(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



BeginPackage["Aerodynamics`PanelMethods`",{"Aerodynamics`CartesianVectors`","Aerodynamics`InfluenceCoefficients`","Aerodynamics`AirfoilGeometry`"}];


PanelMethods::usage="is a package providing functions for specific implementation of three panels methods. Also included are functions to compute aerodynamic characteristics of airfoils from model results.";


ForceAndMoment::usage="ForceAndMoment[singularity,strength,rP,\[Alpha],rC] computes measures of lift and pitching moment given the name of the singularity, values for the strength parameters, angle(s) of attack \[Alpha], and collections of vecors locating panel endpoints, rp, and collocation points, rc.";


PMDoubletConstantDirichlet::usage="PMDoubletConstantDirichlet[rP,\[Alpha]] implements a panel method using doublet panels of constant strength and internal Dirichlet boundary conditions. Panel endpoints describing the discretized airfoil are located by rP, and the angle of attack is \[Alpha].  The function returns a list of strength parameters for the doublet panels, the strength parameter for the wake panel, locations of collocation points and a list of surface speeds at collocation points.";


PMDoubletQuadraticDirichlet::usage="PMDoubletQuadraticDirichlet[rP,\[Alpha]] implements a panel method using doublet panels of quadratically varying strength and internal Dirichlet boundary conditions.  Panel endpoints describing the discretized airfoil are located at rP, and the angle of attack is \!\(\*
StyleBox[\"\[Alpha]\", \"TR\"]\).  The function returns three lists of strength parameters, usually denoted \!\(\*
StyleBox[\"a\", \"TI\"]\), \!\(\*
StyleBox[\"b\", \"TI\"]\) and \!\(\*
StyleBox[\"c\", \"TI\"]\), for the doublet density function, the strength parameter for the constant-strength wake panel, locations of collocation points and a list of surface speeds at collocation points.";


PMVortexLinearNeumann::usage="PMVortexLinearNeumann[rP, \[Alpha]] implements a panel method using vortex panels of linearly varying strength and external Neumann boundary conditions. Panel endpoints describing the discretized airfoil are located at \!\(\*SubscriptBox[
StyleBox[\"r\", \"TI\"], 
StyleBox[\"p\", \"TI\"]]\) and the angle of attack of the airfoil is \!\(\*
StyleBox[\"\[Alpha]\", \"TR\"]\). The function returns strength parameters for the circulation density function, locations of collocation points and surface speeds at collocation points.";


SurfaceSpeed::usage="SurfaceSpeed[Alpha,t,Tau,Theta] computes surface speeds for a van de Vooren airfoil of thickness parameter \!\(\*
StyleBox[\"t\", \"TI\"]\), trailing-edge angle \!\(\*
StyleBox[\"\[Tau]\", \"TR\"]\) and angle of attack \!\(\*
StyleBox[\"\[Alpha]\", \"TR\"]\) at a list of values for \!\(\*
StyleBox[\"\[Theta]\", \"TR\"]\) using an exact solution.";


Options[PMDoubletConstantDirichlet]={Epsilon->10.^-10,InterpolationOrder->1,XWake->10.^10};


Options[PMDoubletQuadraticDirichlet]={Epsilon->10.^-10,InterpolationOrder->1,XWake->10.^10};


Options[PMVortexLinearNeumann]={Epsilon->10.^-10};


Epsilon::usage="is an option for PMVortexLinearNeumann, PMDoubletConstantDirichlet and PMDoubletQuadraticDirichlet that is used to offset collocation points normal to the assocuated panels to insure that they are inside or outside the airfoil as desired for the application of boundary conditions.";


XWake::usage="is an option for doublet panel methods that locates the downstream endpoint of the wake panel.";


Begin["Aerodynamics`PanelMethods`Private`"];


Unprotect[Epsilon,ForceAndMoment,PMDoubletConstantDirichlet,PMDoubletQuadraticDirichlet,PMVortexLinearNeumann,SurfaceSpeed,XWake];


SetAttributes[{ForceAndMoment,PMVortexLinearNeumann,PMDoubletConstantDirichlet,PMDoubletQuadraticDirichlet,SurfaceSpeed},NumericFunction]


PMVortexLinearNeumann[rP_CartesianVectors?TwoDimQ,\[Alpha]_,opts___?OptionQ]:=Module[{a,b,epsilon,icA,icB,im,mBA,mD2,mVA,mVB,m\[CapitalGamma]A,nP,rC,tQ,un,v\[Infinity]N,vel,vel\[Infinity],vC,\[Delta]},
{epsilon}={
Epsilon}/.Flatten[{opts}]/.Options[PMVortexLinearNeumann];
(***** Airfoil geometry *****)
un=PanelNormals[rP];
\[Delta]=PanelLengths[rP];
nP=Length[\[Delta]];
rC=PanelPoints[rP]+epsilon un;
tQ=ThickAirfoilQ[rP];
(***** Influence coefficients *****)
{icA,icB}=ICVortexLinear[rC,rP];
{mVA,mVB}={icA.un,icB.un};
(***** Panel-to-panel value conditions *****)
im=IdentityMatrix[nP];
mD2=im-RotateLeft[im];
(* modify mD2 for Kutta condition *)
m\[CapitalGamma]A=If[tQ==True,ReplacePart[mD2,1,{-1,1}],ReplacePart[mD2,0,{-1,1}]];
(***** Solution procedure *****)
mBA=-DiagonalMatrix[1/\[Delta]].m\[CapitalGamma]A;
v\[Infinity]N=If[Length[N[\[Alpha]]]===0,
vel\[Infinity]=UniformFlow[nP,\[Alpha]];
vel\[Infinity].un,
vel\[Infinity]=Map[UniformFlow[nP,#]&,\[Alpha]];
Transpose[Map[Dot[#,un]&,vel\[Infinity]]]];
a=LinearSolve[mVA+mVB.mBA,-v\[Infinity]N];
b=mBA.a;
(***** Compute speed at collocation points *****)
vC=If[Length[N[\[Alpha]]]===0,
vel=vel\[Infinity]+icA.a+icB.b;
Sqrt[vel.vel],
vel=vel\[Infinity]+Map[Dot[icA,#]&,Transpose[a]]+Map[Dot[icB,#]&,Transpose[b]];
Sqrt[Map[Dot[#,#]&,vel]]];
(***** Arrange output format *****)
{a,b}=If[Length[N[\[Alpha]]]===0,{a,b},{Transpose[a],Transpose[b]}];
{a,b,rC,vC}]


PMDoubletConstantDirichlet[rP_CartesianVectors?TwoDimQ,\[Alpha]_,opts___?OptionQ]:=Module[{a,aOut,aW,epsilon,ic\[Phi],interpolationOrder,kutta,m,nP,rC,rhs,s,sC,soln,un,vC,xWake,\[Delta],\[Phi]F,\[Phi]\[Infinity]},
{epsilon,interpolationOrder,xWake}={Epsilon,InterpolationOrder,XWake}/.Flatten[{opts}]/.Options[PMDoubletConstantDirichlet];
(***** Airfoil geometry *****)un=PanelNormals[rP];
\[Delta]=PanelLengths[rP];
nP=Length[\[Delta]];(* number of panels *)
rC=PanelPoints[rP]-epsilon un;
(***** Boundary conditions & Kutta condition *****)
ic\[Phi]=IC\[Phi]DoubletConstant[rC,Append[rP,{xWake,0.0}]];
kutta=Flatten[{-1.,Table[0.,{nP-2}],1.,-1.}];
(***** Solution procedure *****)
m=Append[ic\[Phi],kutta];
rhs=If[Length[\[Alpha]]===0,
\[Phi]\[Infinity]=UniformFlow\[Phi][rC,\[Alpha]];
-Append[\[Phi]\[Infinity],0.],
\[Phi]\[Infinity]=Map[UniformFlow\[Phi][rC,#]&,\[Alpha]];
Transpose[Map[-Append[#,0.]&,\[Phi]\[Infinity]]]];
soln=LinearSolve[m,rhs];
{a,aW}={Most[soln],Last[soln]};
(***** Compute speed at collocation points *****)
sC=Accumulate[\[Delta]]-0.5\[Delta];
vC=If[Length[N[\[Alpha]]]===0,
\[Phi]F=Interpolation[Transpose[{sC,a}],InterpolationOrder->interpolationOrder];
Abs[D[\[Phi]F[s],s]/.s->sC],
\[Phi]F=Map[Interpolation[Transpose[{sC,a[[All,#]]}],InterpolationOrder->interpolationOrder]&,Range[Length[\[Alpha]]]];
Map[Abs[D[#[s],s]/.s->sC]&,\[Phi]F]];
(***** Arrange output format *****)
aOut=If[Length[\[Alpha]]===0,a,Transpose[a]];
{aOut,aW,rC,vC}]


PMDoubletQuadraticDirichlet[rP_CartesianVectors?TwoDimQ,\[Alpha]_,opts___?OptionQ]:=Module[{a,\[DoubleStruckA],aOut,aW,b,bOut,c,cOut,epsilon,im,interpolationOrder,kutta\[CapitalMu],m,mB\[DoubleStruckCapitalA],mCB,mSB,mSCInv,m\[CapitalMu]A,m\[CapitalMu]\[DoubleStruckCapitalA],m\[CapitalMu]B,m\[CapitalMu]C,m\[CapitalPhi]A,m\[CapitalPhi]\[DoubleStruckCapitalA],m\[CapitalPhi]B,m\[CapitalPhi]C,nP,rC,rCAll,rPWake,s,sC,un,vC,xCE,xWake,zCE,\[Delta],\[Phi]F,\[Phi]\[Infinity]},
{epsilon,interpolationOrder,xWake}={Epsilon,InterpolationOrder,XWake}/.Flatten[{opts}]/.Options[PMDoubletQuadraticDirichlet];
un=PanelNormals[rP];
\[Delta]=PanelLengths[rP];
nP=Length[\[Delta]];
rC=PanelPoints[rP]-epsilon un;
(* locate and append extra collocation point *)
{xCE,zCE}=Mean/@Transpose[{First[rC],Last[rC]}];
rCAll=Append[rC,{xCE,zCE}];
(* wake panel *)
rPWake=Take[Append[rP,{xWake,0.}],-2];
(***** Boundary conditions *****)
{m\[CapitalPhi]A,m\[CapitalPhi]B,m\[CapitalPhi]C}=IC\[Phi]DoubletQuadratic[rCAll,rP];
(* append influence coefficients due to wake panel *)
m\[CapitalPhi]\[DoubleStruckCapitalA]=Transpose[Append[Transpose[m\[CapitalPhi]A],Flatten[IC\[Phi]DoubletConstant[rCAll,rPWake]]]];
(***** Continuity of doublet density & Kutta cond. *****)
im=IdentityMatrix[nP];
m\[CapitalMu]A=im-RotateLeft[im];
m\[CapitalMu]B=DiagonalMatrix[\[Delta]];
m\[CapitalMu]C=DiagonalMatrix[\[Delta]^2];
(* modify with Kutta condition *)
kutta\[CapitalMu]=Join[{-1},Table[0,{nP-2}],{1,-1}];
m\[CapitalMu]\[DoubleStruckCapitalA]=ReplacePart[Transpose[Append[Transpose[m\[CapitalMu]A],Table[0,{nP}]]],-1->kutta\[CapitalMu]];
(***** Continuity of slope of doublet density & Kutta cond. *****)
mSB=ReplacePart[m\[CapitalMu]A,{-1,1}->1];
mSCInv=0.5DiagonalMatrix[1/\[Delta]];
(***** Solution procedure *****)
mCB=-mSCInv.mSB;
mB\[DoubleStruckCapitalA]=-Inverse[m\[CapitalMu]B+m\[CapitalMu]C.mCB].m\[CapitalMu]\[DoubleStruckCapitalA];
m=m\[CapitalPhi]\[DoubleStruckCapitalA]+(m\[CapitalPhi]B+m\[CapitalPhi]C.mCB).mB\[DoubleStruckCapitalA];
\[Phi]\[Infinity]=If[Length[N[\[Alpha]]]===0,
UniformFlow\[Phi][rCAll,\[Alpha]],
Transpose[Map[UniformFlow\[Phi][rCAll,#]&,\[Alpha]]]];
\[DoubleStruckA]=LinearSolve[m,-\[Phi]\[Infinity]];
{a,aW}={Most[\[DoubleStruckA]],Last[\[DoubleStruckA]]};
b=mB\[DoubleStruckCapitalA].\[DoubleStruckA];
c=mCB.b;
(***** Compute surface speeds *****)
sC=Accumulate[\[Delta]]-0.5\[Delta];
vC=If[Length[N[\[Alpha]]]===0,
\[Phi]F=Interpolation[Transpose[{sC,a+0.5\[Delta] b+0.25\[Delta]^2 c}],InterpolationOrder->interpolationOrder];
Abs[D[\[Phi]F[s],s]/.s->sC],
\[Phi]F=Map[Interpolation[Transpose[{sC,a[[All,#]]+0.5\[Delta] b[[All,#]]+0.25\[Delta]^2 c[[All,#]]}],InterpolationOrder->interpolationOrder]&,Range[Length[\[Alpha]]]];
Map[Abs[D[#[s],s]/.s->sC]&,\[Phi]F]];
(***** Arrange output format *****)
{aOut,bOut,cOut}=If[Length[N[\[Alpha]]]===0,{a,b,c},{Transpose[a],Transpose[b],Transpose[c]}];
{aOut,bOut,cOut,aW,rC,vC}]


ForceAndMoment[sing_String,strength__?MatrixQ,rP_CartesianVectors?TwoDimQ,\[Alpha]_List,rC_CartesianVectors?TwoDimQ]:=Module[{a,b,cl,clD,clF,cl\[Alpha],cm,cmD,cmF,cm0,nS,xac,\[Alpha]\[Alpha],\[Alpha]0,\[CapitalDelta]cl,\[CapitalDelta]cm},
nS=Length[{strength}];
{cl,cm,\[CapitalDelta]cl,\[CapitalDelta]cm}=Switch[nS,
1,
a=strength;
Transpose[Map[ForceAndMoment[sing,a[[#]],rP,\[Alpha][[#]],rC]&,Range[Length[\[Alpha]]]]],
2,
{a,b}={strength};
Transpose[Map[ForceAndMoment[sing,a[[#]],b[[#]],rP,\[Alpha][[#]],rC]&,Range[Length[\[Alpha]]]]],
3,
{a, b, c}={strength};
Transpose[Map[ForceAndMoment[sing,a[[#]],b[[#]],c[[#]],rP,\[Alpha][[#]],rC]&,Range[Length[\[Alpha]]]]]];
{clD,cmD}={Transpose[{\[Alpha],cl}],Transpose[{\[Alpha],cm}]};
{clF,cmF}={Fit[clD,{1,\[Alpha]\[Alpha]},\[Alpha]\[Alpha]],Fit[cmD,{1,\[Alpha]\[Alpha]},\[Alpha]\[Alpha]]};
{cl\[Alpha],\[Alpha]0}={clF[[2,1]],\[Alpha]\[Alpha]/.Solve[clF==0,\[Alpha]\[Alpha]][[1]]};
{cm0,xac}={cmF/.\[Alpha]\[Alpha]->\[Alpha]0,-PolynomialQuotient[cmF-cm0,clF,\[Alpha]\[Alpha]]};
{cl\[Alpha],\[Alpha]0,xac,cm0}]


ForceAndMoment[sing_String,strength__?VectorQ,rP_CartesianVectors?TwoDimQ,\[Alpha]_,rC_CartesianVectors?TwoDimQ]:=Module[{a,b,c,cl,nP,rM,nS,ud,ul,\[Delta],\[CapitalDelta]cl,\[CapitalDelta]clA,\[CapitalDelta]clB,\[CapitalDelta]cm,\[CapitalDelta]cmA,\[CapitalDelta]cmB,\[CapitalDelta]cmC,\[CapitalDelta]clC,\[CapitalDelta]f,\[CapitalDelta]fA,\[CapitalDelta]fB,\[CapitalDelta]fC},
\[Delta]=PanelLengths[rP];
{nS,nP}={Length[{strength}],Length[\[Delta]]};
(* unit vectors in direction of drag and lift *)
ud=ReplicateVector[MakeCartesianVectors[{{Cos[\[Alpha]]},{Sin[\[Alpha]]}}],nP];
ul=Cross[ud];
(**** Lift & moment from Kutta-Joukowski theorem ****)
Switch[sing,
"Doublet",
(* \[Gamma] (x) = - \[DifferentialD]\[Mu]/\[DifferentialD]x *)
Switch[nS,
1,(* constant doublet density -> point vortices at panel edges *)
\[CapitalDelta]cl=2ReplacePart[strength-RotateLeft[strength],0.,{-1}];
\[CapitalDelta]f=MakeCartesianVectors[{-\[CapitalDelta]cl Sin[\[Alpha]],\[CapitalDelta]cl Cos[\[Alpha]]}];
\[CapitalDelta]cm=Components[ExtendXzToXyz[Rest[rP]]\[Cross]ExtendXzToXyz[\[CapitalDelta]f]][[2]];
{Plus@@\[CapitalDelta]cl,Plus@@\[CapitalDelta]cm,\[CapitalDelta]cl,\[CapitalDelta]cm},
3, (* quadratic doublet density -> linear circulation density *)
{a,b,c}={strength};
{\[CapitalDelta]clB,\[CapitalDelta]clC}=-{2b \[Delta],c \[Delta]^2};
{\[CapitalDelta]fB,\[CapitalDelta]fC}={MultiplyByList[\[CapitalDelta]clB,ul],MultiplyByList[\[CapitalDelta]clC,ul]};
rM=PanelPoints[rP,PanelOffset->2/3];
\[CapitalDelta]cmB=Components[ExtendXzToXyz[rC]\[Cross]ExtendXzToXyz[\[CapitalDelta]fB]][[2]];
\[CapitalDelta]cmC=Components[ExtendXzToXyz[rM]\[Cross]ExtendXzToXyz[\[CapitalDelta]fC]][[2]];
{\[CapitalDelta]cl,\[CapitalDelta]cm}={\[CapitalDelta]clB+\[CapitalDelta]clC,\[CapitalDelta]cmB+\[CapitalDelta]cmC};
{Plus@@\[CapitalDelta]cl,Plus@@\[CapitalDelta]cm,\[CapitalDelta]cl,\[CapitalDelta]cm}],
"Vortex",
Switch[nS,
1,
"model not implemented",
2,
{a,b}={strength};
{\[CapitalDelta]clA,\[CapitalDelta]clB}={2a \[Delta],b \[Delta]^2};
{\[CapitalDelta]fA,\[CapitalDelta]fB}={MultiplyByList[\[CapitalDelta]clA,ul],MultiplyByList[\[CapitalDelta]clB,ul]};
rM=PanelPoints[rP,PanelOffset->2/3];
\[CapitalDelta]cmA=Components[ExtendXzToXyz[rC]\[Cross]ExtendXzToXyz[\[CapitalDelta]fA]][[2]];
\[CapitalDelta]cmB=Components[ExtendXzToXyz[rM]\[Cross]ExtendXzToXyz[\[CapitalDelta]fB]][[2]];
{\[CapitalDelta]cl,\[CapitalDelta]cm}={\[CapitalDelta]clA+\[CapitalDelta]clB,\[CapitalDelta]cmA+\[CapitalDelta]cmB};
{Plus@@\[CapitalDelta]cl,Plus@@\[CapitalDelta]cm,\[CapitalDelta]cl,\[CapitalDelta]cm}]]]


SurfaceSpeed[\[Alpha]_,t_,\[Tau]_,\[Theta]_List]:=Module[{a,aA,b,bB,dD0,dD1,dD2,k,q,r1,r2,u,ux,uy,w,wx,wy,\[Epsilon],\[Theta]1,\[Theta]1x,\[Theta]2},
k=2-\[Tau]/\[Pi];\[Epsilon]=t/2;a=2^(1-k) (1+\[Epsilon])^(k-1);b=Map[Which[0<=#<\[Pi]/2,0,\[Pi]/2<=#<(3\[Pi])/2,1,(3\[Pi])/2<=#<=2\[Pi],2]&,\[Theta]];r1=Sqrt[(a Cos[\[Theta]]-a)^2+a^2 Sin[\[Theta]]^2];r2=Sqrt[(a Cos[\[Theta]]-\[Epsilon] a)^2+a^2 Sin[\[Theta]]^2];
Off[Power::infy,\[Infinity]::indet];
\[Theta]1x=ArcTan[ Sin[\[Theta]]/( Cos[\[Theta]]-1)]+\[Pi];\[Theta]1=ReplacePart[\[Theta]1x,-(\[Pi]/2),Position[\[Theta]1x,Indeterminate]];On[Power::infy,\[Infinity]::indet];
\[Theta]2=ArcTan[ Sin[\[Theta]]/(Cos[\[Theta]]-\[Epsilon])]+b \[Pi];
aA=Cos[(k-1)\[Theta]1] Cos[k \[Theta]2]+Sin[(k-1)\[Theta]1] Sin[k \[Theta]2];
bB=Sin[(k-1)\[Theta]1] Cos[k \[Theta]2]-Cos[(k-1)\[Theta]1] Sin[k \[Theta]2];
dD0=a (1-k+\[Epsilon] k);dD1=aA (a Cos[\[Theta]]-dD0)-bB a Sin[\[Theta]];
dD2=aA a Sin[\[Theta]]+bB (a Cos[\[Theta]]-dD0);
Off[Power::infy,\[Infinity]::indet];
ux=2 r2^k/r1^(k-1) (Sin[\[Alpha]]-Sin[\[Alpha]-\[Theta]])/(dD1^2+dD2^2) (dD1 Sin[\[Theta]]+dD2 Cos[\[Theta]]);
uy=ReplacePart[ux,0,Position[ux,Indeterminate]];
u=ReplacePart[uy,0,Position[uy,ComplexInfinity]];
wx=-2 r2^k/r1^(k-1) (Sin[\[Alpha]]-Sin[\[Alpha]-\[Theta]])/(dD1^2+dD2^2) (dD1 Cos[\[Theta]]-dD2 Sin[\[Theta]]);
wy=ReplacePart[wx,0,Position[wx,Indeterminate]];
w=ReplacePart[wy,0,Position[wy,ComplexInfinity]];
On[Power::infy,\[Infinity]::indet];
q=MakeCartesianVectors[{u,w}];Reverse[Sqrt[q.q]]]


End[];


Protect[Epsilon,ForceAndMoment,PMDoubletConstantDirichlet,PMDoubletQuadraticDirichlet,PMVortexLinearNeumann,SurfaceSpeed,XWake];


EndPackage[]
