(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[      8063,        275]
NotebookOptionsPosition[      5620,        200]
NotebookOutlinePosition[      6771,        231]
CellTagsIndexPosition[      6728,        228]
WindowTitle->Airfoil Geometry - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "GuideColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[CellGroupData[{

Cell["Airfoil Geometry", "GuideTitle",
 CellID->942062912],

Cell["\<\
This package provides functions for discretizing airfoil geometry in \
preparation for using computational models to compute flow over the airfoil.\
\>", "GuideAbstract",
 CellID->2001916300],

Cell[TextData[{
 Cell[TextData[ButtonBox["AirfoilFromDataFile",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " imports airfoil data and fits cubic spline"
}], "GuideText",
 CellID->203374175],

Cell[TextData[{
 Cell[TextData[ButtonBox["AirfoilSurfacePoints",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " collection of vectors locating points on airfoil surface"
}], "GuideText",
 CellID->612614724],

Cell[TextData[{
 Cell[TextData[ButtonBox["FitAirfoil",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " fits a B-spline to a collection of vectors specifying points on an airfoil"
}], "GuideText",
 CellID->31839155],

Cell[TextData[{
 Cell[TextData[ButtonBox["NACA4DigitAirfoil",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/NACA4DigitAirfoil"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " thickness and camber properties of NACA four-digit airfoil"
}], "GuideText",
 CellID->16749258],

Cell[TextData[{
 Cell[TextData[ButtonBox["NDiscretizeUnitSegment",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/NDiscretizeUnitSegment"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " list of x values used for discretization of airfoil"
}], "GuideText",
 CellID->56389064],

Cell[TextData[{
 Cell[TextData[ButtonBox["PanelAngles",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PanelAngles"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " angles of panels describing discretized airfoil"
}], "GuideText",
 CellID->5001441],

Cell[TextData[{
 Cell[TextData[ButtonBox["PanelLengths",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PanelLengths"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " lengths of panels describing discretized airfoil"
}], "GuideText",
 CellID->146539965],

Cell[TextData[{
 Cell[TextData[ButtonBox["PanelNormals",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PanelNormals"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " collection of unit normal vectors to panels describing discretized airfoil"
}], "GuideText",
 CellID->43477983],

Cell[TextData[{
 Cell[TextData[ButtonBox["PanelPoints",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PanelPoints"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " collection of vectors locating points on panels describing discretized \
airfoil"
}], "GuideText",
 CellID->396586081],

Cell[TextData[{
 Cell[TextData[ButtonBox["ThickAirfoilQ",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ThickAirfoilQ"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " does airfoil have thickness"
}], "GuideText",
 CellID->312758],

Cell[TextData[{
 Cell[TextData[ButtonBox["TrailingEdgeAngle",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/TrailingEdgeAngle"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " angle between trailing-edge panels"
}], "GuideText",
 CellID->166696838],

Cell[TextData[{
 Cell[TextData[ButtonBox["XToTheta",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/XToTheta"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " converts x values to \[Theta] values using van de Vooren transformation"
}], "GuideText",
 CellID->95899216]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["1.1 Introduction To Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]], \
"GuideTutorial",
 CellID->30833798],

Cell[TextData[ButtonBox["Table Of Contents",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/TableOfContents"]], "GuideTutorial",
 CellID->64701022]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Airfoil Geometry - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 55, 30.8166000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Guide", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "This package provides functions for discretizing airfoil geometry in \
preparation for using computational models to compute flow over the airfoil.",
     "synonyms" -> {}, "title" -> "Airfoil Geometry", "type" -> "Guide", 
    "uri" -> "Aerodynamics/guide/AirfoilGeometry"}, "SearchTextTranslated" -> 
  ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[597, 21, 75, 1, 70, "GuideColorBar"],
Cell[675, 24, 191, 5, 70, "LinkTrail"],
Cell[CellGroupData[{
Cell[891, 33, 58, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[952, 36, 201, 4, 70, "GuideAbstract",
 CellID->2001916300],
Cell[1156, 42, 304, 9, 70, "GuideText",
 CellID->203374175],
Cell[1463, 53, 320, 9, 70, "GuideText",
 CellID->612614724],
Cell[1786, 64, 314, 8, 70, "GuideText",
 CellID->31839155],
Cell[2103, 74, 315, 9, 70, "GuideText",
 CellID->16749258],
Cell[2421, 85, 318, 9, 70, "GuideText",
 CellID->56389064],
Cell[2742, 96, 288, 8, 70, "GuideText",
 CellID->5001441],
Cell[3033, 106, 293, 8, 70, "GuideText",
 CellID->146539965],
Cell[3329, 116, 318, 8, 70, "GuideText",
 CellID->43477983],
Cell[3650, 126, 323, 9, 70, "GuideText",
 CellID->396586081],
Cell[3976, 137, 271, 8, 70, "GuideText",
 CellID->312758],
Cell[4250, 147, 292, 9, 70, "GuideText",
 CellID->166696838],
Cell[4545, 158, 307, 8, 70, "GuideText",
 CellID->95899216]
}, Open  ]],
Cell[CellGroupData[{
Cell[4889, 171, 62, 1, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[4954, 174, 201, 5, 70, "GuideTutorial",
 CellID->30833798],
Cell[5158, 181, 163, 3, 70, "GuideTutorial",
 CellID->64701022]
}, Open  ]],
Cell[CellGroupData[{
Cell[5358, 189, 62, 1, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[5423, 192, 155, 3, 70, "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[5593, 198, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

