(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[      6470,        218]
NotebookOptionsPosition[      3955,        145]
NotebookOutlinePosition[      5309,        178]
CellTagsIndexPosition[      5266,        175]
WindowTitle->PanelMethods - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "GuideColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[CellGroupData[{

Cell["PanelMethods", "GuideTitle",
 CellID->942062912],

Cell["\<\
This package provides functions that implement some numerical panel methods \
for steady flow over two-dimensional airfoils.  These models formulate and \
solve a system of linear algebraic equations for singularity strength \
parameters.  Included in the package are functions that compute common \
aerodynamic properties of an airfoil from the strength parameters.\
\>", "GuideAbstract",
 CellID->2001916300],

Cell[CellGroupData[{

Cell["Models", "GuideFunctionsSubsection",
 CellID->32271],

Cell[TextData[{
 Cell[TextData[ButtonBox["PMVortexLinearNeumann",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " linear vortex panels & exterior Neumann b.c."
}], "GuideText",
 CellID->212983962],

Cell[TextData[{
 Cell[TextData[ButtonBox["PMDoubletQuadraticDirichlet",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " quadratic doublet panels & interior Dirichlet b.c."
}], "GuideText",
 CellID->1463276848],

Cell[TextData[{
 Cell[TextData[ButtonBox["PMDoubletConstantDirichlet",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " constant doublet panels & interior Dirichlet b.c."
}], "GuideText",
 CellID->172791454],

Cell["\t", "GuideDelimiterSubsection",
 CellID->311258892]
}, Open  ]],

Cell[CellGroupData[{

Cell["Aerodynamic Properties", "GuideFunctionsSubsection",
 CellID->200503920],

Cell[TextData[{
 Cell[TextData[ButtonBox["ForceAndMoment",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/ForceAndMoment"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " force and pitching moment characteristics"
}], "GuideText",
 CellID->371265540],

Cell[TextData[{
 Cell[TextData[ButtonBox["SurfaceSpeed",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/SurfaceSpeed"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " surface speeds for van de Vooren airfoil"
}], "GuideText",
 CellID->641452831]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS", "GuideTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["3.1 Introduction To Panel Methods",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"]], "GuideTutorial",
 CellID->806871991],

Cell[TextData[ButtonBox["Table of Contents",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/TableOfContents"]], "GuideTutorial",
 CellID->1885805579]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "GuideMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"PanelMethods - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 55, 37.8210000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Guide", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "This package provides functions that implement some numerical panel \
methods for steady flow over two-dimensional airfoils. These models formulate \
and solve a system of linear algebraic equations for singularity strength \
parameters. Included in the package are functions that compute common \
aerodynamic properties of an airfoil from the strength parameters.", 
    "synonyms" -> {}, "title" -> "PanelMethods", "type" -> "Guide", "uri" -> 
    "Aerodynamics/guide/PanelMethods"}, "SearchTextTranslated" -> ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[593, 21, 75, 1, 70, "GuideColorBar"],
Cell[671, 24, 191, 5, 70, "LinkTrail"],
Cell[CellGroupData[{
Cell[887, 33, 54, 1, 70, "GuideTitle",
 CellID->942062912],
Cell[944, 36, 420, 7, 70, "GuideAbstract",
 CellID->2001916300],
Cell[CellGroupData[{
Cell[1389, 47, 58, 1, 70, "GuideFunctionsSubsection",
 CellID->32271],
Cell[1450, 50, 310, 9, 70, "GuideText",
 CellID->212983962],
Cell[1763, 61, 329, 9, 70, "GuideText",
 CellID->1463276848],
Cell[2095, 72, 325, 9, 70, "GuideText",
 CellID->172791454],
Cell[2423, 83, 58, 1, 70, "GuideDelimiterSubsection",
 CellID->311258892]
}, Open  ]],
Cell[CellGroupData[{
Cell[2518, 89, 78, 1, 70, "GuideFunctionsSubsection",
 CellID->200503920],
Cell[2599, 92, 293, 9, 70, "GuideText",
 CellID->371265540],
Cell[2895, 103, 285, 8, 70, "GuideText",
 CellID->641452831]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[3229, 117, 62, 1, 70, "GuideTutorialsSection",
 CellID->415694126],
Cell[3294, 120, 194, 4, 70, "GuideTutorial",
 CellID->806871991],
Cell[3491, 126, 165, 3, 70, "GuideTutorial",
 CellID->1885805579]
}, Open  ]],
Cell[CellGroupData[{
Cell[3693, 134, 62, 1, 70, "GuideMoreAboutSection",
 CellID->23220180],
Cell[3758, 137, 155, 3, 70, "GuideMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[3928, 143, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

