(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     10271,        314]
NotebookOptionsPosition[      8023,        245]
NotebookOutlinePosition[      9238,        277]
CellTagsIndexPosition[      9195,        274]
WindowTitle->Aerodynamics - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACLET SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Table Of Contents\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/Aerodynamics\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/Aerodynamics"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
Aerodynamics.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/Aerodynamics", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Aerodynamics", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      StyleBox["Aerodynamics",
       FontSlant->"Italic"]], "InlineFormula"],
     "\[LineSeparator]is a collection of five packages providing functions to \
implement numerical models for two-dimensional steady incompressible \
potential flow over an airfoil using panel methods. "
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{
      Dynamic[
       If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
        RGBColor[0.92, 1, 0.59], None]]}}, "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->1072863367],

Cell[TextData[{
 "There are five packages included in the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " software."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 " defines a data type for specifying collections of vectors in Cartesian \
coordinates.  This data type facilitates vector operations involving \
collections of vectors and provides convenient pattern matching for input \
arguments in defined functions used in the following packages."
}], "Notes",
 CellID->137363931],

Cell[TextData[{
 Cell[BoxData[
  ButtonBox["AirfoilGeometry",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilGeometry"]], "InlineFormula"],
 " provides functions for specifying airfoil shape, discretizing the airfoil \
and computing geometric properties of the discretized airfoil."
}], "Notes",
 CellID->86597757],

Cell[TextData[{
 Cell[BoxData[
  ButtonBox["InfluenceCoefficients",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/InfluenceCoefficients"]], 
  "InlineFormula"],
 " provides a collection of functions for velocity, velocity-potential and \
stream-function influence coefficients for commonly used two-dimensional \
potential-flow point and line singularities."
}], "Notes",
 CellID->208577819],

Cell[TextData[{
 ButtonBox["PanelMethods",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PanelMethods"],
 " provides functions for the implementation of panel methods:"
}], "Notes",
 CellID->264646838],

Cell[TextData[{
 ButtonBox["Graphics",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/Graphics"],
 " provides functions for plotting some results such as airfoil shape and \
surface pressure coefficient in conventional formats used by the aerodynamics \
community."
}], "Notes",
 CellID->121740225]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["Table Of Contents",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/TableOfContents"]], "Tutorials",
 CellID->20558713]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "MoreAbout",
 CellID->1665078683]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Aerodynamics - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 55, 39.8646000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics", "status" -> "", "summary" -> 
    "Aerodynamics is a collection of five packages providing functions to \
implement numerical models for two-dimensional steady incompressible \
potential flow over an airfoil using panel methods. ", "synonyms" -> {}, 
    "title" -> "Aerodynamics", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/Aerodynamics"}, "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[593, 21, 76, 1, 70, "SymbolColorBar"],
Cell[672, 24, 191, 5, 70, "LinkTrail"],
Cell[866, 31, 2606, 60, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3497, 95, 55, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3555, 98, 653, 16, 70, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[4245, 119, 581, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->1072863367],
Cell[4829, 138, 158, 6, 70, "Notes",
 CellID->1067943069],
Cell[4990, 146, 483, 10, 70, "Notes",
 CellID->137363931],
Cell[5476, 158, 336, 8, 70, "Notes",
 CellID->86597757],
Cell[5815, 168, 407, 10, 70, "Notes",
 CellID->208577819],
Cell[6225, 180, 215, 6, 70, "Notes",
 CellID->264646838],
Cell[6443, 188, 311, 8, 70, "Notes",
 CellID->121740225]
}, Closed]],
Cell[CellGroupData[{
Cell[6791, 201, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[7213, 215, 159, 3, 70, "Tutorials",
 CellID->20558713]
}, Open  ]],
Cell[CellGroupData[{
Cell[7409, 223, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[7831, 237, 150, 3, 70, "MoreAbout",
 CellID->1665078683]
}, Open  ]],
Cell[7996, 243, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

