(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     18171,        502]
NotebookOptionsPosition[     15380,        413]
NotebookOutlinePosition[     16766,        452]
CellTagsIndexPosition[     16681,        447]
WindowTitle->AirfoilInteriorQ - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACLET SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Airfoil Geometry\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/AirfoilGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/AirfoilInteriorQ\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/AirfoilInteriorQ"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
AirfoilInteriorQ.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/AirfoilInteriorQ", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["AirfoilInteriorQ", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{
       ButtonBox["AirfoilInteriorQ",
        BaseStyle->"Link",
        ButtonData->"paclet:Aerodynamics/ref/AirfoilInteriorQ"], "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          StyleBox["x", "TI"], ",", 
          StyleBox["z", "TI"]}], "}"}], ",", 
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["p", "TI"]]}], "]"}]], "InlineFormula"],
     " \[LineSeparator]returns True if the point ",
     Cell[BoxData[
      RowBox[{"{", 
       RowBox[{
        StyleBox["x", "TI"], ",", 
        StyleBox["z", "TI"]}], "}"}]], "InlineFormula"],
     " is interior to the discretized airfoil specified by the collection of \
vectors ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     ", False if not."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{
      Dynamic[
       If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
        RGBColor[0.92, 1, 0.59], None]]}}, "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->3446597],

Cell[TextData[{
 "Collections of vectors are of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 "."
}], "Notes",
 CellID->1067943069],

Cell["\<\
AirfoilInteriorQ uses the function PolygonInteriorQ furnished by Wolfram \
Technical Help.\
\>", "Notes",
 CellID->94660197],

Cell["\<\
AirfoilInteriorQ is used primarily to exclude the computed flow field \
interior to the airfoil for flow visualization.\
\>", "Notes",
 CellID->89738067]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->530746571],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->223441854],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->489055632],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1004438945],

Cell[TextData[{
 "Import the data file for the Eppler-205 airfoil from the folder Airfoil \
Data and arrange the data in the form used by ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "ExampleText",
 CellID->373278422],

Cell[BoxData[
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->317289154],

Cell["\<\
Generate and show graphics that plot the airfoil as described by the data, \
and that the show the region interior to the airfoil using RegionPlot.  \
\>", "ExampleText",
 CellID->618169264],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gAirfoil", "=", 
   RowBox[{"PlotAirfoil", "[", "rD", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.05"}], ",", "1.05", ",", 
     RowBox[{"-", "0.1"}], ",", "0.201"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gTest", "=", 
   RowBox[{"RegionPlot", "[", 
    RowBox[{
     RowBox[{"AirfoilInteriorQ", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "z"}], "}"}], ",", "rD"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "1"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gAirfoil", ",", "gTest"}], "]"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->507800874],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3ftzVOUdx/F0+pf0ByugvY21ahC8C7F1qIptEUVhyqKiGDNChQxykZsh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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->209009266]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "MoreAbout",
 CellID->1665078683]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"AirfoilInteriorQ - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 55, 49.0842000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics", "status" -> "", "summary" -> 
    "AirfoilInteriorQ[{x, z}, r_p] returns True if the point {x, z} is \
interior to the discretized airfoil specified by the collection of vectors \
r_p, False if not.", "synonyms" -> {}, "title" -> "AirfoilInteriorQ", "type" -> 
    "Symbol", "uri" -> "Aerodynamics/ref/AirfoilInteriorQ"}, 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5730, 176, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->530746571]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 16538, 440}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[597, 21, 76, 1, 70, "SymbolColorBar"],
Cell[676, 24, 191, 5, 70, "LinkTrail"],
Cell[870, 31, 2366, 53, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3261, 88, 59, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3323, 91, 1198, 36, 70, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[4558, 132, 578, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->3446597],
Cell[5139, 151, 251, 8, 70, "Notes",
 CellID->1067943069],
Cell[5393, 161, 134, 4, 70, "Notes",
 CellID->94660197],
Cell[5530, 167, 163, 4, 70, "Notes",
 CellID->89738067]
}, Closed]],
Cell[CellGroupData[{
Cell[5730, 176, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->530746571],
Cell[CellGroupData[{
Cell[6216, 193, 148, 5, 70, "ExampleSection",
 CellID->223441854],
Cell[6367, 200, 132, 6, 70, "ExampleText",
 CellID->489055632],
Cell[6502, 208, 101, 3, 70, "Input",
 CellID->1004438945],
Cell[6606, 213, 234, 7, 70, "ExampleText",
 CellID->373278422],
Cell[6843, 222, 217, 8, 70, "Input",
 CellID->317289154],
Cell[7063, 232, 200, 4, 70, "ExampleText",
 CellID->618169264],
Cell[CellGroupData[{
Cell[7288, 240, 1064, 31, 101, "Input",
 CellID->507800874],
Cell[8355, 273, 6343, 111, 201, "Output",
 CellID->209009266]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[14759, 391, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[15181, 405, 157, 3, 70, "MoreAbout",
 CellID->1665078683]
}, Open  ]],
Cell[15353, 411, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

