(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     11762,        355]
NotebookOptionsPosition[      9503,        286]
NotebookOutlinePosition[     10742,        318]
CellTagsIndexPosition[     10699,        315]
WindowTitle->CartesianVectors - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACLET SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"A1 CartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/CartesianVectors"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"MakeCartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/MakeCartesianVectors"], "\<\"Components\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/Components"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"CartesianVectors\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/CartesianVectors"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/CartesianVectors\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/CartesianVectors"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
CartesianVectors.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/CartesianVectors", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["CartesianVectors", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      ButtonBox["CartesianVectors",
       BaseStyle->"Link",
       ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], 
      "InlineFormula"],
     " \[LineSeparator]is the name of a data type specifying a collection of ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     "-dimensional vectors in a rectangular coordinate system.  \
CartesianVectors is also the name of the package defining this data type."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{
      Dynamic[
       If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
        RGBColor[0.92, 1, 0.59], None]]}}, "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->33858485],

Cell[TextData[{
 "The data type CartesianVectors simplifies the logistics of handling \
collections of vectors used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  The data type is used both for pattern matching input arguments for \
defined functions, and for basic vector algebra.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions such as Dot and Cross, defined for single vectors, are \
overloaded for use with element-by-element operations for collections of \
vectors.  The data type CartesianVectors is formed by replacing List with \
CartesianVectors as the Head of suitably a structured lists.  Consequently, \
many operations for manipulating lists are overloaded to provide analogous \
operations for collections of vectors of the data type CartesianVectors."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The output format for the data type CartesianVectors uses the infix \
\[UpArrow] between lists of components, eg., ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["x", "1"], ",", 
     SubscriptBox["x", "2"], ",", 
     SubscriptBox["x", "3"], ",", 
     SubscriptBox["x", "4"]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["y", "1"], ",", 
     SubscriptBox["y", "2"], ",", 
     SubscriptBox["y", "3"], ",", 
     SubscriptBox["y", "4"]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["z", "1"], ",", 
     SubscriptBox["z", "2"], ",", 
     SubscriptBox["z", "3"], ",", 
     SubscriptBox["z", "4"]}], "}"}]}]], "InlineFormula"],
 " denotes a collection of four three-dimensional vectors."
}], "Notes",
 CellID->820573],

Cell["\<\
The constructor for the data type is MakeCartesianVectors and the selector is \
Components.\
\>", "Notes",
 CellID->37659213]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["MakeCartesianVectors",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/MakeCartesianVectors"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["Components",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/Components"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->653943820]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["A1 CartesianVectors",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/CartesianVectors"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["CartesianVectors",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/CartesianVectors"]], "MoreAbout",
 CellID->1665078683]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"CartesianVectors - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 55, 56.0730000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics", "status" -> "", "summary" -> 
    "CartesianVectors is the name of a data type specifying a collection of \
n-dimensional vectors in a rectangular coordinate system. CartesianVectors is \
also the name of the package defining this data type.", "synonyms" -> {}, 
    "title" -> "CartesianVectors", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/CartesianVectors"}, "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[597, 21, 76, 1, 70, "SymbolColorBar"],
Cell[676, 24, 191, 5, 70, "LinkTrail"],
Cell[870, 31, 3155, 72, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4050, 107, 59, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4112, 110, 807, 20, 70, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[4956, 135, 579, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->33858485],
Cell[5538, 154, 832, 16, 70, "Notes",
 CellID->1067943069],
Cell[6373, 172, 821, 25, 70, "Notes",
 CellID->820573],
Cell[7197, 199, 135, 4, 70, "Notes",
 CellID->37659213]
}, Closed]],
Cell[CellGroupData[{
Cell[7369, 208, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[7789, 222, 433, 15, 70, "SeeAlso",
 CellID->653943820]
}, Open  ]],
Cell[CellGroupData[{
Cell[8259, 242, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[8681, 256, 163, 3, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[8881, 264, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[9303, 278, 158, 3, 70, "MoreAbout",
 CellID->1665078683]
}, Open  ]],
Cell[9476, 284, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

