(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     39665,       1043]
NotebookOptionsPosition[     35270,        893]
NotebookOutlinePosition[     36582,        931]
CellTagsIndexPosition[     36497,        926]
WindowTitle->FitAirfoil - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACLET SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.2 Airfoils From Data Files\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilFromDataFile\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/AirfoilFromDataFile"], "\<\"PlotAirfoil\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/PlotAirfoil"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Airfoil Geometry\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/AirfoilGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/FitAirfoil\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/FitAirfoil"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
FitAirfoil.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/FitAirfoil", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["FitAirfoil", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{
       ButtonBox["FitAirfoil",
        BaseStyle->"Link",
        ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"], "[", 
       SubscriptBox[
        StyleBox["r", "TI"], 
        StyleBox["d", "TI"]], "]"}]], "InlineFormula"],
     " \[LineSeparator]fits a b-spline to a collection of vectors specifying \
points on the surface of an airfoil."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{
      Dynamic[
       If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
        RGBColor[0.92, 1, 0.59], None]]}}, "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->14533967],

Cell[TextData[{
 "This is a temporary function using code furnished by the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Development Group to use a re-parametrization method to provide a better \
interpolation function for B-splines."
}], "Notes",
 CellID->1067943069]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->59296152],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->118571419],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->7484878],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->175506032],

Cell[TextData[{
 "Use AirfoilFromDataFile to import data from the file for the Eppler-205 \
airfoil and rearrange the data to the form used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  This data file uses the counterclockwise numbering convention, which is \
the default value for the option NumberingConvention."
}], "ExampleText",
 CellID->158695459],

Cell[BoxData[
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->128851687],

Cell["Use FitAirfoil to fit a B-spline to the data.", "ExampleText",
 CellID->124701399],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"spline", "=", 
  RowBox[{"FitAirfoil", "[", "rD", "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->63002811],

Cell[BoxData[
 TagBox[
  RowBox[{"BSplineFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"0.`", ",", "1.0000000000000004`"}], "}"}], "}"}], 
    ",", "\<\"<>\"\>"}], "]"}],
  False,
  Editable->False]], "Output",
 ImageSize->{215, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->137712546]
}, Open  ]],

Cell["Plot data points and spline fit.", "ExampleText",
 CellID->554708867],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gD", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->271980122],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PFPUdx/Fr+pf037hEueMOFAk9D4vYWmrhZAtt1Wo0MagRalsIQrrx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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->60115126]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Possible Issues",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1305812373],

Cell[TextData[{
 "The B-spline is an approximation of the airfoil shape.  Some details of the \
approximation need additional study to determine if the fit can be forced to \
conform to conventional properties of an airfoil.  This will be done after \
the next major release of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " when increased support for B-splines is planned."
}], "ExampleText",
 CellID->58615840],

Cell["Import data for the NACA-4412 airfoil and plot.", "ExampleText",
 CellID->659725730],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\naca4412m.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"spline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"grD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"grD", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->337273913],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3e1vFNcVx3FX/Uv6byC1QIzTEqdxHZU6iuMmmGYFzVNbgRo5fQFSC1UE
6QoSgXmICY+xTVOTRktQTEWwREFKgoqwFGWRkKNG5kUsJCykroTkHu+xr9c7
u+N9mN25Z+b70SZx1vb6zO7Mb+7ce2fmR4NvPDv4w46Ojk/kn//9oKNj8esF
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAC8kct0iHXZfKWng88DAJrS7tSV
F87kiv9d9dr5bDa39G2SHgCi0/bUzWfXFf9g4E+uVETOA0Bk2p+6yzuWai+d
yyx9e0kHAKAhjaVu88L3LMvHEatyPtoCACANXHjWm7oRqNJTpMUUn8plS75B
zgNAA1bCs87UjcTqkd+lCpbGfYtKjyHIeQBoQGl41pW67UfOA0ADDIWnoVIB
wB+GwtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6G
SgUAfxgKT0OlAoA/DIWnoVIBwB+GwtNQqQDgD0PhaahUAPCHofA0VCoA+MNQ
eBoqFQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/DIWnoVIBwB+GwtNQqQDg
D0PhaahUAPCHofA0VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0Ol
AoA/DIWnoVIBwB+GwtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoqFQD8YSg8
DZUKAP4wFJ6GSgUAfxgKT0OlAoA/4g3PXEb+fse6bH710/nsOnk6k1v9LDkP
AA0oDc+6UjcC8veKr5rLBP7m4h8l5wEgAivhWWfqNm/lRSv8SXIeAKLhwrPe
1G3e8o6FnEc0bt749o3X/3kp93XchQB+ceFZb+o2r4H2vBN1LYjM5Gf5bVsv
DB++ceur7+RxZfLuyPtfVHwcOnhdYrmux66duWqvJo+en59a/+PhTRuP6Z+W
x5df/nf6zv243xIgHsHMbH97nv55b5U1jCUqg6Htsre357Skq3tsXH9M/r1h
/VH97p7dk9VieXzstgvkGh9SWEjOD24dlz/9bO8ZV9sLz3+4Y/tEaXndm0dK
dxyyr3G/fnFi2v2hmZkH8X4EQIRi7J9fKB/5LT2k6AgOCZPzrSOxJuEmwa6J
9+SmE4tZ/ZOlbJSoDIa2i8S5uUelL3X2zC35lQP7p+JalnDz84XSHYfsa9wS
Sc0u/wf6R92uYdvgBX1SfqBsj5D/5nt9WbqM4LPq823WSN32I+ebJ7k0de1e
aYO8q/O4RJnEmny9b+9V/dZf912VJyXV467XC/KmaapLvJftEWQXoPsC3Sfq
UYweI7ijFbcvAOJiKDwNleqD8dH/vDgw9tbQZQ2lvi3ntGkqz5Q2yAuFx3FX
mgTyfsrbO/Tmp+4YwY0+uH2B7kzdEMOVybvywwwfoA0MhaehUttP42X4yE3X
Yb6p2FB/YsMx7WSYnX0Yd41pp51jbohBDpfkw3LDB6XjBboLYIwAUTEUnoZK
bbX8N9/rkKjrSdaIOHvmlusw97yfHGVKxwt0F6CfrBs41r2D7Cbkx+bnC/pb
jAugFobC01CpkSjdhCXY5QvtCtDuFx0SpdWXeG7gWHN+186crAMS/roadHWe
0KO2qWv3SvMfKGUoPA2V2jzZYHt7TunQnmzUskXv23tVh/biLg2+kL3//rc/
l5Uk8/JHbw1ddvmv86OGj9x07X9GYVLOUHgaKrUxs7MPr0zePbB/SlJdNtjn
+s7LNvvHXZdopKEuer7D2TO3XPu/q/O4PNzMWG38x10m2sdQeBoqtXaS7Zdy
X0tbvW/LOXnIZnhxYpqZeIicNOndmW7a+Hed/zqfVlr+TP5JKkPhaajUcNI+
lwaVtNs122Urk6hnPgzaTzv/9fw4afnr5J/entPupDBmaiWDofA0VGpFMzMP
xsduy9YkjShpUEm7nS0IHpqbe+ROCnNnXpQ2+3Xon6k+hhgKT0OlOrItvPTr
8VdfuShtpIH+0UMHr8szcRcF1K202a8TPt0JGnJwSk+j5wyFp6FS5XBYNoo9
uyf1dPjOjUfLLgIDWPe3A1Nuqo+e8+vOtib5fWMoPP0vVeNdVvXuzSNykKtj
XpyshJRwV09yyb9j+4TO8JHDWHopY+R/eDp+lior8OuvffzO/muyPmu8y6oe
d1GAF6bv3NfWzq6dOdfPL82e0hO30QZ+hmdFHpY6M/PgZz8dKV50/aisz3GX
A/hOR3jdhZh0Yqf8r2w+dPW0TlThqTcLb+nli/3J+fn5gqyrA/2j8vjT0GV6
ZoDG6MROad7L4bB29WiDXy++F3d1yRFZzucX0z2fzbTuIvU+5Lwch+7be1Xa
IbIqcmEZIHLa4NeLabtLOWk/j54YznzOBkQZnpXuRBWhuHJe16t39l/bsX1C
HrKCcbUQoD300qzaz9O9eaRvy7knu07oXYC5HkjtogvPXKZ4+6l8LpekfhuJ
9Gee/kB74DkrHIjX7OzDv/z5X3oXYI1919qn9RUiqvB03fMdLbmR7KI257ys
NuNjt2VFevk3H9EDD3hIL/2nrf2uzuMD/aNc1rUiHzq9a9S2Ul3CHzp4nalf
gBUzMw/KbtOgQ7rM5CHny0xdu0fCAwkg8a5Dum4mj7T8ZQNP4aZNzjvTd+7v
2D7x1tDlFK4GQOLpBE7ZwHt7TktbTr7Qjv2462oHcv7mjW9ffeXj3xXvyMxI
K5AGs7MPpWGvHft6fQY5hL+U+zqpk6XJ+V88c0ovNdaKFwfgP2ngjY/d3rf3
6kD/qF42XJr6SWr1pTznZSf+wvMfMp0GgNLbAElTX++6Ig1+vd9KofDY7ila
Kcx5/bAm/nFH/i078UheE0AiScLr/Va6Oo/rJfc7DZ6ilcKc1+vmbVh/NCVD
MAAioZfc7/vVue7NIwP9owf2T1m542cKc/7I4RvyYe3ZPRnJqwFIoZmZBxcn
pvftvWriLs9py3n5IOQQzNuPA4A5kicS8j5nfqpyvlB4vGP7BCfHAWiRipk/
N/co3jHcVOW8vueRFAMA4Vzm9/ac7txwTMdwYzkNMz05f2XyLn3yAGLx3rv/
lpx//rnzkvk6hjt17V7b5u2kJOcLhcdyDMUFDQDETsdw3xq63L15ZMf2iZH3
v2j1OVl+5nwuU+EOhM2UKrtOeVebrgsAoiQJLzkvad/VeVwyanzsdisGEEvD
s2K6xkDqKF7Evuz+VM3kPD3zAHxWKDyW5uihg9e3DV7o7TktkXVl8m5UXRAr
4VklXdsvn123dK+S1aU0nPM3b3zb/dTJ4cM3oqgOAFpL4l3apXt2T0rgS+xL
+J/64Mtmpuu48KyWru23vMOpkPNOXS+oJ8B2bjwWaZkA0qWZuZEN/27+m+/H
x25veuJ4AyEWzMxq6dp+kbfnz565JW/RiwPjUVQHIE61BOb0nfu3vvou/HFl
8u7I+1/U9ejqPKGXTJEC6n3o7zb52Li+wcaqh+35VvTPz88XBvpHmW8DRCUk
b6UJWi1dpdEVEqS7duZCorL2PNyxfWLN4N2ze7LenN//9ufy4kNvfrrmTiT4
0N9t+Lq42lht+Nc97J9faMF8mwWm3CCJGu4NmJl5UHsTVzacuhqo2wYvVEvX
4SM3Q4JUFickKrXyJhMvnXycb1NF81NAZUVi9YBRwdy7ODHdvfmkhN4TG45p
TuotsAOBfDwYxXJ4W3sTV9pIkTdQ0U5+zp+vKJJSZdPY+tKFV347wTRLRKha
uzrYbJafDAZpLX0UwZ+RjP3D7z+Rb2Ve/khfZ3zsdjCQC4XHsbwn8Efacl70
9izdKJD1H0Hz84VgVEp+liWzZGygE+NoLc3mXTtzwZyv1kcBRCKFOe+m3/Rt
OSdfT36WN3ovMJSam3t0/uytbVsvDB++oVFZlqUV+zS6N4+UJbM8E/wx+d2y
V5MDw0AnxhE6MeCnFOa8I8kwfOTmxvXF68htOMb1iuMV7HwOT+mybufentNl
097Kkrlin4a5G8ABDUhzzitt3m8b/Pu2wQt6q3d321/9Abs3/21YA4s8O/tQ
k1Mb1e+9W94ALpuzMdA/Gt7/LG3j8JQOdrsxEwOoiJwvpbd6d7f9lSZi8dD+
pBySb6py819v9wINFObOLtEhDDeLI9gdHRwo7NtyrnS6nTSqw+dszMw8aN2y
AyhFzoeYm3skiVSc0nD02d4z2pfrAk37bHVim+wFKnYFtOdE6YpDh08/dbIs
q4OzoKudXfJc3/nSWRzB7uiQgUIa1YBvyPl6uQ4KnYOhE9u2/PJsyNBetenK
4Q/XMK44/3nNocM3Xvu4LKuDs6Djfi8BtIMn4VkLQ6U6rnFb8fTDWk6UHnrz
U+Z/AmiGofA0VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/
DIWnoVIBwB+GwtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoqFQD8YSg8DZUK
AP4wFJ6GSgUAfxgKT0OlAoA/DIWnoVIBwB+GwtNQqQDgD0PhaahUAPCHofA0
VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/DIWnoVIBwB+G
wtNQqQDgj3jDM5eRv9+xLptf/XQ+u06ezuRWP0vOA0AD1gzPuqK4PvLSxRfI
ZQIvv/j6UeZ8OvcRKVxqFjkNUrjICy0NwDqjuC4rv1/h1cn5CKRwqVnkNEjh
Ii+0MgDrjeK6LO9DyPlWSeFSs8hpkMJFXmhlANYbxWEvVKr4Wg205wEADYgw
iuvTyk4hAEBNWhzFqwd5S48eOkq/AQBoHaIYAAAAQFyqzP9PoCpLmuQDr7AP
N6EjPCGLrOe0JO+DDj2FJ4kLvCKK05TSIGx8IVmqLGk+m80tfTtx70DYh1vM
huRtH9UXWRIhcR9wUbVFXt6Pr3QlJ1NC2yuRCp0vlChrLmny3oCQRc5lMrkk
bh9VF3m5bZu8fVv1T3kp4Bc/63hKa48krsdRC53/nyhrLWkCt4aqi6zLmsTt
o9oiu8Z88g7bwlbshB60rZbE9ThqtOeXv5tN3qpSZZFXmrbJ67ut9ikneD0P
a89rcz5pH3IZcr4Gqe+fX1hp7OWyCXsLwj/cRG4f1RZ5Oe0SuNAhi1x8PrED
E0sS+JG2Qlrn27jey5XGbfJWl2ondyxK6PaxxvksKVpkd+CWxGVetnq2XNLH
nAEAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAASIilqyZmsov/5WKCAJBE
i1EvOZ/AW7kAAJYk/q5GAJBmevcH7gEBAImUX+qX1356GvUAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAJT4PyTV5Wc=
    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->45267141]
}, Open  ]],

Cell["Magnify the region near the leading edge.", "ExampleText",
 CellID->16563098],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", 
  RowBox[{"grD", ",", "gSpline", ",", 
   RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
   RowBox[{"Axes", "\[Rule]", "True"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"5", " ", "72"}]}], ",", 
   RowBox[{"PlotRange", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.001"}], ",", ".03"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.016"}], ",", "0.027"}], "}"}]}], "}"}]}]}], 
  "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->133166750],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3U1LHPuewHGHeSXzNgKHw+UwixCYM2eYTBYhXEYH0XsG3ehGN8lGNwqi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    "], {{0, 459}, {360, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{360, 459},
  PlotRange->{{0, 360}, {0, 459}}]], "Output",
 ImageSize->{364, 459},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->31436670]
}, Open  ]],

Cell["\<\
Note that the spline approximation to the airfoil causes small changes in the \
chord and angle of attack.  Locate the point where the slope of the spline is \
infinite.\
\>", "ExampleText",
 CellID->403140097],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"t", "/.", 
   RowBox[{"FindRoot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Apply", "[", 
       RowBox[{"Divide", ",", 
        RowBox[{
         RowBox[{"spline", "'"}], "[", "t", "]"}]}], "]"}], "\[Equal]", 
      "0."}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0.48"}], "}"}], ",", 
     RowBox[{"Evaluated", "\[Rule]", "False"}]}], "]"}]}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->306868033],

Cell[BoxData["0.4948001638596905`"], "Output",
 ImageSize->{46, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->191804981]
}, Open  ]],

Cell["Evaluate the spline for this parameter value.", "ExampleText",
 CellID->223903497],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"x1", ",", "z1"}], "}"}], "=", 
  RowBox[{"spline", "[", "t1", "]"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->396694564],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.0004163470824214804`"}], ",", "0.003159071525059488`"}], 
  "}"}]], "Output",
 ImageSize->{184, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->319216424]
}, Open  ]],

Cell[TextData[{
 "Taking this as the leading edge of the airfoil, compute the increase in \
angle of attack in degrees if the chord is taken as the line from ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox[
     StyleBox["x", "TI"], 
     StyleBox["1", "TR"]], ",", 
    SubscriptBox[
     StyleBox["z", "TI"], "1"]}], "}"}]], "InlineFormula"],
 " to ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{"1", ",", "0"}], "}"}]], "InlineFormula"],
 "."
}], "ExampleText",
 CellID->440778306],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ArcTan", "[", 
   RowBox[{
    RowBox[{"1", "+", "x1"}], ",", "z1"}], "]"}], "/", "Degree"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->469983748],

Cell[BoxData["0.181076253522208`"], "Output",
 ImageSize->{60, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->311998165]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["AirfoilFromDataFile",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["PlotAirfoil",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PlotAirfoil"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->124604311]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["1.2 Airfoils From Data Files",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "MoreAbout",
 CellID->98375866]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"FitAirfoil - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 56, 21.8442000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics", "status" -> "", "summary" -> 
    "FitAirfoil[r_d] fits a b-spline to a collection of vectors specifying \
points on the surface of an airfoil.", "synonyms" -> {}, "title" -> 
    "FitAirfoil", "type" -> "Symbol", "uri" -> "Aerodynamics/ref/FitAirfoil"},
   "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5753, 166, 460, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->59296152]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 36355, 919}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[591, 21, 76, 1, 70, "SymbolColorBar"],
Cell[670, 24, 191, 5, 70, "LinkTrail"],
Cell[864, 31, 3144, 72, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4033, 107, 53, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4089, 110, 731, 20, 70, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[4857, 135, 579, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->14533967],
Cell[5439, 154, 277, 7, 70, "Notes",
 CellID->1067943069]
}, Closed]],
Cell[CellGroupData[{
Cell[5753, 166, 460, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->59296152],
Cell[CellGroupData[{
Cell[6238, 183, 148, 5, 70, "ExampleSection",
 CellID->118571419],
Cell[6389, 190, 130, 6, 70, "ExampleText",
 CellID->7484878],
Cell[6522, 198, 100, 3, 70, "Input",
 CellID->175506032],
Cell[6625, 203, 368, 8, 70, "ExampleText",
 CellID->158695459],
Cell[6996, 213, 217, 8, 70, "Input",
 CellID->128851687],
Cell[7216, 223, 88, 1, 70, "ExampleText",
 CellID->124701399],
Cell[CellGroupData[{
Cell[7329, 228, 134, 4, 70, "Input",
 CellID->63002811],
Cell[7466, 234, 375, 14, 36, "Output",
 CellID->137712546]
}, Open  ]],
Cell[7856, 251, 75, 1, 70, "ExampleText",
 CellID->554708867],
Cell[CellGroupData[{
Cell[7956, 256, 553, 18, 65, "Input",
 CellID->271980122],
Cell[8512, 276, 5842, 103, 201, "Output",
 CellID->60115126]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[14403, 385, 231, 7, 70, "ExampleSection",
 CellID->1305812373],
Cell[14637, 394, 420, 9, 70, "ExampleText",
 CellID->58615840],
Cell[15060, 405, 90, 1, 70, "ExampleText",
 CellID->659725730],
Cell[CellGroupData[{
Cell[15175, 410, 847, 27, 70, "Input",
 CellID->337273913],
Cell[16025, 439, 5895, 104, 70, "Output",
 CellID->45267141]
}, Open  ]],
Cell[21935, 546, 83, 1, 70, "ExampleText",
 CellID->16563098],
Cell[CellGroupData[{
Cell[22043, 551, 631, 19, 70, "Input",
 CellID->133166750],
Cell[22677, 572, 8014, 139, 70, "Output",
 CellID->31436670]
}, Open  ]],
Cell[30706, 714, 220, 5, 70, "ExampleText",
 CellID->403140097],
Cell[CellGroupData[{
Cell[30951, 723, 462, 15, 70, "Input",
 CellID->306868033],
Cell[31416, 740, 176, 5, 36, "Output",
 CellID->191804981]
}, Open  ]],
Cell[31607, 748, 88, 1, 70, "ExampleText",
 CellID->223903497],
Cell[CellGroupData[{
Cell[31720, 753, 175, 6, 70, "Input",
 CellID->396694564],
Cell[31898, 761, 267, 9, 36, "Output",
 CellID->319216424]
}, Open  ]],
Cell[32180, 773, 497, 17, 70, "ExampleText",
 CellID->440778306],
Cell[CellGroupData[{
Cell[32702, 794, 181, 6, 70, "Input",
 CellID->469983748],
Cell[32886, 802, 175, 5, 36, "Output",
 CellID->311998165]
}, Open  ]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[33122, 814, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[33542, 828, 433, 15, 70, "SeeAlso",
 CellID->124604311]
}, Open  ]],
Cell[CellGroupData[{
Cell[34012, 848, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[34434, 862, 180, 4, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[34651, 871, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[35073, 885, 155, 3, 70, "MoreAbout",
 CellID->98375866]
}, Open  ]],
Cell[35243, 891, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

