(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     30275,        907]
NotebookOptionsPosition[     25796,        757]
NotebookOutlinePosition[     27252,        797]
CellTagsIndexPosition[     27167,        792]
WindowTitle->NDiscretizeUnitSegment - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACLET SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.1 Introduction To Airfoil Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilSurfacePoints\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/AirfoilSurfacePoints"], "\<\"PanelPoints\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/ref/PanelPoints"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Airfoil Geometry\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/AirfoilGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/ref/NDiscretizeUnitSegment\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/NDiscretizeUnitSegment"]}, Visible -> 
            False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
NDiscretizeUnitSegment.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/NDiscretizeUnitSegment", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["NDiscretizeUnitSegment", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{
       ButtonBox["NDiscretizeUnitSegment",
        BaseStyle->"Link",
        ButtonData->"paclet:Aerodynamics/ref/NDiscretizeUnitSegment"], "[", 
       StyleBox["n", "TI"], "]"}]], "InlineFormula"],
     " \[LineSeparator]returns a list of ",
     Cell[BoxData[
      RowBox[{
       StyleBox["n", "TI"], "+", "1"}]], "InlineFormula"],
     " numbers in the range ",
     Cell[BoxData[
      RowBox[{"0", "\[LessEqual]", 
       StyleBox["x", "TI"], "\[LessEqual]", "1"}]], "InlineFormula"],
     " dividing the unit segment into ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     " sub-segments."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{
       ButtonBox["NDiscretizeUnitSegment",
        BaseStyle->"Link",
        ButtonData->"paclet:Aerodynamics/ref/NDiscretizeUnitSegment"], "[", 
       RowBox[{
        StyleBox["n", "TI"], ",", 
        SubscriptBox[
         StyleBox["n", "TI"], 
         StyleBox["p", "TI"]]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]returns the list of numbers to precision ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["n", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{"Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {
     Dynamic[
      If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
       RGBColor[0.92, 1, 0.59], None]], {
      Dynamic[
       If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
        RGBColor[0.92, 1, 0.59], None]]}}, "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->209081018],

Cell[TextData[{
 "Use NDiscretizeUnitSegment as a convenience for generating the list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values used to discretize an airfoil."
}], "Notes",
 CellID->1067943069],

Cell["The following options can be given:", "Notes",
 CellID->250107128],

Cell[BoxData[GridBox[{
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["Layout",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/Layout"], 
    ButtonBox["ConstantSpacing",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ConstantSpacing"], Cell[
    "equally spaced points", "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["PanelOffset",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PanelOffset"], "0.", Cell[
    "panel endpoints", "TableText"]}
  }]], "3ColumnTableMod",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {None, {None}}, 
   "RowsIndexed" -> {}},
 GridBoxDividers->{"Rows" -> {{True, True, True}}}},
 CellID->4888558]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->131556767],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->1729252],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->396165825],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->230514364],

Cell["\<\
Divide the unit segment into 10 sub-segments using using the default value of \
Layout.\
\>", "ExampleText",
 CellID->297313774],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NDiscretizeUnitSegment", "[", "10", "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->30360763],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.`", ",", "0.1`", ",", "0.2`", ",", "0.30000000000000004`", ",", "0.4`", 
   ",", "0.5`", ",", "0.6000000000000001`", ",", "0.7000000000000001`", ",", 
   "0.8`", ",", "0.9`", ",", "1.`"}], "}"}]], "Output",
 ImageSize->{345, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->502144524]
}, Open  ]],

Cell["Specify 25 digit precision.", "ExampleText",
 CellID->293570029],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NDiscretizeUnitSegment", "[", 
  RowBox[{"5", ",", "25"}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->431048280],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "0.2`25.", ",", 
   "0.3999999999999999999999999999999999999999999999999999999999`25.", ",", 
   "0.5999999999999999999999999999999999999999999999999999999999`25.", ",", 
   "0.7999999999999999999999999999999999999999999999999999999999`25.", ",", 
   "0.9999999999999999999999999999999999999999999999999999999998`25."}], 
  "}"}]], "Output",
 ImageSize->{408, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->688061241]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Options",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(2)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->2061341341],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Layout",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->1757724783],

Cell[TextData[{
 "Subdivide the unit segment into ",
 Cell[BoxData[
  StyleBox["n", "TI"]], "InlineFormula"],
 " pieces for each value of Layout."
}], "ExampleText",
 CellID->105341754],

Cell["\<\
n = 10;
xCon = NDiscretizeUnitSegment[n, Layout \[Rule] ConstantSpacing];
xCos = NDiscretizeUnitSegment[n, Layout \[Rule] CosineSpacing];
xHCos = NDiscretizeUnitSegment[n, Layout \[Rule] HalfCosineSpacing];\
\>", "Input",
 CellLabel->"In[1]:=",
 CellID->182816592],

Cell["\<\
Illustrate the layout options by graphing the three lists of points generated \
above.  Constant spacing is shown in red, cosine spacing, in blue, and \
half-cosine spacing, in green.\
\>", "ExampleText",
 CellID->68003369],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"ListPlot", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"AspectRatio", "\[Rule]", ".1"}], ",", 
      RowBox[{"ImageSize", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"7", " ", "72"}], "}"}]}], ",", 
      RowBox[{"Ticks", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"Automatic", ",", "None"}], "}"}]}]}], "}"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gxCon", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{"{", 
       RowBox[{"xCon", ",", 
        RowBox[{"Table", "[", 
         RowBox[{"0", ",", 
          RowBox[{"{", 
           RowBox[{"Length", "[", "xCon", "]"}], "}"}]}], "]"}]}], "}"}], 
      "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"PointSize", "[", "Large", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gxCos", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{"{", 
       RowBox[{"xCos", ",", 
        RowBox[{"Table", "[", 
         RowBox[{"0", ",", 
          RowBox[{"{", 
           RowBox[{"Length", "[", "xCos", "]"}], "}"}]}], "]"}]}], "}"}], 
      "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", 
        RowBox[{"PointSize", "[", "Large", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gxHCos", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{"{", 
       RowBox[{"xHCos", ",", 
        RowBox[{"Table", "[", 
         RowBox[{"0", ",", 
          RowBox[{"{", 
           RowBox[{"Length", "[", "xHCos", "]"}], "}"}]}], "]"}]}], "}"}], 
      "]"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Green", ",", 
        RowBox[{"PointSize", "[", "Large", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"GraphicsColumn", "[", 
  RowBox[{"{", 
   RowBox[{"gxCon", ",", "gxCos", ",", "gxHCos"}], "}"}], "]"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->99122243],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     GraphicsBox[
      {RGBColor[1, 0, 0], PointSize[Large], 
       PointBox[{{0., 0.}, {0.1, 0.}, {0.2, 0.}, {0.30000000000000004`, 0.}, {
        0.4, 0.}, {0.5, 0.}, {0.6000000000000001, 0.}, {0.7000000000000001, 
        0.}, {0.8, 0.}, {0.9, 0.}, {1., 0.}}]},
      AspectRatio->0.1,
      Axes->True,
      ImageSize->{504},
      PlotRange->Automatic,
      PlotRangeClipping->True,
      Ticks->{Automatic, None}], {264.6, -27.090000000000003`}, 
     ImageScaled[{0.5, 0.5}], {504, 50.400000000000006`}], InsetBox[
     GraphicsBox[
      {RGBColor[0, 0, 1], PointSize[Large], 
       PointBox[{{0., 0.}, {0.024471741852423234`, 0.}, {0.09549150281252627, 
        0.}, {0.2061073738537635, 0.}, {0.3454915028125263, 0.}, {
        0.49999999999999994`, 0.}, {0.6545084971874737, 0.}, {
        0.7938926261462365, 0.}, {0.9045084971874737, 0.}, {
        0.9755282581475768, 0.}, {1., 0.}}]},
      AspectRatio->0.1,
      Axes->True,
      ImageSize->{504},
      PlotRange->Automatic,
      PlotRangeClipping->True,
      Ticks->{Automatic, None}], {264.6, -81.27000000000001}, 
     ImageScaled[{0.5, 0.5}], {504, 50.400000000000006`}], InsetBox[
     GraphicsBox[
      {RGBColor[0, 1, 0], PointSize[Large], 
       PointBox[{{0., 0.}, {0.01231165940486223, 0.}, {0.04894348370484647, 
        0.}, {0.1089934758116321, 0.}, {0.19098300562505255`, 0.}, {
        0.2928932188134524, 0.}, {0.412214747707527, 0.}, {0.5460095002604533,
         0.}, {0.6909830056250525, 0.}, {0.843565534959769, 0.}, {
        0.9999999999999999, 0.}}]},
      AspectRatio->0.1,
      Axes->True,
      ImageSize->{504},
      PlotRange->Automatic,
      PlotRangeClipping->True,
      Ticks->{Automatic, None}], {264.6, -135.45000000000002`}, 
     ImageScaled[{0.5, 0.5}], {504, 50.400000000000006`}]}, {}},
  ContentSelectable->True,
  ImageSize->{541.2, 174.54000000000002`},
  PlotRangePadding->{6, 5}]], "Output",
 ImageSize->{545, 175},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->768585128]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "PanelOffset",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->1295379749],

Cell["\<\
Points interior to the sub-segments are specified by the option PanelOffset.  \
Locate points at the middle of each segment.\
\>", "ExampleText",
 CellID->914405104],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NDiscretizeUnitSegment", "[", 
  RowBox[{"n", ",", 
   RowBox[{"PanelOffset", "\[Rule]", "0.5"}]}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->46504520],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05`", ",", "0.15000000000000002`", ",", "0.25`", ",", 
   "0.35000000000000003`", ",", "0.45`", ",", "0.55`", ",", 
   "0.6500000000000001`", ",", "0.75`", ",", "0.8500000000000001`", ",", 
   "0.95`"}], "}"}]], "Output",
 ImageSize->{397, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->15263648]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Applications",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->258228157],

Cell["\<\
Compute two collections of vectors, one locating panel end points for a \
discretized NACA 4415 airfoil, and the other locating collocation points at \
mid-panel.  Plot the two collections of points, joining the panel endpoints \
to show the discretized airfoil.  The resulting graph illustrates a \
discretized with collocation points at mid-panel.\
\>", "ExampleText",
 CellID->21363168],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4415", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"5", ",", 
        RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rC", "=", 
   RowBox[{"PanelPoints", "[", "rP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"grP", "=", 
   RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"grC", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rC", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"grP", ",", "grC"}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->333885958],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3e9rFdkdx/GU/iX9NwLd1QYfpGmR2BUfiE9iuavQruyisBufKAUtJcJF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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->144119235]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["AirfoilSurfacePoints",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["PanelPoints",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PanelPoints"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->539278887]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["1.1 Introduction To Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]], "Tutorials",
 CellID->341631938]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "MoreAbout",
 CellID->586699732]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"NDiscretizeUnitSegment - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 57, 46.5834000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics", "status" -> "", "summary" -> 
    "NDiscretizeUnitSegment[n] returns a list of n + 1 numbers in the range 0 \
<= x <= 1 dividing the unit segment into n sub-segments. \
NDiscretizeUnitSegment[n, n_p] returns the list of numbers to precision n_p.",
     "synonyms" -> {}, "title" -> "NDiscretizeUnitSegment", "type" -> 
    "Symbol", "uri" -> "Aerodynamics/ref/NDiscretizeUnitSegment"}, 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[7625, 220, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->131556767]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 27024, 785}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[603, 21, 76, 1, 70, "SymbolColorBar"],
Cell[682, 24, 191, 5, 70, "LinkTrail"],
Cell[876, 31, 3233, 74, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4134, 109, 65, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4202, 112, 1701, 48, 70, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[5940, 165, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->209081018],
Cell[6523, 184, 224, 6, 70, "Notes",
 CellID->1067943069],
Cell[6750, 192, 72, 1, 70, "Notes",
 CellID->250107128],
Cell[6825, 195, 763, 20, 70, "3ColumnTableMod",
 CellID->4888558]
}, Closed]],
Cell[CellGroupData[{
Cell[7625, 220, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->131556767],
Cell[CellGroupData[{
Cell[8111, 237, 146, 5, 70, "ExampleSection",
 CellID->1729252],
Cell[8260, 244, 132, 6, 70, "ExampleText",
 CellID->396165825],
Cell[8395, 252, 100, 3, 70, "Input",
 CellID->230514364],
Cell[8498, 257, 138, 4, 70, "ExampleText",
 CellID->297313774],
Cell[CellGroupData[{
Cell[8661, 265, 118, 3, 70, "Input",
 CellID->30360763],
Cell[8782, 270, 382, 10, 36, "Output",
 CellID->502144524]
}, Open  ]],
Cell[9179, 283, 70, 1, 70, "ExampleText",
 CellID->293570029],
Cell[CellGroupData[{
Cell[9274, 288, 142, 4, 70, "Input",
 CellID->431048280],
Cell[9419, 294, 524, 13, 71, "Output",
 CellID->688061241]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[9992, 313, 223, 7, 70, "ExampleSection",
 CellID->2061341341],
Cell[CellGroupData[{
Cell[10240, 324, 228, 7, 70, "ExampleSubsection",
 CellID->1757724783],
Cell[10471, 333, 185, 6, 70, "ExampleText",
 CellID->105341754],
Cell[10659, 341, 274, 7, 70, "Input",
 CellID->182816592],
Cell[10936, 350, 233, 5, 70, "ExampleText",
 CellID->68003369],
Cell[CellGroupData[{
Cell[11194, 359, 2193, 69, 70, "Input",
 CellID->99122243],
Cell[13390, 430, 2092, 49, 196, "Output",
 CellID->768585128]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[15531, 485, 233, 7, 70, "ExampleSubsection",
 CellID->1295379749],
Cell[15767, 494, 175, 4, 70, "ExampleText",
 CellID->914405104],
Cell[CellGroupData[{
Cell[15967, 502, 182, 5, 70, "Input",
 CellID->46504520],
Cell[16152, 509, 396, 11, 36, "Output",
 CellID->15263648]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[16609, 527, 227, 7, 70, "ExampleSection",
 CellID->258228157],
Cell[16839, 536, 399, 7, 70, "ExampleText",
 CellID->21363168],
Cell[CellGroupData[{
Cell[17263, 547, 861, 25, 70, "Input",
 CellID->333885958],
Cell[18127, 574, 5441, 97, 70, "Output",
 CellID->144119235]
}, Open  ]]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[23629, 678, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[24049, 692, 435, 15, 70, "SeeAlso",
 CellID->539278887]
}, Open  ]],
Cell[CellGroupData[{
Cell[24521, 712, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[24943, 726, 196, 4, 70, "Tutorials",
 CellID->341631938]
}, Open  ]],
Cell[CellGroupData[{
Cell[25176, 735, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[25598, 749, 156, 3, 70, "MoreAbout",
 CellID->586699732]
}, Open  ]],
Cell[25769, 755, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

