(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    383788,       7644]
NotebookOptionsPosition[    372036,       7252]
NotebookOutlinePosition[    373807,       7296]
CellTagsIndexPosition[    373720,       7291]
WindowTitle->PMDoubletConstantDirichlet - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACLET SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.1 Introduction To Panel Methods\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"], "\<\"3.2 \
Validation Example\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ValidationExample"], "\<\"3.3 \
Implementation\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Implementation"], "\<\"3.5 Convergence \
and Timing\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"], "\<\"Table of \
Contents\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PMVortexLinearNeumann\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMVortexLinearNeumann"], \
"\<\"PMDoubletQuadraticDirichlet\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"], \
"\<\"ForceAndMoment\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ForceAndMoment"], \
"\<\"PlotSurfacePressureCoefficient\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/ref/PMDoubletConstantDirichlet\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/PMDoubletConstantDirichlet"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
PMDoubletConstantDirichlet.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/PMDoubletConstantDirichlet", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["PMDoubletConstantDirichlet", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{
       ButtonBox["PMDoubletConstantDirichlet",
        BaseStyle->"Link",
        ButtonData->"paclet:Aerodynamics/ref/PMDoubletConstantDirichlet"], 
       "[", 
       RowBox[{
        SubscriptBox[
         StyleBox["r", "TI"], 
         StyleBox["p", "TI"]], ",", 
        StyleBox["\[Alpha]", "TR"]}], "]"}]], "InlineFormula"],
     " \[LineSeparator]implements a panel method for uniform flow over an \
airfoil using doublet panels of constant strength and internal Dirichlet \
boundary conditions.  Panel endpoints describing the discretized airfoil are \
located by ",
     Cell[BoxData[
      SubscriptBox[
       StyleBox["r", "TI"], 
       StyleBox["p", "TI"]]], "InlineFormula"],
     ", and the angle of attack is ",
     Cell[BoxData[
      StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
     ".  The function returns a list of strength parameters for the doublet \
panels, the strength parameter for the wake panel, locations of collocation \
points and a list of surface speeds at collocation points."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{
      Dynamic[
       If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
        RGBColor[0.92, 1, 0.59], None]]}}, "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->149220581],

Cell[TextData[{
 "Collections of vectors are of the data type ",
 Cell[BoxData[
  ButtonBox["CartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/CartesianVectors"]], "InlineFormula"],
 "."
}], "Notes",
 CellID->1067943069],

Cell[TextData[{
 "The discretized airfoil is specified by ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " straight-line panels, where the doublet density for each panel is \
constant.\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["\[Mu]", "TR"], 
    StyleBox["i", "TI"]], "=", 
   SubscriptBox[
    StyleBox["a", "TI"], 
    StyleBox["i", "TI"]]}]], "InlineFormula"],
 "   for  ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["i", "TI"], "=", "1"}], ",", "2", ",", "3", ",", 
   RowBox[{"...", 
    StyleBox["N", "TI"]}]}]], "InlineFormula"],
 "\t\t\t\t\t\t"
}], "Notes",
 CellID->275911084],

Cell[TextData[{
 "There is one unknown strength parameter for each airfoil panel for a total \
of ",
 Cell[BoxData[
  StyleBox["N", "TI"]], "InlineFormula"],
 " unknowns.  Strength parameters for the discretized airfoil are grouped as \
a list.\t\t\t\t\t\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["a", "TI"], "=", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["1", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["2", "TR"]], ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["3", "TR"]], ",", "...", ",", 
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["N", "TI"]]}], ")"}]}]], "InlineFormula"]
}], "Notes",
 CellID->6298250],

Cell[TextData[{
 "Additionally, a doublet panel of unknown strength ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 " extends from the trailing edge of the airfoil to a point far downstream on \
the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis.  This panel is used to satisfy the Kutta condition."
}], "Notes",
 CellID->252999978],

Cell[TextData[{
 "Boundary conditions at collocation points provide ",
 Cell[BoxData[
  StyleBox["N", "TI"]], "InlineFormula"],
 " equations, and the Kutta condition (zero circulation at trailing edge) \
provides an additional equation for a system of ",
 Cell[BoxData[
  RowBox[{
   StyleBox["N", "TI"], "+", "1"}]], "InlineFormula"],
 " linear algebraic equations.  The system of equations is solved by the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function LinearSolve."
}], "Notes",
 CellID->631099542],

Cell[TextData[{
 "Output of PMDoubletConstantDirichlet is a list of strength parameters, ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", strength of the wake panel, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 ", a collection of vectors locating collocation points, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 ", and a list of surface speeds just outside the airfoil near collocation \
points, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["v", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 "."
}], "Notes",
 CellID->86481803],

Cell[TextData[{
 "If a list of angles of attack is input instead of a single angle of attack, \
nested lists of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["v", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 " are returned."
}], "Notes",
 CellID->490569405],

Cell[TextData[{
 "The same example airfoils and computations are used in the documentation \
for ",
 ButtonBox["PMDoubletQuadraticDirichlet",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"],
 " and ",
 ButtonBox["PMVortexLinearNeumann",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"],
 ", providing a convenient way to compare results among the three \
panel-method functions in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Notes",
 CellID->509195412],

Cell["The following option can be given:", "Notes",
 CellID->75751524],

Cell[BoxData[GridBox[{
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["Epsilon",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/Epsilon"], 
    SuperscriptBox["10.", 
     RowBox[{"-", "10"}]], Cell[
    "offset collocation points to inside airfoil", "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["InterpolationOrder",
     BaseStyle->"Link",
     ButtonData->"paclet:ref/InterpolationOrder"], "1", Cell[
    "degree of polynomial used for interpolation", "TableText"]},
   {Cell["      ", "TableRowIcon"], 
    ButtonBox["xWake",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/XWake"], 
    SuperscriptBox["10.", "10"], Cell[
    "large number used to locate endpoint of wake panel", "TableText"]}
  }]], "3ColumnTableMod",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, 
   "Rows" -> {None, None, {None}}, "RowsIndexed" -> {}},
 GridBoxDividers->{"Rows" -> {{True, True, True, True}}}},
 CellID->14117100]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->277908699],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->357770921],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->77267041],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->326492779],

Cell[TextData[{
 "This example uses the Eppler-205 airfoil specified by a data file located \
in the folder Airfoil Geometry.  Use AirfoilFromDataFile to import the data \
and arrange it in the form used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "ExampleText",
 CellID->48545141],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"NumberOfVectors", "[", "rD", "]"}], ",", 
   RowBox[{"Head", "[", "rD", "]"}]}], "}"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->317289154],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"61", ",", "CartesianVectors"}], "}"}]], "Output",
 ImageSize->{155, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->335608642]
}, Open  ]],

Cell["\<\
The collection of 61 vectors of the data type CartesianVectors locates \
endpoints of 60 connected panels. Generate graphics to plot the discretized \
airfoil.  This plot will be incorporated in a subsequent graphic.\
\>", "ExampleText",
 CellID->167522],

Cell[BoxData[
 RowBox[{
  RowBox[{"grD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->493524749],

Cell["Specify an angle of attack for the airfoil.", "ExampleText",
 CellID->20057380],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"6.", " ", "Degree"}]}], ";"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->163234459],

Cell[TextData[{
 "Solve the model.  Output consists of the list of strength parameters ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", the strength of the wake panel, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["w", "TI"]]], "InlineFormula"],
 ", locations of collocation points, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 " and the fluid speed associated with collocation points, but just outside \
the airfoil ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["v", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 "."
}], "ExampleText",
 CellID->442126641],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "vC"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->53812608],

Cell[TextData[{
 "Flow visualization provides a good plausibility check for model results.  \
The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ContourPlot can be used to plot streamlines for the flow field.  \
Specify an arbitrary field point and compute the steam function at the point \
as the superposition of the stream function for uniform flow and the stream \
function due to the collection of doublet panels describing the discretized \
airfoil and wake."
}], "ExampleText",
 CellID->205450438],

Cell["\<\
Although the strength of the wake panel is included in the output of \
PMDoubletConstantDirichlet, the location of the endpoint of the wake panel is \
the value of the option XWake for the function.  The default value was used \
above to solve the model.\
\>", "ExampleText",
 CellID->99130043],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"xW", "=", 
  RowBox[{"XWake", "/.", 
   RowBox[{"Options", "[", 
    RowBox[{"PMDoubletConstantDirichlet", ",", "XWake"}], "]"}]}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->544103304],

Cell[BoxData["1.`*^10"], "Output",
 ImageSize->{54, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->412113092]
}, Open  ]],

Cell["\<\
Append the wake panel to the airfoil panels and the strength of the wake \
panel to the list of strength parameters for the airfoil panels.\
\>", "ExampleText",
 CellID->33026124],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"a", ",", "aW"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->81885161],

Cell[TextData[{
 "Compute the stream function at the location of an arbitrary field point ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["f", "TI"]]], "InlineFormula"],
 ".  Since the primary focus of this software is numerical computations, the \
standard form for output is a list of results.  In this case, the result is a \
list of one lengthy function of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 ".  Remove the extraneous list brackets to obtain the stream function. "
}], "ExampleText",
 CellID->19233943],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rF", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "x", "}"}], ",", 
      RowBox[{"{", "z", "}"}]}], "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Psi]A", "=", 
   RowBox[{"IC\[Psi]DoubletConstant", "[", 
    RowBox[{"rF", ",", "rAll"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", "\[Psi]", "}"}], "=", 
   RowBox[{
    RowBox[{"UniformFlow\[Psi]", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "aAll"}]}]}], ";"}]}], "Input",
 CellLabel->"In[8]:=",
 CellID->133172956],

Cell["\<\
Generate the graphics for a streamline plot for flow over the airfoil.  Some \
experimentation with option values were made to obtain the plot below.  The \
number of contours plotted was used to avoid a streamline very near the \
airfoil, PerformanceGoal->\"Speed\" was used to obtain a contour plot in \
reasonable time, and MaxRecursion was set at a minimum value to avoid \
noticeable numerical artifacts in computed streamlines.\
\>", "ExampleText",
 CellID->76299467],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", ".5"}], ",", "1.5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", 
       RowBox[{"-", "0.4"}], ",", "0.4"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "18"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "grD", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[9]:=",
 CellID->808226483],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJx0vQmcj+X3/6+04hNpL9GiSMQnKUpdfYq0F0VKlrSXJaWFFipLRVSyJEtI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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5005, 5748, 5731, 5746, 5747, 5091, 6069, 6435, 6434, 6768, 
          5568, 5570, 7523, 5569, 4851, 6245, 5688, 4921, 6066, 4922, 7107, 
          5040, 7106, 5039, 6942, 6941, 6940, 7086, 7085, 7349, 5988, 6994, 
          6993, 6992, 5531, 6249, 4857, 5834, 7125, 5095, 6436, 5096, 5754, 
          5753, 6095, 6094}]},
        RowBox[{"-", "1.4500000000000002`"}]],
       Annotation[#, -1.4500000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5008, 5749, 5732, 6391, 6392, 6390, 6389, 6091, 4939, 6395, 
          6396, 6393, 5006, 6394, 4776, 5623, 5300, 6229, 6230, 6231, 5792, 
          5943, 5944, 5378, 6058, 6059, 5649, 4856, 5969, 5833, 5744, 5745, 
          5572, 5640, 4835, 6412, 5019, 6214, 6213, 6212, 5000, 5717, 5716, 
          6084, 6083}]},
        RowBox[{"-", "1.5`"}]],
       Annotation[#, -1.5, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5799, 5693, 4844, 6335, 6336, 6334, 6400, 6399, 6799, 7487, 
          5007, 7539, 6398, 7538, 6397, 6092, 4940, 6403, 6404, 6401, 5009, 
          6402, 4777, 5624, 5301, 6232, 6233, 6234, 5794, 6678, 6679, 6330, 
          4976, 6166, 6167, 6606, 6607, 6221, 6910, 6911, 5791, 6220, 6219, 
          6218, 5292, 5719, 5718, 6370, 4999, 6369, 6372, 6371, 4934, 6283, 
          4933, 6284, 6374, 6373, 4770, 7263, 5289, 6603, 5290, 6604, 5277, 
          5938, 6215}]},
        RowBox[{"-", "1.55`"}]],
       Annotation[#, -1.55, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0btLFmAUx/HjINnktXxfL5XU5tRf4KK4pIjgEF4SQbGliGqyocFJN0Er
qCHCC4WO5aUEQY2yMnPV0fS9WgotCvlx+PI7l985z8Pz1PXdb79XEBFjOK6I
eFQT8Z0+pD/oY5orj8hjoCxi6HLEE9y4FHEdA5UR/9UHxac0zddcG5GiG3wn
avXyb3b9SzoDy+K/9BM9op/53vHVJSKu4Q56kFM71P+DHD7y5+k6/1X9KxiR
N6m3OqNbfrE6Iqu2pJamizRD18zU6tdgWN6o3mKmS15kJqW2oLZP5+kBXTVT
rV+Fp/IG9VtmOuUXzGTc77n4BcbxDBMo5H2r1+ft3pv5Lf9Ap+ybRKIqIokl
3gfeKUGT+MW3jTZx2P+ad1N/S23O/E86S3vtva2X4kvjAPv44i+az/9KfIjM
+Rsiizfu02Nu2vxXe2boKzteosxdiv1fKS2h5fSuPSPuMIo587PYteOmM3bo
Kc8JOvj37O3nb7NrRf8MKsZj8Q==
          "]]},
        RowBox[{"-", "1.6`"}]],
       Annotation[#, -1.6, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LtPVGEQhvFZ7KBilwUBIbHijwCUSwMdLCFaEZdLYigA6dUocl3AUpCE
BjsIS2ejiSTcxNgABVSAMdTgpUERfxRPnm/O986cOedu92BmIBERb/AqFXF4
O2KUj7iHE7yHmqqI+WTEvvNbbr8TkUEbFkoiOniO18vd8yee50xpxGNzNtSL
6vv6G9CIBXfHZl1VRDSpt2Q2UeRciH59eU67L8Gi/Hf5Vs9asGzeV/lf6Yjf
mJL/4XmOf95keLcs4ty51/5ZvRfOXeZk7ZvXf6B/jd95toQzmQ8yHzFbGTGD
YXP69U/YYRLP9NTqecp1/JyPZf7ov6fOemc9X6qn5F+6b1C/4EYe4RP5K/fN
6m75Jv6rzslPY0xmHKOYxARO9YRsAlW+91r+Hx7Yedr9DHJ4jVl8ky+QvYVq
+TLfsMJpXuZSfqg3718Uq1c5yWu8bZ8h/swDvMODfCk/zLvqJ/yF+/Q8MquX
U/ZJ4r13ptSdvMP/Af54Xpw=
          "]]},
        RowBox[{"-", "1.6500000000000001`"}]],
       Annotation[#, -1.6500000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwN0D1Pk2EUh/FTRKZitKWKtonxY+jKBAONWBMYHCTO8gF0YJCBxDcEJxlo
i5q4KPIFShugRnRDa1+UFxtfZsNAEP0NV67n/M+5z33nuTQ5NXY7ERGz+J2K
mB+MWMBjPMEcXmcjXurVLkRU0ac+iV+y7+cjutjFPvZQOxvxQm/NbAW9Zk/g
p+yb/g7a+IoOquaf62XcNYACruGHrKnfQgOrAxFfeM18Wj+Fu+or8n/2H2NM
1sOf5W/l27zCn7ji3Bn907ijviz/a/YIV2UJfsQj6YgHPMwP+T7/4WXvOcj4
V+pRdR5d2Q17L9r3Rr2CJPpxCiV31PXK/I63nN3kono8FzGBhH8UmPbddqaD
Fpqon4sYcqbsbctYQglFPHN3wZnrqNq5aGeN6+b77Ut6a8X5Gd95b7wlz+lv
cJbX+b09T/Uadh2ae+XOSfk9+U3+wB+xhU31f2RVXYg=
          "]]},
        RowBox[{"-", "1.7000000000000002`"}]],
       Annotation[#, -1.7000000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ctLlWEUxeGdOEgkA2+pqeSscSpWXgiicSgWYYp6Ch1oOu0iUZBHAydd
MfHSLA1LLE2bFAQNKkotDXRwKv+FBl6a9Dj4sd717r3Xfj++skRvQ8++iBjH
+eyI9vyIBL7kRKwejvhMR/hNtUH+LvoxgCRaiiPa1H7mRbxFxaGIcqTM7ar/
wxZ2sI1m/a361/RmYZo/QF/QJ/b8UVsskIV5LOANzrlPFtmL1/yQrF475pwz
ndvM9vCzdl+hN0vscfeRH5U9hkv6LqNC/SlfSaf455jEIz6jMGI/Dsrt5l/R
r/SIu2+0jKZ7Q6f8Y/yEnHKaMN+O93i3t8dcs/1dai+9YQYX+SY0+pYLtIF+
cF8qswSf9J6R22f2lIwbdDM34o4daXae5pN6ftNdetRMyvkXalGHlMzb+pfN
r2AJP/AdHWbu2ddJ6+2v0b+h/7HzOq3mT+I6riFHX425q8713nXfW87K3vaW
HTw0d0LtOKrwQP2vfzksa8zcLfOjdALje/+If6ZvCn3yJukq/gNjQWOb
          "]]},
        RowBox[{"-", "1.75`"}]],
       Annotation[#, -1.75, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ttPz3Ecx/F3QsjUKsovLoxqzOkmJtSFs2LjkrvYXNgc5lAySrUVV06L
zZlVjAubCy30Y/wBzIUJFTPnzWEzYQ6P38Vzr8/7/Xp/Xp/P5/v7TareunZL
WkT0Yn9OxOtxEfX0SiLiMiIv4lVhRNnYiCnq7RP5uRGn9NLVLRMiWlFfELFI
r4EuNvPATGJ8xO38iDtYLXeN3GZ7kuoac7uRY66P14/nzrrI++isT/hn/1/U
2VfEL0aTfpWMVViBSqxEnpxGXpqz0zEEDeplvOVYggPqpXSTnA7rXDOdNI8O
Ovukd/yibfQnPUETcgesm6xf0gr3K0eFt5ZjIfr1G/mlspK0jr8X7epq523E
BhSmspz9wp4+TE29T28abafNvE5zHZhr731Zf/Svy8rnFaDMeh4eWj9CLY6b
6dbLtL4ptwtFKEYG7y5t9a0PoiX1++gdoiX68+177P5dNGF/IbY5e447lGKz
uUH3+4HRvG57bmE6ZiBT1r1UDm8BLqlnqmdhNrJ92yxUye/lP8MN+ZPpKLP7
Ut9c9m/0mn+K7/ol6jZ6mP/F/Sr1q1Ap57P6mP43eoR+pUfpgP/JW/oOb/AB
79HBX0fb6Xp62twT7zpD0+kQXMNVjPHmHc7eiaHqHvVZcz3ue44O1xuW+u/y
dyGLX0NrkaGfVJ83lzR/gY7UG4E6/h5k8/8DDZyANg==
          "]]},
        RowBox[{"-", "1.8`"}]],
       Annotation[#, -1.8, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV00lsTlEUwPGLEl0Qc1HVlqDVyayhkhojLNCaac3sEBoENduK0lJCKFZs
0BISNS1NRT+Cjxi7NESiJNrwe4t//++ce8659773NX3lhqL1bUIISf7M6BbC
zF4hTO8Xwv4eIdzoE8IB/iz3CSN6hzAcm/qGcFA+IymEqZiCJXIVPUNYzEd4
KY/nAgxKCaHc7INm7OQPySF8xHv86x5Cgv3ao1Btolkd8Vxtjtwq+2Tzas7l
FvVdra8RX+Dven5grfgoj7ZXippp1vrzGHGTtUzzqqzni9Pkq6yn8lhxpfxP
3MJ0cYL6OuvtuUlfm8jO2i56F9yWv3AJ17jnWZzBH/Pi8r95gLMORCrSkRbd
z6wMzsQQLNM/lP+q72K/5eLz/NU5vmEM7lpfIZ/FK/kBP8Qi51yICWoG27vc
8yCO6c/23bKQLv5oz0nONBHlvt0h9RPVFOK4tWqcNXe+mWd4AV/2jRrkK8w4
gsOYGb1fLjGzk706Y56Z85Hp/NXmZfBccTHKrNeIb3rOwDnPuXKJ3GpWR84R
x6LviO2ed2AbZok7OMdsjjlHI07Ye464mov4JJc6y2n3Ga8ny7wTzjCUx4lP
ye/Vtye6o7rjuG1mjnw9Z3MDP8UjPMFj7HH3Z7yXL+m5iA7Rb0/9AHM3y+8z
s4wHivPky+y7GSPcczg2eM9b5SfrjevdZd5utDpzC17L1ap/wwX8iv9Y36Ln
F/9GM4rtUWyvl+btc68XvJ+v2eM6rqIOtYirvWfOWn33eR3H1dbrTzb3irgf
X+VK68XOfjt6lzxS/MuMUbxR7q2+POeqEpdE/4v6GnFMfMf6Vj1jPTfryedN
cu/0DIt+U+Jl0W8ZMT2lzlXq+T9OqaA7
          "]]},
        RowBox[{"-", "1.85`"}]],
       Annotation[#, -1.85, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV00dsj3Ecx/EfqkRLjdasWm2NQ43WKAkxDirU1hqxDqXrX+OAoCctYkZt
iZW2TiQkdh3sPSoRnDki1sVoeT2Hdz+/7+c7ft/n6f/pt7JiTqxFCCHLn4SU
EH51DWFUcgilPUP4102iewgNvFviBjTzRsqv7RFCO/UVdIT4D/8vmnBD/U21
N/BbPFw+pu41Hc0bgzJz03uFENclhLfq3zl/d85Q94Nm0iG8wXijb1NqCNdw
FeVmLuQXogAn7dGpN9/MGN6Z9x4D+f34FZ1DqBTH6LfoHnzFT/xAlvnl7htG
y+hQ2tk9XfDYrk9w3Nx8exWpX4WO6pLQ4Hze3BZqp+oLNI9OcPcid4+nF+Q/
8+aJv9Bp9s9Dunm5vE+8Guc6s86hHvHiXWpa0/52f+280ayh6tvzdos70GHi
Tfx0NYni5XY9LV5Gz9AV9BTNkc9GEfby9uChGRfkSpx70kSzjtEEepR+t/dY
uXXOuXQ9HUfXRu/VnL4oUdNozhZetr1zMBJ7eWPoaIxCs/fShCn613iXtXZt
7/2liXtjg/ol6hZjEc6al6HusvmJcoOcByIT072v22bdQa3eOhwyK59/kB7B
YcwQF+stwQPzVtNVKDZzpp5ZyMcczMZ2NfGevQ1aY4e4uz3nyc3HXBRgAXbK
DYp+Y9hsXjkvFv2ucTP6LqLfP23jvni0smtLnLBXob1SnAtoMq1GFT5E35W+
j3SSnkrPmur+tna5L/8A9/AID1EvN0B9rnv6R98Vfczfoncr9osL+fvoQjqZ
1w4JuM/7i0vRtyG+SP+Ib6FMXIorZqUjAy/0v8QddUlyd6P/u/gVlto3U803
vdUYbt8Uux1QV4MqXhxvG+3g+Z95rud4ipd4gUnqq+hEWk0b8Qo17smL3pE7
ruvdTvuYmYYc+Wz8BzIUo+g=
          "]]},
        RowBox[{"-", "1.9000000000000001`"}]],
       Annotation[#, -1.9000000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV01tM1nUYwPGf3XiVicLL++Kx5QrcNIVwM9OltVnKKedh5qEFoiQbeADU
Gpp5hxFYtBagbnksSy90U8BDbSVONy+yRFNx09QrTVti2PHzu/jy/T/P73me
3/N/ed+nS6vnVA0IIczx59jQEIamQujgdF7AvcNDuI7bWSHcwdkhIYzIDOEc
70mGsBdbhoXwAd5Sn6nvFGfwSU7ww/QQWszo4095o3yr+npeNSKE7ebuwDXn
ueKE/FR3ZHKeuFd+YCKEAr0Nnu+rfYDf8Yn8WbO+VZt013ec4k40qu2x5798
kTc7W4t39OzS+wX6xVf11/Az3mUMjqhN03+Yc9XkYSKqM0J4gau4wX5PxLvM
aueX43vwYDQ6e1fN3/Z9j/v5LzxGufoU15rdatbn6NNzTf4quuTPyVWYVxJ3
tdvH8T2xTnzdvF7sdMeA+PnqybFzY9xNT0quxVmD+EOUym3lenUrURnrxR12
yOZOrlJ/zL2XPRfLXeISfl9+EzqdFYln4Ednr/AFns49/Cr/xBfxMzaor5d7
290b415Y4DmP15uXZsfbGCVXprbCTq85Wyx+nWdiNpaIZ/FN3MB8/8dsuQ49
g8yZJy6UXypXwJfj7vgFbzorkV/EC5FpryQSWCxeZ8YW99bxGHW5eurMrMVT
divEaPllzms9T/Ncx8/hMzXP8lj9X8Z30XsLbyBH3Yn4XVNzPP5W3DcSWRiO
YTiv75CzH/xPv8cs+/waPxu5r8ybYcYBHh/3MGcc/6HmipoVapp8Jyp4m97m
+F0RF6sp50l8EO3ybaiUW4l/1H9t1jfIcH86jso9KT6jrhunMdc9aXac4K6J
2GfWdPvsj78PtKlvR7ezIv1LkB9/T+Ih6nZzs/MmTHHvi5iM4Ow/c9f4rq7G
82Zt5tHuHIUN8bMX12AtNpmzRm41qrEqnsu9pLaGp/L9eLf8Ay7TU4pydyzD
Q7/ffPFdd9/Db1gu16f+ETrsfFz8kfft4iY+wY/MnaRvtn3HyVXK/am+H116
xnOz/D7exkXxf6onZacsTvL/PFbDdA==
          "]]},
        RowBox[{"-", "1.9500000000000002`"}]],
       Annotation[#, -1.9500000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0VtMz2Ecx/FvZBRhTh1l3CUUF7okGzZmGeaC6sJhHXTY3MZNNkS4cbxg
XDC2NmaUzZxuHLbaHC5M2Eoh1YQibA6v/8V779/n+3yf5/k+///cbfUb6pIi
4iUapkecyI44hZPYLH9Ij3iYEfEIT/AYzdMiVuRE7JkRUcx7OUdfNt5nRfSi
Bw3qy61vcc5p551Bv/oA2p0xyB28Vs9BvWu4iXu4F114h24803eEj6IZA/qO
cR8f5n5u4k98iEe5f6aMC+YqNMPSWRHlZihDi3yH09SuWC+Si3xXeF8l9qsd
QLn9BeYs40JuMUey+jisdkfJ7IhVfFy9zf40d0/GJEzFFLRa+2P/X1xFF2rU
kvTXcrd8DSH/42X8i3/jnHvO4o3vt4kec6ZY/2zfZfUhTpXnmXkM35XPq3/U
24cH8kX5EpKtj8V9tUb5Hnck/gu8Nmcnnqq36xlwz23fg9whv+J05+V6b6l9
xWor5a2J9/NGv0Gq/RMxHimYgF166/TWY5O+YXeN4Ll6nvNfcK61ObhufTbn
YJ16Pi/ADfVFvDCRzbGYl+Bm4j/hAuzWP+y+GrVvXMsjXMVfE3PwEFfzFy7R
n2n2NjmDb3EWt/IP531HnjftyIzYie3YZ+5GrLe3Uq5CBebrq+ZOs/1MzCiX
els+jyb+E9//AXT7hQc=
          "]]},
        RowBox[{"-", "2"}]],
       Annotation[#, -2, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DtIlWEYwPFH3BIyvOs5Qg45nKNrJIRWVg7RkhFKg3kcM8tF8xxRoSXI
oIsauWSuTlHYRSlSjKYyTB11KKghMkijzd8Z/vzf5/q+31eXuX6hryAi5rBe
EpGsjXhbHpHgd1zLz7ixMqIBryoiHul7XeOMPeziq1xKPY3xsojzyYg7PCU/
b2aSX+Z71TPVEd34jxa5E2jGKZxE2n0L5hcxKZ5Cv/lRe27whHjNnpS+pPPf
0ohhvZvuy/IG57hJ/yCvigf4C9/k3/o31HbyTkT84U2e4Xo7L/nmI7yup1Cu
QfzZ7HHnD5wS//OOj+rf5M6Kv/NjHPBN01zECTv39B311l2uEVfjmjf0Yg7t
ahex7L55LrbzEHrEVXorUYEV917Vf1DtF2fUy+TLcdtcl7gbXd5yhc+5b8k7
WsXLvIBb+t7wfTvu4ZOep1URs8jZk8Vl9SEeMjfizlZ3/fCPf+Kw+TqMyZ+W
H+UzfNeeB/n5/G6ewEN0ip/Y3cEzPGZHm942dw8755DFC/UV/c95XG1bbgt9
9h+zP202hSW8R4l/uw/8JWeE
          "]]},
        RowBox[{"-", "2.0500000000000003`"}]],
       Annotation[#, -2.0500000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0L1L1mEUxvEjkZBtz2OWWailvRCCW6BWCmHPoKUOLmVETpov2Oy7bYmK
g4aDUTpFVBZuig5i5KLi4lKrpVRY/gF+fsOX6z7Xuc79Vvy0u6krKyJGUZOK
OHEm4s2FiLdow1puRDtdoa20Wf+Alp2PeHIuYpV/lB9xXX0NqbMRaUzLnboY
MUVzaIb3Mi/iqzN+yH1Hp9mb9jrJz0aPulL9nFbRxXTEeoEZ9Mn34675jGyG
1jv3PupQaOYB3THzQf+n8/ZQxG/gnzabg0brJvzSW1CP2eej/Dgdkq117jC9
rH8Jm/zfcn/wGV8wwXvvTq+cNeIPbtFhepu+oH9lyhPfux/x5qz37XWAWfUn
s6/pAr2TvJG/IfNMvhMd6OV30W60oUXdK/+YzsvesK5w72r1THJP/7yFUusS
HOr9wzveEf2PXdkreleRx9/GQ2+558399htI/hlDGMSW3qS5JXPf1Mu0Xl2X
oHcM8o5TXw==
          "]]},
        RowBox[{"-", "2.1`"}]],
       Annotation[#, -2.1, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0L1L1WEYxvH7DEGBZKSmZg7nmNafkGGKkNoJUqnMlzLFocCh4xtWY+Cg
gWaBQg5NpTa0hJ7+gsBBN21vSexFrdwK6/MbvlzPfV/Xc9/P75ceyF1/kIqI
OVQXRcyXRrRVRLTiGhZKIgbLIg5PRkzxqvQySOM+7x7vgDfBO6FXiOPo5G3o
DfDGT0U8xJfiiMu8JjSjhZ+2M2RfnYlI0fZkPy7o11VG/HPnDzqc/9KXctu0
xf0ryOKp2Rn5q84fveex86zzI7oov4TXOGL+GzotP4Mhc2rVOXqRTnrrO/2l
8ohlLGKU95Z+Ph1xSaYeX83eQSnvGy2jjTKb9pV7+5i6mxbbt44ecxvcG5Id
Rg6jGMEa75mdeXfX1B+S71BnE3g3ze3ADXTiFrZk9pNvtqcn+Yd0T/3EG7v5
Veouejbx6Sf59/wfst9RrX9b/6i33aE16l39FfdXsSL7U72PPZzj98odk79L
z6t/6edln3vjqvwLeqD3G/0yfSjg/wdWVVYH
          "]]},
        RowBox[{"-", "2.15`"}]],
       Annotation[#, -2.15, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0D1Pk2EUxvHDgpDAom21pQgyMKD4yupLVBxgsiYsbiaVmkgZsK06YkhQ
P4JfQFuVvqh8ALowaIyGdMEgIFvDZ/DX4Z/rPtc557qf5z73aDlX7IuINeRP
RnSSERPIn44YH4kYw2P1cTZi2HkIW4mIgvpZJmIFjVMRbf59umNv60zE5GjE
qry28yt61fw1XEHB/gxdpAnzv2RcUl/GkpynyMkq0gf0nYzfZtpy1lMRr9Hk
l3hlVHCPN6t/Ix1xE9eRkH+L/jTb1U+pF9yXpB/cVcV79Pu/2+bS/D39DXl1
1Hp3ydzn/cVdM3cwqLdr7w+mnC/gPE7IKdAnGOjVeGO/Lqdm7yOq+IxP2Nab
9m8XcSD/EG95DfN1/ZLv2aBl2qAHvXcyO+dts7w53j87R2jZaakr/CZ9Tr/R
TXzBC/XXXpbdUeeutz1L53kd75OR8VL9kH73Hz+QxH8blk11
          "]]},
        RowBox[{"-", "2.2`"}]],
       Annotation[#, -2.2, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LkuhHEYxeFXwSBRWMYyQ6HSWAsi0U1s0ygsQWWrLKGxdRrhHpgZSiQu
wAUoUBHb0EtIuAbPFL+cnPNu/+9rX96a3CyLiBUM1UectEacYiEdsYg+2URT
REcqoqIlIoGl5ohqtQKdUc/RaZqnB/p+GyNSDRFz5tL0j39SHzebxQjGMFqi
LmLHrl1sYx97uJU3m/+07wNFTCXN2jWMJrV5PQ+yazTySTy686Ne5+4NX0u/
+Q16r+8Os/J1/kp+iSL/jkHvqZFn2yI6fcuzmy84k58jhwLyuDB36P6Af7Vq
pp+u0WP5EcpL/wt9eqvkr/a/4UstwffKK2kX7UE3MvZlSm/FP5/INao=
          "]]},
        RowBox[{"-", "2.25`"}]],
       Annotation[#, -2.25, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwNzksug2EUh/EjMUJc+rkEFboBA5dKEZPu4ItBW0akjDCSsIAyNzGukmi0
JqzBEhDKUJuwCL/Bkyfn/57Lm9s7SY/7ImIbW5MRBzMR9xMRVW7yIVe4NhWx
w7toJxHJbEQGY7hWP/AVP2cjcnqeeIH3zc3zoj1l/pOXuKN+ybg3F3E77R7f
cR0N3KDlP5d6vu1IzbyaecM7UlnDrq67PbRlj7jQ/2uuxi31l75kPGLTvn47
NnhU/SH/RAdd/T2MyAveh3mdz82f4dTfBmU/7gzxmrcBXuU8lrCCZRTdLdp1
hH8iiDEQ
          "]]},
        RowBox[{"-", "2.3000000000000003`"}]],
       Annotation[#, -2.3000000000000003`, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdk30wG3YYx5lT61Adxm3OMK509dbSpdrVd5ptZah5K/Uy0izGKMtWi3dV
VFALahKltqxa6qWaeK3L/NRwipY2Y7KezVuuWpWE9Mpym0629Z89d889z5/P
53Pfx+pkUgDjJQ0NjcNbrZ7/lgIvNhUtTxG7IEecebDvzuTnvUwz/VmHUjkM
pp67jXEpxEYmeMrXlSPd+PrRubQgsr3Igi4Lk8H3ARHer/uUNJ7197p0ZgWD
nL0sB6sk0npkxK8x8wkiSTQlIfw0KTCR3F0PXMZ0p12bx65Ucqln7UStzmM0
t0xHfe2XRSoklhfO1yxhqOgNn58UOcSe8fp8qOlDmDjnhLOUeeTOIyFvOkOK
Ff1e56q32WQgI1Xz2b0FVAr1mM3FxWT5+LiX3GweEc0Nt1z1OGShP6fUPnoW
y8V5nTXeZWSHUqo9fXQGtASrYY/tF4iiaDZ7j0SCxLj9W2gVxIJ0PRrPmoKp
aGmPp0slKZQPNJtSxVg9I9wdksIlB60/Ss91moBdd6h/bhyP3AspjbG/OAIp
jT/lKucRup3tr8GegwjIClgRr/IIoySN/SGrD1X6+24Pn+CR1d0aQy6x3Vg0
2FabbsAlV2kbbJ6hAIbroaoJ7W9JtiikxWfjCjT3dyS0OJUTo2pOPHWQh3Nh
YtGCcQlZzK290ziej/c+N7JdP5RHclWVbpEmdHT2esaO3jxNVmMX2B0bT92d
Ei9rirSPoPXlza5y83hoHBzjmkZnw/47aeoXrHPwYQZLWB+UvuCC6v1am0iX
KpgUuJCLWnX4R58FH94WPd/3Mq7jK+8mKybnCn4xXPMYj+mA5f2kKJ2717C4
/s5fEo4ITEsbXV5fC7Sox7vSjvWDflLJPNVwA6ydhwKXREPo2xX/M9VViDUd
rfEfC0aRXcfWa0ppA4X5JaH6T8CTZJof+KwdiT5+ofOaYoS4jypnZe1w+SOm
2kw8Ca98Qb6lTgeSFPm3g25Nw7zCWfystR2r9LFI7+EHKI/YpKpW2sDOV4v6
DZEf/76vflSInr2HaYU6cxAMaI1cjRCAEnReSeHPQ2/4E/lmSitozAPKJrdF
DAgct4w1o61DLVIKo+5kil9yA9THO/o/xLvrbzq6yerA72Uc4w4uoSGgTScs
hw/2q/2XS6wfQ53OoIIa5M7cOOUfswxfh/CtyFVCUT/kTSl8gg21JuMy1Pxg
ffPPghWUVc9ty8hiYzh9ciQvSobkbzQlyzPZKHuFruuxQw7thmuM+Ewm5riq
1MAyOerfem3I5GzEf3xySG0n3c05bvj///4NQNjiEA==
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{-0.5, 1.5}, {-0.4, 0.4}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 224},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->325168051]
}, Open  ]],

Cell["\<\
Note in the streamline plot that the flow appears to leave the trailing edge \
smoothly, satisfying the Kutta condition.  There are no streamlines inside \
the airfoil indicating that the computed velocity there is zero, as it should \
be.  The streamline nearest the airfoil shows some waviness, and increasing \
the value of MaxRecursion in ContourPlot does not improve smoothness of this \
streamline, suggesting that the discretization is not sufficiently fine for \
this low-order model.  Constant-strength doublet panels are equivalent to \
contra-rotating point vortices at panel edges, and it is probably the effect \
of these point singularities causing the waviness of the near-airfoil \
streamline.\
\>", "ExampleText",
 CellID->14395635],

Cell["\<\
The function ForceAndMoment computes panel contributions to lift coefficient \
and pitching-moment coefficient (about the leading edge) and sums these \
contributions to obtain airfoil coefficients, which are displayed.\
\>", "ExampleText",
 CellID->80454523],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "cl", ",", "cm", ",", "\[CapitalDelta]cl", ",", "\[CapitalDelta]cm"}], 
    "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Doublet\>\"", ",", "a", ",", "rD", ",", "\[Alpha]", ",", "rC"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{"cl", ",", "cm"}], "}"}]}], "Input",
 CellLabel->"In[10]:=",
 CellID->499718542],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9980191091077664`", ",", 
   RowBox[{"-", "0.3066586747308935`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->145104453]
}, Open  ]],

Cell["\<\
Note that the wake panel represents the starting vortex which should be equal \
and opposite to the circulation around the airfoil.  For the dimensionless \
formulation of the problem, lift coefficient is just twice the circulation.\
\>", "ExampleText",
 CellID->119738758],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"2", "aW"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->288057467],

Cell[BoxData[
 RowBox[{"-", "0.9980191091077661`"}]], "Output",
 ImageSize->{68, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->263944]
}, Open  ]],

Cell["\<\
Surface pressure distribution is computed using Bernouilli's equation.\
\>", "ExampleText",
 CellID->753558053],

Cell[BoxData[
 RowBox[{
  RowBox[{"cp", "=", 
   RowBox[{"1", "-", 
    RowBox[{"vC", "^", "2"}]}]}], ";"}]], "Input",
 CellLabel->"In[12]:=",
 CellID->435840926],

Cell["\<\
The function PlotSurfacePressureCoefficient provides the conventional plot of \
surface pressure coefficient versus the chordwise location on the airfoil.  \
It is also conventional to insert the airfoil shape in the graphic.\
\>", "ExampleText",
 CellID->139054009],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xC", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rC", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"gCp", "=", 
  RowBox[{"PlotSurfacePressureCoefficient", "[", 
   RowBox[{"xC", ",", "cp", ",", "rD", ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"PointSize", "[", "Medium", "]"}]}]}], "]"}]}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->671472236],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3d2LJWdeB/AR7/ZGbxRBwTdY/wIvHBEvwl4k8R02QYdojHBIUDasJCrZ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    "], {{0, 330}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 330},
  PlotRange->{{0, 504}, {0, 330}}]], "Output",
 ImageSize->{508, 330},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->73750055]
}, Open  ]],

Cell[TextData[{
 "If a list of angles of attack is specified, nested lists of ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["v", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 " are returned."
}], "ExampleText",
 CellID->571494656],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]List", "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"-", "5."}], ",", "0.", ",", "5."}], "}"}], "Degree"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aList", ",", "aWList", ",", "rC", ",", "vCList"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]List"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Dimensions", "/@", 
  RowBox[{"{", 
   RowBox[{"aList", ",", "aWList", ",", "vCList"}], "}"}]}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->42630738],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "60"}], "}"}]}], "}"}]], "Output",
 ImageSize->{153, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->262461228]
}, Open  ]],

Cell["\<\
These results can be input to the function ForceAndMoment to obtain \
aerodynamic properties of the airfoil in the linear range of the lift curve.  \
Listed below are values for lift-curve slope, angle of attack at zero lift, \
location of aerodynamic center and pitching-moment coefficient at zero lift \
(couple).\
\>", "ExampleText",
 CellID->24035690],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl\[Alpha]", ",", "\[Alpha]0", ",", "xac", ",", "cm0"}], "}"}], 
  "=", 
  RowBox[{"ForceAndMoment", "[", 
   RowBox[{
   "\"\<Doublet\>\"", ",", "aList", ",", "rD", ",", "\[Alpha]List", ",", 
    "rC"}], "]"}]}]], "Input",
 CellLabel->"In[15]:=",
 CellID->34907031],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6.718083511588108`", ",", 
   RowBox[{"-", "0.04402568206056109`"}], ",", "0.25662302616339844`", ",", 
   RowBox[{"-", "0.05064591665902646`"}]}], "}"}]], "Output",
 ImageSize->{298, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->19329667]
}, Open  ]],

Cell["\<\
Note that the location of the aerodynamic center is close to the thin-airfoil \
theory result of the quarter-chord point.  Display below the ratio of the \
lift curve slope to the thin airfoil result of 2\[Pi], and the angle of \
attack at zero lift expressed in degrees.\
\>", "ExampleText",
 CellID->47127959],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"cl\[Alpha]", "/", 
    RowBox[{"(", 
     RowBox[{"2", "\[Pi]"}], ")"}]}], ",", 
   RowBox[{"\[Alpha]0", "/", "Degree"}]}], "}"}]], "Input",
 CellLabel->"In[16]:=",
 CellID->274981366],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.0692161989734057`", ",", 
   RowBox[{"-", "2.522485772254972`"}]}], "}"}]], "Output",
 ImageSize->{135, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->73845521]
}, Open  ]],

Cell["\<\
Consistency between these two approaches of computing aerodynamics \
coefficients from a single angle of attack and linear aerodynamic properties \
from a list of angles of attack can be checked by computing the lift and \
moment coefficients at 6\[Degree] using the linear properties. \
\>", "ExampleText",
 CellID->328772922],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"cl\[Alpha]", 
    RowBox[{"(", 
     RowBox[{"\[Alpha]", "-", "\[Alpha]0"}], ")"}]}], ",", 
   RowBox[{"cm0", "-", 
    RowBox[{"xac", " ", "cl\[Alpha]", 
     RowBox[{"(", 
      RowBox[{"\[Alpha]", "-", "\[Alpha]0"}], ")"}]}]}]}], " ", 
  "}"}]], "Input",
 CellLabel->"In[17]:=",
 CellID->762946865],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9992842689444066`", ",", 
   RowBox[{"-", "0.30708526975301936`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->438828141]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Scope",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1293636265],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Alternitives for discretization",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->421192202],

Cell["\<\
Fitting a B-spline through the points in a data file provides a means for \
alternate discretizations.  This is currently accomplished by the function \
FitAirfoil.\
\>", "ExampleText",
 CellID->5610347],

Cell[BoxData[
 RowBox[{
  RowBox[{"spline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->559405563],

Cell["\<\
Graph the data file as points and the spline as a curve to check that the \
curve provides an adequate representation of the airfoil shape.\
\>", "ExampleText",
 CellID->444428628],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"grD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"grD", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->249332617],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PFPUdx/Fr+pf037hEueMOFAk9D4vYWmrhZAtt1Wo0MagRalsIQrrx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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->51606828]
}, Open  ]],

Cell["\<\
Rediscretize the airfoil using the spline description of the airfoil.\
\>", "ExampleText",
 CellID->27068391],

Cell[BoxData[{
 RowBox[{
  RowBox[{"n", "=", "100"}], ";", 
  RowBox[{"layout", "=", "CosineSpacing"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"spline", ",", 
     RowBox[{"NDiscretizeUnitSegment", "[", 
      RowBox[{"n", ",", 
       RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->343427282],

Cell["\<\
Graph the endpoints of panels for this discretization and the spline fit to \
the data.\
\>", "ExampleText",
 CellID->126580234],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"grP", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"grP", ",", "gSpline"}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->274596420],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PE3Uex/G93F9y/wYJbJcFFb8h+AWjogJSQT2/kxgICuK3Q71r/BK3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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->184133198]
}, Open  ]],

Cell["\<\
Solve the model using this discretization, displaying the airfoil lift and \
pitching moment coefficients.\
\>", "ExampleText",
 CellID->33075010],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aP", ",", "aWP", ",", "rCP", ",", "vCP"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "clP", ",", "cmP", ",", "\[CapitalDelta]cl", ",", "\[CapitalDelta]cm"}], 
    "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Doublet\>\"", ",", "aP", ",", "rP", ",", "\[Alpha]", ",", "rCP"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{"clP", ",", "cmP"}], "}"}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->252090154],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9618055866726263`", ",", 
   RowBox[{"-", "0.2894976675565213`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->88947904]
}, Open  ]],

Cell["\<\
Compute the percent difference of the results for the discretization \
specified by the original airfoil data compared to the finer discretization.\
\>", "ExampleText",
 CellID->169245216],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"cl", "-", "clP"}], ")"}], "/", "clP"}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"cm", "-", "cmP"}], ")"}], "/", "cmP"}]}], "}"}], 
  "100"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->208999159],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3.7651603335369517`", ",", "5.927856800788106`"}], "}"}]], "Output",
 ImageSize->{127, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->4570479]
}, Open  ]],

Cell["\<\
Compute the surface pressure coefficients for the relatively fine \
discretization and show the pressure coefficients at collocation points in \
red.  Compare the results of the two discretizations by superimposing the \
results of the finer discretization on the graph of surface pressure \
coefficient shown in the basic example above.\
\>", "ExampleText",
 CellID->511124957],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xCP", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rCP", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cpP", "=", 
   RowBox[{"1", "-", 
    RowBox[{"vCP", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpP", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xCP", ",", "cpP", ",", "rP", ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gCp", ",", "gCpP"}], "]"}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->59705953],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3X+MHOV9x/FzwDZJRA0KBCelxS3qpTSAVX6I5ir/kCz/AVcCTklQbS6m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    "], {{0, 330}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 330},
  PlotRange->{{0, 504}, {0, 330}}]], "Output",
 ImageSize->{508, 330},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->20320370]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Options",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(3)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->2061341341],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Epsilon",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->30521753],

Cell["\<\
Collocation points are displaced inward a small distance to insure that they \
are computationally internal to the airfoil for application of the internal \
Dirichlet boundary conditions.  In the following example, the model is mapped \
to three values for the option Epsilon, the default value used for the model, \
$MachineEpsilon and zero.  Transposing the result groups like results \
together.\
\>", "ExampleText",
 CellID->120830056],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "vC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMDoubletConstantDirichlet", "[", 
        RowBox[{"rD", ",", "\[Alpha]", ",", 
         RowBox[{"Epsilon", "\[Rule]", "#"}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"10.", "^", 
         RowBox[{"-", "10"}]}], ",", "$MachineEpsilon", ",", "0."}], "}"}]}], 
     "]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->210128799],

Cell["\<\
Differences are computed between results for strength parameters using the \
default value for Epsilon as the reference.  The maximum of the absolute \
values for these differences is displayed for each case.\
\>", "ExampleText",
 CellID->112154987],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aD", ",", "aME", ",", "a0"}], "}"}], "=", "a"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"aME", "-", "aD"}], "]"}], "]"}], ",", 
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"a0", "-", "aD"}], "]"}], "]"}]}], "}"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->165825942],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2.6260413843459673`*^-9", ",", "4.469013458713605`"}], 
  "}"}]], "Output",
 ImageSize->{163, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->35197929]
}, Open  ]],

Cell["\<\
There is a discontinuity in velocity potential across a doublet panel so if \
numerical errors place a collocation points on opposite sides of a panel, \
large errors will occur.\
\>", "ExampleText",
 CellID->937232338]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "InterpolationOrder",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->233233938],

Cell["\<\
The list of external surface speeds associated with collocation points is \
computed by using the jump conditions for the velocity potential to determine \
the velocity potential just outside the airfoil.  An interpolation function \
is fit to these values of velocity potential, and the velocity is computed \
from the gradient of the velocity potential.  The default value for \
InterpolationOrder is 1 giving linear interpolation as a conservative default \
value.  Higher-order interpolation may give better results, but risks \
excursions of the interpolation function fitting the data. \
\>", "ExampleText",
 CellID->381223408],

Cell["\<\
To prepare the graphics below, the model is mapped to values of the option \
InterpolationOrder of 1, 2 and 3.  Flow speed just outside the airfoil at \
locations associated with collocation points is plotted versus distance along \
the discretized airfoil.\
\>", "ExampleText",
 CellID->507899588],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "vC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMDoubletConstantDirichlet", "[", 
        RowBox[{"rP", ",", "\[Alpha]", ",", 
         RowBox[{"InterpolationOrder", "\[Rule]", "#"}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"s", "=", 
   RowBox[{
    RowBox[{"Accumulate", "[", 
     RowBox[{"PanelLengths", "[", "rP", "]"}], "]"}], "-", 
    RowBox[{"0.5", 
     RowBox[{"PanelLengths", "[", "rP", "]"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"data", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{"s", ",", "#"}], "}"}], "]"}], "&"}], ",", "vC"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{"data", ",", 
   RowBox[{"AspectRatio", "\[Rule]", "1"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<s\>\"", ",", "\"\<\!\(\*SubscriptBox[\(v\), \(c\)]\)\>\""}],
      "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.3", " ", "72"}]}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"Green", ",", 
        RowBox[{"AbsolutePointSize", "[", "6", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"AbsolutePointSize", "[", "4", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", 
        RowBox[{"AbsolutePointSize", "[", "2", "]"}]}], "}"}]}], "}"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->49740593],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3X2IJOl9H/BOZCnjlyiDMIoiHGQjWxpsC0Hs2JOLo5VsuAw5dJIPmXAY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    "], {{0, 528}, {526, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{526, 528},
  PlotRange->{{0, 526}, {0, 528}}]], "Output",
 ImageSize->{530, 528},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->141919443]
}, Open  ]],

Cell["\<\
Except for a small region near the maximum surface speed, results for the \
three interpolations are reasonably close for this example.\
\>", "ExampleText",
 CellID->520516665]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "XWake",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSubsection"],
  $Line = 0; Null]], "ExampleSubsection",
 CellID->104403840],

Cell["\<\
Results should be insensitive to values of XWake if they are sufficiently \
large.  Differences are computed between results for strength parameters \
using the default value for XWake as the reference.  The maximum of the \
absolute values for these differences is displayed for each case.\
\>", "ExampleText",
 CellID->391229248],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"a10", ",", "a5", ",", "a2"}], "}"}], ",", "aW", ",", "rC", ",",
      "vC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMDoubletConstantDirichlet", "[", 
        RowBox[{"rD", ",", "\[Alpha]", ",", 
         RowBox[{"XWake", "\[Rule]", "#"}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"10.", "^", "10"}], ",", 
        RowBox[{"10.", "^", "5"}], ",", 
        RowBox[{"10.", "^", "2"}]}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"a5", "-", "a10"}], "]"}], "]"}], ",", 
   RowBox[{"Max", "[", 
    RowBox[{"Abs", "[", 
     RowBox[{"a2", "-", "a10"}], "]"}], "]"}]}], "}"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->451142620],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.3479012517247924`*^-6", ",", "0.001350820109622175`"}], 
  "}"}]], "Output",
 ImageSize->{177, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->96248378]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell[TextData[{
   "Possible Issues",
   "\[NonBreakingSpace]\[NonBreakingSpace]",
   Cell["(1)", "ExampleCount"]
  }], "ExampleSection"],
  $Line = 0; Null]], "ExampleSection",
 CellID->1305812373],

Cell["\<\
In the specification of some airfoils, the trailing-edge points for the lower \
and upper surfaces are not coincident.  This example uses the Clark-Y airfoil \
specified by a data file located in the folder Airfoil Geometry.  Import the \
file and plot the discretized airfoil.\
\>", "ExampleText",
 CellID->416625656],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{"AirfoilFromDataFile", "[", 
    RowBox[{"\"\<Aerodynamics\\\\Airfoil Data\\\\clarky.dat\>\"", ",", 
     RowBox[{"NumberingConvention", "\[Rule]", "ForeToAft"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"gD", "=", 
  RowBox[{"PlotAirfoil", "[", "rD", "]"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->112529997],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c1rFOkWx/Fc7l9y/43AMAtxIS7ECyGLkI0XGt3MbHSjm2SjmwQa3RiQ
60INJLmLyFxKhAzIBMTAMIFgQOyBIcIQFwZBEW5gIPekT+dJpV46/VIvz3nq
+6HmhY52n0p3/erUU091/ePaj/+89veJiYn/yj//+9vExPH/HwEAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAA4I2oNSEm252sh9OPAwDGUnXqyhO3ou5/zzx3
p92Oej8m6QGgOJWnbqc92X3B1EueVkTOA0Bhqk/dkx1L3lNHrd6PeyYAACMZ
LXXH13/PcnIccSbniy0AAJrAheewqVuAnJEiLab7UNSO/YCcB4ARnIbnkKlb
iLNnfnsV9M77dsWPIch5ABhBPDyHSt3qkfMAMAJD4WmoVADwh6HwNFQqAPjD
UHgaKhUA/GEoPA2VCgD+MBSehkoFAH8YCk9DpQKAPwyFp6FSAcAfhsLTUKkA
4A9D4WmoVADwh6HwNFQqAPjDUHgaKhUA/GEoPA2VCgD+MBSehkoFAH8YCk9D
pQKAPwyFp6FSAcAfhsLTUKkA4A9D4WmoVADwh6HwNFQqAPjDUHgaKhUA/GEo
PA2VCgD+MBSehkoFAH8YCk9DpQKAPwyFp6FSAcAfhsLTUKkA4A9D4WmoVADw
h6HwNFQqAPjDUHgaKhUA/GEoPA2VCgD+MBSehkoFAH8YCk9DpQKAPwyFp6FS
AcAfhsLTUKkA4I96wzNqyetPTLY7Zx/utCfl4VZ09lFyHgBGEA/PoVK3APJ6
3WeNWqnXPH5Rch4ACnAankOm7vhOnzTjJcl5ACiGC89hU3d8JzsWch4ASuTC
c9jUHd8I/bxTdC0AEJp0ZlbfzzM+DwAVqHF8/ih55jd+SDGRPiVMzgPACPLn
25yTutUj531zcPBt+7c/08vmL388/vev5y4//vBTIcviwmafV3kRvUtXKJXX
/csDqmMoPA2VapHLQAlGF5L37r6KJ+rlS4+//27JLVevPMkM3ju3Xw6S85n7
iBGW5+u7fV4lsQq6SOXxFZEl/tP5uY3M3cTh4V91v0vAKAyFp6FSPbS//0WS
6ueN3xO9dDroJBjzmuGvXw/rXo+yxFfT/ZYSu4mLFx7p70r2d+mjibXVHf3r
nfef6l4b4AxD4Wmo1BrpWIq2uLduRq53nZ5ajneqLtPqrtck2d+ljyYe3H+t
yf+va/9J7A7cr33zlz/4taN6hsLTUKmVcTnj4kXHUrTJ3HrzgbHoGrndgTtA
uHP7Zfwwyg18LT3ckp/KW8kAEcpgKDwNlVqSzvtPeopTGnXpz3WwRSOd4QKL
3InsZ0+35U2UtzI+QDQ7sxI/2cEuACMzFJ6GSi3K3t5naQWl2dMOUDp23eql
Ud/f/1J3dSiXvPvxyUvxXcCN6+vy/w/uv3ZjQeQ/+jAUnoZKHZkc6et2rZNb
pKObn9uQZo8RXcTtvv0oH4m11R03FhTPf3lQPzPyx+quFF4wFJ6GSh2KHLxL
0y7H7NKuS7Zrxx725BaURPNfPj96DCix704H67lgPQXAKZumMRSehko9l2xo
L6J39+6+mp5avnrliWyDsgEywI4y6OlgPRespwB0CpbsBW7djNzsWfqKgBkK
T0Ol5pF2Sxot6dtlQ5OQl+2LYXbURT6NW28+uGsEpO2/eOGRu4BCfkTjEQxD
4Wmo1DhJcslz6dhlO5IO6tnTbTYf+Onw8C93QbS0+jpTV6f9uDEf2n6LDIWn
oVKPuvG+trojW8r01LI0SHLUzAYCi3Tajxvzoe23yFB4mig1Hu8P7r9mE0B4
Mtt+N+lXWhrm+fjGRHgqn0s9OPhGvKPJ3EV883MbOs9ndmZFz/PK42wR9Soq
PPVm4aV+fbGfOS/di3ywr155QrwDcXt7n/U8r/T5rufX6Z1c6FexwnK+c5zu
nXarvC+p9yrn5TO89HBLp0RK1NddDmCANEI6vVO/uEPH+RcXNvWrPrmktzxF
hmfWnagK5EPOy0fxRfRODkvlmPTZ022uNwFGpuP8z9d39as+JfYl/PlmjzIU
F55Rq3v7qU4UBTluI5EuHz+dPMNpJqAMku3xb+rThl/2ArIvYKMbR1Hh6Ybn
J0q5keyxunJePnt64ap8/Gjggcpow7+2urO4sKnndt0IPzP5h+LDYMiAqi/V
JfyL6B2Dh0Dt3Ai/zuS/euWJm9LDOE8f5Hwm6dulhdCEr+xFAQxFtlM3pUe2
Vv3GNmI/jZxPe76+OzuzIv+u5uUAFEK/sY3YTyPn4+So8Mb1denkGfoDrMuL
fTkEaNqJNnLe0QtaOa0PBMnF/q2b0dUrT2Rx93oI/uwbOX/U/QDIO/7g/uuS
nh+Ab6Sld/duu3jhkd67Ta/Yqru04pHzOlYj73gZTw7ABL0Xs16xpRM4Fxc2
X0Tvwvgyk4bn/NabDxLy8hYX/swA7JJ4f76+e+/uKwl8vVzL9MB+k3Nedta3
bkbBD80BGIderuUG9qenlufnNvRO63WXNqjG5vza6o7srAt8QgBNsL//5eeN
3/VO63qb3Qf3X0vT6POwQDNzXnbNnHUFML7dtx+1aZydWbl86bFen+vbHJ4G
5ry8C7IU8lQA4Hz9eqjX5+ocHj2ZK81/7VdpNS3nZc9LJw+gAnoyd35uY3pq
We9VIflTyxU6jcp5OZjixCuA6h0cfNN5mzeur7sJPJWdyfUz56NWxh0IxyxV
fs+zMyt8oQGAerkJPHom103aLC+d4uGZma41kDq6X2KfuD/VmDnPDf4AeMhN
2rx86bG7PqvYIf3T8MxJ1+p12pO9e5WcLWWcnO+8/yS/wLFLA4ASueuzdEj/
zu2Xa6s741+T68IzL12rd7LDych5Z9jnfPZ0e+nhVnE1AkC5Dg6+SWpJg/r9
d0vS6o/wTenpzMxL1+qV0c8zlxJA2fb2Pm//9md80WmWiWV+buPHH35KLNK9
S54nlhvX190fGGfY2cN+vozxedkVLi5sjl8agDDo1xQnlrXVnUQm6zebJZbZ
mZV0Jssijyf+pF42lVgksdMvXeo353g4Pn9Uwnyb/f0v8hYwoxIwavftx0Qw
6o1iE4vkajqW9WayiUzW244kFkn1xBPqNxUnFp+/4iCTj/Ntcow534ahG6Aa
0lals3HwZM7sluMjGLrMz22kn23rzYfMl274hGo/589nGr/UO7dfcl9vIC3d
Letkv8Ry7+6rdA+czuTpqeXMpnrAZDbXLfuvUTkv5IO69HCLARzYpVfZJJbn
67uJ/NQvVBzkZF9mt6wX7yQWaZPSL1337wPna1rOH3XnWMqnmmumUI3O+0+J
YNTb1Q3YLaeTWa+aTyyLC5uJZ9MvSK/yZB+81cCcP+puenIIKVuQ9Dzc+BtO
5sBy5tS4zIFlneqcWOTBxB/T208P2C2TzBhfM3NeyRakvb00SLLNav/DkI4h
6enKZQwsZ06NyxxYDuNeoghPk3PekWyXbVbHMyXzZWPXTVunuZb0og1U+MBy
eroyA8tAGjmfJgfveqiul63FR0QlghK9XMAtXHpgOW9q3IDJzMAyUAtyfhCu
EZUISozNxodk01depC+7cIv0sZljDiMveWf33CKJmtn95nXL6YHlvKlxJDPg
M3K+QOkrqdOXUQ+SuqMteWf3BtmzkMlAwPwPT8dQqQDgD0PhaahUAPCHofA0
VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/DIWnoVIBwB+G
wtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoqFQD8YSg8DZUKAP4wFJ6GSgUA
fxgKT0OlAoA/DIWnoVIBwB+GwtNQqQDgD0PhaahUAPCHofA0VCoA+MNQeBoq
FQD8YSg8DZUKAP4wFJ6GSgUAfxgKT0OlAoA/6g3PqCWvPzHZ7px9uNOelIdb
0dlHyXkAGMG54TlUFA9Hnrr7BFEr9fTHz19kzjdzH9HAtWaVm6CBq3xUagAO
GcVDOf37Gc9OzheggWvNKjdBA1f5qMwAHDaKh3KyDyHny9LAtWaVm6CBq3xU
ZgAOG8X9niiu+1wj9PMAgBEUGMXDKXNQCAAwkJKj+OxJ3vjRw0T8BwCA8hDF
AAAAAOqSM/8/QDlrGvKBV783N9AzPH1WWa9pCe+N7nsJT4grfKqIy5SaoN/5
hbDkrGmn3Y56Pw7uN9Dvze1mQ3jbR/4qSyIE9wZ35a3yyX78dCg5TIH2K4Xq
O18oKOeuaXi/gD6rHLVaUYjbR+4qn/S24e3b8t/lXsAfv9f1lFaNED/HRes7
/z8o561pgFtD7irruoa4feStsmvmwzts6/fBDvSg7awQP8dFo58/+Wk7vI9K
ziqftrbhjd3mvcsBf8779fPazof2JieQ8wNo/Pj80WmzF7UD+xX0f3OD3D7y
Vvkk7QJc6T6r3H082BMTPQG+pWVo6nwbN3p52tyG93HJu7jjWKDbxznXszRo
ld2BW4jrfOLsbLnQzzkDAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAABCI
3rcmttrH/+XLBAEgRMdRLzkf4K1cAAA9wd/VCACaTO/+wD0gACBInd64vI7T
09QDAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAABDzf3mA5o8=
    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->5545174]
}, Open  ]],

Cell["\<\
Note that the trailing-edge points for the lower and upper surfaces are \
close, but not coincident\
\>", "ExampleText",
 CellID->390242666],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptsTE", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"First", "[", "rD", "]"}], ",", 
    RowBox[{"Last", "[", "rD", "]"}]}], "}"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->238007975],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1.`", ",", 
     RowBox[{"-", "0.0005993`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.0005993`"}], "}"}]}], "}"}]], "Output",
 ImageSize->{241, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->48979307]
}, Open  ]],

Cell["\<\
Specify an angle of attack, solve the model, and compute the lift and \
pitching-moment coefficients.\
\>", "ExampleText",
 CellID->25666953],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"5.", " ", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "vC"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rD", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "cl", ",", "cm", ",", "\[CapitalDelta]cl", ",", "\[CapitalDelta]cm"}], 
    "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Doublet\>\"", ",", "a", ",", "rD", ",", "\[Alpha]", ",", "rC"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{"cl", ",", "cm"}], "}"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->592619],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.18969802405527325`"}], ",", "0.23030473695897258`"}], 
  "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->169567890]
}, Open  ]],

Cell["These coefficients are not plausible.", "ExampleText",
 CellID->722709344],

Cell["\<\
Plot streamlines near the trailing edge, using a scale that makes the gap at \
the trailing edge visible.\
\>", "ExampleText",
 CellID->492689119],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"xW", "=", 
   RowBox[{"XWake", "/.", 
    RowBox[{"Options", "[", 
     RowBox[{"PMDoubletConstantDirichlet", ",", "XWake"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"a", ",", "aW"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Psi]A", "=", 
   RowBox[{"IC\[Psi]DoubletConstant", "[", 
    RowBox[{"rF", ",", "rAll"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", "\[Psi]", "}"}], "=", 
   RowBox[{
    RowBox[{"UniformFlow\[Psi]", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "aAll"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax", ",", "contours"}], 
   "}"}], "=", 
  RowBox[{"{", 
   RowBox[{"0.95", ",", "1.05", ",", 
    RowBox[{"-", "0.03"}], ",", "0.03", ",", "13"}], 
   "}"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "contours"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "gD", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->266881112],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.95`", ",", "1.05`", ",", 
   RowBox[{"-", "0.03`"}], ",", "0.03`", ",", "13"}], "}"}]], "Output",
 ImageSize->{196, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->119372697],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxs3QW4VVX6P3AQEBMTe3QcMTFAsWOrY3cHGICBAYpgB0ooJqIighIqxtj6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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwlkTlLA1EURj8RW4kimehktxEEEw0qGAlYiI0iDEJEXDAupVqGlE40PyB2
BuPyH8Q1ajHRSsEFLELQP2BstEjlCRaHk/fufd+9TEKpTWujSVIa9julvpAU
gShkvdKIT9rBMc4D0A8F+g5MKW9Ia2HpmvMqLuF1fIWzQancJfV0Sw5+xW/w
Aja1XXJy8OmXPqBK1jKzUjBF/w09t3AHSea3UjexFyap+/ASvYuw0DD1bXJs
GIY41N2SEZA8UGffWY/0zLs5fMoeZ1CAGfZ9x9PYghaIkTnImzhUqbnYw+G+
g9wHnCEnDRVqY3gchrhP4FF6EvDEuZd3zew2wQ5Jsh6ZXcNfUOG+jft2cMEv
/MA83/AQH8EWnMAxBMhbwUEcIT8K5/y+J6fG3G+45CxmXjR6ydmjVjT//9s/
SQtIpQ==
          "]]},
        "0.406`"],
       Annotation[#, 0.406, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0rtL1mEYxvHbUIkcRIMOnkob8hRlKOirRoMaFAS+b2bqoDQ0qBSVIO4q
/gFOOogO6uKgkCb4eqhIC6LAwcHFpTaHQCG3PtHw5frdp+t+nodf6fNXyZcZ
ETGN19ciGq9G5JdFJOiPGxHp6xGbaFBbvhKRS1dogg6URgwiWRSRwjf5Fvk2
tOLc5YhKs0GbS6Cnjm8LvY9D/sN0BLvyF/V8orlmynETFUgURtTibnFEpno7
HuPAfIpO0w66TtfQ5Rz7PCfkzsztOcsuzi5FnJf/Q5f41tt3wV1z0K8+Kh5D
iZ5xHn32jdG3akN4gyaejXRV/qPeh74f4bt4W7yFNN6Jn/J58e9sdNG+BWTa
lY0sfNX3BZ+9z6yebbkdbOFeQcQHOs8nj8eiuNxbV6ASR/pv0WpUYcq5HvAZ
1NtvJls8Jz/g+8TeU/Ty60On/DN0YZLPmlwz/1W6jvdYMLehnsYd97uNNP/f
+jdoj3w3dszVuMOxNw21n/pm7P5Fn/BJIYk555pFRtn//+wvNh9bDw==
          "]]},
        "0.392`"],
       Annotation[#, 0.392, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0j9IlWEUx/Ej/REdGsLSW3bNgpbSslxscXDRITOHqMXAJQ3MxUHrBqWD
OgRRdIeWFG42ZGVCkRZS4VVbQiOX/jkW1RARRRHW56Xhy+95zvmdc57ned/K
9u7W0wUR8QQ3KiK60xEd2+nOiObSiE7rQ/TM1ojPKVoZcRYZpLdFpMoieq37
8BJLOIwWTMtl9XlE03jFv4xmvT5tjjhezos9PFXYjb2oxuKWiAO0FvuwHzVo
cpZzevx0rh/4uiOin7fNvAW5GesM30Pr63xTdE5u3PxZeovm6W06T9+4U47/
LS1Huzf4K76Ki87WqP6COf04j1n9R/UdQV7uF/8ddXnvdkRNK76oW7Cfxxzu
yv/mG3Pv7zRnPyP+FM8wzN8mXiJ+gm6i15xpRa5Lv3fmFJlXjCW5Kufoc+9e
HJN7rKYDnTiJap4iPcf5CulBferwXG5Q7pL1ZVxJZttn3WnavKt0wIyjYpPq
uuyzalrsB+k3ZxkQX2Nfw1eLOnd64VvuMqsYG7AxuTdPifWf5NvTjLpl8Qa6
ilHxMTpC62m9XIG7lFo3mFVGJ+RS9J7cJJrEG5N35MuZu2jGe/FTyXcUG+J9
Tdc7VyEmnH8dXZv8B3I35R7w38dHfT6gR59cxf9//x9l23qu
          "]]},
        "0.378`"],
       Annotation[#, 0.378, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl01tozmEcwPFnXmFlFtrBmNmGuVDMyK42mzmVQ3axqa1kF2ubtjnsdUjI
MVtiRBFm3OByCDmULDltpIRY4YacEiunl/J5c/Ht+zy/0/M8/3fLrmkub0oI
IfxCXk4IH8eF8Amf0ZIRQg1H+ZXceD7Dufw8NYSsMSE84+W5IeSIJY4KoU3+
He/ht9zKnWkhrONb9lHu5kfqb+h7yNdHh3ANscwQXsil8OrsEFbhsZo1YgX6
IvY3ndlofz7L/biL1/J0+SLnXJa/ghVid825h4jZFeYM4Eruc+fJ3vWSc8e6
O5LVp+pLwUX7C8iw3pIewlLezE3OakYEM9XPEz8tnuCMY2YHvi9eKv5E/ybO
dE5MzUhOw3DxJE7EBPS6Q6HedN9hu/qt6PfOr9jtnMM4hGq1XXJl5u+1b4vH
9TQ4sz7+Trla7z9k/gH0yvdgivqY/Xp3iMbPjvf5Bj14gB3iZ521k4fJRXmI
eS08yMw8Pa/dMWI9EPnO7PKtp/J5/sLzzRmh/ql3JOODuvc4bs4lTJJ7I/7X
+g/63emA9xxEO76ZscCMOj1z3KUMs3FVvEP9CRSasU9NTKyc58r3cqv4OetB
GIz39vt5cbyel3A71/s2R/TW68m33mDmNG6I/32J17pfn1iD2DbrYusibESB
WKPYT+sfKFa/jGdxCfqcsdC3W4xFSLa/I1+ipxTDvPeUd3biJJLsf7vTUK5G
hztU8VG9t613OS8j/vtzp+8/h+vMrMJ3c6s5Zm6N3Ay5ErFSrDS7Nf43wW1c
KVaBbr/TxJz//9//AClilAM=
          "]]},
        "0.364`"],
       Annotation[#, 0.364, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1VlsV0UUx/GBGCkKsSwtCIW24gOIgQdIoIhiBISWslQWo5HFQkRk11IM
aCkoCFZlKVHBFhKhUSkoiyyCRl/coOwKQtkhQguUnRpDrJ95+PY3Z5lzzsz/
3tv03KnPTakXQqjxp1fLECpbhVDVIoSpbUJ4yXpYWggjaXbbEAaijVjnlBDG
4X2x02KL6Bm63p5yXJbzS+sQfsWVaOMK38/sMfK+aRfCy3QUKuzdi32YJm86
XsceuTX2nLKeYH1K7kL7qvm+4luHk3y9+U7QPvT75BBuyc15JIRlD4cwhBbT
o2Y9gln2LmcX0kr2XDrTvk/4ytHE3iwMQBc1B9JsHNarlbs5FmdyB5U0hX2d
vzU9zh5KZ9Cn1TzH39B6JvLRi2+nfrswVp/b4rk0UexJHDL3hVR9aFv1q1GF
+9X8wHyXrGekh5CH43yX6F5nq8AkteayCzEHteYu02ctPscaLOC7IXc+vUl3
x98O3c1V49478LXHGDNlqLGav9T6NK1Svxo/mfNHjOabx35AXkMkYIm6/cTO
mj9d3YVmzGT3xwG5zdjV8i6jCifZD6kzSnyQvWfkTLGeitnWs9AHfbFUPNGe
JmiKcXwDzXzdndxAJrIwABfVXRHvi3bFDrld6D9qLKHtzNZM31x9bvG9rV4B
Oohtl/sYHWB/NrJwJz4D9K67SBXvKH6b70/6t31H6EX6OB0v/ipGiI+3J1mf
Cfol0cHsIRgUa4nn0KH0FfnD6Nfs4fEe1DlhtlS6VSwtPmP8i503Cz2t+9Mn
aCYdImeic7+G8ex3+bfE94D9jLl20WJ2vjpl7OaesSS8x06md8Tv479L8zAH
BXghvr9mf5EWsvfI3Y1Z7iFHn+f1GUx/wzv8RegtVkAXYT5u6lEQ3zG0kN/N
uSo8H4/ak4BGSFQ7mTbFv/JT6B51Ttj/Oy2xp87+lbS+s+7j689+FpPl1EN9
zFbnLYwwb3f2cDrdubvFmfkDraP91C+xLsUk9mRMwVb1tmGS+qvEJtLVtFz+
TTN3iu+SMzRScw09zj6GAn3y+T6Wf9d6gxlraRHfOXn71T7IXsN/iB5kH4j3
KbZA7R3OtTeu5R8WXyuvgr2T/zvc0/uq2mXiNfQKruMa0uO3wW/3IYqQzf5D
/kb7NqERtmBzfAfV3ka3ozG24tv4DshfR8vxIDZgPebILzFnKf4zUx1+kLsT
K/X6DCswUs8S2pPWyh1NS9mj6Ebn24Ras3Zip6nRmZ7HObTWI4c2EL8Xn3Fn
zGB/JG9x/L7zvRm/zfJG+C0+pV/q8QW6srtgmVkXyl2EFtbFmCevsX1v6J2H
jPi+oAfSxC/pl6RPG3uSaRX7qpp/ibVkX4u16FF2+/g+xP9Zap6P3zzrwng/
6MF3li+DTmPfwW301fuy2ZPoetqcbqDL5RZjlfyN8fvOl6DPZr4G8ZvOfirW
YV8wUwpNoPn0gG/zQezHWHmHaC7tGP/P0P8BH+Q+fw==
          "]], 
         LineBox[{6050, 7733, 7890, 7732, 9155, 9154, 5873, 8361, 8362, 7891, 
          8552, 6100, 8668, 6341, 8666, 6340, 8667, 5886, 7258, 5887, 8649, 
          6318, 8648, 6319, 8650, 6160, 8340, 7713, 7075, 6049, 8313, 6050}], 
         LineBox[{6059, 6259, 9106, 6258, 6980, 6979, 7626, 6161, 8654, 8655, 
          8651, 8653, 8652, 5900, 7261, 5899, 8721, 6456, 8720, 6457, 8998, 
          8999, 8319, 6850, 8997, 8891, 8892, 8890, 8894, 8893, 6334, 6440, 
          6439, 7317, 6083, 6851, 7509, 7508, 7507, 6020, 7562, 7563, 7061, 
          7060, 5823, 7680, 6102, 8267, 8268, 6723, 8373, 8374, 8375, 7260, 
          5898, 7252, 8985, 8986, 8984, 6835, 7578, 7577, 6059}]},
        "0.35000000000000003`"],
       Annotation[#, 0.35000000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1GtM11Ucx/FjympzreWF+P8BuVkuL02tsNkT0y6rFZCXBrYoQ/RJ0VoP
igFqaYtCvNQqNcU1LTUtK0CbyjRl6GZu6SbVzEwu4pxOUZehYr3Ogzef872e
7zk/zj/n1TemVwwIIdT7MzEZQn9OCKmJEE5kh/AnTmI+f0FuCMv5n6MraCGt
uCeE9SNCWEwT6hqsH8gKYRwOqxvB35oXwlh2gdgMTEcW/0w6C4XoQhGex9q0
EM7Q22g3HUh76JtqVtp3nb5rcYn/kj0v0gyzPCOWoOlI4gOc4+tKDWGOGZ6S
dwp/YRB+TQ/hKA5khrAf91uPxkyxDjqeXuafEOfAKj266Gx3UYJVzlSs/0tm
K43nNl9jhrPTl833Ckrxltw59F46Rb/76Fa9RtEX5S7gW4ha9pU4N03wl+mV
RsvNcgf242ccQ4o5Kumz4u1qC60LomKI9Tt4TKyaXaXf0/Qh2kcfpqniVajG
MBSbZzgtoUNpq70PoA5T9ZnrfOVY4r7fx2LMc6bz5r2Ai/F7oBd71LSg3N2s
128uTdFjoX1/ZC+i1ViAH9g1dBmarJfTsdim5xj6LX3SzPnq+6yv41/cxA1U
6/26PSvwGq7x/YOrvnmT79OMRuzEDlTK30V34ycUqSlAUv932e+hBrVYgl73
XOMsi5Bqnnx3c1jvPDW3YzDuci/D6d3ok5+k6TihPkHTUK9/Ns3BLf4j5si1
bqJX9f5Nj3b8wn7b/Flite74I3yIP8R+x3F9NslfKqcedehVsyy+TUxUd5n9
IB0U35g77Tf3DHqL7naXu/CoOf9jz+I/pf9B9knaFt8HLbLH13p8hY3xHesZ
9M+kI9XsU5tL99K8+Lb0/Fv8NMrkd9IO5Is30mY0ocfdFet9lpape0R8EnKs
W2g23UPPO+sws7TSnc4xLr6vuOarktPMt8W32EF77H0Wm9mb4pvWu8EMUzEt
vjv7jkSJ2V9gz4vnpuV0tp4lqFNXqVeLPpnWx3GnvdrYU6y749sR/469HV3s
febopOfQbs9S9jQ1k7FSzsf4Uu06dZ/E3yx6UM4htGG7fbfgG6wWWyF/DZ3k
/vMxVM4QLNWj0+w3xJJyrtMO9mr7HBG7yf6ev5+eMcsX+q3BGP9no5ErZ2/8
HYi/n9ZZYluxDU+Y63FkqMvEBjmnfZ8ONNjjqLpjSFH3qbzP8Rm6nfdKfBNi
/wPFkfxW
          "]]},
        "0.336`"],
       Annotation[#, 0.336, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1FdsjWEcx/GnhFoRIXTR9rS2itbe64LGDonVitaebUIQF6gRIjrcUDMh
0QoJImJFNHGBEuHCqD1izxskEoTPe/E9v+e//8973nNic4onF8WFEMZGH8kh
HM0MIY6OTgohISOEXJpIPyey24Xwiv2dZqaE0B5Jzono2DaETtjCtyIhhK18
K+kN9k00x1e+r3y1zufbhJBv1lB9t/Nli7U09zhfK5qjVzYapYYQjztqgtzT
8uLoI1TKfUg7i3XCFbF/et12TldTbe+YXh/SQ/iITyhmz6JptEo8nY5iN4iF
cE9dM/2+6dOU3mUfpxPs8du+U9kNcQonkWDWxLQQJmEeey6eqO2uJhfz7PdZ
XZkZX+h89kDn8XLGYYz65Xyb+Y447+Kromv5TvFtdIdZ7BL+jSjl+2HnHfQn
7SD22t5vMNwON9QNN3cTe0R0D/qJ7nO//fhjvwL2QbXxeuyjB7AfS9xxjX79
9RiAfqiTUx3thGK9ilBoTop3oIAm01J1STRLj4W0K11Au9FDtC9dRfvQ1bQf
7Ur3qltHL+uzno7kH4Eydz5kdjmtwHV5dXJqaYm8i84baE10X3e6iTLncpRg
Jyrwwz2b2bsFUtyjl7tf8x3kRDU0We/myJM7A7/kv6Cv8RjP8Qylnkmhvebg
L/uC2f/oGfqHnqO/6Vk6xYzJ2GbGAzOf6P8Uz1DfHo/0WuwZDol2N++lcy0d
K/eEO4+h+fIeytvGvs9eENXrmSGvLophevQeysnzTOLpDHY8anAZlfx7sBtd
7PVCzRL+pVikZ567VPHPpNU01aw0rLf3RfH6ZtZDHIbwDUYT/tnsAhSiiP1c
fbb578XfIZOdgSzxblGt2Fw6zNyrdujBXub8Vm4+/0Ezp5lf6XlMZU9DLPr9
Rn3oPXl30YbdOnqfMEif3WJ9PLveOBb9vvV8jyn67BKr1qcKPdk52Ot8S/yX
WI36VOc0dPd8snDJ+xCL/l9QjrLo/0RultwmdmyKxjhs1k6xCvwHMiHCvQ==

          "]]},
        "0.322`"],
       Annotation[#, 0.322, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1XtM11UYx/GDdrEV6cypCBaIoNFyzVJaIZpy03JNbMsgbJVONwRchtJf
rYtkLVuhchGjf7qQtyWXIenUUv9I2SptS9m8JWBWTstleWu9zh9vPud5zvN8
zvM7v+/3R9pLVcWVCSGELn+GjAphSnoIU9GRGkI72jBzTAgP3xvCpZEhzBob
wpfjQtiMVky0t4W26d1l3asmi96PGr3baTVdhZWYpj8Hzeq389yGLWqqUkKo
RAXGpoWQgmR8ai9D/XiMQzo2yd1qxttxG9J5let72xyr8RZOJYVQSy/aO239
0egQdvBrRxsuyWfyum5vvbp6bECduiY62BkNdBBtpC20Npk/rum7SzyJJtJ6
uTX89okzrDOxns879FX5Pvlquibu+bwn3FG7/Q3xnulme43xXulnasarycAi
ex+qWUxb7dWraUCbM0fK76Cj6H49d9j/1ZzncQ6/4zesVDNAa2gfXUX76Rv0
pL503n/G+6W/uJfD9Aw9RKffF0IuZpjlZ/69dK786fg5aK++563LcEZcSktw
PN6D+JR1XvR31gT6nHyBdSF6xLPpVvMX0RJ7pfjJeq/cUXpB/490j/gIzcMB
s+fTrz1Lu7AbC333iebrtt6JFeZ/BefkKnhWRm89S/mkyKWqL+MzDbnIwQxM
jznPw0H6tPpvee3HTXO8aN5k/Z3yj/F+HFt5neW1jSbQQehx1mG8rP6Gmnv4
XafD6Q33VyO/kqa513EYj3TM51Ejvwr/4Saq5eeYZa3eY9Yf0B5nrDDHYVru
nAXx3THjcsxHMfLVva6+gJaI11lfMcNlZ9+pZyjG6Jvscx30LD6k5jua5MxE
lKhfgH/iM0Gb+BynjbRJzwlnN9IrtEtuCn0ELRiOEdinN0fPXnrU2efVzxQ/
43MuN/8T1q85qxs78ZV8s7lmqJ2OXHSZcYm9WfY6eObRy2b9C5Pcy4N4Ek/h
Tf7l9k/wWGK9FMvEDfor4vtFj+hLtc5S06LnE2TLDXHOo/EO7W2Mv1fmq7Je
ra5JfIu4mSbQwRgU30n7i+N3zWOevUO0mK5Dp/06OiK+++box1Vxt/ps93HN
+mz8HXF3U8Vp8lPoVfFJ38MwWizON3MBCjEgLnZn81Akniy+SP9V+wKynTWa
5zfOTqIL5TaqLaPNdA/fQjPWigvobBThAp9n7f8R/dz5Mvc8kVcp797422Hd
ar5+Oie+sxjg3yc+QA/Qj2ldjNUlWOfge+fN5fUD/ZzPMHN9QdeqeR8TnJWJ
DAy1dzcaeHeIG2kn/Zvn7vis8kvGA/yy0M0zRbyI3zHf23H0xv8xajP1vGvv
PQzEuVEj9z8knQ2n
          "]]},
        "0.308`"],
       Annotation[#, 0.308, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkDFLQnEUxU9g4ZQJ6nupSaKQTn6CPkCrS6KIILTWoDnp1CBatoQOihgE
oqCj4ewH0ETU1pacjUa1fg4/zr333P+5j+dP30VvDyRdQ86QimeS6ZcMcMGr
W4r5pCBe3ystz6UFzOHqVDrCTwakoUsK4YfhkFmHmQUNk/fL25opVaEOY7wG
WmD+yX4erZLVhze8HvpNXprapDYgS849pMiccPsDppBh5oEn6go8woqdZ/QE
tcOIrJv9t3PTRt8l9xiN8G7H7UvmCWbv7LWYD9A4/R93dxBiz8m/cEAT34r3
4JG+qMvUP2RcsLNht02/RV/InEGJfo3/D5hTN40=
          "]], LineBox[CompressedData["
1:eJwl01tMz3EYx/Fv5ipJyaGcKq7Y0kwzHdwpNjfWRqLmXGGLykbnuHCIGza6
sGHYbKFCl5TMIRfIhFpIbHKBi8jcGK//XLz3eb7P83me3/P7/v7/1G1788uj
QgijqEgKoWpeCFGpIUzA/Tkh5M9UEP9NCaGYFkXycjd5GxeEcE88xtdDm5xv
yV+aHcLdxBAanLvlh8RdtN75jviL+PD8EN7zNtF39BDdNyuEDp4KmiEXK7eU
TqF/ZoRQ4jmr1L+q59FvtHNuCLcRa+535yKzK9Vq9Q3q2SKeJI7GUf0Z6nvk
Njo3i1O8zwlaqv+UfAmdbs40lPGUy61R79Lbjd1y6fbZRePkN8v16Xkit1bP
I9qLx9hurw882WbkIAuDPCvoAL2qPqLezpvu3EYX09XyabTenXfSGlqHWmTz
93hev71f4xVynRf6bovwyTsnJIfwkV5Tu45WtOEG8njH+H7iB34jx8z1nl2A
dSjEBsR7x3i7TMUzPEWub5uHAbV6nkY0oNncI2Ydw3FEu5et5p4UTxbXqLfS
ahpDV8r32fMFzohf0v3upVRPv7iMJtj3rFql965Ai7to19uGTc4FKMRp/nHe
WPsN2eWN3TLFSfxL7PrAXaTz9dJE82Iiv3GeYn2/7DMSebaZVyJ3QYfxFi16
x+l5nmU0TU8CvYgLqJU/R+toHK3X/5Ae4Mu2x0G6nGYhEx0YtsNnnlFMNP+5
PS/z7bRfjX130GpapVaJjuT//8t/igOPxw==
          "]]},
        "0.294`"],
       Annotation[#, 0.294, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{5825, 8020, 8021, 7686, 7685, 7684, 6210, 6382, 5824, 6821, 
          8478, 8479, 8257, 6667, 8254, 8256, 8255, 7469, 6023, 7841, 7842, 
          8685, 8684, 8344, 8345, 7204, 7205, 6356, 6131}], 
         LineBox[CompressedData["
1:eJwl01tMznEcx/FvbhNzmByq54lLZmmEIafNqIfFDRcurEVZTeLGbC6ojZUL
cWczW7kRcxgj5qYetMLE5iEXZnXDbOXQM6c5vMzFe5/v6fc9/J89xdWN2/bl
RMRDfElE3C2KWFsccT0/4l1BxLTCiOnoni0+L2IdyrEGqzGs7kQyYoSOeDuM
oVkRR+ZEbBcb1OMZzmK++BS6gE6mD8ybRLMzIsq8K2WfmxmRoufpG7oCnXIH
6Vv1WfGdNPTIFc9Hn/cL7ZCmJeYus+9SZNS8nhuRxx53U7t39e7YobaBXtOz
SD6lrkXNH3a4KUl/4x77Fz0p/9zbDF6gjf/NnFb6lD/Vnbl6rf/37dAjVi6X
pvfRi2Z+m1wrjiMplsAhXPI2Y9ZlesteBexCrLTzKlzV/5O6K/Sw+Ef2bT3G
6CiqfZPddA9qMMF9Oai1ex1+sn/gOxrVXtCzxD4T5RbRPNor/lJ8CK8wYNZX
+/TTo2b22avKzf20mf9YvMM3e0Jb+APij9jH2Gm9SvWtt0sD6uy6FxvkR80a
QzvG1ZyiWXqansFiO5ZhCXaZX4v3ftdOfgeWm1NpbgUKzbmDblTYeRMGsREp
VOKGmR/0uEm38Dejy7z99qkS65HbSg/wm9Bl34tocuvnxP//xF8D74Gq
          "]]},
        "0.28`"],
       Annotation[#, 0.28, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0j9I1GEYwPHHxPQaDEnMK+/0TggMISgUdKkMB42GMnUs3S6koMEQBXPQ
RRwikIZSTyN1c87BhrKmpg5xDBX6o7UpNqgfaPjyfd/3+Xs/LtP/5N7jooiY
xrPaiBvJiMpsxHU+xx/rIj6hRmyyOiLJaaTQfz5iqSZiGWPy29IRH7yvoUIs
633/YsRX9wMuuH9Drz6r9RGvMhEZM+6oreXbnOIt8xrVbzjnLkSM8CPe5IKc
5pS4fj/0uMLfOWX2LlfxGRybMWjWoRlP+YV+LVzv3soL4te4Ex3ow0FVxC0z
2rFu53Gz3nh/jbPqynHa+3O1i86j/I7X7fsZw2I31QxxG49wsV476ufd79o7
z9vux/b+a95Vu65469LnPrqxJ75q3x7nZj2WzWniJS7TL4FSXFL3Uu57uXNy
85hBTt4sF/QvER9wfys/7RvmeQHzmBBLiE35bjt6zdkp533N+YH6h+jDobx/
+KJ+3O+8LL8Bf/Q/UvPLbr/xE3vYRVaPU9n//6kTMERdTA==
          "]]},
        "0.266`"],
       Annotation[#, 0.266, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0UtL1FEYB+BX1LzQ1ky8NaNdIGgn+AHctXLA1jqOtZsQ1EChhdhWTRQ/
gRqIjisRF3lbqjMkIm0UiUJoQKSFDUTiIy4efuf8zznve85MIv0+lS2LiI9M
NUUkGiK+tEZk5YhcYpG/POUZjbzgOf31EVeNEX8YdyavRq9xH5OPI3KJiDXm
mOWNbzPy7EnEvDyXP/nBS7WOkxEnjHHKd47V7Wi2ruZFW8QreS6bWiKKsk7W
cKP2sP4ldQfltHqdMmmeYJMFe9blBq+t/ZLffEsbXz+K6NKvR/0JPYvyNxl1
ar31IZVUU0WJZVYYJcfq3e/k7H/1tmXKvbvVqPS+Bxy45yF5Lu15p/9bhvTd
sf/IewbMO9lljw+MUOueBetFZ7fMt9nhKxVqp+Un6/2y3DwjC+5e0icv9537
bNyuzz/Z1nD/n98CxztUkw==
          "]]},
        "0.252`"],
       Annotation[#, 0.252, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwl0UsrhVEUh/FFJjqMXcPBhFKY+AgGBiSmlIEBDi9mZoxxXMoXoFBiKPkG
SuqcDIhOrkUmRi4hPxk8PXvtvfZ/r7c3PTLVP1kUEaNoqY9Yqoyo5kXeroq4
bIroTke0N0ZMqTu4E+8NEb0VEcu1ESsYtzeBMUwi82cZm+5m+ETOvjsH+KyL
SOx98DTP6J39y8c0EnzJXMOP/m98IfH+kLNd3jPjDpfyvIwSXuAUl3vzpSai
R30mY8A6Z31thlscOt9SH/ExmrHsW7r0ZjnhN3vfeu70F3vniZ9RY+YCt8l8
5Afknd9zjq/MN4dz5NEq78bsLVzgdbmrGJS9wSlcuJtVD8s89Xaf9at1Wfr/
n/wCLOJG1w==
          "]]},
        "0.23800000000000002`"],
       Annotation[#, 0.23800000000000002`, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdVX001Wccpw7ZbUK02pRi63hba46JrWOf5iYuS1KRRRnVZLIUR7gllUh1
DL/lUm6KishLXi4Lj6QYl9a0rpIbufI6Kb0sWq17e55/+p5zzz3PeZ7f5/l+
P8/3+/kY+//isWWKmppa5Nuf8v9djMHQmz/uks0nhU49UQaFj5DiUFayvWAV
8Umqf2bm8wgax2faNd1dR3iy7B32vEdoO7XPdlS+gVQaxQ2vrhxFy9oIw8TH
fsRABTAKba65ue/ZVhJqr/ziH7i3z4v6biKYtEqVMQLB4UrOLSaUmKsuGMGQ
mX+wMDaMKNHeQuJI0/TlFw5EEGU2Tj1D+Dyw3LAjLoqo4OyH0Dpt07hGwh6S
ka6MQUjWHOz72HEfeW6mBBxAwqfeN7+8Hks8VAkOYP34ohonpwO0vn6YX52S
t7HpIK2nHy9TZFy44BDZqoqHMCs+O5LeHE+uPlMC9sGrddfyWtfDZL4qwT4o
bJdqjUkTSbSqYAXKp+k+1XQ7SlRwsl6YJg7v0r1xjFirohddobcM36xKIio6
kx6gunKA+Af+SkZU5fbA7n7vealZMnF+VzD013RM8geTSY4SLrsbHxRVpvbm
pZB379UN0/6oZBKUSvmXY94Zw3y5JUf57oK+b3XDfyMc5fce2p3iPKUXf6N8
3sVXh7gJ7ZDjlL87yJ6YU7N5URrlqwOtrjprZcNplB8ZBM+1bPgXRJSPvxH7
Mvxx07Z0Wv8txE1rLYy2zKD1tkMvMtY5fCyD1ncTcybvfWJSeYLW8ycsIy3V
+6NP0vzbkO3u53rHOpPmK0WnPEQwoyOT5vcHjrbxJbl+YppPI6ZYzP3O4pqY
3t+Aux1LbutMiul9V1AcMFPh+UJM8WshMvfQLC0UU7wqSIxu7A6exb4vQ+5V
nkT4WSY9X4jzK/jGvo0n6Pk8HFC/35SWn0HP5yA/BBY6c9PpeTFObWs3GVmY
Rvc5/K6to8mv58jE3qDqsNQEDA6Le3b+mExe5T3+a6X/PljJHd1mORyj6x1I
5CcLTsfH0bUfzEhWd83rGLp2x9QKefN2z910vQy2wRu4Owt2EbX3gu1/0Wip
a28eAPa9qUCY6poQAobvVlV/2achAuz+nzmY56zYD5bfA03h3jnGR8Hy5z+5
37pZkAJWn0dMTf3+yXSw+jttjC9aDWSB8TPf9lLPKa0cMP4yE3lHluWfA+N3
2Yq+luLnuWD8+/30ckeMogDsfbgWm8s5C4rB3u9h9m1Hw+QSsPet3uB9plR6
Cez9BVVZAQWdpWD9YVb0zezhujKw/hEJIj1aosvB+mvGm+V1Qu0KsP4TyLmV
BsIKsP40kIr227RVgPVvY+hx5/U8CVh//+sXkHtriQSs/7sUPoufeEnA5oPn
296qs10CNj8iLtRBFiYBmy+XqYqx0bf7bP6SXDyHXntKwObToSB7lftiCdj8
Or/26656UQE23/nh8VWCEpa/HEZP9TJ5vhVg+uDdsXvdh6/KwfQj0cu70yCp
HExfmu8ZBzl9VA6mP81Hvq8rSi4D06cHEpndavUyMP3aWViNr7eUgunbqGmA
UFTD+FdAunR89vzpl8D00XahzkSCWwmYfmpcX1cnSigG01cjNamuSVgRmP5a
cN2BJo0XwfS5LLThByGvAEy/o62Dal2+vQCm72lXamTl/rlg+r/HK89xMOIc
mD+slOn7zt6bA+YfmRvb9MThZ8D8RUvjbGDfxiww//GtrBmtDTsJ5k/X7GL0
eXoiMP/qyt1kzUtNBfO30ojh0ybjx8D8b8LqpPH0+INg/hhvez3PShgD5p+J
CstYriMczF/VazurQz0C8Z7/4n9EbM4S
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{0.95, 1.05}, {-0.03, 0.03}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 313},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->183533631]
}, Open  ]],

Cell["\<\
The flow field near the trailing edge is dominated by the point vortices \
associated with the endpoints of the trailing edge panels for the lower and \
upper surfaces.  Notice that streamlines are closer together near the lower \
surface than the upper surface.  By placing the cursor over a streamline, its \
value is displayed.  This tool can be used to follow a given streamline as it \
approaches and leaves the trailing edge.  This flow does not satisfy the \
Kutta condition that the flow leaves the trailing edge smoothly.\
\>", "ExampleText",
 CellID->38752132],

Cell["\<\
Repeat the computation after altering the trailing edge by making the \
trailing-edge points coincident.\
\>", "ExampleText",
 CellID->254018838],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rTE", "=", 
   RowBox[{"Mean", "[", "ptsTE", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "\[Rule]", "rTE"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "\[Rule]", "rTE"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gA", "=", 
   RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "aW", ",", "rC", ",", "vC"}], "}"}], "=", 
   RowBox[{"PMDoubletConstantDirichlet", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "cl", ",", "cm", ",", "\[CapitalDelta]cl", ",", "\[CapitalDelta]cm"}], 
    "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Doublet\>\"", ",", "a", ",", "rP", ",", "\[Alpha]", ",", "rC"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{"cl", ",", "cm"}], "}"}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->230503503],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9661805505780308`", ",", 
   RowBox[{"-", "0.3249486575058804`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->97863317]
}, Open  ]],

Cell["\<\
These are plausible values for the lift and moment coefficients.\
\>", "ExampleText",
 CellID->88671999],

Cell["Plot the flow field near the trailing edge.", "ExampleText",
 CellID->211503654],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"{", 
      RowBox[{"xW", ",", "0."}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aAll", "=", 
   RowBox[{"Append", "[", 
    RowBox[{"a", ",", "aW"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Psi]A", "=", 
   RowBox[{"IC\[Psi]DoubletConstant", "[", 
    RowBox[{"rF", ",", "rAll"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", "\[Psi]", "}"}], "=", 
   RowBox[{
    RowBox[{"UniformFlow\[Psi]", "[", 
     RowBox[{"rF", ",", "\[Alpha]"}], "]"}], "+", 
    RowBox[{"ic\[Psi]A", ".", "aAll"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gStreamlines", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "contours"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gStreamlines", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", "\[Times]", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->78818121],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxsnQeYTcf7x1m9B0kQvXcS0YUhekkE0XuJ3tbqLLbbxbJrd7GrdxK9E44u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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{4546, 4927, 6556, 4499, 5345, 4731, 6583, 4732, 4659, 6437, 
          4983, 6435, 4982, 6436, 5466, 5467, 5062, 6613, 5061, 5897, 5858, 
          5859, 5660, 5661, 4412, 5339, 4411, 5659, 5056, 6107, 6108, 6109, 
          5516, 4490, 6372, 6373, 6369, 6371, 6370, 5342, 4410, 5658, 5857, 
          5856, 4843, 6592, 5690, 6106, 5895, 4919, 5894, 4918, 5341, 4724, 
          5855, 4842, 6301, 4841, 6637, 5893, 6642, 4917, 6641, 5972, 4350, 
          5973, 5892, 5581, 5689, 5688, 4489, 5423, 5687, 5686}]},
        RowBox[{"-", "1.8681`"}]],
       Annotation[#, -1.8681, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LkuhGEUh/EzRs3sZirR2ZcJhZtASOglWhI6XIBSVLaKa6Ck1mlmzGKP
rZCInvD7iifPd877f8+cd/qW1+ZXUxGxgq9MxC/a5YiOnog06qWIBm6wkYsY
KUZM5iOWeLAQ8SD7iIZsE1Nyl5WIF9mm7wKKmM5G3MqPujfn/MKMWb7kc/0z
ZN1vybZRwon6NOk5q6jL6MGr2Tn9PK7Mm+E7/Xf9W/7ge37jI7sdYx+HOMC3
N+6a2fTdSnbHmD2Gkjdhx6615O1YQM35jzt1vsGf2Z92fzJ/T/3M13rrPOz+
uFmL3O83BjChDnOqnOJeuS75bmRQVW9xmje5k7eT/1z2HwYVPzQ=
          "]]},
        RowBox[{"-", "1.872`"}]],
       Annotation[#, -1.872, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0d1PzXEcwPFPGCH0cE4PrqpRqnMaTqO5lGkLXdjcJe10S+bOPWVkHubg
ToiN2orxPzCU9aBNKjZ3ZeYf0Hr9Lt57f7+fp+/n/E5d/vLZgaKImMCf0ohC
dcRDPMBjPMLv8oj5yohvmMEcZrGm/lRVxOZ0xE815/gXb+Er/FXNCk/xMk9z
l/qP8nt4N5bEd4qXYBcWUhHPxJ9ih/sP+S98sSIiVRNxzz738UR+BEfL7Kbn
CG9X16oux/vk9qMRDbghf13/IK4hpWeIi7jJPgfw3blY7zZsxaK3Z73VYuYM
N/O6+H/sdd8kP27GIffDGPJGgzmNyKqdU5Pheb5r9j/f67RdziCTzEW9GcPJ
dxI7zm1iORScK1Anf4vzPMLvzHmPnLlL6lawjEGxO/LtdijnMtxErb5JO7Y5
F3tjVe0xvR3uJ3Bb/UH7DnOzfAua8EHNefEe9OKqeyb57/ikvk4U9FyQ60Or
3Ce5LH/mfrE80u6VyW9W/1L9gli3c5XYc3u9Eqt2zjqP2u0FJt3f4C1eJ+/J
jXEnD3Cp+CVOu//1TTcAjVJd+w==
          "]]},
        RowBox[{"-", "1.8759000000000001`"}]],
       Annotation[#, -1.8759000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0b1L1XEUx/Fzrw+zpqYuml7NG2KF2iBqD5vmpLbcyILAhoKyQbBGLWjw
Ac1F0JYWEx30DxASQVN7QBNtcw6MtE0bfDm8+Zzv95zP53x/91Y8ftH1PBER
H9FfEHFQGPH6QsSnoog5dDtP5EekiyMKMXEx4j1acRNHJZTvUO8P6nlanR+V
RqzSB/QhenCYFzEq7xvPZb2vtIZW81ThrT3XZTa5u0Vv4w72eZr1W3Cgl3GX
w5uLAXevcGTmGAvqefSqn6DNfNjdTqd816S6zJ4ZvYQ6iR9y0vrf6RW6pLeI
GXs+4B3KeHPoOOrN1To3q1uQOf+95PyTe5//mM46Z9Rr3vLbdw/LG0Ha7Lbf
aQeD5jpkDdHPZrt5Vug9+pI2mTmhWzK61J34q5dyt05P7F6jezJH5OzTDXd9
dMeuKnOnztX0P/3JX0eXeX6Z2VQHEtg1/0XGBtaxhU288bar+teQRBYa3O2Z
T3lXO57Kn6bPaKUdKZSjApdwVy+BJIr5smU0ythW3zj/L8xMY8y+URR43xn5
41uh
          "]]},
        RowBox[{"-", "1.8798000000000001`"}]],
       Annotation[#, -1.8798000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0UtIlVEUxfEdDYRG+aycdK9QjXsMKsoelAWVpD2IcmRNUlCDCrUUmlR6
I0rJwtusUUSTQkvrVmAPGvQwrVmJQVD4BI0Kg/rdwZ919tprn3O+8yWr6yvr
5kXEM8zlRkwtjpjG2KKIcYwURDTmRVTwmuhh3hFU4aZeuoiiFQX6LbSQ7kbP
kohddDY/oty6l+6hD+ipwoiTWKW+rF5J12A1atyjTa+Wdqv79Nc7a6n6Nt1m
z07eoF4HbZad1PuLFO/Hwoiv/Bx3fsobtb7Fz1h3yj6hZfa4z9tOE+o07cZZ
/evyH/Ru0C6c4a3FOnxWH/KN8+39yNyk+5zmT9Fp5OCOupoeRZn8jOxPzOKa
M457uxrUIi2zkV+KDdiMTei3d5VsLu66Sx7dinvWW+h5c/l0xv7jsl3OnKAH
eb0yB7LfIvON16fXj1Z1Cx7zrqo7MGK+Qf4LPZF9a70r/If0mGwRdqp3oNh6
ATJyn+yfyb4lAv/4H3kpd7+Ei2hHG777H+36qew/dM4b3/0W75Awe85Z760H
sRcvzcyZeUV/eevfGMALPEedc5LmEvaqpMvsuxxJ9R/9Mfcbkrlgfpi+5tXT
EtkSmf10hfw+2qAeoOXq/2TveQ0=
          "]]},
        RowBox[{"-", "1.8837000000000002`"}]],
       Annotation[#, -1.8837000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ktsjGEUxvHXJUETyXRQrYWkVmLHioXqdCp0ioS9Sxt1SahoiZX7LRpC
UlVqo9WmkpZIBNXEpRbaaplhIyFCgikJ0tpYMMPvW/zzvM85zznfO983pbV7
NtRPCiG8RzYWwoniEE7iOF7HQzhF03SoKIRX9BnN0EFaNTeE3OwQXvDr54Tw
kub53fSp/igdoCO0sCSEOGLYOSuEJ+rP1XPm8viHjNmrdrbhLz+s/1huh/wx
9xjiL+tdwZJCz5NfTFv5P/KPZB9iJn9b74jeqPMI5tvx1Y5vyGIpP0ZX0Vr9
GmzBGc9oV+/ANfzgE3aVo9MzctEe5zFs47N0tef0mu3BT/lqmWG/M0Xb9Bep
nXNOq52ljebum4mpF9Eqme20O3oX+tV8KtqvNkOtjt6z+y7yZi7ptUR38L2a
7KxRy8ockD1vPqN2gSbVu+WK9ZrVWvkx53a5DizTm8f30BLaKLMPDfhifj/9
RC/KDup38Z/5dfYkvZuVqMRHvRaZD9FvkBmX+YU3zhO0kt6gm91/EzbiIF9n
ZiuS5hb4bv1qfXoP0I8mvV16C/UKnKdG/xu6xn37nAfsnUxvucMU2kNvohdl
5g7LLqdl8h36h/jv7v7b3HV+hfo05+koNBPHhHxCbjz6Pvxa+QpagAb5d+p3
9N/SvXzauYJWRe/HOUWTfKk7H+U7o+9gfxctpxVIoFnmNP4DJSCNSw==
          "]]},
        RowBox[{"-", "1.8876000000000002`"}]],
       Annotation[#, -1.8876000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV01dsiFEYxvEj1Kpdq1p7xIpxwZ0ZpWarolYqVmwViXFBqHlh76S1m1Ik
toRIhVKbUErChURiE70xqubvu/j3ecfzvud8p23LKfPTMiuEEJL8eFMnhORG
IbyID2FA3RCOiJNo64YhZCMH3eqF0KNxCN1pT7pVf1yDEBbGhZBgrikS0Vq/
JW2FZmiB5piAbWY+2v0Bo+xIQ2f1/fQAjqkfxyC+JPWB9Kg8H+dwFhPVfjgj
g1Z2fhWUyQ+bz8cR1Nc7Xj+EOJouH4PR2MkXY29lVLdrrLtXEseKa+Cg78zF
IaTxrtF/oL+afrLvcfQ28mHyGP1SDFf7rLdRvAmp8hN2nUQeDqOPWpZ79qUt
7B7AnxLtUhtBV5pbgSzURIK71qKJtKLzRjjvGV8v/qd0uPyxON2evfJN4n10
qN3v9O6L39N7dKQ9g3HeniE0GXniIr2U6L6oji3Rm6i9NRdLt8qXiZdiqnga
JmEKJkfqrLZoh5LoXqhtbrNdl/S30EVYjI3yf/7GKth1wXvM4vumPpbGmBtH
v8v/IfDMjN6G9qOr9PvTfHlP39ubJ1+eQWNRA8XyTs5YxdOedkQHzDSbqzdX
/FRvDn1Gy/ADVc1WwzGedfSi/lr62jnZavvkd/ja+MZh8ln2zcZatW16tdS2
89akg/yukjEQQzAYJeqn+Z/QTL4n4rn0ofyU+BGdJx9v/wxaKM9R34N4+Stv
Vo7F4iUodW659/uF39jMd9bMX/EfzHenTKTbNxov9abTJnrxWMB/Ru2nuAzP
7ZvHv8E3f7B/PT3KkyJORZreSDRCYzREE8RHmP1CS/ERN6K/Yd98kz4wex/f
orP5vtIE+p3edm5nWmDmFm+B+Kczy/HQzFU7Enmv0GsoxHJ3vmPuLnZgJ7qo
FfM3M9ccIfp/x3N769EV+gX2lMgv06a4btcNFKGr/j2+bnS3fbuwUj0Ltc38
B7DPucs=
          "]], 
         LineBox[{4662, 4502, 5225, 4553, 5533, 5115, 5164, 4357, 5268, 4358, 
          5165, 5116, 5534, 4504, 5204, 4503, 4663, 4753, 4692, 4569, 4950, 
          4951, 4948, 5438, 5733, 5439, 4949, 5567, 5191, 5192, 5508, 5507, 
          4437, 6476, 5040, 6475, 5041, 5511, 5510, 4439, 5039, 5173, 5036, 
          4438, 5509, 5035, 6474, 5038, 6472, 5037, 6473, 4386, 5446, 6412, 
          6413, 6411, 4967, 6414, 4568, 5553, 5554, 5232, 5231, 4518, 6561, 
          4669, 4752, 4662}], 
         LineBox[{6131, 6129, 4506, 6525, 4507, 5207, 4516, 5428, 4515, 5074, 
          5166, 4361, 6192, 6193, 6191, 4645, 6190, 4646, 4644, 4442, 5205, 
          4443, 5123, 5947, 5122, 6526, 5948, 5949, 5704, 5706, 5705, 4427, 
          6130, 6131}], 
         LineBox[{5237, 4573, 5558, 4572, 5556, 5557, 5236, 5235, 4522, 4671, 
          4754, 5987, 5988, 5989, 5284, 5992, 5991, 5990, 4755, 6228, 4655, 
          6119, 6120, 5701, 6156, 6157, 6158, 5729, 5582, 5946, 4550, 5224, 
          5728, 5237}]},
        RowBox[{"-", "1.8915000000000002`"}]],
       Annotation[#, -1.8915000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ltMz2EYwPHXFjI2hA5Om8PmCpNSNsxG3ZZqk0PnSDLd0FE5XnDDXMQd
0Y1jDtUF4cJ5iFzVnFejhY3NhpTx+V98932f532e531///c/p7gya+eoEMI3
vJsUwpq4ENaiLDaEcrxOCGEbv+G2ySEk2ns1JYR/HOJDGIVj00Jot5evrgB5
GC//QO6tvpPiAV4UE8InPiH+yE3cq7dD3QF7R+WirJunhpAsPoSD6Bc3ya92
7mOeaHaSfAp/c4/vSBVf07+cn6gpMrdW/VO5Oj5rxhmcwmn0yGU4PxOX1S9T
38ob7W1CFDYgF8eRjwKMQREKcUl9sr6L/MsdfiLN3G65sXItaqL5pbhXfp2z
spCNFLkr9jarycNorNJ/VS7Hfgz3qEmTz7I+bb3EOhE54iReimb5bHG6daNv
7+MGbpFfYb0y8ts5+7zfakj+HP/hCzwSeQ9nDfNA5O3MaXeHQl4gnm3dprdY
fNi8Ir4uXmxmB5eIo+VL+W/kPeRTsd46F0PyaWak473/1kJnfuBqb1KDKnw2
p5YHuZ6/ch1/4VJ3KEGFmc/NanVWF28XT+BydS/UbeNu3s7PuIyf8xbu4q3c
59x9PE7vXj6A/YgWj2AY9925wR0bUYc9qEe/3iP2b1nHmtfJcTzdfRLQ4C4P
feMjZOCG/d96bvJuVGEXalCNCufM0DcTE82diyH1t+3dwQ/fPOhud9XViu9x
p9wOnqUn3tmZPE/ffFSK/wOr748J
          "]], 
         LineBox[{5102, 5488, 5487, 6446, 6447, 6209, 4679, 6152, 6153, 5723, 
          4547, 5724, 5578, 5579, 4324, 6537, 4602, 4534, 5490, 5000, 6606, 
          5001, 4672, 5285, 4373, 4698, 6220, 4699, 4623, 6546, 4369, 4944, 
          6602, 4945, 4943, 4947, 4946, 4567, 4691, 5168, 4848, 4433, 5357, 
          4849, 4850, 4852, 5860, 4851, 5861, 4368, 5271, 5272, 4999, 4998, 
          5483, 5484, 5076, 5936, 5075, 5937, 4444, 5485, 5486, 6524, 5101, 
          5944, 5102}]},
        RowBox[{"-", "1.8954000000000002`"}]],
       Annotation[#, -1.8954000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DtPlHEQxeHhIwgiUCCxkZIQK1fAqBAaA4HSkqgkkEhBIxAEbDTKRemQ
y67fAdxd7iRKIKEhfAC5BxFsRLxUPm9x8pszc+ad/+6Ntq7WZwURMUmzRREf
aKeMxwmqUU9jCqfwDpZciei9GnGAueKIQ+zjO65F9OA+n9Xfw+f8F/UAnvAL
6m84yHfKv8Tv/KL+KQ7x5VjnTsa9WvyId3EG++Uq7F2nNF9eElGafKPU2/iv
/C7tUXXyNjygfaoxP5NrkT/HH3Skf0xjbj5yYxRbzYupwX49PbZ3j2aSN5jf
0ktjKrlJtylVGPGa8jKv8Mh+RibN37e7qn6AmeT3qE/NX/Bb/EDyP9upVA+p
b+Igtrvzy9su6KHfu61Xpa7HWT5r56fMG7239Nk3R3GEhiln3u1tT+Tn5PP8
hfyY2Ttal//NXyZ3aN78k9xT+Swu8H/038uO04b8Ji3pN+kvJnO+0Swvv8zn
cAXnsQP/ma9hs/wq/uXX1f8Bvy1phw==
          "]], LineBox[{6645, 4387, 6646, 6644, 6643, 6645}]},
        RowBox[{"-", "1.8993000000000002`"}]],
       Annotation[#, -1.8993000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzrdOnFEQBtDhDUhLEFBQuCFISETBM9C4slto8bK75FAQbTA0NBZPYFG5
QaIDvAQh8QAIBA9AtqGwycvZ4uj75/4z905tT//HZEFELFNbFHFaEnFUGXEm
v9Pke0n2lUd8IcnP0oi0TNHP54qIT/wojniUT/zjgf/M6n+RrzyT441V/VXm
qzlJRGRklR3W9FfLQfUAbXZoZcUebep2mmmlhU6zSwyb6Ta74ztLr/4udZ3z
1TK7kiJJhjQj/m26Y4sppql31mG/Qf+HGCDnbFiOym09v+xTY/8P+mbUs3Sp
R/WEHJEFckz+zt+Zf19vijTr6js73rNrxwt1sX3PZYm8lGPeypo999aGuzLm
5tTz7Ju51lOq90om5I0cN7Pj/y5f+UaDswP9f/z/yy33+feZ8G8vfx8LLNLo
7FB/ud0evF0hc+4vcnasLpSTet4BOapZCQ==
          "]]},
        RowBox[{"-", "1.9032000000000002`"}]],
       Annotation[#, -1.9032000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0EcyhFEUBtBLmYndQmmhaZS0FwYmqmRNy2GCFUgDC2ALLU2kvaDEFYhl
QDn/4NT33/e+2/Wqc/n1obWSiNhiIh0x0BgxKQdlo8zKjNxpiCj1PSyLmYjf
lHPzgfnR/MwTPbov8pUHmsxtes1yV7fM93ZdxJ/9w3rnvltI1UZ8yaLekfNj
+W0uyEpzFT86rc7eaiIu/M4oY4wwwThz+tW6NZTrZnnXv3SXo4N2uuhkwztO
dE6Zsrvnzbf6++Z72aPTSzf99LFp58z9ebLjTft27nQ/uHI/TZ4pZplh3m+n
vClNhb02PvWvkzcn9xRYZCGhX6tbR6VuO1/6N+6WWWGJNVaTPf163Ybkv9LN
8a0/YP4HYwQ9Ng==
          "]]},
        RowBox[{"-", "1.9071`"}]],
       Annotation[#, -1.9071, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0TcvhWEYxvHbyKaTMBh9AkzKymTV4jAqk95LYhIGn0CvR19ERHwPvXcJ
dj/DP9dz39f/PW/e8xQlOms7UiKiB9M5EaMYwwgmMI6f9IjMvIhJ59yMiELn
HHmRH1GeG1GBJd283aJckMuyRf5k6WR1tk5+myv5q/pl84pckWuyVf7q12QN
f1WmeUd9ZkSqrJPrvE37c32X3+nGkWeuzJe4wJZ+g9fA35RJ87V9D7cXx/w7
8y1usK3f4jXym3zbjrmP148BnPAfeQ+4x64+yW/ib8s985P9IHcIp/xX8wue
sa/f4TXzd+WB+c1+mDuCM/6n+QPvONTv8RL8KVlg/rKfcZ5FsbnU/1GGEszx
KvSf7qlN3/5/T74j7/+eUOUdfzaSUHA=
          "]]},
        RowBox[{"-", "1.911`"}]],
       Annotation[#, -1.911, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz8kuw3EUxfFbojXUrIalxFTDS/AARKKtrb154Sm6NDTm2EoUC6REjA+B
p5AUS58uvjn3nPO795//4NLq/EoiIpZR1xmR7Io4H4hI0Srf0x+RQa4vIo+Q
ffMJusgXMMfPYstOwe4iFpBHDjOZiGks27n3vlKjO+JDNiS76Yko27/EBa5x
hRXdgZvbbgybb71rsPfMJ+mq7FC/w4+Y7/Qp+QvfSNdkR/pdftRc0TfJX/lm
ui471u/xY+YHfYv8jU/TDdmJvsRnzY/6Vvk730Y3Zdled/lT787Mn/7pC0+1
W7S99j3aQdN03JsX876dov2CbKN2C0VZvaxkd9K9Mj9Ff2Q/+j+66/0vrfIT
3v0DwK88JA==
          "]]},
        RowBox[{"-", "1.9149`"}]],
       Annotation[#, -1.9149, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdVX801WcYpw7ZbUK02pRi6/i11hwTW8c+jcTNklRkUUY1mSzFEW5JJVId
w10uRYqKyI/8uCy8kmJcWtO6Sm7kys9J6cei1brX9/mn55x77vc97/s+7/N8
nuf5fAx9f3HbOkVFRSX83U/xP2mjoK8Cx+4IvYLHSLIrLd6Rv5p5JdQ9N/F6
DLUTM20a761nPGnWTlveY7Se3m89ItvIKgxihtZUjKB5XZh+/BMfpqd0MAJN
YVNT7/NtLNhWceMfuLbNi/huPJC1SBQ2DP6RCqFLVDAzVT4wjEET30BBdAhT
eHvnEkcbpy+/eDCMKaJx7B7E5/5l+u0xEUzpznYQLdM2j6nF7WVpqQobgHjt
od6PHfazFyYKh/2I+9Tz1pc3opmbMsB+bBhbVO3oeJBN5tcH02tTcjc1HuLy
6cOrJKkwlH+YbVPaI5gUnRtObYpl154rHPbCo2X38hrnI2y+MsBeyK2XaoxK
4lmkMmE5yqZpP1N3OcaU7qQ9MI4f2q198zizVFoPOoNv679dncCUcCY8RFVF
P/P1/5UNK9Pths2DngsSk0TmNJkwdNe2T9gPJLJshbusLnxQWJHck5vEJqvU
BeO+iEQWkMzhL8O8s/p5MnMhh3cndL2r6v8bFnL43kebY4y75NJvHJ738NVh
4bhm0AkOv7vIGp9TvWVRCodXO1qctdZJh1I4fKTgv9Cwsr8o4vD4G9GvQp80
bk/l8r+NmGktBZHmaVy+bdAJj3YKHU3j8ruFORP3PzGqOMnl8yfMw81V+yJP
cfG3IsvVx/muZToXrwQdsiD+jPZ0Lr4/cKzVXpzjk8HF04ApZnO/M7uewb1f
j3vtS+5oTWRw711Fkd9MufvLDM5/DUSmbuolBRmcv0qIDW7uCZxF90uRc40n
FnyWzp0vwIUV9obeDSe587k4qPqgMSUvjTufjbwgmGnNTeXOZ+D09jaj4YUp
3L4Qv2tqqdvXCdn4voCqkOQ4DAxldO/6MZG9zn3y1yrf/bCQObjMsjvOrXci
3j6RfyY2hlv7wIRldlW/ieLWrphaLmva4b6HWy+DdeBG4d0Fu5nKe0b7XzSY
a9ua+oHuG/MFyc5xQSD/LpV1V7zqw0Dv/yyEafaKA6D4HqoL9s0xPAaK3/7p
g5Yt/CRQfm5R1XUHJlJB+XdYGV6y6M8E4TPf+nL3aY1sEH7p8byjy/LOg/Bd
tqK3uehFDgh/n59e7YyS54PqI2y2upK9oAhUv0dZdxz0E4tB9a3a6Hm2RHIZ
VH9+ZaZffkcJqD9MCr+ZPVRbCuofET/crTmyDNRfM94urxVoloP6jy8TrtIT
lIP6U08iOmDVWg7q34bgE04beGJQf//r45dze4kY1P+dcq/FTz3EoPngebe1
aO0Qg+ZHJAy2k4aIQfO1cqp8dOTdPs1fwkr3wTfuYtB82uVnrXZdLAbNr9Mb
n67Kl+Wg+c4Lja3kF1P8Mhg800nneZeD+MGzfc/6D1+Xgfgj3sOzQy+hDMQv
TfcNAxw/KgPxT9PR72sLE0tB/PRQLLVZo1oK4q9dBVX4emsJiN9GjP0EomrC
Xw7J0rHZ86dfBvGj9UKt8TiXYhB/qt1YXyuKKwLxq4GKRNsopBDEv2bCLn+j
hksgfi4Nrv9BwMsH8XekZUDNym8vgvg95Wq1tMw3B8T/ez1yHQbCzoP0YZVU
13v2vmyQfqRvatXJCD0L0hcNtXP+vZsyQfrjXVE9UhNyCqRP122idHk6IpB+
deZstuQlJ4P0rSRs6IzR2HGQ/o1bnDKcHnsIpI+x1jdyLQRRIP2Ml5tHC9tD
QfqqWtNRFezmj/f1938r+8dg
     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{{0.95, 1.05}, {-0.03, 0.03}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 313},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->458229526]
}, Open  ]],

Cell["\<\
Evidence of point vortices at panel edges show clearly in the two streamlines \
nearest the airfoil, but overall, the flow appears to leave the trailing edge \
smoothly, satisfying the Kutta condition.\
\>", "ExampleText",
 CellID->667775740]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[{
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMVortexLinearNeumann",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["PMDoubletQuadraticDirichlet",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["ForceAndMoment",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/ForceAndMoment"],
   FontFamily->"Verdana"]], "InlineFormula"],
 " . ",
 Cell[BoxData[
  StyleBox[
   ButtonBox["PlotSurfacePressureCoefficient",
    BaseStyle->"Link",
    ButtonData->"paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"],
   FontFamily->"Verdana"]], "InlineFormula"]
}], "SeeAlso",
 CellID->110623922]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["3.1 Introduction To Panel Methods",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["3.2 Validation Example",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/ValidationExample"]], "Tutorials",
 CellID->499723042],

Cell[TextData[ButtonBox["3.3 Implementation",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Implementation"]], "Tutorials",
 CellID->336290191],

Cell[TextData[ButtonBox["3.5 Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]], "Tutorials",
 CellID->351529306],

Cell[TextData[ButtonBox["Table of Contents",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/TableOfContents"]], "Tutorials",
 CellID->8122694]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "MoreAbout",
 CellID->107272168]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"PMDoubletConstantDirichlet - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 58, 18.6102000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics", "status" -> "", "summary" -> 
    "PMDoubletConstantDirichlet[r_p, \\[Alpha]] implements a panel method for \
uniform flow over an airfoil using doublet panels of constant strength and \
internal Dirichlet boundary conditions. Panel endpoints describing the \
discretized airfoil are located by r_p, and the angle of attack is \\[Alpha]. \
The function returns a list of strength parameters for the doublet panels, \
the strength parameter for the wake panel, locations of collocation points \
and a list of surface speeds at collocation points.", "synonyms" -> {}, 
    "title" -> "PMDoubletConstantDirichlet", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/PMDoubletConstantDirichlet"}, "SearchTextTranslated" -> 
  ""},
CellContext->"Global`",
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[12366, 371, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->277908699]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 373576, 7284}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[607, 21, 76, 1, 70, "SymbolColorBar"],
Cell[686, 24, 191, 5, 70, "LinkTrail"],
Cell[880, 31, 3998, 88, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4903, 123, 69, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4975, 126, 1403, 35, 70, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[6415, 166, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->149220581],
Cell[6998, 185, 251, 8, 70, "Notes",
 CellID->1067943069],
Cell[7252, 195, 648, 23, 70, "Notes",
 CellID->275911084],
Cell[7903, 220, 740, 25, 70, "Notes",
 CellID->6298250],
Cell[8646, 247, 409, 12, 70, "Notes",
 CellID->252999978],
Cell[9058, 261, 519, 14, 70, "Notes",
 CellID->631099542],
Cell[9580, 277, 668, 22, 70, "Notes",
 CellID->86481803],
Cell[10251, 301, 439, 17, 70, "Notes",
 CellID->490569405],
Cell[10693, 320, 548, 16, 70, "Notes",
 CellID->509195412],
Cell[11244, 338, 70, 1, 70, "Notes",
 CellID->75751524],
Cell[11317, 341, 1012, 25, 70, "3ColumnTableMod",
 CellID->14117100]
}, Closed]],
Cell[CellGroupData[{
Cell[12366, 371, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->277908699],
Cell[CellGroupData[{
Cell[12852, 388, 148, 5, 70, "ExampleSection",
 CellID->357770921],
Cell[13003, 395, 131, 6, 70, "ExampleText",
 CellID->77267041],
Cell[13137, 403, 100, 3, 70, "Input",
 CellID->326492779],
Cell[13240, 408, 302, 8, 70, "ExampleText",
 CellID->48545141],
Cell[CellGroupData[{
Cell[13567, 420, 369, 12, 70, "Input",
 CellID->317289154],
Cell[13939, 434, 220, 7, 36, "Output",
 CellID->335608642]
}, Open  ]],
Cell[14174, 444, 264, 5, 70, "ExampleText",
 CellID->167522],
Cell[14441, 451, 217, 7, 70, "Input",
 CellID->493524749],
Cell[14661, 460, 85, 1, 70, "ExampleText",
 CellID->20057380],
Cell[14749, 463, 147, 5, 70, "Input",
 CellID->163234459],
Cell[14899, 470, 662, 22, 70, "ExampleText",
 CellID->442126641],
Cell[15564, 494, 268, 8, 70, "Input",
 CellID->53812608],
Cell[15835, 504, 521, 11, 70, "ExampleText",
 CellID->205450438],
Cell[16359, 517, 304, 6, 70, "ExampleText",
 CellID->99130043],
Cell[CellGroupData[{
Cell[16688, 527, 210, 6, 70, "Input",
 CellID->544103304],
Cell[16901, 535, 164, 5, 36, "Output",
 CellID->412113092]
}, Open  ]],
Cell[17080, 543, 189, 4, 70, "ExampleText",
 CellID->33026124],
Cell[17272, 549, 363, 13, 70, "Input",
 CellID->81885161],
Cell[17638, 564, 622, 16, 70, "ExampleText",
 CellID->19233943],
Cell[18263, 582, 659, 20, 70, "Input",
 CellID->133172956],
Cell[18925, 604, 483, 8, 70, "ExampleText",
 CellID->76299467],
Cell[CellGroupData[{
Cell[19433, 616, 1027, 26, 70, "Input",
 CellID->808226483],
Cell[20463, 644, 89426, 1530, 70, "Output",
 CellID->325168051]
}, Open  ]],
Cell[109904, 2177, 759, 12, 70, "ExampleText",
 CellID->14395635],
Cell[110666, 2191, 269, 5, 70, "ExampleText",
 CellID->80454523],
Cell[CellGroupData[{
Cell[110960, 2200, 431, 14, 70, "Input",
 CellID->499718542],
Cell[111394, 2216, 260, 8, 36, "Output",
 CellID->145104453]
}, Open  ]],
Cell[111669, 2227, 283, 5, 70, "ExampleText",
 CellID->119738758],
Cell[CellGroupData[{
Cell[111977, 2236, 89, 3, 70, "Input",
 CellID->288057467],
Cell[112069, 2241, 191, 6, 36, "Output",
 CellID->263944]
}, Open  ]],
Cell[112275, 2250, 121, 3, 70, "ExampleText",
 CellID->753558053],
Cell[112399, 2255, 162, 6, 70, "Input",
 CellID->435840926],
Cell[112564, 2263, 276, 5, 70, "ExampleText",
 CellID->139054009],
Cell[CellGroupData[{
Cell[112865, 2272, 447, 12, 47, "Input",
 CellID->671472236],
Cell[113315, 2286, 7153, 125, 351, "Output",
 CellID->73750055]
}, Open  ]],
Cell[120483, 2414, 304, 11, 70, "ExampleText",
 CellID->571494656],
Cell[CellGroupData[{
Cell[120812, 2429, 613, 19, 70, "Input",
 CellID->42630738],
Cell[121428, 2450, 339, 12, 36, "Output",
 CellID->262461228]
}, Open  ]],
Cell[121782, 2465, 365, 7, 70, "ExampleText",
 CellID->24035690],
Cell[CellGroupData[{
Cell[122172, 2476, 318, 10, 70, "Input",
 CellID->34907031],
Cell[122493, 2488, 336, 9, 36, "Output",
 CellID->19329667]
}, Open  ]],
Cell[122844, 2500, 321, 6, 70, "ExampleText",
 CellID->47127959],
Cell[CellGroupData[{
Cell[123190, 2510, 236, 8, 70, "Input",
 CellID->274981366],
Cell[123429, 2520, 258, 8, 36, "Output",
 CellID->73845521]
}, Open  ]],
Cell[123702, 2531, 337, 6, 70, "ExampleText",
 CellID->328772922],
Cell[CellGroupData[{
Cell[124064, 2541, 353, 12, 70, "Input",
 CellID->762946865],
Cell[124420, 2555, 261, 8, 36, "Output",
 CellID->438828141]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[124730, 2569, 221, 7, 70, "ExampleSection",
 CellID->1293636265],
Cell[CellGroupData[{
Cell[124976, 2580, 252, 7, 70, "ExampleSubsection",
 CellID->421192202],
Cell[125231, 2589, 213, 5, 70, "ExampleText",
 CellID->5610347],
Cell[125447, 2596, 154, 5, 70, "Input",
 CellID->559405563],
Cell[125604, 2603, 190, 4, 70, "ExampleText",
 CellID->444428628],
Cell[CellGroupData[{
Cell[125819, 2611, 555, 18, 70, "Input",
 CellID->249332617],
Cell[126377, 2631, 5842, 103, 70, "Output",
 CellID->51606828]
}, Open  ]],
Cell[132234, 2737, 119, 3, 70, "ExampleText",
 CellID->27068391],
Cell[132356, 2742, 437, 13, 70, "Input",
 CellID->343427282],
Cell[132796, 2757, 138, 4, 70, "ExampleText",
 CellID->126580234],
Cell[CellGroupData[{
Cell[132959, 2765, 312, 10, 70, "Input",
 CellID->274596420],
Cell[133274, 2777, 6364, 112, 70, "Output",
 CellID->184133198]
}, Open  ]],
Cell[139653, 2892, 156, 4, 70, "ExampleText",
 CellID->33075010],
Cell[CellGroupData[{
Cell[139834, 2900, 667, 20, 70, "Input",
 CellID->252090154],
Cell[140504, 2922, 258, 8, 36, "Output",
 CellID->88947904]
}, Open  ]],
Cell[140777, 2933, 198, 4, 70, "ExampleText",
 CellID->169245216],
Cell[CellGroupData[{
Cell[141000, 2941, 305, 12, 70, "Input",
 CellID->208999159],
Cell[141308, 2955, 237, 7, 36, "Output",
 CellID->4570479]
}, Open  ]],
Cell[141560, 2965, 388, 7, 70, "ExampleText",
 CellID->511124957],
Cell[CellGroupData[{
Cell[141973, 2976, 639, 19, 70, "Input",
 CellID->59705953],
Cell[142615, 2997, 9779, 168, 70, "Output",
 CellID->20320370]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[152455, 3172, 223, 7, 70, "ExampleSection",
 CellID->2061341341],
Cell[CellGroupData[{
Cell[152703, 3183, 227, 7, 70, "ExampleSubsection",
 CellID->30521753],
Cell[152933, 3192, 449, 8, 70, "ExampleText",
 CellID->120830056],
Cell[153385, 3202, 589, 18, 70, "Input",
 CellID->210128799],
Cell[153977, 3222, 259, 5, 70, "ExampleText",
 CellID->112154987],
Cell[CellGroupData[{
Cell[154261, 3231, 434, 15, 70, "Input",
 CellID->165825942],
Cell[154698, 3248, 245, 8, 39, "Output",
 CellID->35197929]
}, Open  ]],
Cell[154958, 3259, 229, 5, 70, "ExampleText",
 CellID->937232338]
}, Closed]],
Cell[CellGroupData[{
Cell[155224, 3269, 239, 7, 70, "ExampleSubsection",
 CellID->233233938],
Cell[155466, 3278, 643, 10, 70, "ExampleText",
 CellID->381223408],
Cell[156112, 3290, 308, 6, 70, "ExampleText",
 CellID->507899588],
Cell[CellGroupData[{
Cell[156445, 3300, 1910, 56, 70, "Input",
 CellID->49740593],
Cell[158358, 3358, 12041, 205, 70, "Output",
 CellID->141919443]
}, Open  ]],
Cell[170414, 3566, 186, 4, 70, "ExampleText",
 CellID->520516665]
}, Closed]],
Cell[CellGroupData[{
Cell[170637, 3575, 226, 7, 70, "ExampleSubsection",
 CellID->104403840],
Cell[170866, 3584, 341, 6, 70, "ExampleText",
 CellID->391229248],
Cell[CellGroupData[{
Cell[171232, 3594, 938, 30, 70, "Input",
 CellID->451142620],
Cell[172173, 3626, 248, 8, 39, "Output",
 CellID->96248378]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[172482, 3641, 231, 7, 70, "ExampleSection",
 CellID->1305812373],
Cell[172716, 3650, 328, 6, 70, "ExampleText",
 CellID->416625656],
Cell[CellGroupData[{
Cell[173069, 3660, 385, 10, 70, "Input",
 CellID->112529997],
Cell[173457, 3672, 5178, 92, 70, "Output",
 CellID->5545174]
}, Open  ]],
Cell[178650, 3767, 150, 4, 70, "ExampleText",
 CellID->390242666],
Cell[CellGroupData[{
Cell[178825, 3775, 212, 7, 70, "Input",
 CellID->238007975],
Cell[179040, 3784, 344, 12, 36, "Output",
 CellID->48979307]
}, Open  ]],
Cell[179399, 3799, 151, 4, 70, "ExampleText",
 CellID->25666953],
Cell[CellGroupData[{
Cell[179575, 3807, 759, 23, 70, "Input",
 CellID->592619],
Cell[180337, 3832, 264, 9, 36, "Output",
 CellID->169567890]
}, Open  ]],
Cell[180616, 3844, 80, 1, 70, "ExampleText",
 CellID->722709344],
Cell[180699, 3847, 156, 4, 70, "ExampleText",
 CellID->492689119],
Cell[CellGroupData[{
Cell[180880, 3855, 2160, 60, 70, "Input",
 CellID->266881112],
Cell[183043, 3917, 270, 8, 36, "Output",
 CellID->119372697],
Cell[183316, 3927, 104636, 1771, 70, "Output",
 CellID->183533631]
}, Open  ]],
Cell[287967, 5701, 580, 9, 70, "ExampleText",
 CellID->38752132],
Cell[288550, 5712, 155, 4, 70, "ExampleText",
 CellID->254018838],
Cell[CellGroupData[{
Cell[288730, 5720, 1158, 36, 70, "Input",
 CellID->230503503],
Cell[289891, 5758, 258, 8, 36, "Output",
 CellID->97863317]
}, Open  ]],
Cell[290164, 5769, 114, 3, 70, "ExampleText",
 CellID->88671999],
Cell[290281, 5774, 86, 1, 70, "ExampleText",
 CellID->211503654],
Cell[CellGroupData[{
Cell[290392, 5779, 1703, 46, 70, "Input",
 CellID->78818121],
Cell[292098, 5827, 76340, 1297, 70, "Output",
 CellID->458229526]
}, Open  ]],
Cell[368453, 7127, 252, 5, 70, "ExampleText",
 CellID->667775740]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[368754, 7138, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[369174, 7152, 889, 29, 70, "SeeAlso",
 CellID->110623922]
}, Open  ]],
Cell[CellGroupData[{
Cell[370100, 7186, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[370522, 7200, 190, 4, 70, "Tutorials",
 CellID->341631938],
Cell[370715, 7206, 167, 3, 70, "Tutorials",
 CellID->499723042],
Cell[370885, 7211, 160, 3, 70, "Tutorials",
 CellID->336290191],
Cell[371048, 7216, 177, 4, 70, "Tutorials",
 CellID->351529306],
Cell[371228, 7222, 158, 3, 70, "Tutorials",
 CellID->8122694]
}, Open  ]],
Cell[CellGroupData[{
Cell[371423, 7230, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[371845, 7244, 149, 3, 70, "MoreAbout",
 CellID->107272168]
}, Open  ]],
Cell[372009, 7250, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

