(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     50137,       1271]
NotebookOptionsPosition[     44961,       1108]
NotebookOutlinePosition[     46872,       1153]
CellTagsIndexPosition[     46786,       1148]
WindowTitle->PlotConvergence - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACLET SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.5 Convergence and Timing\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"], "\<\"A2 Graphics\
\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Graphics"], "\<\"Table of Contents\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Graphics\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/Graphics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/PlotConvergence\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/PlotConvergence"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
PlotConvergence.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/PlotConvergence", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["PlotConvergence", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{
       ButtonBox["PlotConvergence",
        BaseStyle->"Link",
        ButtonData->"paclet:Aerodynamics/ref/PlotConvergence"], "[", 
       RowBox[{
        StyleBox["n", "TI"], ",", 
        StyleBox["c", "TI"]}], "]"}]], "InlineFormula"],
     " \[LineSeparator]generates a graph showing convergence properties of an \
aerodynamic coefficient. The list of number of panels describing a \
discretized airfoil is denoted by ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     " and the list of coefficients is denoted by ",
     Cell[BoxData[
      StyleBox["c", "TI"]], "InlineFormula"],
     ".  The coefficient is  plotted versus the reciprocal of the number of \
panels, and a straight line is fit to the resulting points.  A reference \
value for the coefficient is determined by extrapolating the straight line to \
zero, and the function returns a list containing the reference coefficient \
and the graph."
    }]]},
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{
       ButtonBox["PlotConvergence",
        BaseStyle->"Link",
        ButtonData->"paclet:Aerodynamics/ref/PlotConvergence"], "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          StyleBox["n", "TI"], ",", 
          StyleBox["c", "TI"]}], "}"}], ",", 
        StyleBox["m", "TI"]}], "]"}]], "InlineFormula"],
     "\[LineSeparator]uses data from the last ",
     Cell[BoxData[
      StyleBox["m", "TI"]], "InlineFormula"],
     " values in the list ",
     Cell[BoxData[
      StyleBox["n", "TI"]], "InlineFormula"],
     " to determine a linear fit and reference value for the coefficient."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{"Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {
     Dynamic[
      If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
       RGBColor[0.92, 1, 0.59], None]], {
      Dynamic[
       If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
        RGBColor[0.92, 1, 0.59], None]]}}, "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->412705712],

Cell["\<\
The straight line fit to data is shown as a dashed line where extrapolated, \
and as a solid line within the range of data used to determine the line.\
\>", "Notes",
 CellID->246272709],

Cell["\<\
Data points used to determine the fit are shown in blue, those not used, in \
red.\
\>", "Notes",
 CellID->74541308]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->304721846],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->289022365],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->337266530],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->207768003],

Cell["\<\
Use the Eppler-205 airfoil, which is specified by a file in the folder \
AirfoilData, as the example airfoil.  Fit a B-spline to the data and show \
both the data and the curve fit to the data.  User-specified discretizations \
are based on the B-spline.  \
\>", "ExampleText",
 CellID->99847148],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"spline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gD", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSpline", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "t", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}]}], ";", 
  RowBox[{"Show", "[", 
   RowBox[{"gD", ",", "gSpline"}], "]"}]}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->524210768],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+PFPUdx/Fr+pf037hEueMOFAk9D4vYWmrhZAtt1Wo0MagRalsIQrrx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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->8892725]
}, Open  ]],

Cell["\<\
Specify an angle of attack for the airfoil and a list of discretization \
numbers.\
\>", "ExampleText",
 CellID->53729757],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"6.", " ", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"n", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Range", "[", 
      RowBox[{"5", ",", "20", ",", "5"}], "]"}], ",", 
     RowBox[{"Range", "[", 
      RowBox[{"25", ",", "250", ",", "25"}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->222327051],

Cell["\<\
Compute collections of vectors locating panel endpoints for each \
discretization number in the list, and display the list of number-of-panels \
for each airfoil.\
\>", "ExampleText",
 CellID->3599668],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"AirfoilSurfacePoints", "[", 
       RowBox[{"spline", ",", 
        RowBox[{"NDiscretizeUnitSegment", "[", 
         RowBox[{"#", ",", 
          RowBox[{"Layout", "\[Rule]", "ConstantSpacing"}]}], "]"}]}], "]"}], 
      "&"}], ",", "n"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"nP", "=", 
  RowBox[{"Length", "/@", 
   RowBox[{"PanelLengths", "/@", "rP"}]}]}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->146492676],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "10", ",", "20", ",", "30", ",", "40", ",", "50", ",", "100", ",", "150", 
   ",", "200", ",", "250", ",", "300", ",", "350", ",", "400", ",", "450", 
   ",", "500"}], "}"}]], "Output",
 ImageSize->{420, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->318993919]
}, Open  ]],

Cell["\<\
Solve PMVortexLinearNeumann for each discretized airfoil, and compute lift \
and moment coefficients.\
\>", "ExampleText",
 CellID->775435343],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "vC"}], "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"PMVortexLinearNeumann", "[", 
        RowBox[{"#", ",", "\[Alpha]"}], "]"}], "&"}], ",", "rP"}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "cl", ",", "cm", ",", "\[CapitalDelta]cl", ",", "\[CapitalDelta]cm"}], 
    "}"}], "=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Apply", "[", 
     RowBox[{"ForceAndMoment", ",", 
      RowBox[{"Thread", "[", 
       RowBox[{"{", 
        RowBox[{
        "\"\<Vortex\>\"", ",", "a", ",", "b", ",", "rP", ",", "\[Alpha]", ",",
          "rC"}], "}"}], "]"}], ",", 
      RowBox[{"{", "1", "}"}]}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{"cl", ",", "cm"}], "}"}]}], "Input",
 CellLabel->"In[5]:=",
 CellID->53143172],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "0.9418031781153716`", ",", "0.9649551596371126`", ",", 
     "0.9724211193147836`", ",", "0.9764192981740645`", ",", 
     "0.9790145434607962`", ",", "0.9844981599700706`", ",", 
     "0.9865689490418403`", ",", "0.9876872778029462`", ",", 
     "0.9883744876132636`", ",", "0.9888448165504262`", ",", 
     "0.9891822786083211`", ",", "0.9894311090452481`", ",", 
     "0.9896214395067449`", ",", "0.9897711520439691`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.29259037588779`"}], ",", 
     RowBox[{"-", "0.2963055615838363`"}], ",", 
     RowBox[{"-", "0.29777117591801894`"}], ",", 
     RowBox[{"-", "0.2987216831002686`"}], ",", 
     RowBox[{"-", "0.2994506097235059`"}], ",", 
     RowBox[{"-", "0.30112789172966004`"}], ",", 
     RowBox[{"-", "0.3018615090008967`"}], ",", 
     RowBox[{"-", "0.3022869506514747`"}], ",", 
     RowBox[{"-", "0.30255033565778633`"}], ",", 
     RowBox[{"-", "0.3027304067949217`"}], ",", 
     RowBox[{"-", "0.30285918275941803`"}], ",", 
     RowBox[{"-", "0.30295338813335526`"}], ",", 
     RowBox[{"-", "0.3030245888433609`"}], ",", 
     RowBox[{"-", "0.3030799885049016`"}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{533, 67},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->346077016]
}, Open  ]],

Cell["\<\
Note that the lengths of lists of numbers of panels, and coefficients are the \
same.\
\>", "ExampleText",
 CellID->2060996],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "/@", 
  RowBox[{"{", 
   RowBox[{"nP", ",", "cl", ",", "cm"}], "}"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->57188429],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"14", ",", "14", ",", "14"}], "}"}]], "Output",
 ImageSize->{82, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->333236422]
}, Open  ]],

Cell[TextData[{
 "Use the last ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["n", "TI"], 
   StyleBox["f", "TI"]]], "InlineFormula"],
 " discretization numbers to compute reference values for lift and \
pitching-moment coefficients and plot the convergence graphs.  Extrapolation \
beyond the range of data determining the line is shown as dashed."
}], "ExampleText",
 CellID->388479374],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"nF", "=", "5"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clRef", ",", "gCl"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{
     RowBox[{"Take", "[", 
      RowBox[{"nP", ",", 
       RowBox[{"-", "nF"}]}], "]"}], ",", 
     RowBox[{"Take", "[", 
      RowBox[{"cl", ",", 
       RowBox[{"-", "nF"}]}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", "gCl", "]"}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->640039878],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0.67, 0.6, 0.6], AbsolutePointSize[5], 
    PointBox[{{0.002, 0.9897711520439691}, {0.0022222222222222222`, 
     0.9896214395067449}, {0.0025, 0.9894311090452481}, {0.002857142857142857,
      0.9891822786083211}, {0.0033333333333333335`, 
     0.9888448165504262}}]}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVz3s41XccB3DRRm5n57gT1XqS+6VTUen5YE3CsTaFYvU793N857Io8zxE
6LYu2xAzt6N1yHFNRGhnJqQjnppO6NnykKflslN0jKL2OX+8n/fz+uP9x3sD
J/4rvraWlhYLo+kntHoli0OBp+2mcZ8aFbyZ/pntyKUgKvLMPo0Z9zJnVvMo
OJ37omEXOjQjXLuDT8HIGln6TnTPqxVXJzEFGQtutt7opochpz9OoGBw0DuS
ic7JnWLK0yiIzQwZcESzLOx/8iikoOpFYps5Wi697nRjiAJW0+avZ6tVwO94
HuZryIalkoI7Pejhw5LUVn820F6/eyZB780QZg+ksSEsoLs/GT3l+o/NfBMb
SluDkr9En+ck5TS/YoO567aZzeg3dhl6bGcOLOqbbvwgU0Ee93NjGw4Hxj+q
txhBe7ibzx8r4cCSdr2yHq2VJRGODHHA0paEnUeXHJE9dTDggkRik8NFb++U
u5kHcCHc3rNoJ1qn5jKdkcYF0z5tjim631b8Z3UrFz7E7pifqlIBx6Irz22e
C2GpvIA/0C6Fykm5Iw8aKoMjitFz5WozFY8HOx4XbElEz24ZeOxSzAPFOoki
BP08qDFJb5gH339j5L4RXTllZvKAwYfGT+MPvL2ugjIG00+yjw/j6oteQ+jc
zqhQ+mk++CpokzJ0++Lu6R87+CC9E7o/G13fUnZ2aY4PSVaZaUc0+xVDHS03
ASQkWx/fhr6rYBqb8QWQNBfnRENPrhcs9koEII13kE1WquDRisU496kAfMOV
0x3oN5ysKGOGECYO+U7noxlfvL0ZzhLCzaO05lg0r8upuDZbCBcyf/HZi86S
j2bNtgmh39j6zAZ0qX9+kEIthF0l7wsXK/A/y6C8xUUEjYFl5BHae+yC0k8k
goML13Sr0K1vu4tGy0SwqlPAzkSvLkmnopUiMFevTYtGh/TsWZ1KF8MhZ28u
E92X3OUlCxTDrydpukZo/wZW0IlsMdgMBR+fkKrgbEJEtLNcDMs5qdI2tNZt
6/f/qsUwGSfOzUMfP/fy5H7PGFgOaQ8m6HVj3/7wuygGrAP7Ovegr8hTyqzK
YuBqjWLJDq1rf+nyJ6MxEGHCXFZfU0Gd6WEjczoB04LwjkG0Z8TLdgMGgZz/
LtA1bir8LmaVCQF6ZKdgAN1mV9A7Y0rA2MqF/gDd6zB0qsuSgF7RKsF99JhP
6ELCegLvSmto3Wg63/+Zwp3ARIU2tw2dV/nwUqcHAbaud+tttOUU5dPiSWBM
GGuksV18euFVJoG/HIZbWtBOKR1hKV4EnshqDZvR/pe237MHAvfrIm/Vo7sH
u0+s9SWwj3bZQONAxsFNDD8CvfFdVB06tCAxc8WfwF0PN4NadFR5g89QAIHf
GnUoGfrvCZju20tgt8mO5io0x36wUB5IoD0xTl9jUfXsgiyIQOvWkaZK9PRs
qlQSTMDrirG+xnEehgfyQwg0qz87WoF+faxI+yKLwNbwlCYpOqnZ6capUAKN
t+rWaPw/nMVdHA==
      "]]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwV0n881HccB3BlEyG/ojLFeshvZedHJXdvxCbuRCcujvss4xJRKOyaSErk
sYrvZH6Gh4vOb3drrBpK09Q8dPthdTLOj0fRt9Va17Hauz9ej9fj+d/rj9fH
+1P2xC3X0NDgYN63QPO2dezhQeavHorG2u9oAMGWVgHVwCo1oPtq0FLThHsM
qpvFndNQVKOfP/8w2oK6xRoptzapQhfHyK2NKDnr56XEE+VoPtVboEspWf39
6oiL6AGXth5t6h9Wa/BanVPojLdXuCuoD6AgnpsoRJe47U6tTTWFE+s+F9ij
+9JKnDMVlrBMpX2YltFQW0rm3ulsgrr0mnoZetkavXrDUgc4k35PKwvNibH4
PfvmFlDVUKcBXdVopX4kYYAk+JX7MrRY1HJyJskdTlTs1L4lpcG2a/5O9X9b
YdzE6XUhOpMly71+2xNyDSSmHDTLiaEqjGFCm8dgiAm6POPKAb8nAEuL91v/
7KZhf5HcpjPcB875pXhVoV31WjPKa32h6VrTTCx6Xja83E3LD3TaRT9uQqec
nyv0i/OHAn+v63NdNDhXRB+RSD6FEq2y39rQD35Zc/GOQQD8FXjWMhXd7e5o
5Bm1C3p7TPO3ob+6nXdX0B8IYy3bzdWdNBieGstZ3MgGM2CP/oAeTyugniVy
YHxR1ZiHpoqPK0tHguHJDuZVf/TOeWXA0KoQOCn+aFgXbWW/coNmZwhs75gz
eNBBw9/q0PNmsaHgyuLkUWgSN9ixV3sPfP9vkAkfPfBF9OHk3j2Qx5gcXY8u
Wkx19k3kQqKeUDrRjn9wkJb7GITBY6PT3WJ0IL2u3rwvDMq+9ZpKQPuwr0ZZ
CvfCw+DizS7oKu9e48c64cC8PFz5uo2GhaGF+MyecFAeq/fsQT98yj7IFESA
aebqVzno63X3Vz4y4kHRksVjX7TXi+ORjdd4UCy5Oa2NthpruuLI3wct6hCD
kVYaVGFvSgr1I8FwOOvARXRywtHNpl2RoDHkNc5D/2G8Oj4rPgpcd5XlmaPF
3E8qL5nxIV84EaRoocHEWZQ71cOH2I4BlwZ0WsWspvahaMjPZvnHo4/U5bjz
TGKAkcE95oSWe1+Ki5XFwFv9iLsvJTQ4po8yAoQC0Ld7wZGhDWsrI5y0CDQd
sn12HC3PS1NUriAgzH69IEKXCwNj9XUIWH99Z+FL9MYtquRnugRq24ULmWi3
G9zT7UYEyl6K59PR+x6tlLqtJ3Aqy+5pErpubZYx042AIN9+jo+OX9p9qcWd
wPpv3sxGvd8zYbNhw1YCDxuHZiPR3WK5/bvtBMJ/SpjloQc9XHz6gABHt3km
DP0kbDbFP4jAjgsO02x067Ybr6RsAqrLamUQOs2CEtkEE5B13lUGopemfM+s
CCXAkB9UBqBXpVVXD4UTsFtzdcoPPRp+1NqTR2DGVjS1E13myW5u3kegYVvQ
lC/aarlaWsQnYBn5dNIbPT09smMpmoDiYO8koJuHxH1JAgIVoqJJFjq5Jfsz
BSHAOxc1yUS7Xth7j7OfgFmV46QX+n/lPmM+
      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->None,
  Frame->True,
  FrameLabel->{
    FormBox["\"reciprocal of number of panels\"", TraditionalForm], 
    FormBox["\"C\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{0.98, 1.},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 327},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->28521870]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cmRef", ",", "gCm"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{
     RowBox[{"Take", "[", 
      RowBox[{"nP", ",", 
       RowBox[{"-", "nF"}]}], "]"}], ",", 
     RowBox[{"Take", "[", 
      RowBox[{"cm", ",", 
       RowBox[{"-", "nF"}]}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", "gCm", "]"}]}], "Input",
 CellLabel->"In[8]:=",
 CellID->153900572],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0.67, 0.6, 0.6], AbsolutePointSize[5], 
    PointBox[{{0.002, -0.3030799885049016}, {
     0.0022222222222222222`, -0.3030245888433609}, {
     0.0025, -0.30295338813335526`}, {
     0.002857142857142857, -0.30285918275941803`}, {
     0.0033333333333333335`, -0.3027304067949217}}]}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVz2lQUwcUBWC2gh0ETdhUFLVFRgNCkKqdFuZKqmCRUAZUpoDy8pKX7RWl
yFJUBA0MdQkVQTahAS2DVaoQsbhEnzQiUFxGQ5HFaSmLhk0oENkEe/PjzJnv
3zlryYNhlJmJiQkfY+yXS6638UkC3lPqphuUjpkcKhRsEBLwiozfYzS76cSw
hYgATcymHjU6JH2vmYYi4Mi36rka9KOx+Y0cGQFzwWrPanTt8+BMyzi0jzqv
Cn0ud9CHSSVg1lRNVKD5Tm453CICpktrpvPRTMVlTk0rAcW9PE0ymtL0hW9b
LIC0tfdyItHtkWVHb/EEcOjjazN+6MB0ScbTVAFYf+AaXNCDG/XOE7UCeDNc
YmaGPkkmnLs5JgCpvvhSjwj/uKQvEriTMKVc8lKLzhPusHUmSWh2j11ZieZ6
OU7El5Lg/zLakIU2UZRJOlpJeH94c7ccXbr/Std6ayE0NGXsC0FvqWc8HQOE
8Jmr+IAX2rwqm8VOFUJ/MjeHjX68Sqa7eksIvh1nAieEOoZ00uZ5TgihvC/m
bCvao6itn9kggiC981Aderzc4DAqEkHBjKyjED2y6elfHiUiWG9e3nEE3Rek
TljULoL9K+Sx0ejKQQe7J2wKLnb9muuHVrF9/Mu+poBz/kXLGnRufVQIK5OC
vdHKAlP03Wm/obMaCh45eyf2kTrmep0qa2acgh2Zu/ob0ar5xeYmnmKQvp/S
X0Y/bPGxdaDE8IxWLpxG968RTzeWiaFTX1v5HfrFvFOPsEsMk0vCe/noSVIR
ZcuWQJhXJ9cbzf5m9sZevgR2R0+w2GiRllPyW4YEio8bLAwCHaNgOhUjdyQw
GnrgVDv6Z15+UItBAvqNKZduo0f41uV1HlJwNY99WoL+vPt0m79UCpavx8+k
om/NNlzoVEnBvV6wj0BblKYR0W1S2B8Q+OArdPCj7RZHWTLY2nH/d1d0c7J2
65WdMggTrWas0LxqflBShgzs5llRekLHZMVFRLszMuiBIyeb0Sa3Vyy8NcjA
cPhJcxU68ceBY6HecqjS1pQq0au7v//pgVQOlRZpijj0eSZFtVwlh+rRP+bC
0VZuyuylnXKI0RdabUZfs4+0cWTRENoRsW4Z2jti4K41m4brH8X84oSuLfpB
bmpHg80m8adG33EpaBy2p6HlVMJaR3Tj+tbj2mU07PDNWWWP7vYNeRe3hoYv
VH86LEWzKN4/LV40uFJ+lovQeZXPlfVcGhQ52zOt0MsGCd86bxr+vbfLwmiX
g2lFF31oUDlGmlmiOSma8JStNCxvSlowR/OUW5rcgAZb9+p3H2J0TMOzhqSV
22iIjahLNHone886tj/uU9yfXECHFBw6Mc+j4eSrx+Pz6Kjyat/WABrMswfe
zqH/7oWh5kAayDtjsUaTbs+KmJ00PHg9NTyLll4deXcliIZjYDk0gx4aOVpR
touGV7SN3OgD3MW784Np+LLQfmAa/V/8BbMzfBqKHzpLjU64yak5HkLD9Ngn
b6bQ/wNm5XeV
      "]]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwVynk81Hkcx3Gb3UEywmK3lWQdjxyRRkqZTzOrVgxyNDnm+K5xTa40zlSP
bD3KY0Ql88A6HpZdJOlgsCy/VTlDHs3so+SoXSyl0Nbuitm1H3+8H6/H8/F4
bw1L8I9Yp6am5o1bq1C9y0J0vNtt5TS92DteQYHQoV4o+4Hplb59noOWG4oH
nWSNzOIkH9aaFxc/4ZvIHjD3xuTOeKJzBEoLPZmSeTKIzvBA82RtWdqyKeay
E32Qjb7veKtVU/ae+WFGZ9UFnfpfTYCG7GP4209HZI6+xvA9UX7CEMSuUVnv
4hRUp+Safdr4FthfUNbbgy7PJ7OrWpZg/+7iVAn6I+MNlRvzbSBPNMCPR3sL
TJ6coRyA5zH/lo0urTJbHqtzglJWzKQBujrj5rd/xDrD+z3f3Z6OVVDWDa97
yv51gdtVVYWt6DRmU2Z7lytMWtMnstFMO6clqcANMjYHMAm6KLUm2v0VgI2+
50lndFi20uoulwW+X5QYaKB3bqhPLSpnw7PChz2jMQrqddPAOgbNHcZsuCN1
6IQrs1L3iAPwri2v+wzavpifWFd3EI531g0FoBWPjPN6dD0gfHQrwxLd6Gyr
5xp6CDrESY0fjimo013n+oX3PCF3g3leP3rj+ZGzK+YcSK7P9CpBT0iyZPMx
3vDLg44tiWhZzqmp/GEfOLfA9WOjv3o95dFHPww3cgdKjdBm29abqt89DLN2
77vmxArq7bLfFSORH5Q+T/FqR5OI7jtHNP2hmfZk7ir6fjj/eHybPxxi570M
Q2evnLBnxwTA4qnVewz0oo28iKUbCCpbz2YttOfC55WbOgPh7MKC5mi0gmJx
boRuiToCYmtJ8i106f42/edaXIgS1LSfR7/pexOZ1sqF8eu7RVz06BznmJvw
KLSIq760RbdXDK0f0wsCbWYCSxWloPb9eSqkqiUI7l0wPDmMNhu5XmPLC4bL
Q3G5P6KXAj9ck+qEgNJmq1oqOl6cvN2wIQROr9y55IV+qv9pZHpkKJS9KE0x
QVcH7CgpNOLBrw/27puPVFAG9hmZk608SNzcbHIfLSmeUdeM48N5aU5IPjqx
4qxzkIEAVDGshmi0cn9hhKhJAFJO9Ywb2jbpsZNHlBAus6LTddEby0uO2tEI
LNfSrKYi8H9OMl6iQSAr7mr4JLooylOko0XAyNGk8ne0ucNS/Lw2ASf5DrPf
0IyOgAu39QiIKd6mCXTw2Ho5YzOBEcVd+lN0xWfp+m4MAi0qwT8P0ZEq38Kb
zgQOUi+d12z7wsrU1IWAMjMpqR/dWK3ctrqHwCJN+rYX3b3LkdUJBKz15HNd
6FeBMwkHvAgUWGq/oND1uzv+knMIWM7KTNcsMZFlWPkQaKg143egVZPsixp+
BB457Hr2M5ouKSvr4xKguX6j/An9mJts4RpEIF81p7/mAldObW0wAXMqxa8F
bbZuWZ7NI8B0vzTUhJ6eHt6r4hMYoBnrrLm2r7ozVkggpO97Lzk6/uaZr8cJ
gdlsO2kjeufVI4PeYQRSfJp7G9D/A/WFa2U=
      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->None,
  Frame->True,
  FrameLabel->{
    FormBox["\"reciprocal of number of panels\"", TraditionalForm], 
    FormBox["\"C\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{-0.31, -0.3},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 322},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->458396289]
}, Open  ]],

Cell["\<\
The reference values for lift an pitching moment coefficients are also \
computed in PlotConvergence.\
\>", "ExampleText",
 CellID->99280306],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"clRef", ",", "cmRef"}], "}"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->202114905],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9911654719698267`", ",", 
   RowBox[{"-", "0.30360736983652525`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->290392090]
}, Open  ]],

Cell["\<\
Sometimes, it is instructive to display the results of some coarser \
discretizations, but not use these results when determining the equation for \
the line.  Coarser discretizations, not used to determine the equation for \
the line are shown in red.  Results from extrapolations should always be \
viewed with skepticism, but the graph provides a useful visualization of the \
convergence properties of the model for the example used.\
\>", "ExampleText",
 CellID->248309304],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"clRef", ",", "gCl"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"nP", ",", "cl"}], "}"}], ",", "nF"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", "gCl", "]"}]}], "Input",
 CellLabel->"In[10]:=",
 CellID->181676159],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 0, 1], AbsolutePointSize[5], 
    PointBox[{{0.002, 0.9897711520439691}, {0.0022222222222222222`, 
     0.9896214395067449}, {0.0025, 0.9894311090452481}, {0.002857142857142857,
      0.9891822786083211}, {0.0033333333333333335`, 0.9888448165504262}}]}, 
   {RGBColor[1, 0, 0], AbsolutePointSize[5], 
    PointBox[{{0.004, 0.9883744876132636}, {0.005, 0.9876872778029462}, {
     0.006666666666666667, 0.9865689490418403}, {0.01, 0.9844981599700706}, {
     0.02, 0.9790145434607962}, {0.025, 0.9764192981740645}, {
     0.03333333333333333, 0.9724211193147836}, {0.05, 0.9649551596371126}, {
     0.1, 0.9418031781153716}}]}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVz3s41XccB3DRRm5n57gT1XqS+6VTUen5YE3CsTaFYvU793N857Io8zxE
6LYu2xAzt6N1yHFNRGhnJqQjnppO6NnykKflslN0jKL2OX+8n/fz+uP9x3sD
J/4rvraWlhYLo+kntHoli0OBp+2mcZ8aFbyZ/pntyKUgKvLMPo0Z9zJnVvMo
OJ37omEXOjQjXLuDT8HIGln6TnTPqxVXJzEFGQtutt7opochpz9OoGBw0DuS
ic7JnWLK0yiIzQwZcESzLOx/8iikoOpFYps5Wi697nRjiAJW0+avZ6tVwO94
HuZryIalkoI7Pejhw5LUVn820F6/eyZB780QZg+ksSEsoLs/GT3l+o/NfBMb
SluDkr9En+ck5TS/YoO567aZzeg3dhl6bGcOLOqbbvwgU0Ee93NjGw4Hxj+q
txhBe7ibzx8r4cCSdr2yHq2VJRGODHHA0paEnUeXHJE9dTDggkRik8NFb++U
u5kHcCHc3rNoJ1qn5jKdkcYF0z5tjim631b8Z3UrFz7E7pifqlIBx6Irz22e
C2GpvIA/0C6Fykm5Iw8aKoMjitFz5WozFY8HOx4XbElEz24ZeOxSzAPFOoki
BP08qDFJb5gH339j5L4RXTllZvKAwYfGT+MPvL2ugjIG00+yjw/j6oteQ+jc
zqhQ+mk++CpokzJ0++Lu6R87+CC9E7o/G13fUnZ2aY4PSVaZaUc0+xVDHS03
ASQkWx/fhr6rYBqb8QWQNBfnRENPrhcs9koEII13kE1WquDRisU496kAfMOV
0x3oN5ysKGOGECYO+U7noxlfvL0ZzhLCzaO05lg0r8upuDZbCBcyf/HZi86S
j2bNtgmh39j6zAZ0qX9+kEIthF0l7wsXK/A/y6C8xUUEjYFl5BHae+yC0k8k
goML13Sr0K1vu4tGy0SwqlPAzkSvLkmnopUiMFevTYtGh/TsWZ1KF8MhZ28u
E92X3OUlCxTDrydpukZo/wZW0IlsMdgMBR+fkKrgbEJEtLNcDMs5qdI2tNZt
6/f/qsUwGSfOzUMfP/fy5H7PGFgOaQ8m6HVj3/7wuygGrAP7Ovegr8hTyqzK
YuBqjWLJDq1rf+nyJ6MxEGHCXFZfU0Gd6WEjczoB04LwjkG0Z8TLdgMGgZz/
LtA1bir8LmaVCQF6ZKdgAN1mV9A7Y0rA2MqF/gDd6zB0qsuSgF7RKsF99JhP
6ELCegLvSmto3Wg63/+Zwp3ARIU2tw2dV/nwUqcHAbaud+tttOUU5dPiSWBM
GGuksV18euFVJoG/HIZbWtBOKR1hKV4EnshqDZvR/pe237MHAvfrIm/Vo7sH
u0+s9SWwj3bZQONAxsFNDD8CvfFdVB06tCAxc8WfwF0PN4NadFR5g89QAIHf
GnUoGfrvCZju20tgt8mO5io0x36wUB5IoD0xTl9jUfXsgiyIQOvWkaZK9PRs
qlQSTMDrirG+xnEehgfyQwg0qz87WoF+faxI+yKLwNbwlCYpOqnZ6capUAKN
t+rWaPw/nMVdHA==
      "]]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwV0n881HccB3BlEyG/ojLFeshvZedHJXdvxCbuRCcujvss4xJRKOyaSErk
sYrvZH6Gh4vOb3drrBpK09Q8dPthdTLOj0fRt9Va17Hauz9ej9fj+d/rj9fH
+1P2xC3X0NDgYN63QPO2dezhQeavHorG2u9oAMGWVgHVwCo1oPtq0FLThHsM
qpvFndNQVKOfP/8w2oK6xRoptzapQhfHyK2NKDnr56XEE+VoPtVboEspWf39
6oiL6AGXth5t6h9Wa/BanVPojLdXuCuoD6AgnpsoRJe47U6tTTWFE+s+F9ij
+9JKnDMVlrBMpX2YltFQW0rm3ulsgrr0mnoZetkavXrDUgc4k35PKwvNibH4
PfvmFlDVUKcBXdVopX4kYYAk+JX7MrRY1HJyJskdTlTs1L4lpcG2a/5O9X9b
YdzE6XUhOpMly71+2xNyDSSmHDTLiaEqjGFCm8dgiAm6POPKAb8nAEuL91v/
7KZhf5HcpjPcB875pXhVoV31WjPKa32h6VrTTCx6Xja83E3LD3TaRT9uQqec
nyv0i/OHAn+v63NdNDhXRB+RSD6FEq2y39rQD35Zc/GOQQD8FXjWMhXd7e5o
5Bm1C3p7TPO3ob+6nXdX0B8IYy3bzdWdNBieGstZ3MgGM2CP/oAeTyugniVy
YHxR1ZiHpoqPK0tHguHJDuZVf/TOeWXA0KoQOCn+aFgXbWW/coNmZwhs75gz
eNBBw9/q0PNmsaHgyuLkUWgSN9ixV3sPfP9vkAkfPfBF9OHk3j2Qx5gcXY8u
Wkx19k3kQqKeUDrRjn9wkJb7GITBY6PT3WJ0IL2u3rwvDMq+9ZpKQPuwr0ZZ
CvfCw+DizS7oKu9e48c64cC8PFz5uo2GhaGF+MyecFAeq/fsQT98yj7IFESA
aebqVzno63X3Vz4y4kHRksVjX7TXi+ORjdd4UCy5Oa2NthpruuLI3wct6hCD
kVYaVGFvSgr1I8FwOOvARXRywtHNpl2RoDHkNc5D/2G8Oj4rPgpcd5XlmaPF
3E8qL5nxIV84EaRoocHEWZQ71cOH2I4BlwZ0WsWspvahaMjPZvnHo4/U5bjz
TGKAkcE95oSWe1+Ki5XFwFv9iLsvJTQ4po8yAoQC0Ld7wZGhDWsrI5y0CDQd
sn12HC3PS1NUriAgzH69IEKXCwNj9XUIWH99Z+FL9MYtquRnugRq24ULmWi3
G9zT7UYEyl6K59PR+x6tlLqtJ3Aqy+5pErpubZYx042AIN9+jo+OX9p9qcWd
wPpv3sxGvd8zYbNhw1YCDxuHZiPR3WK5/bvtBMJ/SpjloQc9XHz6gABHt3km
DP0kbDbFP4jAjgsO02x067Ybr6RsAqrLamUQOs2CEtkEE5B13lUGopemfM+s
CCXAkB9UBqBXpVVXD4UTsFtzdcoPPRp+1NqTR2DGVjS1E13myW5u3kegYVvQ
lC/aarlaWsQnYBn5dNIbPT09smMpmoDiYO8koJuHxH1JAgIVoqJJFjq5Jfsz
BSHAOxc1yUS7Xth7j7OfgFmV46QX+n/lPmM+
      "]]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwVz3s0lAkYBvBpE0qsoU3ntELHqliUS+qofboZoayitjpqa86KMlKKRuVO
pmmyhDSJyGAkchvmi62zS6ILVkOEXNpGLt/3YScNW2u//eM9z/md8/zxvGbc
4L1+X7FYrN3M/Z/TZ3UaYj0DsSGH1s6X0Djpzsl8IApE6qC6NZxxn2lMSO+z
QFArv8rwZNzwctp0vSsPkvwl36nzaFxfNRQxgiCwizdu9WBs3SN38loXjFFZ
XPhkLg07s8oOPeEZ3H5uNLr5Do0AwVG3Xl4o5qbxvFNMQ8vBWLO9OwIZgVb9
Jkk0hFXBLWM9sRhe1ldqG8f0d8nLrGsuIy+Hwwvj0+hvL//yrVIIG47HRQWP
hu6KUqe0l0k4a76UM3OMxkix9Snf+BQkml7iOu6nwddSe+jdTIXWJzFP6k6D
+2S7qYYgHVs0I+2GQEM8O1zUlpuBnfZf3OwdaFhd17UczBBjkXibsccaGic+
FDiy4zPh1I234hU0/pbdnDa6lQV7p66JOQMa3e91SrmiOxh5uf7x99o0EhoD
dbcpchAcu7b/9GcK58KvXuH25eIoe2h8bIJCSt8YK19xFwu7D1ZbKyn4RKjb
2wfyEPeI9+flNxRcWlxyVvZIUN3rIi1opdB7CE2HR/MhX/awra+eQlPjlEyq
KoCFrVLpIacw9E6Z/Q1ZiN2RXfLoEgp3NyUOBc1KsWPMr/dFLoVkToDZh49F
UCUTtf03KChs51uEjN+DHvuJYI2IwiMf94ThT8WYKQ0sFEZT6LRs74+cvI+s
hK6YilAKaZs00+exSsG+MFAyeYKC1Fr1ddrCB5jYmfPU/wiFj/7s7RJWGQRe
FlkSbwojnZFWqstlmAh1bVC7Mv9ni9j7NMrBVtgUGm6mcG/f+Tebo8qRtbjl
vJ8dhUpLuddtjQrMcLVvtVhQ8D4gbtCJqYDelDb/3+UUGvPa3k9oVWIqW1Lo
ok+hzVWqMZxQieR4zVTvBRQSenw1/eZVQbzDlHdzloTuNX/ZgcQqhKzsuKGi
SFhlGHAPa8igsdErwugvEsaPDeteX5XheNBx2Z4uEkTnpLpbvxoOlxx6/nhB
Ypxz2p4UVeNhfVPd7O8kphQ7locZ1kB/jZp2qybxay3B0k+pgclT8u3PxSRe
p+g0r1sshxiZtfdzmH5C5Q/NKXK863g3op9OQtL+T/kZPQJKydQrZyGJfX78
Pdn6BGr4yaE6jDXVqolnBgQEu2yMeq6Q8DembM2NCFipThy6wNgyYKCk04RA
8PbBAbmARNmXhiLndQTUg62kYyKJutVJuQt8CCwyKdFaG08iqHbRVrv9BHqn
3Ivm4kis8EwcOHKAQEnjB/dWxlFno03kvgR+PGWedIrxtkchWSf9CKT9lrmk
JJZEk/dP4tYwAsa+QjPLGBLhylcbPvMJULar62eimT3hXl2rLxJ4PL/xl2bG
V7PdjGKjCBwrnicNYOw54pzuICBQMMu3KYwiMXdR7nBMSOB8y9K2MMZleo6K
ayICO+9WneEwNnCwMRxOJjDqNlH1PpJEfeO9CsNUAnXGSftljM8dXLV3SzoB
0aSVOp6x+XjeJC+DwJEnzWIfxh2RpiliMQFbsb+zOeP/ANkpfaE=
      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->None,
  Frame->True,
  FrameLabel->{
    FormBox["\"reciprocal of number of panels\"", TraditionalForm], 
    FormBox["\"C\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{0.92, 1.},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 330},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->592916800]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"cmRef", ",", "gCm"}], "}"}], "=", 
   RowBox[{"PlotConvergence", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"nP", ",", "cm"}], "}"}], ",", "nF"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", "gCm", "]"}]}], "Input",
 CellLabel->"In[11]:=",
 CellID->402096401],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[0, 0, 1], AbsolutePointSize[5], 
    PointBox[{{0.002, -0.3030799885049016}, {
     0.0022222222222222222`, -0.3030245888433609}, {
     0.0025, -0.30295338813335526`}, {
     0.002857142857142857, -0.30285918275941803`}, {
     0.0033333333333333335`, -0.3027304067949217}}]}, 
   {RGBColor[1, 0, 0], AbsolutePointSize[5], 
    PointBox[{{0.004, -0.30255033565778633`}, {0.005, -0.3022869506514747}, {
     0.006666666666666667, -0.3018615090008967}, {
     0.01, -0.30112789172966004`}, {0.02, -0.2994506097235059}, {
     0.025, -0.2987216831002686}, {
     0.03333333333333333, -0.29777117591801894`}, {
     0.05, -0.2963055615838363}, {0.1, -0.29259037588779}}]}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJwVz2lQUwcUBWC2gh0ETdhUFLVFRgNCkKqdFuZKqmCRUAZUpoDy8pKX7RWl
yFJUBA0MdQkVQTahAS2DVaoQsbhEnzQiUFxGQ5HFaSmLhk0oENkEe/PjzJnv
3zlryYNhlJmJiQkfY+yXS6638UkC3lPqphuUjpkcKhRsEBLwiozfYzS76cSw
hYgATcymHjU6JH2vmYYi4Mi36rka9KOx+Y0cGQFzwWrPanTt8+BMyzi0jzqv
Cn0ud9CHSSVg1lRNVKD5Tm453CICpktrpvPRTMVlTk0rAcW9PE0ymtL0hW9b
LIC0tfdyItHtkWVHb/EEcOjjazN+6MB0ScbTVAFYf+AaXNCDG/XOE7UCeDNc
YmaGPkkmnLs5JgCpvvhSjwj/uKQvEriTMKVc8lKLzhPusHUmSWh2j11ZieZ6
OU7El5Lg/zLakIU2UZRJOlpJeH94c7ccXbr/Std6ayE0NGXsC0FvqWc8HQOE
8Jmr+IAX2rwqm8VOFUJ/MjeHjX68Sqa7eksIvh1nAieEOoZ00uZ5TgihvC/m
bCvao6itn9kggiC981Aderzc4DAqEkHBjKyjED2y6elfHiUiWG9e3nEE3Rek
TljULoL9K+Sx0ejKQQe7J2wKLnb9muuHVrF9/Mu+poBz/kXLGnRufVQIK5OC
vdHKAlP03Wm/obMaCh45eyf2kTrmep0qa2acgh2Zu/ob0ar5xeYmnmKQvp/S
X0Y/bPGxdaDE8IxWLpxG968RTzeWiaFTX1v5HfrFvFOPsEsMk0vCe/noSVIR
ZcuWQJhXJ9cbzf5m9sZevgR2R0+w2GiRllPyW4YEio8bLAwCHaNgOhUjdyQw
GnrgVDv6Z15+UItBAvqNKZduo0f41uV1HlJwNY99WoL+vPt0m79UCpavx8+k
om/NNlzoVEnBvV6wj0BblKYR0W1S2B8Q+OArdPCj7RZHWTLY2nH/d1d0c7J2
65WdMggTrWas0LxqflBShgzs5llRekLHZMVFRLszMuiBIyeb0Sa3Vyy8NcjA
cPhJcxU68ceBY6HecqjS1pQq0au7v//pgVQOlRZpijj0eSZFtVwlh+rRP+bC
0VZuyuylnXKI0RdabUZfs4+0cWTRENoRsW4Z2jti4K41m4brH8X84oSuLfpB
bmpHg80m8adG33EpaBy2p6HlVMJaR3Tj+tbj2mU07PDNWWWP7vYNeRe3hoYv
VH86LEWzKN4/LV40uFJ+lovQeZXPlfVcGhQ52zOt0MsGCd86bxr+vbfLwmiX
g2lFF31oUDlGmlmiOSma8JStNCxvSlowR/OUW5rcgAZb9+p3H2J0TMOzhqSV
22iIjahLNHone886tj/uU9yfXECHFBw6Mc+j4eSrx+Pz6Kjyat/WABrMswfe
zqH/7oWh5kAayDtjsUaTbs+KmJ00PHg9NTyLll4deXcliIZjYDk0gx4aOVpR
touGV7SN3OgD3MW784Np+LLQfmAa/V/8BbMzfBqKHzpLjU64yak5HkLD9Ngn
b6bQ/wNm5XeV
      "]]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwVynk81Hkcx3Gb3UEywmK3lWQdjxyRRkqZTzOrVgxyNDnm+K5xTa40zlSP
bD3KY0Ql88A6HpZdJOlgsCy/VTlDHs3so+SoXSyl0Nbuitm1H3+8H6/H8/F4
bw1L8I9Yp6am5o1bq1C9y0J0vNtt5TS92DteQYHQoV4o+4Hplb59noOWG4oH
nWSNzOIkH9aaFxc/4ZvIHjD3xuTOeKJzBEoLPZmSeTKIzvBA82RtWdqyKeay
E32Qjb7veKtVU/ae+WFGZ9UFnfpfTYCG7GP4209HZI6+xvA9UX7CEMSuUVnv
4hRUp+Safdr4FthfUNbbgy7PJ7OrWpZg/+7iVAn6I+MNlRvzbSBPNMCPR3sL
TJ6coRyA5zH/lo0urTJbHqtzglJWzKQBujrj5rd/xDrD+z3f3Z6OVVDWDa97
yv51gdtVVYWt6DRmU2Z7lytMWtMnstFMO6clqcANMjYHMAm6KLUm2v0VgI2+
50lndFi20uoulwW+X5QYaKB3bqhPLSpnw7PChz2jMQrqddPAOgbNHcZsuCN1
6IQrs1L3iAPwri2v+wzavpifWFd3EI531g0FoBWPjPN6dD0gfHQrwxLd6Gyr
5xp6CDrESY0fjimo013n+oX3PCF3g3leP3rj+ZGzK+YcSK7P9CpBT0iyZPMx
3vDLg44tiWhZzqmp/GEfOLfA9WOjv3o95dFHPww3cgdKjdBm29abqt89DLN2
77vmxArq7bLfFSORH5Q+T/FqR5OI7jtHNP2hmfZk7ir6fjj/eHybPxxi570M
Q2evnLBnxwTA4qnVewz0oo28iKUbCCpbz2YttOfC55WbOgPh7MKC5mi0gmJx
boRuiToCYmtJ8i106f42/edaXIgS1LSfR7/pexOZ1sqF8eu7RVz06BznmJvw
KLSIq760RbdXDK0f0wsCbWYCSxWloPb9eSqkqiUI7l0wPDmMNhu5XmPLC4bL
Q3G5P6KXAj9ck+qEgNJmq1oqOl6cvN2wIQROr9y55IV+qv9pZHpkKJS9KE0x
QVcH7CgpNOLBrw/27puPVFAG9hmZk608SNzcbHIfLSmeUdeM48N5aU5IPjqx
4qxzkIEAVDGshmi0cn9hhKhJAFJO9Ywb2jbpsZNHlBAus6LTddEby0uO2tEI
LNfSrKYi8H9OMl6iQSAr7mr4JLooylOko0XAyNGk8ne0ucNS/Lw2ASf5DrPf
0IyOgAu39QiIKd6mCXTw2Ho5YzOBEcVd+lN0xWfp+m4MAi0qwT8P0ZEq38Kb
zgQOUi+d12z7wsrU1IWAMjMpqR/dWK3ctrqHwCJN+rYX3b3LkdUJBKz15HNd
6FeBMwkHvAgUWGq/oND1uzv+knMIWM7KTNcsMZFlWPkQaKg143egVZPsixp+
BB457Hr2M5ouKSvr4xKguX6j/An9mJts4RpEIF81p7/mAldObW0wAXMqxa8F
bbZuWZ7NI8B0vzTUhJ6eHt6r4hMYoBnrrLm2r7ozVkggpO97Lzk6/uaZr8cJ
gdlsO2kjeufVI4PeYQRSfJp7G9D/A/WFa2U=
      "]]}}, {{}, {}, 
    {Hue[0.67, 0.6, 0.6], Dashing[{0.01, 0.01}], LineBox[CompressedData["
1:eJwV0Hs0lHkYB/ChSacddyu1UdPlUBwVkV2qJ9ooW86IIqmVXZdcVi5NY61q
W8qOKY3L0ZRQUUgaxmXeU3lqm0TnkPf3aoTKpWVSKjTh6GLf/eM53/M55/vP
91kSGrczTJfD4exg7/+cSOSpTvhEQ0ZJ9tDkzwxGeXteuCmJBu29MY9O1s/5
fyY8exQNIb2ConrWqtYJ/jqvGPjhO6PAw6yzbQZShyEWXmdltozvZ9C+R+ki
cIiD7SlplW/3Mei4RPHEUBwPJjuFCS/3MhiZEbLtWcxhOD+z52tbAINznKz0
SFcq9Lt8DZnwYVBcG9f2pucEuKGlWuHJ9rcr5fYNJ0ERp35wfiODvaT6i+WQ
GEK+xHCIM4MGi6pcclvPAL87dUeRPYPD1+1/C06Tgnlu9ODt5QyK5kz9ZHgu
B1Yc0U7OsmQw9MFmPjcjD3zKJy37zRiUTWvK2y/lQ7myU2LIY9Au28C2P18G
8y0cjYgugwdfXXU2SbsApYU2+jhN8EPduQmL8xchbNa03+cxgl2DvKpQSRG4
NnhWPR4mmN4UbeDRUQybtx7y+9BHMCk58+/Q55cgStc5cdNTgtLnbzilHZeh
lJdUuqydoH/qFCF9V+Db8aylAQ8JbmnbUry0pwTybaxr+EjwWRA073tdCj6u
MolrPcHmpvG6Mu1VWCa81p96g+DAy6FC87fXQF9nzbaAUoKX158aiJ0ug5UH
8z4dLyB41jNyyauP5RDEP3x3Uy7BjtWzrBNGKqCcMzqzUkKw0d87XTN5Hewb
vcS//EVQbUt6j45VgurRApu1KQRz1+vl6XCqIKu22dAvgWCZvdYod+5NED00
jS84SPBjhMnmEo4chCs7B1MOEBxWH7XTnpTDtUhR6fVAdn+hxGQXtxpGJnKO
xAsIVuw60r3hWDUIqrxVIVsJKmyVggJuDbzyeLG3GAj6BcpUvD9rIMd6ZLbQ
hWDTlfbB0TkKSNSopy+sItjuVcbVpCsgfNJNN9Ca/XdPsF6YTi2I1qxP27CI
oMHpiLrAU7Vwx3OBb6I5Qbt809B93DqwqDy0BgwIWqHZ7c7MOih5VxK0l0uQ
Uo9NdRnXgyAoaab4E40jnofWvpXUg7F5Y/0fH2gc7/hxodCsAbg3/lWWvaYx
6xbFMZY2gH5uemvUAI2dUl6Lg74SHEvuBe/uZvvpio0tUiVIpN3rsmkaS8in
6nhDCqZbY5f92kLjrjCRb6ExBU0vPhYAa70p7egjUwqy36fOW8g6wurd6uUW
FNiZnJlLmmm0jey7oV5MQbB/1XtgLf+iKndzoAC7391a+JDG2yvOXJrtT0Ga
Js6fUdEYe+sbd8fdFPhOTbZWsV7kc6pvfyAFVnOPe4lZH0s8vlgZTEG97VlX
d9YejQkXo8Io0MTK+Tfv09jsFyB7LKTAWzs6Iv6HxuQh5vvPIgrmzU4OD2dt
myx4uiKFggFznT531pmF2yxOHKMgxcWEmbpHo8+wW55TBgWVvztQ4axnUpRO
B8QUiDIpRw/WckPnjtMSCrYUuFdasTZ1WmWmOUvBizu+RR13abzfVFFjlkNB
RVvXfDnrpD02OzflUSDsPZCdyXr5yJWxmHwKPEaHeRGsnxzlS2UyCox0EtI9
WP8HgEaUWQ==
      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->None,
  Frame->True,
  FrameLabel->{
    FormBox["\"reciprocal of number of panels\"", TraditionalForm], 
    FormBox["\"C\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->{-0.31, -0.27},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 322},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->641705327]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["3.5 Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]], "Tutorials",
 CellID->111584306],

Cell[TextData[ButtonBox["A2 Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Graphics"]], "Tutorials",
 CellID->424464613],

Cell[TextData[ButtonBox["Table of Contents",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/TableOfContents"]], "Tutorials",
 CellID->115061498]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Graphics"]], "MoreAbout",
 CellID->4015565]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"PlotConvergence - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 58, 10.6074000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics", "status" -> "", "summary" -> 
    "PlotConvergence[n, c] generates a graph showing convergence properties \
of an aerodynamic coefficient. The list of number of panels describing a \
discretized airfoil is denoted by n and the list of coefficients is denoted \
by c. The coefficient is plotted versus the reciprocal of the number of \
panels, and a straight line is fit to the resulting points. A reference value \
for the coefficient is determined by extrapolating the straight line to zero, \
and the function returns a list containing the reference coefficient and the \
graph. PlotConvergence[{n, c}, m] uses data from the last m values in the \
list n to determine a linear fit and reference value for the coefficient.", 
    "synonyms" -> {}, "title" -> "PlotConvergence", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/PlotConvergence"}, "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[6905, 194, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->304721846]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 46643, 1141}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[596, 21, 76, 1, 70, "SymbolColorBar"],
Cell[675, 24, 191, 5, 70, "LinkTrail"],
Cell[869, 31, 2843, 64, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3737, 99, 58, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3798, 102, 2126, 53, 70, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[5961, 160, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->412705712],
Cell[6544, 179, 195, 4, 70, "Notes",
 CellID->246272709],
Cell[6742, 185, 126, 4, 70, "Notes",
 CellID->74541308]
}, Closed]],
Cell[CellGroupData[{
Cell[6905, 194, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->304721846],
Cell[CellGroupData[{
Cell[7391, 211, 148, 5, 70, "ExampleSection",
 CellID->289022365],
Cell[7542, 218, 132, 6, 70, "ExampleText",
 CellID->337266530],
Cell[7677, 226, 100, 3, 70, "Input",
 CellID->207768003],
Cell[7780, 231, 306, 6, 70, "ExampleText",
 CellID->99847148],
Cell[CellGroupData[{
Cell[8111, 241, 816, 26, 83, "Input",
 CellID->524210768],
Cell[8930, 269, 5841, 103, 201, "Output",
 CellID->8892725]
}, Open  ]],
Cell[14786, 375, 132, 4, 70, "ExampleText",
 CellID->53729757],
Cell[14921, 381, 426, 14, 70, "Input",
 CellID->222327051],
Cell[15350, 397, 211, 5, 70, "ExampleText",
 CellID->3599668],
Cell[CellGroupData[{
Cell[15586, 406, 539, 16, 70, "Input",
 CellID->146492676],
Cell[16128, 424, 358, 10, 36, "Output",
 CellID->318993919]
}, Open  ]],
Cell[16501, 437, 152, 4, 70, "ExampleText",
 CellID->775435343],
Cell[CellGroupData[{
Cell[16678, 445, 972, 30, 70, "Input",
 CellID->53143172],
Cell[17653, 477, 1384, 32, 88, "Output",
 CellID->346077016]
}, Open  ]],
Cell[19052, 512, 134, 4, 70, "ExampleText",
 CellID->2060996],
Cell[CellGroupData[{
Cell[19211, 520, 157, 5, 70, "Input",
 CellID->57188429],
Cell[19371, 527, 216, 7, 36, "Output",
 CellID->333236422]
}, Open  ]],
Cell[19602, 537, 387, 10, 70, "ExampleText",
 CellID->388479374],
Cell[CellGroupData[{
Cell[20014, 551, 536, 17, 70, "Input",
 CellID->640039878],
Cell[20553, 570, 4036, 77, 348, "Output",
 CellID->28521870]
}, Open  ]],
Cell[CellGroupData[{
Cell[24626, 652, 467, 15, 70, "Input",
 CellID->153900572],
Cell[25096, 669, 4051, 78, 343, "Output",
 CellID->458396289]
}, Open  ]],
Cell[29162, 750, 151, 4, 70, "ExampleText",
 CellID->99280306],
Cell[CellGroupData[{
Cell[29338, 758, 123, 4, 70, "Input",
 CellID->202114905],
Cell[29464, 764, 260, 8, 36, "Output",
 CellID->290392090]
}, Open  ]],
Cell[29739, 775, 488, 8, 70, "ExampleText",
 CellID->248309304],
Cell[CellGroupData[{
Cell[30252, 787, 355, 12, 70, "Input",
 CellID->181676159],
Cell[30610, 801, 6141, 111, 351, "Output",
 CellID->592916800]
}, Open  ]],
Cell[CellGroupData[{
Cell[36788, 917, 355, 12, 70, "Input",
 CellID->402096401],
Cell[37146, 931, 6202, 115, 343, "Output",
 CellID->641705327]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[43409, 1053, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[43831, 1067, 177, 4, 70, "Tutorials",
 CellID->111584306],
Cell[44011, 1073, 147, 3, 70, "Tutorials",
 CellID->424464613],
Cell[44161, 1078, 160, 3, 70, "Tutorials",
 CellID->115061498]
}, Open  ]],
Cell[CellGroupData[{
Cell[44358, 1086, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[44780, 1100, 139, 3, 70, "MoreAbout",
 CellID->4015565]
}, Open  ]],
Cell[44934, 1106, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

