(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     15864,        475]
NotebookOptionsPosition[     12985,        384]
NotebookOutlinePosition[     14443,        424]
CellTagsIndexPosition[     14358,        419]
WindowTitle->SurfaceSpeed - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACLET SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.4 Airfoils From Conformal Mapping\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"], "\<\"3.2 \
Validation Example\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ValidationExample"], "\<\"Table of \
Contents\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"See Also \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilSurfacePoints\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/AirfoilSurfacePoints"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/SurfaceSpeed\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/SurfaceSpeed"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
SurfaceSpeed.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/SurfaceSpeed", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["SurfaceSpeed", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{
       ButtonBox["SurfaceSpeed",
        BaseStyle->"Link",
        ButtonData->"paclet:Aerodynamics/ref/SurfaceSpeed"], "[", 
       RowBox[{
        StyleBox["\[Alpha]", "TR"], ",", 
        StyleBox["t", "TI"], ",", 
        StyleBox["\[Tau]", "TR"], ",", 
        StyleBox["\[Theta]", "TR"]}], "]"}]], "InlineFormula"],
     " \[LineSeparator]computes surface speeds for a van de Vooren airfoil of \
thickness parameter ",
     Cell[BoxData[
      StyleBox["t", "TI"]], "InlineFormula"],
     ", trailing-edge angle ",
     Cell[BoxData[
      StyleBox["\[Tau]", "TR"]], "InlineFormula"],
     " and angle of attack ",
     Cell[BoxData[
      StyleBox["\[Alpha]", "TR"]], "InlineFormula"],
     " at a list of values for ",
     Cell[BoxData[
      StyleBox["\[Theta]", "TR"]], "InlineFormula"],
     " using an exact solution."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{
      Dynamic[
       If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
        RGBColor[0.92, 1, 0.59], None]]}}, "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->329962348],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->904363220],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->58415392],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->606767683],

Cell["\<\
Use a van de Vooren airfoil with maximum thickness of 0.12 chord, \
trailing-edge angle of 9.81\[Degree] at an angle of attack of 6\[Degree].  \
Compute surface speeds from the exact solution using SurfaceSpeed.\
\>", "ExampleText",
 CellID->771933209],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"t", ",", "\[Tau]", ",", "\[Alpha]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{"0.12", ",", 
     RowBox[{"9.81", "Degree"}], ",", 
     RowBox[{"6.", "Degree"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"n\[Theta]", "=", "50"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Theta]", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", 
     RowBox[{"2", "\[Pi]"}], ",", 
     RowBox[{"\[Pi]", "/", "n\[Theta]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rE", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"t", ",", "\[Tau]", ",", 
     RowBox[{"Take", "[", 
      RowBox[{"\[Theta]", ",", 
       RowBox[{"n\[Theta]", "+", "1"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"x", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rE", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vE", "=", 
   RowBox[{"SurfaceSpeed", "[", 
    RowBox[{"\[Alpha]", ",", "t", ",", "\[Tau]", ",", "\[Theta]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"Transpose", "[", 
    RowBox[{"{", 
     RowBox[{"x", ",", "vE"}], "}"}], "]"}], ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<V\>\""}], "}"}]}], ",", 
   RowBox[{"Joined", "\[Rule]", "True"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->663191981],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdlXtQE0ccxyOkvESBAWNKGpQSFSjiC5Xa6g9LLRXFR9tUGDGKiAoiqCCi
OALysFFQCCo+QqIJmgEUQgsICIKpGJHRAlVi5CEBCRBydzkSkBhhaqfj/tGd
udm72dm93d/3+/2s6+7YnyLMKBRKxMfn3/6/poNPb9bhgDPGCHBRsanyeA3s
zU9MZnYQsMz+vTh03wj4cyHHppAAh4OUfRrDCES6qg7l7yQgIrws902IFvZj
pcpoawIaS15ZXy7SwudLQhJXinCgW+6Iq9dqgYw6n3nXE4fKvDMqcgEGL1ny
rhwxBlxT6RcDOzCItrwfVTYDA6HtoHBtDgZhf0xX9EVqgfduy3JaIwbMlWyR
Xc0I/F3RkTATxyBJ4kphfdDA6/Tf2njOOIgC+Ok0Hw10KXMaOtfhYGvKvP82
bBha7kk6J2NxYBUMVOVmDoFLT2A1eQWHBPmQkCUehIV1Do5XG3HwzuvZKqlW
Q3ZGRpBMjcOirFfUeU8G4IAVrdJzOgFRq2xti9rfwjHOMW3yQgLOrc0vXq3s
hwreZ5JdQQS02CymjHT1AVdAtoZGffwu4Mys6lHBpdnXFo2lEWC4R+8QvukF
anj48WtXCTj1p6Xz+odv4FQVX1lSTIBNUXD70aZu4H890RxfRYA9O9hc8qwT
DhQccvGsIyAm3C5D2qkEdSrfXl37UaeKlv5tpAKsreLMxOUEZG7N4P5s3wEW
sUV5PkIC9o6fNgtc9QL+KnI4ueY0Aa1uFSFth9qBzxHsdg8hYEVXPDemshVU
tTt5M74koIuSYPJwfA6kRobdfo1D4nLzwI3dT0ERbFnnlYKDb8BqRl+LHKza
RHVedjhMxNkz9qgewVW1evrmNAyuYPIB1lwZzGUWcKYptEA/66274dQA0+7t
cTOZaaFx0eg2hX8tXDywXZ4+qYEtInGqpK0SAkrzvDg1wyC9ebgz6WE5RCSk
9lmyhmBXf7mh0+sO5JOCMCs3NfTSneXnbt+Cb3W3xMkX+sGPvUwwMCyEtFKa
ifTuBcrzhm9KfS8D/ZdMZuVmBVDOEs27UrOg11kYoX/cBJQ9vx6ptkuBqewz
vYPSGqBssGNnXY4GwctY0ZBfD1CcSnoU1zeC0syPdWIJjnKRQpvHfFo9jsbH
A9jxIqMJzbdw6zYaXabQ+hs2UFubN02h/5vfeH48qXQS7S/t1A/cWe0f0P4d
58V3OOpN6HyFOwcZdJYJnb8yT+frEvke1WcyKfCY00Mjqt/w90tXJngYUX3b
tknrrwgnUP2DHrBy6K4TSJ8761Ni9CXvkH4VviHS9DXvkL4Mc5koUTGO9J9a
4HD9wdFx5I8SwazdJ+njyD+KH7mF5g1jyF+Dv0eGNu8fQ/67xF2n5cwaQ/6M
Y/Zfj35kQP49HxKw8O5RA/K357L9QRp3A/L/HPeRMmO3HuWDN+1WWe1FPcrP
Cwsl2ylIj/KVyxZI9RZ6lD/hg8KIQNkoyqdMRTsyI3kU5VcT0GTjs3oU5Zv/
Ym1u1QcS5Z/l65GXXU8iPsy3SHoiTSYRPxjBHCrNn0R8kZwQWkktScQf9+jW
+Qef6RCf3LNUOcsv6hC/NHPqvxrerkN8e+y/lB/K0iH+lUdbLy5+TSA+evjF
cagHCcTPFmbMiqEpHPF1ssmZN/sCjvgr5nXfrmfgiM+hzc6JVBGG+P3dXgc7
HyaG+G6guhiz07WI/2GHT2+SPRqBT/eDPS9GXnxTA/+/P/4Bzq8f9A==
    "]]},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"V\"", TraditionalForm]},
  ImageSize->504,
  PlotRange->Automatic,
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 327},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->426318428]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["SEE ALSO",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "SeeAlsoSection",
 CellID->1255426704],

Cell[TextData[Cell[BoxData[
 StyleBox[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"],
  FontFamily->"Verdana"]], "InlineFormula"]], "SeeAlso",
 CellID->40167389]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["1.4 Airfoils From Conformal Mapping",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["3.2 Validation Example",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/ValidationExample"]], "Tutorials",
 CellID->818994628],

Cell[TextData[ButtonBox["Table of Contents",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/TableOfContents"]], "Tutorials",
 CellID->140088713]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "MoreAbout",
 CellID->262017434]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"SurfaceSpeed - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 58, 49.5450000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics", "status" -> "", "summary" -> 
    "SurfaceSpeed[\\[Alpha], t, \\[Tau], \\[Theta]] computes surface speeds \
for a van de Vooren airfoil of thickness parameter t, trailing-edge angle \
\\[Tau] and angle of attack \\[Alpha] at a list of values for \\[Theta] using \
an exact solution.", "synonyms" -> {}, "title" -> "SurfaceSpeed", "type" -> 
    "Symbol", "uri" -> "Aerodynamics/ref/SurfaceSpeed"}, 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5502, 152, 461, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->329962348]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 14215, 412}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[593, 21, 76, 1, 70, "SymbolColorBar"],
Cell[672, 24, 191, 5, 70, "LinkTrail"],
Cell[866, 31, 3298, 75, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4189, 110, 55, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[4247, 113, 1218, 34, 70, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[5502, 152, 461, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->329962348],
Cell[CellGroupData[{
Cell[5988, 169, 148, 5, 70, "ExampleSection",
 CellID->904363220],
Cell[6139, 176, 131, 6, 70, "ExampleText",
 CellID->58415392],
Cell[6273, 184, 100, 3, 70, "Input",
 CellID->606767683],
Cell[6376, 189, 262, 5, 70, "ExampleText",
 CellID->771933209],
Cell[CellGroupData[{
Cell[6663, 198, 1692, 50, 70, "Input",
 CellID->663191981],
Cell[8358, 250, 2274, 47, 348, "Output",
 CellID->426318428]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[10693, 304, 417, 12, 70, "SeeAlsoSection",
 CellID->1255426704],
Cell[11113, 318, 236, 6, 70, "SeeAlso",
 CellID->40167389]
}, Open  ]],
Cell[CellGroupData[{
Cell[11386, 329, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[11808, 343, 194, 4, 70, "Tutorials",
 CellID->341631938],
Cell[12005, 349, 167, 3, 70, "Tutorials",
 CellID->818994628],
Cell[12175, 354, 160, 3, 70, "Tutorials",
 CellID->140088713]
}, Open  ]],
Cell[CellGroupData[{
Cell[12372, 362, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[12794, 376, 149, 3, 70, "MoreAbout",
 CellID->262017434]
}, Open  ]],
Cell[12958, 382, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

