(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     20699,        609]
NotebookOptionsPosition[     17205,        494]
NotebookOutlinePosition[     18599,        533]
CellTagsIndexPosition[     18514,        528]
WindowTitle->XToTheta - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "SymbolColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS PACLET SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.2 Validation Example\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ValidationExample"], "\<\"Table of \
Contents\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Airfoil Geometry\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/AirfoilGeometry"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/ref/XToTheta\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/ref/XToTheta"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/ref/\
XToTheta.html"], StandardForm]], "Input", TextClipboardType -> "PlainText"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/ref/XToTheta", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["XToTheta", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{
       ButtonBox["XToTheta",
        BaseStyle->"Link",
        ButtonData->"paclet:Aerodynamics/ref/XToTheta"], "[", 
       RowBox[{
        StyleBox["t", "TI"], ",", 
        StyleBox["\[Tau]", "TR"], ",", 
        StyleBox["x", "TI"]}], "]"}]], "InlineFormula"],
     " \[LineSeparator]converts a list of ",
     Cell[BoxData[
      StyleBox["x", "TI"]], "InlineFormula"],
     "-values to a list of ",
     Cell[BoxData[
      StyleBox["\[Theta]", "TR"]], "InlineFormula"],
     "-values using the van de Vooren transformation for an airfoil of \
maximum thickness ",
     Cell[BoxData[
      StyleBox["t", "TI"]], "InlineFormula"],
     " and trailing-edge angle ",
     Cell[BoxData[
      StyleBox["\[Tau]", "TR"]], "InlineFormula"],
     "."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{
      Dynamic[
       If[CurrentValue[{TaggingRules, "ModificationHighlight"}] === True, 
        RGBColor[0.92, 1, 0.59], None]]}}, "RowsIndexed" -> {}}},
 CellID->982511436]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 ButtonBox[
  FrameBox[
   StyleBox[
    RowBox[{"MORE", " ", "INFORMATION"}], "NotesFrameText"],
   StripOnInput->False],
  Appearance->{Automatic, None},
  BaseStyle->None,
  ButtonFunction:>(FrontEndExecute[{
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], All, ButtonCell], 
     FrontEndToken["OpenCloseGroup"], 
     FrontEnd`SelectionMove[
      FrontEnd`SelectedNotebook[], After, CellContents]}]& ),
  Evaluator->None,
  Method->"Preemptive"]]]], "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->867080230],

Cell["This function is specific to the van de Vooren airfoil.", "Notes",
 CellID->1067943069]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->55892326],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection",
 CellID->299782951],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "ExampleText",
 CellID->311150377],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->85203508],

Cell[TextData[{
 "Specify the shape parameters for a van de Vooren airfoil and a list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values to be used for discretization."
}], "ExampleText",
 CellID->450653109],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "=", "0.12"}], ";", 
  RowBox[{"\[Tau]", "=", 
   RowBox[{"9.81", " ", "Degree"}]}], ";", 
  RowBox[{"x", "=", 
   RowBox[{"NDiscretizeUnitSegment", "[", "10", "]"}]}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->475849417],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.`", ",", "0.1`", ",", "0.2`", ",", "0.30000000000000004`", ",", "0.4`", 
   ",", "0.5`", ",", "0.6000000000000001`", ",", "0.7000000000000001`", ",", 
   "0.8`", ",", "0.9`", ",", "1.`"}], "}"}]], "Output",
 ImageSize->{345, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->135653920]
}, Open  ]],

Cell[TextData[{
 "Compute a list of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 "-values corresponding the the list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values."
}], "ExampleText",
 CellID->105439063],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Theta]x", "=", 
  RowBox[{"XToTheta", "[", 
   RowBox[{"t", ",", "\[Tau]", ",", "x"}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->75917125],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.`", ",", "0.5910009410620326`", ",", "0.8632693526618288`", ",", 
   "1.089793647075696`", ",", "1.2980488921524616`", ",", 
   "1.500228040203691`", ",", "1.7048671489501932`", ",", 
   "1.9208649027663334`", ",", "2.1618151247310737`", ",", 
   "2.4595129237834996`", ",", "3.141592653589793`"}], "}"}]], "Output",
 ImageSize->{368, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->117058701]
}, Open  ]],

Cell[TextData[{
 "Using this list of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 "-values, points on the surface of the van de Vooren airfoil are computed \
and plotted."
}], "ExampleText",
 CellID->109577907],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"rP", "=", 
  RowBox[{"AirfoilSurfacePoints", "[", 
   RowBox[{"t", ",", "\[Tau]", ",", "\[Theta]x"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"rP", ",", 
   RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->261648493],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "1.`", ",", "0.899999980287212`", ",", "0.7999999863643488`", ",", 
     "0.699999995417208`", ",", "0.5999999985460058`", ",", 
     "0.500000000029821`", ",", "0.40000000164066973`", ",", 
     "0.30000000157567475`", ",", "0.20000000338593604`", ",", 
     "0.10000002231542593`", ",", "0.`", ",", "0.10000002231542593`", ",", 
     "0.20000000338593604`", ",", "0.30000000157567475`", ",", 
     "0.40000000164066973`", ",", "0.500000000029821`", ",", 
     "0.5999999985460058`", ",", "0.699999995417208`", ",", 
     "0.7999999863643488`", ",", "0.899999980287212`", ",", "1.`"}], "}"}], 
   "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"0.`", ",", 
     RowBox[{"-", "0.010312012043222865`"}], ",", 
     RowBox[{"-", "0.021464527601938982`"}], ",", 
     RowBox[{"-", "0.03240179792112437`"}], ",", 
     RowBox[{"-", "0.04243950621293667`"}], ",", 
     RowBox[{"-", "0.050919001767322294`"}], ",", 
     RowBox[{"-", "0.05708721539362682`"}], ",", 
     RowBox[{"-", "0.059944555567619906`"}], ",", 
     RowBox[{"-", "0.057898210452918875`"}], ",", 
     RowBox[{"-", "0.04751306356231155`"}], ",", "1.1102230246251565`*^-16", 
     ",", "0.04751306356231155`", ",", "0.057898210452918875`", ",", 
     "0.059944555567619906`", ",", "0.05708721539362682`", ",", 
     "0.050919001767322294`", ",", "0.04243950621293667`", ",", 
     "0.03240179792112437`", ",", "0.021464527601938982`", ",", 
     "0.010312012043222865`", ",", "0.`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{1., 0.899999980287212, 
    0.7999999863643488, 0.699999995417208, 0.5999999985460058, 
    0.500000000029821, 0.40000000164066973`, 0.30000000157567475`, 
    0.20000000338593604`, 0.10000002231542593`, 0., 0.10000002231542593`, 
    0.20000000338593604`, 0.30000000157567475`, 0.40000000164066973`, 
    0.500000000029821, 0.5999999985460058, 0.699999995417208, 
    0.7999999863643488, 0.899999980287212, 1.}, {
    0., -0.010312012043222865`, -0.021464527601938982`, -0.03240179792112437, \
-0.04243950621293667, -0.050919001767322294`, -0.05708721539362682, \
-0.059944555567619906`, -0.057898210452918875`, -0.04751306356231155, 
    1.1102230246251565`*^-16, 0.04751306356231155, 0.057898210452918875`, 
    0.059944555567619906`, 0.05708721539362682, 0.050919001767322294`, 
    0.04243950621293667, 0.03240179792112437, 0.021464527601938982`, 
    0.010312012043222865`, 0.}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{544, 91},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->663639947],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3b+O61odBeAgnoTXGAlxBaKBDglRIaRbpOQ28A4IgZQWQQHoQkMJMi0S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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->756864300]
}, Open  ]],

Cell[TextData[{
 "Note that the points on the upper surface of the airfoil have ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values corresponding to the values list specified for discretization."
}], "ExampleText",
 CellID->318374758],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Take", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Components", "[", "rP", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}], ",", 
   RowBox[{"-", "11"}]}], "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->281182168],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.`", ",", "0.10000002231542593`", ",", "0.20000000338593604`", ",", 
   "0.30000000157567475`", ",", "0.40000000164066973`", ",", 
   "0.500000000029821`", ",", "0.5999999985460058`", ",", 
   "0.699999995417208`", ",", "0.7999999863643488`", ",", 
   "0.899999980287212`", ",", "1.`"}], "}"}]], "Output",
 ImageSize->{345, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->130677185]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["TUTORIALS",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "TutorialsSection",
 CellID->250839057],

Cell[TextData[ButtonBox["3.2 Validation Example",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/ValidationExample"]], "Tutorials",
 CellID->341631938],

Cell[TextData[ButtonBox["Table of Contents",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/TableOfContents"]], "Tutorials",
 CellID->812039269]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 CellID->38303248],

Cell[TextData[ButtonBox["Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/AirfoilGeometry"]], "MoreAbout",
 CellID->6407222]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"XToTheta - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 59, 6.5958000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Paclet Symbol", "language" -> "en", "paclet" -> 
    "Aerodynamics", "status" -> "", "summary" -> 
    "XToTheta[t, \\[Tau], x] converts a list of x-values to a list of \
\\[Theta]-values using the van de Vooren transformation for an airfoil of \
maximum thickness t and trailing-edge angle \\[Tau].", "synonyms" -> {}, 
    "title" -> "XToTheta", "type" -> "Symbol", "uri" -> 
    "Aerodynamics/ref/XToTheta"}, "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[5541, 163, 460, 13, 70, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->55892326]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 18372, 521}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[589, 21, 76, 1, 70, "SymbolColorBar"],
Cell[668, 24, 191, 5, 70, "LinkTrail"],
Cell[862, 31, 2715, 62, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3602, 97, 51, 1, 70, "ObjectName",
 CellID->1224892054],
Cell[3656, 100, 1135, 33, 70, "Usage",
 CellID->982511436]
}, Open  ]],
Cell[CellGroupData[{
Cell[4828, 138, 580, 17, 70, "NotesSection",
 CellGroupingRules->{"SectionGrouping", 50},
 CellID->867080230],
Cell[5411, 157, 93, 1, 70, "Notes",
 CellID->1067943069]
}, Closed]],
Cell[CellGroupData[{
Cell[5541, 163, 460, 13, 70, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->55892326],
Cell[CellGroupData[{
Cell[6026, 180, 148, 5, 70, "ExampleSection",
 CellID->299782951],
Cell[6177, 187, 132, 6, 70, "ExampleText",
 CellID->311150377],
Cell[6312, 195, 99, 3, 70, "Input",
 CellID->85203508],
Cell[6414, 200, 229, 6, 70, "ExampleText",
 CellID->450653109],
Cell[CellGroupData[{
Cell[6668, 210, 262, 8, 70, "Input",
 CellID->475849417],
Cell[6933, 220, 382, 10, 36, "Output",
 CellID->135653920]
}, Open  ]],
Cell[7330, 233, 253, 9, 70, "ExampleText",
 CellID->105439063],
Cell[CellGroupData[{
Cell[7608, 246, 173, 5, 70, "Input",
 CellID->75917125],
Cell[7784, 253, 492, 12, 54, "Output",
 CellID->117058701]
}, Open  ]],
Cell[8291, 268, 233, 7, 70, "ExampleText",
 CellID->109577907],
Cell[CellGroupData[{
Cell[8549, 279, 321, 9, 70, "Input",
 CellID->261648493],
Cell[8873, 290, 2657, 51, 112, "Output",
 CellID->663639947],
Cell[11533, 343, 3192, 60, 201, "Output",
 CellID->756864300]
}, Open  ]],
Cell[14740, 406, 252, 6, 70, "ExampleText",
 CellID->318374758],
Cell[CellGroupData[{
Cell[15017, 416, 237, 8, 70, "Input",
 CellID->281182168],
Cell[15257, 426, 480, 12, 36, "Output",
 CellID->130677185]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[15798, 445, 419, 12, 70, "TutorialsSection",
 CellID->250839057],
Cell[16220, 459, 167, 3, 70, "Tutorials",
 CellID->341631938],
Cell[16390, 464, 160, 3, 70, "Tutorials",
 CellID->812039269]
}, Open  ]],
Cell[CellGroupData[{
Cell[16587, 472, 419, 12, 70, "MoreAboutSection",
 CellID->38303248],
Cell[17009, 486, 154, 3, 70, "MoreAbout",
 CellID->6407222]
}, Open  ]],
Cell[17178, 492, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

