(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     51551,       1478]
NotebookOptionsPosition[     44689,       1268]
NotebookOutlinePosition[     47301,       1322]
CellTagsIndexPosition[     47258,       1319]
WindowTitle->3.6  Aerodynamic Coefficients - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 StyleBox[ButtonBox["Mathematica",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:guide/Mathematica"],
  FontSlant->"Italic"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.5 Convergence and Timing\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"], "\<\"3.7 \
Analysis of an Airfoil\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/AnalysisOfAnAirfoil"], "\<\"Table Of \
Contents\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PanelMethods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/\
AerodynamicCoefficientsFromPanelMethods\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
             "Aerodynamics/tutorial/AerodynamicCoefficientsFromPanelMethods"]}\
, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/AerodynamicCoefficientsFromPanelMethods.html"], StandardForm]], 
              "Input", TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/AerodynamicCoefficientsFromPanelMethods", 
             ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["3.6  Aerodynamic Coefficients", "Title",
 CellID->509267359],

Cell[TextData[{
 "Computation of aerodynamic characteristics of airfoils in previous sections \
has played a supporting role in the discussions of validation and \
convergence.  This section focuses on how pressure, lift and pitching-,moment \
coefficients are obtained using ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " functions.  Choose ",
 Cell[BoxData[
  ButtonBox["PMDoubletQuadraticDirichlet",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMDoubletQuadraticDirichlet"]], 
  "InlineFormula"],
 " as the panel method and the NACA-4412 airfoil for the example in this \
section.  "
}], "Text",
 CellID->131238241],

Cell[TextData[{
 "The reason for choosing the NACA-4412 airfoil is because its geometry is \
available in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " both as a data file and as equations for thickness and camber properties.  \
A secondary topic for this section is to evaluate the effect on aerodynamic \
characteristics of discretization based a B-spline fit to the data in a file. \
 Three discretized airfoils are specified: first, a user-specified \
discretization based on equations for thickness and camber properties, \
second, a discretization by the data in the file, and finally, a \
discretization based on a B-spline fit to the data in the file using the same \
user-specified discretization parameters as the first case.  Comparison of \
surface pressure distribution for the three discretizations provides a \
measure consistency among the three methods of discretization."
}], "Text",
 CellID->92798100],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->836781195],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[0]:=",
 CellID->2058623809],

Cell[CellGroupData[{

Cell["3.6.1 NACA-4412 Airfoil", "Section",
 CellID->259819136],

Cell[TextData[{
 "Specify angle of attack and discretization parameters.  Generate the list \
of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values on which the user-specified discretization is based."
}], "Text",
 CellID->267612329],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"4.", " ", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"n", "=", "50"}], ";", 
  RowBox[{"layout", "=", "HalfCosineSpacing"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"x", "=", 
   RowBox[{"NDiscretizeUnitSegment", "[", 
    RowBox[{"n", ",", 
     RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->17065800],

Cell[TextData[{
 "Discretize the airfoil using with thickness and camber properties.  Note \
that the trailing edge is not closed, but that the mean value of the two \
locations is ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{"1", ",", "0"}], "}"}]], "InlineFormula"],
 " as expected."
}], "Text",
 CellID->96382008],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP1", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4412", ",", "x"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"First", "[", "rP1", "]"}], ",", 
   RowBox[{"Last", "[", "rP1", "]"}], ",", 
   RowBox[{"Chop", "[", 
    RowBox[{"Mean", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"First", "[", "rP1", "]"}], ",", 
       RowBox[{"Last", "[", "rP1", "]"}]}], "}"}], "]"}], "]"}]}], 
  "}"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->238618617],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.9998334737126852`", ",", 
     RowBox[{"-", "0.0012489471548601018`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.0001665262873145`", ",", "0.0012489471548601265`"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"0.9999999999999998`", ",", "0"}], "}"}]}], "}"}]], "Output",
 ImageSize->{389, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->558659956]
}, Open  ]],

Cell["\<\
PMDoubletQuadraticDirichlet requires a closed trailing edge to provide \
plausible results.  Modify the collection of vectors locating panel endpoints \
to close the trailing edge of the airfoil.\
\>", "Text",
 CellID->231578421],

Cell[BoxData[{
 RowBox[{
  RowBox[{"pt1", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"First", "[", "rP1", "]"}], ",", 
      RowBox[{"Last", "[", "rP1", "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP1Closed", "=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{"rP1", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "\[Rule]", "pt1"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "\[Rule]", "pt1"}]}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->133089996],

Cell[TextData[{
 "Specify a discretized airfoil using a data file for the NACA-4412 airfoil.  \
The original data file downloaded form the UIUC Airfoil Coordinates Database \
has been modified as discussed in the Possible Issues section of the \
documentation for ",
 Cell[BoxData[
  ButtonBox["AirfoilFromDataFile",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"]], 
  "InlineFormula"],
 ".  Note that the trailing-edges are not exactly the same as those defined \
by thickness and camber properties, but that the mean value of the two \
locations is ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{"1", ",", " ", "0"}], "}"}]], "InlineFormula"],
 " as expected."
}], "Text",
 CellID->583936543],

Cell[CellGroupData[{

Cell["\<\
rD = AirfoilFromDataFile[\"Aerodynamics\\\\Airfoil Data\\\\naca4412m.dat\"];
{First[rD],Last[rD],Mean[{First[rD],Last[rD]}]}\
\>", "Input",
 CellLabel->"In[8]:=",
 CellID->108778828],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1.`", ",", 
     RowBox[{"-", "0.0013`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.0013`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.`"}], "}"}]}], "}"}]], "Output",
 ImageSize->{263, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->570743163]
}, Open  ]],

Cell["\<\
Modify the data to close the trailing edge.  This is a relatively coarse \
discretization compared to the user-specified discretization as indicated by \
the number of panels for this discretization.\
\>", "Text",
 CellID->113191753],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ptD", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"First", "[", "rD", "]"}], ",", 
      RowBox[{"Last", "[", "rD", "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rDClosed", "=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{"rD", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "\[Rule]", "ptD"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "\[Rule]", "ptD"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Length", "[", 
  RowBox[{"PanelLengths", "[", "rD", "]"}], "]"}]}], "Input",
 CellLabel->"In[10]:=",
 CellID->92323877],

Cell[BoxData["34"], "Output",
 ImageSize->{18, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->198740885]
}, Open  ]],

Cell["\<\
Fit a B-spline to the data for the NACA-4412 airfoil and rediscretize the \
airfoil using the same user-specified discretization.  Once again, note the \
coordinates of the trailing-edge.\
\>", "Text",
 CellID->11042490],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"bSpline", "=", 
   RowBox[{"FitAirfoil", "[", "rD", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP2", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"bSpline", ",", "x"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{
   RowBox[{"First", "[", "rP2", "]"}], ",", 
   RowBox[{"Last", "[", "rP2", "]"}], ",", 
   RowBox[{"Chop", "[", 
    RowBox[{"Mean", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"First", "[", "rP2", "]"}], ",", 
       RowBox[{"Last", "[", "rP2", "]"}]}], "}"}], "]"}], "]"}]}], 
  "}"}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->601516212],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0.9999999999999997`", ",", 
     RowBox[{"-", "0.0012999999999999793`"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1.`", ",", "0.0013000000000000281`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0.9999999999999998`", ",", "0"}], "}"}]}], "}"}]], "Output",
 ImageSize->{256, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->2523348]
}, Open  ]],

Cell["\<\
Modify the collection of vectors locating panel endpoints to close the \
trailing edge of the airfoil.\
\>", "Text",
 CellID->2918513],

Cell[BoxData[{
 RowBox[{
  RowBox[{"pt2", "=", 
   RowBox[{"Mean", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"First", "[", "rP2", "]"}], ",", 
      RowBox[{"Last", "[", "rP2", "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rP2Closed", "=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{"rP2", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"1", "\[Rule]", "pt2"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "\[Rule]", "pt2"}]}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[16]:=",
 CellID->805149539]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.6.2 Surface Pressure Coefficient", "Section",
 CellID->6814950],

Cell[TextData[{
 "In addition to strength parameters, the model solution provides locations \
of collocation points and surface speeds associated with collocation points.  \
For doublet models with internal Dirichlet boundary conditions, collocation \
points are located just inside the airfoil, and the velocity potential at \
collocation points is set to zero.  Jump conditions (see ",
 Cell[BoxData[
  ButtonBox["ICDoubletQuadratic",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/ICDoubletQuadratic"]], 
  "InlineFormula"],
 ") across a panel are used to determine the velocity potential just outside \
the airfoil .  An interpolation curve is fit to the external velocity \
potentials as a function of arc length along airfoil, and surface speed is \
obtained by taking the derivative of this function with respect to arc length."
}], "Text",
 CellID->115793861],

Cell["\<\
Solve the panel method PMDoubletQuadraticDirichlet for each of the three \
discretized airfoils.\
\>", "Text",
 CellID->42676348],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a1", ",", "b1", ",", "c1", ",", "aw1", ",", "rC1", ",", "vC1"}], 
    "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rP1Closed", ",", "\[Alpha]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"aD", ",", "bD", ",", "cD", ",", "awD", ",", "rCD", ",", "vCD"}], 
    "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rDClosed", ",", "\[Alpha]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a2", ",", "b2", ",", "c2", ",", "aw2", ",", "rC2", ",", "vC2"}], 
    "}"}], "=", 
   RowBox[{"PMDoubletQuadraticDirichlet", "[", 
    RowBox[{"rP2Closed", ",", "\[Alpha]"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[18]:=",
 CellID->821293751],

Cell[TextData[{
 "Bernoulli's equation is used to obtain surface pressure coefficient from \
surface speed.  It is conventional in the aerodynamics community to plot \
surface pressure coefficient as a function of fraction chord.  The collection \
of vectors locating collocation points is denoted by ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["c", "TI"]]], "InlineFormula"],
 ", and the list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values for the plot is obtained using the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["Components",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/Components"]], "InlineFormula"],
 ", which is the selector for the data type CartesianVectors."
}], "Text",
 CellID->250599266],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xC1", ",", "cp1"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Components", "[", "rC1", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"vC1", "^", "2"}]}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xCD", ",", "cpD"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Components", "[", "rCD", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"vCD", "^", "2"}]}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xC2", ",", "cp2"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Components", "[", "rC2", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"vC2", "^", "2"}]}]}], "}"}]}], ";"}]}], "Input",
 CellLabel->"In[21]:=",
 CellID->47000734],

Cell[TextData[{
 "Superimpose the plots of surface pressure coefficients for the three \
discretized airfoils.  The surface pressure distribution for the \
user-specified discretized airfoil based on thickness and camber is used as \
the reference case, and is shown as a line joining values of ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["C",
    FontSlant->"Italic"], 
   StyleBox["p", "TI"]]], "InlineFormula"],
 " associated with collocation points.  Surface pressure coefficients \
computed from the discretized airfoil specified by the data file are shown as \
medium-sized red disks.  Results obtained for the user-specified \
discretization based the fit of a B-spline to the data in the file are shown \
as red dots.  Additionally, the shape of the airfoil is shown ."
}], "Text",
 CellID->139613108],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"g1", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC1", ",", "cp1", ",", "rP1", ",", 
     RowBox[{"ImageSize", "\[Rule]", 
      RowBox[{"7.8", " ", "72"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gD", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xCD", ",", "cpD", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"PointSize", "[", "Medium", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g2", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC2", ",", "cp2", ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"g1", ",", "gD", ",", "g2"}], "]"}]}], "Input",
 CellLabel->"In[24]:=",
 CellID->314038555],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJw9kXs0FPgCx4U1XplFWqtWdbfX3s0VyWZOzXebtNmobNFINaJsp1IssrFi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     "]]}, 
   {RGBColor[1, 0, 0], PointSize[Medium], PointBox[CompressedData["
1:eJxTTMoPSmViYGBQAmIQvfqQkbGx8Xv7NSUOSknZp/cn5c8Egrdwftp3kPxr
+wvxZ7RENp7a37vkPxA8tw9IPPRFfs+p/bsCzwDBE/uCsP0Hl504tf8XD0j/
Q/uczo6bE8qB6jefBsrfsd+w3P6GQtKp/dzKqWlpadfsP/xnOWU1//R+C48Z
QPUX7VuO5Agr/T69n0vrFMhA+yvHfRUvXzm9v/ZOClD9Mfs383iUFApP7fe2
ADqa4YD9nfKkB1wbTu9/uzobKL/N3mfD5OnVPOf3n/jABZTfYM9hVHDggs+l
/RcXJALdv9j+1/9CnxM/r+//K38HyJ9sv7F4p9hJlkf778gdmzdzZqW9hNND
yajrb/bbBm6fD+L/aG35diT4xv62QFsTkPpjGcu1w3w+278WnQ0270bBjFdm
Bt/tUx5pgu3bIv3VacWqr/bR3J1g9xw66x0yv/Wrfd5VXrB7Xabs2RXT/sU+
164A7J8LM//78ph/sdcsvQz2r1BaMOf2VZ/tf7fMBodHkaH+f4P/n+x7mtLA
4SXXwvzxS+xH+0gFSHiu0jgb7vj8rb2+1gxweNfVPLjnVfXC/vnlU+D42PCN
QdXzymP7zRP+guNrReeqRzuD79mfdDUAx2fGtiWKU9ZdsV/vNQ0c3wvmOgfF
HThifyJIH5weGBK+pkT5pNgDACK9HRU=
     "]]}, 
   {RGBColor[1, 0, 0], PointBox[CompressedData["
1:eJw91Ws41FkcB3BbWGwYi2ioTSbVRrdtM2T9WrFRsi3WZZs2BrOFXVs0bEtu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     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  Epilog->InsetBox[
    GraphicsBox[{
      Hue[0.67, 0.6, 0.6], 
      LineBox[CompressedData["
1:eJxdlfs7FPgCh4mQynoUSzcNmiQt4uRSfOREzyrVSqqp1bKUI5TCtrmkGpeI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       "]]}, {FrameLabel -> {
        FormBox["\"x\"", TraditionalForm], 
        FormBox["\"z\"", TraditionalForm]}, FrameTicks -> {{{-1.2, 
          FormBox[
           RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
          FormBox[
           RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
          FormBox[
           RowBox[{"-", "0.7999999999999999`"}], 
           TraditionalForm]}, {-0.5999999999999999, 
          FormBox[
           RowBox[{"-", "0.5999999999999999`"}], 
           TraditionalForm]}, {-0.3999999999999999, 
          FormBox[
           RowBox[{"-", "0.3999999999999999`"}], 
           TraditionalForm]}, {-0.19999999999999996`, 
          FormBox[
           RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
          FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
          FormBox["0.20000000000000018`", TraditionalForm]}, {
         0.40000000000000013`, 
          FormBox["0.40000000000000013`", TraditionalForm]}, {
         0.6000000000000001, 
          FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
          FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
          FormBox["1.0000000000000002`", TraditionalForm]}, {
         1.2000000000000004`, 
          FormBox["1.2000000000000004`", TraditionalForm]}, {
         1.4000000000000001`, 
          FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
          FormBox["1.6`", TraditionalForm]}}, {{-1., 
          FormBox[
           RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
          FormBox[
           RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
          FormBox[
           RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
          FormBox[
           RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
          FormBox[
           RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
          FormBox[
           RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
          FormBox[
           RowBox[{"-", "0.3999999999999999`"}], 
           TraditionalForm]}, {-0.29999999999999993`, 
          FormBox[
           RowBox[{"-", "0.29999999999999993`"}], 
           TraditionalForm]}, {-0.19999999999999996`, 
          FormBox[
           RowBox[{"-", "0.19999999999999996`"}], 
           TraditionalForm]}, {-0.09999999999999998, 
          FormBox[
           RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
          FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
          FormBox["0.10000000000000009`", TraditionalForm]}, {
         0.20000000000000018`, 
          FormBox["0.20000000000000018`", TraditionalForm]}, {
         0.30000000000000004`, 
          FormBox["0.30000000000000004`", TraditionalForm]}, {
         0.40000000000000013`, 
          FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
          FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
          FormBox["0.6000000000000001`", TraditionalForm]}, {
         0.7000000000000002, 
          FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
          FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
          FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
          FormBox["1.`", TraditionalForm]}}, None, {{-1., 
          FormBox[
           RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
          FormBox[
           RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
          FormBox[
           RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
          FormBox[
           RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
          FormBox[
           RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
          FormBox[
           RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
          FormBox[
           RowBox[{"-", "0.3999999999999999`"}], 
           TraditionalForm]}, {-0.29999999999999993`, 
          FormBox[
           RowBox[{"-", "0.29999999999999993`"}], 
           TraditionalForm]}, {-0.19999999999999996`, 
          FormBox[
           RowBox[{"-", "0.19999999999999996`"}], 
           TraditionalForm]}, {-0.09999999999999998, 
          FormBox[
           RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
          FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
          FormBox["0.10000000000000009`", TraditionalForm]}, {
         0.20000000000000018`, 
          FormBox["0.20000000000000018`", TraditionalForm]}, {
         0.30000000000000004`, 
          FormBox["0.30000000000000004`", TraditionalForm]}, {
         0.40000000000000013`, 
          FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
          FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
          FormBox["0.6000000000000001`", TraditionalForm]}, {
         0.7000000000000002, 
          FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
          FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
          FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
          FormBox["1.`", TraditionalForm]}}}, ImageSize -> {504., 180.}, 
      PlotRange -> {{0, 1}, Automatic}, PlotRangeClipping -> True, 
      RotateLabel -> False}], {0, 0}, {0, 0}, 1],
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox[
    "\"\\!\\(\\*SubscriptBox[\\\"C\\\", \\\"p\\\"]\\)\"", TraditionalForm]},
  FrameTicks->{Automatic, {{-1, 
      FormBox["\"1\"", TraditionalForm]}, {0, 
      FormBox["\"0\"", TraditionalForm]}, {1, 
      FormBox["\"-1\"", TraditionalForm]}, {2, 
      FormBox["\"-2\"", TraditionalForm]}}, None, None},
  ImageSize->561.6,
  PlotRange->{{-0.05, 1.05}, {-1.2, 2}},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{566, 365},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[27]=",
 CellID->69621107]
}, Open  ]],

Cell[TextData[{
 "Except in a small region near the negative ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["C", "TI"], 
   StyleBox["p", "TI"]]], "InlineFormula"],
 " peak, there is close agreement between results for the two implementations \
of the same user-specified discretization, one based on a specification of \
the airfoil by thickness and camber properties, and the other based on a \
spline fit to data specifying the airfoil.  There is a small-magnitude \
oscillation of ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["C", "TI"], 
   StyleBox["p", "TI"]]], "InlineFormula"],
 " on the upper surface near the pressure peak for discretization based on \
the B-spline which suggests the need for additional smoothing.  The plot also \
shows a small but noticeable difference in pressure distribution between the \
relatively coarse discretization provided by the data file and the finer \
user-specified discretization."
}], "Text",
 CellID->20411779]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.6.3 Lift and pitching moment from pressure coefficient", "Section",
 CellID->252223901],

Cell[TextData[{
 "Lift and pitching-moment coefficients can be computed from the surface \
pressure distribution on the airfoil.   The lift coefficient can be written \
as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["C", "\[ScriptL]"], "=", 
    RowBox[{"-", 
     RowBox[{"\[ContourIntegral]", 
      RowBox[{
       SubscriptBox["C", "p"], " ", 
       RowBox[{
        OverscriptBox["n", "^"], "\[CenterDot]", 
        OverscriptBox["\[ScriptL]", "^"]}], 
       RowBox[{"\[DifferentialD]", "\[ScriptS]"}]}]}]}]}], TraditionalForm]]],
 ", where the integral is over the airfoil contour, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "p"], TraditionalForm]]],
 " is the pressure coefficient, ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["n", "^"], TraditionalForm]]],
 " is the outward unit normal to the airfoil surface, and ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["\[ScriptL]", "^"], TraditionalForm]]],
 " is a unit vector perpendicular to the free stream velocity in the \
direction of positive lift.  The moment coefficient can be thought of as the ",
 Cell[BoxData[
  StyleBox["y", "TI"]], "InlineFormula"],
 "-component of the vector ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    RowBox[{"\[ContourIntegral]", 
     RowBox[{
      OverscriptBox["r", "\[RightVector]"], "\[Times]", 
      OverscriptBox["n", "^"], 
      SubscriptBox["C", "p"], " ", 
      RowBox[{"\[DifferentialD]", "\[ScriptS]"}]}]}]}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["r", "\[RightVector]"], TraditionalForm]]],
 " locates a point on the airfoil surface.  The integrals are approximated by \
considering the pressure coefficient constant over each panel, computing the \
contribution to lift and moment for each panel, and summing the results.  The \
user-specified discretization of the airfoil based on thickness and camber \
properties is used in the following example."
}], "Text",
 CellID->126296105],

Cell[TextData[{
 "There are ",
 StyleBox["Aerodynamic",
  FontSlant->"Italic"],
 " functions to compute panel lengths and unit vectors to panels."
}], "Text",
 CellID->195800744],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Delta]", "=", 
   RowBox[{"PanelLengths", "[", "rP1", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"un", "=", 
   RowBox[{"PanelNormals", "[", "rP1", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"nP", "=", 
   RowBox[{"Length", "[", "\[Delta]", "]"}]}], ";"}]}], "Input",
 CellLabel->"In[28]:=",
 CellID->41528148],

Cell[TextData[{
 "Collections of vectors in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " are of the data type CartesianVectors."
}], "Text",
 CellID->489123521],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", "un", "]"}]], "Input",
 CellLabel->"In[31]:=",
 CellID->992671459],

Cell[BoxData["CartesianVectors"], "Output",
 ImageSize->{116, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[31]=",
 CellID->568497138]
}, Open  ]],

Cell[TextData[{
 "There is not an ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function to compute the collection of unit vectors in the direction of \
positive lift at collocation points.  Compute this collection of unit vectors \
using ",
 Cell[BoxData[
  ButtonBox["MakeCartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/MakeCartesianVectors"]], 
  "InlineFormula"],
 " to construct the data type."
}], "Text",
 CellID->126606522],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"uL", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"Transpose", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"Sin", "[", "\[Alpha]", "]"}]}], ",", 
         RowBox[{"Cos", "[", "\[Alpha]", "]"}]}], "}"}], ",", 
       RowBox[{"{", "nP", "}"}]}], "]"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Head", "[", "uL", "]"}]}], "Input",
 CellLabel->"In[32]:=",
 CellID->440390728],

Cell[BoxData["CartesianVectors"], "Output",
 ImageSize->{116, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[33]=",
 CellID->3144434]
}, Open  ]],

Cell[TextData[{
 "Compute dimensionless lift coefficient and pitching-moment for each panel \
and sum over panels to obtain airfoil lift and moment.  When computing panel \
contributions to moment, the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["Components",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/Components"]], "InlineFormula"],
 " is used to select the list of components, and  the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function Part is used to obtain the list of ",
 Cell[BoxData[
  StyleBox["y", "TI"]], "InlineFormula"],
 "-components of the collection of moment vectors.  Note that the pitching \
moment is about the origin of the coordinate system, which in this example is \
the leading edge of the airfoil."
}], "Text",
 CellID->25066311],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalDelta]cl", "=", 
   RowBox[{
    RowBox[{"-", "cp1"}], " ", 
    RowBox[{"(", 
     RowBox[{"uL", ".", "un"}], ")"}], " ", "\[Delta]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalDelta]cm", "=", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      RowBox[{"Components", "[", 
       RowBox[{
        RowBox[{"ExtendXzToXyz", "[", "rC1", "]"}], "\[Cross]", 
        RowBox[{"ExtendXzToXyz", "[", "un", "]"}]}], "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}]}], " ", "cp1", " ", "\[Delta]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"cl", ",", "cm"}], "}"}], "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Apply", "[", 
     RowBox[{"Plus", ",", "\[CapitalDelta]cl"}], "]"}], ",", 
    RowBox[{"Apply", "[", 
     RowBox[{"Plus", ",", "\[CapitalDelta]cm"}], "]"}]}], "}"}]}]}], "Input",
 CellLabel->"In[34]:=",
 CellID->391129333],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1.009797741425677`", ",", 
   RowBox[{"-", "0.3771136977179274`"}]}], "}"}]], "Output",
 ImageSize->{135, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[36]=",
 CellID->639679893]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
3.6.4 Built-in function to compute lift and moment coefficients\
\>", "Section",
 CellID->190767290],

Cell[TextData[{
 "The ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["ForceAndMoment",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/ForceAndMoment"]], "InlineFormula"],
 " computes the lift coefficient and pitching-moment coefficient from the \
Kutta-Joukowski theorem, ",
 Cell[BoxData[
  RowBox[{"\[ScriptL]", "=", 
   RowBox[{
    StyleBox["\[Rho]", "TR"], " ", 
    StyleBox["U", "TI"], " ", 
    StyleBox["\[CapitalGamma]", "TR"]}]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData["\[ScriptL]"], "InlineFormula"],
 " is the lift per unit span on the airfoil, ",
 Cell[BoxData[
  StyleBox["\[Rho]", "TR"]], "InlineFormula"],
 " is the density of the fluid, ",
 Cell[BoxData[
  StyleBox["U", "TI"]], "InlineFormula"],
 " is the free stream speed and ",
 Cell[BoxData[
  StyleBox["\[CapitalGamma]", "TR"]], "InlineFormula"],
 " is circulation.  This computation is made for each panel, summed over the \
panels and expressed as dimensionless coefficients.  For panel methods using \
doublet panels, the basic relationship between doublet density and \
circulation density is ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{
    RowBox[{"-", 
     StyleBox["d\[Mu]", "TI"]}], 
    RowBox[{
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], "/", 
     StyleBox["dx", "TI"]}]}]}]], "InlineFormula"],
 ".  This relationship is discussed in more detail in ",
 ButtonBox["Section 2.1.2",
  BaseStyle->"Hyperlink",
  ButtonData->{"IntroductionToInfluenceCoefficients.nb", "relation"}],
 ".  Pitching moment with reference to the origin of the airfoil-fixed \
coordinate system is calculated using ",
 Cell[BoxData[
  RowBox[{
   OverscriptBox[
    StyleBox["m", "TI"], "\[RightVector]"], "=", 
   RowBox[{
    StyleBox["r", "TI"], "\[Cross]", 
    OverscriptBox["\[ScriptL]", "\[RightVector]"]}]}]], "InlineFormula"],
 "."
}], "Text",
 CellID->43157970],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "clPM", ",", "cmPM", ",", "\[CapitalDelta]cl", ",", "\[CapitalDelta]cm"}],
     "}"}], "=", 
   RowBox[{"ForceAndMoment", "[", 
    RowBox[{
    "\"\<Doublet\>\"", ",", "a1", ",", "b1", ",", "c1", ",", "rP1", ",", 
     "\[Alpha]", ",", "rC1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"{", 
  RowBox[{"clPM", ",", "cmPM"}], "}"}]}], "Input",
 CellLabel->"In[37]:=",
 CellID->239099752],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0.9990448221237438`", ",", 
   RowBox[{"-", "0.3725662829947557`"}]}], "}"}]], "Output",
 ImageSize->{149, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[38]=",
 CellID->128627423]
}, Open  ]],

Cell["\<\
Compare results between coefficient values computed from the circulation \
distribution and those computed from surface pressure coefficient.\
\>", "Text",
 CellID->35177595],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"clPM", "-", "cl"}], ")"}], "/", "clPM"}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"cmPM", "-", "cm"}], ")"}], "/", "cmPM"}]}], "}"}]], "Input",
 CellLabel->"In[39]:=",
 CellID->449270340],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.010763200072520107`"}], ",", 
   RowBox[{"-", "0.012205652875023386`"}]}], "}"}]], "Output",
 ImageSize->{171, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[39]=",
 CellID->532959204]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["PanelMethods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["3.5 Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]], "RelatedTutorials",
 CellID->101392790],

Cell[TextData[ButtonBox["3.7 Analysis of an Airfoil",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AnalysisOfAnAirfoil"]], "RelatedTutorials",
 CellID->14999323],

Cell[TextData[ButtonBox["Table Of Contents",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/TableOfContents"]], "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"3.6  Aerodynamic Coefficients - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 59, 12.6798000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "Computation of aerodynamic characteristics of airfoils in previous \
sections has played a supporting role in the discussions of validation and \
convergence. This section focuses on how pressure, lift and pitching-,moment \
coefficients are obtained using Aerodynamics functions. Choose \
PMDoubletQuadraticDirichlet as the panel method and the NACA-4412 airfoil for \
the example in this section. The reason for choosing the NACA-4412 airfoil is \
because its geometry is available in Aerodynamics both as a data file and as \
equations for thickness and camber properties. A secondary topic for this \
section is to evaluate the effect on aerodynamic characteristics of \
discretization based a B-spline fit to the data in a file. Three discretized \
airfoils are specified: first, a user-specified discretization based on \
equations for thickness and camber properties, second, a discretization by \
the data in the file, and finally, a discretization based on a B-spline fit \
to the data in the file using the same user-specified discretization \
parameters as the first case. Comparison of surface pressure distribution for \
the three discretizations provides a measure consistency among the three \
methods of discretization. Load the Aerodynamics packages.", "synonyms" -> {},
     "title" -> "3.6  Aerodynamic Coefficients", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/AerodynamicCoefficientsFromPanelMethods"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[610, 21, 78, 1, 70, "TutorialColorBar"],
Cell[691, 24, 209, 6, 70, "LinkTrail"],
Cell[903, 32, 3032, 69, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3960, 105, 66, 1, 70, "Title",
 CellID->509267359],
Cell[4029, 108, 645, 16, 70, "Text",
 CellID->131238241],
Cell[4677, 126, 930, 16, 70, "Text",
 CellID->92798100],
Cell[5610, 144, 125, 6, 70, "Text",
 CellID->836781195],
Cell[5738, 152, 101, 3, 70, "Input",
 CellID->2058623809],
Cell[CellGroupData[{
Cell[5864, 159, 62, 1, 70, "Section",
 CellID->259819136],
Cell[5929, 162, 252, 7, 70, "Text",
 CellID->267612329],
Cell[6184, 171, 456, 13, 70, "Input",
 CellID->17065800],
Cell[6643, 186, 316, 9, 70, "Text",
 CellID->96382008],
Cell[CellGroupData[{
Cell[6984, 199, 587, 18, 70, "Input",
 CellID->238618617],
Cell[7574, 219, 482, 15, 36, "Output",
 CellID->558659956]
}, Open  ]],
Cell[8071, 237, 239, 5, 70, "Text",
 CellID->231578421],
Cell[8313, 244, 569, 20, 70, "Input",
 CellID->133089996],
Cell[8885, 266, 729, 18, 70, "Text",
 CellID->583936543],
Cell[CellGroupData[{
Cell[9639, 288, 192, 5, 70, "Input",
 CellID->108778828],
Cell[9834, 295, 402, 14, 36, "Output",
 CellID->570743163]
}, Open  ]],
Cell[10251, 312, 243, 5, 70, "Text",
 CellID->113191753],
Cell[CellGroupData[{
Cell[10519, 321, 665, 22, 70, "Input",
 CellID->92323877],
Cell[11187, 345, 160, 5, 36, "Output",
 CellID->198740885]
}, Open  ]],
Cell[11362, 353, 230, 5, 70, "Text",
 CellID->11042490],
Cell[CellGroupData[{
Cell[11617, 362, 657, 20, 70, "Input",
 CellID->601516212],
Cell[12277, 384, 461, 14, 36, "Output",
 CellID->2523348]
}, Open  ]],
Cell[12753, 401, 144, 4, 70, "Text",
 CellID->2918513],
Cell[12900, 407, 570, 20, 70, "Input",
 CellID->805149539]
}, Open  ]],
Cell[CellGroupData[{
Cell[13507, 432, 71, 1, 70, "Section",
 CellID->6814950],
Cell[13581, 435, 882, 16, 70, "Text",
 CellID->115793861],
Cell[14466, 453, 139, 4, 70, "Text",
 CellID->42676348],
Cell[14608, 459, 848, 25, 70, "Input",
 CellID->821293751],
Cell[15459, 486, 827, 22, 70, "Text",
 CellID->250599266],
Cell[16289, 510, 1022, 35, 70, "Input",
 CellID->47000734],
Cell[17314, 547, 811, 16, 70, "Text",
 CellID->139613108],
Cell[CellGroupData[{
Cell[18150, 567, 964, 27, 70, "Input",
 CellID->314038555],
Cell[19117, 596, 13473, 261, 386, "Output",
 CellID->69621107]
}, Open  ]],
Cell[32605, 860, 960, 21, 70, "Text",
 CellID->20411779]
}, Open  ]],
Cell[CellGroupData[{
Cell[33602, 886, 95, 1, 70, "Section",
 CellID->252223901],
Cell[33700, 889, 1957, 52, 70, "Text",
 CellID->126296105],
Cell[35660, 943, 178, 6, 70, "Text",
 CellID->195800744],
Cell[35841, 951, 382, 11, 70, "Input",
 CellID->41528148],
Cell[36226, 964, 171, 6, 70, "Text",
 CellID->489123521],
Cell[CellGroupData[{
Cell[36422, 974, 102, 3, 70, "Input",
 CellID->992671459],
Cell[36527, 979, 175, 5, 36, "Output",
 CellID->568497138]
}, Open  ]],
Cell[36717, 987, 474, 14, 70, "Text",
 CellID->126606522],
Cell[CellGroupData[{
Cell[37216, 1005, 524, 16, 70, "Input",
 CellID->440390728],
Cell[37743, 1023, 173, 5, 36, "Output",
 CellID->3144434]
}, Open  ]],
Cell[37931, 1031, 840, 21, 70, "Text",
 CellID->25066311],
Cell[CellGroupData[{
Cell[38796, 1056, 931, 29, 70, "Input",
 CellID->391129333],
Cell[39730, 1087, 259, 8, 36, "Output",
 CellID->639679893]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[40038, 1101, 110, 3, 70, "Section",
 CellID->190767290],
Cell[40151, 1106, 2012, 60, 70, "Text",
 CellID->43157970],
Cell[CellGroupData[{
Cell[42188, 1170, 465, 14, 70, "Input",
 CellID->239099752],
Cell[42656, 1186, 260, 8, 36, "Output",
 CellID->128627423]
}, Open  ]],
Cell[42931, 1197, 184, 4, 70, "Text",
 CellID->35177595],
Cell[CellGroupData[{
Cell[43140, 1205, 280, 10, 70, "Input",
 CellID->449270340],
Cell[43423, 1217, 283, 9, 36, "Output",
 CellID->532959204]
}, Open  ]],
Cell[CellGroupData[{
Cell[43743, 1231, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[43811, 1234, 158, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[44006, 1242, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[44081, 1245, 184, 4, 70, "RelatedTutorials",
 CellID->101392790],
Cell[44268, 1251, 182, 4, 70, "RelatedTutorials",
 CellID->14999323],
Cell[44453, 1257, 170, 4, 70, "RelatedTutorials",
 CellID->280152894]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[44662, 1266, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

