(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     38730,        921]
NotebookOptionsPosition[     34373,        794]
NotebookOutlinePosition[     36558,        843]
CellTagsIndexPosition[     36515,        840]
WindowTitle->1.4 Airfoils From Conformal Mapping - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"1.3 Airfoils From Thickness And Camber Properties\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/\
AirfoilsFromThicknessAndCamberProperties"], "\<\"1.5 Panel Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PanelGeometry"], "\<\"Table Of \
Contents\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/AirfoilGeometry"], "\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/AirfoilsFromConformalMapping\"\>\
":>None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/AirfoilsFromConformalMapping"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/AirfoilsFromConformalMapping.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/AirfoilsFromConformalMapping", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["1.4 Airfoils From Conformal Mapping", "Title",
 CellID->509267359],

Cell[TextData[{
 "The example used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " for airfoils from conformal mapping is the van de Vooren family of \
symmetric airfoils.  The van de Vooren airfoils provide a collection of \
airfoils with finite trailing-edge angles for which exact solutions for \
potential flow are known.  See Section 6.6 of ",
 ButtonBox["Katz and Plotkin (2001)",
  BaseStyle->"Hyperlink",
  ButtonData->{"References.nb", "katz2001"}],
 " for details.  Such airfoils are useful for validating numerical models."
}], "Text",
 CellID->1534169418],

Cell[TextData[{
 "The function ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 " computes a collection of vectors on the surface of the airfoil from two \
shape parameters and a list of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 "-values locating points on the circle that maps to the airfoil under the \
van de Vooren transformation.  The range of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 " is from ",
 Cell[BoxData["0"], "InlineFormula"],
 " to ",
 Cell[BoxData[
  StyleBox["\[Pi]", "TR"]], "InlineFormula"],
 ", and points on the lower surface of the airfoil are obtained from the \
mirror symmetry of the symmetric airfoil."
}], "Text",
 CellID->455105011],

Cell[TextData[{
 "Load ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->23403085],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[0]:=",
 CellID->71496216],

Cell[CellGroupData[{

Cell["1.4.1 van de Vooren airfoil of 12% thickness", "Section",
 CellID->320326959],

Cell[TextData[{
 "A van de Vooren airfoil of 12% maximum thickness is used to validate \
results of numerical models in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  User-specified discretizations need to be based on a list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values to be consistent with the methods used for airfoils from data files \
and airfoils specified by thickness and camber properties.  The function ",
 Cell[BoxData[
  ButtonBox["XToTheta",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/XToTheta"]], "InlineFormula"],
 " provides the tool for  converting ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values to ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 "-values."
}], "Text",
 CellID->26627051],

Cell[TextData[{
 "Specify shape parameters for a 12% maximum thickness airfoil and a \
relatively finely spaced list of ",
 Cell[BoxData[
  StyleBox["\[Theta]", "TR"]], "InlineFormula"],
 " values.  Compute a collection vectors locating points on the surface of \
the airfoil.  This will serve to specify the shape of the airfoil."
}], "Text",
 CellID->310923134],

Cell[BoxData[{
 RowBox[{
  RowBox[{"t", "=", "0.12"}], ";", 
  RowBox[{"\[Tau]", "=", 
   RowBox[{"9.81", " ", "Degree"}]}], ";", 
  RowBox[{"\[Theta]", "=", 
   RowBox[{"Range", "[", 
    RowBox[{"0", ",", "\[Pi]", ",", 
     RowBox[{"\[Pi]", "/", "100"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"r", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"t", ",", "\[Tau]", ",", "\[Theta]"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->18936054],

Cell[TextData[{
 "Compute the collection of vectors for a user-specified discretization of \
the airfoil.  Note that this discretization is specified in terms of a list \
of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values."
}], "Text",
 CellID->985219100],

Cell[BoxData[
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"t", ",", "\[Tau]", ",", 
     RowBox[{"XToTheta", "[", 
      RowBox[{"t", ",", "\[Tau]", ",", 
       RowBox[{"NDiscretizeUnitSegment", "[", 
        RowBox[{"4", ",", 
         RowBox[{"Layout", "\[Rule]", "CosineSpacing"}]}], "]"}]}], "]"}]}], 
    "]"}]}], ";"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->130944639],

Cell["\<\
Generate and show superimposed plots of the airfoil and its user-specified \
coarse discretization, shown in red.\
\>", "Text",
 CellID->367777498],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gr", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"r", ",", 
     RowBox[{"PlotRange", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.15"}], ",", "0.15"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"grP", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gr", ",", "grP"}], "]"}]}], "Input",
 CellLabel->"In[4]:=",
 CellID->8862207],

Cell[BoxData[
 GraphicsBox[{
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdl3s0VWkfx/c0TRdS1pi8SomhmkylUmmUvqeLdKFCYnTTTaKUyZRJIwxd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     "]]}, 
   {RGBColor[1, 0, 0], 
    LineBox[{{1., 0.}, {0.8535533879177306, -0.015469287303172485`}, {
     0.500000000029821, -0.050919001767322294`}, {
     0.1464466265038108, -0.05376892827817453}, {0., 
     1.1102230246251565`*^-16}, {0.1464466265038108, 0.05376892827817453}, {
     0.500000000029821, 0.050919001767322294`}, {0.8535533879177306, 
     0.015469287303172485`}, {1., 0.}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2000000000000004`, 
      FormBox["1.2000000000000004`", TraditionalForm]}, {1.4000000000000001`, 
      FormBox["1.4000000000000001`", TraditionalForm]}, {1.6, 
      FormBox["1.6`", TraditionalForm]}}, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}, None, {{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.9, 
      FormBox[
       RowBox[{"-", "0.9`"}], TraditionalForm]}, {-0.8, 
      FormBox[
       RowBox[{"-", "0.8`"}], TraditionalForm]}, {-0.7, 
      FormBox[
       RowBox[{"-", "0.7`"}], TraditionalForm]}, {-0.6, 
      FormBox[
       RowBox[{"-", "0.6`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.29999999999999993`, 
      FormBox[
       RowBox[{"-", "0.29999999999999993`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], 
       TraditionalForm]}, {-0.09999999999999998, 
      FormBox[
       RowBox[{"-", "0.09999999999999998`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.10000000000000009`, 
      FormBox["0.10000000000000009`", TraditionalForm]}, {
     0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.30000000000000004`, 
      FormBox["0.30000000000000004`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.7000000000000002, 
      FormBox["0.7000000000000002`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {0.9000000000000001, 
      FormBox["0.9000000000000001`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}}},
  ImageSize->{504., 180.},
  PlotRange->{-0.15, 0.15},
  PlotRangeClipping->True,
  RotateLabel->False]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->124495677]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.4.2 van de Vooren family of airfoils", "Section",
 CellID->36431957],

Cell["\<\
The effect of the two parameters on the shape of the airfoil can be seen \
using the following dynamic example.  Use the sliders to change the parameter \
values and the shape of the airfoil.  Note that the maximum thickness of the \
airfoil is included as part of the plot label.\
\>", "Text",
 CellID->302187830],

Cell[TextData[{
 "Note:  If graphic does not show properly, quit the kernel and evaluate as \
you would for any ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " notebook."
}], "Text",
 CellID->583385869],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"r", "=", 
     RowBox[{"AirfoilSurfacePoints", "[", 
      RowBox[{"t", ",", "\[Tau]", ",", 
       RowBox[{"Range", "[", 
        RowBox[{"0", ",", "\[Pi]", ",", 
         RowBox[{"\[Pi]", "/", "200"}]}], "]"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"thickness", "=", 
     RowBox[{"200", 
      RowBox[{"Max", "[", 
       RowBox[{
        RowBox[{"Components", "[", "r", "]"}], "[", 
        RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"PlotAirfoil", "[", 
     RowBox[{"r", ",", 
      RowBox[{"ImageSize", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"7", " ", "72"}], "}"}]}], ",", 
      RowBox[{"PlotLabel", "\[Rule]", 
       RowBox[{"StringForm", "[", 
        RowBox[{"\"\<`` % Thick van de Vooren Airfoil\>\"", ",", 
         RowBox[{"NumberForm", "[", 
          RowBox[{"thickness", ",", 
           RowBox[{"{", 
            RowBox[{"4", ",", "2"}], "}"}]}], "]"}]}], "]"}]}], ",", 
      RowBox[{"PlotRange", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "0.4"}], ",", "0.4"}], "}"}]}]}], "]"}]}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"t", ",", "0.12", ",", "\"\<t\>\""}], "}"}], ",", "0", ",", 
     "0.25"}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Tau]", ",", "0.171217", ",", "\"\<\[Tau]\>\""}], "}"}], ",", 
     "0", ",", "1.5708"}], "}"}], ",", 
   RowBox[{"SaveDefinitions", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->186362284],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`t$$ = 0.12, $CellContext`\[Tau]$$ = 
    0.17121679962064373`, Typeset`show$$ = True, Typeset`bookmarkList$$ = {}, 
    Typeset`bookmarkMode$$ = "Menu", Typeset`animator$$, Typeset`animvar$$ = 
    1, Typeset`name$$ = "\"untitled\"", Typeset`specs$$ = {{{
       Hold[$CellContext`t$$], 0.12, "t"}, 0, 0.25}, {{
       Hold[$CellContext`\[Tau]$$], 0.171217, "\[Tau]"}, 0, 1.5708}}, 
    Typeset`size$$ = {504., {194., 198.}}, Typeset`update$$ = 0, 
    Typeset`initDone$$, Typeset`skipInitDone$$ = False, $CellContext`t$469$$ =
     0, $CellContext`\[Tau]$474$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, 
      "Variables" :> {$CellContext`t$$ = 0.12, $CellContext`\[Tau]$$ = 
        0.171217}, "ControllerVariables" :> {
        Hold[$CellContext`t$$, $CellContext`t$469$$, 0], 
        Hold[$CellContext`\[Tau]$$, $CellContext`\[Tau]$474$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, 
      "Body" :> ($CellContext`r = 
        Aerodynamics`AirfoilGeometry`AirfoilSurfacePoints[$CellContext`t$$, \
$CellContext`\[Tau]$$, 
          Range[0, Pi, Pi/200]]; $CellContext`thickness = 200 Max[
           Part[
            Aerodynamics`CartesianVectors`Components[$CellContext`r], 2]]; 
       Aerodynamics`Graphics`PlotAirfoil[$CellContext`r, ImageSize -> {7 72}, 
         PlotLabel -> StringForm["`` % Thick van de Vooren Airfoil", 
           NumberForm[$CellContext`thickness, {4, 2}]], 
         PlotRange -> {-0.4, 0.4}]), 
      "Specifications" :> {{{$CellContext`t$$, 0.12, "t"}, 0, 
         0.25}, {{$CellContext`\[Tau]$$, 0.171217, "\[Tau]"}, 0, 1.5708}}, 
      "Options" :> {}, "DefaultOptions" :> {}],
     ImageSizeCache->{553., {254., 259.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    Initialization:>({$CellContext`r = 
       Aerodynamics`CartesianVectors`CartesianVectors[{
         1, 0.9996546935660066, 0.9986705449365171, 0.9970760018163566, 
          0.9948871125859398, 0.9921163268177845, 0.9887745683047701, 
          0.9848720875875345, 0.9804188923526854, 0.9754249904979132, 
          0.9699005366923474, 0.9638559238698904, 0.9573018408350747, 
          0.9502493077614609, 0.9427096965840446, 0.9346947406792314, 
          0.9262165367191765, 0.9172875406747675, 0.9079205593663864, 
          0.8981287385863137, 0.8879255485640948, 0.8773247673713698, 
          0.86634046273841, 0.8549869726639975, 0.843278885132562, 
          0.831231017200605, 0.8188583936737255, 0.8061762255628429, 
          0.7931998884813267, 0.7799449011221671, 0.7664269039350171, 
          0.7526616381061386, 0.7386649249294652, 0.7244526456437819, 
          0.7100407217991227, 0.6954450962047174, 0.6806817145010252, 
          0.6657665073894812, 0.6507153735454498, 0.635544163232493, 
          0.6202686626293582, 0.6049045788750279, 0.5894675258317305, 
          0.573973010560962, 0.5584364205032498, 0.5428730113486349, 
          0.5272978955815675, 0.5117260316811301, 0.4961722139551553, 
          0.48065106298488314`, 0.4651770166552899, 0.44976432174505, 
          0.43442702604927863`, 0.41917897100768653`, 0.40403378481055696`, 
          0.3890048759549646, 0.3741054272239256, 0.35934839006159314`, 
          0.34474647931826785`, 0.3303121683397444, 0.31605768437643744`, 
          0.3019950042887296, 0.2881358505260865, 0.2744916873586587, 
          0.2610737173412788, 0.2478928779910382, 0.23495983866087655`, 
          0.22228499759289932`, 0.209878479136403, 0.1977501311168316, 
          0.18590952234313263`, 0.17436594024215046`, 0.16312838860986267`, 
          0.15220558547035878`, 0.14160596103452572`, 0.13133765575140144`, 
          0.1214085184460959, 0.11182610453906405`, 0.10259767434234757`, 
          0.09373019142913419, 0.08523032107371731, 0.0771044287595446, 
          0.06935857875362816, 0.061998532746109, 0.05502974855421061, 
          0.04845737889022428, 0.04228627019351605, 0.03652096152680906, 
          0.031165683537286215`, 0.02622435748318752, 0.02170059432678606, 
          0.017597693894683908`, 0.013918644106472478`, 0.010666120272810109`,
           0.00784248446398017, 0.005449784949954073, 0.0034897557129424106`, 
          0.0019638160332975385`, 0.000873070149588373, 
          0.00021830699348401872`, 0., 0.00021830699348401872`, 
          0.000873070149588373, 0.0019638160332975385`, 
          0.0034897557129424106`, 0.005449784949954073, 0.00784248446398017, 
          0.010666120272810109`, 0.013918644106472478`, 0.017597693894683908`,
           0.02170059432678606, 0.02622435748318752, 0.031165683537286215`, 
          0.03652096152680906, 0.04228627019351605, 0.04845737889022428, 
          0.05502974855421061, 0.061998532746109, 0.06935857875362816, 
          0.0771044287595446, 0.08523032107371731, 0.09373019142913419, 
          0.10259767434234757`, 0.11182610453906405`, 0.1214085184460959, 
          0.13133765575140144`, 0.14160596103452572`, 0.15220558547035878`, 
          0.16312838860986267`, 0.17436594024215046`, 0.18590952234313263`, 
          0.1977501311168316, 0.209878479136403, 0.22228499759289932`, 
          0.23495983866087655`, 0.2478928779910382, 0.2610737173412788, 
          0.2744916873586587, 0.2881358505260865, 0.3019950042887296, 
          0.31605768437643744`, 0.3303121683397444, 0.34474647931826785`, 
          0.35934839006159314`, 0.3741054272239256, 0.3890048759549646, 
          0.40403378481055696`, 0.41917897100768653`, 0.43442702604927863`, 
          0.44976432174505, 0.4651770166552899, 0.48065106298488314`, 
          0.4961722139551553, 0.5117260316811301, 0.5272978955815675, 
          0.5428730113486349, 0.5584364205032498, 0.573973010560962, 
          0.5894675258317305, 0.6049045788750279, 0.6202686626293582, 
          0.635544163232493, 0.6507153735454498, 0.6657665073894812, 
          0.6806817145010252, 0.6954450962047174, 0.7100407217991227, 
          0.7244526456437819, 0.7386649249294652, 0.7526616381061386, 
          0.7664269039350171, 0.7799449011221671, 0.7931998884813267, 
          0.8061762255628429, 0.8188583936737255, 0.831231017200605, 
          0.843278885132562, 0.8549869726639975, 0.86634046273841, 
          0.8773247673713698, 0.8879255485640948, 0.8981287385863137, 
          0.9079205593663864, 0.9172875406747675, 0.9262165367191765, 
          0.9346947406792314, 0.9427096965840446, 0.9502493077614609, 
          0.9573018408350747, 0.9638559238698904, 0.9699005366923474, 
          0.9754249904979132, 0.9804188923526854, 0.9848720875875345, 
          0.9887745683047701, 0.9921163268177845, 0.9948871125859398, 
          0.9970760018163566, 0.9986705449365171, 0.9996546935660066, 1}, {
         0, -0.000029995190636323425`, -0.00011687323292118567`, \
-0.00026009404253466226`, -0.00046008728097634904`, -0.0007175032038973551, \
-0.001033028403759335, -0.0014073028223394674`, -0.0018408718312102955`, \
-0.0023341541748097324`, -0.002887418467516138, -0.0035007649802909387`, \
-0.00417411109377358, -0.0049071795458857995`, -0.0056994889765039115`, \
-0.00655034647027106, -0.007458841908328746, -0.008423844001819284, \
-0.009443997915252543, -0.010517724407377624`, -0.011643220427300913`, \
-0.012818461107917185`, -0.014041203099505191`, -0.015308989184976238`, \
-0.016619154115686013`, -0.017968831603507374`, -0.019354962401444686`, \
-0.02077430340171746, -0.02222343767714466, -0.02369878538896847, \
-0.025196615482023108`, -0.026713058086472258`, -0.028244117544201128`, \
-0.029785685977395282`, -0.03133355731683066, -0.032883441707964436`, \
-0.0344309802139752, -0.03597175973646333, -0.037501328076534876`, \
-0.03901520906139551, -0.040508917664365235`, -0.04197797504931279, \
-0.043417923473860906`, -0.04482434098930453, -0.04619285587893725, \
-0.04751916078036811, -0.04879902644140571, -0.05002831506311321, \
-0.05120299318769441, -0.052319144092914785`, -0.05337297965873372, \
-0.05436085167577157, -0.05527926256902643, -0.05612487551398132, \
-0.05689452392579435, -0.057585220305705724`, -0.05819416443203215, \
-0.05871875088623772, -0.05915657590744568, -0.05950544357154912, \
-0.059763371293568276`, -0.05992859465432676, -0.059999571554679526`, \
-0.059974985702556965`, -0.05985374943991381, -0.05963500591836979, \
-0.059318130633811944`, -0.05890273233161358, -0.058388653295309345`, \
-0.05777596903265758, -0.05706498737395155, -0.05625624699824894, \
-0.05535051540391506, -0.054348786340452426`, -0.05325227671908811, \
-0.05206242302002584, -0.050780877214567754`, -0.049409502220587265`, \
-0.04795036691002563, -0.046405740687199715`, -0.04477808765680902, \
-0.043070060400569876`, -0.041284493381351484`, -0.03942439599372749, \
-0.03749294527973285, -0.035493478328536165`, -0.03342948437865373, \
-0.03130459664118579, -0.029122583862432827`, -0.0268873416441045, \
-0.024602883539186216`, -0.022273331941410734`, -0.01990290878609735, \
-0.01749592608001216, -0.01505677627776425, -0.01258992252212826, \
-0.010099888765554843`, -0.007591249790065924, -0.005068621142583468, \
-0.0025366490027270983`, 1.1102230246251565`*^-16, 0.0025366490027270983`, 
          0.005068621142583468, 0.007591249790065924, 0.010099888765554843`, 
          0.01258992252212826, 0.01505677627776425, 0.01749592608001216, 
          0.01990290878609735, 0.022273331941410734`, 0.024602883539186216`, 
          0.0268873416441045, 0.029122583862432827`, 0.03130459664118579, 
          0.03342948437865373, 0.035493478328536165`, 0.03749294527973285, 
          0.03942439599372749, 0.041284493381351484`, 0.043070060400569876`, 
          0.04477808765680902, 0.046405740687199715`, 0.04795036691002563, 
          0.049409502220587265`, 0.050780877214567754`, 0.05206242302002584, 
          0.05325227671908811, 0.054348786340452426`, 0.05535051540391506, 
          0.05625624699824894, 0.05706498737395155, 0.05777596903265758, 
          0.058388653295309345`, 0.05890273233161358, 0.059318130633811944`, 
          0.05963500591836979, 0.05985374943991381, 0.059974985702556965`, 
          0.059999571554679526`, 0.05992859465432676, 0.059763371293568276`, 
          0.05950544357154912, 0.05915657590744568, 0.05871875088623772, 
          0.05819416443203215, 0.057585220305705724`, 0.05689452392579435, 
          0.05612487551398132, 0.05527926256902643, 0.05436085167577157, 
          0.05337297965873372, 0.052319144092914785`, 0.05120299318769441, 
          0.05002831506311321, 0.04879902644140571, 0.04751916078036811, 
          0.04619285587893725, 0.04482434098930453, 0.043417923473860906`, 
          0.04197797504931279, 0.040508917664365235`, 0.03901520906139551, 
          0.037501328076534876`, 0.03597175973646333, 0.0344309802139752, 
          0.032883441707964436`, 0.03133355731683066, 0.029785685977395282`, 
          0.028244117544201128`, 0.026713058086472258`, 0.025196615482023108`,
           0.02369878538896847, 0.02222343767714466, 0.02077430340171746, 
          0.019354962401444686`, 0.017968831603507374`, 0.016619154115686013`,
           0.015308989184976238`, 0.014041203099505191`, 
          0.012818461107917185`, 0.011643220427300913`, 0.010517724407377624`,
           0.009443997915252543, 0.008423844001819284, 0.007458841908328746, 
          0.00655034647027106, 0.0056994889765039115`, 0.0049071795458857995`,
           0.00417411109377358, 0.0035007649802909387`, 0.002887418467516138, 
          0.0023341541748097324`, 0.0018408718312102955`, 
          0.0014073028223394674`, 0.001033028403759335, 0.0007175032038973551,
           0.00046008728097634904`, 0.00026009404253466226`, 
          0.00011687323292118567`, 0.000029995190636323425`, 
          0}], $CellContext`t$$ = 0.12, $CellContext`\[Tau]$$ = 
       0.17121679962064373`}; Typeset`initDone$$ = True),
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 ImageSize->{557, 515},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->27600532]
}, Open  ]],

Cell[TextData[{
 "Note that this interactive example can be used to obtain the shape \
parameters for an airfoil of desired maximum thickness.  Suppose that shape \
parameters for a 15% thick van de Vooren airfoil are needed.  Click on the \
\"+\" box to the right of the slider for the ",
 Cell[BoxData[
  StyleBox["t", "TI"]], "InlineFormula"],
 " parameter to display additional controls including a box listing the value \
of ",
 Cell[BoxData[
  StyleBox["t", "TI"]], "InlineFormula"],
 ".  Change the value in the box to 0.15 and enter.  Note from the plot label \
that this produces an airfoil of 13.88% thickness.  Use the ",
 Cell[BoxData[
  StyleBox["\[Tau]", "TR"]], "InlineFormula"],
 "-slider to obtain a thickness value close to 15%.  Click the \"+\" box for \
the ",
 Cell[BoxData[
  StyleBox["\[Tau]", "TR"]], "InlineFormula"],
 " slider and insert a value for the trailing-edge angle (in radians).  Trial \
and error quickly yields a value of 0.2162 for a 15% thick airfoil.  "
}], "Text",
 CellID->860335157],

Cell["\<\
Convert the trailing-edge angle of 0.216  radians to degrees.\
\>", "Text",
 CellID->311424989],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{".2162", "/", "Degree"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->58885892],

Cell[BoxData["12.387347530728398`"], "Output",
 ImageSize->{53, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->53166743]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/AirfoilGeometry"]], "TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->529874125]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["1.3 Airfoils From Thickness And Camber Properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"]], \
"RelatedTutorials",
 CellID->806871991],

Cell[TextData[ButtonBox["1.5 Panel Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PanelGeometry"]], "RelatedTutorials",
 CellID->242123264],

Cell[TextData[ButtonBox["Table Of Contents",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/TableOfContents"]], "RelatedTutorials",
 CellID->860107027]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"1.4 Airfoils From Conformal Mapping - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 59, 15.4254000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "The example used in Aerodynamics for airfoils from conformal mapping is \
the van de Vooren family of symmetric airfoils. The van de Vooren airfoils \
provide a collection of airfoils with finite trailing-edge angles for which \
exact solutions for potential flow are known. See Section 6.6 of Katz and \
Plotkin (2001) for details. Such airfoils are useful for validating numerical \
models. The function AirfoilSurfacePoints computes a collection of vectors on \
the surface of the airfoil from two shape parameters and a list of \
\\[Theta]-values locating points on the circle that maps to the airfoil under \
the van de Vooren transformation. The range of \\[Theta] is from 0 to \\[Pi], \
and points on the lower surface of the airfoil are obtained from the mirror \
symmetry of the symmetric airfoil. Load Aerodynamics packages.", 
    "synonyms" -> {}, "title" -> "1.4 Airfoils From Conformal Mapping", 
    "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/AirfoilsFromConformalMapping"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[616, 21, 78, 1, 70, "TutorialColorBar"],
Cell[697, 24, 191, 5, 70, "LinkTrail"],
Cell[891, 31, 3136, 71, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4052, 106, 72, 1, 70, "Title",
 CellID->509267359],
Cell[4127, 109, 578, 13, 70, "Text",
 CellID->1534169418],
Cell[4708, 124, 827, 23, 70, "Text",
 CellID->455105011],
Cell[5538, 149, 120, 6, 70, "Text",
 CellID->23403085],
Cell[5661, 157, 99, 3, 70, "Input",
 CellID->71496216],
Cell[CellGroupData[{
Cell[5785, 164, 83, 1, 70, "Section",
 CellID->320326959],
Cell[5871, 167, 806, 22, 70, "Text",
 CellID->26627051],
Cell[6680, 191, 363, 8, 70, "Text",
 CellID->310923134],
Cell[7046, 201, 491, 14, 70, "Input",
 CellID->18936054],
Cell[7540, 217, 277, 8, 70, "Text",
 CellID->985219100],
Cell[7820, 227, 420, 12, 70, "Input",
 CellID->130944639],
Cell[8243, 241, 157, 4, 70, "Text",
 CellID->367777498],
Cell[CellGroupData[{
Cell[8425, 249, 567, 19, 70, "Input",
 CellID->8862207],
Cell[8995, 270, 7897, 166, 201, "Output",
 CellID->124495677]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[16941, 442, 76, 1, 70, "Section",
 CellID->36431957],
Cell[17020, 445, 324, 6, 70, "Text",
 CellID->302187830],
Cell[17347, 453, 209, 7, 70, "Text",
 CellID->583385869],
Cell[CellGroupData[{
Cell[17581, 464, 1693, 46, 70, "Input",
 CellID->186362284],
Cell[19277, 512, 12461, 189, 70, "Output",
 CellID->27600532]
}, Open  ]],
Cell[31753, 704, 1025, 22, 70, "Text",
 CellID->860335157],
Cell[32781, 728, 105, 3, 70, "Text",
 CellID->311424989],
Cell[CellGroupData[{
Cell[32911, 735, 100, 3, 70, "Input",
 CellID->58885892],
Cell[33014, 740, 175, 5, 36, "Output",
 CellID->53166743]
}, Open  ]],
Cell[CellGroupData[{
Cell[33226, 750, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[33294, 753, 167, 4, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[33464, 759, 157, 3, 70, "TutorialMoreAbout",
 CellID->529874125]
}, Open  ]],
Cell[CellGroupData[{
Cell[33658, 767, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[33733, 770, 229, 5, 70, "RelatedTutorials",
 CellID->806871991],
Cell[33965, 777, 169, 4, 70, "RelatedTutorials",
 CellID->242123264],
Cell[34137, 783, 170, 4, 70, "RelatedTutorials",
 CellID->860107027]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[34346, 792, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

