(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     66535,       1712]
NotebookOptionsPosition[     59033,       1474]
NotebookOutlinePosition[     61613,       1528]
CellTagsIndexPosition[     61570,       1525]
WindowTitle->1.2 Airfoils From Data Files - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.1 Introduction To Airfoil Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"], \
"\<\"1.3 Airfoils From Thickness And Camber Properties\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/\
AirfoilsFromThicknessAndCamberProperties"], "\<\"Table Of Contents\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"AirfoilGeometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/AirfoilGeometry"], "\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/AirfoilsFromDataFiles\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/AirfoilsFromDataFiles"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/AirfoilsFromDataFiles.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/AirfoilsFromDataFiles", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["1.2 Airfoils From Data Files", "Title",
 CellID->509267359],

Cell[TextData[{
 "Michael Selig of the Aerospace Engineering Department at the University of \
Illinois at Urbana-Chanpaign maintains the ",
 ButtonBox["UIUC Airfoil Data Site",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.ae.uiuc.edu/m-selig/ads.html"], None}],
 " that provides links to useful information about airfoils and to data files \
for a collection of over 1500 airfoils.  This site serves as the primary \
source for specifying airfoil geometry in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellID->1534169418],

Cell[CellGroupData[{

Cell["1.2.1 Structure of data files", "Section",
 CellID->339343814],

Cell["\<\
Several data files from the UIUC Airfoil Coordinates Database have been \
downloaded and are located in the folder Airfoil Data for use as examples.  \
In the listing below, repetitive parts have been stripped away from path \
names.\
\>", "Text",
 CellID->100647037],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"StringTake", "[", 
     RowBox[{"#", ",", 
      RowBox[{
       RowBox[{
        RowBox[{"StringPosition", "[", 
         RowBox[{"#", ",", "\"\<Airfoil Data\\\\\>\""}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "2"}], "]"}], "]"}], "-", 
       RowBox[{"StringLength", "[", "#", "]"}]}]}], "]"}], "&"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"FileNames", "[", 
    RowBox[{"\"\<Aerodynamics\\\\Airfoil Data\\\\*\>\"", ",", "$Path"}], 
    "]"}]}], "]"}]], "Input",
 CellLabel->"In[0]:=",
 CellID->161538214],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"clarky.dat\"\>", ",", "\<\"e205.dat\"\>", 
   ",", "\<\"e374.dat\"\>", ",", "\<\"l7769.dat\"\>", 
   ",", "\<\"naca23012.dat\"\>", ",", "\<\"naca4412.dat\"\>", 
   ",", "\<\"naca4412m.dat\"\>", ",", "\<\"naca642415.dat\"\>", 
   ",", "\<\"naca747a315.dat\"\>", ",", "\<\"s1223.dat\"\>", 
   ",", "\<\"sd7003.dat\"\>", ",", "\<\"usa35b.dat\"\>", 
   ",", "\<\"clarky.dat\"\>", ",", "\<\"e205.dat\"\>", 
   ",", "\<\"e374.dat\"\>", ",", "\<\"l7769.dat\"\>", 
   ",", "\<\"naca23012.dat\"\>", ",", "\<\"naca4412.dat\"\>", 
   ",", "\<\"naca4412m.dat\"\>", ",", "\<\"naca642415.dat\"\>", 
   ",", "\<\"naca747a315.dat\"\>", ",", "\<\"s1223.dat\"\>", 
   ",", "\<\"sd7003.dat\"\>", ",", "\<\"usa35b.dat\"\>"}], "}"}]], "Output",
 ImageSize->{572, 67},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[0]=",
 CellID->69847345]
}, Open  ]],

Cell[TextData[{
 "There are three numbering conventions used to order the points specifying \
an airfoil: clockwise, counterclockwise and fore-to-aft.  Clockwise and \
counterclockwise conventions start and end at the trailing edge, traversing \
the airfoil in the specified direction.  The fore-to-aft convention traverses \
the upper surface of the airfoil first and then the lower surface in the \
specified direction.  ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " uses the clockwise convention because it is consistent with the convention \
used in American textbooks for clockwise circulation producing positive lift. \
 Data files in the UIUC Airfoil Database use both counterclockwise and \
fore-to-aft conventions."
}], "Text",
 CellID->448529733],

Cell[CellGroupData[{

Cell["Counterclockwise numbering convention", "Subsection",
 CellID->397338901],

Cell["\<\
Look at the data in the file for the Eppler-205 airfoil as an example of the \
counterclockwise numbering convention.\
\>", "Text",
 CellID->16776698],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{
   "Import", "[", "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}], 
   ",", "5"}], "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->464615637],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"E205\"\>", ",", "\<\"(10.48%)\"\>"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1.`", ",", "0.`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.99655`", ",", "0.00039`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.98649`", ",", "0.00174`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.97049`", ",", "0.00427`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.94916`", ",", "0.00778`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.92285`", ",", "0.01196`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.89175`", ",", "0.01668`"}], "}"}], ",", 
    RowBox[{"\[LeftSkeleton]", "47", "\[RightSkeleton]"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.89034`", ",", 
      RowBox[{"-", "0.00497`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.92195`", ",", 
      RowBox[{"-", "0.0038`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.9486`", ",", 
      RowBox[{"-", "0.00252`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.97017`", ",", 
      RowBox[{"-", "0.00125`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.98635`", ",", 
      RowBox[{"-", "0.00036`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.99651`", ",", 
      RowBox[{"-", "0.00003`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1.`", ",", "0.`"}], "}"}]}], "}"}],
  Short[#, 5]& ]], "Output",
 ImageSize->{544, 67},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]//Short=",
 CellID->521761]
}, Open  ]],

Cell[TextData[{
 "Note that after an initial descriptive list, the file consists of ordered \
pairs of numbers locating points on the surface of the airfoil.  It is \
conventional to plot the airfoil in a normalized airfoil-fixed coordinate \
system with the leading edge of the airfoil at the origin and  the trailing \
edge at ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     StyleBox["x", "TI"], ",", " ", 
     StyleBox["z", "TI"]}], ")"}], "=", 
   RowBox[{"(", 
    RowBox[{"1", ",", " ", "0"}], ")"}]}]], "InlineFormula"],
 ".  Observe from the data that the first and last data points are coincident \
and locate the trailing edge of the airfoil.  By looking at the ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 "-values, observe that the data points are ordered such that they traverse \
the airfoil in a counterclockwise manner.  You may want to display the entire \
data file to verify this."
}], "Text",
 CellID->496848717],

Cell[TextData[{
 "The function ",
 Cell[BoxData[
  ButtonBox["AirfoilFromDataFile",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"]], 
  "InlineFormula"],
 " imports the data file and casts the data in the form used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages and note that the default option for AirfoilFromDataFile is \
Counterclockwise."
}], "Text",
 CellID->271030335],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->26732741],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "AirfoilFromDataFile", "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->182211284],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"NumberingConvention", "\[Rule]", "Counterclockwise"}], 
  "}"}]], "Output",
 ImageSize->{280, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->29776540]
}, Open  ]],

Cell["\<\
Import the data file and compute the collection of vectors locating data \
points.\
\>", "Text",
 CellID->521166357],

Cell[BoxData[
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{
   "AirfoilFromDataFile", "[", 
    "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}], 
  ";"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->613288026],

Cell["\<\
Count the number if vectors in the collection, and verify that the collection \
is of the data type CartesianVectors.\
\>", "Text",
 CellID->88025627],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"NumberOfVectors", "[", "rD", "]"}], ",", 
   RowBox[{"Head", "[", "rD", "]"}]}], "}"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->546386324],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"61", ",", "CartesianVectors"}], "}"}]], "Output",
 ImageSize->{155, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->61089721]
}, Open  ]],

Cell[TextData[{
 "For the data type CartesanVectors, all the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-components are listed together and all the ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 "-components are listed together.  An infix arrow separates the two lists of \
coordinates.  Observe that a clockwise numbering convention is used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellID->273275556],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rD", "=", 
  RowBox[{
  "AirfoilFromDataFile", "[", 
   "\"\<Aerodynamics\\\\Airfoil Data\\\\e205.dat\>\"", "]"}]}]], "Input",
 CellLabel->"In[6]:=",
 CellID->58269839],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "1.`", ",", "0.99651`", ",", "0.98635`", ",", "0.97017`", ",", "0.9486`", 
     ",", "0.92195`", ",", "0.89034`", ",", "0.854`", ",", "0.81343`", ",", 
     "0.76911`", ",", "0.7216`", ",", "0.67149`", ",", "0.61938`", ",", 
     "0.56591`", ",", "0.51174`", ",", "0.45751`", ",", "0.40388`", ",", 
     "0.35149`", ",", "0.30097`", ",", "0.2529`", ",", "0.20783`", ",", 
     "0.16627`", ",", "0.12864`", ",", "0.09534`", ",", "0.06669`", ",", 
     "0.04291`", ",", "0.02419`", ",", "0.01065`", ",", "0.00233`", ",", 
     "0.00002`", ",", "0.00331`", ",", "0.01097`", ",", "0.02292`", ",", 
     "0.03909`", ",", "0.05937`", ",", "0.0836`", ",", "0.11157`", ",", 
     "0.14302`", ",", "0.17764`", ",", "0.21508`", ",", "0.25496`", ",", 
     "0.29699`", ",", "0.34101`", ",", "0.3868`", ",", "0.4341`", ",", 
     "0.48265`", ",", "0.53217`", ",", "0.58218`", ",", "0.63204`", ",", 
     "0.68108`", ",", "0.72866`", ",", "0.77412`", ",", "0.81684`", ",", 
     "0.85624`", ",", "0.89175`", ",", "0.92285`", ",", "0.94916`", ",", 
     "0.97049`", ",", "0.98649`", ",", "0.99655`", ",", "1.`"}], "}"}], 
   "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"0.`", ",", 
     RowBox[{"-", "0.00003`"}], ",", 
     RowBox[{"-", "0.00036`"}], ",", 
     RowBox[{"-", "0.00125`"}], ",", 
     RowBox[{"-", "0.00252`"}], ",", 
     RowBox[{"-", "0.0038`"}], ",", 
     RowBox[{"-", "0.00497`"}], ",", 
     RowBox[{"-", "0.00614`"}], ",", 
     RowBox[{"-", "0.0074`"}], ",", 
     RowBox[{"-", "0.00876`"}], ",", 
     RowBox[{"-", "0.01023`"}], ",", 
     RowBox[{"-", "0.0118`"}], ",", 
     RowBox[{"-", "0.01345`"}], ",", 
     RowBox[{"-", "0.01516`"}], ",", 
     RowBox[{"-", "0.01689`"}], ",", 
     RowBox[{"-", "0.01859`"}], ",", 
     RowBox[{"-", "0.02021`"}], ",", 
     RowBox[{"-", "0.02168`"}], ",", 
     RowBox[{"-", "0.02292`"}], ",", 
     RowBox[{"-", "0.02384`"}], ",", 
     RowBox[{"-", "0.02435`"}], ",", 
     RowBox[{"-", "0.02436`"}], ",", 
     RowBox[{"-", "0.02378`"}], ",", 
     RowBox[{"-", "0.02252`"}], ",", 
     RowBox[{"-", "0.02053`"}], ",", 
     RowBox[{"-", "0.01776`"}], ",", 
     RowBox[{"-", "0.0142`"}], ",", 
     RowBox[{"-", "0.00988`"}], ",", 
     RowBox[{"-", "0.00506`"}], ",", "0.00055`", ",", "0.00766`", ",", 
     "0.01589`", ",", "0.02461`", ",", "0.03344`", ",", "0.04211`", ",", 
     "0.0504`", ",", "0.05811`", ",", "0.06507`", ",", "0.07111`", ",", 
     "0.07606`", ",", "0.0797`", ",", "0.08177`", ",", "0.08214`", ",", 
     "0.08081`", ",", "0.07785`", ",", "0.07342`", ",", "0.06782`", ",", 
     "0.06147`", ",", "0.0547`", ",", "0.04777`", ",", "0.04088`", ",", 
     "0.03419`", ",", "0.02786`", ",", "0.02199`", ",", "0.01668`", ",", 
     "0.01196`", ",", "0.00778`", ",", "0.00427`", ",", "0.00174`", ",", 
     "0.00039`", ",", "0.`"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[CompressedData["
1:eJwB9QEK/iFib1JlAQAAAD0AAAAAAAAAAADwP/phhPBo4+8/cRsN4C2Q7z8k
7rH0oQvvPwmKH2PuWu4/qoJRSZ2A7T+vQspPqn3sP4cW2c73U+s/nbryWZ4H
6j+U2SCTjJzoPyxlGeJYF+c/zemymNh85T/EfHkB9tHjP+tWz0nvG+I/48KB
kCxg4D8O8+UF2EfdP/CK4H8r2dk/H7+36c9+1j+J78SsF0PTPzojSnuDL9A/
0VeQZiyayj9jKCfaVUjFP2WNeohGd8A/8SkAxjNouD+mYfiImBKxP3+8V61M
+KU/E5uPa0PFmD/kg57Nqs+FP4P6ljldFmM/8WjjiLX49D6vCP63kh1rPyyf
5Xlwd4Y/8YCyKVd4lz8WhzO/mgOkP0sfuqC+Za4/1xLyQc9mtT/k9zb92Y+8
PwNDVrd6TsI/dxA7U+i8xj/yBwPPvYfLPzdxcr9DUdA/bkxPWOIB0z8z+Wab
G9PVP2/whclUwdg/8WPMXUvI2z+IhVrTvOPeP7oxPWGJB+E/N1SM8zeh4j9h
cTjzqznkP7O1vkhoy+U/U+i8xi5R5z+hvmVOl8XoP4gRwqONI+o/g92wbVFm
6z/wp8ZLN4nsP5yiI7n8h+0/yHvVyoRf7j+RCmMLQQ7vP+GX+nlTke8/iIVa
07zj7z8AAAAAAADwP7Ee+1o=
    "], CompressedData["
1:eJwB9QEK/iFib1JlAQAAAD0AAAAAAAAAAAAAAGkdVU0Qdf++D9b/OcyXN797
FK5H4XpUv03bv7LSpGS/xY8xdy0hb79dv2A3bFt0v9i2KLNBJnm/rK3YX3ZP
fr/GihpMw/CBvxYwgVt384S/7Q2+MJkqiL+WsgxxrIuLv1ysqME0DI+/64uE
tpxLkb9hbCHIQQmTv25pNSTusZS/F7zoK0gzlr/xgLIpV3iXvyi4WFGDaZi/
5WGh1jTvmL9SflLt0/GYv5kNMsnIWZi/5A8GnnsPl78S91j60AWVvwE1tWyt
L5K/hlrTvOMUjb9AahMn9zuEv7a+SGjLuXS/L26jAbwFQj8AOsyXF2B/P0xx
Vdl3RZA/+kSeJF0zmT+sqME0DB+hP3NLqyFxj6U/INJvXwfOqT9xICQLmMCt
P2Ni83FtqLA/wCZr1EM0sj9/pIgMq3izP0i/fR04Z7Q/Vz7L8+DutD9JhbGF
IAe1Px2UMNP2r7Q/klz+Q/rtsz9dUN8yp8uyP7HEA8qmXLE/DMhe7/54rz/E
sS5uowGsPxO4dTdPdag/PPceLjnupD+oUrMHWoGhP4Fbd/NUh5w/TS1b64uE
lj/4Nv3ZjxSRP5Gb4QZ8fog/dY4B2evdfz8FqKlla31xP6ciFcYWglw/5SfV
Ph2POT8AAAAAAAAAAPf66Iw=
    "]], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{551, 254},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->289128254]
}, Open  ]],

Cell[TextData[{
 "Plot the points specifying the airfoil using the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["PlotAirfoil",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PlotAirfoil"]], "InlineFormula"],
 "."
}], "Text",
 CellID->681250186],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"rD", ",", 
   RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellLabel->"In[7]:=",
 CellID->13906675],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c+KK2kZwOGIVzK30SCKooiuHBBXgzBIljMu9BocRCFbUWRGRje6Gym3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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->76284173]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fore-to-aft numbering convention", "Subsection",
 CellID->9113640],

Cell["\<\
Look at a few lines of data from the file for the Clark-Y airfoil as an \
example of the fore-to-aft numbering convention.\
\>", "Text",
 CellID->120129970],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{
   "Import", "[", "\"\<Aerodynamics\\\\Airfoil Data\\\\clarky.dat\>\"", "]"}],
    ",", "5"}], "]"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->15342918],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"CLARK\"\>", ",", "\<\"Y\"\>", ",", "\<\"AIRFOIL\"\>"}], 
     "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"61.`", ",", "61.`"}], "}"}], ",", 
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.`", ",", "0.`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.0005`", ",", "0.002339`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.001`", ",", "0.0037271`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.002`", ",", "0.0058025`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.004`", ",", "0.0089238`"}], "}"}], ",", 
    RowBox[{"\[LeftSkeleton]", "110", "\[RightSkeleton]"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.9`", ",", 
      RowBox[{"-", "0.0042718`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.92`", ",", 
      RowBox[{"-", "0.0035373`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.94`", ",", 
      RowBox[{"-", "0.0028028`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.96`", ",", 
      RowBox[{"-", "0.0020683`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.97`", ",", 
      RowBox[{"-", "0.0017011`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.98`", ",", 
      RowBox[{"-", "0.0013339`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.99`", ",", 
      RowBox[{"-", "0.0009666`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1.`", ",", 
      RowBox[{"-", "0.0005993`"}]}], "}"}]}], "}"}],
  Short[#, 5]& ]], "Output",
 ImageSize->{541, 67},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]//Short=",
 CellID->348512777]
}, Open  ]],

Cell["\<\
The format for the information in the file differs from that used for the \
Eppler-205 airfoil.  After identifying the airfoil, the number of points is \
given and then an empty list before beginning the ordered pairs of numbers.  \
Even with this truncated listing of the file, the data can be identified as \
using the fore-to-aft numbering convention rather than the counterclockwise \
convention.\
\>", "Text",
 CellID->398895014],

Cell["\<\
Import the data file and plot the points specifying the airfoil.\
\>", "Text",
 CellID->583917234],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rDCY", "=", 
   RowBox[{"AirfoilFromDataFile", "[", 
    RowBox[{"\"\<Aerodynamics\\\\Airfoil Data\\\\clarky.dat\>\"", ",", 
     RowBox[{"NumberingConvention", "\[Rule]", "ForeToAft"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", 
  RowBox[{"rDCY", ",", 
   RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[9]:=",
 CellID->294412045],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3T+P7FYZB+BBfBK+xkrh5uoq/BeCSCiiQEgppkwooESUUIA0LTcIEApQ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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->295259753]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.2.2 Discretization of airfoil", "Section",
 CellID->636677227],

Cell["\<\
The data in the file can be used to specify panel endpoints for a discretized \
airfoil.  This is visualized by joining the data points in the file.  For \
example, here is the discretized airfoil determined by the data file for the \
Eppler-205 airfoil.\
\>", "Text",
 CellID->203189468],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", "rD", "]"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->812310790],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c+LG+cdx3GV/iX9NxZKD8aHkINxwfhgfNkFkVyai31JLu2lvdggkosN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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->299384336]
}, Open  ]],

Cell[TextData[{
 "More flexibility in discretization is obtained by fitting the data in the \
file with a B-spline and basing a  user-specified discretization of the \
airfoil on this spline function.  The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " 7 function Interpolation has an option Method that accepts \"Spline\" as a \
value.  Unfortunately, there are problems with this function for the problem \
of interest.  The temporary ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["FitAirfoil",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/FitAirfoil"]], "InlineFormula"],
 " contains code furnished by the Wolfram Development Team to accomplish this \
task.  The spline function obtained from FitAirfoil is used together with a \
user-specified list of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values in the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " function ",
 Cell[BoxData[
  ButtonBox["AirfoilSurfacePoints",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilSurfacePoints"]], 
  "InlineFormula"],
 " to rediscretize the airfoil. "
}], "Text",
 CellID->20949403],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"spline", "=", 
  RowBox[{"FitAirfoil", "[", "rD", "]"}]}]], "Input",
 CellLabel->"In[12]:=",
 CellID->47856896],

Cell[BoxData[
 TagBox[
  RowBox[{"BSplineFunction", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"0.`", ",", "1.0000000000000004`"}], "}"}], "}"}], 
    ",", "\<\"<>\"\>"}], "]"}],
  False,
  Editable->False]], "Output",
 ImageSize->{215, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->112951616]
}, Open  ]],

Cell[TextData[{
 "Use the function ",
 Cell[BoxData[
  ButtonBox["NDiscretizeUnitSegment",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/NDiscretizeUnitSegment"]], 
  "InlineFormula"],
 " to generate a user-specified discretization list."
}], "Text",
 CellID->284760479],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"list", "=", 
  RowBox[{"NDiscretizeUnitSegment", "[", 
   RowBox[{"5", ",", 
    RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}]], "Input",
 CellLabel->"In[13]:=",
 CellID->11317880],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.`", ",", "0.04894348370484647`", ",", "0.19098300562505255`", ",", 
   "0.412214747707527`", ",", "0.6909830056250525`", ",", 
   "0.9999999999999999`"}], "}"}]], "Output",
 ImageSize->{332, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->11898749]
}, Open  ]],

Cell["\<\
Compute panel endpoints for the rediscretized airfoil and display them and \
the spline fit to the airfoil.\
\>", "Text",
 CellID->269235534],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"spline", ",", "list"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gP", "=", 
   RowBox[{"PlotAirfoil", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"Joined", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gFit", "=", 
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"spline", "[", "p", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"p", ",", "0", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gP", ",", "gFit"}], "]"}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->354096070],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3f+LFPcdx/Er/Uv6bxyYO3ux1F6qGLD+INc0miz6SyVBISiU9IfGEM52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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->49247270]
}, Open  ]],

Cell["Here is the user-specified discretized airfoil.", "Text",
 CellID->319617898],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlotAirfoil", "[", "rP", "]"}]], "Input",
 CellLabel->"In[18]:=",
 CellID->9442322],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c+LFPkZx/EO+UvybwyEHJY9LHsICSx7EC8jNHrJXpKLXtyLXhQavSgs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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->68516052]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.2.3 Data from the UIUC Airfoil Coordinate Database", "Section",
 CellID->114536833],

Cell[TextData[{
 "Data files for airfoil coordinates can be imported directly from ",
 ButtonBox["UIUC Airfoil Coordinates Database",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.ae.uiuc.edu/m-selig/ads/coord_database.html"], None}],
 ".  As an example, locate the data file for the Eppler-550 general aviation \
airfoil.  Place the cursor over the link to the airfoil data file, and note \
that the URL (address) is displayed at the bottom left of the screen.  Use \
this address in the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function Import to view a few lines of the data file to identify the \
numbering convention used."
}], "Text",
 CellID->342063873],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{
   "Import", "[", "\"\<http://www.ae.uiuc.edu/m-selig/ads/coord/e550.dat\>\"",
     "]"}], ",", "4"}], "]"}]], "Input",
 CellLabel->"In[38]:=",
 CellID->322390100],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"EPPLER\"\>", ",", "550", ",", "\<\"AIRFOIL\"\>"}], "}"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"35.`", ",", "38.`"}], "}"}], ",", 
    RowBox[{"{", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.00001`", ",", "0.00036`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.00013`", ",", "0.00208`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.00045`", ",", "0.00389`"}], "}"}], ",", 
    RowBox[{"\[LeftSkeleton]", "65", "\[RightSkeleton]"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.90695`", ",", 
      RowBox[{"-", "0.00114`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.93922`", ",", "0.00064`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.96522`", ",", "0.00124`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.98434`", ",", "0.00099`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.9960501`", ",", "0.00035`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1.`", ",", "0.`"}], "}"}]}], "}"}],
  Short[#, 4]& ]], "Output",
 ImageSize->{537, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[38]//Short=",
 CellID->63427531]
}, Open  ]],

Cell[TextData[{
 "The numbering convention used in this file is fore-to-aft.  Import the file \
into the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " software and display the discretized airfoil specified by the data file."
}], "Text",
 CellID->2374947],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rD", "=", 
   RowBox[{"AirfoilFromDataFile", "[", 
    RowBox[{"\"\<http://www.ae.uiuc.edu/m-selig/ads/coord/e550.dat\>\"", ",", 
     RowBox[{"NumberingConvention", "\[Rule]", "ForeToAft"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", "rD", "]"}]}], "Input",
 CellLabel->"In[20]:=",
 CellID->458862603],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c9rFecex/Fc7l9y/41A6UJclC6kF4qL0o2Fg27ajW50oxvdJHBINgbk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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[21]=",
 CellID->122830795]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.2.4 Surprises", "Section",
 CellID->577231104],

Cell[TextData[{
 "If you are importing airfoil data from a file prepared by someone else, \
review it carefully before using.  Data files in the UIUC collection are \
acquired from different sources, and conventions used may not be what you \
expect.  I had several surprises using the limited number of airfoils \
selected to test functions in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  Some of these surprises are listed in the Possible Issues section of the \
documentation page for the function ",
 Cell[BoxData[
  ButtonBox["AirfoilFromDataFile",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/AirfoilFromDataFile"]], 
  "InlineFormula"],
 "."
}], "Text",
 CellID->9676428],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["AirfoilGeometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/AirfoilGeometry"]], "TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->80129204]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["1.1 Introduction To Airfoil Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]], \
"RelatedTutorials",
 CellID->574311372],

Cell[TextData[ButtonBox["1.3 Airfoils From Thickness And Camber Properties",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"]], \
"RelatedTutorials",
 CellID->895221539],

Cell[TextData[ButtonBox["Table Of Contents",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/TableOfContents"]], "RelatedTutorials",
 CellID->6988088]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"1.2 Airfoils From Data Files - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 59, 18.9354000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "Michael Selig of the Aerospace Engineering Department at the University \
of Illinois at Urbana-Chanpaign maintains the UIUC Airfoil Data Site that \
provides links to useful information about airfoils and to data files for a \
collection of over 1500 airfoils. This site serves as the primary source for \
specifying airfoil geometry in Aerodynamics. Several data files from the UIUC \
Airfoil Coordinates Database have been downloaded and are located in the \
folder Airfoil Data for use as examples. In the listing below, repetitive \
parts have been stripped away from path names. There are three numbering \
conventions used to order the points specifying an airfoil: clockwise, \
counterclockwise and fore-to-aft. Clockwise and counterclockwise conventions \
start and end at the trailing edge, traversing the airfoil in the specified \
direction. The fore-to-aft convention traverses the upper surface of the \
airfoil first and then the lower surface in the specified direction. \
Aerodynamics uses the clockwise convention because it is consistent with the \
convention used in American textbooks for clockwise circulation producing \
positive lift. Data files in the UIUC Airfoil Database use both \
counterclockwise and fore-to-aft conventions.", "synonyms" -> {}, "title" -> 
    "1.2 Airfoils From Data Files", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/AirfoilsFromDataFiles"}, "SearchTextTranslated" -> 
  ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[609, 21, 78, 1, 70, "TutorialColorBar"],
Cell[690, 24, 191, 5, 70, "LinkTrail"],
Cell[884, 31, 3147, 71, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4056, 106, 65, 1, 70, "Title",
 CellID->509267359],
Cell[4124, 109, 567, 14, 70, "Text",
 CellID->1534169418],
Cell[CellGroupData[{
Cell[4716, 127, 68, 1, 70, "Section",
 CellID->339343814],
Cell[4787, 130, 277, 6, 70, "Text",
 CellID->100647037],
Cell[CellGroupData[{
Cell[5089, 140, 607, 18, 70, "Input",
 CellID->161538214],
Cell[5699, 160, 897, 18, 88, "Output",
 CellID->69847345]
}, Open  ]],
Cell[6611, 181, 766, 14, 70, "Text",
 CellID->448529733],
Cell[CellGroupData[{
Cell[7402, 199, 79, 1, 70, "Subsection",
 CellID->397338901],
Cell[7484, 202, 160, 4, 70, "Text",
 CellID->16776698],
Cell[CellGroupData[{
Cell[7669, 210, 213, 7, 70, "Input",
 CellID->464615637],
Cell[7885, 219, 1529, 46, 88, "Output",
 CellID->521761]
}, Open  ]],
Cell[9429, 268, 965, 22, 70, "Text",
 CellID->496848717],
Cell[10397, 292, 499, 16, 70, "Text",
 CellID->271030335],
Cell[10899, 310, 99, 3, 70, "Input",
 CellID->26732741],
Cell[CellGroupData[{
Cell[11023, 317, 121, 3, 70, "Input",
 CellID->182211284],
Cell[11147, 322, 245, 8, 36, "Output",
 CellID->29776540]
}, Open  ]],
Cell[11407, 333, 126, 4, 70, "Text",
 CellID->521166357],
Cell[11536, 339, 217, 8, 70, "Input",
 CellID->613288026],
Cell[11756, 349, 160, 4, 70, "Text",
 CellID->88025627],
Cell[CellGroupData[{
Cell[11941, 357, 192, 6, 70, "Input",
 CellID->546386324],
Cell[12136, 365, 219, 7, 36, "Output",
 CellID->61089721]
}, Open  ]],
Cell[12370, 375, 467, 13, 70, "Text",
 CellID->273275556],
Cell[CellGroupData[{
Cell[12862, 392, 192, 6, 70, "Input",
 CellID->58269839],
Cell[13057, 400, 4579, 92, 275, "Output",
 CellID->289128254]
}, Open  ]],
Cell[17651, 495, 309, 11, 70, "Text",
 CellID->681250186],
Cell[CellGroupData[{
Cell[17985, 510, 169, 5, 28, "Input",
 CellID->13906675],
Cell[18157, 517, 3536, 65, 201, "Output",
 CellID->76284173]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[21742, 588, 72, 1, 70, "Subsection",
 CellID->9113640],
Cell[21817, 591, 166, 4, 70, "Text",
 CellID->120129970],
Cell[CellGroupData[{
Cell[22008, 599, 214, 7, 70, "Input",
 CellID->15342918],
Cell[22225, 608, 1624, 50, 88, "Output",
 CellID->348512777]
}, Open  ]],
Cell[23864, 661, 444, 8, 70, "Text",
 CellID->398895014],
Cell[24311, 671, 108, 3, 70, "Text",
 CellID->583917234],
Cell[CellGroupData[{
Cell[24444, 678, 427, 11, 65, "Input",
 CellID->294412045],
Cell[24874, 691, 3697, 68, 201, "Output",
 CellID->295259753]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[28632, 766, 70, 1, 70, "Section",
 CellID->636677227],
Cell[28705, 769, 298, 6, 70, "Text",
 CellID->203189468],
Cell[CellGroupData[{
Cell[29028, 779, 109, 3, 28, "Input",
 CellID->812310790],
Cell[29140, 784, 5002, 89, 201, "Output",
 CellID->299384336]
}, Open  ]],
Cell[34157, 876, 1198, 32, 70, "Text",
 CellID->20949403],
Cell[CellGroupData[{
Cell[35380, 912, 135, 4, 70, "Input",
 CellID->47856896],
Cell[35518, 918, 376, 14, 36, "Output",
 CellID->112951616]
}, Open  ]],
Cell[35909, 935, 286, 9, 70, "Text",
 CellID->284760479],
Cell[CellGroupData[{
Cell[36220, 948, 220, 6, 70, "Input",
 CellID->11317880],
Cell[36443, 956, 348, 10, 36, "Output",
 CellID->11898749]
}, Open  ]],
Cell[36806, 969, 151, 4, 70, "Text",
 CellID->269235534],
Cell[CellGroupData[{
Cell[36982, 977, 698, 22, 83, "Input",
 CellID->354096070],
Cell[37683, 1001, 5372, 96, 201, "Output",
 CellID->49247270]
}, Open  ]],
Cell[43070, 1100, 83, 1, 70, "Text",
 CellID->319617898],
Cell[CellGroupData[{
Cell[43178, 1105, 107, 3, 28, "Input",
 CellID->9442322],
Cell[43288, 1110, 4859, 87, 201, "Output",
 CellID->68516052]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[48196, 1203, 91, 1, 70, "Section",
 CellID->114536833],
Cell[48290, 1206, 690, 15, 70, "Text",
 CellID->342063873],
Cell[CellGroupData[{
Cell[49005, 1225, 224, 7, 70, "Input",
 CellID->322390100],
Cell[49232, 1234, 1176, 35, 85, "Output",
 CellID->63427531]
}, Open  ]],
Cell[50423, 1272, 264, 7, 70, "Text",
 CellID->2374947],
Cell[CellGroupData[{
Cell[50712, 1283, 369, 9, 83, "Input",
 CellID->458862603],
Cell[51084, 1294, 5930, 105, 201, "Output",
 CellID->122830795]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[57063, 1405, 54, 1, 70, "Section",
 CellID->577231104],
Cell[57120, 1408, 708, 17, 70, "Text",
 CellID->9676428],
Cell[CellGroupData[{
Cell[57853, 1429, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[57921, 1432, 167, 4, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[58091, 1438, 156, 3, 70, "TutorialMoreAbout",
 CellID->80129204]
}, Open  ]],
Cell[CellGroupData[{
Cell[58284, 1446, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[58359, 1449, 205, 5, 70, "RelatedTutorials",
 CellID->574311372],
Cell[58567, 1456, 229, 5, 70, "RelatedTutorials",
 CellID->895221539],
Cell[58799, 1463, 168, 4, 70, "RelatedTutorials",
 CellID->6988088]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[59006, 1472, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

