(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     84168,       2479]
NotebookOptionsPosition[     74308,       2144]
NotebookOutlinePosition[     76711,       2195]
CellTagsIndexPosition[     76668,       2192]
WindowTitle->A1 Data Type CartesianVectors - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"References\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/References"], "\<\"A2 Graphics\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Graphics"], "\<\"Table of Contents\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"CartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/CartesianVectors"], "\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/CartesianVectors\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/CartesianVectors"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/CartesianVectors.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/CartesianVectors", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["A1 Data Type CartesianVectors", "Title",
 CellID->509267359],

Cell[TextData[{
 "Collections of vectors play an important role in implementing panel \
methods.  Points defining panel endpoints and unit normal vectors to panels \
can be represented as collections of vectors.  ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " functions for computing influence coefficients require the collection of \
vectors locating panel endpoints as input arguments.  Computation of \
influence coefficients requires vector algebra between two collections of \
vectors, and the resulting velocity influence coefficients are collections of \
vectors.  Defining a data type simplified pattern matching and programming \
for the packages.  My original intent was to leave the data type \"hidden\" \
in the packages, but the need to construct the data type occurs several times \
in the documentation.  Additionally, this introduction to the data type could \
be useful to those wanting to modify the packages or to those working on \
entirely different problems which involve the manipulation of large \
collections of vectors."
}], "Text",
 CellID->65021125],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->280757486],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[0]:=",
 CellID->107795920],

Cell[CellGroupData[{

Cell["A1.1 Constructor and Selector", "Section",
 CellID->152704095],

Cell[BoxData[GridBox[{
   {"MakeCartesianVectors", Cell[
    "constructs the data type CartesianVectors", "TableText"]},
   {"Components", Cell[
    "selects lists of components from CartesianVectors", "TableText"]}
  }]], "DefinitionBox",
 CellID->801716347],

Cell["Constructor and selector for the data type CartesianVectors.", "Caption",
 CellID->104944398],

Cell["\<\
Components of collections of vectors may be simple lists or matrices.  \
Vectors may be of any dimension.  Specify several collections of two- and \
three-dimensional vectors.  The heads of all these collections of vectors are \
CartesianVectors.\
\>", "Text",
 CellID->63591597],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"uS2D", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"ux", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"uy", ",", "5"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vS2D", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"vx", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"vy", ",", "5"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"uS3D", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"ux", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"uy", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"uz", ",", "5"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vS3D", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"vx", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"vy", ",", "5"}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"vz", ",", "5"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"uM3D", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Array", "[", 
       RowBox[{"vx", ",", 
        RowBox[{"{", 
         RowBox[{"4", ",", "5"}], "}"}]}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"vy", ",", 
        RowBox[{"{", 
         RowBox[{"4", ",", "5"}], "}"}]}], "]"}], ",", 
      RowBox[{"Array", "[", 
       RowBox[{"vz", ",", 
        RowBox[{"{", 
         RowBox[{"4", ",", "5"}], "}"}]}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Head", "/@", 
  RowBox[{"{", 
   RowBox[{"uS2D", ",", "vS2D", ",", "uS3D", ",", "vS3D", ",", "uM3D"}], 
   "}"}]}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->101936636],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "CartesianVectors", ",", "CartesianVectors", ",", "CartesianVectors", ",", 
   "CartesianVectors", ",", "CartesianVectors"}], "}"}]], "Output",
 ImageSize->{377, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->148063649]
}, Open  ]],

Cell["\<\
Display a couple of these collections of vectors, noting that an infix arrow \
is used to denote the data type CartesianVectors when displayed.\
\>", "Text",
 CellID->139029545],

Cell[CellGroupData[{

Cell["uS2D", "Input",
 CellLabel->"In[7]:=",
 CellID->203056159],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{479, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->549800377]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uM3D", "//", "MatrixForm"}]], "Input",
 CellLabel->"In[8]:=",
 CellID->186875196],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnT1y4zoWhT01K5lteA+TzAq6XuDqbBwqe6FCO3Po0KFrknboUKEW0EGH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    "], {{0, 177}, {333, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{333, 177},
  PlotRange->{{0, 333}, {0, 177}}]], "Output",
 ImageSize->{337, 177},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[8]=",
 CellID->414205713]
}, Open  ]],

Cell["\<\
The function Components selects the list of components from the collection of \
vectors of the data type CartesianVectors.\
\>", "Text",
 CellID->133331613],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Components", "[", "uS2D", "]"}]], "Input",
 CellLabel->"In[9]:=",
 CellID->228036416],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}]}], "}"}]], "Output",
 ImageSize->{495, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->376346877]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A1.1 Vector algebra", "Section",
 CellID->98413141],

Cell[CellGroupData[{

Cell["Basic addition, subtraction and multiplication by a constant", \
"Subsection",
 CellID->578648157],

Cell[TextData[{
 "By overloading the ",
 StyleBox["Mathematica ",
  FontSlant->"Italic"],
 "functions, basic element-by-element arithmetic operations are defined for \
collections of vectors of the data type CartesianVectors.  Collections of \
vectors can be added, subtracted or multiplied by a constant."
}], "Text",
 CellID->110468743],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uS2D", "+", "vS2D"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->337721988],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ux", "[", "1", "]"}], "+", 
      RowBox[{"vx", "[", "1", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "2", "]"}], "+", 
      RowBox[{"vx", "[", "2", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "3", "]"}], "+", 
      RowBox[{"vx", "[", "3", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "4", "]"}], "+", 
      RowBox[{"vx", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "5", "]"}], "+", 
      RowBox[{"vx", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"uy", "[", "1", "]"}], "+", 
      RowBox[{"vy", "[", "1", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "2", "]"}], "+", 
      RowBox[{"vy", "[", "2", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "3", "]"}], "+", 
      RowBox[{"vy", "[", "3", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "4", "]"}], "+", 
      RowBox[{"vy", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "5", "]"}], "+", 
      RowBox[{"vy", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`ux[
      1] + $CellContext`vx[1], $CellContext`ux[2] + $CellContext`vx[
      2], $CellContext`ux[3] + $CellContext`vx[3], $CellContext`ux[
      4] + $CellContext`vx[4], $CellContext`ux[5] + $CellContext`vx[
      5]}, {$CellContext`uy[1] + $CellContext`vy[1], $CellContext`uy[
      2] + $CellContext`vy[2], $CellContext`uy[3] + $CellContext`vy[
      3], $CellContext`uy[4] + $CellContext`vy[4], $CellContext`uy[
      5] + $CellContext`vy[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{485, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[10]=",
 CellID->204399871]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uS3D", "-", "vS3D"}]], "Input",
 CellLabel->"In[11]:=",
 CellID->716287734],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ux", "[", "1", "]"}], "-", 
      RowBox[{"vx", "[", "1", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "2", "]"}], "-", 
      RowBox[{"vx", "[", "2", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "3", "]"}], "-", 
      RowBox[{"vx", "[", "3", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "4", "]"}], "-", 
      RowBox[{"vx", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{"ux", "[", "5", "]"}], "-", 
      RowBox[{"vx", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"uy", "[", "1", "]"}], "-", 
      RowBox[{"vy", "[", "1", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "2", "]"}], "-", 
      RowBox[{"vy", "[", "2", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "3", "]"}], "-", 
      RowBox[{"vy", "[", "3", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "4", "]"}], "-", 
      RowBox[{"vy", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uy", "[", "5", "]"}], "-", 
      RowBox[{"vy", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"uz", "[", "1", "]"}], "-", 
      RowBox[{"vz", "[", "1", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uz", "[", "2", "]"}], "-", 
      RowBox[{"vz", "[", "2", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uz", "[", "3", "]"}], "-", 
      RowBox[{"vz", "[", "3", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uz", "[", "4", "]"}], "-", 
      RowBox[{"vz", "[", "4", "]"}]}], ",", 
     RowBox[{
      RowBox[{"uz", "[", "5", "]"}], "-", 
      RowBox[{"vz", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`ux[
      1] - $CellContext`vx[1], $CellContext`ux[2] - $CellContext`vx[
     2], $CellContext`ux[3] - $CellContext`vx[
     3], $CellContext`ux[4] - $CellContext`vx[
     4], $CellContext`ux[5] - $CellContext`vx[
     5]}, {$CellContext`uy[1] - $CellContext`vy[
     1], $CellContext`uy[2] - $CellContext`vy[
     2], $CellContext`uy[3] - $CellContext`vy[
     3], $CellContext`uy[4] - $CellContext`vy[
     4], $CellContext`uy[5] - $CellContext`vy[
     5]}, {$CellContext`uz[1] - $CellContext`vz[
     1], $CellContext`uz[2] - $CellContext`vz[
     2], $CellContext`uz[3] - $CellContext`vz[
     3], $CellContext`uz[4] - $CellContext`vz[
     4], $CellContext`uz[5] - $CellContext`vz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{493, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[11]=",
 CellID->243454251]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a", "*", "uS2D"}]], "Input",
 CellLabel->"In[12]:=",
 CellID->67522338],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", 
      RowBox[{"ux", "[", "1", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"ux", "[", "2", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"ux", "[", "3", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"ux", "[", "4", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"ux", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", " ", 
      RowBox[{"uy", "[", "1", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"uy", "[", "2", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"uy", "[", "3", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"uy", "[", "4", "]"}]}], ",", 
     RowBox[{"a", " ", 
      RowBox[{"uy", "[", "5", "]"}]}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`a \
$CellContext`ux[1], $CellContext`a $CellContext`ux[
      2], $CellContext`a $CellContext`ux[3], $CellContext`a $CellContext`ux[
      4], $CellContext`a $CellContext`ux[5]}, {$CellContext`a $CellContext`uy[
      1], $CellContext`a $CellContext`uy[2], $CellContext`a $CellContext`uy[
      3], $CellContext`a $CellContext`uy[4], $CellContext`a $CellContext`uy[
      5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{295, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->99853864]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Multiplication by a list", "Subsection",
 CellID->87368199],

Cell[TextData[{
 "I was unable to define the multiplication of a collection of vectors by a \
list by overloading ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions, so an exported function MultiplyByList is defined to accomplish \
this task.  First, each component of a collection of vectors is scaled by a \
different factor.  A second definition provides multiplication of each vector \
in the collection be a different factor."
}], "Text",
 CellID->91427808],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplyByList", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"sx", ",", "sy", ",", "sz"}], "}"}], ",", "uS3D"}], 
  "]"}]], "Input",
 CellLabel->"In[13]:=",
 CellID->87964621],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnEuymzAQRUllJdlChiwigyzhDZhmK6yGzXgxDj+DhFoyPHS7pfI9Valy
8JM4yGrAH91fX//+fv1smub3+O/Pj6aZHj8JIYQQQgghhBBSHo++bZq2f+ju
0Nvj0I0bmm5I/1URxOT9jdXIb/ZVyr+2OjOnEvnl/8ehF+WPR+gyvnqRZ0SH
TnlYHn3v2Y1HMur2reA89OW9ZJL8xDjqbe8+Vb78VOaLfTBlypefGbq2D2dO
+fJj+cbORpK8WKTTa9YN3eliDwZwP9107TKG63l/fLw/cve3blieXGZ7qon4
kmWqdCP5tbOblW4of7/SLeQf86TJUOnq8hcrPdy4vmDDMJy+sofn+ZfCfsaf
R2I+HLFTZ6j32R5vAqx0K/nna9bVJ79O8rvFYiA/V8tTmjnly7t372emTeyI
5ko/STCA2/tO3yEyIybEo443QV7TjeS3l7hG+XXXN6/p6vJbeWa4phuOfDD0
mEpP3ETs58xNqG3bS0cdaYKsdGcnavLTNAnKvBZ5qau5h+LlnavicfzLl490
Nfcgykcq9cY13TnDOEe93dQ4dzeu6nZ23cc83uRWpe9vkQQM5FMfjJYuP3+G
+NgfViXv9vP+ml6YvDPy/rlFlI9tfN6t9OZwJ/N6T7F/1HA8u+xf1fTLiCab
JE/Ob+5kkq+YvvzDd69L3vMPOi5f3u+iMvkLIx/d+Lxa6dbfsiU4HuDyftjy
GxTKW/Gp8kvzHEVawC9nTv/VpV8KQKC8FZ8qr16fhBBCCCGEEEIIIYQQQggh
hJA0BXzJPjCeQgXGU1jBeIoJxlNowXgKK6qKp3DX4PsreqJL9eUmFvEUYPm1
M1A8hYL8/Uq3kIfHU8Dki42n2Df6jxfEZX3xJtrxFAryz9esq08eGU+BlQfH
U0Dly42n2OZxOKFjC3ijTdTjKfDywHgKvPy665vXdHV5eDyFzsgHQ28eT7H8
pXTQ0aX6kSYG8RRQeXQ8BXrkg67qkFeIpyho5LXiKZZnpvW50pQWl+rHm2jH
U0Dl0fEUSHl4PAVS3u3n/TW9MPlC4yk2okt/g6X6ySaq8RRYeXA8BVbe888f
TwGX97uoTL7seArxsnu9iU3OAOUp/w3s5ZfmSvEU4gcWaRJNlHMGKJ/WCqD8
G5NvNGE8BSGEEEIIIYQQQgghhBBCCCFVwiSKKzCJwgomUdx2YBLFBZhEYQWT
KDI7OKeb16r8wy+DD/NZXJWfaqKbRKEivzbKnkShJo9IosDLA5MowPK3kyi+
wbUkivMr+OJNzJIocPJi53XIqyRRQOS1kigQ8rmSKK5wLoli05NX8MlrdSNN
1JMo8PLAJAq8/Lrrm9d0dXl4EgVS3t/3+/Xptyv9QhJF+D50I3rUchODJAqo
PDqJAj3yQVdzB8XLKyRRFDTyyUo9x7kkCueuJixJeVV+tIl6EgVSHp5EAZTH
J1Hgp43Yc/Hy2ZIornAqiSIRFuE2clflx5toJ1EA5fFJFNCRRydRYOX9LiqT
z5REcQX7b9kSZIwUyATlrfhU+aW5UhJFXpQjBfJCeSs+VZ5JFIQQQsjn8R8h
uGyk
    "], {{0, 50}, {334, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{334, 50},
  PlotRange->{{0, 334}, {0, 50}}]], "Output",
 ImageSize->{338, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->187093065]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"MultiplyByList", "[", 
    RowBox[{
     RowBox[{"Array", "[", 
      RowBox[{"a", ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "5"}], "}"}]}], "]"}], ",", "uM3D"}], "]"}], ")"}], 
  "//", "MatrixForm"}]], "Input",
 CellLabel->"In[14]:=",
 CellID->100168776],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnT9yG0uTxBW7J9lryNwj7AkUn6GQtzTpfSZN0qMpU6ZiHdCkSZMHGEMm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    "], {{0, 177}, {578, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{578, 177},
  PlotRange->{{0, 578}, {0, 177}}]], "Output",
 ImageSize->{582, 177},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->5396401]
}, Open  ]],

Cell["\<\
For most applications, the number of vectors in a collection is large, so a \
conflict between the two definitions should be rare.  If the number of \
vectors is equal to the dimensionality of the space, then multiplication of \
components by elements of the list takes precedence.\
\>", "Text",
 CellID->256903099]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dot and cross products", "Subsection",
 CellID->130895041],

Cell["\<\
Dot and cross product are overloaded to provide elemen-by-element products \
for collections of vectors.\
\>", "Text",
 CellID->564988326],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uS2D", ".", "vS2D"}]], "Input",
 CellLabel->"In[15]:=",
 CellID->1023740835],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], " ", 
     RowBox[{"vx", "[", "1", "]"}]}], "+", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], " ", 
     RowBox[{"vy", "[", "1", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"ux", "[", "2", "]"}], " ", 
     RowBox[{"vx", "[", "2", "]"}]}], "+", 
    RowBox[{
     RowBox[{"uy", "[", "2", "]"}], " ", 
     RowBox[{"vy", "[", "2", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"ux", "[", "3", "]"}], " ", 
     RowBox[{"vx", "[", "3", "]"}]}], "+", 
    RowBox[{
     RowBox[{"uy", "[", "3", "]"}], " ", 
     RowBox[{"vy", "[", "3", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"ux", "[", "4", "]"}], " ", 
     RowBox[{"vx", "[", "4", "]"}]}], "+", 
    RowBox[{
     RowBox[{"uy", "[", "4", "]"}], " ", 
     RowBox[{"vy", "[", "4", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"ux", "[", "5", "]"}], " ", 
     RowBox[{"vx", "[", "5", "]"}]}], "+", 
    RowBox[{
     RowBox[{"uy", "[", "5", "]"}], " ", 
     RowBox[{"vy", "[", "5", "]"}]}]}]}], "}"}]], "Output",
 ImageSize->{518, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->62967172]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wS3D", "=", 
  RowBox[{"uS3D", "\[Cross]", "vS3D"}]}]], "Input",
 CellLabel->"In[16]:=",
 CellID->285324683],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztne2R67gVRKfsSPzTKUwYDmGrrL9OZaJRMhMMPZJGEgmCEC7QTYH7zqna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    "], {{0, 101}, {553, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{553, 101},
  PlotRange->{{0, 553}, {0, 101}}]], "Output",
 ImageSize->{557, 101},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->25903942]
}, Open  ]],

Cell["\<\
The cross product is perpendicular to the original vectors element-by-element \
in the collections, and there were five vectors each collection.\
\>", "Text",
 CellID->378189741],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "/@", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"wS3D", ".", "uS3D"}], ",", 
    RowBox[{"wS3D", ".", "vS3D"}]}], "}"}]}]], "Input",
 CellLabel->"In[17]:=",
 CellID->110286018],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 ImageSize->{215, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->8780141]
}, Open  ]],

Cell["\<\
Generalization of the cross product  is convenient for computing \
element-by-element normal vectors for a collection of two-dimensional vectors.\
\>", "Text",
 CellID->113463184],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wS2D", "=", 
  RowBox[{"Cross", "[", "uS2D", "]"}]}]], "Input",
 CellLabel->"In[19]:=",
 CellID->221821086],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"uy", "[", "1", "]"}]}], ",", 
     RowBox[{"-", 
      RowBox[{"uy", "[", "2", "]"}]}], ",", 
     RowBox[{"-", 
      RowBox[{"uy", "[", "3", "]"}]}], ",", 
     RowBox[{"-", 
      RowBox[{"uy", "[", "4", "]"}]}], ",", 
     RowBox[{"-", 
      RowBox[{"uy", "[", "5", "]"}]}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{-$CellContext`uy[
      1], -$CellContext`uy[2], -$CellContext`uy[3], -$CellContext`uy[
      4], -$CellContext`uy[5]}, {
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{519, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[19]=",
 CellID->509912641]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wS2D", ".", "uS2D"}]], "Input",
 CellLabel->"In[20]:=",
 CellID->317318721],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}]], "Output",
 ImageSize->{97, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[20]=",
 CellID->84639185]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Outer", "Subsection",
 CellID->337431051],

Cell["\<\
In the following example, two collections of vectors locate red and blue \
points.  Outer is used to compute displacement vectors from one collection of \
points to another, and yields a collections of vectors whose components are \
matrices.\
\>", "Text",
 CellID->18165939],

Cell[TextData[{
 "Define two collections of vectors, where ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["a", "TI"]]], "InlineFormula"],
 " locates the red points and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["b", "TI"]]], "InlineFormula"],
 " locates the blue points in the graphic below.  Use Outer to compute \
displacement vectors locating every blue point with respect to each red point."
}], "Text",
 CellID->435648623],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rA", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0.", ",", "0.", ",", "0."}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0.", ",", "1.", ",", "2."}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rB", "=", 
   RowBox[{"MakeCartesianVectors", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1.", ",", "1.", ",", "1."}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0.", ",", "1.", ",", "2."}], "}"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"(", 
   RowBox[{"rBA", "=", 
    RowBox[{"Outer", "[", 
     RowBox[{"Plus", ",", "rB", ",", 
      RowBox[{"-", "rA"}]}], "]"}]}], ")"}], "//", "MatrixForm"}]}], "Input",
 CellLabel->"In[21]:=",
 CellID->256028157],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztXLGS2jAQZSZfkjafwGfkC5grGLpQuqOkxB0lJSVVjpLSpT+AgtIlJaWy
jpOcg7VrS9r1SWbfDDfMyV6tnp61tlfL17cf39++zGazb/D5CZ/6u+nH+xKO
my3fBxwasYVQNB4I+CA3tL8uz+b5ld96g3xeu57P/f2PwUIgro0Hpqacl2u5
of1zuZbJPA83+Hg8drsd0leo/yNb2Gw2WBMMc7Va3W43rLVLQqvrWh+DvRgK
YeUP9ZkgDXA6naqqsjYlJw9sLDD7WZat12ugwqoQ64ntrmWudEF5XAdbJwQA
wJYOk6A8QAb7/b77/+12e7/fm8sE/sJhTwdYSUhXHnVYHLzcYaQByrI8n8/Y
icnJw/ye6O7sN8BWUYyE4OBSz1ILzwMJIIeyXN+bOnqLkQaXFXFW1//GreGj
Gt8CzHVRFNYmTB4YCdd8+eeG9ONbvwPDEU6Ozab9fD/SMMYwcQ73/7MsQBDB
lkpssOi92cc1+t+DC88khpHT5zAPaXC35utKvHCVwSRJcIWVnMViMboj4nCV
xyRJcIWVHPqZN1G4ymOSJLjCSsIk11UNLh7Q4KLBhYDKQ+VBQOWh8iDgKI8Y
0vGeFvjkwZx2F9se0O2EhzSMmRjS8d4WuOTBm3aX2x7QBi9p9LqaYs7FcAcX
LnlIbw9A+hoKlUc08hDf46TyIKDyUHkQiF4e4cGFf6sAizziT+gbeXn45VWl
twe0wUUacdOeaELfsD65cKbdxbYHsPisr8X0tRgBlYfKg4DKQ+VBQOWh8iCg
8lB5EFB5qDwIqDxUHgSs5GRZhhz+Qgn9PhKe31T4QvAHB8K3CjjJ46US+jQJ
NTiq3eWq8lm2CmjOJSC4sKXghWt1/f10DC5P/XoiCXnQJBiXavdeSJdye9u3
kuNaY+uK8S1UVXU8Hq1NmDxoEpyq3XsRpzww0oAZrJ7du6/PtXC5XMqytDYR
8sBI8Kh2pxFncMFIA9k4/YBD/Al9rP7a4PLASPCrdqchUZUfvlWAIA355ZMk
E/ow0YfDATNINFlI8K12xyBVlW9CtwrQpBVFga3GyQGGScRKAlMiwRW9pEHo
GcsXWYQMZDIkuOJlB65QKNjxC+ZtDTI=
    "], {{0, 42}, {181, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{181, 42},
  PlotRange->{{0, 181}, {0, 42}}]], "Output",
 ImageSize->{185, 42},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[23]=",
 CellID->412680610]
}, Open  ]],

Cell[TextData[{
 "Use ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["r", "TI"], 
    StyleBox["b", "TI"]], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["r", "TI"], 
     StyleBox["a", "TI"]], "+", 
    SubscriptBox[
     StyleBox["r", "TI"], 
     StyleBox["ba", "TI"]]}]}]], "InlineFormula"],
 " to locate the blue points, noting that arrows are drawn to all blue points \
from each red point."
}], "Text",
 CellID->732586039],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"gA", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{"rA", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Red", ",", 
        RowBox[{"PointSize", "[", "Large", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gB", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{"rB", ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Blue", ",", 
        RowBox[{"PointSize", "[", "Large", "]"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gArrows", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"Map", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Arrow", "[", 
        RowBox[{"rA", ",", 
         RowBox[{"rA", "+", 
          RowBox[{"rBA", "[", 
           RowBox[{"[", "#", "]"}], "]"}]}]}], "]"}], "&"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gA", ",", "gB", ",", "gArrows", ",", 
   RowBox[{"Axes", "\[Rule]", "False"}], ",", 
   RowBox[{"Frame", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellLabel->"In[24]:=",
 CellID->275335666],

Cell[BoxData[
 GraphicsBox[{
   {RGBColor[1, 0, 0], PointSize[Large], 
    PointBox[{{0., 0.}, {0., 1.}, {0., 2.}}]}, 
   {RGBColor[0, 0, 1], PointSize[Large], 
    PointBox[{{1., 0.}, {1., 1.}, {1., 2.}}]}, {{
     ArrowBox[{{0., 0.}, {1., 0.}}], ArrowBox[{{0., 1.}, {1., 0.}}], 
     ArrowBox[{{0., 2.}, {1., 0.}}]}, {ArrowBox[{{0., 0.}, {1., 1.}}], 
     ArrowBox[{{0., 1.}, {1., 1.}}], ArrowBox[{{0., 2.}, {1., 1.}}]}, {
     ArrowBox[{{0., 0.}, {1., 2.}}], ArrowBox[{{0., 1.}, {1., 2.}}], 
     ArrowBox[{{0., 2.}, {1., 2.}}]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->False,
  Frame->False,
  PlotRange->Automatic,
  PlotRangeClipping->True]], "Output",
 ImageSize->{184, 112},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[27]=",
 CellID->357478985]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A1.2 List Operations", "Section",
 CellID->23446292],

Cell[TextData[{
 "Since the data type CartesianVectors is constructed from a list of a \
particular structure, the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions for manipulating lists can be overloaded for CartesianVectors.  \
This has been done as needed for the application packages, so not all list \
operations have been overloaded, and those that have been, may not be as \
general as the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " list operations.  The following examples illustrate how overloaded ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions are used for list-type operations."
}], "Text",
 CellID->4393084],

Cell[CellGroupData[{

Cell["Get pieces from a collection of vectors.", "Subsection",
 CellID->69497392],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"First", "[", "uS3D", "]"}]], "Input",
 CellLabel->"In[28]:=",
 CellID->191503734],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ux", "[", "1", "]"}], ",", 
   RowBox[{"uy", "[", "1", "]"}], ",", 
   RowBox[{"uz", "[", "1", "]"}]}], "}"}]], "Output",
 ImageSize->{145, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[28]=",
 CellID->653815230]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uS3D", "[", 
  RowBox[{"[", "3", "]"}], "]"}]], "Input",
 CellLabel->"In[29]:=",
 CellID->703666199],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ux", "[", "3", "]"}], ",", 
   RowBox[{"uy", "[", "3", "]"}], ",", 
   RowBox[{"uz", "[", "3", "]"}]}], "}"}]], "Output",
 ImageSize->{145, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[29]=",
 CellID->320971388]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Rest", "[", "uS3D", "]"}]], "Input",
 CellLabel->"In[30]:=",
 CellID->87307636],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}, {
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{395, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[30]=",
 CellID->224807230]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uM3D", "[", 
  RowBox[{"[", "3", "]"}], "]"}]], "Input",
 CellLabel->"In[31]:=",
 CellID->442110006],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"vx", "[", 
      RowBox[{"3", ",", "1"}], "]"}], ",", 
     RowBox[{"vx", "[", 
      RowBox[{"3", ",", "2"}], "]"}], ",", 
     RowBox[{"vx", "[", 
      RowBox[{"3", ",", "3"}], "]"}], ",", 
     RowBox[{"vx", "[", 
      RowBox[{"3", ",", "4"}], "]"}], ",", 
     RowBox[{"vx", "[", 
      RowBox[{"3", ",", "5"}], "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"vy", "[", 
      RowBox[{"3", ",", "1"}], "]"}], ",", 
     RowBox[{"vy", "[", 
      RowBox[{"3", ",", "2"}], "]"}], ",", 
     RowBox[{"vy", "[", 
      RowBox[{"3", ",", "3"}], "]"}], ",", 
     RowBox[{"vy", "[", 
      RowBox[{"3", ",", "4"}], "]"}], ",", 
     RowBox[{"vy", "[", 
      RowBox[{"3", ",", "5"}], "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"vz", "[", 
      RowBox[{"3", ",", "1"}], "]"}], ",", 
     RowBox[{"vz", "[", 
      RowBox[{"3", ",", "2"}], "]"}], ",", 
     RowBox[{"vz", "[", 
      RowBox[{"3", ",", "3"}], "]"}], ",", 
     RowBox[{"vz", "[", 
      RowBox[{"3", ",", "4"}], "]"}], ",", 
     RowBox[{"vz", "[", 
      RowBox[{"3", ",", "5"}], "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`vx[3, 1], 
     $CellContext`vx[3, 2], 
     $CellContext`vx[3, 3], 
     $CellContext`vx[3, 4], 
     $CellContext`vx[3, 5]}, {
     $CellContext`vy[3, 1], 
     $CellContext`vy[3, 2], 
     $CellContext`vy[3, 3], 
     $CellContext`vy[3, 4], 
     $CellContext`vy[3, 5]}, {
     $CellContext`vz[3, 1], 
     $CellContext`vz[3, 2], 
     $CellContext`vz[3, 3], 
     $CellContext`vz[3, 4], 
     $CellContext`vz[3, 5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{343, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[31]=",
 CellID->3485984]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"uM3D", "[", 
  RowBox[{"[", 
   RowBox[{"3", ",", "2"}], "]"}], "]"}]], "Input",
 CellLabel->"In[32]:=",
 CellID->59296511],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"vx", "[", 
    RowBox[{"3", ",", "2"}], "]"}], ",", 
   RowBox[{"vy", "[", 
    RowBox[{"3", ",", "2"}], "]"}], ",", 
   RowBox[{"vz", "[", 
    RowBox[{"3", ",", "2"}], "]"}]}], "}"}]], "Output",
 ImageSize->{199, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[32]=",
 CellID->43783898]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Take", "[", 
  RowBox[{"uS2D", ",", "2"}], "]"}]], "Input",
 CellLabel->"In[33]:=",
 CellID->542072997],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2]}, {
     $CellContext`uy[1], 
     $CellContext`uy[2]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{203, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[33]=",
 CellID->548154160]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Drop", "[", 
  RowBox[{"uS2D", ",", "2"}], "]"}]], "Input",
 CellLabel->"In[34]:=",
 CellID->15858325],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{295, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[34]=",
 CellID->94194539]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Join two collections of vectors.", "Subsection",
 CellID->292283586],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Join", "[", 
  RowBox[{"uS3D", ",", "vS3D"}], "]"}]], "Input",
 CellLabel->"In[35]:=",
 CellID->94031565],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}], ",", 
     RowBox[{"vx", "[", "1", "]"}], ",", 
     RowBox[{"vx", "[", "2", "]"}], ",", 
     RowBox[{"vx", "[", "3", "]"}], ",", 
     RowBox[{"vx", "[", "4", "]"}], ",", 
     RowBox[{"vx", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}], ",", 
     RowBox[{"vy", "[", "1", "]"}], ",", 
     RowBox[{"vy", "[", "2", "]"}], ",", 
     RowBox[{"vy", "[", "3", "]"}], ",", 
     RowBox[{"vy", "[", "4", "]"}], ",", 
     RowBox[{"vy", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}], ",", 
     RowBox[{"vz", "[", "1", "]"}], ",", 
     RowBox[{"vz", "[", "2", "]"}], ",", 
     RowBox[{"vz", "[", "3", "]"}], ",", 
     RowBox[{"vz", "[", "4", "]"}], ",", 
     RowBox[{"vz", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5], 
     $CellContext`vx[1], 
     $CellContext`vx[2], 
     $CellContext`vx[3], 
     $CellContext`vx[4], 
     $CellContext`vx[5]}, {
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5], 
     $CellContext`vy[1], 
     $CellContext`vy[2], 
     $CellContext`vy[3], 
     $CellContext`vy[4], 
     $CellContext`vy[5]}, {
     $CellContext`uz[1], 
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5], 
     $CellContext`vz[1], 
     $CellContext`vz[2], 
     $CellContext`vz[3], 
     $CellContext`vz[4], 
     $CellContext`vz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{483, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[35]=",
 CellID->30317732]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Prepend, append or insert a vector to a collection.", "Subsection",
 CellID->234066801],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Prepend", "[", 
  RowBox[{"uS3D", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[36]:=",
 CellID->59182926],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"y", ",", 
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"z", ",", 
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`x, 
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {$CellContext`y, 
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}, {$CellContext`z, 
     $CellContext`uz[1], 
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{523, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[36]=",
 CellID->490854444]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Append", "[", 
  RowBox[{"uS3D", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[37]:=",
 CellID->473326778],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}], ",", "x"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}], ",", "y"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}], ",", "z"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5], $CellContext`x}, {
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5], $CellContext`y}, {
     $CellContext`uz[1], 
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5], $CellContext`z}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{523, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[37]=",
 CellID->128482476]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Insert", "[", 
  RowBox[{"uS3D", ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}], ",", "3"}], "]"}]], "Input",
 CellLabel->"In[38]:=",
 CellID->209839352],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", "x", ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", "y", ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", "z", ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[1], 
     $CellContext`ux[2], $CellContext`x, 
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5]}, {
     $CellContext`uy[1], 
     $CellContext`uy[2], $CellContext`y, 
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5]}, {
     $CellContext`uz[1], 
     $CellContext`uz[2], $CellContext`z, 
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{523, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[38]=",
 CellID->468341332]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Rotate position right or left.", "Subsection",
 CellID->70600885],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RotateRight", "[", "uS3D", "]"}]], "Input",
 CellLabel->"In[39]:=",
 CellID->257261906],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "5", "]"}], ",", 
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}], ",", 
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "5", "]"}], ",", 
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}], ",", 
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "5", "]"}], ",", 
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}], ",", 
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[5], 
     $CellContext`ux[1], 
     $CellContext`ux[2], 
     $CellContext`ux[3], 
     $CellContext`ux[4]}, {
     $CellContext`uy[5], 
     $CellContext`uy[1], 
     $CellContext`uy[2], 
     $CellContext`uy[3], 
     $CellContext`uy[4]}, {
     $CellContext`uz[5], 
     $CellContext`uz[1], 
     $CellContext`uz[2], 
     $CellContext`uz[3], 
     $CellContext`uz[4]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{487, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[39]=",
 CellID->333322532]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RotateLeft", "[", 
  RowBox[{"uS3D", ",", "2"}], "]"}]], "Input",
 CellLabel->"In[40]:=",
 CellID->59964444],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ux", "[", "3", "]"}], ",", 
     RowBox[{"ux", "[", "4", "]"}], ",", 
     RowBox[{"ux", "[", "5", "]"}], ",", 
     RowBox[{"ux", "[", "1", "]"}], ",", 
     RowBox[{"ux", "[", "2", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uy", "[", "3", "]"}], ",", 
     RowBox[{"uy", "[", "4", "]"}], ",", 
     RowBox[{"uy", "[", "5", "]"}], ",", 
     RowBox[{"uy", "[", "1", "]"}], ",", 
     RowBox[{"uy", "[", "2", "]"}]}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"uz", "[", "3", "]"}], ",", 
     RowBox[{"uz", "[", "4", "]"}], ",", 
     RowBox[{"uz", "[", "5", "]"}], ",", 
     RowBox[{"uz", "[", "1", "]"}], ",", 
     RowBox[{"uz", "[", "2", "]"}]}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
     $CellContext`ux[3], 
     $CellContext`ux[4], 
     $CellContext`ux[5], 
     $CellContext`ux[1], 
     $CellContext`ux[2]}, {
     $CellContext`uy[3], 
     $CellContext`uy[4], 
     $CellContext`uy[5], 
     $CellContext`uy[1], 
     $CellContext`uy[2]}, {
     $CellContext`uz[3], 
     $CellContext`uz[4], 
     $CellContext`uz[5], 
     $CellContext`uz[1], 
     $CellContext`uz[2]}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{487, 33},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[40]=",
 CellID->279662032]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A1.3 Summary", "Section",
 CellID->23564291],

Cell["\<\
The discussion and examples in this section provide an introduction to \
working with the data type CartesianVectors.  Refer to CartesianVectors.nb \
for details of implementation.\
\>", "Text",
 CellID->379619690],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["CartesianVectors",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/CartesianVectors"]], "TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->829829444]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["References",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/References"]], "RelatedTutorials",
 CellID->383413863],

Cell[TextData[ButtonBox["A2 Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Graphics"]], "RelatedTutorials",
 CellID->806871991],

Cell[TextData[ButtonBox["Table of Contents",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/TableOfContents"]], "RelatedTutorials",
 CellID->352542221]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"A1 Data Type CartesianVectors - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 59, 42.4134000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "Collections of vectors play an important role in implementing panel \
methods. Points defining panel endpoints and unit normal vectors to panels \
can be represented as collections of vectors. Aerodynamics functions for \
computing influence coefficients require the collection of vectors locating \
panel endpoints as input arguments. Computation of influence coefficients \
requires vector algebra between two collections of vectors, and the resulting \
velocity influence coefficients are collections of vectors. Defining a data \
type simplified pattern matching and programming for the packages. My \
original intent was to leave the data type \"hidden\" in the packages, but \
the need to construct the data type occurs several times in the \
documentation. Additionally, this introduction to the data type could be \
useful to those wanting to modify the packages or to those working on \
entirely different problems which involve the manipulation of large \
collections of vectors. Load the Aerodynamics packages. Constructor and \
selector for the data type CartesianVectors.", "synonyms" -> {}, "title" -> 
    "A1 Data Type CartesianVectors", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/CartesianVectors"}, "SearchTextTranslated" -> ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[610, 21, 78, 1, 70, "TutorialColorBar"],
Cell[691, 24, 191, 5, 70, "LinkTrail"],
Cell[885, 31, 2990, 67, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3900, 102, 66, 1, 70, "Title",
 CellID->509267359],
Cell[3969, 105, 1086, 18, 70, "Text",
 CellID->65021125],
Cell[5058, 125, 125, 6, 70, "Text",
 CellID->280757486],
Cell[5186, 133, 100, 3, 70, "Input",
 CellID->107795920],
Cell[CellGroupData[{
Cell[5311, 140, 68, 1, 70, "Section",
 CellID->152704095],
Cell[5382, 143, 259, 6, 70, "DefinitionBox",
 CellID->801716347],
Cell[5644, 151, 99, 1, 70, "Caption",
 CellID->104944398],
Cell[5746, 154, 289, 6, 70, "Text",
 CellID->63591597],
Cell[CellGroupData[{
Cell[6060, 164, 2111, 68, 70, "Input",
 CellID->101936636],
Cell[8174, 234, 316, 9, 54, "Output",
 CellID->148063649]
}, Open  ]],
Cell[8505, 246, 187, 4, 70, "Text",
 CellID->139029545],
Cell[CellGroupData[{
Cell[8717, 254, 64, 2, 70, "Input",
 CellID->203056159],
Cell[8784, 258, 1048, 34, 36, "Output",
 CellID->549800377]
}, Open  ]],
Cell[CellGroupData[{
Cell[9869, 297, 105, 3, 28, "Input",
 CellID->186875196],
Cell[9977, 302, 6486, 114, 198, "Output",
 CellID->414205713]
}, Open  ]],
Cell[16478, 419, 166, 4, 70, "Text",
 CellID->133331613],
Cell[CellGroupData[{
Cell[16669, 427, 109, 3, 70, "Input",
 CellID->228036416],
Cell[16781, 432, 682, 21, 36, "Output",
 CellID->376346877]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[17512, 459, 57, 1, 70, "Section",
 CellID->98413141],
Cell[CellGroupData[{
Cell[17594, 464, 104, 2, 70, "Subsection",
 CellID->578648157],
Cell[17701, 468, 338, 8, 70, "Text",
 CellID->110468743],
Cell[CellGroupData[{
Cell[18064, 480, 99, 3, 70, "Input",
 CellID->337721988],
Cell[18166, 485, 1848, 51, 54, "Output",
 CellID->204399871]
}, Open  ]],
Cell[CellGroupData[{
Cell[20051, 541, 99, 3, 70, "Input",
 CellID->716287734],
Cell[20153, 546, 2649, 75, 71, "Output",
 CellID->243454251]
}, Open  ]],
Cell[CellGroupData[{
Cell[22839, 626, 95, 3, 70, "Input",
 CellID->67522338],
Cell[22937, 631, 1445, 40, 54, "Output",
 CellID->99853864]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[24431, 677, 65, 1, 70, "Subsection",
 CellID->87368199],
Cell[24499, 680, 476, 10, 70, "Text",
 CellID->91427808],
Cell[CellGroupData[{
Cell[25000, 694, 201, 7, 28, "Input",
 CellID->87964621],
Cell[25204, 703, 2079, 42, 71, "Output",
 CellID->187093065]
}, Open  ]],
Cell[CellGroupData[{
Cell[27320, 750, 318, 11, 28, "Input",
 CellID->100168776],
Cell[27641, 763, 10499, 180, 179, "Output",
 CellID->5396401]
}, Open  ]],
Cell[38155, 946, 325, 6, 70, "Text",
 CellID->256903099]
}, Open  ]],
Cell[CellGroupData[{
Cell[38517, 957, 64, 1, 70, "Subsection",
 CellID->130895041],
Cell[38584, 960, 148, 4, 70, "Text",
 CellID->564988326],
Cell[CellGroupData[{
Cell[38757, 968, 100, 3, 70, "Input",
 CellID->1023740835],
Cell[38860, 973, 1244, 42, 54, "Output",
 CellID->62967172]
}, Open  ]],
Cell[CellGroupData[{
Cell[40141, 1020, 132, 4, 28, "Input",
 CellID->285324683],
Cell[40276, 1026, 5046, 90, 116, "Output",
 CellID->25903942]
}, Open  ]],
Cell[45337, 1119, 188, 4, 70, "Text",
 CellID->378189741],
Cell[CellGroupData[{
Cell[45550, 1127, 210, 7, 70, "Input",
 CellID->110286018],
Cell[45763, 1136, 364, 12, 36, "Output",
 CellID->8780141]
}, Open  ]],
Cell[46142, 1151, 189, 4, 70, "Text",
 CellID->113463184],
Cell[CellGroupData[{
Cell[46356, 1159, 131, 4, 70, "Input",
 CellID->221821086],
Cell[46490, 1165, 1148, 36, 36, "Output",
 CellID->509912641]
}, Open  ]],
Cell[CellGroupData[{
Cell[47675, 1206, 99, 3, 70, "Input",
 CellID->317318721],
Cell[47777, 1211, 233, 7, 36, "Output",
 CellID->84639185]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[48059, 1224, 47, 1, 70, "Subsection",
 CellID->337431051],
Cell[48109, 1227, 285, 6, 70, "Text",
 CellID->18165939],
Cell[48397, 1235, 479, 14, 70, "Text",
 CellID->435648623],
Cell[CellGroupData[{
Cell[48901, 1253, 865, 28, 65, "Input",
 CellID->256028157],
Cell[49769, 1283, 1619, 34, 63, "Output",
 CellID->412680610]
}, Open  ]],
Cell[51403, 1320, 445, 17, 70, "Text",
 CellID->732586039],
Cell[CellGroupData[{
Cell[51873, 1341, 1220, 38, 70, "Input",
 CellID->275335666],
Cell[53096, 1381, 822, 20, 133, "Output",
 CellID->357478985]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[53979, 1408, 58, 1, 70, "Section",
 CellID->23446292],
Cell[54040, 1411, 665, 16, 70, "Text",
 CellID->4393084],
Cell[CellGroupData[{
Cell[54730, 1431, 81, 1, 70, "Subsection",
 CellID->69497392],
Cell[CellGroupData[{
Cell[54836, 1436, 105, 3, 70, "Input",
 CellID->191503734],
Cell[54944, 1441, 305, 10, 36, "Output",
 CellID->653815230]
}, Open  ]],
Cell[CellGroupData[{
Cell[55286, 1456, 124, 4, 70, "Input",
 CellID->703666199],
Cell[55413, 1462, 305, 10, 36, "Output",
 CellID->320971388]
}, Open  ]],
Cell[CellGroupData[{
Cell[55755, 1477, 103, 3, 70, "Input",
 CellID->87307636],
Cell[55861, 1482, 1235, 40, 54, "Output",
 CellID->224807230]
}, Open  ]],
Cell[CellGroupData[{
Cell[57133, 1527, 124, 4, 70, "Input",
 CellID->442110006],
Cell[57260, 1533, 1887, 61, 71, "Output",
 CellID->3485984]
}, Open  ]],
Cell[CellGroupData[{
Cell[59184, 1599, 147, 5, 70, "Input",
 CellID->59296511],
Cell[59334, 1606, 379, 13, 36, "Output",
 CellID->43783898]
}, Open  ]],
Cell[CellGroupData[{
Cell[59750, 1624, 127, 4, 70, "Input",
 CellID->542072997],
Cell[59880, 1630, 641, 22, 36, "Output",
 CellID->548154160]
}, Open  ]],
Cell[CellGroupData[{
Cell[60558, 1657, 126, 4, 70, "Input",
 CellID->15858325],
Cell[60687, 1663, 776, 26, 36, "Output",
 CellID->94194539]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[61512, 1695, 74, 1, 70, "Subsection",
 CellID->292283586],
Cell[CellGroupData[{
Cell[61611, 1700, 129, 4, 70, "Input",
 CellID->94031565],
Cell[61743, 1706, 2458, 76, 71, "Output",
 CellID->30317732]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[64250, 1788, 93, 1, 70, "Subsection",
 CellID->234066801],
Cell[CellGroupData[{
Cell[64368, 1793, 188, 6, 70, "Input",
 CellID->59182926],
Cell[64559, 1801, 1517, 46, 54, "Output",
 CellID->490854444]
}, Open  ]],
Cell[CellGroupData[{
Cell[66113, 1852, 188, 6, 70, "Input",
 CellID->473326778],
Cell[66304, 1860, 1517, 46, 54, "Output",
 CellID->128482476]
}, Open  ]],
Cell[CellGroupData[{
Cell[67858, 1911, 198, 6, 70, "Input",
 CellID->209839352],
Cell[68059, 1919, 1517, 46, 54, "Output",
 CellID->468341332]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[69625, 1971, 71, 1, 70, "Subsection",
 CellID->70600885],
Cell[CellGroupData[{
Cell[69721, 1976, 111, 3, 70, "Input",
 CellID->257261906],
Cell[69835, 1981, 1439, 46, 54, "Output",
 CellID->333322532]
}, Open  ]],
Cell[CellGroupData[{
Cell[71311, 2032, 132, 4, 70, "Input",
 CellID->59964444],
Cell[71446, 2038, 1439, 46, 54, "Output",
 CellID->279662032]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[72946, 2091, 50, 1, 70, "Section",
 CellID->23564291],
Cell[72999, 2094, 224, 5, 70, "Text",
 CellID->379619690],
Cell[CellGroupData[{
Cell[73248, 2103, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[73316, 2106, 169, 4, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[73488, 2112, 157, 3, 70, "TutorialMoreAbout",
 CellID->829829444]
}, Open  ]],
Cell[CellGroupData[{
Cell[73682, 2120, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[73757, 2123, 155, 3, 70, "RelatedTutorials",
 CellID->383413863],
Cell[73915, 2128, 154, 3, 70, "RelatedTutorials",
 CellID->806871991],
Cell[74072, 2133, 170, 4, 70, "RelatedTutorials",
 CellID->352542221]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[74281, 2142, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

