(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    413344,       7424]
NotebookOptionsPosition[    407047,       7234]
NotebookOutlinePosition[    409529,       7286]
CellTagsIndexPosition[    409486,       7283]
WindowTitle->3.4 Flow Visualization - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 StyleBox[ButtonBox["Mathematica",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:guide/Mathematica"],
  FontSlant->"Italic"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"3.3 Implementation\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Implementation"], "\<\"3.5 Convergence \
and Timing\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"], "\<\"Table Of \
Contents\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Panel Methods\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/PanelMethods"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/tutorial/FlowField\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/FlowField"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/FlowField.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/FlowField", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["3.4 Flow Visualization", "Title",
 CellID->509267359],

Cell[TextData[{
 "Flow visualization provides a useful plausibility check for numerical \
models in fluid dynamics.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ability to work seamlessly with symbols and numbers is used to write \
functions describing the flow field for uniform flow over an airfoil.  ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions for function visualization are supplemented by ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " functions when the conventions used by the  aerodynamics community differ \
substantially from those used in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ".  "
}], "Text",
 CellID->823686949],

Cell[TextData[{
 "Load the Aerodynamics packages and specify an arbitrary field point, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["f", "TI"]]], "InlineFormula"],
 ", as a collection of one vector of the data type CartesianVectors.  This \
provides the input expected for computing influence coefficients in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " and yields results that are functions of position instead of numerical \
values."
}], "Text",
 CellID->34146804],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"<<", "Aerodynamics`"}], "\[IndentingNewLine]", 
 RowBox[{"rF", "=", 
  RowBox[{"MakeCartesianVectors", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "x", "}"}], ",", 
     RowBox[{"{", "z", "}"}]}], "}"}], "]"}]}]}], "Input",
 CellLabel->"In[0]:=",
 CellID->35199140],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", "x", "}"}], "\[UpArrow]", 
   RowBox[{"{", "z", "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`x}, \
{$CellContext`z}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{55, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->16583519]
}, Open  ]],

Cell["\<\
A member of the NACA four-digit family of airfoils is selected as the example \
for this section for two reasons.  First, by changing the identification \
number of the airfoil, calculations can be made for many airfoils including \
reference airfoils without thickness (camber lines).  Second, discretization \
is based directly on the equations defining the airfoil instead of a spline \
fit to data in a file.\
\>", "Text",
 CellID->972502991],

Cell[TextData[{
 "Any of the three models for potential-flow over an airfoil programmed in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " can be used to obtain a solution for a member of the family of NACA \
four-digit airfoils at an angle of attack.  The model using vortex panels of \
linearly varying strength, and Neumann boundary conditions is chosen because \
these airfoils have an open trailing edge.  Using ",
 Cell[BoxData[
  ButtonBox["PMVortexLinearNeumann",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/PMVortexLinearNeumann"]], 
  "InlineFormula"],
 " avoids the minor inconvenience of altering the airfoil geometry to close \
the trailing edge which is required when using the doublet models."
}], "Text",
 CellID->70657603],

Cell[CellGroupData[{

Cell["3.4.1 Solution of the model", "Section",
 CellID->314388262],

Cell["\<\
Specify the four-digit identification number for the airfoil, angle of attack \
and discretization parameters.\
\>", "Text",
 CellID->75137487],

Cell[BoxData[{
 RowBox[{
  RowBox[{"id", "=", "4412"}], ";", 
  RowBox[{"\[Alpha]", "=", 
   RowBox[{"5.", "Degree"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"n", "=", "50"}], ";", " ", 
  RowBox[{"layout", "=", "HalfCosineSpacing"}], ";"}]}], "Input",
 CellLabel->"In[2]:=",
 CellID->108183604],

Cell["\<\
Compute the location of panel endpoints for the discretized airfoil and plot \
the discretized airfoil.\
\>", "Text",
 CellID->731882235],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"id", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"n", ",", 
        RowBox[{"Layout", "\[Rule]", "layout"}]}], "]"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"gA", "=", 
  RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], "\[IndentingNewLine]"}], "Input",
 CellLabel->"In[4]:=",
 CellID->401362744],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c9rFHkax/Fe9i/ZfyMgcxAP4iFkQTyIF4VGL+NFL3qJl92LQpNcEpDJ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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->346796653]
}, Open  ]],

Cell["Solve PMVortexLinearNeumann for the example specified.", "Text",
 CellID->19157149],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "vC"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP", ",", "\[Alpha]"}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[6]:=",
 CellID->450780366]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.4.2 Stream function", "Section",
 CellID->1708000],

Cell["\<\
If the stream function is single valued, it can be used in a straightforward \
manner to plot streamlines of the flow.  Such plots can be useful either for \
viewing the overall flow field, flow near the airfoil or flow near the \
trailing edge.  Contours of constant value for the stream function are \
computed and plotted for the interior of the airfoil as well as the exterior. \
 The absence of streamlines interior to the airfoil is indicative of a \
plausible solution for a model using vortex panels.\
\>", "Text",
 CellID->35129489],

Cell[TextData[{
 "Use the stream-function influence coefficients and the strength parameters \
from the model solution to compute the stream function for the flow at a \
field point ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["f", "TI"]]], "InlineFormula"],
 ".  Extraneous list brackets are removed from contributions to the stream \
function."
}], "Text",
 CellID->48905279],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", "\[Psi]\[Infinity]", "}"}], "=", 
   RowBox[{"UniformFlow\[Psi]", "[", 
    RowBox[{"rF", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]A", ",", "ic\[Psi]B"}], "}"}], "=", 
   RowBox[{"IC\[Psi]VortexLinear", "[", 
    RowBox[{"rF", ",", "rP"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "\[Psi]A", "}"}], ",", 
     RowBox[{"{", "\[Psi]B", "}"}]}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ic\[Psi]A", ".", "a"}], ",", 
     RowBox[{"ic\[Psi]B", ".", "b"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Psi]", "=", 
   RowBox[{"\[Psi]\[Infinity]", "+", "\[Psi]A", "+", "\[Psi]B"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[7]:=",
 CellID->17324249],

Cell["Display streamlines describing flow over the airfoil.", "Text",
 CellID->275199393],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "0.5"}], ",", "1.5", ",", 
     RowBox[{"-", "0.5"}], ",", "0.5"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", "25"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"g\[Psi]", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[11]:=",
 CellID->666894011],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJx8vQeYnkXV/0/xxS7IC4hSRKQIiCAgRcBBRF4BQUBQUYog6ksxVBGQEiki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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{6520, 8083, 9760, 9759, 9758, 7558, 8432, 8431, 6839, 10168,
           7046, 10167, 7045, 7903, 7902, 10653, 7710, 10053, 10054, 6838, 
          8430, 10555, 10165, 10166, 9939, 9940, 9941, 7085, 10203, 7086, 
          10204, 6861, 8518, 8519, 7944, 7943, 6736, 10686, 10683, 10684, 
          10682, 7778, 10685, 7777, 8523, 7155, 8165, 7026, 8072, 8038, 8039, 
          7030, 8881, 9909, 6725}]},
        "0.559`"],
       Annotation[#, 0.559, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{6728, 9910, 8886, 7034, 8042, 8041, 9257, 7028, 9256, 9255, 
          8576, 6885, 9259, 9260, 9258, 7031, 7870, 7869, 9254, 7027, 8875, 
          8876, 6722, 8036, 8127, 8347, 8348, 7787, 8533, 7425, 8349, 10077, 
          8942, 7511, 9630, 7160, 10227, 7161, 9358, 7013, 8428, 8429, 8070, 
          8069, 6750, 8346, 9938, 9937, 8911, 7423, 10425, 7427, 10424, 
          7426}]},
        "0.516`"],
       Annotation[#, 0.516, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{6500, 9750, 8569, 7125, 7959, 7958, 6729, 9113, 9114, 9112, 
          10127, 6949, 10126, 9263, 10128, 7032, 10836, 9262, 10761, 9261, 
          8577, 6886, 9265, 9266, 9264, 7035, 8726, 8725, 9109, 8723, 8878, 
          8877, 6723, 7956, 6724, 8880, 8879, 7029, 7868, 6617, 9233, 7016, 
          9232, 9235, 9234, 6942, 9087, 6941, 9106, 9591, 10849, 9592, 10850, 
          7424, 10422, 10423, 9594, 9593, 7118, 9340, 7117, 10218, 7952, 6711,
           8008, 8009, 8345, 8344, 6933, 9342, 7120, 9341, 7119}]},
        "0.47300000000000003`"],
       Annotation[#, 0.47300000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ktvzGEcxfEjUSyEtNqKTjv/mV5oK25h4w2wl3gDU6Ul2LIoOhLt1F1Y
eRcu6bRuL0Es3WKB1n1BLBDiM4tvzu9yzvN/5mmrtZMHTqxIchvNnmRtX/Jt
JJlU/yySv6WkYzT5R+cwaz+in6HfK8kE31f+5/ox9R+edvvf9Id8zeyTfZv9
4/5kwO4RvTacXMcNHBtKPvQmR+lHmV32O9Ghr5t3yzzUL25O+tUP1JflruAq
JvmW+SZa58hvtR/Fev1Z8y6ZRf2CfEVdxUW5S1iyP8L3nh5unSM/xDuIdfoz
5p38C/qmfFldoCE7h3f243xv6SG6JF/hLbBSP2W+gb+pn5efp9sHfGuL72KH
etkbbaN7BpPdeLkpKfH14C7uYB9Pl7P30/v6ezjtjZ+Ukxf8fXLH9b20hE77
SuvvpW7gVNXM98edUZifN3tG6/SLe3xGu91e559zl2k0zGaxkW+GruKtqdvo
GF1N3/D/cnbBX8Yas6fuc8HbvPLbD3qD17Tuf2Uat9zrJv4DHTJf8g==
          "]]},
        "0.43000000000000005`"],
       Annotation[#, 0.43000000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkTsvg2EYhu9i0a9OCaH09PWrHhKHH6AtVmHQ1eLYoEkbA4OS1ITEQoXB
8QewIAZWRolowuJQ4ieUiBhcHa7ceZ7nfp/7fb/PHM8k0jZJGzDilPq90nxE
WoCbdunOLd2iBZeU80mnQWndLw3RG4RfU/qBxjZp0ZKyARQG2JPHtwWbMMn8
0iPdk1HLvA58+Iut0h/n31GDngOqIYnfjg6zJwEp6gP2HEIP3ml6UfSI+hiW
2bHNPhNfHm3injuon3oNtVBvORM8kKF2cq4FmiFAvYtvhvee8S6BDVzMJtj1
zL2LZL6gHnpuqGReAedgBwPM8lvxv+L7wP+GWvT8UMPcARcwS84eeUFyg8ws
7hSAOeoIdRhCZaj38aXw13Mux3/ooD9FRhI+yeim7oJOWGHegK8KfxZdggf6
BRjle5Xwx/i3cYhCH/TCKjlfIekbSnAdlq7AYM8Y557IeoQ4sxic0PsHAbBN
1A==
          "]]},
        "0.387`"],
       Annotation[#, 0.387, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkT8vQ1EYxh8DauiE0lZRlPAFapH6WwmJgYEIShtEWrelTL1tF/Hndq9v
gMHGhJ1FQvgEFjFI1IColp/hl+ee9zzv855zrjccnzQqJO3AZoeUhGi7ZHdL
z17pBcZ80jjst0mnndJVi3TN/gbeGzSOJwGzTmmUvRCaIsMEAxx4iugM+o1O
o2X8JShChloasuRN0N/rkg6ZlYdVvHbqF83SJcRYN7J2gROe8Jpk1PFt65Hq
0Sl0DZ/VJbk5r4NaA+wxIwcW9NFXy6xXPA94d9l/RGPUohDhDj/USlCAXyjD
G56BVmkQUh6pHw3AMv4q3qgaKqEGbFDAP8J+EEz8w+gQrOBfZ44BCTxH3PWM
s55DiFqA8x1TO4EM95sjK8i9kvQ14ZlnvQAfnOkLPuGdnAOyLUgzK897beHP
sV4kcwn81CJo+P8/MeOO/HvIMsNHxm23tE2Phxl/DmpSEw==
          "]]},
        "0.34400000000000003`"],
       Annotation[#, 0.34400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkbkvw3EYxp+GGDR6TW5+vZXJaHAMGEgklTgHFIuhjWumEmkFg4TEtbgG
JBIW11+gk05MrkrckWCvT4dPnvd4fu/7flsjFAmGTZIWoKFQqnRJVeCHAFRA
m1u6LZAcTskOB7AP9fhDHmkQ+mAA+iFVJt3gt+Gxwh7sQi1+L/N84AIPuMFg
vhMsePJgidqij5tgHnaonXilkoC0TRzzS3F4KpZ62feI9mSUvWb6uZAD49S7
qT+gXeg9ulIqLcNbuXRHHke/i6Qf9o+R1zF3FO3A3wlbzDlmt9htgiHuGYZn
3uLkzhRqoGFmZOPNgjnytCGtEa9CDR4Ht20Sb8A6jOA/544kv1OU3gyckZ9C
jDif/jQ3RWGCWTZusEMLvVZI0J+kd4W+cI+VWjV7jvBE2G+lPkvdgjbyjiaI
Zf47+odwgf8Skry1md41OsXbE9zzyfdf8AHv8Mp+M3P+iNvRXzSI/gPRsVVi

          "]]},
        "0.30100000000000005`"],
       Annotation[#, 0.30100000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkLsvg2EUh38tgqQTA6VC3YWNoaUVI4NLikWiqkiohIW6tMSlZWnLSPwF
TIz+AYuEpC5RoqNIRBe3lMXzDU+e95z3nPOe77P75z1zJkkJ8FuljirJBasV
Uid2Q6Zc+quVwjYpi0P4FxfXS07uV4hfjR7cjt+p/+J+mTiDl/AnLqLewf0i
8QsO4nSl9Ax+7sbBUi0tkP+gfpod9mskG/MKyBfCPTvmcneHfdTncM4nv0dd
AnaJj+qkQXofcAqueesKPOT8TdIEeOkdg1vmOOm5wU/kHZzzmBdnVgyixFG8
AybyZkhSaybvYy8TlgF5K3GEcxIH7ezI28fMPIEt+rchAim+9xEGoB/ayqRe
3IrjzInBCDO8zDrEo/gAf9M7ZPxbPIx/cJqeN94q5VwCWbhkv41GaRO62SHJ
/+zBF/Sc884k/+GUeAqfYQv5GWO3BimA13l/DVqY3UV9Mw4Th6CPWjc5F8zy
bgD+AXx4WMM=
          "]]},
        "0.258`"],
       Annotation[#, 0.258, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ctL1GEUxvGnP6OZ0bIbJbRokVREiyBolaOOdtGRatFKM7oQVJCRFhhU
2s5sFUG1j2xpELToAhFpRXRxzPJC0MJaRR8XX573nPec55zf+2s42td6fEWS
u9hcSHasSi6tSfpxEeVi8qg+eb4ymV2bNIu/05/4gU96zjUkX52/oYZpLG3k
V5cMrU+6G5OXpeSC3Cs6bsb0puQxfbAhuY/LZr12t1v9Ar859PLpwel1ybD8
FrOuqRtChz3+uivqGXU/htvolJ9YnXTRfv4Tap9i0nkKW+20l/d133QDTeIR
3i16W/GPZ1XvGTXdtKJnxq5ty73mt9OauIPOuW9Ut53PFx7b6Dt62Fudld+v
pqDmnn2K9ID4mV2OmfNQvOg8pv4OKvao49tO62kLHZUv05L4CM+D+g+hJNdF
O9Fkpyrtxguep8w9jz3evKrnJAredx+vPrlmOm/+IhYwr2fAfoOYQQ03fccw
3nrbsvqqvh7nXf5fL/213KvvitpJ8SwdUX8LU+I2Pe9phZ6gv+2zhD/4bM+r
6j/I7xR/pIN8B/DEd47jDZ//SNZxPw==
          "]]},
        "0.21500000000000002`"],
       Annotation[#, 0.21500000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ltMznEcx/GvG8yhGS48T4UkUxe4UzZjJlvmsLrhomYOlZlquKCYNrlD
NXOoKzZuncatsrEeI+ZGbTnklJyKpHJjeT0X733+38/ve/r9nidnT11Z7ZSI
eIHq7IjViyPqsiL68iL+5ESMoWdRRMWyiHKMimud/1oaUSW/PjdiXqaYP4Ln
iYjfvGe0L5/n+yf/GD2KenUb9StGjz6l5rykZbTJ7Nf8brXDaobQuiTisz4t
tE7O2uXm0zY6pK6Ln8Jj/FO7PRnRac8OXJZzCbvtN6b3OE7LacJxed36NNAv
CyMG3eWO89s4wcuyazYSyEQS1Xbfj36z3uINzuu9k9erR6N+c/Tp5T/x/RQf
xAPOPmGu+yTEeQURH+kgr9Pd9tIue6awTk6J+JY9buKVXbfZ55GeD3FRzgXU
6N0s3mX+pN1a7dCCk/wj8kf0fW/GdT2uIddbzaeFvEn1RTTkJHFF/hbxVToq
XuP7rx3axQvSO2NCPI71vm84L5dXgWLvvAmz9E6ascP5QXkZ4tlI8H7YaQgz
xTMw7PuAnO/0W/q3NOe+e3agiv+Vd0ifw9jAq0n3TP8X+A/ct4WussOEe6yk
ld6zCvvQ7OyUdymVN+xd2r1JG6aZOx0r5J+Rcw5nkW92hhkFtEFdPbaqHVB7
1y73UCLuF2+m7+hU2qVXCkVqC9GI/w0xju8=
          "]]},
        "0.17200000000000001`"],
       Annotation[#, 0.17200000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0klszVEUx/GDGrpRZWOoPjW1RSLEsBASKmJoDTVHYkFi2FAzCxUtbSUl
QUtiWCiJKSkJSxULYmhLbbVa8yyISChtfN7i+37nnHume98/a83mwk1dIiLV
z5OBEd+GRvzAd9Tw7w2JOE/HZEe8FZs6OKJqhNzMiCZM43fPjeiJHqgbFLEu
K2I9NmBKImLFgIiNyZj8m/wbOKpnqt63UIcZzqdjtH4V+h9CGcbwy+kZPS6z
V5lxhR6jqfo1OutFE/bLxGCMd/4nJ+KX+G86jj9IPAPHzb1t3jE6kD8A1ahC
rt1P0FH07LCI4exT8tLkp6MvFtpxATIyIvrxX3uTJjs8xXzzWmkBbaMp8rJw
X4+u7rxRvJBfa59u9A+/HRfNWzM84hJdSx/Kn+R8Mq46v+Du/d3jPJ03MiIf
/+xXa7+VajrYuewcTNa7U+51OddQyi5BmX0n6ldOZ9u7C51DK+hPWqnutFkl
Zszll9IjYoeRr64Ar5xvsVs6eyt9rO8TNOIpmnBHvJ424AEe4SEq9HvrTd7h
DT7gPQr1/Ott2pFilw536ES23XNQpq7S2RLvshSLsRzLsNesTmfN+rfgGTLt
+5y2olT/gziAhLwGfc95r3paQxtpvR4X2L1HeV/6Vf0E9hfazz0/00W0w56d
yEK+b++o+Dw6HwVI8NvMfIkX6OMu1XZMo935/8xv8a00o9Y+11CC/ag394Nd
PmK3ml3oI3Yy+d3RPLV13mKL/7kvv0xeETtNrDdWe6Nt/O3Yip3YgbHuNktt
kX531W2m92ilOxSpKbZXnvkzk/+Hs0/4mEROsdn7sCf5DST/d/mv+f8BY6ys
ew==
          "]]},
        "0.129`"],
       Annotation[#, 0.129, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV011M11UYwPGHF81aKfEi8mIKvovCH0jUqWsBc+rmjVfi2ny7EL3Q5aZX
qbO0ItZmsQRbChjqVurUDbrpQgRTWmvMIvXKzVdsbhpXaqJ+fhdfvr9zznOe
5znn/CnZtGPN9rSIqPEnVRjRMydi/XsR3byBf+HlvGdqxCcYmhIxUBBxvyTi
Hj4vjTiIcnvzpkVMRDZykYPHeIK7Yu+g0fdnsyI+xX4csPe7GRET1MlCnbl6
nJdvRK1zXDM9YtjaQl6AXeJ342VxxJ/2X/HdODdiG7bi+mR5xd/gq9bG86+z
I97hzdYHzH1p/QvscJZm59tsbb5xBcrRhp/V7uVLuIh+9KFRfJveWtFs/9fO
9a/9r2a6SL2n4YK72md/hu903FH/Nl6I+R/95vrwhrlcOfeKfWl+FL+Zv4xB
czetT5b7qflneI5X6r7r7GOtZXOz8VcJehmUq0X8t3iIaucdTnqTrx0d2CJH
I+6Jv2J8l1eIa3C3i93nSW+0y/gE7zb3lzp/4xr+wVByl+pmcCYq5EqhSZ4m
fVT7rkIlDqm9Sk/jxd8QW2R8nQv5G7nTzR9XZ36Rt1W7Bgtx2Piq3lp5gOdx
h7gy7uQWb1ivx4/UTJd/iXxpnImM5M3E1XmXXu7RUzd+EttirkvNW2LGuN91
3imTc/WUl9xnct/WczjLWp1eavEh2s0Xq/tIfKF8PxhvEHeUN3I+v82TuNbZ
61CPY8aF5mdwAc/kIp7Ok9QowFq/qbN6WsenuYHP8Ck9t6q52vk+dt6dOGK8
ktv4d738KC4l/jhXclfyO+ZMPWbgmJgq41p7qjkLy3xP4CL38Yf1Ur0M2dep
3io9d/BKXoESa/n6ncqj/L5eXnA198lxCePs/SD5H/JW5eiSM9+5uvXYg/Xe
+ntvcd89PsCb4t9K+hSbQiWy5bkmNodbrZ2XbyOf45P66ZfvBJ+2vljfi7DU
2kTzeSiTYy5q/D5G1F/A//FrXkG3sg==
          "]]},
        "0.08600000000000001`"],
       Annotation[#, 0.08600000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ldszlEcxvFja6kROt6i1dpuiNGardGktCohsXdTMTuUWImk1oUSdy5E
i9SWIKi4kZQgZmpEREJsaocYTbjxeS++73N+5zy/cc7/TSssnVrSJISQ6WdB
WggLMSg1hMHI6RLCyvQQitG2fwgltAylSE0Ooap3CGN7hXCzawhj6C26ul8I
a7AKt7uFUE578daniJ0fkXMUx7Cxewi9nR22PoTBffTGMXs3nR2nQ8VDMEat
HH1b2pvrbBf/TlTihrMyfar12I+j+hT1VYcupkfoHDknaJ77HaeJ7jOD9yRq
IiEk0Cx7iTQeEWSLk2hfPe/oMUB8ly5CIW7oOU3uArVn0XLeDMwXz0Od9+ug
f0fEoT3aoZo3yXtFkIguSMY+c12mV1CHYvVXYjP/Wt8jxV4NTzd6iM7SYyaa
ma/aG+3HPgzUt4oekJfrfVJ5E/RNofn8581epG4tLRBPwhveiL08e0k0nza1
3wSd5LWWH4sY7FF/dk/fAXNQqf9Fvh20tb6t8JXvC6q8R6V679RrMMsHfMQJ
/s/0E574rvHOH/MWqDHOuo2c8bQt/aDfMLpeHOG/Y30Xpd4kPfrNrXvQAzSX
J5FOpPF0Ak2gFeqmWb8zUzqtjb4DWpq1BZqju/23zg+qtVPOe/FP/X7hN07x
b1Cn0foPrvNewyXrs/LPRf+nyMZpvtv8Z6L3EY9FOzXjcFn9b/L6qB9jPxZt
onNE50Fn752p5jAMx173HEVHYgRmqDEdu/nWiddjA6ZE39R3HO09s/Dcf71C
3Ym+UR62esdX9l7iBVqJ/zr/hzT56ZjEt81+Pp2MApS6Q43aJfQZz1O0N/t2
1JtjnZ6b/CdjxdvU3YotaFTnh9m/476ce6jHBXevxVXnS3r4z9DX/Iv1emCW
h/hs/i/Ilhvr/BGNoVk0To33vEvd/y1dRhtog/MMc64Qv6bL6dDo3Gr/B0bl
tnE=
          "]]},
        "0.043000000000000003`"],
       Annotation[#, 0.043000000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1HlM13Ucx/E3XmtqqQvbAjKQPCsLUkujtg6sVQOWmhx2rLUSVAiPWlBt
GcgsNDQBj80iBe3YWra2trRr1VJR/1FqhWWuTG1la+YqbfX4/PHk9X3f78/n
+/2R93DdvbUZEdHgT25WxLWXRywdG7EE9ZiVHfHOmIhPL43oy48oZh+mX+Mr
9Kt5KjeiSU4zGtU3oANtcl+eFDGUP3dCRB7Wy/9Ifjs9pn+m2J9yetg5eRHd
tFZeHRbhd/Zi2qNXy/iIldjBnsYunRxxhL1Mzh96rOabgq32uoZuo2VyT5kz
UO6v6Xx8K81pwXQ5xepm4Vbcjttw1u67055qP8aHWCP3QbGH8ADeviziAj26
5J1hj3KOziucG1vMeAVTzS1j32H2MffXqscuvXZjAWpQjf/EvtevUN5R2o/v
UJDOR5fbZ4ucQ+a8xX4TB8dFDDfzAB1G+8w6jMFqDtEN+m7EJnRiu31eo4V2
qHFnC1Ekb5D835yhQLwQq5yp2azXPTfRTP1H42J05kRcQjvoXO+wWY8mbGDP
n2hHe82140CxQbjTXv2o0L+dfdo5Hkc95pgzG5+Iz1Y7B4v0WowT5v4lp5Jv
vd4VtJ1W0TNiA+gwdgYdSv9OfdXVoRYvpv3EfjG3nK5jN/A34mmEXd5Dvn1L
7HBCzs84jkfscxLPqe131rs9f0vvoUfoXUlRKb7CWavoPj334l31O1Htuctd
F7jrIe63nn3S3iVmnqJL2UvQaq/75G+2xzy6mr2MfzmeQKn8MuwSX2DWB7Sa
PqPHNv27sTW92/T7wEK7jHSeGnpa3jfyz+MGO5yjG9W9xF+Rfm/scrqdVtIV
dKK8nfR5zOBrojNpi36Z+o7GWb5WvjXp94b7xTrsOJ/ut3Mv+lKP1NP527yb
fL7z6Ttnj6AjMQoZ+g3ADvk3ybkZm+zYpnaMui462V2/Sq+k+XLH4kuzrmJ/
Rv8R69b3c8+5Yl/Qc3yTxH/0fW9mZ/FPMG88xuFCsaNirWLv2/2i5MNw/GuX
ZvkN9lhrh3W4Jd2h2AzMxB41b6i90XNROo97G4E95u5N70fNs5hizjzxclRg
f7onWoVKlOqbLf9qeb1iJexiswerHYKedDZMxTRch+sxHft8K9lm5iALj7KL
fEOP0Z/8zznuThr17GX/QFeZ8QKeNON/r6/q3A==
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1FtM13UYx/GHWhNSa9VWSxNTAamLTmA2/pK6QnO5oehqHdbKwjYtta1a
a5qk2VyUCP+/gAfEs7byIi+60DSzqR3dPAPplnLlCVBXnOzw+l28+Xyfw/d5
nu/3x/c/Ytb8inlZEfG+P8fvjyhCMcpyI86OiDh8L//wiEP0pYKIM3zteRHn
sTNfrtgYlMvfhYPyFrJflPsCUoURlfdFvPxgxCujI44MiWilbTiFFpxGjj3j
1CxFCiX4YGTEs0Mjvhbfia8wlb3sgYhP0aJ+K9qwwcwbkTMsYooZs+kM82zE
F2aaLr8Cj8sdg2I0yumzpxdX5UzQ7wDfx2JLsBSD1eoWO6HOSTzJLsVU5xg7
KuI6rWXX4Ud7z+jRL/829u342Z6fUMW3Q/5iuob9qNgj+M7+9c402P000Rq+
Fcm9y+mTW2LdSy/Ku0nOBXpIrIevWGyi9ZdYxf6F1tPZZngTldhtpoHybkXK
+UowXF6v2B61cvl302q9u/jesu61Zzm9W+we9LDH6t1Nh/lO++lD7B9oA8rt
q6fTaI+77DLDn3qcQwkeVqORrkaRPllqHFD/6eTOaIZvoHpp2qTOOtxQZ5lY
l3UnvWbvJ7TIngnWO5DRZ51aabqdPR5X0YW/+AbJ/ZtW+Y6L8b1aW8Vq+dba
t5JuYZci2P/RzuSu6SWksAkrkm8mXkM3s8fhCi5jKH9Hsi/538A21Mmr4W+x
bsVk/IE2XBOb6WzX6UdmWoS097HBbMfdXyN/k70N9Bi7mf9f+f+g2vn7krsR
OytW7L6a1XzMOd8Ta0/uwPftlpuRtwpH5X0uv9N6kbc4DdNRjs/0raADaLee
2TQHV+TPUKOD1rIPqruSXmZX8NdYz1V3n5pz6F76rv0FzpKPPFTzTRQLuXfR
Sew7aBm9ky4w70W989kXaAG9RPPot3osEe+wLnTGWu8+jTos5e8Xn6nG6uSN
0zXJWfXPYK8akzAZZZiCZ3CzOZ6z73m0+906j+Xu7dfk26uZss6iv7GfcIdH
6Wh2If8C63ewzyw5zpaNAdjD3qXnN2hmr0cTDuv1oZ6/J2+Y9mOk2UehQc39
Zqinab9xGdRhkD0n/D48peYc/eciV06VOeeJV4rNxinznE7mth4v91V59Xq+
wb6F/Rp9HW97T7NoK/4HYD34LA==
          "]]},
        RowBox[{"-", "0.043000000000000003`"}]],
       Annotation[#, -0.043000000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV01tMz2EYwPFHqBnXzlR0JEYmcxilC8yhrRFu4w61MRXhBjmWnK7MyGFz
uqWQTGKuHC8MG9mcLoiSqTl+uvju+z7v+7zP+77P7/9PLS0vLusXET1oGhWx
YrwB30iNaMaRjIj27IijXJkVUYFjxsdxITniIhrlr5K7GkUjIqaaK+a7coeO
iWjlYXw5JeISKkZGvBgXMdxchzpf8Q1XrFVae2ktZWzE0syIZShCstot1hYb
L0Er7uCzfV9wyh1O47ac/XIP4CAOm5tsfRLWyc/hLnSiSm63Mxd63yIswCF3
qnP/WvSMjkibENHLO5HrXtPQ7m0d1hfY25QecR2NuIkb2KfGfjxSb4fcV31j
TFHrMT/EFmtPuZKfcBW/lXvL/jZxM9/jFm7gf846wxPUOMt/5Z7nieJzHN76
x9xrde6LP3lbmXE59nrrPiSYGygvEUn4pccnzT13Rr34mXixONu4xngPNquV
q8Zp7y/1HRt4De/iQt7N+TwfBdjiDr/1/Kde96IH78Qd/BXD3eOBeuvljdbD
jdiENmd9MDfI2R+5ybmTvO06X0UjcsTXeI/1Lp4u/s6d+IE8cTfXeEOmegXi
LE5DBvLF6Vxi/z95M8Uh/mucwLPE/bje/kSeI07i/hiI2eIBfML+FVwoLuHl
qPN9atGhH9v7vju2YhuqsdKeFE5FMsZjHEqcnclZSEMG0vvW3SFPX2agWf/W
6stgvRuCDWpV6/Vc320e3qT57aAd7+VWukdV3/9M7b3uVia/XfwfHfGgKA==

          "]]},
        RowBox[{"-", "0.08600000000000001`"}]],
       Annotation[#, -0.08600000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0klIVlEUwPFTUIsiq0VBNmqm+dk8aMMiJUlpWghqOSu0CwtSq43DMrBF
aJERpBaV1aphER+oZTSRBLUpG6AWgVq7rF30c/Hn/+49557z7nkvreFESeOM
iHiLjNSIB9kR1Ssi7nMNP+RCblkV0Yx3KyOeLYn4lBYxhtPpYkh3dsHqiIWY
h/lIwVeMyP8o9wOa5X5Ts4m/c5pz/ZkRtYmIyWURXWsjfvLxNRHj3KlnUnxY
/oT1lHhiecRvzuYCsXzswWLxufYXcS1f4Wru4Roe8B55XGmdy1W8g4/wdq7g
rXyUt/E5zlE3gWzMtg77s/gQH8Z+HMQB3FZ/I2/CBmzBZpTLX8frp98ZOdN3
QCFanLloDpfQjRSzHnHnp7hqNkUZEcXYh3rU4XWW+Ztri5l95jGc8nwSxUsj
vlg/N7/35jOuZrk6k+q1W2fJ6eA2eW2cad3KE+KP5HXYn2OvnfvcuRd39Cuy
LrGf53lAXi7XWf91t+vucQOv5JaJ/fKPdHrP89iLVmdHfb8m79Qnrx8v5L5E
qfwuvbvxR61rYlPcpHav5zJuRDnq/T8NqMIP/Su5Ao/Vv6fnXfSoeRnH9Nut
3y4Myikzk5nyzjp3BrXiF+Sl6n/TuVsY1nen/CFu1jvp2ww79wRDGERSrX/O
jHouUO8N5/N/qYOCHg==
          "]]},
        RowBox[{"-", "0.129`"}]],
       Annotation[#, -0.129, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0btPk2EUgPHDZCITF0VEVLwAMnjbdGRRo4OoiS6ltIVJAgObl6ijgxon
/wJtRYWaOBFFBWEyKkZdCBSksHpBE6Kt8uvw5PnOey7v5WtJD54ZqIqIEZxs
jPi9N+JUR8Ta1oihtohjuyNO4DhqrFXviChYX8DFPRHPmiL6+b9ch9w//UW5
ZZR9T+2LKHGVuhctEZPmvME0pvDcnvntEa+3RaR3RWTQi2Gzmsy81xqREl8y
7zKu4Ly+CbM67XvV3Lfy0+pK9iqjR00KX/Qn+Z360+70nru4U88GbtDzwdoM
EkiiG836Ru2fx5jahL260eWsGT7LD9ojsriPh8jhjzs84REM4zEe4YJ5eS7J
j3KZn/JfvuG8n5sjrvEnvs7nzH8pXyMe51rO7rQnFpy5gHnUWX8lv1J5P64X
F60vYwU59Uv8FYsoyt80f0D9N/Xt4lrvU4d6jKm/5aybfW/CDzXfsar+Z8Vo
sL4Fd9TdNes2/5Lvw0f9Mxj0nZXL4Yieo0i4a1vlHXm/mkXvepB7xHOc5AO8
0Tnn/NeUeJbTPM9L6g/L94oLnOFD3GfmOo3nfCE=
          "]]},
        RowBox[{"-", "0.17200000000000001`"}]],
       Annotation[#, -0.17200000000000001`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkMkuA1AUhn9hYdywQg01VAdJH0DtDBssKoTYVCWGBF1oqhZq6APY8wpI
uiFUa2grWgvTwhih3kKi9d3Fl/+ec/9z7jnX6g94l0skRaG2QfK1StNwbJEc
ndIRGrBLsXop0S4V2qQi/MET/jNynkbpFO1B4+iIU/rlLtQlrcIwuUVyQ2gd
+eoWKU/+B5Y6eIO6bnIukI3+vHtHfoC3Y+ggmuXuBi6t0gWcwwN397CPf4Le
k5Ain0VfHczLzDnOSTTMDmvgwXtFv0N6HYCFeWbZZQ7S1GZMb/x99O2HCuqf
8ZSjL2glusuM8/jXmT8CG+Ajn6P2FnbYp8rMYHZF39AZPH4oMlcB9uixYP6Q
+0copeaEfy9D3cz1zV0N+k5tnHykmXcgxexOYi9q461R1EX8Rd0Kmib+QDfx
bkGGuJedp/j/Mc52/mDceNgtT00Q7zXxJ7qNPwrBJukf9NBZng==
          "]]},
        RowBox[{"-", "0.21500000000000002`"}]],
       Annotation[#, -0.21500000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkElOAlEURW+JugQTe9BSKAEXoMGJA8EWEZuIUeNAnIiOXIAbcAk6sCPO
SEwcCiY0CUhvg/vQqecPTm7d9+9/77/yHJ3FUpakS4j0S44jXU9IN5D0Scfw
NCS94UvDUg4touExaQHsUelvQMqiv+gj52m4hwS5B/SWszt4po8Pn6LnKWSY
lxuXLOgCQYseLvRqUtqkv5/8FMxw1+bOLJqxpc4gb3NLL/hP9JxZFyPSCT2T
eC/z0l4pRK5Avzl0mloQ+piRoLYHu2Y2syyIcteFNukfpm8DjaAt9If7Dn2X
8N/ootkF7SXv558FoIfvPNqNLnOe90gxvjegzNwSvggFWOG8Td993v2BruLL
1ONmb6iQr+LfoQJrZk9yB+S/0Ci+Rn2L7DZUyTfxDajDunknuUPyHTRu9qC+
Q/aVPevks2ib2jz/LcguAajBP2/SUqw=
          "]]},
        RowBox[{"-", "0.258`"}]],
       Annotation[#, -0.258, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkEkvQ2EUhl+J6Qew0AFFa4ihVu3KuFKx0MTUpUSCtmhVtNR/aOyonZUh
QVWiSLRsaLGTiDH+gVURG08XT957zn3O9517LZPz7rkiSTGIG6VIjeQgRy2S
k+xukHqgC6LUxmop3ywlDJIZx0C9RW0iN8g4bMIw79drJTeZsUlp+DJJl3WS
i16aezrxfGbJTvrJML1lqAUL7OMPFHwYhG165cy66qlhqVEKwS+9oRbp2yr9
wCe9E3YdoZen/uP9DrseMZOAJBwXqJIO2DvFPeOcPwGVuFPsXUGOUo/BLrN7
kMS/Ya4ESqEYHvHKyBjf58G1Nkk26OXMU/7ZYeFeyLJ7Ds7oP3H+MveG4Zy6
lR097NrHcyOz/WSK2Ve+YRr3gvqZjOCvQABnAdqYa4cM7jVcwRszXlwfBPin
q/gz7B3E78C1wy1eDrLwju/HnYMgfhR/Fj+E78B1wh3eA9zDB36g4MIi/hq+
F/+F+h8Gu1xQ
          "]]},
        RowBox[{"-", "0.30100000000000005`"}]],
       Annotation[#, -0.30100000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ctL1GEUxvHjykV/gE6pOabmpYRSg0QthK5GXrZqCyFCg+yy1P/BlYgm
ldsZx/ukI3SDaBkpKm1atCs3ZWSguOgziy/PvOd9zjnP753k4Ejvo4KIeIXE
yYjF2oi+sogF2k+XaAd9XB4xgi+nIzYSEblkxDruVUQMoFhvc1XEJVxEExrx
mT/Hv8b7Gv28fSjiHzJvGG9LIi7wvqHPaszl/6RvjfZUR2T1va+MeIc0X/cZ
dayor+bn5vM4Z+lXedf9XlbvoneR0jNQap7+q3Zk3M1jAR/t6bajC3POV9wf
8j+ok1VPnXlHzuVytrnboe00SXdP2W1mq/MKzWIVd+SoMrcaSzxDaov029mI
WbUTvr1Nrod2tNOn6k9woN6plla7zJ+iGcxhy92EfJv0j/drcT+vnrYr4/9J
yXBN/ToKeUrkG8e093yOKfzimaF/9e7JcUB/2/uP7tP7+l7oeYlC337evgb0
mP89/+407T3G7NznGaWTaud46pHTf+x8w6wfcnxw/kmb5GzEtvoObuMWbmLG
7oQsxfgP9gRq9w==
          "]]},
        RowBox[{"-", "0.34400000000000003`"}]],
       Annotation[#, -0.34400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwNj0svA2EYhY+NRVe2epvOdKYzI2FFwr6VLpCYlnTNJGiiXWDhR0hIhEWJ
1k7SqrBzW7ivkGiKjT/gJyDxLJ6c+b7vnPc9Y85Vg0qPpHUIo9K3L/UmpK2U
tAn5fqnjSMNJ6Q0dQR1TssGLSzE8LhpFd7jbBp/zvi2VmNMgM5qWxmCXtzba
ZUeTuSd8t9BFT1oCA2+LbN2QDiBCn42MNGFJcd4TkIQGs2vM2oMbfLdwTK5D
hxX6rUKbc4p5pQFpkG+D3BB6SPY9Js3jDaFCxzX8C/Qo4zHJWJCl2zjkoI8e
EXZ8udIp2TIzzlCfOw8myV5wdwnncA1X8IHHZocDVfakUQum8N/z/gB38ASP
8Inf5T1Hzy6aRTPoD30K9PilR4D+oUU0JPNKpyNyRXQGZqHJuYAGMA0v8AzL
/Oc/chNJbw==
          "]]},
        RowBox[{"-", "0.387`"}]],
       Annotation[#, -0.387, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVkLtOQlEQRTelBdqKCMrlpeAP+EiAlgRUoKGlxYDfIQm1mnARGmmAi4L0
IvEDKPwECwoJD0WxcFms7DNzZs/MOZ5cMVWwScpDc0taGdIvLOEHvmEUlDLc
DQPSC3SJHyHmk9qoBSXO7/vSFTolHu9IaTwD6p/B3JMqkKHfZkhyQJ74Arrb
UtBPTzSAvuJteKR7sOh3SN5LnQFp/GX6+al7+M+jHdSHDvHV8dSgie8OTVF/
Dhvs8ol+wQwWMIe+lxkOqUqtCUlyCbBTn9uV1tEz4lOwmNNy82bqn/C10Dfe
3ONcwRtFI1Bnl6yL+exwgOeauxu4BRc93RAmb3IfQqvoBM8HxPmXOTMv+cMj
ep3AMayRM3ifCW2n9AdUd0nU
          "]]},
        RowBox[{"-", "0.43000000000000005`"}]],
       Annotation[#, -0.43000000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVjz1SAlEQBtvcC6DAsoCKJiqimPoTWSUBGYYGGqmgl8ALcAB3F1AsCDRR
TyBehUSgSkPboKvf+2Zmd154flO/XgBO5WoJ7pdhsg4XngeeL/X7GnzIm7TN
Mln4sefF2lcOXvWv9+MMdFbgTuZpaOmZvtWTwN48jOVTpuZN8289KsK+jkNI
pCuB362tupdE3qvWh/ad+Y/IuURi2TUPS7Cnn62X9UBX9KPe0U96/r+LNDZg
yyznzLbuW9vUXd2TSBKJ5STlfs5M5aAAR3Ioi7634Hse5A/yJjW9
          "]]},
        RowBox[{"-", "0.47300000000000003`"}]],
       Annotation[#, -0.47300000000000003`, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdlfs7FPgCh4mQynoUSzcNmiQt4uRSfOREzyrVSqqp1bKUI5TCtrmkGpeI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     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->568.8000000000001,
  PlotRange->{{-0.5, 1.5}, {-0.5, 0.5}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{573, 301},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[13]=",
 CellID->437519550]
}, Open  ]],

Cell["\<\
Placing the cursor over the two streamlines nearest the airfoil in the \
overall flow field provides information needed for focusing on streamlines \
near the airfoil.  Smoothness of streamlines near the airfoil and absence of \
streamlines interior to the airfoil suggest that the discretization is fine \
enough.\
\>", "Text",
 CellID->130290240],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"g\[Psi]N", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "0.02"}], ",", "1.02"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", 
       RowBox[{"-", ".1"}], ",", ".14"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.02"}], ",", 
        RowBox[{"-", ".03"}], ",", 
        RowBox[{"-", "0.04"}], ",", 
        RowBox[{"-", "0.05"}], ",", 
        RowBox[{"-", "0.06"}], ",", 
        RowBox[{"-", "0.07"}]}], "}"}]}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "3"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"g\[Psi]N", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[14]:=",
 CellID->176178541],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxs3QXYVVX6NnAdux17DOzuGnWMNXbr2N2J3diBCioGiIKIdAvSHR5AHFux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    "], {{{}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1GtM1WUcwPGnklZbdgOK+0RuQmUKol2mNHVtiRW4lhZslesy7wasZVt5
aDiwTTJOYWm5NrsIrV4V0zZflFacNsOMBYQWJU7TDG12XbY+z4sv3+fyu/3P
/xwKl69bsvaiEMJOf/qvDyGFxqwQGjA3I4R9eSH8nRlCT04I/3Av/8kp91tz
QziWHcJf9rudt14XwinnCW533oEc+5/VqbU+xbn269xfkPsfLjgbzA/hXx53
dxzPmeEY/4T3kal2BtJxp9wC8Ws5n9dwv/M6/oLrOcUHzfSa2bejQZ9G5OjR
pF42rxKzGiuREPs8Sgo8A88UU4l8TIh92uxp1o+IXe/+Mr4cndbpctJ4vb5v
8zT7d7hFfDMG08V5nmedHTbDNxjEHeaqQb/6V4m7Gs/oswEpZ1faT+Y/PGNK
biUy3GXiA/Xy4lxicnnALCv0/yo+D7/lfBf2yXkB15jpF3F5zs5wgxqN6FJn
1Hz1zlfpsxpPYFyNx/k4V4grR4O8Mb5dziHrfjkDvEb9XXwvB32WWN9nPRX5
9kWccDZDfCtPyK/nFrXqeJmYQWcPxefgRfEduavlLHeHnH1sf5hz7O923my/
mL92dsT8JXpc67Msje+DL+Ukut0n9S22ruKP5PWhx7oXnzpvtU/K6cJLeCrm
4Ki67Z7/Fp/DbbgVp8Xnid/jfi8KrPNRZpZf5d4s/gzP4AmezkP69LgvEp/m
vc/zju5SZy4v4hrui98HXmxfKq+WZzmrxiw1euUPq/MeV9ufV+t3VFqfFV/F
53gmv2rOYnev8DZ040vP8pn8z3EA59TfbO7feH/8HNCo9gl5J1GsbxGa4/sS
d1b+bv7E3Y3O6q2XYhkedfYAP8YJ3OMdbeQxfX/EHGwyV7P5WtAZn0PfjXIS
aBFbEntxKR/Uq8tnkcRW9y+iE2/G32j8DsXvsJi97nPU3cO5PAez0S1uG+ow
391CLMD95j4v9zvrUYzgKI7gYXdb9P4+/qbVmGfmLXzSfoxHzDaMMnO8bL/D
esIMC8R12c/nJC/kNq7m2ahUswpDYpfbT9W/CE3xmeP3EpPM+SEvdb9f7g96
HuC++H2N/0tRjor4G5XXjgH1msRNQznKcAMqsAE3YTq2m3cHvhX/Ou/EG9jk
rlffJ72PNutCM07BiBmGMIxCTMHFYiZhsvhM7/YS6yusOzDuc9vMWcjGCfsH
zT7MbeYdif9r7NPdvStvlFfosxI17jrEnMb/YI7yTA==
          "]]},
        RowBox[{"-", "0.02`"}]],
       Annotation[#, -0.02, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1nmQz3Ucx/Gvm10dJq3KOlKqKaRyteTMWiG7i1035ehypJgKEWbMuKIm
hXF1OBqLdZRFJbG2QVMp5FjUTPVHNM6aItvj/cdzXp/3+Tl+38/3+7vz6bG5
Y8olSVJcPknurpkkg2snScntSXIUR9CTb/odSTIDUzENb2DWbUnSXmyU/Bfw
cXqSrMWXtZJksfgSfKP+abHm8mbIHyM+FpPFXkfPukmSg2x0kLOnTpLsxVfY
h2Lk6tcb3eT0okNRR+4/Yo/cmiSpyNWzF3oiB9loZb7x5piAL61jF75AM/mt
UV+/DFpZv5r6/R3zpiXJblTiyxSvSF/S6y7cHWu3lxeRKb8VuzMdwx6NBuw7
0UbP2jQdw/iHo4W8DHZLOpI9AnXZdfAibrXWNIyyvhewybybMUGvQroRz8fv
ob6mvN3GS9UtwxC9huIesb720JBuk/8KvpP7LSbGetVMosvlLkMV8T/xGl9L
sVfpOfZYXFBzHvNjDWjiLEqtJU/eKbpL7CRd6WzfxzKswHKsit9VPBsr5EwK
Px1trVP55pmnmvlTUBUP659KHwoVT8HFWzxP5mrOV83cn6vpHL8fTWG34D+g
Zj8mxu+p92zMwkr1f8m7KvYvnvF7pjuXmvK6qjsulqVXF+Od/Fm0iO7AdhzE
BfMv5X8qng3zVVHbU01V2t8cA9AXb7P70Xw9T8pNwxpzro37gAw1b4ld498n
9x25JXQhLQ7b2ewNmxbEeWONmtVooH6z2Fy58zAHX7CHWV+pPb0sPh5H1c5k
/0iP4DB+R1fzlso9hZOYaA2n6Wg6Tv/H9GprvnzaJfLRy16zaG+6QH2Z8fw4
SzWT4/nBk+Yax66hdmb8JuzqsWa+YbQHu1RsvdoNSOFLxfPxW8YZ4gHxQWJt
9P7PmgYaf8vf0Vo622cmOmEm+/G4R/I72WtHdMBW/qn8W+gisZ18O9DY3Gfj
HvL/QRuxt/EXYTsWxjvLfPP529EFtAMdJ3+K/E30dVoYv008h2ou4DxGyq0d
7yX7KEHd2KfcevRU3Hd6Mu40PUZP4Dha6/On56m9Hu2Q72y76N9QLIuOsPeR
GI7x7Jb8TzqXH8xxl14fqVmF1Tis15ti5eSuM94jf7Zz+1DuL9Y2Le6z+olx
F+TMxaR438b9MJ4T91P+79bzkfEjaIaHcb9eD9EHaHO9m2EBCtSe5vtA7Az9
kO6ylptpbXupz1dDTh47H0PjvWCOFXiQXY99e7xL43lQ14Zm8mXQznF3Q/kf
Rz95PfjujTOgg+K7IL5RLN24kFZiV473jv7941m37zP2M8D4Z/qOM+iltjTe
VTiNHLFs9Hd2u/X+Kr5TuC5vD/2PvqbP3877VfoXXRLfMWwTvyI+y9yzMSe+
j85lsF5z41ugb148t7QPnSZ/Ot7AJ+zFcpfgsvFqOZfoGtpXffe41+xusV/0
QC77vviOxbvGnruZozt6oKJ1bVa7FVtQgV1Iy9NN8dvGN0Tf3+y5Pt91WoYk
ngOxU7hBn/b2VT3uEq0lnoYzcr5W8xydYu40ejDWyXdIz9Nq84yrOp/8+EbS
KqiMm8THit9I8/Ttg0X2cECfAfbwnvF+43dpOfkJNiDTvAf5t6j9XM8q8e0z
3ozu7ApyutGK9EG+evSAOY7EXcATYuX5UqwzFW/JeRsj+C/SS1hoj795Ls4Y
/0q3yv8EW7ANn8Z/DPkH4r1H98d7kJapu46O1jzDGo+Z7zhOIIuvCxrr2QR9
4u7zl+L7+P7y3WIvRXQ7jor/FL+HudqKH4p3oHM5rm9j+j27Ef0uvuvmPxbP
iv6rcJ5vib6LUS7Wx7dRThptKu82WgsN+OrqcS3O33iw8RDxpmK92Tn82ahu
/kHx/Mba9MuxpiKaG78J3zjxs+Y8h3V4Iubl6yS2gb0eBdip9zx9d9ACeo2u
o1fjHRbnEt9bfW+wjhL1hfGeZrdnf0Yniz9mbW3xrLqB/M/Qsnh+5V6niViZ
+Y9Y22H0VXOJ73LcYf492Itj8kvoPhSjlf5X5FTlr4YM9l69W9Ni+mg8a/Ia
+X0r05V6r8BVz0c/+f3jv4vYcvwPdGWf5Q==
          "]]},
        RowBox[{"-", "0.03`"}]],
       Annotation[#, -0.03, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1nl8FtUVxvEhAbR1ISKL4tJWLWIJBCu20lLQigTC1gYFUVlEQhKoKCik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          "]]},
        RowBox[{"-", "0.04`"}]],
       Annotation[#, -0.04, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV1GtsD1YYx/HTko1ESxWlV3+3ICEu07G66zZvvJnKkJghYYy2b1pL1khW
rWtH6h7ENVGRBRv2xgu3hAQVfyKuGaHmLlhsyjY+58W3v/M8z+88zznn//83
MbPsq9KUEMJ3/kzsEEKvLiGU0Jq8EGrxOjuEFlkhnO8cwjm8Un+JdWoJ+cM5
IRxBgz0j5PfQj+3J5L1gPVtuH98S/qXYwrsVlWqFahW0MXrxA98p9JMrUOtP
S+0pwwI86xRCOX1C+6nPVH9qPa4jf24IxbQrTaUZ6pN4j6mX0ON0oRmVyLCv
WPyO93P6Lx1Lb6mdVmuTr6c4DdvVkvpto5foDrqZJnlbqU80pzbeD63FlRhq
fwWt4tnC2816iFyCfqZ/rfwS7FZbZ/96JDN5vdtFmuJ9UhHQEi2Qpf8a9+hC
62lnukKvv2kaavX4h75BOv7Xa2F8J97+uCRO4qg9Y3jHYgDfHrW9aMB2tRXO
NdB6EAbgEwzGPGfdiz7u0UBX8tWhjxm3vUlveofOoFn839NC3vm0Wd/LclPN
uxjPgmScKz/SOdKdKw1t0I1nOh6pP/QWvcx4RDfKzcY0/hf25csn+BP0nXiR
/FvaVTzK+RJmj6YDMdn7TcFU1Iu/ds48mk7boW38/O0t06Mc58w5i1ZyGWa8
sH6JcvyFV1ig9iOdYE4VrbFvMW5Y38Q9915sXjV+wj3z6uUeu8t96z/RhF/N
/g37cRAHcIFnvHuc16cRPfXtgWIzm8T3URpn4C6u8E/3Xjf1/4beot/S67S7
fePsey++Ia7hz8YGuSK1jTRHvJ6muMtcZ6rme2r+JrlUuVz1rPjZ05biQ/LN
6m+xI3r1bnaG+eI39En8LslvQ767FyAHecjFA75ss3NwJP4PQbX1Sv46PLS/
SZ+e8u3wpZkZtC1Wq2XSn2l7uopOcp6r1h1QZz0s9qJ9Y2+6S8+d8fdu7izx
c+vf4+8VVeL2/Gfkxrhjkfi/+FujJfLLrJfjI/W14jWoxy7eRb4bO+k19HXG
q/QL3nyMFxfQZfYVOeNwFLr7p/jD3eY4y206TDw0/j/0uVfgFz1O6nsKx3AC
x9GIDzYs0X0=
          "]]},
        RowBox[{"-", "0.05`"}]],
       Annotation[#, -0.05, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV00lQz2Ecx/HHAaMhSztZopixjHUoClnGWA7GTDMiW3KxLwflqDGTFC6M
a3FxEY4ozJRSifrHcMBEu3FxIMvB63d493m+z/fzXX5PlVF8atfJUSGEBD9e
JoQwZ3oImZiNucjCguQQhtJDGEY/BjGAO4khfKFf0YM+9GI+f/3UEBrwCE/w
GC3656eGsAE3kkIo4o3TfxyK1FzSb5m75ViKwzzJ7tqdX6ENr9GB0fy/9BzB
T1zWowKr9V40LYSFWCA+y7dqRgilPCXqKmidvqXO9+h5msubhzWo5M9T90Vu
nvh9Wggf8EzfC+IylOIzXwlfNl9xtCsO4BAO4kf0DnZ/mhLCO55/vCvEDfos
p8fkj+MoalGDvdE7oFvvQhqje2iG+Xf1qFZb7C7N/KmY5VuG6Tf9vsvFxKdx
BjORKtesvgUDzoMYwg79+mkfelFgx/Xq16Hd76iPv8ycUqTI75RPpMlIQo95
W+27DafMKfC+0+w3QZxOr0dv5+4aPSe/CtlYibXucugo3kyawpfsbqFzlb2q
cUi8W1xMC+lXMz9FM+U+Rn9z0Q7Ybq96ux7ha7Nrvl6t0fvwvaQJPJt5JtEp
mIxy9d2+8w/Pb4wgnreRb7HcE7kl9K++Xe47MYunw4xHcoXiifps4k+z2yfn
t/ypPD3iEzgZvT3i5TbyTbRXN88WnhiNF29wX44m751Ht7pbS3Px0Ox1tI4+
wH1cVBcz/7aetajBeJ48d7lIle/0pl14g27EMNa+V9RXoRJxanL428ztjf63
9dnPtw+3fOM1eh1X8Vwcp2+T8ws0ogXNCPq26nNT/dzoW3mz6JjofxP/ASsU
oVM=
          "]]},
        RowBox[{"-", "0.06`"}]],
       Annotation[#, -0.06, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DtP03EUxvHjpFAMcpO2SrkstDiBAQIvwSgYg2zEoIwGwuBSUkgMgTiQ
6K6RgIkuLqIygIoiMEKY2CT0JXhBbgmfDt885/L8zjn/f/PDsXujFyJiATO1
EeXXIxK4iDJcQhdu6B03ROzURWzjytWI9kxEJZ3Wq6U35XW0CjXokFfT3np1
nv/eb8o3cJqOOMMRTnCMZp5O2kJ/piLW0WPXjNosFuVvUKyJeJKMOKC919yu
9szMPZ73/Gl7RuRJ+oimaCt/FkPyovmT4pe8rzCLCfUe9+ZpoTSbp1q96O2W
PUm1X2oFu/bpA96UWoOdGfT5xltq/fQ2vnt3h36la/hW+nZzUv5lGm3uziGL
Snvuqw1iAHPyjB2fzU245xOtoMu0nO7aO6WfML+bt83scfVWmkMWeZ4cz7D5
h26eFP+jBfqFruIP5vVf46/4B4bETSVP6XY8xipW8Fu/Xm+FLskbxXfFT8UV
9l3Gmnueu/MFlvQ+ooxvVK/f7D58UFt27zu1t9jQ7/Ku0f/rpOddaVow
          "]]},
        RowBox[{"-", "0.07`"}]],
       Annotation[#, -0.07, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdlfs7FPgCh4mQynoUSzcNmiQt4uRSfOREzyrVSqqp1bKUI5TCtrmkGpeI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     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->568.8000000000001,
  PlotRange->{{-0.02, 1.02}, {-0.1, 0.14}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{573, 156},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[15]=",
 CellID->352096662]
}, Open  ]],

Cell["\<\
Magnify the area near the trailing edge to verify that the flow leaves the \
trailing edge smoothly, satisfying the Kutta condition.\
\>", "Text",
 CellID->11571494],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"g\[Psi]TE", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0.8", ",", "1.05"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", 
       RowBox[{"-", ".05"}], ",", ".08"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"Contours", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "0.02"}], ",", 
        RowBox[{"-", ".03"}], ",", 
        RowBox[{"-", "0.04"}], ",", 
        RowBox[{"-", "0.05"}], ",", 
        RowBox[{"-", "0.06"}], ",", 
        RowBox[{"-", "0.07"}]}], "}"}]}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "2"}], ",", 
     RowBox[{"PerformanceGoal", "\[Rule]", "\"\<Speed\>\""}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"g\[Psi]TE", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[16]:=",
 CellID->144480356],

Cell[BoxData[
 GraphicsBox[{GraphicsComplexBox[CompressedData["
1:eJxlXQVUVc0WVrE7UewOLGzF2CZ2goKtIHaAYqJSIgoq4r0gInhBGgUFAwM5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    "], {{{}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVzT1LwlEUB+BjaUH1EXJwdQnsjYZAt3L0I7iKulVDSw0tUU1BDiItQvbe
akKtUZOfo60XsYKe//DwO/eee+7JVZuVRioiKgzSEd+ZiBEffPHJo/v1qYhT
9QQdUiyxyKVejw322eOCF3N98tMRBe/SnDFI/mSVG+82mVQfyGuueDW3YK7o
foUuy7Rp8az/JHf44Zc/zskwZpdbf91xz4PzWrKT7aTHFj1qZO1ryHlZl4e8
JXsomS/yrj6Sc3ozclYeyxPKDPX/Aa5HLow=
          "]]},
        RowBox[{"-", "0.02`"}]],
       Annotation[#, -0.02, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwVz7krx2EcwPFHuf8Icp85chYxWVzlmCiKHatrYLZRFslisjCi2Nz3qFxF
uQdnFry+w6v35zl+z7dfWt9g+0BMCGGM6rgQEuNDSNIDDjmLDWFKf7jmhg02
SXI3WY/IN1dSxbB1tU7oJIOkJIQwpKnar3XO69nw/j5v5mxyeOCeJvcyonu6
QAfLrDBKjrdGNFdvvVHh7l1U63LzubNOxjm1/2yv1fykr7zQYp2pzfqom+7N
mucoYYlS0p3V6S5l5kzfyKLH3Mui/QbrNv22btcP/eKTY+9m6Un0f53t0cg6
a/amdYYiVtlmJ/pvfpNHt/lXg2/8aZf1BZdcMU+B/ULSKI7epcb8pn++saWB
d2rt/wOAqEQH
          "]]},
        RowBox[{"-", "0.03`"}]],
       Annotation[#, -0.03, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0ctLlFEYwOFjeVmU1xnvLRwlI0i7rZISJPdeSksz0FHzQijd9kEF2SpL
rbQUNKgg1ymtXbRpk5pCpW1aSKX9AUrPt3j4nfec7zs6M4nkcPNQSgjhI4m0
EF6lh1CiL3WGaWLmBm2kiXHGWEsNYcXZKvfND7hHLk+J22/WC2yzw0VaaGXW
+Rx1nMgI4bP7Jq0neO78Bc8oMk/qhvMrWmye0t/mfh1kgE3zYWc/9bT7Dlnf
5as57rzMOqYF5PPL/jt7eZ6NkUvSHNcBLdJB7aOfa9FdrLEe/c/uOM66e2o9
m219TnM0izzPPNHRqBTSZm7nh3cm9JG9Cu/c0nK9rfO8p8p5NQfINn9hmVnz
HJ10Uem9O/aT1t28te6ki6POFvytT/SaC8yF5NNjTuhNLdMbesr7J8mMfkfz
Qc3Rbfbc8Vf/scM382v9rpc9d4mG6Fyva7E7Sykh0/w4+l50VLe889B6hCOc
8UylLtrPsN7HflK4aj9VO/QNvd7vId1eGmftfYi+e13QY/qHXXctRZ/NXo3n
6jlPlb3/JbtTdA==
          "]]},
        RowBox[{"-", "0.04`"}]],
       Annotation[#, -0.04, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0LcvRmEUB+BX+5gleosyaQkRBInOoETMLAY2dgOD8ml/BImya7tuU4IB
EwPCZhU8d3hyfuec97259xaPTQ1PJoQQpilPCSGDmtQQHpNDyJOf1Fp9pjzD
eiyEDUYY5cX+W91jn10OOaDA+RLq3X93rkz+UBv0pfIsRdTpX82H5Caa9c1q
Ie1yume1qTF9XE5Vf+mX+2i1+3G/Ua2gkjn7Jbs09Y8EOXDn3IA6SAdVzp6Y
zTuzrF8hkWSSuLe7srvmiE6zU7MFeVVeIyV6J2I82N3Y3XJMl9mZ2aK8KW+R
RQ7ZvNk92T1zQrfZuVlc3pZ3yCWfPD7txtUJeqj2/hdmhdHOnR79l3oa/Vuz
Fv2lfW/0vdH/Mf8HrM46Og==
          "]]},
        RowBox[{"-", "0.05`"}]],
       Annotation[#, -0.05, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], LineBox[CompressedData["
1:eJwV0DdOA0EAhtFBIucr0MERoCTTIoFxY0s0dHASQs8BoARsbDIm55xzhisQ
JZ6Lp0+z82ul3aruvvbenBBCDyO5IfTnhTBANQmSjOaHMKlTpBmzm/Fslmnm
maPe3ZIus0CGRcbtV3WNFTZYp8F+R3fZZJstJuz39YA9jjik0f5MzznmlBMS
9pd6xQU3XNNk/6CP3HLPHUn7Z33hiRa7V23WD23TN23V9+y3239pp/OnRvRb
O/RPo/qjXfqrKfuighDizoVaQjEx5wqtpJRyykjbD7obooYUtd5Tl/2X7oad
/wGP5ET8
          "]]},
        RowBox[{"-", "0.06`"}]],
       Annotation[#, -0.06, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {GrayLevel[0], Opacity[0.4], 
         LineBox[{1199, 1513, 1514, 1511, 1275, 1512, 1198, 1664, 1274, 1662, 
          1273, 1663, 1197, 1661, 1272, 1659, 1271, 1660, 1196, 1658, 1270, 
          1656, 1269, 1657, 1195, 1655, 1268, 1653, 1267, 1654, 1194, 1652, 
          1266, 1650, 1265, 1651, 1193, 1649, 1264, 1647, 1263, 1648, 1192, 
          1646, 1262, 1643, 1645, 1644, 1190, 1509, 1510, 1506, 1508, 1507, 
          1565}]},
        RowBox[{"-", "0.07`"}]],
       Annotation[#, -0.07, "Tooltip"]& ], {}, {}}}], 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdlfs7FPgCh4mQynoUSzcNmiQt4uRSfOREzyrVSqqp1bKUI5TCtrmkGpeI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     "]]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->568.8000000000001,
  PlotRange->{{0.8, 1.05}, {-0.05, 0.08}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{573, 310},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[17]=",
 CellID->22095040]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.4.3 Velocity field", "Section",
 CellID->56297006],

Cell[TextData[{
 "Plotting arrows for the velocity on a grid provides a relatively fast \
direct visualization of the velocity field.  Use the velocity influence \
coefficients and the strength parameters from the model solution to compute \
the velocity at an arbitrary field point ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["f", "TI"]]], "InlineFormula"],
 ", induced by the panels representing the discretized airfoil, ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["r", "TI"], 
   StyleBox["p", "TI"]]], "InlineFormula"],
 "."
}], "Text",
 CellID->253577469],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icA", ",", "icB"}], "}"}], "=", 
   RowBox[{"ICVortexLinear", "[", 
    RowBox[{"rF", ",", "rP"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"velA", ",", "velB"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"icA", ".", "a"}], ",", 
     RowBox[{"icB", ".", "b"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vel\[Infinity]", "=", 
   RowBox[{"UniformFlow", "[", 
    RowBox[{"1", ",", "\[Alpha]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vel", "=", 
   RowBox[{"vel\[Infinity]", "+", "velA", "+", "velB"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "u", "}"}], ",", 
     RowBox[{"{", "w", "}"}]}], "}"}], "=", 
   RowBox[{"Components", "[", "vel", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gVO", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"u", ",", "w"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gVO", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[18]:=",
 CellID->20343141],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnXuvbVWZ5unuT9Ifo0/S2X/YGo1d7ZVGQEBAt6BUoRZ4AUvBC9jdoDto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    "], {{0, 326}, {569, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{569, 326},
  PlotRange->{{0, 569}, {0, 326}}]], "Output",
 ImageSize->{573, 326},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[24]=",
 CellID->771285384]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.4.4 StreamPlot", "Section",
 CellID->256023531],

Cell[TextData[{
 "With suitable values for options, the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function StreamPlot can be used to plot streamlines directly from the \
velocity field.  This would be particularly useful for panel methods using \
source panels, where the stream function is not single valued.  In the \
following plot, streamlines with direction of flow indicated by arrows are \
forced though points specified by the symbol tuples."
}], "Text",
 CellID->117098232],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalDelta]", "=", "0.07"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"z0", "=", 
   RowBox[{"Range", "[", 
    RowBox[{
     RowBox[{"zMin", "+", "\[CapitalDelta]"}], ",", 
     RowBox[{"zMax", "-", "\[CapitalDelta]"}], ",", "\[CapitalDelta]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"x0", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"xMin", "+", "\[CapitalDelta]"}], ",", 
     RowBox[{"{", 
      RowBox[{"Length", "[", "z0", "]"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tuples", "=", 
   RowBox[{"Append", "[", 
    RowBox[{
     RowBox[{"Transpose", "[", 
      RowBox[{"{", 
       RowBox[{"x0", ",", "z0"}], "}"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"xMax", "-", "\[CapitalDelta]"}], ",", "0"}], "}"}]}], "]"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSP", "=", "\[IndentingNewLine]", 
   RowBox[{"StreamPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"u", ",", "w"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"StreamPoints", "\[Rule]", 
      RowBox[{"{", "tuples", "}"}]}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}], ",", 
     RowBox[{"StreamScale", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Full", ",", "All", ",", "0.015"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gSP", ",", "gA", ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]}], "Input",
 CellLabel->"In[25]:=",
 CellID->826808409],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnf2PVde532n7l/SfqFSkq8QvYGAwmDjESShxDDcTSOKQ3JrmXrsvdpvA
L6YdYd1biDEDxuDwkgTHuYe4xTIy9/oa27oTUWiRx6qF3RZU2UJylEqWrjT9
zvnOPF6svfc5Z/brWmt/P1qyhzNwzt777P181/OsZz3PP92x5ys7/smqVav+
2T9ateqf/+NVqxZ/XmieVUIIIUQF2pGqlv9h394qwEPSW7X/Pnqr9t9Hb9XO
+7QmVeVkMcCrHeZbBXhIeqv230dv1f776K2afp++eVViQnTN20fXvH10zdsn
Cq+q5X8oSqNr3j665u2ja94+CUuVEEIIIakSQggROJIqIYQQgSOpEkIIETiS
KiGEEIEjqRJCCBE4kiohhBCBI6kSQggROJIqsSI+/fSPn3/+D10fhRCiXwRe
A1CExh/+8PmWzSe+9CeHt287veeHrz7z9MXZo++dfGlu7u//N1Ss66MTQqRG
FDUARThApKBHFwY39j4xgFS546knX8Ovuj5AIUSySKpELvPvf3Ll7Y/gMe3f
dwne05r7jkytn8UP+ON/eOYiFQqvHHj28q1bn3V9sEKIxJFUic8//wf4RK+c
vw5hghht33YaMrRzxzl4T3gFbhR+665PPXfwrUe2njp75ir8rA4PWwjRHyRV
fcOE6fChKxCmLZtPwGPCD/CPIEz41c2bd0a/w/Vrt9s5VCGEIJKq5OECU1aY
lAsRI/jKqozXL36ACckkg7dHLUP3mKiOpCox5t//5PKbH8LU7H1iwFAeF5gk
TG2C6zxaIJ47+Ba+l+zw8lWyI/dfTT6YsTnJ4NymlsGUUW9wyrSiQce/RpFV
BLshrrz9Ee4f/LfGjS2SqqhhVh6eTZig3bvOc43pqSdfwwOL+2RsKE9MwvVr
t824uSYR19msKC67a4dhnEcLxNkzV3ONZ9fn2h4MRK9ocDm1RpGdWj87dm7A
TRljBx+6ScaKTjnSYDvsEqYivICwS7XIlqQqLuA0cYrOySqz8nAn4MVI7+qu
uHXrs6zLk+sI4Fmz112bA+/VTAq+l65PSDQC5nuTaApDGZOMFTmSnH/GODZt
PJ4VfTxrpb8ISVXIcOaJGfj+fZfMaeIUXdG80UA7cImYb49x4NnLfPYZFMV4
ZOuprMuj5RUhqoPHx8K88DfhDlff0iKpCgp8obSue58YwJYynv/cwbcuDG7I
afJg8JPxT1wxKI6rRNB0/JH59hh4WPiXFRQVomlgxGC18N8a31NS1S2MwHOR
HcIEeaJ1xbesrbXEXSTi8hCuEjcg0y06fOgKfvX6xQ+kREKkiqSqTWBIGdOG
ybX0vAPPXuYie8/LwDJ7wY3nuzlv7vKQRFyIvqFytY0C8+s6TZAnZgrB5PZ2
/s98Bqgz45y8Mpa94K4ZdX2kQojuUbna2mEWxMmX5vbvu8TsZZjfPjtNXE5i
jjFUiZkhzGeAfDPO2c8rI4RYEZKqKnz66R+ZBQFfybIgDh+6cmFwo4fZy0UV
brlzE79SZogQohySqhVx8+Yd29YEO7xl8wlmQVx+88NeLaC47tKeCSrcCiFE
FSRVo3EXm7iLzbY19acqC4Xp7JmrvA7QaNddUt6dEKJpJFUu2S237mJT10fX
EkXChD/yOvRHo4UQgdBzqbLFpt5uubWeIBImIUSw9E2q3HIQWzafsMWm/my5
tb1drjqb5yhhEkIESDkRGUwvZrmvnpnP+d38zOrpQS2fUgtwjqxbk1cOoid1
3lynicttVnq9J+rcGtlq4bj3sgVL3ZLs3uiwvmjFEuJWukp74kQTlBERCNVQ
jAbTWbEailh3UuXubLLFJuvW1IecNK40sSYe93a5TlPXRxcT1vvDLb3u9Zny
ukhkezDh3stadbckezhNQCqWELeCwJNo7uhmVUVNtXI7pyilpyeUEJEv3KaM
Vg2mpwftelUjdjb1ZLHJXW6ztiCsidfDvV1jsd4f1vzIbW/E6oJfurv3h2s5
PWupeGk5RjerKmpVnNuP0goUeyNXBCeUvz7MaRulieeihIgsO1UZqVoUqsYD
gNrZ5GmTu9zWk5DmaNj+w8xdUe8PaynrNo3tyS3UB3JFcEL5s7aA7rA7xx1W
ut8bozsRJ+8JwhrDLmGOhwlhXSdbn1eFl61Wk7+MtepuJv8sS5xmeKGfO5uk
TR42J2f7DxbHsCZ0bP9hrlBPjINoGvPH3WEN0bwxuhNxkSfo3sBV1g1DGOsf
OGpnVFq2SgvHEqPWqmr2qiBSTJzmom25N4kOaROhcWCkjnrEZ9xWOtj+g8Ux
ehLvFcnDsECVdcMQxsYNd3UBhgWDMa8yY6wjA/CLiODwQoeVARgFlguBaVI/
tQlCQ3VmQIaSxJALI3XUIzlHQoQPHmdGwPA4Q09rWfvr276qQHC1CQbZciHw
tSavTcw0m13OGYMuM4eB6sxlbkmSEPGC57f2ZV9JVTtYFv0zT1/EZMPVprSX
8jG/YoYDRJnLSdy/M7u8Eyd5aRZCVEdS1RxWTnDnjnOWRQ+7nbDLYJteLYgH
eWKGA0RZy0lCiHJIqmqEZdgPPHuZ7oOVE0x1f5N5TLYjyTa9KognhKgRSVUV
PG1iGXa8kqT78Omnf/TqYJjHpB1JQohGkVStCHgK8JLgK/VBm+AMwmniDhHu
tlYdDCFEJ0iqRsPiGFbtdvu20/v3XTp75mqS2uT1kYTrBKeJ++57sttaCBEm
kiqPW7c+87TJzHV6lcHmet9HMjTG7vpM7yYUYhIkVfAXrDTE1PrZR7aeSlWb
3LLzVnS9V30kK+JKRra7R1FrD6v4NMkYW0sntzzdiMFtESNGURU7aaUIitak
qmQRpwYwi23bb600RHphLvpN3Mzllp3XYtPC8kZst5wgk0bMjHN7cq6OZLt7
FLX26HYa4J5j7iiqYjehVma7XGWvjPeJHV4NESNtyke3IsWn1bbfmsVOcvst
NIhJia7flPZmriLMSrOKoFUQtU4frsfBcoJMGjGjqu3JY8l2ucr6m56WeX6f
NVvxvghP/eX495wkpSq3bFGq228tgMkZLxSKSYk98ZvYAJFiZA0dOPM3JWIV
QevLkN78JGqshaXn3noxVS+O6gU2vW6MXktizToSIA2pGlFSL0m7RNdp/75L
rNHEAGbC6wiM2VKRzTly5+QUI2uTl+p1EIYX2PS6MXotibOxXK/5lDUvy3rW
UrpAiFSq+qZNdJ0sL5Gu04XBjfScRG40ppcECaYkMWZLRTbnqOsjFRHjNZ+y
ltDZ9cpcpcvtKpXbPYodAfomfDBWMFCwxjWu/sciVbi12JzFa8ueqjYt5LlO
6eUl2nzYitlyozFnubjhJUkiTLJdpXK7R7HP2ljhmzAL1PP+Rg8+PvWOCaUH
f83Sb6Dpzx18CwdT0XAFLlWYk9CPgOvESUvCLZxyV52ScZ0YxIP40jfk02qr
DCpmKwQZoRSe9zd6MChR78CcecJdEps2Hs/6oXjMS1+WwKWKfnpKfoRHwqtO
rMs0u1zMlkE8iC99w1TnG0IIPN3Wqh7GDXag+vMeuFSlB1wnBgoSc53oNLm1
L1iXScVshegb0KaK7emzSKpagEVuzXViJDN218lrFkmnSbUvhBBNIKlqAppx
K9bEIrexu0652pTqbjUhRFBIquqCrhPLRNCMx16sSdokhAgESVVpcl2n2MtE
uHWZpE21wJ1iJUbU8WEh6qWH5WqrcP3abWYOmJcRu+sEQ3r5zQ+ZQM5ciF7V
ZcrFK/Xj5QBn94ey3GLR4E6xEmPCIurZvagc2exl2z1to8/fsoiF/pSrrYJZ
cuvoxMyBqL2MueXC8rCiGPiBRqzr42oW825sq6bVaNpzdw1Vr4CqZ/CzVXe6
NfjZvai5Cuudb67IuoX1rPaCV0+vwzMVfUZSlYWuEybPj2w9ZZY86qw2T3D3
LBeWT2xzE8XIlMizxubdmBF2vYyujz0I3MJ6dhm9enqutDGwkKvp9j5RPzgi
HCRVC5kgGF0nTJ6j3g2Eg2eGfDKCazBA527JZ+0LipEpUQguT9pwuTbXjzP9
ciuiu7FQa2tlRYY1YRAj6K1U5bpOsT8sTIrASeGMcF7MkI9acG16z1pqrOvC
AJ1b6Cwx9zBV3AwTa2tlrVtcl80W4KzqncUh05huiZXSH6mCGcfTgefCXXWK
3XVaWF54YhYikyJqKWPSPrRjOBfaLisSSC+JFarjTV8RK8IW4KzqncUhXTfN
NM4aMrqVzLs+CVEnCUsVi7G7jTMwM4cnlcCsjMEWzkL5nEaXhcjqjuYuWXQI
58KIUIxqK1rGVMkaMrqVzE3Rtm877UUd3VwRbQqIgpSkyppYsVEIi7En0zgD
CkvvifLEtZiuD2oFcIZMj8l629k0uOujEylz8+YdL+ro5orYpgA3AdJtK+wl
+cceh4mUqKWqqMFiMo1CTJ7wNO3edZ7eU9cHNRH21eCRZw4e1x3oMXV9dELk
4CZAum2FvSR/mJqi9lLmtXlpkIHsa2gNnGbt23nikqo+NP/NlacovELbruV+
NXjke/J4in7iKpF5bV4a5Ijd4t4mvj3F/RPdVElvhLbNE488Iyc1ZnYFLlV9
0KaFaOXJLXvobtdK6asRolG80ihzxf0T3VRJb2zfdnp0SZPSJVNKj40bjrkH
gCNkC/vSFypwqcJEIkltWohWntyEwzTKHgqRPKULUZYb777z8YZ1R92tB7Bv
FSP/qgHYJpb1EZ08xZ5wKIRoDXgWmMo+8/TFC4MbtfT/VQ3AFvDsPLM+Apcn
HB4O0i3EFF3CYWm8Cgzu8ro7WCWjoZH7ibnL9GmkD4nEaCICJqmqHbgbkdr5
69duU1Xh9MEUJ1OIaWF5GxfG2TNXs2ldbqjfq2vnJi27g1UyGhpFUpUVNW6U
zj1yd3XeW5GXRyyiQ1JVC5jcYvrNBAN4vhHZ+fn3P4Eds5hkLKqahXtnrMgq
vR5LuOI2LoznDr6V3SwTWgJVOTx/0F2d91bkWZ/K2x5rtRNdIZaoiUCQVJWG
+W9WDxbT71gSDFjJlqUCYcxhx8KPSRqUJDpH1CNu4aTJNXtLYxvF19Ettj3W
VN4Nb5qomaKxhJEJfRoqL8JHUrUi3HqwzH+LpR4spsewRfD7cNjc7xB+qUCm
LUGVGKzj1kvaTDpH1KNYRDZqTNFYwsjCpxY7LdKyKJ4OET6SqrHMLe9sja4e
rGVHMCyJU4DOBjsNtnK1XlVAqBLtnoxeyBRpGScYto7GFTRbO9M0Q0yIpCqL
175qz/LO1lji9nD9YC7c7Igw42D0mLiMgkO1crWqCpgeto7GFTRbO2Pw1jao
0lmWkIkskioSe/sqyKu7/FR9w10TUJvgNFl1C5gm2iUZpT5jG1S5BOkJmaXE
eG0+dM/0inIiMphe3JC1emY+7+Xs64FK1dxy4QUmv8XYvgoKy/geFCrA5Sdc
TBwSLqyrTTA1YXp5Ikxso4HX5sPzyLhGZt09JGSJUUZEoEjTg+H/7xKl+ZmZ
wdKvfbEKRKpyI3sRJb8RPLl4HuH3UWFDi++Zf8o+LPgBfwzqCEVKmEfGNTLr
7pErZEpcjJcSIjI/s3qoVBmtWib7codSFXtkz4CkwiVh/h4LP4ajsF49Q/qn
Qfl3op94QpZNXDQts20O5pfpBg6KEiKy7FQVSdVgeunXlT6lNJ7rFGlkj7gO
FIQAmhvObJDJG9HVMxTCxbTMtpWZX2aVQNyWi25HqsS2kAdO7V7VchjQ/xSX
UkdaCJOLmE+Ouytq14nYClRoDhSTN/C0WvKG5Ekkj9ty0e1IlVuYy5JAvAIg
2XpcCXttOLtaavVUFY6CtaqFoYoNXxrM3P2L2uWJYT3mknHLBvPJ4/326UA9
8/RFTOFCW4HykjdqqZbcIbiwRdX2YFu8InvZXni1DNegucPqPnG8fvEDO9QY
wwI9xJJAvAIg2SrHbv3GoluCS2wrLUHZuelga0Xu5awrFlRHBuCSdC3l/w3x
QoDVpYplIuBisAYsw3oJ5JJ5K1AwTYHs3rLs9wClczTe7NfUx+2eU1TDHLal
nafeNWjusGq6HLj+dqhun3QrDeGpWyCWSpQg95bgEttKC/tPPr/K9iCua0yt
v6u1IkvPJdxaEQaHRoZlIqBWUdSAHY1ViMV5hbYClc1+D0Q6PbjEQD3i4gLn
MEzsdNcUTH26PuQ6sdIQnrp5liqbM6BNScIl24O4luG1VsQdWH2uG7hUcaZR
78F0Akuv2yoPK8R2fVBLwGrBnvPYAnSgvEqAtMM0wtQjpmwlMIepnWzOgLcp
yRIGrGOIgo2iOrjfal9nD1yqYsdK8NH/DWqVB0fC9EKGlAM5NkuSoa9kBZes
EmBQMho7ljBgHUPcYKMJGVdMrN5R10ctQqeJUIykqnasQSHjewE6KbBLXNUN
Ib0QxwNB56ISLKQlydBXUpyqQ0zIuGJi9Y7cJT+ulDHKGmasWKSBpKoWvD2w
ATYohBlxFarDw3M3F3BfNq4bzZ3iThHBREqulDF3hc2tmMnG7slcGtPXKqoj
qSoNsyMYQAt2DywUCuaCB9mhQnFfNosBupsLZMTSg+vLr1/8wJbGGE5kIJcJ
HnSZ9e2LyZFUrYibN+9Ya0VmR4TZHMRVKJiLKjmipWEmCZs5cl+2igH2GeZ4
MMGDC5HZXpkMJIawZipCQ1I1FrcxPR6rkFsrdq5QTCbkTjFmkoTczFGEABPv
3UAi98ZyLYwZHcrwFJKqXBjci6UxfecKxXqAu3edX3PfkRr3p4s+w7UwZnTY
vjlGjxVF7CGtSVUT1f/qhakR1pg+/PLglsvXiULRgYKPCWeT9QA17xVNw5yc
0VFEFh0KbdVYlKO54rFFH9fCp5SAVeDc8uBhrj25eNnmLWdKuB4c/osjiWJa
a+UdmBg/ukANt8dWH275IxtMjcst5RTFlQwTN4rI75RfIgsHWRRRGfXx0jep
wlyLxmrPcndFTsPCn4OFo1D04IK6Ym7WmVuTwSqCmmowMX502c+6Ts0tf2TD
DjJbILeoyp/VLLWySHJgJ4SFgyyK6GXUW72pudRLnSdAH6SKbdNh3pkpTWMV
2r6nInDwmCtictjVfihcOhat7SqT0CUb/2H/BXcvj2vS4zU+rsxZzVKTYCt4
aFX+rDISxVdrhaPh3MbqTbkTG6vRYXWAVQQ4BFKVKlZiZ/Nftk3HHyO62XCo
eEZYcR2PTPuzaBwArB8eW1w6qFXLn05gb+kCs8A4phnZVfWe22Sr8meVkRj+
sg5KjIBRyKjgPb9iY7EaHVYHuKgIsJVEtsbB8nabIxmpwg1mkT0YNFZij675
L+52CBMTFXAu7VsVmD7M4WHocAB4AFsO7NM+0F1iEIwuMNs2BRVyjAVGwChk
dMqoYvQdLByqcuuTY9MDazRjjYPN23UVbfQCZZLqhpOqfVUlaqlig0W6Tnj0
LLIX3RNnhc1xV+OMOpFX3FpMzj986EprEmltyDhlZdSFc9R2DqC30HewJBM3
n4SOmElYkra0BUzRRi9QuuqW22DRkzkbbtvNotHVucOCWfsPHGpdHYGrv0lr
nwITii8I1sxtsBid60RgK2wZqMPC5nSj8IBA6NsJ9FnWJTcFsA1ZRLHZ5KEj
ZhLGZ41eGM0mjaRS6Zog22DRkzkbbtvNolFX2mqJsfnBF923ZZJAwq0V8Thw
6ZOpO7iM+ILgd8Q702OaBE+Hy0BdPfKuG9Wo1jPrEp9Co2dZl7J1EUEvjGaT
RpKpdBZF5EKYvtPYyU1bXel4952PbccHywJU39USuFQxQZqBiKifAi5CWZpE
h755a24U7kzcn9yzxmYo8U4wRBEWReRCGPSLqS+u/xVdQF5UBN87ywLUaOgC
l6qogShAahni4yJUt8lX7bhRbu7igWcvh9PsWLQDNxS4/hfEi48AExGVhShK
IKmqHfgO1vkXU80OQ3yEnaqadqOgUN3mLoqQYfyQiYiWhbh713nGTHBbyuMW
o1ENwFpwHSg8gIF0/sXjz8RCzGYbcqMgSdBlKlRXuYuT4EXgbcuMV2Rp13d/
vfOxc+5gOly2zEXusJbu2jQ6Cbg/uRKNB4frmLiLKF5wxjXbEUQ1ACvCbAFr
eBGCA2UHBt3Es4+BH5pYKbCPwLlDlztXKHefy5Hn3/3B916h0Ky9f2lt98Gp
41Mbjtm4/97nc9Oftj58EtbSHVaaqWisW3v0ka+d4vjqwyfx5vbHLQ998W/v
v+d5HM8Pvv8bt7KE7a/Rag6BuFO8MDGg52U7bVUjsedIqlYKG15YtkDnK1Au
eJYhndTNhiIq+Ai4UY1+xOhPp0/03MG/heV/dPsZCgEEYt0DR/nzvV/++bcf
PUuhsdo4+Mpcf6fGGYW7fYbD9bNwreiUPf6D3+B41q19wRW4TQ8e3zh1HD/c
++XnTc6+/73z1DLz0fosZDYDsRqJmCAxZ0OeV6+QVE0Cm/9afxD2pu/6oO4C
c1HoJp5l2LeGPDucMj8CZr8d5xFSCEd15j/9za7pXw09lJ+vue8FJsFOrT+G
L+JHe161sjZxRdtM4GwTLuSJcrZxw/GlcgcPnYAEb970IoXsoU0vQsh++Phv
XI+sh7aad4XreVnYENdEa16pIqkqwu1Nz+a/AfYHscxzPKoNqSem9JA/ZmU0
KtBcef+rv/y7H3zvlU0bF831A0MfBD8/9u3FncLcudOfKBC34toOUIgUbDIE
i54jPDII2cYNx5aE+zGY69+ZLxaXcFfEwoZ4CrjmxZoPNo0J7bEVJZBUuWTl
KdjuijgwPJiNZp6bSD138K2GPgLW+Bcv/x7aBGN73z3P33fPkXVrj3770bPw
pHCCvbK3K4IJ4XNOn3eoGK4hrPT6B2YXJWxqUes3DiWMaz29KjHBQLFV59Nu
rwSQVEUkTwvDo7WC58215HBFqvZLASvxHw9c5jIT/CYuLeGDcDrBXvaIoJXm
+g4cYRZXxEXevOlFDIZPoV//7t/+V4sidn3IbZC728st1E9vtIcB1Vjop1TF
JU8Lw0AfHiUcKkajBc8bEin4R3/1l39HeYJ92Ljh2I/2vKrSf63hGWroF3xY
RhGZ4gj/6zs7f4kvnfknPYmyuu3PcO624YuFo217AluAhWwf+kB/pIpl2Bk0
i0KeFoaPkgX68Cg1PeXDZ+HK1CVSOHgYgZ/8qwsPrDm69v7FyB7lqSdmMHxo
qM3/+tZwIrFh3WL8EP/Fz5ha/MWf/66HuXZMerGMF7YAo6bTEXMLntuOA2V0
NErCUuX2pscNxjLssUSZcOdDTPF0NBroM/CUMQe4Fh2B5D3+/Vfuv/fImnuP
fOPrLz//83fkPcUC44fwrXD7/enOX8L5whxj49TxB4eLXzsfO/ezn75BzyuK
56h2bJXQ0l2so4fbzsPt5cEAow13X0Oq6sYszXqDP4lJFXvTQ5K83vSxLKHi
K6ZC4Q5vp7MhrhhrBVR/anCd/+WP//q+e47Avk1/51e/PPff+mnNEoMuBsWL
kUN6XpCwB9a8MP2nvzp86IpajHm4vTwYYLThdsrwmlXl9qvCwBUuKpDidmnM
HZ2YPutXxb2NOMjqhxG7VHGSQycdFp696c+euRrXNN4K6OHWxVPfjoWHDuIR
2L7tdMXCgHifQ//5bZgviNRjj549/+trPUkz6y2WX/dv/vV/+eY3XuaGAsbH
vvbwScYMe+t2VSTbr4rzhCKpcrs05g5rxlFxuE2NJxncb+IOGLeE+1Xlwhzd
/fsuwcwydMxwenSPhlviteUCelyWwiNQZbZz8+adHz7+KhQKOgW1iu76i7rA
nXxhcAMzn+9O/xpu19T6Y5sePA63a92aF77/vfORPp7CJVuVZcJ+VeywUD0e
GEW5Wj4IXltzvBLpUi8eW0b5OinxiovJReEqpuPSG/+T5YB2fffXv5/7PzUe
nkgA3NKMev3Zj3+7ccOiZlG5Htr04t69Ay51xRKTFyVgbR/8txYTHUW5WugR
1BkmHT5U1LnNzOXDWUytn937xKC1KJ8LZjVcxauyrPDXv72x5aETa+9/Yf++
NzRPFpOA+4QJTj98/DdwuB5Ys5ingXnO1x4++bOfvsENTV0fowidwKUK1jXq
2ZfbCReODJS3q3Uc7pbCf0u/w3+//n/hScHOwCXUapQojSkXo4UbNxzjOte2
b7zM8lnaziCyBC5VkQLvj90VQ+iEi0/nbqnS+oJ/+JO9F9befwSelERK1MvN
m3cY3n/s2+dYpHfDutmN649ZqLDrAxRBIKmqC9hwhvi4CBVCd0UYATh0GFUi
xu+9+/HmB1989FtnIl0ZFHHBhDfctFPrF3cibx6W533s0cV2BrFsihRNIKmq
yPVrt2ePvrd713nW5cP8MISn6datz5ghWdGhOzjzt+semH3pxN/XdWBCTI5t
k9z2zV/c++WfP7TpxXVrj27Z/CIetLqW7EUsSKpKwBUoa08PqQpn1zlF6pGt
pyCaFd/qp//+9Ye3vCSDIELAis9w2+xXtrz04NTxjRuO/cWf/y66fZSiBJKq
CWF878Czl+Gq1LVToF6gKXWJFPjJ3gvb/8XpqHNaRMIwTvijPa/ed8/zWx46
sehwrXlBspUwkqoRcCLHBAnG98IMO+CxZUfUWkQKvPrq/5hafywoIRaiiOvX
bkO2/uzHv3Vl66knX5NspUQ5ERlML27IWj0zP+HrcUkVi8awJgkkIIQEiVyg
pNAmyCgOtcYkw08//X9r7n/hvXf/V11vKERrmGx96U8Of/Xhk1/ZcmL92iXZ
CnCSKSanjIhAkKYHw//fLUpFrwcvVXAfLr/54XMH32IYnJWaQs6SvXXrMxzt
ls0n9u+7VLuMHpz5m61fPVnvewrRPniEDx+6Mv2dxd1b3/j6y/C2Nj14nDVC
Q366RS4QkfmZ1Ut1K7LuUB74+0NF8jWp6PWFIKWK3ZRYvWFq/SznXeFkR+TC
khdw9LiZt6EA3davnoJaNfHOQnSCPezf/PrLU+uPfWv7WXb5ZBkcKpdqPQXO
olTNL8rK/Mz0REL1hfPkS1LR6wtOEafWSjnlQu8JEy14T2vuOxJRNNuag2BO
2PSG4nVrj/7i5d83+hFCVAHP7OSlU71hmx8hVWdOX70wuMH+v1aE3EqI43VW
L2eLEw4t4LZJjnBkY3bFxOVV3br1GffCb992mt7TyZfmAveeDBw8jhZHzuYg
7TwmG9bN7vvZGy18kIgLPDW5lr+oPwUGpoWTNIxgm/jJB9dnq4/c1SsrIY55
LM8C1sP+CWxI7mHg77gnjssSQt+orsB0uvZ+30MRGUwPNWZ+MJhIroJfq2Jv
ejgg7GDFUrcRLariYcEBQ55w8HjYWz5yPoNROJvCI7fbEe6lrIjgocia7myn
P6/fUPafWN/23MHKSGNHRM+mi+vceRcZl2V03yhrfO8Nc+VyxyQXM4QSBDgG
6wCCO62WwggQEVuqWo7fjefuTD838tdNBiBude5qx3fNZwo/45UQvrXJoULh
FKCwmKR14vphOrT3iQGOBIdRV+q7GEvWYcHFH+ueTNhD1pvwc+D9s1YulmhD
Aljje2+YK5c7JnEVGdvsfGzccCzrC/etteLCcNWJW9eHtcJmt287HW9ij6dQ
HZ4CHh9cTx4ArjCmQzCGsGm9il2siElMjRs+4oCgjHVYcPHHuidSFhEmsB7m
RbJnffUV9oikCg8mTDpL27HrE7uLRrrQeevWZzBrISgUwWV86snXvDsK15w9
trj9OaWo4GgvJrcVeHbqOEkAx12U51CTC5E2MCO1F0QNXKrY0p2BDvyX7Wwi
jWwTHDxmyOwPglPrXKEIJGlEbVsmybMLM2Pshw9d4Tyh2/Vib2mGnWdtWGZX
dkE8qzKeF4OBd/MkpqvTFEIELlWMqCQQ6IBVh+WEPEERYOfDcU8s0Dfh/Icx
dqgto6/uerFr+UfE28eOrDuTqzXZpRn8Q/d9bL+M0oyFiJ3ApSpqYPzhA7IA
O6w6LGdQkR+IDlsD15U+4epCFanKujPSGiF6jqSqdliFbOeOcyx8FFoB9gVH
pKq0BhZCiNaQVNWC60Dt3nU+qBCfiytScWXyCyH6jKSqClyB2r7tdLAOlAFh
mj36nkRKCBEjkqqVwvje3icGbBESfnMBHN6BZy9DpCBVEikhRIy0JlXdFqqt
iCtPjO/Bnwp8Yyy7WeFo4fS9cv564EcrhBC5tCkfMYpUjPJE5t//hGXY9++7
lECevxBCSKpcmGUdozwtDFejWIZ9545zrZVhF0KIFui5VLGWIDtYWf/fiORp
YXgKFwY3IK9wo9ovwy6EEC3QQ6m6deszKxPEzbkRdbAyqFBPPfkaTmH/vktN
91sUQogO6YlUsRYQDDs7WMVbfNVTqCpF9YUQogmaKPufqlRZZI8FsVlhFYY9
0mxtaxQihRKJYV13kxkKwsMLYPvIGmNWKUkVrg/suRfZi7ogtpVhD6RRiEgS
t5vthFUcWam4aGQLC48YuLcn6RgY0di+7XRzLQtjuVxT6+9qrUjZ6mFrRcLm
v+Y6waTDnkca2XOBoQizDLsIhGxjeta6H9s4ONvVkQ9O9m+Olir2fykayj5t
jiic0Hff+dgaLuCHp558DWa5YrHuuKTK1SZcATb/jd11IvgerYogzi60Muyi
Rjwvhv2s3YHbwBOO0d1P9ix3EBvbOFg3lWgBeE8wzrU0/zUClypMz9gvL6tN
EeWTj8CqCML4BF5FUBhZp8aTCVdE4LOM9mLYz9oduA289+/6jIXomMClitlu
eHiT0aaF5RwJnFcsVQRTxYul4Isokhtv9SHr1Hha476t4rdCVEc1ANsBOksH
ynIk4C0mI74hcP3abVMHzHBMNXCpiyJp3go1vp0iudFcQohOWOXQzse18CkB
AvsJo7d713lzoDTHHosbXmP4N3cRB0Lj6g4usv1q/75L9k/gwCqSJkTsSKpq
B2IESbJCgrCW0ZXCqJ0//OHz7LKO6++4mWlueI3h39xFnEi3yAkhSiCpqoWb
N+9g9o45P6b6O3ecY+pLf+J7TGkzD8g23Zjjw7RGb1nH9XeUmSaEGIGkqjSu
PG3fdho+Aub8CU/16RnhHKEy0GJ36YcpbeYB2aabhK+GEKJNJFUrgk2sWIIv
bXmCo0QvCSdLSaJnBGnGi2fPXNXSjxCiNSRVo2HmHlOXuXbPWoLp7X5itvb+
fZe4Dwj/pZeEk5UkCSG6RVKVhV1CmFjOzD2mLnd9XDVz8+adC4Mbpk3M1sYr
ylEUQoSGpIpcv3YbPsUzT198ZOspdglJL7EcHuLccptjBjChU9ImIUT49Faq
vC4hu3edh08BZyqxVDTzELm3a89ym+P0AphCiITplVSZ6wSHIo0uIblwY5fn
IWpvlxCiHWBtas83S1uqWImdPoXrOqVXJ2du2AaC+465sSs9D1EIEQWwPNzG
UmOadGI1ABnWc5djUvUpvNREtoHo1b5j0Qdy2z6OHm4FyLqGV0myyvBKTYY2
uDml+ti08bjXUADWuFxrxWRqANKhYLyLYb1Ul2PYEgWSZEUFk0xNFGGSqxrc
Ep47bJ947piw6W1u28fRw60AWdfwKklWGV4B/9AGN6dUHG5rRZjlWqpzRydV
rJlAbbIsa1jvVONdcJxxaviucbIQYsx5cO7pOYmiCZjz6Y5ss+Dc1vO5Hedz
VYNbwnOH7RPPHV1fG9EgMFmYTtfb3ih8qaI20TOl32TalGqWtckTXOYtm0/A
GmBSl+rJiiKyQjM2VuOJC3M+3ZFtFpzbej69iISIncClCrpMbaJnmqTfRKBE
0KP9+y5Bjk2e0kv/6CFey2DXr8HX7eqI1y84KzRjYzVdn6sQTRG4VKUNKwoy
AwRmCm7UhcGNhOU4dooaaXF3ng2vkZbXMtj1a/B1u0Ijx1mIIiRVbWL1ImCy
2M2KaXuKt3SCG2FjOmXW2XF1p6iRFnfn2UiyfLEQ3SKpaprcehEwaMoqb4ib
N+9kHZ/cIJsbYWM6ZdbZke4IEQKSqibIrSiotL2KZLMLLHvNFaDt205nHR8F
2YSImnIiMphe3JC1emY+7+Xs6+lLFebe3PG0J+mKgs1hrpDtOjEZMg3KZhdY
9poESIi0KSMiUKTpwfD/d4nS/MzMYOnXvlglKVXM5sLUfcvmExjc8aQsrCLg
VOLivHL+uhuRgwfkuUK2l99kqOsDF0J0TwkRmZ9ZPVSqjFYtk305DalyV51s
6zGcKS1nGMyRY2UbJidYaA4XDX888OxlNyKnbHwhxCSUEJFlp6pIqgbTS7+u
9CkhYLuP9z4xgN+kVSeDuXMsqkNJwsWxHDlWtmFygkJzQojqjBGRpdWnZYbK
NNqrWg4D+p/iUtvR1w0tMKs2sVHInuUGT312nVjSH9cBl4Vp9sydY1EdSpJW
5YQQ9VJVOArWqhaGKjZ8aTBz9y9Clie3iRUtMKs29TM2RUcSF4QpIvSVdu86
T1XCZVGavRCiferIAFySLtcD80KAQUlVf5pYjQWuIr1I9jiAUtORxAVhioh8
JSHESrny9kdsU1WjAenDviqmQ9BNgDVOuInVaCy8acK0ZfMJepHscSB3SQhR
HcyB3SYg+/ddql4yLkmpwjVhpQK2M2A6BN2EXlljXAfWC2JRegtvSpiEqBGG
zZsYNTa3yu32MuHIbQozenitFWF8YIVgjkpf5DSkav79T8xvojaxUkHf2hnA
T4QMsaUd7g1cB9YLSrsovUgJ7r8rbbpX1K7XK2VfejBs3sSosWVwbreXCcdK
rag1rGdfxSoKZUQqVa5NZtM385t6qE24GWzdDT+zpZ2cJtEQuWpS1CY+1wIz
XSd3cP9dadO9ona92knREJxU1Lv6H4VUWeff/fsucSLk2uS6DjIKcrUJr/Rt
3U2sFKbQjBYXr3/WnkxDkxFqUtQmPlcj5OOLlRK4VMEIu51/8XD1bSKEmQnO
2vxHaVOf8dZEuNmtSGg8z4UpNKPFxeufNafC8iIYApcqPCl9m4Axl97NV4RJ
6aH/mDZWnjerOK6a0HcuWhPhZrcioenbgyPSJnCp6gNuvqLl0vcwXzFqTCDc
Fo1e2pWbRmXlebOK48qNfGchiKSqfbK59JavKG0KB7c9PWYO1BFGYrPrOPai
26LRS7vqVcKPEPXSmlQFXv2vUUZok8xX+2Qzn22Vxw24ue3pDx+6wr/JSKzW
cYRojTblo28iJW3qBJbjmHNa1dsuG1eDspnPtsqjgJsQYSKpqgUvF0La1BCW
jeB5Q6ziwnIce5xW9bbLRhokRNRIqsqRzdNTLkQteElxTEuwqgKWjeB5Q7rm
QqSNpGpCvJietKk0TFewGB3FyDYBeUlxTEvo22Y6IYSHpKoI12/SetNK4bKR
ZW5Tj5itzXQFi9FRjLQJSAgxAkmV4dUVdP0madMITJIo60xg4LKRZW5Tj3QZ
hRDl6K1Ucdp/9sxV1RWcEBaRg5rPDruKWDKDSRJlXQkMQvScC4MbbFl7+c0P
69rT0R+pYv00WFS3Nz0uZg/rCo7FVSXIEB0lFpGDms8Ou4pokU4I4ZYIs/Hu
Ox9vWHfUrc0Cu1Gxu2LCUmWJEDC2MLOsn4Y/9rM3/Qgs446+kqdKV97+SI6S
EK1h2wPHjpU2Xpy8mdeILi3ecEuEuWNq/TGvGn/FtiApSRUsKoQ7u7kJxlaF
BQhdS9wzvEq8IS3jjr5S18coRLOMbdpr6aljx4SWf0VddG174Nix0saLkzfz
qpjmBElidWUYFkyDa1mkjleqGKRiQM8Wm/bvu6QEcoOXCMLEXBHcOXQt8Ude
JeXdiRAo8iOKJGO0AffK0Zdo2mvpqWPHhJa/bwlFbL5Z73tGJFWu04SbjUEq
BvS02LSwLEzQILr5mJvxEkGYmCvSt+dFNAeeOM8a51pyt7B8bpnf0X5EkWSM
1gXFq5Mk8HK1uPHgKFnHW3OaZHUXhhfHsutNmCDldPPlV4oivI7A3pIHZ4Pu
2L7ttCcuO3ec8/5Orqa4heVtKBovVkQU5Wrnhm21NU1aGGaJcPsSppo0Hcx8
oMckYeohbvKVFysbvVbidQT2ljw4G3SH0pBE5wQuVb3Ftn2Z0/TI1lPcvgSj
JNORGO5C/wjR8fKy3OQrL1bW87USkR6SqkCYf/8TS62HRbJtX3KaIsVVnwuD
GyYi3BHAkbvQP0J0lAYjeoukqhMsBcJW4nbuOGep9bJIwWKtgd3G9GPVB9+y
/WXuCODo+myEiAZJVTswBcJLX8QftRIXCLbuYx6Q25ueJaTc1sBuY3qpjxBN
I6lqAtg9N5pnKRBKX+wEi8WZDJkfZPkGtu5jHpDbm14BWCG6RVJVHUuB4IYm
2j1F81rDUq9ZtNC289hWUIvFmQyZH6RpgxBRIKkqgec0WQoENzR1fXQJYitE
XikzxuUs9ZpFC207jyKrQiSDpGoscppawGJ01Jpnnr7oukW2QuSVMlNcToie
IKnKIqepIazU28mX5kyPuJ3ZYnQUo9cvfiC3SAhhSKrkNNUOF4+Yw7B/3yVc
VdYTtlJvhw9dMT3SdmYhxFgCrwHYBPPvf8L2TNz+L6epNFR5eqBM7aaLxMUj
5jBAsPB3VE9YCFGCVQ7tfFwLn5ILF0G8zbZsz6Tt/5PDqv5Wy5RXkipPD5Sp
3XKRhBBNkJ5Uub1CrPmvNttOCB0lxu64lsSdR9AmLiRxa5iupBCiTWKXKkuB
sHrjbq8QtRsYzfz7n7BAENfpoEp0lBi741qSdh4JITonLqni/hpO+BmDshQI
1RsfDS+dtaqnrO/ccY4FgrhOJ1USQoRJ4FIFt8hta8v9NZzwKwY1AnOXTNN5
6axVvWRdCBERgUsVJEltbUfD1SWrheu5S9J0IUQCBC5VwqNodclq4cpdEkKk
h6QqZOAQWdIId9FqdUkI0UPKichgenFD1uqZ+Zzfzc+snh7U8il9w7LEIUaW
zWhJI9pFK4ToLWVEBEI1FKPBdFashiImqRqHt8C0ZfMJ9rRilviVtz9SHE8I
IYwSIvKF25TRqsH09EBeVQZ3X61bzckWmLT/SwghRlBCRJadqoxULQpVYQCw
5VJOHWKlnFgWDx6Tu69W1ZyEEGISViAcXJQyhspU4FXhZe8v3vWJjZxJAFCY
bGvt1PpZK+XEsnjymIQQoiJ1r1UlnlZhHUPoMcFdojDZ1lql5AkhRO3UkQH4
RURwkbSkimtMlvxgHUPoMWlXshBCtID2Vblcv3abrazY/9ey8pT8IIQQHdJn
qbIei+4+JrayUv9fIYQIh/5IFYQpW/nBhEn7mIQQIlhSlapcYVLlByGEiJE0
pErCJIQQCROjVLmdfyVMQgiRPOFLVbZLoNv5V8IkhBDJE7hUnT1zVV0ChRCi
5wQuVUIIIURrUtWHQrVCCCFqpOU65xIpIYQQpZFUCSGECBxJlRBCiMCRVAkh
hAgcSZUQQojAkVQJIYQIHEmVEEKIwJFUCSGECBxJlRBCiMCRVAkhhAgcSZUQ
QojAUQ1AIYQQwaIagEIIIaJAUiWEECJwJFVCCCECR1IlhBAicCRVQgghAkdS
JYQQInAkVUIIIQJHUiWEECJwJFVCCCECR1IlhBAicAKXKmlc++iat4+uefvo
mrdPlWveH6lK+60CPCS9Vfvvo7dq/330Vu28T+DlagO82mG+VYCHpLdq/330
Vu2/j96q6fdpuVytEEIIUZoWpEoIIYQQQgghhBBCCCGEEEIIIYQQQtTFYHox
5WP1zLz/i/mZ1QW/ESUous6F119URvd26/DKTg8yv9B93hiF1zyp+xw30PAU
B9P+6QxmUji/UCi6zsXXX1RF93Y3wD5mzKbu82bJu+Zp3edfnKF3Dy3JcZ5S
i5VTdJ0Lr7+ojO7tjsgxm7rPGyZPqtK6z5cnOwW30OLJJnKm3VJ0ncdcf1EB
3dsdkWM2dZ83TL5XtfSb6O5zBouN4R0zdrYzPzOtO6s68qraR/d2R8irap9i
qUrmPh8XQx5Mx6bIYaK1qvbRvd0NWqtqn1FSlcx9fndmztIttRTmTCbQGQC5
1znzuqgT3dvtsxS8Wbrous/bIPea6z4XQgghhBBCCCGEEEIIIYQQQgghhBBC
CCGEEEIIIYQQQgghhBBCCCGEEEIIIYQQQgghhBBCCCGEiJDlfqgzi/9XoWkh
hBBBMmyGOjMzI50SQggRLoNptUsSQggRLuw390WnPyGEECIk5pfWqLhmJddK
CCGEEEIIIYQQQgghRF/5/+4ak7w=
    "], {{0, 310}, {569, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{569, 310},
  PlotRange->{{0, 569}, {0, 310}}]], "Output",
 ImageSize->{573, 310},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[30]=",
 CellID->100775212]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3.4.5 StreamDensityPlot", "Section",
 CellID->2809397],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function StreamDensityPlot is used to plot the scalar pressure-coefficient \
field as well as streamlines.  Compute the list of surface pressure \
coefficients using Bernoulli's equation and record the minimum pressure \
coefficient for use in color coding.  In the color function, Hue is used to \
specify color, saturation and brightness.  Positive pressure coefficients are \
shown in orange and negative pressure coefficients shown in blue.  Minimum \
and maximum values of pressure coefficient have maximum saturation and a \
pressure coefficient of 0 has a saturation value of 0."
}], "Text",
 CellID->176900651],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cpC", "=", 
   RowBox[{"1", "-", 
    RowBox[{"vC", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cpCMin", "=", 
   RowBox[{"Min", "[", "cpC", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"colorFunction", "[", 
    RowBox[{"c_", ",", "cMin_"}], "]"}], ":=", 
   RowBox[{"Hue", "[", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"c", "\[GreaterEqual]", "0"}], ",", ".0483667", ",", 
       "0.633338"}], "]"}], ",", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"c", "\[GreaterEqual]", "0"}], ",", "c", ",", 
       RowBox[{"c", "/", "cMin"}]}], "]"}], ",", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{"cMin", "\[LessEqual]", "c", "<", 
        RowBox[{"-", "1"}]}], ",", "0.6472", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "\[LessEqual]", "c", "<", "0"}], ",", 
       RowBox[{"1", "+", 
        RowBox[{".3528", "c"}]}], ",", 
       RowBox[{"c", "\[GreaterEqual]", "0"}], ",", "1"}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellLabel->"In[31]:=",
 CellID->460900760],

Cell["\<\
Compute the scalar field for pressure coefficient and prepare the graphic.  \
Since the flow interior to the airfoil is being computed, there are abrupt \
changes in pressure coefficient across the surface of the airfoil at most \
locations.  Although this graphic provides the most information on the flow \
field, it is also the most expensive in terms of computational time because \
the value of the option MaxRecursion must be set to a relatively large value \
to reduce numerical artifacts.\
\>", "Text",
 CellID->76766512],

Cell[BoxData[{
 RowBox[{
  RowBox[{"cp", "=", 
   RowBox[{"1", "-", 
    RowBox[{"vel", ".", "vel"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gSDP", "=", "\[IndentingNewLine]", 
   RowBox[{"StreamDensityPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"u", ",", "w"}], "}"}], ",", "cp"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"ColorFunction", "\[Rule]", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"colorFunction", "[", 
         RowBox[{"#", ",", "cpCMin"}], "]"}], "&"}], ")"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}], ",", 
     RowBox[{"FrameLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
     RowBox[{"MaxRecursion", "\[Rule]", "6"}], ",", 
     RowBox[{"RotateLabel", "\[Rule]", "False"}], ",", 
     RowBox[{"StreamPoints", "\[Rule]", 
      RowBox[{"{", "tuples", "}"}]}], ",", 
     RowBox[{"StreamScale", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Full", ",", "All", ",", "0.015"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gField", "=", 
   RowBox[{"Show", "[", 
    RowBox[{"gSDP", ",", "gA"}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[34]:=",
 CellID->146745562],

Cell["\<\
Prepare graphic for surface pressure coefficient, color coded by the same \
color function used in the field plot.\
\>", "Text",
 CellID->614303007],

Cell[BoxData[{
 RowBox[{
  RowBox[{"xC", "=", 
   RowBox[{
    RowBox[{"Components", "[", "rC", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gCpSurface", "=", 
   RowBox[{"PlotSurfacePressureCoefficient", "[", 
    RowBox[{"xC", ",", "cpC", ",", "rP", ",", 
     RowBox[{"ColorFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"xC", ",", "cpC"}], "}"}], ",", 
        RowBox[{"colorFunction", "[", 
         RowBox[{"cpC", ",", "cpCMin"}], "]"}]}], "]"}]}], ",", 
     RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", 
      RowBox[{"Thickness", "[", "0.015", "]"}]}], ",", 
     RowBox[{"ShowAirfoil", "\[Rule]", "False"}]}], "]"}]}], ";"}]}], "Input",
 CellLabel->"In[37]:=",
 CellID->318266012],

Cell["\<\
Display the graphics for a combined flow field and pressure field, with a \
plot of surface pressure coefficient serving as a key to color coding used in \
the pressure field.\
\>", "Text",
 CellID->42228577],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphicsColumn", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"gField", ",", "gCpSurface"}], "}"}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7.9", " ", "72"}]}]}], "]"}]], "Input",
 CellLabel->"In[39]:=",
 CellID->448653773],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzsvQeUpcWR56s1772z782e3Tdzxu7OzM68nd0xGiONNDOySEIOEEIYCSFA
Eggk4YQRVggnEF54JDw00I723tG+ae+9rfbee1PVVS8zIzMyIjLy+75b1Q0U
+r4T3HPrdnV1VXFv/u4/4h8Rf3Hptedc+h8+9KEPvfnvP/ShT5j/7P22+qqv
+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv
+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv
+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv
+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+qqv+uqE14fq
q77qq77q6wNxvdc8OYlX+366E/I7qb/I+/aLvE++jfqL1F+k/iLv8j/9vr3a
h+PO/v+0/iKd4tuov0j9ReovUvHza1V1Av9WfZVe9S/2ZFz1b/UkXfUv9iRd
9bGcXvXv5H111b/Yk3HVv9WTdNW/2JN01cdyen2wf7r6qq/6qq/fkuuDfZgX
/3Qbtx7cuIXFBhObbaz3cWD9Jh/rNtpYS2PDgTUb9kOsXu+jycQ6E/sgVq31
sXKNjxVr9tpYvXc5RNPeZTb2LFu1Z6mLJSt9LF6xG2LRch8Ll+3CWLB013yI
JTvnuZi72MecRTtMzF7oY9aCHTMXbLcx38aMedunz9tmY+62aS6mzjGxders
rVNMzNo62cTMLe+4mDTDxOaJ031MmL55/DQXUzePm7pp3JRNY12MmbwRYvQ7
G0dP2vA2xMQNoyZuGDlhvYkRJsbbGG5i3LphJsbaGAoxZu2QMWsHj44x6O01
A02M8jFg5GqI/iN89IMY3mSir4lhPvoMbeozzEbvoasgeg2R8dbglTIGrezZ
SPQYtCLESozs5w/2d5R/d/DK9Nszgd88BPxEfcLP2Nf9yP1C4O+kf/gtDRgV
f3XmNwlBf71D3C986Bj/+7f/L9z/lOHj7P8g+D81wv2PMzHK/X80/zffDv9n
7f9lF/A/HZ4D5slgnhLjIabZp8oEeNrMgGfRlknhSWXCPMemuOebedbZ596c
bdPm2KfidBfmKWpjvn/Gzlpgwj6N4fkMT+85i/yzHZ785lVgXgsLXMSXSXjh
wOtoyUoT/lVmwrzi7Ouuyb4G4cW4wgW8TvGVawJey/gaNxFe+P4osIfDJh/2
0LDhTxIT/njZamPT1kMQm7f52GJiO8bhrSZ2+Ni2M8Z2G0e277KxA2O3jZ00
9hw1sYvE7r0y9uyDOEZjr4n9LPZhHMBo3k/jYIwDIg7JOOijxcdhH4dIdOQw
7+xX8U9nnjCCVgmqDnpOYRBUrdmIz9j9a8jT2KFqv0TVWoKq1Smq9npUrVRo
tSjQamGg1YIyVFFOUVTBIQCc8qiaszVyarbjlETVFoCUDccpC6mUU+H4gtNs
1CR7vo0Kxx2gKnJqHIPUkNE2EFIQKqcCoVZHQg1HSK1ykYUUBQTigwPI3DoA
DWwoVhb9aZ5iCbZWqfDytIIIzOojgRVpFYE1cs0ADqyUVhZYhFbDITK0GuX+
t1JaeVQ5WgGqxgVUAa38m5wZmycFYAVUEVolqPK0AlS5py7QCp/V5hk+V6Jq
l42l7qUROKWiCmmFqFrWFDllXp4r1zBahbed9l2o51RE1X4FVZZWByOqtkRU
IacYqginthBOSVTtOoJRjCrKqV0VOLW3fZw6kIFUwqmDLCKkBKdqVBX8qY6q
LfHtUDGqqKrKompdA6hKhNXuxSsTVDlawbtHxymPKsKpDKrma6iao6OKcsqj
aprnlJNU7lwCVKmcInqKSSrBqbHrhnA9JSE1iokpVFJ9iZKKpzdXIvG0H7yK
qqe3KKQGunB3ekAknOoOMcDFQHKrfUj/SgG2egxMZFeisyiz7E80JKOwhhFg
ud8Pk1cjVw9MFFaOVkOjtrK0QmAxWk3kqOLCamyg1TgQVoxWeVQhreYSWs2z
tJrJhZXnlIaqeQ5V8LpYkFFVjlYKqqKqWhNRFVVVQFUiqfbjUYCoWi9QtUVH
FXKKoOqwlFQ7yiTVrjJOZSVVwql9GqcOsFDFVKmekpw6pEOqRlUlVG1VVJXI
AeZQVZoDbAhVS4OwEgnAxSWqapd9qS62gdm/OQxV2wtQ5U4Jf2i47J89Riin
OKrMe2aPKscpRVLhmTZygqVVzPuNXz8spP6inhoj9ZQ9VN/OiakIqT6EU1Vy
fUJG5aRT5E6Ibi7wTkF0T2Mgi1LBVZAe7E2ygl5eDdVoReXVCMwHrq5Iq2GU
VuPWUW2F/1tVYRW11VSeBpwmhRVB1RbzfJsyKyOs5m1Lc4AUVUireVRYFaJK
U1V7ZQJQoGpdFlVrbVRGFc3+bbORomqrkFQ7KKcqZP+qpv5K8n4qqtqR98tB
KsepGlUFfwrPnAJhRXOA68uEFUPV+oJyVUg1BFotC7RqFFULPKdKUeVV1Yy0
UEWrVFxS0exfRJU7glBSIafGJJIKVFU+71ekpwZYTq2BXJbCqeElkBKnvUjx
2ciJJoKn7gJP/ZfHgA/Fg/hHNuBPI+NSfglsuW9vRUFtS61kQVawb1rDIvKq
fyKvGqBVLhOoaauYBiRFqwlcWyGqoFwlhNU0Kqzm+YpViqrZHFVIK3jnVqqq
lmZUFSYAV0ACcK1MACqoQk5t1DiVy/5VlFQ7OKd2NiapOlSiOtBRSVXAqRpV
6lUVVZmKFaUVFVasYpVFVa5ctVcIq4CqPUubQrmq0FmxgKJq6S6fAOGeijma
p2KGV1XUUJFm/zinZvgS1QRM/cUq1cacpBqJboogqaieGjJ2LeXUoLcDpzyq
XJGFOih4xs9zCmtSTnH0GrKSJvpK8ESydt1yPCLRFW77+ftVgsBruSeXqrxS
neVu08QgQ1WmhiWSgUr1KsgrWbpCWqWlK54GjC4L1V9BilZUWFFUiRwg0Eox
V8xjOcBZQlgJVbW43aqKo4pwaiXL/qmo0jwVJwJV26SkOpJKqp1cUu0qK1Hp
VSpVT0lOVdBTxWKqgp6ycaRGVfYiVc5yVOmqKiOsVhehSpoA4cUSTYAJqhal
qFoaE4AKqhbqCcCoqoKkAlXlOTVbl1QT0PgHkmqqlFRK6k+VVBXrU5D0Qz01
fLUmpppU10RPkuhTpNNAmdbzCqh/NQb18/Emv4MfxsDPT8nFxFeh2sokBgWz
CLD06hUAa0BqDkycgR5VjlbDtLqVp1Vx0Sqh1YQkDUiElahYbVOElZYDzDgr
EmHVaK1qdfBUyELVvqqFKs6pDcWcymT/ylN/u4ryfu1O/XW8PtVOTh3x0ZHD
vLNfxRM5GKq2FuYANzNUleYAV+fKVWt1v3oGVbulA5A41YtQFczq0lMx30kq
lv0jqKKGipleUk0kxr/xoWIe3BSbEFXMTVHAKV6fGjSamf1O+czzIvpp9omC
jB8tSKUpPsEmiacIoGUulr/p75NH+i5Toh+N5TQkvFRs9V+BsosBSyisPLNk
DSsoLPjtKcnAQlphMlBxsE8ImUCVVgRVtGjFcoBKxSoRVoWomiXKVTQBqPnV
S1G1PEkAaqjavzqoqjV5VYUJwCqSasu2vPFPk1Q6p9pdoqqgp9rho2jIRIGc
Ohyifcf4B+OqrKpk0QpppbvWy3KA7URVQQJwBa9VEQcgRZWz/+3MomqeVFXR
UzFLz/5R79+4VFJNlsY/yqnU8pfTUymkHKdWMzFlT+NETLmkXzjJV0gxxcpP
mnTiZIF4I7DpjX6WRG+0K+xfZOTSKKZkCxNgJSIr15BFgNVE84F9Q++VMAcW
04pqK+Zgn8C1leqvmMKF1TTmrxA9Vh5VqrnComqbKFfNUstVwlaRqCqgVTYB
uFomAAvsfyCswFLlUZXhlIKqRiRVQS9V466/CpzK+SgKOEUIdaADeb9SVFU5
zDv71XFU5XKAXFjtbygHKFG1mqNK+NWFrYLkAJmqWsJQlfNUgKqaRlDF2qlm
RUkVs3/5KlVM/RVyahjhlNFTKZXARyHF1PC8mBocxVQsSJEWJ5blo1Wnfime
Snj0eh92PxdvYBTAK6owRXZhUYzlBoMwlApL87ezfCBpvxJWdswEDsj0COv9
VuPXD1dbg4khcEyuKXja5pgGBGE109Mqda3THCDtBZ61QCtXJapqPlVVFWpV
y3mtynufMp6KmP1LPBXrGpFUtJ2q0E1xmEmq0i6qhFN6F9UJ4VS1vF9FSNWo
Kv7pzBNG0qooB8jNFZvajyrprEgGViCqsg7AXAJQQZV9gaOkiqiCQlW+nYpl
/3iVKu2lYu50kvpT5ZL0+wVHuoWUO05p0g8O3kJI+aZd2uUUILWiW39WPCpi
kwMNoc9SFUldervHza29QyP7t/wXT+EVmCWARRKDugdDTQnmWob7Euck9VpQ
WuVsgUl3MLdYlKUBU39FKqxKzRVZy3rGry5aqyrUqvakoypytaqC5t/1m8iQ
ioyk2qS1/apuim1ET21vF6f2QJwkTnXMlA6VqcM1qsjVIKqy5gq1HXitlgNM
eoEZqlZlULVctFYhqlJVVZgAnEuyf7MXBE+Fmv2bm9rUizzq44RHXTOop1Ty
qT9qTdeGUZiTUwxKYmKKlaUop2RLFOKpm57i89WlvEqK6DFIandUgpfCrGWi
tkU8hBFYicU9AktJBvJxTLFupU1hqqit1GYrhVaqsKJWwDJzRTpkSe+uCibA
+akJsGJfVZIArOBU1z0VUlJtqWD801J/20pTf2UlqvZxal8pp7KQqlacOpLl
VI2qgj/dvM2/1VHNFYoVsEqD1XohrPLlqrQROEGVSAAu0lA1P4OqWQsLUaWo
qogqKqkmiOxfSP2lYAJJNRzdFGNZ6o8N93t7zQDfP0U6p8CXzu3owjvBXOi8
aRfTfWkRStVQpUh6rZfy4WvujhLuj0r4xSkmpNabPD0odZajsLS4u1+CkFeq
ku3UIVDFnRXcry5bq0odgGn/L1FVOFIpTQAWGCoyEyoSQ0XWTaFwSkqqowJV
laZSnFBfenvElJb0q1EFVymqIFi5qqBilaBqneYDLBZWbG6tcFYUoIqbAGOt
iqNqTjCr+wTgwiJUTQ2oonP/1OwfGCrEAcLcFNqgv6Hc9Zf2+bImX9BTorc3
nd2nWfu6I6Gil28Z01AyuSc59VoBg1gsebXX0lffcrf2vo3XfGThpVIs1Vkn
6mBn1Ss+87ZvcAb60hWbaFGireCNR7RYTGBFq7fV2YBT+GxAUrHSzRWgqmbL
HOB7zsrSQJv6+nyhqrqkUhqpNFSVlajaq6c6wKlDCarwVwSQqlGVu6qialul
NGDRSECKqg3tLlel45USB6CcAbgzcQCSseqaqpqWzKlQClXTN6evx3FOUo19
x6sqMe4vcoqgKh2dNPBtlvcTnOqbOChEVy8YJ5iS6r9CSff1Xf5moqHy6mlJ
wNASi6ETEr2WNHrciapWEFkhMUhNF/2Xq03ERF6tYl6LISXJwLRuJQbbxjTg
WN5pNV73VwCtHKo2UlTJilVm1vq0tFylja0o96uHibWlqmoFV1XF0/+i/S9I
qveclZ0rVE4dOVqjKnupqCqfCtjIkKXVYXTtqkRYlZSrcsPVE1VFx6ons2q3
F6oq5qkQTyfs/KW9VGMzVaqRGqdo6i+OpKCcGkkmJvGRs2rSr8eglaxJSvr6
lrFcHzU89PYaShVKrwV9hKB5RY2e/rbi6zHlFxFfS1B5qVKL5gaxYyupYTnf
BR8wqFSv3G0sXXGvhd8h4luu+GBb99ZisBt6X1C0EqPXLarADVhNWKVzlmQv
cCmqFumoWlDaAqxNVUoLVauT/t9o/9Pm/m1khSppU88a1HdkJFXZmo/y1F9Z
C1X5fD+9PhXE1CFFTKWqqghSjlM1qgr+lM43rmIF3KC1A+vmikRYlcythZcM
NQFWTAAu1WpVBagKnor0aJWGirCaKqJqikdVOkZJbfgVJarU8ofDKPpU4dTA
laKTt2tqPmdmCVsbSstPPnenSaGKMHq5x+KXey55uedid8fGK/ChvWPvxwiY
U2UX4lJzZeQMGMsls4iznXZgpc3CwsfOUEUahNMtV4CqIWNjGnBYMno9N8kW
njMqqpKRgGwtSG50bU5Vzc341X2RN4uqPWqtSl1TpRaq1icdVQqnAqpkL9X2
tOe3LPXXMU5V8fudQEe6wVAOTwmqjpvoyGHe2a92owrTgBsU43p+g1XhTpCi
7qomxbJesLIqV6tSUBXmVIjzVp1Sqw5TCpyKhopslSodTKGPolWG+/XhpvS0
Z4qIqdQ7IU197sxfhnjCAEaAbiqBkRYvuaB3xJ/GD3uyiOTqmZCLl7r0YpZw
XwCwYgex6MNSvBZZZ6DWcqV2B7OxS+M8rRTvOm2zUo3rxe3AcYOVpVUwAZKF
IKV+dZ4AzDkA1Vm1bKpSMv1P8f5tjN6/1FORbqdCVGU4lRjUi11/HeJUc0eL
U4dKnH6lhAp5v+MYHTnMO/tVHVUJsCpXrKqpqlJhtawQVbFcJSarq6pq0Y70
+FXmVOQLVaHzdxNBFR2i7o+jghlK0vXn9VTCqWEk9afY0UVLLxmIFMYcSds5
11BFPEKU9Ai3CYxidKexKN4Xf0r+igI7ii1zJ0kPultFagmR9SaXV5gPFIb2
Yis7HWfRV9BqlHNZJKhi82z5wKWCNiu5eDGds6TnALeWdleli6tkAlDrq1qa
SwCuYahqWsvsf0XzlPjOXzH3T11NlZn4p0iq6lYKOpWiwgqq5tSXXm1i0glw
+qWQqlFVPDzKPm2yqDpEUZUsCC5YYqUKK4aqVWu0HOBqPgkwhyriV6cJQHEO
s1m16Uz1ZEN9DlUk+xcm/pElvyMn8o1UXFJJTlE9NcIPTeqDekpMnSXj0JFT
wjtBlJQ9zPWCUS9SewJA9FxC2fSyxqMXMbotstFdi27JHYewF7sLrjGEacxS
c4NLSoHF8oH9YhMWmcukyysx1IKMt6XdwcoM9iGj5YTAUlqNeUcdtaTMWp9c
kAMsRNWcRFXlEoB+YT1BVdGoCiGpOKpKber5LYq68U+ZoZSk/qpPT6qwKrFS
cSrpnGou4VQVQmmQKkZVPQMwRZWkVdFUwHLXeummxXQYIHVWqH51TACKM1lJ
AJJV9elMdZz+F+cp8c7fXPbvbY4qTVKx1B+z/AU9Ra0UavOUH0AxgDkoDKfA
YKBSibrNmTuCFI9yeGIAIvFCtXiR3FER9mKCLVHnouUtorOWCvdF7MmitOq7
jDVhEbtFlFeDPbDUNmG66IqMYZf29SGiL5iutSoWVokVUAgrtRdYTliar5er
pKqiXcA4A1BF1So5ALDA/hdRlRaqkioVLVQVSKoig/pujqrCEpWYnlTKqYp5
v/a195YpqSynalVV8Kf+ySNQRQyBhVMByytW+gr7gl5gfRKgR5U4mfUE4GJm
Vhe2CkRVMlC9aEjFWDKfVvWoi82JwvXnu32ZlYJxClAlRqOjnlKpRL0ToiDF
PBKAp5DlU6STo0kBg57vmsbCF7oufJ7Fohcg8C+KD5k0I8DqnpdaRGGBAaNL
bM5ahgpLFLC4vCLJwLBuuMBoYfuthpG9IXyqrTMErlE6rQKtYtGK0CrXY6UI
K4qqxAco9tdn/ep0aK02WMmjapU0q6tO9WD/KxpSAagqmE+bZv9Kq1Rqiap4
F1UjLVTNqt+vgp4qEVMVIJUlVI2qtnJUHcZhXJkeK1a0qjrANgirdB5gYlnX
VJVDVXo+pybArAOQjFUvQdWsBFUz4ojadEKFmv1zmxPXS06N9laKOJViZMz7
UWt61FODlEkL3ObHylISUr0YpGIdKkDqxUQ9CbhQHuXiuTdjPP9m8gn2kUWS
Yt0iuTyzeM6wFFjw09GUYOoSzM24IPKK1a3eoib2oU19h5JJ7HzLFTEE0k4r
WrRiqAJavT0xK6zEZhCxFmRKzAFmhgEm45UUVC3VZgASVUXHqiueirVl85QK
hymlqCqXVNnUXxGnivVUkS+9qim9kpiqoKTKUXW0RlX+UlGVDgZktGqwHVih
Vaa7ShzRRUNrJapEa5VPABagCpuq6DpFFVU5STWSZ//UNR+iiwpH0eI8ihRM
rDgVHG50xiyFFHKK9UMJGdUdIMXx1BUVUwmMaPzmjXhbJRSQOQ5GqUWYpeYG
NWYlNSwCLGEOjPMDeTIwo634WkZCK25fT8YDhi0hcTZgXljBGMnxhcKKompq
DlULy1SVhipv/xO1Kt5UtWpNIary2T9li+JWyaktOqcakFQlnEpTf41zqrEt
8x3L+EVOHbPRkcO8s18VUKUJK2lcP1RRWJW51lm5SpzSqbAqnq8u/ep6ApDZ
KpT+38yU2vFJ9k/vpZqg9fwmnMLB6SKij0Ix+/mh6EJMySYpLd1H61CAA5nZ
C5BKYeSD3vexoDzi31qgw0ukDbtxbEF6sEcWWOi0R1rBregaZpNvidciZwuM
4yxo3SoRVrohMPQFM1QFWuUqVhRVE9QcYLSsb/OW9Vxr1UIysTbUqtKd9RRV
1Kwem6qC/a9kSm1+QRXqKaCVbvwrHffXiJVC5v32FSX9IqoIofZXK061I+9X
XUkdDZyqUVXwp1s9qg6nwirnr1CHV1QUVuKgFpb15dyyrparcqji+6qKUBVn
qs9h9j/YH+QlVRVUEY+6VqXKrPQdJq0UYrKfd6QPWBHE1PI3+kU7uiqmFEgR
pwTP7ym6CbAioPNriNfJnYqhIExqMfweWIYwKL6gs6RpMK1hvRZqWHSWOyld
LaPySl0jYlE1mC1nFCuu6NglSasEVZ5WExJhxYdXRGEFc5YUc4VuWU/XLCrz
1RNVtbAoAbg3mwAEVPHOX6qqSiVV6vrbkiz78BupiJ5iUyl2H6k45a8s75cv
Th3K1afKONVhB8URDqkaVSWo2nE4J6yUMbY5KyCdYUuEVXpWNyVDlsLo2uzu
quyCRe5XjwlAiartiqpyeRXpqZgZbepiSMUYsp2Ktf2OZ9k/8cOC8S8O+sOp
FKw+tSr4/VbKwbPEjs4qU73D2Fji7qPpPpHrE1k+S403FaFEcDMf7jzr7jzb
Zb65Y6MLxAIb8Lj9owXx8UKEUXKl2BI6K00MYhlLmNvhV2F+LQXtY+9mwPNh
BK1YZVzr47FiBcKKLLJXJyx5YUX86jQB6P3qqVk9zABclCQAl4kEIFdVq1RP
RX7rh5BUepWKSypn+TssNlJl11FVbvXN9k81NDEpl/c74mdQFHOqUTH1QUVV
gcE+fbwUVUJYyaJVAa2SNKB42arCSvEBCmHVVNZdpa0CiRNrtdWKUlUJVM2M
CUDcUF8qqdJjCpZS0dkUfnZ6avlL51FEPbU85P3ipFkppnqVQCqY91xtSMnv
Bd3kkbTAA8iHp9IzNF7jd14jET7nWRYBZ69LhCGzTiwjUnlFq1fRGZj2CCd1
KzrVNg6ygHm2I5NOK77a/t0BouJXZ7WqnZAATFuAqQOQTlXKTP8rKlStK9xO
xbbSb1OqVN6aXq1EZdi0K1OiUvRU9eapXN4vn/ErhVQpp3KQ+uChCr9blUrt
QRVEBVTRbVYgrMQraL2wAm5Qllg1rS+yrDdQruKTAIFT87RaFd1WD2NqwAGo
r6mankVV+v6ZTVJK3BTU9dePdPsyazoOTSKrpt7keqqLQ1XcwcEh9XIP5pog
DvO0CCWVTgDTgmcTKj39GsY8dv9V+kc+nnF/ZG4bOm8juQJDo856MxowsFHL
NxeTMtYrRGS9lvEHMlrRHuEB0RPI6lbqnEBtQmA6woIZ13nFarSsWKFlfZPi
A6SWdShXcWfFSeVgrlClDlNSUVVQpeKpv6LBFFW3e2TzfmXDKAqTfjlOdcQ7
UUCoGlXqp9GLo0ovWiGq0me16LFS/BUFqCrsBU5zgOrMClKuiqiay1UVogqy
f9OVqUo6qsQPK4f+OU6F8RT2sFJcf2SAUp+kRMXrU77JNw70I470pDIlLOi0
DYqVoqKGep1qqKCbXpNh0fMq8Gieu2PjqXBbesrhXwGiIcK87HqN6C+qtt7A
CpdqIKTWi8VobgdSvwI1LN4yTGn1RmJl70ZKV2K8rdJspa25T4WVdAOOV1ZZ
8YrVJuZa18pVDFVzSlurMg5AOQMwmaok7H+J2emEBKBKHZ+uutNz3b7VRvw1
v8ucKlZS8BsoFlO/VaiCD3P8ys3loCs4tyaoEs+3ko7gbI9VhQ1WeoNVyAFq
20AUVPGVVTIBOI/XqmZLVE2auUX8vLRQxTp/43Da9VlJNTKf+kvyfoAqVp/K
c+plN3GC9klRRx9yihkkXtcyeyiLIl9KYPTkK/NoPOXC3nk1fPhqjPhl4Z8I
+itJGOr1rMQ3SNuyvLwSfVhiYDsDlkIrP9FCONgzFovVdOaSMnBJTFtKJq7L
AbZEWE1IptdSH6B3VtBtIPmBFXIPSCOoytn/skMqNsdhSmySktRTVlKdVBnY
waCcaq8dPSujEFU0UkIdC6Ee+AUH+Pv5KkCV+uMU/3TbFFRZWsGvNDvGNucG
1K2A0rUeJ1ekg9bF2AptwpKOqpADzJnVZ8palT0BxFPI2yr4nAqlo2rCBpH9
o8Y/tuGXD/qTqT9c6hH7p6Ke6hLWS8HWXd8wFTN+UUwxy0RaiuJ4Kk7TRRK9
PBdun0jiyRBPvAIfms8PD3KWIc4Yv5BZVGQ5kmo2QgYs52NciNaLlziwKK1e
lebAmAyMC0SSulXabyVWMZIJgcwNGNKAXFj5ies51zr6ADdVQdV07qwoGAOo
zABMNoBonopkTRWZp6TNpz0gDBXZvVTopqCNVFVKVHvyLVRpfWq/75wq1VMH
uZ569+Goc6rZRsXDv1NcBQlA9cEGUOWCWSwqugHLjOsFc5bkXuAMqrywUhdX
gbOiaGBFrFWJ58yUtP+Xoop5KtxpI+bTkiHquqQawVx/vbk1nXIq1KeWizEU
dI6f5xRvlSLpPm+uo7k+9ZXiwWFRMvcpzxQFSRCPv+TuvOQCH3SPQ9DHkWX+
jvvKLqTsomoL04/oKuTAkg3FdOpFkT8wQyuQV3TdVRi+RBzsMRNId4WQNGCy
LziiaixZvIgVK74ZBGeti3KVzAEmc2ujX70oAbizGqqIU10bqF5oqJCSyqOq
QiNV0eZEMuivIqeKm3zbmfQrs/nZylSZmMJ0XxGeaHzgUNWW1KRy93OP0Ms+
edyzKKJqR94NWLZ4MdsRnLcCqpMrsj7ABFULyMwKgSp4FYtTmtgqYgKQ2v/S
kUrpkAoxoYJO/BOzKVTXX8/B1EoRh9CK5inRNvWy75ZaJDJ+mO5TqfQMT/Fh
yi5ChFLGcScwaA7GYy/OefxF9iEE/RzyVwjFKL8IsCi2bFWLAquLBBZmBZ8j
bcVeYQVmvUQLWGkyMCldMaNFGGch+63UebboBgy0ksIqWRMsK1bCtS6GLM2g
vcC8ERiH1kZUMb962lclHICJ/U/OVC/P/m2S2T9h/EvHU5TM+ssNpKX1qb2F
nNqfcKrQkX6g1I5e1jNV3NtbWpBK8USj9ORvHBed5ipH1c7DUltV8q43Iqyq
uNZz66vUNYtaIzCiip7V0FqlJwCTAYBx94fL/o1Lp6kn82lz4ynimg/ippAl
qv5kHgUfP0v1lLejEzGlUolaJqhHAoUMZvZUcZSSiMUL/s6vXrChf86LKcXm
Irw8CjVyUZ3FgeW1IWGWNwpKhSUGCfYkXgtZutJsgWFvSGqxUPqCkw2M6bQl
NmoJUaWMBIy9wOleYJywRJ0VehdwYQuwQBVVVR5VazRUJdPUlXaqzArFQkl1
uMpgiiqjaPUhtGmHb0MOipKyVEc9fgWQOtbcaqIjh3lnvyqhqphWVdqscvMr
CqyAvB04P7mCzaygvcDYCCwObVquKphWMYWoqkkZVGmFKo+qYdns3+pYpRKc
Iq4/OjfpjX6KnsJpfimVvLjwRzevSQGkgmcPZdQTmnp6LAETkIjHbHP76POz
H33Bx69YzCG3s/FvKdgi/GJlL17SomYMWsyiKUHRPvxCV54PTGn1VtRWXSAT
SIex9yeeQC6sWNFqqLRYpMIqjLH1S+2psMqYKxQfoL4QBLeBaM4KTACy7YrL
Mg5Avv5jBaKqWqGKDqkollT6+PRMiarK1PTyYRT5timGqsMWVY0OoGh3q5QC
KQVVrTWqin+67TsP52iV9a5rtKq0I7haO3AUVk1yJwiqKnF0yy5gObE2qqoZ
8/gAwNnannpSqBrLm3/5KHW2QhH3UoUqlTf+0R0fcAbi3nlq+bNiyuspuUfe
repY/CKvTCGknqXGCSqjQqKPVp28zKFsingyGJrzKFCJxyMh1D+KH75gb3/l
bzm8JLlYkStYMuain/BpbsCIncUwSYMAi9Iq9Vr40lVP5mNnLot+lFb2f0oP
7gmEsUtia7AfuFTQZpVWrDKTK9KdIFlnRVxeX4oq3awuEoAFnoq087egSiX2
UmE2Zqvs+c3PUMpMpciNTmqUU1Xs6LnKVDuGIwk8HS0iVIRUs4uOHOad/WoI
VTlDYI5WGzU3oBxjGzuCs7vssw1WoRdYnN6LRblKMQFSv3pA1fwkAZiiynGq
AFXUpp5m/yAv5N54O0k11KCqyXLKoapHcP25+tQK2Difiiaf9OsZe3uxOIWV
qWBBZ87zp0Q16mVZe+LSySsjwSMaDz83y8QjLvCOuz+bPzLbR4qwKMQyauul
OZgbfDIkBkVW8OnEdBELWBk3OwKLTg6UdSu0WAhPYKZoFddaBVqJ9YsF/goq
rNJF9jQHyBeCJM4KoaoWsoEVwqyezKrdXWD/I/OUlK0fhahSjH+apCJ6ahcr
Uel5v70NcyqtTwlO5bZ4NNTbe7SynipM90lI1agqQ9WRLK22+0hplRVWyeLF
9RkrIEkD8nbgkAMUpzfzAdIcIDMBykmAElVsBqBElR2pNIMsqU93VJHmX7rt
Fw4oXqVS1iai60/8aDLvRwbPIqdggIMYOvFslwWxPSpWo6JTwuPpJZbfe/SF
RBl50Hge2fhNUTwUIvs5FGEcW2qSkFa1vAbESlYAlnAJ+urV67o/sCAZyCfc
slXCqK3oIAubBrQ5wJW4454JqxGasBrthleMWStQJc0VHlUbZbmq3ASodwHr
top0BbBXVdFTQafUqt4/PqHigFqlimOUdDdFY1aK3Y1zqsouD7UylRFT7U/3
5QnVSgklIFWjqgRVu46AMC/TVoeUulWye5GPsWXzKwoXBNsXiDjAhWs9RVVB
I3B2YAUfrDSVzACkCUCLKkersZM3wZRaNk092U4lJFVapRI/WjeW91sGO21L
ONUVd0X5pF9okvK5sidfZZ5zW4dKUnwUUpYgwCYVRr82MTOJWQ/+eiZGfBzI
9WsXKcIos55jCUPMECq5QWK9UHuycDqukFcmaOnKAkvVVkmDsNNWy+KcQNUQ
qFoseJsVLrWPqBLmCtlgtWkszQFOEznAjAlQa60yz3mGKlqrCjPVmadiNfFU
aKP/MqhSJJXc85tZ81FQoirSU/vynKoipnIevxIx1TCnjpUrKY8nnVMtNjpy
mHf2qxxVu45IbbUzT6ucG1BNA/LFi4JW4gBvEg1WpGIlylVQsZKWdYGqxRxV
WKuauz03A9APqp1OZ6pHT0VA1YYRdDuVK0z4rfTE+Cd+NHRTdA9WCkj9vUHm
URRwKoqpMHciKCkipnhBKmgoXzMKbJodM3hRHDnWBO48+KwSDyQh/jR+SCnm
+PUwlWDPMXJxZkVgnaS+yxMYYjYg91fIGbbF5gqKqgkaqpBWcb56EFZ5B2C0
VSh7FR2q+JoqtqQ+V6gCTq1P234Lxv1pqb8K9amjlXwUmUnp7bb5FVvQC8pS
x+htPtGnyiiEVI2qyqiy0W6LBRat0v0giCrxesc0oGwHXqNWrMTkCmXEeoKq
nQqqnLAqqVXx9R8EVXH0Hxaq1NVU0PZLjX+448N3UZGR6bho/hWcmBTHUCxC
EwU6KJgLnSspsExghg1llEEDT+s5KiURSDQD4v5n3J1nZpo7LvydB57xfxQ+
nPkAfKb/ux5b7QaBBdZLXmQx64VwCWI+kLjZWeMVWdqoZQKXpplAv5CRF63S
CYEnnH10dG35zIp5pagiqxWzTvU9SvNvYedvNUklZ1NkZ9JqJaqi+pTonwqE
iqgSA9LzPVN5GaWgqhhSmYxfa7mMatGjI4d5Z7+Kh0ftCMtiFG1V0WKxNUsr
8XosM67rPVaZQetKDjBF1ZxkDwjWqqaRBKCKqrFTJKpooUr8aDihAg0V6Vha
OpgidPsu7QKc6uX7p+j4WWaigKRfmHkeIfWKN59zJUWLUEmN6ddB/kSVNCPA
iMUvn9aj4vHr+BVUGE0bOimH3xVWtVSR5X0XieniaVLACmuzYu+VaBN+kdKq
h2y5SmjFR9o6VPUIu0KYG5BMsqXCirrWsccqHQlIc4AdwVyoVe0kCcCdrP93
ReJUR0mVQVXa+QsZEnWSEuOU4vqrmvqrMo9CGUZxKM37teTs6O0xolexoDck
o1rag6rOOAOw0av4p4On0ImilXgRyTRghRm2sscq32CloyrZsUidFbgHBFA1
ZU4yrYIkAAWqxI8GHVWDeaEq7k8c2iTcFNhIBV1UdBSt45T3+73UHfch+kkU
zOnH7BPzoncCLRO8FEXwNOshppuc/MkjqfhsvO+p6Ri/NPE0jRlwez9E0F+o
th6IecJZgK1cYjApY/EClnBckJZh1ims+dhf0WgVi1aUVmHmUrooJFuxSmhF
K1bUXJGWq8bhUuAZvBE4o6pgtlJHMJd6KpR2KrXtN9lK711/fIK6cFN4SOVK
VKqeymz0YMWpg1JM6T1TCaqKjejl1j7Ff97aULoPoiXctvzWq6qCP91BcsgK
rXZIT6CglXjmq51WeVodoKhaTYSVum8RK1a+XLUyGVtBltfTcpVTVcxZMW1u
XlU5YQWooj9abP6NI2rXofdvQEQVM1SwGUr9Q4mqr+AU0VPMlB7apoKJ4ile
mUL/uShIwcmPFagHQwXqAZLEs/H09NwJdu+T00NMc0Hv44fT73PhPyT8uu9p
ijCPP/x3Q6rQMwuNGdSDgXZ3CyxuuiAKKzZh0RkXcXigqq26ZzOBKq26J7SS
k2zZ/IrMqKVUWGm9wLS7ytCKomoKaa2Ks5X4LmA6rnZBMKvHVfUrdos5FeaV
1RHMxdkUwfgXOYVuinboqX0lwyjynVOZ6egZm1870n3HypRU1jKRkVEtSXTk
MO/sVwmqdh/RhFWWVuLpmmqrgqJVunixUo+VNsBWDFliPsAlbMS6YgKcxxyA
U8i0CvHTyaYqZv9jnb92jjoZTssk1aCVpOFXTqUgLVTMl+6LU128I50k/ZBT
c4n/fA6HlD38SYrPCqgCfYQk+sUT034Bt9mYmv0jpNhTDlsMXjOC2ppOE4wP
MJHlsMWzgszizuc1PUG6hp8SpSu0sos2YWFiFw723pJWbObSALkoBN2AYtoS
Na4PGa20A6eWdTa2IrGsC1RNS1RVcADuFLWqZKpSnFMhl9RrhSrYi7pWGCpI
229HMJdGed6vxOznM37p4NmK9okIqTynNHdfoqFaJKHMT1cFUjWqylG1WxVW
nlbiGUU6rZJmq/zc9dy0JWFcr1SxSgetEx9gwdzamUm5CgZW0J9uAq9V0alK
EVXgqeBbP5ikopNpB4XUn0OV5vqLSz3QR4HT0Z/lvb3goLD2iRelmELXhBEp
6lEAsBBsggA83SPi8al3Pz71nsenuZhKYtrdNvx9f/tE/AoKvJ4kCUMitUiG
UIosAiwir17g8ornA+ksJkorzAQ+zzOBqbaKRas+rDU4Wiy0NKDcD5LMsKVz
lsTkCtoLLBeCEFRNnqW0Vs0o6KviAwDFpiq2poqP/tN3/vKFv6KXSt30ka5N
LLKmO0idWPCdkChUUq1CSRWUosSXzUGq5biNjhzmnf0q/ul27vYdeQJY+Itt
qG6lWyx4U7BOK81fkS6yjxWrZCcI7q93qFJygKiqxDPHJgDpuFpqqwiqCpuq
hofFHzhN3Umq1YgqHKL+1uBVTlLFGUqkkSpOTY/WdOJLVzlFG6a8wc+e4bPS
l9gDPtcHdajpSCjkFFVDFEN303iMxhSIux6betdj4o/I51O0BXh5pWZpCHCk
2BKJwWjAoGUsCyzeQeyBRQtYL899UpFXzBmII5ioy4IuDdFphVMstPGAven8
CiKs1KmAuQarKKzoMEBSrpoSTICaqtoe+6ooqpaloyqC/U+gKnZU6e1UqaEi
N+5PrPmozqk9mSbfggm0REwl9alsWUqrTB3LyqiTB0EVUjWqSlEFocgr4WAX
gywyDna9L7igaMXTgOoeq5gDXK3kAJm5goytoOOV6PNkxry4ByT0VW1lqOJO
dUTViARVIfsXVn4kQ9QNqqibAlBFt3ukLVSMU0lxKn3aP4y+vl8TL5+jAMXT
vSSz50ST000cOndZGE2561dK3Olj6p3hQ/YJj/mQ/Irwiv+6qrPQBs8UVujM
ojWsNB/oq1f5ZCD2CMth7Ohgd8OBsxaL4AZM9wVTN6AYDGhpNXqNtAJmcoBj
U1Th3FriV5eoml9kVk9RlU7/46P/ePaPDVOKhgpt4l/i+hN6KrfdIz+MQl87
VWVWUlKWal8zb7FrolmIqUwdCm6zeOKQqlFVEVU7OapoMjDXGryF0koxBMqi
ldwUHGilpgHlHqu4GYRPr1UbrJbIWeuqCRC6gDUHoNpUtQEHqqdDKmj2z+/5
DZNpg6SiboolmPpjnHoDS1TAKaunxA+C3VK2MsUgNRPNEggpSqh7goBCxeQR
Q3n0qInJ7tbfucPGZBdTwi19xIb//JRf9utHckGqkOcGpxGdFayDmsJizvbn
4zhcNAeS6hVzBqalK8wE6tqK0IruuO+WjAcUQ9eLKlbUXBGm16qWdeGsSFE1
VZutFM3qaQJQR1Xc/aHY1OncP8qprKQ6vCWVVLkWqqp6qpRT+j7Eikb0Ymtf
gXGiou08m+JLNBSN4y46cph39qsEVeYNT0ZY6Z5AQasqzVbawKWyNCDJAa5N
KlarWDswmivokY6W9TmLY3cVQxXbrigSgAqq4u4PO6QiRVVTv7DtN+ml8mNp
0U1hXX9+KoUYSWFPVMEm7O1FOzoY0RFSDzpDHSqpXz4V61BghPACCgn12FTU
R3cGPN3xiIyf5+Jh+Yj/K/aL5OHls4gotWwQbEWdpQCL17BiPtAVsET1SnQK
0x5hrq2SfitcyJhqq35kX7A2bYnuB6GrrHTXeiYHCOWqcXy8Ep0EOHU22wVM
UYWqam4GVVioWspHKsHraxXd+mE5xdupmKSS4/5iI1XS8Fto+dM4tT9v9tMc
FMWcynknyt192riJUkdfSymnMnhCSNWoKkcVRC4NWLXfSp9nmw6zZRaLjYxW
cjmINmrJbwZxLzpxqjNzxZJd3llBUaWaAGdVQNUkYv8bKz0VuJ0qZP9gJf0K
cI6RKlWcSYvbEtM9iVifiiYKzykmph6kYoqk+5iMemIa4gkEFOADxREDkI13
4Pb2NB5652cuzB33yGS8DZ/PKObhFYAF5LrrV1NTnXWPzwpGYIHRHUwXOAeD
Kay83SIkAxmtnkkygQ5Y0mVBi1aBViislkXvuja/IrZZ5YXVMOJaLy1XcWcF
2Vo1h4wBzDkAVVRpa6pW5qapkznqxZKKVqmg4XdHhzh1TI5LOpSMS6rk8ava
z5vfyZtpj8q3RDWkoQShXLQdL1kC/FuNKngWUVplgZU42ItptVmllVa0WqvR
CrSVMK4v550gbHot+gCXFljWdb+6rFURB6Cuqsauw12KYTd9E6IKChk9B4JH
Pc6m6NLHc0qwScyj4Jzyg5KozQ/GxoLigAZe4ZdA/95dkOVD6WTwZIRPFEQK
kn72EMYkuHObiQcnhbD38XPo/Z89hF8EmPUOU16ArUeDznos5AY5sHwl6yld
YUmLIBraVVq9PFdMZRfzl6i2ohtDXpXrQrh33dBqINBqBRdWq/pwK2A2B6j1
Ao95RytXpc4Kgar5iq2CTlWi6z9kocrZ1Fep2b90QkXOTZEpUeVGJ9HV86ov
Xe3wLe2ZKk335YZONDBoolpXVCxCJZBKBZSAVI2q4okc+FwStNp54mjlutr1
opW6IqR44rqwAhaYK9iaxUVKDnDqHHRW6A5ABVVh829s/gVU8UIVZv9AUokf
AV1/PPXnl1xQPQXH76PPz4l6KnCKuvuckpqeQkrk90A9JUjyYCI8inHrAzQm
mlv2IPvk9Gt6eKHaclILgDX55NmrTkjETqveS4MbcJniBtSsgNJc8TbJAY4L
FavM2AooVwlUMb86HVihmdXpsirdU0Gyf0FSSVThhIoCSUVTf9sLS1Rpq2+q
p9TiFNneq3CqymIpNeOn5foAT8el8zyjpBqSURk8MUKVoqoerLSLvPOpTqvt
SKudSt2Kooq5LPIWi4Kh67TNSlgB5az1ZNC6mLAEOcA4tBbnq7scYDpYKW2q
CnMqQFKtRU+F6PztwZdSubG0y3B8esopOo+CcupXL85hU5KIfYI2SaHtHHJr
kVAuHJ4mYx4PlFGOR7c8MPEWezvplvsnpnGzeMTCy/KrHSxIdJYrZoVeLaxh
pbZ2aWh/TngteOkKbYGwPSSZbRtnWdAhgYnybShijxWpWEE7MDx/JKombRjz
DksACmeFaK3iqoo7AJMBgLSpimb/ovdPFKqyxr9C11+627ddnCoYkN4OMVVB
SRXOQaqipBIj3/EgoBJIKWDy0RqjI4d5Z7+KfzrzFNJodbSIVruyM9gbaA3m
RSvxYgdtFY3r6xIrYG6ArRNWC2KDlRNWaSMwQdVUHFgRZivhakW9/zdBFQyp
ED+CeeOdGP/cbApXogJUPdd14W/ejJY/1z/FmqdIcWomEVMz7ntqhofUk0FJ
YTUqOPRAyNAyE6TsqES6lWMI4qZfTrzZxoSbw53Sw5nyy39lc/vgJAgAItNZ
JDF4JwEWoPYebhc0mvG+J1k+8AGaD8RkIKcVHxuouCx0WnWP9nXZaZU0BRPj
OqtYtYNuVFWN4yPWBaqmzvGqaiZPAM5dhNvqQ18V2v/I8l+2TtFxahXp/F0b
UJWk/sSyD+v+Za6/ZAc9tfylez3IOg+dUwdwoN+hEi96yqkig59mO+9Qrq99
AqpVEgqitUZV/oI3PIJWu3jdamcjtNpK9jAqtAqoEi/VqK02sE4roBXfEcz3
WCU+QHXQOpmyzspV0zRnha6qJm4YkTjV6Y+Ao9Qh+xfaqVyVisymIIMpfBeV
3z/lCitPvDwP9/ZifYra0YV9Aq19d0W/RIBUxBMllNNNwKZAoptcFBykN947
4cb7IMab25v8fXvHh/sK8AWRepxck1zykOUGmQ0DmBUcg7GMRUwXqLAorR5i
8oqMuiU9wk+mtIqjAuPGEFG0snUrzbtu/RUDQpsVX2rPKlYjpRWwwAdYHWqi
tYrUqpj9z6kq6VQXqEollVNV+z2qkl4qIalIieqwSP3JElVAVTKENilOsQW+
DYspFVKqZaLiFNkqhNKKUBqYNDYBnmh05DDv7FcVVAlaRYVVkAxM9lttI2vu
VW1FX3EbtaLV2nxTMJm4HvdY4ZwlORKQzwOkvcBYrmLjlRJUjcfBSomqoj/F
QBz9F+YpwZAK307lnGNMUoUqVUz9dSFWCljnAXm/F2Y/EqbOPsgdFCHjNzWI
qakeUqQa9bPghbjVQeqW+yEcRH45UT0Df3rveBa/GH/DL+ytD/fgDfxDE5Zi
5tYhLPILycVzhgFYLkQ9i1WyosK629vap4LjgtJKAovQitsC2UJh4gmUk5fC
IAtmsfCoCrSKTcGhzUoZXsHHrdMc4NDgAxyR6wXOTgL0fvXqUKNNVdRTgdPU
KaoUQ8Vmn66n+xOdQT24/tQuKt1KcSzlVK44lUv6pZCiqPKQOqoR6piipIqX
RumcqqShSvJ7Kp5qVMFVEVWeVnuzpStqC4x1q11Zo4V5SovXTqqt0mG2eVrF
/SBiKqCaA2TLFtNylRVWZGYFGVgxIQysgMFKY5INIKCqBqFTPaCq99AmLFRZ
VAWPepc+sUrlDeqki+rZLmj5m/dE8KXjMkSvp0hx6t4wr+9u4j+/w+f6OKTy
VSTgC9JHxPX3jMO4IYR8JPlbyC9HLsGsKLhChcsnBpnIol5B6MlK2ocVefXs
jOAMnMW0VUor1FavSQc73cbI5tnq3vVlXek2K94RDDlAgaqBYiRgwdgKcFZM
S50VrFZFbBUhAZg0VVXBWSKpkirVVo4qLqli6o9bKdJViRX1lNreW26fKFdS
3jLRAKGOZwdKNAqmKoSqUdXWCKosrfYyWunaajfPBHJa0RdC6rIQM5fSolWe
VmyVVSqslmrCaj4TVjvmiHIVWNbVLuCpfAPIpKiqKKrsSCXqqUi8fziewksq
kvpzkspPT6JWCsIpavabce9T0/1c2cen0YyfL0g5SKVnEcACUnYol5xiGkeR
BHHd3VrcNfa6uyHgkbHXhw+vv5v9dfM1BblQcAG2LLO4yMoprJ8/kpguSNdw
7MDCZOAzM2jpihgtZut1qzB5qbRoxZuC2WzAdJuVEFaKucIusi/qrsK5tTlU
TaUJwAWsBVhMVcKliuBUp6P/MPtXXaNhlYo1/IKbIpQJkrxfmJu0L2meErsR
M9umOKSOKxm/XCcvz/UdK1xoWL1dt1xDNc6mGlV4Ff90Nofs3vxIYIlkYH6i
hXhKU09gSAb61IFHFe+3wjTgBoVWcYQFG2O7JrZZJWlAbq5wqJqfjq4lOUA6
Xon+ILZcxRKAGyiqQlPVGooqkf2jE/+C8c+iCmfS2u0etkRFrRShPgV5v6Cn
0EHhjOjehW4EiPjl3xqce0bCIKGMhiLqyWEl4dG1d41N4yd3+hCP0M8hFGPw
AuVFpBbRWfcxkUVN71JhPeIVFs5oIl4L2Xv1QDAHkq4rhVZPNkgrTAOyNivR
EUyEFZoriLBao+YAU8s6DgNMJwEqqko6AO0zPDrVXfZvUbKmakWw/9G5f3rb
L1n1m26mK43oo9inDfcr5VTjZSnV2tew4Twx8gUkaXjSilDtwFOMjh3mnf0q
Q5XrHHelT0VhafJKPCeDwjpMS1epyyJ1sOvT10VfcGY2IPorcI9VSAPuZsIK
G6xCxQr312N3Ff1ZJs3cYt7HEr+6PT3gTa9QVbH/d0RY+wvZP0BVYqgASfUi
c1PEEhW1/HlOubxfKE5B0s8qC/HLt2c7Gs6hGmUgBc4HmuLj0onzaIyIa+5I
gj8In8P/lgKviC0itSA9GESWAiwir+LUC1bASrwWfLRFHGqR01a2O/gVPtX2
dTnPFiwWdOBSQqu4zUqZCkiEld8OnDRYCWHFuqsUvzqbrTQjQdVcaqtInOrL
RKEq6fzF+bTo/RO9VMlgClaiQr8fSKoOLvVoCFK5QRON+SUaskk0yCbzE1WB
FERHDvPOflVClQ2NVu4ZKJ5I2WQg3cmo7A5WPIHKIAvRF5yZZMsmruOoJZ4D
XEQqVthgBZZ1+uPg3FoYC6BOrKW2ioCqNdSpTpt/cT4tzf5hLxVtpDJHJeyd
xxYqmEcBespyKlnUe+ejWJmKZSmqpEBDMUKFPJ5nU8CTANDVPx+dxlW3u1vx
uP1k+/mCXymz4FZKLbRk3BcVFiQGb71/Im0oJsCacoeSDJwKY28jrZ6WmcCH
Ba1eytAqWcX4ghhmC0UrsiIE5lcUCCtmruCTK/wGKxRWOVQxv/rWyXRirYaq
2P9LUCWafzmq7OptnPuncirv+vOpP7T87WZ+P1eZgltuSkdHOjoo6D7Ew25/
R0cw12jk2VREqIa0kvgXEUmlYGr0MO/sV/FPtzeiCmh1DJ5v9HdbpXRFnIGH
twsTe9wYojrYFVqtK6IVTwPyrYtyeAWpWNGfaNbCHWLCEh2vRJ0V6FcnqArr
PwBVw1fjnIqeg0mhKrRTxexf2JyIkgo4ZQ7P0EJlT1fxxMbeXvSig33CQ+qB
STHdB5ByWT6b6OOZPYonAaOr3O2Vt7991e1vX+nu2NufvR3DPkL/1N63f/F2
iTZBLs+su4LOilUtlhu8KTBLGAWFrR3lFUy+pVZ2ngm0y1CwR/gRagt0tMKd
jI5WYRtjutmK0yrXZoVjbHF4BdsMwpdYyXmAiKqJiQlwqmytiruAVVSFAYBi
VT2iSk5T1zp/2XgKblBnrj/eRZX40lMfhcv4HVAGz+aM6NQ7YaTTkRIlVVlD
JU6JdxOLGCfpMO/sVwmq3NNpzz4Rmrzay0tXdMLtrryPvYK2Qk9gqq0yy4K1
/SCJcV3OWidDlmaFihV2V6WTAMdpqMK+qoiqEatxToXvqLKL6Z33L0yogGUf
L3RdFN0UXfxgCkj90e/T7pyyemoGdk4FOzoTU7dSSN2HkBrvUnxe0UB1Keom
BylADLAJSHSFj1H+zm2jfnzbqCtivP3j+MjbNtynCZal5OLYoulB9GCME5Us
5xWcgC5BOq8JaeXlFTYLE1rRuRbexE62CaOJnc2y0EbaKp1WfFnw626VM0PV
gKywwpGAcjVw4gNMxytpqBKqajuiam7o/10ITVW0UEXnKa1hO3+FRz0dT0El
lTKQNs8pV59qRhOFmOmnGtENoUq9E4RTvIe3YIosmcvHNZTQTa3YnNvaqHpq
Y7dtJKHXcUJVPMw7+1U8PCoOjUxolcsHlsor0SC8VWsQLqaVNAQGVBXQCo3r
9NgXFSturiBjKyiqZpIFi7qqiqiC6X84pbZHQBUWql7ttRQnKaGkAjcF/T4f
Da2+4PdjHb6BU0ZMwaF9qxVTvkPqJnBNEJM5zfI5PPkMnpdOP4t4AgDR+NGt
I12MinHLyPCgffzHELeROyq8bvdizWULbUDBywErSC1XzDoZb1lj6UpmAmfB
OAtftHoJLBZzhcWC7grxtOJpwC6pv4JbAenKxViuSnuBx8W9wMJZQVFlAleB
TKWqal6iqpz9b742/Q+zfytooYov/MW230RS8Vl/RFKlI9OZmNofNnpodnTV
5lc8Y1ZRUlUGH+k2CWV2RGu7rRFtVYtNJ+kY/2BcVVQV0ioFVgmt9hSOYCKl
K+EJpMCitNqQZgK5xcLTKqQBTYiTSuyxovsWublCTFmPlnU6CVBTVTEBGDYq
rgL7n/VUhJ2/JPsXPOpvLqTf55PEnf7wc7OtNT0Mob33KcspmDqLxSmYOxHE
lLUlgP+cmvpiog8ElM/sjb4yyKUrPJIClW6xMPohjZtHwJ3Lbx4B8cOb6Z+O
xA8DxUZ1BCvgP6RlLPC3g8Ki8y4gH4irRsDKDkPahY/9hPAuV7RCf4VcE8wq
VqvYANs0BzjaWdbD2ArhrGCLq4JfHWYrsVpViqrF2UG16KlYQQtV3PtXWqUy
r+V0B71c6SuafJUO36yeym2V4v5z3TJR4JEoMu+1i01tbQxP7+b124wqaHbI
ayuNVqk5sKhNuGAnY+IJFJutnG+WaiukFT1VVq2NbsBlQVsx43oyEhB9gDPj
lHW6DcS+lQVVNRZRBVuAhaoaIVVVdzKkAiZU0G/VV6lI6u9RcKc7y5/vn3LD
KKKeesTrKUz6eRd6NE74dB+a967+Oc/v3eZ1EAglCqbLbx55+U0jLmMxnH84
ovRUj/By+PuxCZJCpBlCmhi0SOUpQVrDIvlA6BqW/sDYe6UkA5NMoDYqkG4M
gXUhfldIF2XBZWngRnsxZ6mvSwNyH6AXVliuGsmXAlNnBSQAuV+dJwDlWHU/
VSlB1d5g/0u8f2E+7Xo+RJ1xKgymKNBTlFPZMRRlYiqvpLLmc9XOVzLmiOT3
GuDUe8Emcf1Wo+rAMUkrXWFJYOXygcWlq63BGZgbbAvJB9IdbF9BgCp6MiQW
C+trSveDUCvgfLACpg1WuBCEzK31qiqiaiOgyteqfP+vTQCCUx3nVEBHFf1W
vaHCG//cWFrvpgjudBiZHqzp2OTr61OxOAWzZEPGzzvPI6R8ru/2WH6KhOJC
CfD0gxuH/+BGuLWRO4Qv/elwE/hpGA5qnmvmC/qvfMsIQi6fJLTMoiWtYCOU
7ovgFURg3XRvsAgCrR6I7VfgtfBWdqXrKtoCWb9VMnnpceeyUGYuvcE6rVyb
1SInrBaTilX0V1TBmZ4DVLurhF+drawitgrXAjyHJwBT+18cqM4LVbjwl05S
EpJKWUeVTE2n8/2onkKn3wE7KEnnVBmhjsvFHJVzfe0zliOS2jrg0zt51287
qgKtVGBRhVXSeFWyPSQ6A1FYbeXaCh3sm0J3MH2lw9s/nGebGgJJU7BD1Spd
WMmKlUXVdhxboTorDKrArz583PphQVVFVA1rAk8FfqtvUO9faPsNxj83myIO
prBTKayVwpWobJ8v5v1CfcoVpyaFypQ9xml7VEz3hUQfFp5QOlHRlKPSJTcM
o/H9EJek8dNhwC+BsEguyBxSqXWbwiyoZCXA8vnAn5I+LJRX1BwotZXvuprG
bYFKdzBOteXrQug82wXPhqJVuiVEzq8gG+1zwsoush+5upRlirNC7QIuqFVp
9j9s/iWdv/ukoSJM/KMzlLZSN0Wh5S+X96sopjKQUmaeK36JgmFH1aVTW9Um
3Pf2qlEVQwPWnv1V7RY7hbxKGq9SWyA4A+k4C/rija3BTlutC7TKWCz2Aq2W
rd5DhRXtCJ7PVwOjD1DkACcaYUX86jixFpeAoK2CfrdOVbFClfP+LY6GCuz5
fXEuLKAnqT9rpYDhfiHv5zcegoniJl+ZGo9i6po7WbovJvqIhiqgkiXR9UNN
fA/juiSu5/fd57twfzfA61IPr2ECW4xZt4wMTgwJLGEUhALWDUo+0PdeYTLw
9odC4xVoq6RH2NIqAEubahuXWxGLxTzRaYW0CsKKWgH94kVYDsI3g3jXet/g
WifLFtfYclWYso4mwGKQCVTNTM3qOVTxHVVgqFjNq1TrN8fJtEFPEUkVS1Tm
ZX4k5RScG2Kdh3BQJJBqOZLUpHTzeXOVglSDrU9tvurUua7fZlTBs6scWFRe
JaMtSgYx7ZK02s6dgeIlqXgCt3hUree0Slfby9mAfOti2GVvaTUntANHc4XL
AVLLOjUBelQ5ZwX9bkP/78qeOKeibyxUmffhMPTvN28sNKefk1Su5zcMpoAp
f78M1nRRn0IThXVQOJtfyPiNTZUUyCgtfWd1kMcTsum6od+9boiNa/3txSJ+
Mph++N1r/efAHYozYJaFV8TWMBRcmB4EwwbUy1BkQSXrqtvfxjIWBdb1d/u5
gmC3iMnA+4nRIsgrpBUYLX7B6laJJ1ArWrGZS7zTigmrnoJWfIZtth3YmivE
Fnucsj7SdVeNQmeF1lpVDLJ5dFRFWqgCVFFJtZ5k/0KVahNDVRxMkSlRCV96
zPvFQUmHqib9pJLKZ/nUOlRF6dTZr99qVB303XkOWM0CWHuzpoujKbNy5kDm
tQjD2OmrLLdBWHWwA61SYUXTgCtoGpD5K3ZzK6DPAc6Yz3qB2eIqLFeRN70w
WGlgrFX5pipYp/hG3+UOVX47FWb/fC9VqFI94gYoAadCicoOoYW9vcipm345
8cb7MOnHxBRAShxZUIS69EZHKI+nYSiOvmvCk8jHRdcMvugng/3tTwZfeI2P
i64ZZB+8xj+In2ACKfZdDC67mNS6cVhkFksMhkqWp9VorrBk9eoGHMpEkoHC
aOFoNfku6BGOw9gDrcIYdhi7JAZZxDTgK95iEb3rfEVI3L3YawlWrHB4RTch
rIbIIUu5ctUohyrMAWqtVYmtYgGqqp3FIMMN9WKYkpRUpEQlJFVc8MF9FHGT
Lxnr57Z4NAtIpd4JvqRDFqREV1QykU9m+TohoVqTD0u+2U6HqpzBXn28OqoU
haUWsArzgemKxh0BWPS1k+kR1oYvqd3BSWtwMa2Ccd3PsA1WQDtrXZgrIAc4
KeQA6feMfvUhY9ahA9CcQtj/C3MqDKqwUOU6fxc+5/Z9wMQ/GKNEU3+UU3e6
SemRU7445R0UUJYSp5A38t3sDRKGDiigHKGiILr42kAlz6NBJr5zdYiryK2L
C64aaD8kj9i/crUL+3cZuQKzHLZiqjAyiwOLWC+gsxiBReSVaBn+aYZWt3la
+ZmBOH+JONhJ3Srst8KRtnJCIBVWXRaoTcFJGpAIK99jpc1aDz5AtbvK5gCT
xVW4tSqdWDvD9f+aZy8OAIz9v5j9C0vqi0GGvVTCTQGGKD84PR2ZTofQCrPf
YZn3S4f40U7e5gJIFTRD5SB1Is/aE3ihTaM4iq7OhSr8bsW3Xfq4eh046Gug
CbDKmMXtFkxe7bUDwdThgcwWuEtpEwZ/7JbcqEBtXYimrULRioywYBWrpZqw
mut3gpgzAb/n8VM3k3JVHFhBVNVqcAASVNFCFWT/gqHCLvm1VSpzQqKkMm/7
veUvcArG+t1MilP01+j85287a5/tckKzBEAq5veuHQrqKcilCCbDIBrfvjKN
AS7Yg/SvSHhRbP2ESK1Q28LEIAILmAXAwhpWyAcGWzv3WlwPXoswi8mvFHlg
otRWj5B1V1C3ioNtvcsC61a43IoXrdI0IExb8mnA2BHs/BWv9yY7glNzxRDS
YBXmAYqlwAUmQAVVfLASbKuHAYDUqS6af4P3T3rUhXlJDZr6Y/PSqYnCd/hW
z/gpgya0RB8bIpEVUCf0jK18Ub6kxCmlEo/W3xZUpZ9Q/Glw4fRIlVkMWNXk
FX2GC3lFq1fbycxAIa9EMpBtDOGZQDp2iRatVhJaLY9D1/fgYEC/x4pXrOh3
LnKAWK5yfvX1Q0kCsB9NAIbdH6/1djt/nU2dZv+kpAqpP3OiQquv59QDtj5F
vx8jMVxLr+/khbKUOep9uu+nQUa5CpQhFM3mUTwRDA04/woW3/px/2/9GG79
Hfu4v88+E0F2wZWMXCC4LorYIjrruqGsmMWANTLUsEaCworVqzuEvBp7fewU
9rS6BeTVg8wW6LVVGBXI61bK2CVWtIJhtmS7vRRWyX6QZNy6awcOS6yID1DP
AaYj1pXWKvCrI6rmeluF36u4iM5U362gCmzqSTuVbfvdEiUVnfVHB6eXsszE
ASKmCiAVxBSDlCBU6pR4r/N7rfy2Moxaw63SSJzcwQbj/PWBRxW9xCfb90Kc
VgcUYBXJK/GMLfEHZvYzplvvk2TgwTjOIl1uFVwWdFGITwPi7sVVRljt8RUr
YlzH79wPWZq7dSrxAdKZFQ5VG4ZLVWVRBTPVDareJKiihSoYThuqVCCpvOsP
plI4azrbjXjDL7wj/Rrg1O3OPuEMfhqkfJbvIld+CgIqiqbzrxjoeITR/5s/
inHeD/vB7Xk/MmHu4IP96KfZvxVYRrAVZJcjFyQMMUMIFbHoxCAiixoFqUvQ
5wOTZGC0soOPPSwToTMDWSbwUZYJpFNtwb7+ELevx5lLRWnA6K8AVFlhhevs
yYJg3VzhhdVqur5qOA4DzLVWzfCzlXAPCJrV4Y1WUFWyqWqZnVK7B4ZUwC5F
MFSsoYYKUaXiqOJTKUR9ipn9quAMQ0n3iVyfKqDa2JC9k3BVVkMN0aeh0M7z
ggP8/XwVo6rRWpVr0GsWwCpWWOJZh/JKuARVuwWTV7uElf3wtqTxCnzsaukK
M4FCWzURbRWEFU0D7hFjbOfCANtQsRINVuaggMVVYbzSBnOqAKrMaQOo6j0U
1394p/prbvQfjKi13j83oSJXpTJnKX4zt7r+qRvvGx85dccYc2KDFx1mTVgx
QtN910Giz8moUHVyhApy6QoGJssjF+dCXN733Mv7ufB3zrm8rwt4pK//HPfJ
8BcFvFRyXXDVoIbOro4HswVyTyB2BzOLxTPRYuE3W8HodUOrl+cYVD35ylxs
Cv41pgEDrXLCqmsQVj2csFKm1/J5gHbTYpxba3siLKreUbqA6coqiiqyAcQv
VYTK7FKPqrj2V5FUwfiXlVSQ99vDS1R0owdfMU/F1NGjWj+vw1MH/1+fiHM0
o4xS7nQ4/PfcXlRVP8zfb1cBquCRhhKAB+NAfoVZCCz6PGEKK00M8pTgnpw/
MJFXO7i8ovOXeOmKOdh9v9UmTissWhFa0R9h0fLor1AqVt5cES3rWK4CZ8XQ
sXG1Ik8AIqqWvOxQ5QtVYKhwyz6wSkW/n58/8g5YKRynJjhTeuCUs6NbL7ob
0AdiKqb7INfnZVRM8YWEnmPTjxiYzrnMx9mX9Tn7Bxh9v/ED+qEN/4j7NPgr
DZ0noLO8yLo6iizICoLLHRuKhePCyqvYfuVLV9gp7LTV2FC68tqqfYcem74e
aIXeddhplbRZeVRF43qvZDPIAOdaH7SC5QA1HyAsBU5zgHFobQ5V84mq4k71
RWr2j3X+7qdD/6JB3aEKZ/0pJSo+gRbODbHF48iRFhw2q6f7wpzzdKxEqqHU
611+F3TCo631OIlW9mG1w7+zXEIJIqFUkViGqhaBKrgjfrcHCoyC+5VhF6q8
ksza7Zkl8oFyZqBSumLrg6UnkFss8EeA2YBWW5n3nGhcR1otlMKK9gJDDhAa
gYMJ0KBq7YCRa/pZW4UfAEhVlTnEDKqw8xeyf+YkxO/HuimecO70R/z+Kd/n
m3IKkn4+42c59b3rQ02KlKJcli8KqPN+2N+z6XIGJgOgb1za28RZGJewKHiJ
fQP5FeAFEsz8K1Rwoc5CPwY4CcE6SE2D3ncRXYLRbvGj4A+88mej0GvBfOzB
aOF3iHiXxSRSt2KZwLuTTKDRVlUOFvRXuPkVVliBcb24YpXLAablKtpdRctV
vrUqDKyAPSAWVXO9A9Ca1Xn/b9L861EFKfHQTuUnVMQh6oFT0Z3Op9Ga95y4
d97rKW6ioMUpq6SQU+nIPjH7SEv0NX61svven9AahVJ7JJLGEfmh+oiMQk4l
UXh1OlQ1dBX/dLCC0wFLMqsoK8i6sfK+iwywsGWYtV/xsYGielWUDIRRgZ5W
trGRPjGiITBOspXTlui4dRBWtMFqghNW3Fmxbqj1q6/1+6qcquoxEFC1rEuf
Za/2Yp4K+v08DFUqcFPYhl/b7QuWP+AULPIATkFxCpJ+KKbQOHEhz/UZOmB+
L+Cpr2XTpX2QSl+/xESvr3+/l7nNncxnfr+XiK9DuL97FmGc+crf4NiyKcQy
ZhGFNRhqWMQlyOQV0uoKPtqC+thvCLbAuJyR1a3euYNmAg2qHtcsFli0Ijut
wF9h3mOUskxBVb/QDpwtV2W7q2i5SrRWTZkVUOWmVdBlVSqqYJ1iIqlE22+U
VIAqlvoDTvE5tDiJgokpYZ/gezpaWhLb+QkgVJtM4jVApcZw05GoRKgaVdVQ
BW+NDkZs8azgQYattIYVK1lpSjDTgcUWNe5hixpTr0Vx6QqWwdGjw7xvjAvu
ceaS32m1R9LKza8AYTV7IcxaR9e63wnifYBhGwg6K6yqsqsVA6rcqIouvZeB
Ux2/n6denW/OPWj7hSrVvU9ZgzpIKkj9GVFgzlvglDmKr/q55RQWp6KYAkg5
MQWQAo+Ez/IFAQXyBxSTZZMGpq99960zYvQMQT+097/2PRfhTuSX/cqMWRZb
hFlg0gBg0UrWBaFR66JrBlnTRZIPpB1YMEWQdgqLoRa4QATGWYC2Et3BbiHj
5Dtx7JKjFa5iNKh6gBStqHcdm4KjGzC1AvZcXAwyzAH2xnmAVljZKeuDR69h
3VWaCRBRZVUVQRWoKooqdaTSigJUheG0yCloIUkkFZvvxzh1GDJ+SrcU85/z
7t2KWb7kIlQqLycdLxRHJyiOn4Sv2YHDvLNfpag6lKLqcEIrrrNSbVXsbK8+
8LZkbCCfcEsHBtLSlV9uFWilGgKXhNmAxA2YVKxmbZ0MwipY1j2qgrNi4ChA
leurAlSRIdvOU2E7qp561bdTPfyc76X6xZNWUoHrz6b+oER1j/f7EU7ZnqlL
fzrs+zdYj5851Smkzr8iJPouDzLKJfccnmQeD+hz+sU2TrvI3552UY+iIJ8M
QYmG5AKNBthCnWVwab4fkhiMzPq2E1nfuXrgd/QClqxeeVrd6gcxoY8dtRVM
tMBMICy64mPYlTQgNFsZeeumr88kFovZ0ruezK8Qq6y8FdBpq2AFjOaKHMKk
ZR03AktUbaIDK2ityqqqBSqqrIMId1TZIRUUVZuiR51JKuL6Qz21Z5+mp5L2
Xs8pPhmJd0hxGVVOqFZyJ0el4yceRrTNOPdh+pmNRmvy9fF+Bw7zzn6VoOoI
23SmYKuiyArY2seZVdCHlTJLnc0ezYEhGUhf74otMAy2XUMc7GixELRK0oBx
KqAXVswHuMnnABFVzgQIqMJvCfp/X3JTas2xBoUq8P499JtZYPyDKhU0/ELq
D+b7gZ6C+pRr73Wcskm/oeY8Nwc7rUkZzWIhFWRUwiariSiSvnphjK98p3th
dIM7X70QosdX3V9HigVsvfW1oLzOjNjyzDobRdZlPjGIjkFMCQbHxSBIBoau
4WHEzR6TgeBj93NurbYajSb260PLVZjEPlExBCpFKy0NGAYu0dmAxLgOtILF
i3qP1RtkzhKtWMUhS65cVSDERk/aMAacFdpspTiulqiq+QRVS1buZp4KTVLh
xD8iqQ5HKwXqqX163i8sQ2yJ9gnI+JHh52qur8KVVJqKcnfVVM/xFvd9tHDE
tMRo5R/GEJ+ZfthCHsl9kfRrpl8t3O/AYd7Zr/ahSpKL0CpXzxK+C6azCKqK
alhcXtHpFvSFnDZe0SXCFlWb4/rgSKt1oTUYacVHWMxbYmnFK1Z+g5XLAXof
oJ1bOz6uAqF5HnMuwaiKVzyqwFNh5/6Zc8+8XYe233uf9LMpbrepP9vt+9N7
rZXiJ3eOvTpwynZO3Tj8EuCUS/qZU50YJzykWI3pe70MMoh0ClRyYPoyxAXd
v3RBtxjf7poJ+6dftp9gb20EeH3lwsgvxBaQC5OErrblTBqos0hW0HzzJCXo
O4ht77D1WpDqVTAHhoG3ZBBT0FaGVte4TKDQVjcV0OpXhFZ8isWDjlZ6UzCf
X5ETVsXmCouqobFcVdAIrPLL16rIEhAcqz6PoMpl/3bjlNpgU2eoElWquOZj
N5VUxEcRJibFpB/p7fUZv0RJtbb6QbKVINWqsqlQ+8hHVJp0tujAYd7Zr+Kf
jk7mPwRxuCXlF1FYUWqpicFMB3EzlLGqDBJExwV9neK0W1690mm1gY6zILMs
HK1s6h5pRQcuQRrQC6sFXlj5kYAzNlMfoDlP8BsbAK1VzqzezaCq33JA1Yvd
FztPxXxzvkGhKmT/XC+Vk1Tg+jMqAEpUgVPmTPYmCiMx0OYHZSkjTOivxeb6
vg+EAkh5+WNQYrFyQXfADcQXz+9K41R+i/FFcsfGt214hF3gEWYikMsDkTLL
66xQ0rLA+kEQWRFYUV65kRfQODw4yCsHrMTK/sNbYibwSj+P3c5FtJlAZ2Ln
sywSWj2Mi4PpPFs/IZB3WrGmYDCu2zQgEVbPB2H1IgqrXksxB/gGyQHqPsAw
Zd0vBEmW14OqgtYq8wzMSTCvqpbG/l9EFS1UYefverLql1apeOqPtFARTqUz
KFRIVZNROS9EQV7uveZIjaqTeVVHlYgikRV1VkuqsDJGwebCJixfyaKvQdoy
zGazS7uFNow9bRCG1mAcaetoRccD2jRg9Ff4PVZoBYQGK/zeoLtq0Oi1A0bZ
1qreYbCSdQD29v2/BlXPdlkQOqpi9s9VqaaYk9O6KVzqL1j+bP/UD28e6TkF
ST9XmaK/FjD1GQSc+f0oo5yGspLHyB/EU6TSt2J8IcTnv+njCy7Yh9+SYf8u
JR3wi2BL6CxiySDAurT32T8A34UzCnqXYCKvrglWdvBawCAmMpjdzQwMEy1u
T1wWfiejQqufk+VWqnfdj17HnVbCXxGEla9YgXEdZthm2oF9DtANWcpZ1ocE
y7oRVoiqMRJVJAE4h9kqcgizhSot+6dJqjiYwnJqL7H8kQm0rDhlbX6taJ8I
fVLMNaFcunTS8PSusqk5RAu5Q/+0RtW7cZWiyjzxioBVXswqq2QdYLdeZBGF
RV9fqkswW8DSrOybtkVabQh1K5oJbCK0ovb1hY5W2BQ8a8EOTAPitzcOylWT
SLnKOStgYAUMVurSe5lD1WKHKrue3nZUPT/bvF2HvVR3PTbVSap3jKQKqT9n
TQdO3TQCOEV/LeZIN2e7OeStr++SXhRSQUP5hJ6FyPkMTJ8nGPpcuP3ceST4
h58PD9K/mGLr1MAsh0WPrZAhBGaByIJKli1jneW87r4ty03AkPLqSu9m97Qi
zkDQVpdHo4WnVVwdEruD5YoroBWuYrwjLA5OpljEDYwkDehphcLqae6vkOPW
ibDK5ADDPEBuWce5taMmJKtAyMRatFUQszqMVWeDapet2qPyS7T9qpKKjqJ1
9akWkfeL82ZDqxSWpbIyqoBN9ogWSbxSpjS3tZA4nrnfkv+03J8eJ3daArxa
muW/zqLgjxrHXAcO885+FQ+PosP5i5mVw9bBxINxIMkNpq1Y9OWzT3O254DF
LIIhHxiBFeQVDmP3LVeb5Ax2oBVZFIJpwJ1AK/p+FYXVuKmbx0zeBDlAaAR2
q0D8GMDuAwFVkAAEVeU9Fdam/uzM+5xH/c4gqUjqb8wVt739I+dLx3/0Qlec
cvaJ/udebt195pz3kLrY5fqgCOU0lEHGqd/2BEGVhPQ55dyup5zn47Pn2jgl
xGfPZY/QPzKf/Dm4zcOLqi0EVoFhoN0huq7AFhg8gW6chaXVWEmrsD6Y2td/
zjuthMWCbgnBolUcY/tKHAwoKlZujxUXVn5yBawFWUFzgCmqXLnKTwL045V4
a1VEFdgq5kVU4agKQJUvVDWx7B9sp8r9etUSFfgosHnqyNEW7JmCxl6f8SM1
Kc3YpxGqimIqhcv7LdLvVrC1BGTukXYd4x+Mq/inE6gqZtahsvQgwZais+hL
Q3YQ59eLyBoWH8/ugcXn3NJ57GJaIB2+BP1Wy8N2eyxa4Tfp19kTK+D4aZud
D3DDKDsMEBqBvQnwrcGrHKrAVuH7f539DwtVLvv3+FRfpfrlRDeYwqb+riQb
El3ez5soLKdATIWMn1VSF/WEUpSBAmoozOmBRGIkOsfFuV0/cw6Js12EDz99
Nv/Tc8Lfcn+xUaxQkWW+VVLGCv52NxYD24cTeTXAJwPByn6dHWrRPsBB3Yqk
Ad8xQScE4oL7e4PFgroBhbB6AitWOWEFKxff8isXNWGl5AANqgaPls4Khqrp
iKotPgE4dxvZVu9QtcRvqloC0/9IocoPqVCyf1FSFfwOrZ5y++XjAIqW47Ey
RcZN2MujqlhD8dxaC9cv2ThWFs38Pv2LzcmD4su+RzhL4dWBw7yzX+1DVXW1
lTcNtojnvOK+iB3Ecmw7mX+ruARdy/DRHdgyHApYdHIgmxlIS1eeVvbd5sq1
sCuEJUywaBX9FaRiZc0VkAM0wsoOA7STAPtavzqqqmWgqrxT/eW5j4ZC1S9c
9s9JKmtQp4uojFIwquH71kdhOXX+FQMh6Xf2ZX1RTEFNCnN9qKFA5kTd5BCD
PPo0j099gwT5sOCwQqh5cgXxJdSW11mknhWA1Q2AhWWsr/l8IFSvrN1C9Vp8
x/vYsW7FTOw/JJ5A0R2M2qoi0XjRynvXH/r1rCJhZQcDho5gRyuSA3TCKmmw
QnOFzwGOaGLDAMN4pREeVRtgaC1XVVumkHG1XlUtdrNql8b1HwZV2FHlUbVh
/7qNvu0XObWVG9RRUkXLn0v6FfzejqcdUql6UjRUCosUK8famvNUag63zWWf
eVLiZLKsA4d5Z7+Kfzo/COWoj/YDK6QHxfsx3pxF0oMHeW6wcEZTNAoSYNEt
w5lRtywZiM7AdXFpiKUVfrfEEIgjLIK/wlWsnBXQtgPDoHXIAQ5GZ8VQO7Ci
q50B6BKA3exMdWf/s6P/HrSFqunm3bt5J/+zh+KyD3O6XhGsFOYovvjaIWD2
M4f2OZdDZao3lKUcpLySsoQyGupbFhM0rQd4Ekj65Fku3J1PuFCPHfgj/EwT
lGgUdh5bTq/FVCHUuQKzoFLGgdUdjYJMXjl/IPYLa7QaHIdaJPOXsG6FnkBK
KzrIgrYG44RA6LQqBhluChb+ChRWz/OK1atxl700VyCq+gyLlvWBtFxFxivR
+ep0DCDdAuxVFUEV2P/CQHXb/LvaZv8UVNmeX4KqPXv9NNqQ92umJgpom4r2
CUekKm8DnG4ShBIKKMMgcb/5aLhNg35CaRyVaGsP6YQcO3EU68Bh3tmvSqjS
ophZ9JY+M5npPV/P8ulBbU/WPjqmKaVVamsXKUFiaM/Kq40xaQ+ZwBWhbrXE
ZQKtxSJ4182BYGgFxvUorFyDlR1bMWYdOivM22bzFtocUKiqzIH2+EtzH33e
zlOC7B/8oyCpzIl6pZ1KITjl9NTl/cwBjmLKHPK2Jerb3TykIMtHCPXpcyKe
ADQWPV+38W9fl2z61zO7/tuZXf/1613tnXArwv7dgDaEHcKLYgtE3ClEZwlg
nRqABZ3FFFhgaEd5ldDKZwIvusZPuCUui2GCVlck2uq6oK28IZDMXPKGQCxa
PTbVWiye8AOXaFOwEVa5Q9hPXE8qVq/0XBzbgVVzBZarRrClwDi3FlE1JpgA
PapmClSFZVW0/9fVqkhHFff+4XDaRFJFy99B2ueLxanjyCnI9REHhZNWMsXn
z97q2doMhk5GEK61aCATqq0DYX+uhhDWgcO8s1/FP92xMLarmFmgvBBV4jmm
Ci60tmasg4WbR2hP1n61kuUdF6iw6Mxblg9EWjl5hd/z+k3UFugnL2Gz1aIV
dvS6H2EBTcHOuA7CylasQoOVOV58uWpo01uDVnbrv8Kg6lWvqhY87UYqPfJc
PO5uf/idW+6fZKtUrpHqx7eNusxZ/r5rhyYFPWWTfn1cZeotEFPgPI+QQkIF
AeXxlGHTv3ytK8TH8c4Z5H64Y+NMf8fwCyKFV8TW2RFbND14Sgos6rsgBSyU
V9AvDKUrGB6oaavBqSdQ9ls5+zrQCscu3fALpBXvtPK08tvtpb/CuwH9GFuW
BnTCKnfkQsXqNSqsHKq6ixyg7QVuwl7gMAxQ7lhMVdVUQNW8FFVeVcGcipXe
qR7aqTyqePbPcGp34BRYKfg8iqNhoB9On4CxSPaKg/h4EUoIjXjs52XRsSPm
tjrUTj7vKqizyqIsfm9KFjGRYx04zDv7VY6q5uOlwErZdDRsT6tqydDIlbW7
E9PgPsIsASwYU7abAevozsTQTr/zTdvsG0vcer+O08q5LGKz1cIwcCmmAd0M
WyqsfA5w5BrnrHBdwH0dqrp7VMG/awtVT1jv320PTbr5l15SXfGzt80Ze+mN
w80JbIcmRU7Z4pQTU3a4hM34USUVsnyWUN/w6kn8D/q4A9DHzrBhqPSx012c
0fWfT7chP3Sf9rHT4+cDyCjOVGxRqSWAFYtZAljO3A6zL9BugclAGMcEg5js
bPZEW10ktdVwrFv96BY6J9DZ1+9I7OuZKRZ3onedr7YX05bEmmBfsXI7glFY
vdR9Ue4UNbQKqFpJfYBpucq8BaqGqugABFTF5b+gqtbta+Kdv77tN0xQ96k/
UqLCVt+gp5zTL0yfiEqK1qRUSEWRklLpiL09sRG+5gnh3bsQmfrXsY4c5p39
Kv7pmu14ybijEwcji1+sQjEitfQiV6Zj6xANva2YqS0yXfAY9mTtS5hFUoJ2
Trt5u4jf/DZnEYR8IE0GojNwzcYDcfgSZAKDtkKLBbgBIQ2IFStzngwfvx4s
6+bYMe+WzTtni6peS/BfN4ebeVtuzr17Hp9mTsVbH5x0o+2lGnvVz0f/6FYj
qWzqz5zA5jSG+pThlDm6T7uoJ4gpyPjZmlQoRRktA+WklE0fP8PjBhhk4qOn
6fGRr4b7X40Pwuf/82mEaAFhRdgKUotWtT6DWcGgsKCzGOVVWr2SpSutbuUy
gaQ7+AbXHXzjcFK0ihtDrokjbf2uENFpFd2AZDyg8FeQaUuKsCIVq4XOCrgQ
5iy9THOAffwie/XIAh8gXbOomwCnhT0gs/1sJURVnFW7PC7/9YWq0PxLvX/Y
9kvdFOY1BeVjmfcDPdXayhqmgFPeLEFcbV5AcTD5OMzhcrjt6GH3IImjJI6V
RnuJZiN8Y+1PHlbWYi3HCgmVRAcO885+laLK0aoVpyIzcnF+IbB0/RXaA48o
/Cpq10KpdVCTWimzUgMGnSiIz4pdISWIBSxqt4B84AYyjB3kFe23srQKfcHQ
FOzmV0RhNfqdDWiu6O/KVThJG1SVOc0cqqyn4m7r/Zt88/0TzTt8qFJdfvNI
SP1dcNUgcyafaznVx3HKVqYw43fKud7R9ylOKEzlAZsohkz8U7j9p6/E+McQ
/0TuxM/5qv+7H6EsO01qMcRWyiw0Y0Rg8RqWARatXn05mAOptvKZwKK6VaAV
TF6yworRCncHO1rFZitLK1gUktIKi1bEDXg/mbaU7gehy0HYguAei0UOMEyv
XYHCqrdrsOo7vCmFF4xYj+OVIqo24xjA6Rqq0P4HTvXYUQWFKrdFEQ0VTlId
dZLKZv9oiSrqKWtH5429oKSg3bcFClIFeDoSGaTyKMYhHofDbT4QWFmuNUox
mZA8ISlEl5PMJw/T/GEHDvPOflVBlYhjkV9IrlbPrGaJraO55GGhqxDNRRJY
CrZkYpDUszyw6CtdiCy2BivIq61heGDYeHUgodVelVYzF/g04KSZW8ZP3zzW
tQNbYTV6LX4DXfvZLcAvdV9s3mkDqh5ynoq7HrPev5vum2gk1dVOUv3gp8O/
e91QZ00fcG7UU4ZT3cCF/jlnnIB0XyTUmT65h3j6CKESwugfvszi778kw//R
l9in/SMEIiwgzzPrtKjXUqn1b2f6YhkmBpld0CmsWMDytLLyStCK1q3y2mow
0ooUrUaGsUsuDfhzsTg4aKu4MniiSAN6f8Xj6K9Q2qzoxHVA1TNMWMkBtqLB
qufAOA+w7zA2uhbKVar4Qr+6Q9W2YlQtb9oTUOVt6us2HxDZP+GmAE65l6F9
zYKe8tP8/FHRagNt50JDSemUYdCREJJN1aOQX0Wi7EjDCGtut/IqFlyJ/qLA
6sBh3tmv4p+upSXukraRAIswK6+5miO8SkwalW3wND1InBgtaB3cT3qKvWkw
sQsSYB2NYy52HmHyCkeyex+7L13ZpSGeVm5CoDMEQtEK0oAwFdC874XvwZw2
5k2yLVf1twMrXu6x+DlXq3rMOtVnmXfptlDlsn/m2HSSasQlNwy96CeDjV4w
B/I3LKd6nXah5xT0SZlDHn/GfzszaKiQ2fvIV6NWEjz68Bdj/N2pXf/uiyHo
/S+yT7NBQRYQRoUYaq6PiiQhMCsRWYrCovLKJwNZpzAMZj8L6laX9fHjbbnL
wm69R5eFtFiEotXPRrldIW9fTaevh6IV966/w3ZacWGFtPJrgqNxPVSsHK3o
HivaDoyTKyAHiA1WKqpYuSqqKluuSuFFalU7Aqp24agKWFJPO6rMkzwO/Uuq
VCH151/CMC6JOyhCexSUVPyRC+e5kEsUTAfbG+Sv0y94tJh0ZfA6KrDVoPg6
YZor79zowGHe2a/iiRwSVWokaqtZk13+kWMEXs1FxsKjJFtYUOESVS36aj0Y
+rNEYpA2ZO3ee4zuw8Ku4W3BIkiBBdUrkFfmHamn1aq9SCuwWJhTwryzxW/D
utZdDtAcO72GWGeFeSNtUPWbN52qemGOeU9ujj5zEt56/yRzVF59x5gf3erc
FDb1Z0tU3/iBz/uZc9tXpsIXNwc+hRQSCvBEwWTQ87enWhiZ27/9gr39my+E
+Hzm/hf8Z0L83ameZR8OIBPkem8r0Uir75AVV9+7bkhisRjBaTUaaOUNgbxo
dev9E9O56zjJ1qMK0oBi1NKLTliJihWYK7paVFlhZRus+PRazQdIy1XYXZV3
VvhaVfr7oVOVEFXUpm6H/oGhIpVUoUSFnIpb/jynEFJCRgU2SdYcCLcH2g4n
wT7tQIP86rAKUytl7at/nRjNRaJdx/gH4ypVVWlQTqWP5EDG4ZWwLJM8lK54
rXuLvh6RXAfjIq0WxqyDkll0RpNhFk0JRsdFGMwO1auQDHQm9qitvCEQvpMp
c1wacNomaAceMmbdADthyZarXu+7zLypNu+xn351vnkT/uCzFlU/f8QWqqyh
4nYrqcwBe+E1g83xe/Zlfc/8fi/kFHxxI0M+eZbVJv/icn0fPV0S6sNBJf0t
YdBff77r/4b4nIz/FUI8/tefZ9EOgrSd/iGMw6f/n1tP/939p/+no6f/H3PO
+GvzyNSv/cO6M/5oz+m/Qz9t3xn/T/Pp/wHuHz/9340+61MnCmcGVV5bxc1W
o+nW4OvvGWu967go5H5btOLLgifThVbKGFuyyupJI6xenYt7rKDHyqAqnbXu
coDLbC/wwBU9BsV5gLhpkeYAh8eZFWFoLUXVbG6rAAcgGQiGQVG1KaBqe8j+
WTeFk1SQ+oMSlff7YXUqciooKTiiqYACjkge7Q+3udAQJokmohBhRajSsFXJ
yNGQYeMkoqrKYd7ZrxJUHbfPzBaybkaFVwsnlw6vsKia3bd3joPPsFlJIbYq
OcMALHzFFZe3ArNspLtIEmZhVtCWlZ2tXZdXmAyE0pWhFX4/5nCY5YpW5tyY
OMNbAYeNsw1WfYc1mffM5lAyZ9Tzby56+rV55lh74NmZ9zw+7faHJt943wRz
Wv74trfN+/+Lr3Wpv8v7fv37vU+/uAd+8c+cYws90Jn7cVeQMpCihAIFFNmE
GDql61+5+J+f9fFXn433aZjPKT3z1532B4Yjm0/73Xe++vfLT/vTDaf9PmXN
uxxHzvi/4M6uM393/9f+886v/5653w6Q2b5gnwYcl44H9LSSK0IU4zq3AsKO
4HngWqcVK9wOTHOAohe4L7esxzWLOLMCTIBhbT0uAjaomjnf16rmh1qVH1Vh
l//afTfpb8C8MQOPOpVU4PoDyx/60v0cP5v0C2IqQopoqAim/W2HIPbZOAx3
woc0DodPgL9SALJUkUWxVqa/OlL5ypa62mc7rFFV6Sr+6Y4DqtRoaZUgSx85
rrAsK75aQv6whaqw41R50ZdVPlXIZmUcSj2EfIKT11mpv52MZoKUIAJro5FX
W0IycP1++H6WN+0Fl4XRVnMW7YSi1TtWWG02737N2QI+QPNu2ZxI5r20eXf9
zGvzf/WiRZU5+n720CRzPJp3+D+8ZaSRVN+5etA3f9T/rEt6wxe3eupcW9P5
xFkOUl8jkPqSLzkZPEk2OfT8fyY+Y+MvP931L+GWxF98qghM5szfftp/Mbd7
T/u/V5/2R+8hj05IHP2ahdrOs35/4fkfm/Gdz1WnGIywuP0hSAMGWj3hhVUY
YzsDhdWjL0AaMEwFDFsXRcVKCKuuIKwGiLEVq2FsBVrWAVUjyXil8VMlqqbP
8/uq/AYQj6o9MFUJmqpAVdGOqvQHR0l1zM2jiH4/mvSjuT4gBRKKwWhv20Ee
5kG4NX8UI4HXof0sSoXYEZJIPFxBcFUiVxVstQtYscJVAVsdOMw7+1WKquJA
ch0/7jvWW/jjORWWirKcFqMvnGbVdqhZNVRjhroCEpgVRdZ+anGPRkEGrG2H
QV7Bd+VKV87E3uTrVpZWC3fOmG+96+YAGTfVWgGHjbVDlnoPbTJnkTmazGFl
jq/HXpxz/zMWVeYkvOGecVfdPvqym0ZcfO2Q868YAF/8yxd0E5z62OnWuvBP
X4kyCgSUwROIJkulgKS/cDD6HyY+6ePPP9n1zz6h4OnY6f+Rner8ww9wNH3z
b7ec82cHz/zP4y85Z+s5fzr74i80nf/3FRFmm4LpmmAurHBH8NO8x4rPWfLC
6g0urPQcIHRXwUZgjireWuVXVgVbhd8AgqrKNVXZnAD1VGzdHtupzBu2/U5S
6W9gYtLvaHD0HYoaCtWTANPBPTYOkFsfCcIO7pVc0/m1j8HrUD5zeESUwDoi
tfIVrhMptTLY6sBh3tmvElS5Gf5+M1pr2L15nCKsDfemaSxra0mgxsQXxRnh
Gn1dFIPsWLRzRP1VaiwUlgxqeo9G9/1OaglguT4s/N589WrTQVu6ckYLuzsY
tNUSb7GYMsu6AcdM9sKq3/CmnoNsueqlHovN2fXYCwZVM+761ZRbHph4nc3+
jbrkhmHwxc/8fq+vXtjjVMepT7mkH4ipCCkno6J6+oxXScgjjD/9N4VN7zkj
3m9xDDTX2Uw8bj/nv28/90+3nPcXiy7+zMZv/a/t5/1ZKb8eZsZ1tALOp1ZA
h6rFsWLlzRXL3+xHfIA0BzgCcoBrh4xZB43ANgfInRWIKjdcfTvsAUEH4EKv
qvYGVO1D+x+MqMXOX8j+2dkUbiatk1RBT9loVX/qAKlAqAOORwd2h1s18BOS
UBF2KCAsp7xKy15HRJGrHeWtPLOOFWOrfZrrSI0quIp/utbWNrpx5rgax8kd
cV+GgFp80HCK4UlwTauU6fxSzIeK7FJt8IdoepDOwQhZQfz2rMJyFkE74MLt
vVq/+cDajb50ZYcvOVqBfR2KVuYkAWFl3h6bd8vmRHq552JzcJnTzLwbv9Og
6v6JP7lzDHz983884BuX9jntoh6nnt/tlPM8pz52hsv4fdlC6m++YN0RBlL/
M8GTodJ/N/GvXf/bv2bxtP+0//Sec6FTR+sZ/97czv3BaeZ20lXfPfCN3y2G
V6hYuRzg62o78NIuzgf4Zhi0TnOAfcPoWocqUq4i64BtaxWgalZcBDx7QRys
BAlAHACIqNpAhlSYd2LoUQeD+pGjLea1Y15oIe/X6vN+x5yeOhLE1KH92Rzy
/l1FkaXY7nJyIbMOFqcKD+jMqmQv7EB6MOskbLeZ8EhHDvPOfpWiqkocL/mj
Voo8vDWEok9pwrjWeEcXcVrVLOdUbGb6yxvmLb9aObZaRJ6QdmnBdwjM8kbB
PUdBYcFEJpBXdgqTSwaucKMCFy23zVbmna05Osw73rFTNo10wsq8TzZnkXkv
bd5gP/6SQdXMOx/1myYuu3HEhdcMOveH/b72vbe+9O1unzuvm+EUJP1ATP2t
g9RfnWI11F98muDJsOlfuv7Jv3T9449LQvU79ZPv+dn+2xPLLzkF7hw8u4hf
IQcYhFWvkAPsK3OAYX0VKVfhmsWJ68FZMQ6cFQ5Vk0MXcEgAUlT5BOCqNWSk
kkdV8P7t9dk/88y3boqjx5tbjnsrBfgobHsv2CcOMiXlJZID0L6dIXYU/BKK
WKbCqyBnKJlVrLYKMoTVpFaJpbCiH6NxqdWBw7yzXycEVdVDPEurQJCirTXq
uNZEvrW2pEW0Aqe921yA5OKCS9rgrXsQfIPBgAHWC8gKemD5WUxWXtmuqzX7
HK2Mtto1a4HttJo43forho1b13/kavOe2RxN5rx64uW58E/ceO+EK28ffckN
w7595YCzLu3zle90//w3bX3KcOqfT7cGv7871ab7/uqzVkb9D0cowJNh0x99
rOsffowRyhyV87/8P97zc7uOvef9sbndfNE/wIcFJzY2WHlh5VDlfYDDoRcY
dletgUZg7K6i64AnhbX1HlULaAuwt1WwURWAKmJTJ9m/VFIBp47aU9eLqQAp
y5edNvbtsLF3e7XYUYVlEm2s4LVXkksvae3T/ISJ2mqgjatUbVUFlv2JqtOq
A4d5Z79KUMUjPg77PVvjh23kc/BDuM+eZvjVWsnfQja1kTtt/MEK3DyeAo4l
HluRaBRkPpFImr/wu6UjCp3UanaO9xZqGozAcmMurD9wi5VX5l2rodWypr2W
Vot3zlywfcpsmwYcOXGDeWNszp83+y2Df+XBZ2fe8ejkn947/se3jfrutUPO
+2G/07/b89Tzu33mHMupj55mxdTffMGm+wBSRkP9t3+1hDJ4+oN/loSq430e
e87/8x0X2p6y5Vd//fjX/mPBmWxzgEP8CnsxYck2Ao9jjcDorJg8cwugCvaA
WFTBakWOKr9RcVPwVMCQCtf5az3qLvtnJZWtUrX6V/JxNy7JcwrElFNSIKP2
AqG2te0xsdXe7t5qY89W/yE+bsJ8Jt4BZjlsuQiKzLIPRVaUWo2irTEOypJW
BWyVjMgodA8eZZsdyv0YHTjMO/t1Mn46+X9fu9go5vRPxZ7r5G+1qg+2MmhW
QBsrmeE33MI8G0Fz2X4uVtUCDwa0Ze3Ze2yXG9gOwNq09ZDtFN7gSldNVlvN
XWzt65NnbRk7ZaM5ZPqPWG3+oVd6Lnn6tfkP/domAG+4Z9yPbhl54dWDvnFp
7y9f0P2Uc219Cjj1vz9v031//klbhPoTR6jf/2jX3/tIhJQ59xZ/+U/f80O4
jvbGv6Mf5k7RfsObBox0OcAxMQc4auJ6O17JOSu8CRC6gOf4LuBoq1i2a/EK
iqp95vm5nqPKd/5i9s9LqlZfomoO9SngFIgp0FDAHcumLW27TGz2t7shtvDY
Sii2zdKN6yzGrP0JtvZnCltqblAxYPCqFs8NnlQONkDMXFWr8PrAo+qETOQo
ZdP74RIEbOWiT2QdQ0bRj+yAbmXElssQHncGQucbtCKr2bzMd+31wNqy/bA5
BNZuPNC0br85HBavsLQyp8ekmZvhn+s1ZFWX3kuffX3Bw7+Zdeevplx/z7jL
bhp+/hUDzvhuzy980w6jMJz68BetBf0vPmUh9cdOQ/3eP0VCDfnCP7/XZ2wd
Jz1yp5nIAQKqJsJ8dYEqq6p2OlSFWlWYqrR+0wFAFUxTB1SB989m/5qPB0nl
Un/mqDzqOGWTfrudktpOCGXwtKltJ42NPszjPjaHINjygkuIrO2KztrHgbU/
8WAczHvgs8Dal0kMqr7Bav3FRw4WiqxDYUQGU1V5DwZhVuaqBysVXFWkU4cv
XVrxO60cQa3k8bbkE+JnVv/OYR0PdXRAtQsEFzoJDzqdte9AM2QFd+62Q5nM
IbBhs7Wyr3S0gn/RHCnm/XC/Eau79l/+fNeFjz4/26DqurvHXXLDsHN/2M9I
qs+cY30UH/6SrUz9+Sdsrs/IqP/3HyOkXv3sF97zI7SOdzl2XfKPR8/NWjXG
hvFKOF99xrxtOFx9vldVEVVNNgHoUBXsf75Q5VAFyz7M2zOfsrCSyqX+oD5l
k37AKQcpoNKODTa2m1jftmO9v7VhHt8YsQUUA2xRwbVnawKsEEJn7dtVAqwC
ZmW7jPcl/Vl5AwajVY5cJ7CzmEThVaMqvSqc8IIX9PFWP5WFPtLayu63trL7
DYRbPOrviMeZH177fO0vhu8BBp9x5eVt9g5brWDJsMxyHgwAllFYkA+Ef3Tx
yt1zF+00x4g5WIaOWfvW4JUv91j8+Etz7vrV1GvvGvu964aedWnvz3/Tlqj+
/ku2VcqIqT/4aNf/+g8s11dHHSZ2X/aRtrzmslurQhewecrNX8JQtXItGVWx
9aBFlZtSC56KAy77B4YK96Js9ZLKnMDAKdBToKQspAyY1rVtX9u2bW3b1jX2
FsM+vs5iaztiC8mFwKIKayurZymJwVReqY5BwSy1RSufGCydRlhp9mBFhdUI
tgqvGlVlV6puWlkkxz7/UDBFhU7lwI05uCm7NRr54u7shkP8Q/gj2BcyVr4C
tmyG8NARC6x9Blj7jpp/2pwG5lgw72aXrNwzZ/HOKbO3vj1pQ/8Rq1/vs/Sp
V+fd8/i0a+8ce9FPBp9+cc9Pn2196UZP/cm/dP1doqTu+MS33/PjsY73bRy8
+C/bMuSis5VgsJJNAK4zqNoPoyoAVTt3H9299xh4KhyqgqHCPPmt6++QPcMP
uvoUcMqwxmiobessnraubttioqltc5O9tbHaPsjIhcza4MPnBkNWkFaymMhK
i1ma9UJhViqytJ5iNSuoyKsEWyfSMVhtGsZJP8zfv1eFn65V3iJlCpCUFUTV
6EPRwGBRxpQWtyDb3Y+EamnORGYrdHNmBWeM8BXY9wbSy2ULXXkLhxbuO9Bs
3q9u32nllXk3a46LOYt2vDNzy/Bx63oMXPHrNxbc99R0o6ouuGrgly7o8fEz
bH3qTz7e9b982HPqqU+dtvMrv3NSD7o6PnjRfO5/VckFMwDBrE5qVXZOxR5n
/zt4yNrUm6FQBYYKL6lcicrAwnJqo5VLhkQWT6vaNq1s27TCxkZ3az40D5oA
Zm2hzEKdlcgrBqwtCbAKFJYAVoMKq2D8hS6yclKrrJJV7hUsU1gdPcw78VXy
07Un85Yh0fHkw6h9jleTNhABRpEXkT4sxZcCqPmYMlbrGL0flnuWBHfm+C8S
0Ga/nxa0arSYF/qxVvM2df+B5l17jpp3sOs2HljWtHfOwh0Tpm0aOGrNq72W
PPSbmdffPfZbP+5/ynm2RGX01O/8neXUqi/94Xt+4tXxAYhj5/+hiq11DlVO
VR1xqurofo+qUKiyz+BQpTKnveGCwYfn1GoLo43L2zYsa9uwtG39krb1S+0d
++Ey+7hgFogsYBbQiuUDObNKgZUaBRsDVqaPmGGL9GRV3FRSfbtWVYXFydWR
w7yTX9VQdTyRRa1FVBIwag1tuiCLWquAqVlBUl4WRTyxNZoEQMeOZqGD4jra
ckhPun0XpD3TDv//7b0H2F3VeefLTOokmZtMMiV3Jsm0xLmZ5GYmN5kkk0zG
yXjSXHDHBRtsbLApxnQMmG46mN47iCIJISSQkIR6772jDkIgmumdfdfade21
3tXevc853znn/3vWo+fTV/fZ3zn7971lvbv4C+ot/UNKu6mw2LsfiiP5MB/n
K64Ar73xnrgmiOvDkztfWbn+hRkL945+bNtVt68+8fw5nz9i/J8fKPv9fvH3
pKdO/9Mv9fwShzWoy9TWSyKqKjrVpaqyQlXeUPFmXqV6Zb+0ibCMCJSEhoSS
9mxMdm9Idq1Pdmdrg3zPnkxbprCUrODzu+v5QFuE9ZyeEnR3tnckwlJqWCGz
LyKcFVjMgqpCVBVeJ9Lio2AlvR+mpPdqSblaAFULlN5W7kNaRkBvVWJSZfTO
m7XnTK4h7R5w9VvtVCNc9Cd5fjD5f8u/x9Inrfih773z4fvvCWe98+4H4oLw
3Atvbdv16op1zz8+e89tD24849KFnzt8/H/9+zzpt/Zjv/nM3/3Ku/84LOPN
sbq/fnLdyeLfd4/6I1NbQlX5kIpsO5V4AountHCBUMaLT0vXSE9tlm7asSbZ
vkquHavl2rkm2bVOvj8T1tOKsPZts4RXe6qOC61F0B9evUBsH24eYbm3YtG2
ahJkBYdXTS7mfY5PVWSsZPzXaqX39P9W9R01cfcu9e+7VUotz91ReiJSc8U2
hHcoMb1j3Du7fF5V3arGHXaqyZnpUz1rkc2GyaSvjvyQyheO3mqbvnbkELNX
5A965+0PP3j/zTdkvmXX068vW/P8Q5O3X3Dt0i8cMf4jf30f2vywurPePewj
6n8//Nwvvn/MH5vaygtVUlUvyPZyYRZhHCEg4SNhqK3Lkq1Lky1L5b9PLk+2
ryyEVURYZnil2Wq/2iWohlfOdgv3xmF/x0VgStA2qSlYWLVZ7o0HXzS5mI88
bHvByPf7VfVBiJI0N71XX46IiWpjqBeV9FcNufQknpK+q4VLb+jhknr3N+3e
OurNC8pnezqBU50hkx9Y9mrKXlnq/sey2alqedonX2jidfTmqx+88/brr729
++lXF6187tb7N3z1e5PgKayRsg7+dUJb4okt5CKkIxwkPbUk2bggWT8v2TBP
vrF5sTTXthXyQzvXpuGV3VbPUrZSXykv1jcOkzuw/O0WYeGVTVjWe4647zzi
aG5vFmT5rvyRrugl5dFqh+19P01cq4OyXG56j2p1qDU86HqiWx209pjst2n5
7auhUy2bV9707ZV63KS66cVqAmcxQKb24q32Pz4lX3TZ7pKsfVem6LflnVHi
pZrVoPN8yHb5meJ19947Lzz38twl+0760RyoCmvErdn3J0f/IaGtJ1ck62Yn
q6YlK6cmq6bLt6WtFklbPZnZak2crfYrtnrR2WvxE3t4RRewXvILS9813CQr
SFWvInZj2Vvcwy7+3ecAhfAv0d4IfD9NiJve97rJbMMj+vH05J6t/8GW1qsk
VTwZasOT65WmylBU6PQaradsgExhqL3F3vz0JbY/fcVlbhKvRCmmLfLlmRed
18mX7Y7VybaV8lUsXstblsgl/gp9crl4/4dPb33vzTexyRcrYn3xl5Mjfjfo
Mz/108mX/6X8N+STP/eLtf8e/jvJ1/9d8smfkm+f8XfJWf9o23ScP3uFrbYU
sZXMBK5Ndqe2eqpuq6xuta+01W4qE+goXZmDcPcbWffu5APrwy4CbzhS2SrS
WWEX/+6T/eioA2CoyiVEW5e4NXSyb1CiesXtyb23jN48d8mp1JPFTVpyr5bf
K+9fUL+FQTYIuphvpoRRT+fb8+WLK9NTaag0aHpqU7JH1VPhpk0LZZ5E/PG5
enqyYmqydFKyaEKy4KFk7mi5Fo6DqgZ/ffrnkh/8TfLt36nec9C/qN7+zM8n
R/4B8VVXfiP57C8khxQTicWXH/Pf5BvH/nFyy3HJ9/+E+JJP/Uzyrf+cv33K
R5NLvpLcf57bMrwlY65Z9yVLH5NP6TWzgr4k23ulbrwqbfX87jhbRVev3PlA
X4ugS1Uxt8fiZQXfJlTluYB3l06ryvXtPNk8i6TsoZMvenK6SZfU67Sh3lKS
ezVJ1Q2lB1CUoV5WDPXC3jzF93yR4lM3P0pJbZYtT1kMJQy1faWMmLYslSmR
DfOTdXOSNTPky3nZY8miR5K5Y5KZo5KptyWPXpeMvTi554fJrSfkP+j4P+v9
FRUrWyIS+eHfJSf8ufTLyX8l3/juf6k++rlfCg1V0tUJU8RpRYQ8GxfKNebi
ZNodyekfS6bcktz5g+Tyr6cfKpZ4uoo1b2yy/HH5x9V6seYmq55IZo2SZhRv
iyWez2tnS0NNvT2Z86D8u0uqaqb8kPiTTPxhJgOrlWQasNFDMNsCNVv9pG4r
YrqFNvPWHHChpgQtmcC4+bcuZ+WPK0hYIzeqSuIDq5ZVZVXSe3roFJbZM6Kn
egxFJvfIwtPb9X5yrQKluonO8ikxlHkXgyLL99Eq0feUkujbWUkqD6OyRN/6
VFJrCkllYVQhqZXTkmWTpKTEFUC85Kfdnjx2fTLusmTU2cktJyRXfjM56+Py
xx3+EfnvF3+l91fpPl3HptPmD/yZ5OBfl29849/n7//yr8l/yxN7sLzjYfKZ
f5b/91u/Lf+99OCOaiVXALk2aksRx8x7a/8VT5s7TknuPTN58EfJkkdrHxLP
q1n3F/9dmK9lk/M3Ni2s3imehOJfIRrxmU/cWfsmmapKYW2YV9hqXiUp+ZSe
JeUlDmD+uGTRePlTxDM8U1WZA8wqVjvK9vUNxTZhbb+VMtFC7WDX61YysOq5
7ruwXLYKu/h3H15kp32+KqnoDsBauGQ07IVn9ojaU9G5947RuVd2bBK/rJAd
TwEtfLV7lVZhlG4oolNiR94pYUpqZ7rBREhKvEg3L047o+Yma2fKV7T461S8
qBc+nMx9MJlxTzLl1mTitclDlyT3niX/TL3ysOT8zyQn/kUqqV9OPv1zHx3y
NGDpF3Md+QeVX8x15j+0drk46S9lqCuu0quny78rxNV4/Zz8uk0vSkDiAu7Q
E2Go+bo1bEuEMPo7F+prU/HvprqqxBsLxtXMJZfxI6rjnFfFVrmqZksriSf2
iikynhIPU7yxeoaMs8Snie+8WVXV6rwbcE+9v0LbbFV2sOvzl57SbFX1sdvm
3P6kHlt579tYlq7IIe3uEYKu/sCg8MrwlDMr6Lvyh2ui7/CpipqPxwydyBjK
yOy984YuKXJDrlqKeqNeilL3QOmdEnqrefYMVwyVtfNlnioLUrvrkZTZMrFW
viRly0RWk8oyfunLWQZTk5PFE+TFYY7w1N3SU49eK+Op+86Vfx5ff2Ry6Vey
anVyxEeSr/4bWbz4+r+tDink2v75f957v3z8AH827Eu/JmySHPn70sjimL/0
q3Id+ptt/kWaXU49a551lepZ/Iiswrjc5JSUd1ViCtYTsSg9RS0t2nILK7NV
pqq1s2SuYNV0+QwXS2hLxFniQ+IzbarK9gXrgdWTlaq0CbelqjRb2TYI+4fc
ektXZq9FwJ2wWPdwLIWVP28DuwSbXMz7HM+je09tjagNLNIvEe85pueRPRLu
LJ+R4qvVoV6jS1H6Lt2iX+I1sxpVbzgvN0O9YE/37duWN5xn486yYEq8BrOy
lOydWCpfoVXvxIzCUxNTTz0g46mpt8kEzsNXJA9ekNx9hgyprv52cvFB8hhO
/IvkqD9IDvtPyaG/Jf8V1/Pj/yw5/WP6eT7uvyff/A/JgT9bU8ARv5t847fk
ZZ8Rp5hLHIn2/b3rK/+qkV+euDO5+3QZxVxxiCzeiVOXLeH6bIkLo1yz87Uu
W3PyJc5ndsFsRVKd09OG+OipE3qyOcujqrqt1ma2mln9psRvJMv+iT/VslqV
VFW5I3gtoSq9d31HfbDtbv0GIh5bmRMtbG2BjrkW9ubAjtnK2XRR7xJscjHv
c4JUZYx4/ag6LIKeGkG2lyuGejuwFGVs1NXmSGiJPrLnvJbr218zVH7rUiXd
l2X8niuaz/NgKt0Y9bQRTJXtE5mnsu2QawtPLc889bBs8Js5ShayJ9+UTLg6
GXtJcv95spx907H5kZzzyeTUj0oNHfWHUlLCWcf+cXLy/5Q1/R99Lrn8a8m1
R0iv3XVa0JX/8N9NDvmNXCJn/K0s0Fz1reSig6TjSrl85ueTk/6yzXBG/F39
42/If+PW9HytztYMxVOlqmbJpdpKU5W6eKrqnqF4klpISKoVQwU5S1OVEltl
4VX2uxDvLEMqU1U71ynzK7KBS6mq9lKq0qbaqve6qm22CsgEem0VGl6ZtrIJ
i+Gs4H1YTS7mfY5PVcYccnUCua0C5W3k04ZIOOpQb73qaTs3N+2q6b7iKVq7
qMrm82fzSKr0VJ5qoMpSeYPfliKY2lBVpraVbX6Up2R9anwyb4xMJU2/S6b+
REg1/sfJmIuSUWfnB3P1YbKR+LwDpVNO+V/JCf8jOf5PkxP+LDn5L5PTPpac
/Ynkwi/IBi2hqpuPLerp58s6l/g+E69JJt0gv+2025Ppd8rY5Lojk8u+FuEX
EeWJ43ziLvmDJt0oK/jLp8g6iFgP/ig590D5CUJA4oeKRyQ+QTwQ8cbjt+RR
jLqiJaWoarWhqjVKSFVFVVpg1ZKqQg0VaSteEcotqUYaWmSshcW/pqqMCIus
W60vz/m8vBlDfEOpqnTIUq6qtAmwNrxCDawyVVG2ctzlqoqtqFuH5MlAcqJF
cDLQlgkMEZa3m50dYTW5mPc50aryFqHepWYc1apRb+qJPn+zBLkriurrq0dS
ip72lzukikhKKUuVvRN5F3q9MlXe4yBP+pmeWiRf2pmnZLNf2kexNG1Kn/+Q
LFEJBciQ6mYpl3GX54d0+ynJTd9LrjlcyCW54AsysBIx1Gn/O/nBXyen/U1y
+v9Jzv54cv5nk4u/JGVx7XeSm7+f3H6ytMb950rZPXxF8shVMl0mIjVpqzuk
DcUPEv8VbwitCEWKn75gnNSlWIsmyJLZkomyTCa7ix9LC+LpEnGfOGC5puRr
RSqs2Q+k3fWT5DgCuQw9MW013QimzJBK8VQVUrWtqg5FUi1k+ZRc36ZYSZlK
sq2FyhuUtjYZzjLzgfkpSo8291QWUi3Pm9XLGbamqsh7hZQVK3pCoBlbFbZ6
yaxbkVuuLLYKmmthE1aUrRwTmXwRVpOLeZ/jV1VQAPW2c46EvZfPtSuqiKFs
BanX6pFU0TVRixqKxgn51M0iqZeKSKqW9CuDqWxL7440mFLb/DbldzoQfxzK
jb2rZHFKjadkfWpO0RaVbu9dPFG2/GUh1Yy7ZXgy6cb8qETAIowjQqSbjpUO
Eia69CvJBZ+XUYyIbs76uJTUuZ+SbYEXHSRFdtVhsvvi5uPSqOqHUlWjL0we
uiwZf6Xs0BCB1eNpfCSiqul3y0yj+IlzHpBZx9JWs+5LbfWItJU4MBHuiVXZ
anK+VGGtKISVrZVT/cJiJv0cqT8tpGqoqth4qleS4sVQ4YYKtJizjLVhQb05
RPGUHF1rqErtrJC3CNlUqcoMrJ7b6eoGtI1diuiy0Gzl7AwMirCoTCBfWHZb
NbmY9zmeR+e/vaCjne/NYpO1oxplMRQZSb1B1qTySKrSU9k4oUxDMoKpuqTM
YKqW9Cs8lW+bWp3HU1Ufxfxk3Vx5IRWXXHExX5am/sqQatYoIZH82EQc9NCl
qarOSu48Nbn1+OSGo2VnxRWHSFtdeJAU1o8+J5eIsy76ktzpI0QmIi/xabcc
n9xxquw9uP+cZPQFaQ7wimTi1TKpmAdWt8t4asY98idKWz2Y2mpsaquHi9hK
sdXSR2uxlVSV01YrmtvKFk8FhlQNa1VRqmKXotjVKEYpKt5Bm5Vl/tchrE3K
gVWZyQXVkeSeKkOqFUZUVd7NamN1E0atuUILrEpVVfNsjaKVp8XCmQk0m9hj
OwP94RWvemVxVpOLeZ/j2bflr0NRwVQ1CcRRjbJ3Tei/blu676VaAKU+CWsT
J56tt0+Ylal6ceqZstOvKE7tyeKpLO+3ulafEq/Qjdn+qdn5Pl+Z+puUdqfn
IVV+eCLwEU4Rqhp3uWz/G3V2ctdpyW0nycDquiOlrX58qAygLvmqLF2JJSR1
+dflRB3xIRFSiU+79UTZiZElAEtVTbha5gAn3SjHDky9vQis7pVhlAysHpTH
IG01Ls8EClVVtppYj61KWxXCWhEfWzXM++Uh1QyPqmxRVWupvxhJbWipoy8u
0RcvptpanC6fqjZrSUIzQ6h8w9JTW9MqVeYp8XrZUYRUZQLQ0Qdoa1wn0oB7
6aJVVJcFOd42tnQVNDkwoIDVQFUjZLBSR4mJqpwzJSpJeUtRak2q/ivTfsWv
G5J6/eWanojNvLWhE3ow9eLT9V70YohfFk89Uxanttaa/aoxFKtq/X65p8rU
XzrZrwipqoMUYc60O1JVXZeq6jKZvrvv3OSeM2RCT8RKNx6TXPddGTpdeZiM
oa44VGpLSOqqb8l3ig/d+D35abefIju6RTh2/3lpAvBSWa4qVZXlAPOKVRlY
1dOAWWyVqyqtW4klVZUuIraihLXSJyx23o8IqSKjqnZU1YWOPiOM6ribTE85
lj3m2qz9rOJLck8VqT9aVWlURXRWlKpyVKzUERa2O90/QxetXvbFVuGZQKJ0
pTQHvu5OBrbRbtHkYt7nBKiKDKNCqlHhXROap7RhfZqelFD91RfqkRQVTL3o
TPrlt+qob++VTRSb83iqnERR7p+S+3yzElUxN0mm/tJuCnE9XzYpP8756Rxa
Ed2IMGfa7ZWqHr5cNkUI3QjpiMDq9pNlF7qQ0fVHpcI6QupJriPkf8U7xYdu
Pk7GX3eeKu0mwrEHzpffQarqx8kjVycTr8vLVVlzxRNpDrAMrGQacEyRBhwn
y2elrZaUgdWjuq2WcW21yqaqgLyfWaVy9VQ0UFUrwVRbLROtSMoVN3nFFKAt
60eXKJJaqqf+8lsD11WlRVVquUrLAeqtgI7edUf7elgmMLyJ3dFo4U4GBmYC
3bZqcjHvczyPjtgYRXaeB0xAIgdNmIYq0n01PZkTJ9Qtva+oklI89WK9ze+F
p/M/zNQOCpn0K5oozKb03FPKPl+tRLV+Xj6VYvX06miFBWRDxdi092+Uoqrr
ZRAkQqGxF8ty1X3nyMLTnT9IbXW8bPATVrrh6OSGo9J1tPzvTd+XHxKeuuPU
NKQ6U4Zj4mvHXCyjs/GFqh67oSpXVTlAtWJVBlbjaoHVYs1Wk/I5OdZMoGmr
wKgqMPUXEFJxVNWSpxoFU23XoZgRUxsrv4VN2kGR3f9X3gK4jKeUKpXaU5Gr
imoCzGatl6rat4NuBXR3A/pbLAK7LBh1q+DSVWCvhU1YTS7mfY5fVbYJ57Sh
zIJU3VP0vL483VfTk20skjl3IveU8neU2UHxwlNEp1/mqezPuWoSxWbKU0XL
39ZidFLR9VcdsAypioaKXFUPSF9Mv6tQ1Q3JhGukX0RANOZCGViJEOnuM6St
7jglue1EObdWBFDCWXIdl0rqxOT21FMi/pKeOid54DxZ6hp7iax5iW+VJQBL
VeXlKrW5IqtYjSYCK6JoVe8JVDOBVlvFqMrVSmF6yh1StauqTmf8ohJ9Pk81
11N2rzTbf0sZudbSwlPL0mBqmaKqlbSqdptR1WZDVduIchVdsbIVrVqxVV1Y
r5qZQPvtrrylK38y0C6sJhfzPsenKsvuXX1vlNNQxKyJXFL6jlRi4oQ5Bf0F
vX3iJ1qbnxZPFZIyPfXsdqOPQtk/tWu94qnyloh5SFUdcxpSpQ0VMvsnL/tC
AcIIsqfigSqqmnKLTNNNvDZ55EoZEMnA6oLUVmfJ2EqY6M5TZTVKiOm2dIk3
RLQlFCZEJocOnSm9dn8WUmXZv8vlt5p4TVqrMqIqmQMcJXOAs1NVlYFVudNq
kWKrJWr7+mN1VSk9gZqtrDnAQFU5WilYIRVfVZ3L+EUVpHgpvthVuCbIRz5V
BXqqpioyqirKVdWsdYuqyMBKL1q5WyxYtgotXQXbKjy8ehOqksSpKiiS8kiq
5qasd4LuQje39BrtE2bS7yUz6ac0UeQzW9RJFGreb3N9HkXaSlGpSnb9VUcu
s39F41+2lyrb9iuCFGGB+eNkhUiq6l65qUqq6lbZ/PBoVq66Ihl3qTSO8I6w
j3CQiJhEeHXX6amzfpAv8XY2Ge/es5JR5+SeGn2h1JzM/qVbgEtVyVrVLYWq
7lJUpeQA1cBqERVYZbZa1iywivVUW6rqkKeYwVSUoZyeatlQPCtpeioltbyQ
VFaiSsdT1KpUavYva6sgoypNVfbA6nl3YOXYadUstrJNC2xqq/zP9dDmwCYX
8z4nSFW2OUj62PPXyVyfHjr5h06Qs9DLpN/zRNLvpbqntLwfOYlC2z9VzqPY
Y6b+VlYHX6tSFQ0Vq6alqpqcqmqCjFlkT8WYPAEoVXWHVJUIfB67Xspl/JUy
JhKRkfCOtNV5MrMnwishLCEmuc5IDXVmKqmz5EfF52SeGnNx3vsnQqoJV8kw
Te6rujFvq5iqRlX3Fqoq+wA1VY03coBG73qtv4JUVUgCMDakmmX1lE1VIQ0V
sZ5iJv3ialItjmHEGozlygQ2uZj3OX5V+ef1EZt562fedjdeLZiyzEdS2/yU
O3cYHen79FlJ+lg/NZ7absRTmw1PrakeQtZNoVap8pCqyP6Jq/fySbmqFqWq
mleq6h6pj6m3SZuI8OfRa2WBafyPZbFp7CUythp9gRzrJ4V1buqss+W67+zU
UOfK9z9wfhVPCU9lVSoRUqmFqsdvURKAWlT1QDLHiKq0wErLAS71qmpKcFTV
PPVnqoqV/YtSFSWpjlyX6N1MTSIpbjZPW1uXVn195soiKTWeknk/W0hVZP+s
tSqjXFXmALUBthEVq+CilWtrcHAmUA+vbKUrPcJyesrIBDa5mPc5nkfnGIhU
9xR9wvWxSNR+3iyYep0YkVTvoCjjKUpSZBPF/j2Gp3ZS8VQt9Vd7FLUq1ZI8
pNJUtaosVKU9FTVVPShlIayRdVZMuUWGPzKwulaKJrPVQ5dIAY25UJauhI+E
leQ6L13p2+L9Mpi6SHpNeipL/V0pPZWFVJNuTAtVt6T7qlRV3ZvXqrKWdS2q
qporJtQb1x+zj7Cw5QAbqsropvB4Kjr715pWmjZOOHN9TatRjd1U05NFUk96
PbU699QOpUpFFKocqtoepiptj5WzvyLaVt5MYEgTe1Bb4EfLRJOndPUqVOX6
sPUGUsbNK+29E9V+Xm3DlDkfibgB4vP5DT3NbVOunVPqJt8y76fOTaqHVHs2
VQ+kCKnqjX9LK1Wp2b9s6F85pGKpoarZ90llSFXdUQRWN+ZpQGmrK6WthH1y
YV0krSSCrNGpnqShMkldLD+hiqey1N81ekiVbQEWqppxt9wFrNaq1N1Veh/g
BKoP0LIj2Fur4lSpjJCKyv41UgwnnupEuq8TfX3h5acQNznCKNVQK4qm9LKP
ooinSlWpDRVVocoRVZEbgbVylS+wCuoGtLVYGKqK6mD3NrF7BwaGdAY2uZj3
OTxV5W6q9knZbiZlZvyUe3YQHRQvGHeYUiTljqfKe3mo9Sl9/1R5Uw/pqdrV
LHsdZZ7auabW+Pfksnwv1WYl+5fdqWd12lOxMu2pkO1/hapkB+BYmXYTppiZ
lquEQfLA6qY0DXhdbivhHdllcZk0kRRWti5O1yWFoS6T62HTU1lIdbOS/Uun
VcxI91VpqlITgAse1idXaJ0V1uEVsQnAmqoa6Sa6R93RTdFWs19jSRG5vkBJ
dT6AInN9papqnlpVrarxT8v+aZ3q9qjK7KxwB1a2VkBXYBWyL7itLovgPcIh
tmpyMe9zPMOjjGDK34Vueqrq8dNu0atl/MxOv5B4ai/R72fm/fJ4anvt6peV
qKquv6JKtXN1PkOpFlItqrJ/2X3lsluglqpaqtSqMlXNHZ02AZY5wCywUm2V
1q2EfYSDpLAuz4MsdQlDiQ89/GMZgmX1qZqnbioaKoqQSs3+5QnAtANw7uia
qvKoyuisCCtXMXUTvec3YDsVZ4xSKx3pqYP8Gb/AMKpzpSi2oUhJlRm/ejy1
I1tkSEUWqtRm9S3tqaq+I9h6lxBHGtDdENgVWzmExbuMDwSeR+eQlGcWuiop
tcfP6PR7td5BYQ70K0N1YqxfvS/9BSPvV3iqulpmrwizO73splBDqnKM0pbF
uaqq7F/RU6FGVWUHoNDB/FRVsrPivrxlPQ+sbpVy0WwlhZXmA4Wz8nVFurK3
VUldW3jqhsJTRZVqWhFS6T0V9Wb1+bqqONIJSgDGV6n07B+7ShXpqbhOv8Cy
VNueajPLt6zWcx5iqCqSKiSl91HUPbUrW0pIRWb/zIEVerkqIAdoG7XkDawC
i1axmcBWbEVWr5pczPucYFU5h06QPX5k0u81sjhVPBn0Tj9y55TmKXUYxVPV
ENr9u2uX1nIX1V61Oz29EVUZUmWe2rmaGE9RNVTUC1WZqlaoqkr3VQkXZOWq
uQ9KX8waJYtHwiNZK2BuqxulboR0MmEJDWXCMpd02TV1T91IeEoJqeK8U02v
tUVV5kZgSlX6AEC7qmyNf569VPGqCqxSBUkqcD9v1D6pdoOpkCwfqaflyjZe
t6TKDgol76fHU6WqqOzfnk26qmwJQLJcZQ2sHKOWQgIrx04rnq18w9i9txKG
quoEqMotKaN9wtrmZ9/b69o5ZW9Kr/ZPVZ6qXYH1VoqsRKUMUKpSfxuMboo0
pMpeyLqq5io9FVmneqaqyYWq0sFKag4wC6ym3yNnyWZpwCq2ulG2RmTCEibK
nXVN7ia5rs0NVQZTxS0aXfbJpipVUdUYvVZl2wW8NHDCEiOqim38a7ztN1BS
Eam/BkMnKjdFdk001RM30UdISk36mfEUlfrLGyqU9r8WVUU3Vxg7gkM3Bfts
5WpfD7dVbOkKqsrxqSpKUrZgipSUZefUT3zxlHGv+dolmixRqd3pauqvVqVa
SzX+Lav3/pU9FfWoSlyuVxRR1RJFVfPHyuF7cx4smivulZtzZRrwjiITeIts
ipiUJgMzYeXOqhZtItlHcUu+57dsUK+qVFr2zxis5FBVdFTl7FSPCqliq1S0
qnipv7Y6KFoMo+IzflGScodRNknpIZUiKbVBnWiosPRUmB2A1iZA25AlR8Uq
qr8i6s5WnbBVQN2qycW8zwlSVbinXJIqBlB4BqR74ynpqdp1O2+l2G2UqOrd
6eUrorwHvVql2mVWqZYrQ/8W5bc9Vdv/xPW2VqtKo6qsCXBhmgNckOUAi4pV
th04SwNmtqrCqzzCIqyUhlEyWyg+oUz6ZX0UuafuqE2oyGeq31fNqs0LVcpw
9QUP62MAHR2AEQlAo1OdOZmW9FR8Q0U7nmo2wY/R2seLpHRVWXJ9VRgV7ilS
UvX6VC2e0kKqDUZI5Yiq6v3qZLnKlgP0tALa0oCW3vXWYitSVYxGC6jKp6rA
jJ+5YcqmKnoMxfP1DgrdU/oF3LGFSutO13ZRqd0UeuNfvUd9a9H7p7X/Vaoq
R1VMzbcAL8361Y0cYBZY5f0V9+Sx1RPSVka4VGirttL355K6JZlaeuqOwlN3
KcOUipCq1vun7P9d8HB9uLpyC3urqhz7quzZv4iGirYa/yJTfx5PBe6Z8kqq
Y2FUVDBl7t7VDaU1+CmS0jrSa6paUytRVak/ajtVmf3zJADJJkBTVWZzhUVV
zP4Kb9EqbGuwt8siJLbKVpOLeZ/jeXT5WI/wpB85dfaletLPOX5W2T9Vu4YT
8/3UUbTm1HRVVUrqTwupdq2vVLWj7FEvev+q7N+i/EaKslOdTABOqXdWlH2A
SmCV2kp3UxleZRGWXLfI/J668vffmksqa6LI+ijyVoq6p9QhFbXev/KWVZb7
gERMq2gxpPJ2U4SPUYptTQ9QFa/NLy6YWpK2mDZvOw/cwNsk40eFVKqnVFXp
IZWS/QuNqtzlKoeqvIFVwKbg2MDKOnYpILYK6bKAqhx4gin7oCSrpxw7p6Sq
6noi9/maW6g0T2klKqpKtcfcS7VGUZVZqMoSgPOrTVU1VU2rqarMAQojzJcV
K73hYWaaCRR+yZKBT6TemVoISzrrttxKU26r3pml+8pmvyqYMj2l9KhnDRXz
x9YLVePDVFXP/rkHAIaoKir1F62qDuT9upPxayeScmzgdQdT9oxfiKd2ODxF
hlR1Ve11RlX7AqMqd2AVUrGybAoOshVvSGCpqhhhNbmY9zmhqiIzflXSz3EX
DyPv90pNVXqUQQ6j0Ef8qbdKJFN/2+luilr2b0N+Uyqt9287VajKNnvmnerp
9L9cVcUW4KpfXeYA9QeV9VfMfbCMrVJhFcnAXFiFszIZSXOVbrq9iqSm3Vk1
UWRJv7xEVeb9jNRfGVIZO6qMewE/FqYqbvavlQb1iNRfh/J+rQRTRU3Ko6qG
3X3cSMoqKdJT7tRfqSotAbjF31axb4fPVg5Vkc0VraQBW4utPprfpC+4btXk
Yt7nBKjqFd+gJMcOX6LZT7+MZ8+Hl339fnrerxiZrnqK7PrLVbWFaPyrGirq
ber5MKVl+Z16qkJVqqparWpGHlWtnFrfqVTkAPNWwIfyxgbVVjMLW8nw6u7c
Prmz1HVH7b+ZofJg6l65qvqUWqJSUn/zjJCKvAMIOQAwUFWh2b82QqoWWima
xFOtSGpxmKSWMCXl6u6zRVL1jF9enKKGJlk9pXRT2FT1lKEqT1RlaQJ05QC9
FSvnNquGA5fiGwJr1w1v3arJxbzP8UzkIDz1SmhxSvFU7ddhHe5XeupZIu/n
mEa7f1dYSKV4KgupzAkVlarKnorF+v5fRVX1AQ5GDlA2V4xXGtfHFC0WSmxV
hldlSrDUljTXXVUAVRlKjaRGKX0Uat6v6Pqr9lIZDRVxber1HVUuVVm2U/n3
Uo2E1J8jnqL29m425/h1oMcv3FD+SMqMp1Zqfz1i9XxZYyveZXwg8Dw6l6QK
Vb1G5P30M6/Xp+g+inrqz36rRPUGH/UBSoqq3Km/+iQlok1dSQCqUdWG2h0l
0lsAF02Auaom5xf8LLBapARWqq1kbKUmAwthzbyn0FYZbd2tvOeeqiO99FQV
TKWt6XPU7vSx+l4qvUc9uE3dfat6XkNF7J7fJqm/fKtv2/EUI5gyVu8vjO5g
avtqSzy1trZq8ZRWpbK0qavZP29UxSlXGTnAkIqVtWjlja2CN1u95oyqHHWr
JhfzPidGVY5g6mX9ya83paubfC3D0l0j/vbW6lPPG54iUn9mSFVXVV6l8vVU
pAnA2kPLtwDPTussM5TOiqlGYDVRtoJnsZVmq0pY90vXVBHWqMJExZJ6KhJ9
maFmjqo6KGbdX9/qa7RSLKjvpbJVqSpV1W8BnHeqmw0VjAkVnWhQbyueChmX
VK2WTWEGU8Q+Ke/EiQYZP/1GHs6kH5H3M2dTlNt+bT0VW/T2P70DMKZfPSgH
GLzHqpXede9mq9fUWpW3gx2qCleVXpnSX2724pShKmoILZH3M1opiNSfJaSq
xijVs3/5q0aLqox5Sqmqao8ui6o2qtMqspuAKOUqLbCSF/+JRhows5UaXpXC
ur+wTxFnVfIaVTPU7LJ9ogimqr50Mp4yPFXeo2qxs0q1jNr5yximFHVL+kpS
nVJVC3KpRVK+eCq0wW9JB3onbO0TpqfMYEproqD60ukSVRFS7aZCKjqqUjxl
2wLsV1VYH6AZWJG2Cg+seFMsrD2BzlGBTS7mfY5PVfSspOqPAX9TOpn3258/
GYh4SrlVopr3M7vTXd0UiqrIbb9Uocr4W1fpVFcSgDVV5Z0Vasv6FJk0KwOr
0lbZDNtMHFWXxWh5T6tKWA8oDirazit/3Z/HUFVZ6oHitomja/Wp+eYuKrOb
wredqnbvj8etVaqaqlRPOcYotTOWli+ahk3pLU9E5+7nDapMBQRT1njKaPYz
835Vy58zpHrKoqoq++dNANrGK5ETlkKaK/bZ04CWwYCh3YCNuiygKhtBqjI7
/fRWCqopnQ6mbK0UVH3K3O1Le2qH0U2xzQipDFWlnqpdwbSRSltJVRXjatUm
QD0HOLmYXFGmAR2xVZYPfLBKCebOeqAKmmYbSzNUnvTLmv3qeb/SU2bqTw+p
JhkNFZP1hgpXSOXopjBUZaT+OMYJ2fAbkvfLlEQIq+EMCmtxqmn7hH9Lb3gw
RXpKK06p8RS1hcqcTOvYTkWryhZV2VXlKVdFBlYvG3e0b2QrUlXcsUuvQ1US
z6MjO9JdE5Ns+3zrcfTL9RKVt5XC3EXlTv0RIVVVqKpd7mo7qozpf5qqNlJR
lboReKWSA8wCq7IbULNVLqyH6sIanWtrjm9ln1am+/JOvzKYKlRV85SzlcIx
TImjqhnR3onupggbTMGfR+Geju4Oo8KTfqyMXyvBFC0pb31qDZX6i6lSkTuq
mqvKVa7iBlaN0oAhM5fctqLCqyYX8z4nTFWapGLzfmZ9ytdKUbthojFAyeym
oKtU+StCvyrqQyrUnoqV+etdjaqybuS8VlX2q9dzgA5b5YGVFlupwioirHLV
wiVyjVHu61Guh+zxVL07ndjzW3hqud1TK6ZGqIcOqex3TmR4qlFIFb/JNzye
amE+UruRVMDeXtpTq+2ecsdTlt6/kPY/h6pq5ardweUqSlWNWgHji1aufcFh
cwKhKr+qfJ6i836W+3rorRRKMEVOTyq7010lqmLc375tZVRVu2xqw5Tynb+x
qirH1ZZjAG2qmppf5LPhFVXRqqxbFcJaaAprbN4l6F9ja5FU1UFR9qWT8VR2
C0XCU0ECCtpLZe/68+yiarXrj9/v52hHd3oqNpiK2jDF7PGzBVOkqqi8H9lK
oQ2m0BvUVU+Zc/+ioqr6tIq4zoqQHKBveEWsrXgVq/BMYJOL+cjDtheMfL9P
VY6kn1ac8s7322+Pp2w3oNcGKJkzlKoJ6vpFlc7+kapaXVeVkQDMtgDru4CV
clXesm50rS/X0oATi81WaZeFZqsyJZiFSGW4NE9ZZQA176HKUJqnspBNbaJI
PeXXkLnn13F3qqAxSs0GU7Bnp4fXp7oaTCkNfm02ose2T7TrKSOk2uNV1ZaY
qGqHPapSb14fUK4KCaxcrYBhFSvH/YJbsZXvyh/pil5SHq122N730wQNoX0h
NJ7yeGpvcOovV1XtSrtvG7Wdyuz9U7N/pqpWVKqq3QGEagLMVKX1AVYjAVVb
GbEVEV7Va1i2taCc5jcu/1cJplwmypsobCUqW96v9FQb4/6a3uE3IJ7iz0tv
1kHRoTl+nmGzy3NDRXgqJOlnz/uZnnKFVJusvX/ESCVVVdsDEoBRnRW8rvUO
pQHttvIWrcIu/n2BTUnmJ7g/LcfvKUu/n9ZHYapKfYa4SlTFbIpUVfrlt7zT
h61Hne79o+ZUqD0Vuqq0zgqqXGUNrIwWi7InsGarQliqs+zL6iO1LKUm/Wr9
fgHd6SvM8RSlpxy3+g3v+muxRBVZn2p9DEW4pOKCqZCJ6MubZfy8wZTbU+tq
rRRk9s9VqApTlWNrlVtV1nJVeA7QDKy8swGb9a57WyzCLv59AUNVKvpn2zzl
7kvXVRXlqdqsP/1SrIRUtZtSWbf9GkMqNFUR7X9mVGVXVZ4DNCtWRX9F1WIx
OY+tMlNopatFRb5O1ZZ0Fh0o6dWocpGSWqKNpFAGKC1Vd/sGjvuzRVW2GUrm
LqpZ8buomk+jjcn7BUnK24vOvm1He73oniYKSlLmrefjPGU2/pGFKnNHVUBU
9ZxTVZ7OCmrIUnQrYONuQP7WYFpVngv4iME8SLeqOLWqUE/Zt1BpU9N9IVXt
mrw/rPFv3za7qjbZo6qyU70cVaG2VSyrVFWOAdRzgFrXurIdOEsDVkUrLRP4
qN4ZmC46UNKXuk+qHkYRHelkMGUO+iPvnzgleNyfY9af21OtDqRt7qmomX6d
2jMVMG82JOkXF0w5+9K9G36js392Vantf+5aVVxnRWAO0BtYdT4N6Cha+XQQ
KI6RgENV2XviEoAuT72oSMp6q0R9qy/Vna5fnG0zlGp7qcwh6kqVSitUmVNq
tVrV9tVUVKWqanG+S5RW1ex6YKXEVmomsBKWjK0sFaUJ+djAamk+shiqCqOo
4X6apPw3+bXsovLPpujkPT6abPWNi6cajksyDKWrirex19HmF+Upe94vyFPr
aVXtIe/3Qe381RoqXFEVqSpeE6BFVb0OrPIXfmDRKuzi3y9okWBpKDJIjFFV
QDxlqkq/EZX0lH59rt0zUSlRqWNp9R51NaoyGypsQyoMVe1wtFVoTYCLFFVp
OcBCVWtm1PsrarGV/qir6lUR/pQ1LLWSlZvICJ0IPRU1KXOH7xKtMkXGU1OM
VgpFVdF5P3J2eiueir0LVfDQpNC8HyOSsu+WCsr7kTfvsGT8XJ5K/yozJ/t5
6lOO3b7uBvWwNvV2oqrYzgr2dmBe47puK1cHlCO28l35+doY8YSq6tWXiHkU
Qa3pcum/C/+GX9sYpV2Gp2zZP0ubOrmpytoBWFeVmQOsmitKW2XDK6YTz8Dl
9WRg2WuhCWvJxCox6FkTa4ZaYkZSxo19l5LTaMnZ6U5PrQrwlG0jVaMtVLY5
tKSq2s77beF5qqXpE03b0VflCQRPPOUrUdVa/jyqCtmsN5yrjK0snrIUrZpc
zPucMFVpeT9VVWQrheEm6y4qYyOVHlKR2353unr/6J4KVVVGW8UOpa1CbQJU
VZWVq9SW9XoaUH/Iemw1pRhm+3jVaKEJq3LWo5W8Fk+sxUra20smErk+rRc9
G+6nd1CQeT/vbRMdM2lHYGt6mKpa6PRrsLfX46nlIRm/nl94rRdkT0gV3v4X
FVVFBlZmH2CnKlZEGtAwlLNo1eRi3ucEqIrK+xkhlf4Ude2iMkMqM/tnCams
DRVmm7p229/izr+77B2A2RZgU1XqzAplbq3+kMtWQLJoZdpKDa/0CKvogqjW
RMNijxIB1JIy11f3lE1SZr+f3kQRmPdzz05XPRXYR6Ft9Z0f46mFdk8Zd/Jt
3VO+Tr+OGMHbROHYPBXaR6HtogppUI/P/rWTAGy9XMWrWDXvrzBs1eRi3uf4
VGX0UaSLeLE4buzrGExBjE9XPbVH99RztpDKvqOq1lNR7P+11qpcOUD9Ia9X
coBVK6CyzWq1GlgptqoaLerhlRlk5avs33uUMpQthqpXpkxD1ZJ+U/KymrWJ
Ijieiq1POUIqT32q8TyKSE+1o5WgjvQVgfEUa+dUvS89NO9H3pHKHE9heMo/
pCJEVdRgJVNVLZarPNuB2wysIhoCm1zM+xzPo7PWp5wtf25P0bP+6uP+/CGV
O/tnFKrK9j/rAEAiqtIvMpvqc2u1ilXWX1HbZqXEVrUui6n18KqMsMqUICms
gLXMaJwgt02ZHRRx9/adHuwpX2t6l/r9asEU3zJ6JLWkA5UpXqdf+PjZejAV
NDrJ3Z1e95S7R72289fb/hfVAahFVXssqgreC8wJrDj9FUGxFVQVoSo670e0
Upjd6Y7UHxFS2atURPbPuOuHqSqtU93VVmHcXVHbCKzmAKvmCqNxvdxmtcZi
q7ItUBPWsslEkFVFW/WwaxkZPWldE450nyPp5yhOsUZSRHgqbsQf0zhB8VST
Doo2tk1FtKNHjkty9/vpd/ew5/2I1J+vR90sVLna/9T9v2GqoqMq1tiK1lTV
fFOwYqsmF/M+J0xVdk9ZQ6qADb8v1kMq1xB1n6r2GoUqbfOvbKtYb6qKyPmX
CcCs5O1VlSOwssZWprDqNqkFRIV6CCtNti5SUsvJSCow6TejXqIymv08/X6h
Jao46ZB5Pz2q4g6jaKMyVU/3tbdtynbrXpenAopThKcs3emuybSOKlVUoSq+
VsXurGDmANkVK6gqGs9EDo+nbIP+VE+5q1TOm33EFarMzb+b66qSUZV+rVOn
VWi7gCtVGS3r6pClMrDSbOXJBE7T84GZsHRt2WVU+chc9oJU/rOiJOW7tUf2
MJ2eirCPf3RS1P4plqda6qCI3tvbyg2n2oyn3K0UDk95GyrCVPUsu1bFU1Xj
wMqxI7gVW/Eu4wNBQFQVEk9pG6nK3yyZ+nPelMqhKnNIxT5L9q8aqK7v7NAT
gNpkdXVrldZZQW6wcqcB19pjK4ewyDiLXmbcZEv01SMpojLFb0oPElBgfcqz
haoLw9LZHenuvb28gX7xO6e8noro9zM9pQ2miAyprG3qIVFVi6rqZA7QnQZ0
qyrEVk0u5n2OT1WOqRS+2el0lYq604euqt2GqqiQytJToV8kqzsqKgMAaVUV
YwBr89WXUaoqJlfQ/RX1+4MQtqKEVdmqri23rVYUodPyouxlM1RtY2940s9/
6/nW5iZ11FMhm3xdO3w7UZmKHDzLjKca5P1cJSpfg7pZpaqNqKV2VHl7Kjyq
CuxXf7oeVblzgD2pWNlt1eRi3ufEqCo29Wd2/TluSkXdQpFWleapbfqVU+9U
V27+q6tKvV+V1q+uqErLAWoVKy2wqqUB63UrW3hFFLDUNUVfy53/dUVS08g9
Ux4TZcEUkffz3tW3xX6/KE+1nvRrUp/iFafigyl2U7ojnuJ0UwSHVIxCFS+q
MkesN8oB+rrWrRWrMFu9ClXRBKnKk/orfnERe34Dtv0SnspVpV9LzUJVufnX
bFYnoiqyX30FoSpyckUtsDJ3WpHh1fTaqEA9HzjNYq4pRMylVaC0VcRQHhmV
e3utxSnvMArGfT3C56V3fvNU67OSWryLR0gw5fWUa/+UtzU93lM8VcV2qgdG
VQ3LVZzmigbbrNxpwCYX8z7Hr6qQ1J/7Nh+OBnXbtt9alWqXfmn19lSYt/+g
VaVurVpdTwCaOUAqsKr1V1BpwHWUrdbMqMKr1dP1IhEprJVqjWkqEX85lVTL
8unT0Z+ob5gKmERh2+TLiKc22PN+zKFJXfOUI5jq4rYpb9LP00dBxlPrnJ5y
pP4c2b8mhapmqiL61WPKVYzAyq2qJmnAJhfzPidAVcYJ95aoHOP+wlSlX2nN
uX+B7X/hqtLKVU/a+gCXUKrSbGXWrcpMoLLlqhRWGdHQTRemuaZ5lKT1SwQ1
Thjt6P7xs2FDaIOGUUTdfCrMU+ETaDlJvwYD/QLjqe2B8VR4v19M6i/UU2GN
f7qnQgpVHY6qIlrWQ1TFrFjJ12xUYNXkYt7neB4dcaq9XX/ekIqoUumXXGv2
z9NTYdyn3rwDSJiqzMDKWrHypQHJDnbNVhZhRSiptJLuI0pPtukTDklxbj4V
ftP58PpUbLMfb7Kf414evLt4LHe2ozuSfrHFqSZ96faWP5unyJCK8BSjUMVt
q6DvXO9W1TPMHCC7FbBuq+p17Q2smlzM+5xIVRnxVGyDehFS6dde+i6K5t2p
jBt/EKraHBRV7QpWlSuw8tmq7LJQbUUlA+1VJDU3WL7tVRK19GpU3VAuT3kl
FXJHj+Y3nwoLpkI8xWyiaFaZik76NfDUTqenolrTQ1N/7apK9VRMVBWkqthy
lR5YhWzQaLKstmpyMe9z4lTl7fqzNP4Rvwv3dirrPCVjoPpeRVV7C1U5alVm
ZwWtKqoPcKvZXKGlAdURFmZsNae0lavRrkrETTdCLaOw5Q2d6DDKYihiBoWZ
8QvooOjITRJjm/1sqnIbakl0B0XgDl9aUo5mv84EU9Z4KrKVIjykcmX/niSb
eEf+Cu5a91es6KjKlgZscjHvc4JVFdCdrqhK/816JykF7fx1DlQnoyozAWje
sF7vVw/IAWp7rNTAqhxhsaEKrIinOhFeUVnB0iylaGwaslmpNhYppGuiHEBR
pCtjxyXpeb95wTunmt1xvoqnWh+X1HjbVFsd6TvCm9KDVNXza37+cuhJVBWV
A7SVq9qoWPlaAV+EqpL2VKU/9156Romq1CrVXq1KVZVB/W3qVKHqGSqqMlVV
3gSkVJWjXKVPWKJUtcVIAxaxFfFKNNsCrflAxVbVPizNXOHLiJ6IYMrb4Nf6
tqnm8VS7wVSHPMWY6UdLqoOOaCf1pzWo+xoqeD0VTVRFbq2KK1exmivce6y8
E9fNwKrJxbzP8QyPKj1VVxUdEdPj/gK2/UYXqjRVGZ3q1gSgRVXazIpyd1WZ
AzS71rfQN7H6aDlD1cwEEo0Ws3UjaNGN1TLTjTcCljWGUhr86F50b9KPrEwF
7Jza2NLOKU881X1P5apqRyiBeb/oeRQ8T3lVFVilchaqQnoqPKryzVe3lasa
BVZkc0X4HiufrXiX8YEgKKoyZ/1FNP65t/06JlTYp6mbd/71q2ojoapd9V3A
pa22qarSAyv9MqKmAfWilWorU1iz9W3CeougFmqZ27IKAVn1ZLeSvwudbES3
BFOEqsxtU610UHRs55R921Qjy7RQnLLtnIqpT9F9FK16KrBHnQ6pjKjqWVsC
sC1VeTsrWuoDjB5e4dxm1eRi3udEquo5a+Of5il3j/oLT4WqqvSUTVXu/b/G
ZHW6X93eB0hcfGoVK3N+hVq30oRlxlZzaok1IsgitWVkCwN9FGqosN1S1jY/
cudU251+rLwfUzcRs5LcTRSMjvTOeWq9Z3pS85CKryrKU9EJQG4TYEQO8Llm
XethaUCoKsWvKjOe8uz5Ddj2SzRU+ApV1PQ/i6qcW4CtqpIvf/qvYj2wWmZR
ldpi4QivKGHVGtoNYVWJwRB5zTQ+R/vygGqUV1K0qmyVqajb+MZ1pMdJxz+G
YlkL9/KIG5fEaPaLaUrntabv3pC/ajrlqXhVPcdQ1R5PVNW0XBXQWeFVVVR/
RZOLeZ/jeXSkp4hxfzHDabWQKuQGVVqhSk8AGh2A5AxA80bARgdUrbOCqFiV
Vy1zKqDWu272BKpbrizVq1rHhcVZtYBrVt1H9uUtQtG5viaN6Iw2v8pTEfbp
whiK8CaK6I70Fj1FGoqV96tUZazw1F94SMXuqQhXVVBU1WDCEt9WUFUoHFW5
Un+tqkrvqdiRe0rd/1uWq9RRFeZk9dRWxIWu1gToaK4gtwM7bUXUrShb5bUe
I7yqtKVqxbdcQZM9gFpb+LHSU+CWXsc0PyqS2jA/VED+4lS7YyjaGJAe0ZG+
qoXNU/5hFM6RFNF5vxYb/6IKVW2p6ukWVOUoVzXJAYbY6lWoKlBVttkUlKqI
hgp771+Eqtyd6kRbhX4BdN0KxNayrk2uoHYEE7Yyuiy02MoVXqnOmmPEWVoo
ZAooXlKkm6KmT6SSCnKQLZJq514eYTecCg+mmt5l3jaJwjnTr3mz306Hp8yt
vqSqvJ5yTqb1bPt1t6lbVGXaSveUs1blvxVIQGdFlkFqvbmCKFdZilZNLuZ9
Ttuq8oZUpKrMNnVDVQEdgMSF0bq1SlOVZXeVK7Ait1ktzq+ltdjKG17Nq13/
SWetrecGCX/ZXOZI7jlLUet1Q/k15LrFvLMy1coYisCMX/jde8N3ToWM9Qsa
l+RooojN+wXWp4I9FT5GiehRj1KVVqhqUquK2gUcNgyQyASSOUBzO7Azqgps
XG9yMe9z2leVeztVNaSCCqlcUdV2I6oib6pouwmIRVXqjat21lvWtcDK3GPl
sNWmkNjKEl6pzrJ1X6gBl1bkon1kJveIYMpjojyMCr7PFPNGHow2v7Zn+nkk
FRlJtVicCp2bxI2ngkpUjCrVVrunYlXFSgDSbRX22wG7coCBfYDNb7mIqKpG
mKocnmq198/Z/kdcPOkEoG1gBdWvbs0BFjcFDgqsHLHVIj22Kve9qrayRVjW
OItcZpRExE0uGbkSfTENfn5JOcYlxd7Iw530W8JK+nW5g6LTm6fCQ6qYeCoi
+9dEVeH7fwtVmQMrbKpi9Ks3UlXUXmDKVk0u5n1OhKqI+ye2kv2jC1XEhbTM
/pFj1a1bgG0JQGcOUG+usO0ILotWZvu6UbfSkoEb59fucrVBaU7QU4LR5vL4
qFqOZomQFnRGj1/YQL8WbjgVf1fEFopTrKTf9tUtDKH1qCow9eftTjdTfwEh
VfjOX9vm39xWuyKjqqd8tnLfuCpQVeFDlhoHVk0u5n2OZyKHO6R6eV9ASBU6
ola/ouqFKmOmulVVtvuA2HZXBTZX2NKAS/U0oKPLopYMtBSw9DhrLh1qpc4K
85G3ha8NQ4Wn+zp+V8TO3WXeEVJRSb+gPgpG3q/teMra8ue900dwjzpTVc72
v7gZgLx+9SYt6x1oBeRdxgcCX1Tl8FR875/Spk5cXT09FRZVkVuAPaoiy1Vm
c4UtsCqvWstqsRWRBnQkA229FnSc5awftbhsU9BDDOWTVPQACkc7OnumnyOe
8hmqtJKuqpUdmETh2OQbq6pATwV2/TUMqQKyf+SmKjIBGH4XYMbAirjdVQ3n
AQY3VzS5mPc57anKlv2z30vR0VPxrENV3mZ1+2wlTw6Qmgdo2upJ0lZGbKXZ
is4ELtTDq430zqNKHOudVa3mbgofhF519/m29Fozfu3umXJ4yt3mF+Op1pJ+
PE+1kvcLuReVtzu99QkVpqdY+6pcqorZWhWdA+zQ5Iq6rZpczPucIFXFh1TE
xTaiUBXYqW6Mq91ri6qochVxR2CHrSwVK0fRymsry0wGo0twviUFN08XDaGw
efV/qWX75g49BXVNFC4OMVTlKfbGXl6bn8NTK5Q3GJ1+bkkxmig6U59idKfb
uv5aCanC29TptgqtsyJAVZyNwJ0sV7kDqyYX8z6nFVUR19vQIRWqquztf6Gq
styznpivHpYDtAVWflvpXRa0ldwFrI3z6eZ2U1ikwhw6o79wvktSG7XlH44U
eZ8pn6SsqnJXpjo0g8IdT7XhqUBVuT3ln57UJKTy7qVqqKqd8araU1shCcB2
mgCDAytXDtBmKyQAJTxV6ZdcMvtnbVN3T6klO9UdY9WjmgDtgZWuKrNiZdsR
bBStjBuFfLQaVafWrWoRluWyb9rKNJc97Apd3ugpuPnck/GL753gB1NttPlx
Zs82Lk6FSqpznjI3UrUdUnEKVcGqss4AjGmrcKvKnQNsOGTJYasmF/M+x6+q
wCpVRJs6deMPuqci5A4gAaoKKVftWmsNrHaYrYA1YRFi2lIfu2Q2sbuEtVCv
YW1Upj2483Ieo80P1VPE3qiQglTrjehdaUdv914ero70yE2+3n2+QfWp8Hgq
IKRqIftnj6pCOwD3BDSre2+wuLfuKZaq/DlAbnNFk4t5n+N5dOZeKk9DRbCq
3IUqV1Rlb6sIagLcaClX1QMres4Sfetwun2d2B1sq16RTReksBbQTmm+QuOm
KEm55050aJqfU1K9GUPhbkqPlJQrniKDqaiBtCHxFKNBPVBV22lPRY2qcM0A
DCxXdbizIsRWUJUBR1VmQ4U3pPKqquWoytlZ4c8B5oEVYSV30UprCAyxlU1Y
WsXHlhus3ihv/m64zDSaaj2GmGJjqA41+LUcTLk7/dwZP97OqTUcTzXvo+C1
UjBSf3Gqisz+Re3/tXqKqyq6VhXWWdGkuaLJxXzk4dgLZr4/TFWMkEp7PoQX
qihV0bWqmF3Ats4KRVWEmGz9FUG2orosiM7AdNnCq5A4y7piQ6SOGSpQUh5P
hWzstTVRBOzqZXb6tXEP3xaGUUTVp+ypv3ZU9SSlKnehymxTb64q77jaJuUq
csR6q4EVuR3Yd+WPEUWPKY+WtFILqnKFVI6oKlJV4W0VDlU5y1WEmEL3WKmB
FTUe0NoTaOtjVyIsWw2LmMvU6dWioRqWpdyGajgdfblzz5Sj08+d7gselxTe
lN5Ov5+zlWKPW1KUp9rJ/tlDqlhVRdwEpAipmAnAhi3rrMAq7OLfF3RAVaqn
wub+NY+qglTljqpqG4EJMXkqVqWq6KKVHliZsZU1vPLlA4msoF0TmVasDYQ2
B5n/Usulp5BEX8DQic2OLvRmU5K2Fn8z8HvRue3ono70dutThaqa1KfcJSpP
g7p326/pqU70/jHuV9VgYIW3X91/+yqHquy2Crv49wU2VWX/tfnLOkLKM0c9
ZO6fo1AVXKui2yqsnRWEmAJb1ulWQLJxnSxakZutzNKVI7xaZImwnM5S5bJR
8ReRP7Qvv5halFTHalLhZanOdVC4JNWZpnTmKFpH3i+gOz2o66+uqiBPNZj+
50kAWmzlTwCG5wCDp6y77gli76+gLviuC/iIwTxIh6rIhxMZVbWU/WtUq9JV
RYiplgAkc4DWipVlj5Vlm5XXVmZsRQqLTgmSZSxfnBW4IuImX5YvfCevp7Wv
O56K3zMVF0+FeSp0EgVjDq3NU+Etf41VRQ6njVBVg+xf3GT1wLaKwJvXR6rK
Y6shjaps7/SpKnyYEkNVZEjl6QAkxORJAKqqCugDdO8IDipardAvj4G2cgjL
GmctqtukscJCKlB0ESrYU/xhs+pp5AVTy6PbJ9opTnUi6Rc1OqnJbt/wxr+t
rtQfQ1XPqatZocrRrO6Iql6M3VoVY6uQitXgqioxalK2t23vqeGtUvEKVcGq
IsQU1a+udVYEbrCiGtdDbfVksK0cwnL3XZTO8vjLYa5Fiox8VjIltUkNoBr2
9S2pP3zWbeW3BkrKnffrxJ6pJpP9GP1+Ua3p4S1/pKfiuylCb1DFU9VuV/bP
3QTIVJWtX53srIgdW2EpV/mu/NG26B8ioqoWVWUpVBFiIhOAjvuA+AMrSwJQ
zwHWbbUj0FYhgRUlrK2BtgrpwXAqJnZtVt6Ii6FCEn1Gro9O97l3SzU0VPN4
KlhSEcUpXt4vvES1yV+iittI5atSBW37DS5UxU5Vcs9WanO+eqyqIvsAm1zM
+5wYVTUvVNVURYjJ1QHo3gWsvmQCc4Du5gqbqoIzgdsMW+l1q+KNLe6UoNko
qPVgUP0YmsVMo5GfSaf1QjJ7RhgVnuXjN/iFlKXcG3sb7u2N6aCI2OEb25Te
VjzVpETFa/wjp6kzeioCVRW/r6q1JkDG7qpCWFBVSnAC0PxN2UIqWlWEmBwJ
wOfqhapnbVGVN7By7K4yVLV7vX5lIEYt2VRlmxBoRFjEVdcy2kK1VUSoZc8Z
8rwTuloMo3qU7gvf2OuXlKODgtfs19LcpLhBf01UFZD666aqIkIq8z4grZSr
GC3rUFVFTK0qMPsnVUWIiVOrClaVv1xFDQOMaK6gNgWbtrL2BK7Qwysiwqpf
uh01LL+wurNCxNS2p6IMxe9Fb+nGiK3t8GUUp5yqCoqn2t3zawmpHG3qTdv/
zLHq8bes4sxXb7tlHapK4UZVNVXRN1L0JQB5HYD+gRVmucraXKG9xn2tgH5b
rQztCdRtFRxhkasnqmpTT4HpvhhJxbb5hXf6NYqnGngqpDjlbU2PKFG1tJcq
LvtneiowqmLvq3I2qzNVFVyuim2uaHIx73N4qqJvpEhu/g3ZVLXfdrMqyw3r
IwZWePsAycCK3GMVlgY0Ayu9dOUYaqHYqryq051yTme1Ka/AtB67GhWyTyok
kirObWzjRETSL6bNL7o4xahPMeIpX32qE1Uql6rMnoqA3r+gngqy/Y9SlSP7
x2+rsKmq8fTaJhfzPidIVWShymyoiFPV7rio6tmogRW+lnV/K2CMrXZo+4K1
0pVqKzO8Wl5dk2uB1TL9eh4VZ21RBhZVbyjbmlSjcUzkcJPhKbXRMWp8X2jG
z9vj13hjb9OOdHdTeuObevj7KDoZT/F61IOGVLSrqiZtFawmwNCNwMF7gZtc
zPscz0SOpj0VVAJwf0ACMKJc9SSRBjQDKz2qcgZWoUWremy1Q7msqZdBYlSg
JR9oi7B0Zy2J1lbLy7ITyp/iC++aiEz3RfX4MbZNhe/tjSpOufJ+7XrKV5/y
x1M9URXVpv4cq/3PMa62SWDVvLMiamwF7zI+EHQ8qtKfMFxV6c9zb7nKfUMQ
bXKFd5vV2sjYyle6IlrZ3QUsYysWsbrmKd/i1KHCDaV4altkg19sxq83knJ7
KrYvPaDlj+mpBtt+9+1gzlMSnuKpine/quaqaqFc9TyiqiREVZ6QKj6qCmmr
IFSllqu2WaIq++4qdw6w1l9hCax2cVW1XbtUusMrW0O7PciqFbY0bS1V/m1F
TEtqMvIHUAGSilZVVxr8/MFUk06/ME/F7Z9q0EfBSf01rFI5GyrcqmqtU70N
VXkTgC3sBVYqVk0u5n1OQFTl6v2LmVMRU6uiVeXtV3c2AYZOr7X0V+yiAiu5
jCtYSGzltZVLWAHaClphIVJoWi8+yxeR61uunCtSUiuDg6lWylLc7b1+T0WG
VJ5+v7BWCs5GqvDUH3vnb/iOKraqbAlAc2uVJaSyjleKnQQIVXlgRlXklFq2
qvQnZFStyrK1Kqhl3bsj2LEpOLB93RZbFdqyzQwMSgyqnQlauwIVbTGN1kb0
xKxGGem+pg1+7Y1Gj93e24N4ylGiIndRUZLqfkNF9Dwl0lOkquK3ANfaKsLn
q0dGVeFNgE0u5n1Oh1VF3amKiKoMVdEdgLZ+9ZDAaovyZ6Rleq1ZsdIDK8tO
q511VWW31SMvg2QmMEhYDlsFB1xNV4yMWqhGLXeGUVGVqbBcX9SeqZZ3TsV7
KiSe6mrqL3jonzukakdVgc3qnVNVZLkqpLMCUZUD3+bf6J6K6FpVyNYqy3gl
M7AybbUnJA0YEFiZsdUOy1XRZit9+9XyBrlBr7x4OmMZamuUm6Ja+wLLUmEZ
v6iyVJtjKLybfG3xlKOJInYLlT2kYndTxKmKvfM3YPMvJwfY3i7gFu9dBVU5
IG/7aytUBSYAzQ5AVwLQXq6iVRUYWG1WVOVMA5KBFRlb6WnA1fUroaN6Re4X
DkkMMpxl81eT7+MOo7iS4s1Cj/aUV1K9SvpZOtIDR/w1j6eadFO4s39BIVXg
Tep9I5Wa76vyqCrAUy3sroKqvKqKbf/b64+qPCEVT1WxOcAQVdk3BZMNgbW2
QPWuwead7ldZS1e0sLxX+GX5v2os053F8ZHPUC43WTwV3d2n9KI3avNjb+/t
2P4pazzVVuqv2XgKdkjFLlS5xqqTqoqMqtosV/ls1eRi3udEquoZZ6Eqqq2C
EVUZW6sYzRV0DtDXX2EOXbc1BGq3C9nusJWvm10Pr0KctbwKZzquJ3bo5DQU
f3ZflKdYSb+4Nr8wT4Un/faE1KeitlDFhlS2VoqYOer8NvVuqqr5cHVGAjCg
D7DJxbzP4aiq4aaqLqtqr/e+wA0Cq8DYytoc6OgPtEVY4c6yKMy0mOND+mc2
jJ7qhvLXoXwtE9vic31Bjejd8ZRDVYxgKjjv17CVonNVKn9UFXhHxXBVNW+r
aGlrVeBG4CYX8z6HkwAMj6pcbRW8BOCO+KjKeU8Qumtds5WjbmWxVaiqzPlL
3qYLZUoD01k9WVGdEgGqYodRHW2fCNo2xU769baVIqzrr0mPOjuk6qiq+GMA
IycBhgwDbHIx73M8w6O6o6rADsCQ3VWue1fZbRWoKrIhULcVGV5pwrK0sgeG
V4Szem4ip5viDOULoBie2hHfhc6cQRHjqTbrU7F5P8cWqsaqisj+RYVUDdv/
mqvKVqty2+rZmH51pqowAzCiVkVk/2JUtd+4Dwi5C9hMA3pV5doO7M4EBgZW
Zt3KDK9MVUUlA33aai1JOOI91X4YFZLuC0/6NYmnmtSnWlKV11PM+1I1HKbk
G/3nn/7XSVW1lQAMyQE2uZj3OTGqii1UubcAuwMrx0Zgo2IVV65qpqqgNGBA
YOXPCnrNFWirdrVlOIhppZVBemIaqpmnOiqp8DCqUTAVX58K9JTjlvTexr9G
qmrQU+FWlbunoq0OwLY6K5pczPucaFW5sn9hCUBysjpTVVS5ymxZJ5srzBaL
PU1aLLTNVgGxlavXwpsVDIiwaHk1ibkaeqqJniKDqaDGCUpVsZ4KUhUr4+ev
T22y91HwWv6iNvxGdVOED1OKSgCGFapGoqq4gVWTi3mf04eq8nVWBPYBhncD
8m3l67LI16pqWbOCqzyX9yh59WZ5j79hJMXVE19S3kjKm/FzT/bj7fN1e4rb
TRHU9dc8+9ehnoooVdl6KjqfA0RUZaeZqp4xVLXX7qlOqMqcsBQ4ZGmL1Vbm
YEBbJpDusjBjq7YjLEZWcAT6iBYTI9GnpPtak1RI3i+mfSK6KZ03LD3EU/F3
om80myJdxHiKmJCK9lRLI5Ui7gI8AspVTS7mfU7bqupyVGUfW9FIVe7Ayr01
2GYrx7TAWFWJCCukB6P7CmsuKVZBimco/pSk5u0THWpK70481YqqmodU3VeV
PQHYpfFKUJUdl6r2GgnA4A5Af7+6XVWezgpnxcq0FTlunQisNhpXFV8mkGgL
JNstojounHFWqLZ6tNqJnmINZZEUI90Xumcqvs0vIunXtqesknJ4iuz6i9pL
1f3sXwNVNWpWjy9XuXOATS7mfc5gPzoAABgSBvtiznt0g31OeghObCfAWe0Q
OLEdApdlE5yTEQVObCfAWe0QOLEdApdlE95EjlbOCb7JiP0mI+Qw8E3wTfBN
Aj9/2AcrtfpV+Cb98k1GyGHgm+Cb4Jt0+UePWA4AAADQ//RaJgAAAAAAAAAA
AAAAAAAAAAAAAAAAoA9wtJqgCyUW2xnDmWwInqUdAs/YToCna+uUJ808ezif
sdhOpuMkgxDwLO0ojrOK08vG9gdA949kMHBfBHBio4CqOgSepR0FquoEiFXb
xfucxIkNB6rqEHiWdhSoqhM4Th3OqhdzHzQuAi0CVXUIPEs7ClTVCaCqdsFF
oEWgqg6BZ2lHgao6AVTVOlr6NHtbC75AIOTJNN8PYsGztEM40iw4sWzIs4qn
KwAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAMBQUc6mxpBqAAAAI5ZSVb0+EAAAAMAKPAUAAGAkg6gKAADA
SAa1KgAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAJsDUnp9FAAAAABNKSnYCgAAwAgHqgIAADDC0VR1
AAAAgG7Rqyv/iIU8OeaJwqkDAIDugOttCNlZOsCIqnp0OAAAMFzgeuvFFoHi
1AEAQHfA9ZYNTh0AAHQHXG/Z4NQBAEB3wPWWDU4dAAB0B1xv2eDUAQBAd8D1
lg1OHQAAdAdcb9ng1AEAQHfA9ZYNTh0AAHQHXG/Z4NQBAEB3wPWWDU4dAAB0
B1xv2eDUAQBAd8D1lg1OHQAAdAdcb9ng1AEAQHfA9ZYNTh0AAHQHXG/Z4NQB
AEB3wPWWDU4dAAB0B1xv2eDUAQBAd8D1lg1OHQAAdAdcb9ng1AEAQHfA9ZZN
a6fu4/gVAACAC6iKTZuqUhcAAIA6UBWbdk6d5ikICwAADKAqNp1VFWwFAAAF
UBWbFk6dw1OwFQAAFEBVbLqhKtgKAACgqgY0PXUhnoKtAAAAqmpA91QFWwEA
hhuoik1XVQVbAQCGGKiKTbdVBWEBAIYVqIpNo1Pn0BBsBQAAdaAqNp1Sle2j
sBUAYFiBqth0UFW2T4CtAABDCVTFprOqcnwahAUAGDKgKjbdUJXtMyEsAMAw
AVWx6ZKqbJ8MWwEAhgaoik33VGX7fAgLADAcQFVsuqoqx1dBWACAQQeqYsM/
dU38EmgrCAsAMEBAVWx6oyrbl8NZAIDBBapi0zNVOb4JhAUAGESgKjY9VpXj
W8FZAIDBAqpiMyJU5fiGEBYAYFCAqtiMIFXZviecBQAYCKAqNiNLVY7vDG0B
APocqIrNSFSV+0fAWQCA/gSqYjOiVeX4QXAWAKDfgKrY9IGq3D8R2gIA9AlQ
FZt+UpXj58JZAIARD1TFpv9U5T4AaAsAMFKBqtj0sarcRwJtAQBGGFAVm0FQ
lfeo4CwAwAgAqmIzaKoqQagFABhhQFVsBlZVJe06a2Q+RgBAPwBVsRl8ValA
WwCA3gFVsRkuVanAWQCA7gJVsRleVZUg1AIAdAWoig1UVQPOAgB0DKiKDVRl
BdoCALQKVMUGqvIDZwEA2gCqYgNVxQFtAQC4QFVsoCo+cBYAIAaoik2bqhpa
4CwAQABQFRvmqfvEP8nXx5UFEtyjBABgBapiw1XVPyUWUEGoBQCoA1WxYZ66
T/4UsYANOAsAAFU1gHnqPvXTxAJe4CwAhhioig3z1B34M8QCUcBZAAwZUBUb
rqp+lliAB5wFwHAAVbFhnrpP/xyxQEPgLAAGGqiKDfPUfeafEQu0BYQFwCAC
VbFhnrrP/gKxQCeAswAYFKAqNsxT97lfJBboKHAWAH0OVMWGq6pfIhboDnAW
AP0JVMWGeeo+/8+JBbqMV0kQFgAjCaiKDfPUfeH/IhboFQiyAOgHoCo2zFP3
xV8mFugtEBYAIxuoig3z1B30K/r64q+0fWiAC5wFwIgEqmLDVdWvGutftH1o
oDEQFgAjCaiKDfPUfenXiAVGLBAWACMAqIoN89R9+V8SC4xwEGQB0FOgKjbM
U/eVf0Us0C9AWAD0AqiKDVdV/5pYoO+AsADoIlAVG+ap++q/IRboUyAsALoC
VMWGeeoO/nVigb4GwgKgw0BVIRyQYr6T872+9n8TCwwGUbb6xD/p9eEC0DdA
VYG0pqqv/ztl/dt8gUEi0FOf+KfJJ38q+dRP9/pwAegDoKpA2lPVbxALDB5e
VX2yUNWBP4s7QQPgBqoKhFRVScQ3OuQ3iQUGFW9IdeDP5Kr6zM/jftAA2ICq
TEgHtRZVHfpbxAKDjSuk+hnpqU+nnvrsL6S32sT9ywDQgaoCaU1V3/j3xALD
gDekylT1+V+StzDDrWEAUICqQiATfcxT983/UFvfSBcYHvSQ6mf1kCrzlLw7
zK/IUcZf+tVeHzEAvQeqYsNV1X8kFhg2vCGVUNVBhaq+/GsYFAmGHKiKDfPU
HfafiAWGE1tI9QUlpPpS6ik5K/JfY7AJGFqgKjbMU/et307Xf64tMMyQIdUX
6yGVUNVXU1Ud/OvptnHsxQPDBVTFhnnqvv07xALADKkOMkIqOYYr9ZTcP/4b
2OYAhgeoig1XVR8hFgAlX7Rn/8qQSqjqkFRVh/5W2kSKzhww4EBVbJin7vDf
JRYAGo7s39cLVR1aqEp2k/5HWfREMhkMKFAVG+apO+L/IRYAJLbsXxVSFao6
rFDVt38bKWUweEBVbJin7ju/RywA3Diyf2pIlanq8N9JDv9IcsTvJt/BX0Fg
QICq2DBP3Xf/C7EACCFvqDCyf1pIlXlKROtCVd/9PfkEO/L3e33oADQCqmLD
VdXvEwuAKNSGCk1Vakj1ndRT4gl25B8kR/2/ch39h70+dAA4QFVsmKdOXDTM
BQADPfuXhVS/kxxRV9WRpar+MDn6vybH/LfkmD9KvvdHvT56ACKAqtgwT524
XJgLgCbQ2b/fq7J/maqOLlQlPPW9/y859o+T7/9Jrw8dgCCgKjbMUyeuFeYC
oBW82b9jDFV9/78nx/1pcvyf9frQAXABVbFhnrpj/ohYALQImf07Ws3+CU8V
qjquUNUJf56c8D+SE/+i10cPAAFUxYZ56o79E2IB0AkyVR1lz/7lIVWhqhNT
VZ30l8nJ/zM5+a96ffQAVEBVbJinTlwTjjMWAB1FNlQ4s3/HFyHVSYWqTvmr
5JT/lZz60eQHf93rowcAquLDPHXiamAuALrDMYWnbNk/4amT0pBKqOrUQlU/
+JvktP+dnP6xXh89GF6gKjbMUydTK8YCoJuEZP+ykOrUv5aqOq1Q1en/Jznj
b5Mf/m2vHwAYOqAqNsxTJ1/+xgKgJ+ghFZX9y0KqTFVnZKr6u+SHf5+c+ffJ
Wf/Q6wcAhgWoig3z1GWvem0B0FtOMEMqI/unqkp46sx/kKo66x+Tsz+enPOJ
Xj8AMOBAVWyYp0680s0FwAghJPtXqSr1VKaqcz6ZnPup5LwDe/0AwGACVbFh
njrxAjcXACONkOxfrqpPSFWdW6jqvE8n538m+dFnev0AwEABVbFhnjrxujYX
ACOWkOxfFlJlqjo/U9Vnkx99Lrngc8mFn+/1AwCDAFTFhnnqzj2QWACMfEKy
f6qqhKcu+LxU1YVfSC76YnLxQcnFX+r1YwD9ClTFhnnqxKvYXAD0ESHZv0pV
qadKVV3y5eTSr8gFQAxQFRvmqRMvXnMB0I+4sn+fzbN/uaoOkqq6pFTVV5PL
vppcfnBy+dd6/RhAfwBVsWGeOvGaNRcAfU1g9k94KlOV8NRlqafEuuLryY8P
SX58aK8fAxjRQFVsmKfukq8QC4DBIDD7V6pKeKpU1ZXfSK78ZnLVYb1+DGAk
AlWxYZ66y75GLAAGjMDsn1RV5qlUVVelqrr6W8nV306uObzXjwGMIKAqNsxT
d8WhxAJgUKlCqi+5sn8ypFJUdU2qqmuPSK79TnLdd3v9GEDvgarYME/dlYcR
C4BhICT7d3VdVdelqrr+yOT6o5Ibjur1AwA9A6piwzx1Vx+erm/XFgBDRUj2
79q6qoSnbjg6ufEYuW76Xq8fAOg2UBUb5qm79jvEAmA48Wf/vpNcr6nqe1JV
Nx+b3Pz95JbjkluO7/VjAN0AqmLDPHXXH0UsAIYcT/ZPUZXw1E2ppzJV3Xp8
cusJyW1indjrxwA6CFTFhnnqxN+E5gIAZJDZv+vr2b+bypCqVNWJyW0nJbef
lNxxcnLHKb1+DKB9oCo2zFN383HEAgBoBGb/hKqykEp46vaTc1XdeWpy1w+S
u07r9WMArQFVsWGeultPJBYAwAaR/TuWyP7lqhKeUlR19+nJ3Wck9/yw148B
NAWqYsM8dbefQiwAgJfA7J/w1J0/qFR1T6qqe89M7j0rGXVWrx8DYAJVsWGe
ujtPk68jbQEAwlGzf7dS2b87y5BKUZXw1Kizk/vOkev+c3v9GEAcUBUb5qm7
+wxiAQB4hGT/7tVUda5U1QPnJQ+cnzz4o14/ABAEVMWGeeruPYtYAICGhGT/
MlUJT92feipT1egLktEXJmMu6vUDAC6gKjbMUyf+ojMXAKAtQrJ/9xch1YMX
VKoae3Ey9pLkoUt7/QAAAVTFhnnqHvgRsQAArROS/Xuw8FSmqodSVY27LHn4
8uThK3r9AEAFVMWGeepGX0QsAEDnCMn+5SFVXVXjf5w8cqVcoNcMtqoOSEk6
8zCZ33PspcQCneSjf3Vz361en7MBJST7JzyVqWp8qaqrkglXJxPFuqbXD2B4
GWBVlQ+tFFaHvn8c464w1uVtH9qwMMCXfdits4Rk/0pVTchUdU3y6LXJo9cl
j10vF+gug6oq9XF16DEyv+34K4kFLODiHA6MxiEk+ydUNbGuqkk3JJNuTCbf
lDx+U68fwLAwDKoaWT9iwjXEGnpwXe0acBmNJ/t3Ta6qxwpVTc5UdXPy+C3J
lFuTqbf1+gEMOEOlqnZLV8xvkmUPtDVM4PI4whl2kYVk/1RVTSlUNfX2ZNod
vT76gWUYVKW93VbpivlNxNPbXIPLMF7rBpohspg/+3dzTVXTUlU9cWcy/a5k
+t29PvpBY1BVlVis1PuoSjyrzTUoDP7lC9gZWIWFZP8yVT2hqGrGPcnMe5OZ
o3p99APCAKvKpN3Gdeb3EX93matvGbSLEugAg6MwR/avDKkyVc1QVDXrvmT2
/cnsB3p99P3NsKmqxcZ15veRf3EZq6/o76sNGDH0scK82T/hqUxVswpVzXkg
mfNgMnd0MndMr4++Lxk2VfX+u8k/tIw1sum/KwnoZ/rJX1VDhSX7JzylqWre
mGTe2GT+Q70+9D5jeFTV+kZg5neTqQBjjTz64CoBhomR7i8y+zdLyf5JTxWq
mp+qasG4ZOHDycLxvT70/mB4VNU6zFMnMwDGGjGMxIsAABZGnL+qkMqe/ctC
qvnjclUtGp8seiRZPKGXh90PQFVsmKdOPkuN1VNGysscgDYYEfKqGios2T/h
qQVpSCU8tWhCsnhisuRRuZY+1oOj7QegKjbMUyeenObqBdATGBJ6Ka8q+zea
zv5lIVWmKuGppZOSZZOT5Y8ny6d07yD7AaiKDfPUieekuboFAigAkp7Iiw6p
ClUtKVS1rFDViinJiqnJymkdP7A+Aapiwzx14q+maj2Wrw4DPQHgpkvymhcS
UhWqWpmqatUTyerpyZoZHTme/gGqYsM8dTK0N1bHgKEA4NFZeZGqMkMq4alV
05PVM5I1M5O1s5K1s9s8hr4CqmLDPHXi6WeutkEYBUDrdMRctuyfGlKVqlo3
O1k3J1k/t50f3VdAVWyYp04868zVEjAUAF2jTXNpDRVmSLVmVq6q9amqNsxL
Ns5PNi1o6aH0AVAVG+apEyG8uRoDQwHQW9pJGGrZPzKkEp7aMD/ZuCDZtDDZ
vCjZvLjthzISgarYME/d+nnE4oIwCoARC99cK6cmq5SGilJVakhVqmrL4mTL
kmTrso49jhEBVMWGeeo2LiRWPDAUAP0Fx1xqQ4UtpJKeWpo8uSx5cnmybUWy
fWWHH0dvgKrYME/d5iXEigGSAmAAiDCXlv0jQqqluae2rUy2r0p2rE52runW
4+gSUBUb5qkTcbq5woCkABhUgrS1vgypxFJCqq1KSLWjUNWudcnu9V18BJ0F
qmLDPHXizx5z+YCkABgePAEXEVIt00OqnWtzVe3ZkOzZ2IsH0TJQFRveqfvg
yZXmcn8JJAXAMEObiwypSlXtWpvsFqpKPfXUpuTpzcnTW3r6IJoCVbHhnbr3
t64yl+2TEUwBADR0bcmQarkrpMpUtXdL8syTyTPben34TKAqNrxT9+7WVeYy
Pw2SAgB40QOuMqTKVaWEVHu3SlXt25Y8uyN5bmevDzwaqIoN79S9vWWVubTP
gaQAAAxq2qqFVIWn9m3PVbV/V/L8nl4fbwRQFRveqXtr8ypzqZ8ATwEAGpGG
VDVtaSGV8NT+3VJVLzyVvPh0rw83CKiKDe/UvbFplbmyDyHpBwBokyKkqmmr
DKkyT724N3npmeTlfb0+Vg9QFRveqXtt42pzJQimAAAdoh5S6eWtzFMvP5v8
5Lnklf29PlYrUBUb3ql7deNqc8FTAIAOYlapipCqpq1Xnk9efaHXx0oDVbHh
nbqfbFitLXgKANAN9CrVXi2k6vhNkBsAVbHhnbqX169Wl3hKiH9bPzYAAKAx
q1RZ6i8LqV57MXntpQ9ff3mkaQuqYsM7dS+uW6Mu8TQQ/7Z+bAAA4MIIqUpP
Ja+//OEbP/nwjVc+fPPVD998PRkZNxuCqtjwTt3za9eUS/zqszdaPzYAAPBT
hlSvFiGV8NTrpade+/Ct1z98+43y03sYakFVbHinbv/aNeUSv+7sjdaPDQAA
Qqmn/tKQqvDUW298+PabH77z1ofvvKN9UZe1BVWFcECK+U7Gt3p2zZpsid9v
+XYbxwgAAA3IUn+1kCr11NvCU29/+O47H7z77gfvvU9+aRe0BVV5KU+Rdq54
p+6Z1WuzJX6n5dstHCUAADRDT/1JVaWeeifz1HtCVR+8/4H7m3RIW1CVl3ZV
tXfV2myJ32P5dgtHCQAAbWCk/kRIVXnqgw8+/MAjq4oWnQVVeXGoqgni19fw
OwAAQF/QyuWuF5f/fuKAVqOqPSvXZkv87navXJutFo4SADA0aMOR+mU1echQ
lZd2VbVrxbpyid9d9kYLRwkA6DA9v9R3s+MuI+2mKFJ/73/4/gfJe+99+O67
snj11tsfvPnW+2+8STdatA5UFQIZfvJO3Y7l68olnnLZG20cIwDDy4AJYqTx
wftCVbJE9d77uqdef+O9V19/rwvHAFWx4Z267cvWqUs8/8W/rR8bACMZSKQf
KT31dt1Tr7z27k9e1XddtQ5UxYZ36p5ctk5beKGBkQ/MAgSap17LPfXuy6+8
8+LLb3f0R0NVbHinbsvS9ebC6xd0CMgFtAvpqZdefvuFl95+/sW3OvdzoSo2
vFO3ecl6c2UfwjUBZCB+ASMZ3VM/eeeFl6Wn9r/w1rPPv9mhHwpVseGduo2L
15tL/QRcZPoX+AUMCa9Snnru+Tf37X/zmefe8H99PFAVG96p27BofbbWK4v8
TFy7ugwUA0A4madeFJ566W3VU3uffePpfa+3/uOgKja8U7du0QZzeb8Kl8RA
IBoAukbuqRd1T+155vXde19r92dBVWx4p27Nwg3miv0mA3ylRVwDQB+Reuqt
Zw1P7Xr6tZ1PvdriD4Kq2PBO3eoFG8zVyvF07QrfVkUGrgFgAFA99ZTiqR17
5Grrp0BVbHinbuX8DeZq/dhigU0AAGxsnipX8x8BVbHhnbrl8zeaq/VjAwCA
btJRTyVQVQN4p27ZvI3mav3YAACgy3TOUwlU1QDeqVsyd6O5Wj82AADoCZ3w
VAJVNYB36hbP2SjWovpq/dgAAKBXtO6pBKpqAO/ULZy9sVibytX6sQEwEli6
ej9/rWlnLVvzfA/WWv9aPiJXdYTy1OXLdYZX7+/OcwmqYsM7dfNnbzJX68cG
BoBFK5/L1uJV+91rScha3XQ18g5UNbiqcq+2Xg5QFRveqZs3a5O5Wj82MNJY
sPzZBSueXUiuQkm2BVVBVVAVVMWGd+rmzNxkrtaPbTiZt3RfB9eyZ+dHLqkn
dUFVUBVUxQWqYsM7dbNmbDJX68c2Qpiz+BlyzV2yj1hLo1dn3dRYVfNDVfUs
VDWEqhqZtoKqBg/eqZs5Y/PM6fpq/di6w8yFe8Wala1Fz4g1u76gKi2wgqqg
qr5TlecMQ1UjHt6pmz59s7laP7Z2mT7/6WzNWJCtvTMX7M08BVVF5QChKqiq
P1S1BqoaHHin7oknNk8zVuvH1pBpc58S64l5Yj0tlqmqGZ1QVbytBkRVKwZB
Vb1+zoIeA1WNWHinbuq0zeZq/dhieXz2nilizXlq6pynMk8VqnqqiapmQ1Vc
VfX6GQHAyAKqYsM7dY9P3WKu1o8tkEkz90yeJVeoquZDVRxV9er3C8DAAFWx
4Z26yVO3mKv1Y3Pz6PTdj83Y/djM3aSqptpVVStXjQxVdfnUAQB6AlTFhnfq
HpuyxVytHxvJhGm7Jj4h16PTd0lVzRCq2l2qKrPVVCWwqlRF5QBDVNWdxwUA
GHigKja8Uzfx8S3mav3YNMZP3fnI1J0Tpu0sVZUFVpOKwOrxWZSqnJ0VQlKd
PmwAAMiAqtjwTt2EyVvN1fqxlYx7fMfDU3aMnyJV9ci0nVlgVarKlgOsl6uk
qjp3hAAA4AWqYsM7deMnbTVX68cmGDtp+0OTd5SqKgIrVw6wVFUnjgcAANhA
VWx4p+7hSVvHGavdAxv96LYxj21LVbVdqCq3VZED1AKrMgfY7jEAAECLQFVs
eKfuoce2mqutQ3pgwpMPTnxy9MRtqa22E4GVoaq2fjQAAHQOqIoN79SNefRJ
c7VyPPeN33r/I0/mtlIDq9xWO6vA6oldrfxEAADoDlAVG96pGy2jHn01P5h7
H94ySqpqq1BVais1sNquBlbNfxYAAHQZqIoN79Q9MGGruRoeyd0Pbb533JZR
D2+5r7CVGlhlacCGPwIAAHoIVMWGd+ruG7/FXOxjuHPMprvGbLp77OZ7araq
BVbsbw4AACMEqIoN79TdO26zuXgHcPuDG+8YvUnaauwmEVjdI75VmgbMbJUF
VrzvDAAAIwqoig3v1N390KZ8ja0W4/vcev+G2x7YqNhqc2qrLbmtHungtmIA
AOgyUBUb3qm7c8xGc8V+k5vuW3/zfRtsthr1cM9GtQMAQCeAqtjwTt3tD24w
V+w3uXHU+sxWtxi2YhwSAACMcKAqNrxTd+v9680V9R2uv2fdDfeuk7YapduK
cTwAADDygarY8E7dzaPWmSv8y6+9a+11d68lbcU4GAAA6AugKja8U3fjvWur
dU++Ar/26jvWXHPnGtJWjCMBAIB+Aapiwzt119+9xlwhX3jlbauuvH31VXes
vvrONdfctTYVVmqre+PyhwAA0HdAVWx4p+7aO1ebK+QLr7h11Y9LWynhFeMY
AACgv4Cq2PBO3dW3rzKX96suu3nF5beslLa6ddWVt62+6vbKVoxjAACA/gKq
YsM7dTKPd2u2VpbL/SWX3Lji0pvkuuzmlbmwivCKdeAAANBnQFVseKfuCuGa
fK0ol/tLLr5h+SU3Ls+EpdqKddQAANB/QFVseKfuMqGbai3PluPzL7hu2YXX
Lbvo+uWlsGQy8GZPIAYAAIMEVMWGfeouEdLJ17JsOT75R9csveDaZVJY11fC
EuEV70cDAEA/AlWxYZ+6i0SUlK+l5SI/89yrlpx39ZLzr1maCevCQlgNjhoA
APoPqIoN+9RdcO3SC64p15JskZ95zpWLz71yiSmsBkcNAAD9B1TFhn3qzhfe
qdZiua5abH7aWZcvOuuKxWf/eHEmrPMKYTU7agAA6D+gKjbsU3euVE+5FmXL
/LQzL190prSVIqyrpLCaHTUAAPQfUBUb9qk7+4pF9bVQLO1zTr90wRmXLvzh
ZWLlwjo7FVbjowYAgP4DqmLDPnVnXrawvhaIpX3OaRcvOO2SBadfIoWVOSsT
VuOjBgCA/gOqYsM+dWfIiCldl2Rrvlja55x60fwfiHXxfOEsIawsyGp8yAAA
0JdAVWzYp+40KSB1zRNL/YSTL5h7ygXzTrlw3qkXziuFJYKsNo4aAAD6D6iK
DfvUnZo6qL7mqp9w0vlzT/rR3JPFEsK6IP2Ei/SwCwAAhgeoig371J2caai2
5pQfPf7c2SecN+dEsc6fkzsrDbJaOmoAAOg/oCo27FN3Ymai2ppdfvS4c2Yd
d85sKazMWamwWjpkAADoS6AqNuxTJzRUrXPyVX70+2fPylYmrCzIaumQAQCg
L4Gq2LBPXSkjdWUfOubMmd8T66yZx8qVCUs6q7WDBgCAPgSqYsM+dd/LfFRf
2YeOPmOGWMf8UK7s/cJZ7R0yAAD0JVAVG/apy3ykrexDR542/ajT5VKd1d4h
AwBAXwJVsWGfuiNPny6UpC3x/u+cOu07P3jiu2Kd9oR8Z+qso86Y3upRAwBA
/wFVsWGfOuEjc4n3H37K1CNOmXaEEJbirFYPGQAA+hKoig371B1+yjRzifd/
+6Sp3z556uFiKc5q9ZABAKAvgarYsE/dt06aaq5vnvD4YSdOEetbJ8klnPXt
1FntHjMAAPQjUBUb9qk77IQp3zTWN46f/I3jHxfCUp3V7gEDAECfAlWxYZ86
oaRyHXpctiYf8v3J4l+xVGe1e8AAANCnQFVs2KdOWKm+Jon19WPlyt4+JHPW
cZPbPWAAAOhToCo27FOXWelr5freY2IdnP4r17GPldpq94ABAKBPgarYsE/d
wcc8pq6vHvOoXEc/mr1xsFips9o9WgAA6F+gKjbsU/eVox9V1sSvHCXXl9Ml
/3t0rq12jxYAAPoXqIoN+9RJKx2Zry8dOSFbBxVv5M46amK7RwsAAP0LVMWG
feoO+u4EZT0i13ce+aJY2dvfzZ3V7tECAED/AlWxYZ+6L6Zi+sIRYo0v1+ez
N74zPvtou4cKAAB9DVTFpsmp+/zh44v1sFif+3a65NvSWWK1eJwAANDvQFVs
mpy6z3774c9+K1vjaku8P3VWi8cJAAD9DlTFpsmp+8xh49L10KfL9U3572e+
NU6sFg8SAAAGAKiKTZNTd+A3HzrwG2KNra1viiWF1eJBAgDAAABVsWly6j51
6NhPHjrGXJ/6xlixWjxIAAAYAKAqNk1O3ScOGfOJQ0Z/4uv5+ni65HsOkcJq
8SABAGAAgKpCOCDFfCf7G/7j10b/49ce1FYmrGZHCgAAAwhUFUi7qvqHgx/I
1t9/tVr/cLAUVrPDBACAAQSqCqRdVamG+ruv3J+t7L/NDhMAAAYQqCoQUlUl
sd+NVJVYLR0sAAAMFFBVIO1GVUlhK9VTUBUAAJBAVYG0rqoMqAoAALxAVSGQ
ib52Tx1UBQAANqAqNjh1AADQHXC9ZYNTBwAA3QHXWzY4dQAA0B1wvWWDUwcA
AN0B11s2OHUAANAdcL1lg1MHAADdAddbNjh1AADQHXC9ZYNTBwAA3QHXWzY4
dQAA0B1wvWWDUwcAAN0B11s2DU/dMJ/5YX7syXA/fDz2oQUXzF6BM89mmB97
MtwPH499aMEFs1f09szjy3v15T0/AHz5cH55zw+gr798mOnrXxy+vH8PAF8+
nF/e8wPo6y8fZg4AAADQLXp9yQcAAAAAAAAAAAAAAAAAAAAAAAAAAMOLrYNl
SDpbhvnhOx7jwD/2xPfwB/sM4Gkf/n4wEih/NdrvyPb+AWOYH77jMQ7Da9b9
8Lt+OF1lmJ/2GY5f+sA/9j5lyJ+03oc5wA8fqtLeKP878A9/yF/1CVTVhwz5
kxaqSiy/+gF+4BnD/Mwf8r9SEqiqDxnmF2zie5jD+dgPUOjFcXWJYX7mD/Nj
z4Cq+o4hf9J6CxYD/PCHWdPJcD/zh/mxZ0BV/Yj297P6KxuG3xr58IcnsiB/
9clwvGCH+ZmPx266aRgeOwAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAABoQjnRGqOtAQAAjFgG/u5gAAAA
BgB4CgAAwEgGURUAAICRDGpVAAAAAAAAAAAAAAAAAAAAAAAw8vn/AWy1wg8=

    "], {{0, 676}, {569, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{569, 676},
  PlotRange->{{0, 569}, {0, 676}}]], "Output",
 ImageSize->{573, 676},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[39]=",
 CellID->57742385]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Panel Methods",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/PanelMethods"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["3.3 Implementation",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Implementation"]], "RelatedTutorials",
 CellID->806871991],

Cell[TextData[ButtonBox["3.5 Convergence and Timing",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]], "RelatedTutorials",
 CellID->334470709],

Cell[TextData[ButtonBox["Table Of Contents",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/TableOfContents"]], "RelatedTutorials",
 CellID->2248688]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"3.4 Flow Visualization - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 10, 59, 58.6062000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "Flow visualization provides a useful plausibility check for numerical \
models in fluid dynamics. Mathematica's ability to work seamlessly with \
symbols and numbers is used to write functions describing the flow field for \
uniform flow over an airfoil. Mathematica functions for function \
visualization are supplemented by Aerodynamics functions when the conventions \
used by the aerodynamics community differ substantially from those used in \
Mathematica. Load the Aerodynamics packages and specify an arbitrary field \
point, r_f, as a collection of one vector of the data type CartesianVectors. \
This provides the input expected for computing influence coefficients in \
Aerodynamics and yields results that are functions of position instead of \
numerical values. A member of the NACA four-digit family of airfoils is \
selected as the example for this section for two reasons. First, by changing \
the identification number of the airfoil, calculations can be made for many \
airfoils including reference airfoils without thickness (camber lines). \
Second, discretization is based directly on the equations defining the \
airfoil instead of a spline fit to data in a file.", "synonyms" -> {}, 
    "title" -> "3.4 Flow Visualization", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/FlowField"}, "SearchTextTranslated" -> ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[603, 21, 78, 1, 70, "TutorialColorBar"],
Cell[684, 24, 209, 6, 70, "LinkTrail"],
Cell[896, 32, 2858, 64, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3779, 100, 59, 1, 70, "Title",
 CellID->509267359],
Cell[3841, 103, 683, 18, 70, "Text",
 CellID->823686949],
Cell[4527, 123, 503, 13, 70, "Text",
 CellID->34146804],
Cell[CellGroupData[{
Cell[5055, 140, 300, 9, 70, "Input",
 CellID->35199140],
Cell[5358, 151, 390, 13, 36, "Output",
 CellID->16583519]
}, Open  ]],
Cell[5763, 167, 456, 8, 70, "Text",
 CellID->972502991],
Cell[6222, 177, 765, 16, 70, "Text",
 CellID->70657603],
Cell[CellGroupData[{
Cell[7012, 197, 66, 1, 70, "Section",
 CellID->314388262],
Cell[7081, 200, 153, 4, 70, "Text",
 CellID->75137487],
Cell[7237, 206, 309, 9, 70, "Input",
 CellID->108183604],
Cell[7549, 217, 147, 4, 70, "Text",
 CellID->731882235],
Cell[CellGroupData[{
Cell[7721, 225, 476, 13, 83, "Input",
 CellID->401362744],
Cell[8200, 240, 5376, 96, 201, "Output",
 CellID->346796653]
}, Open  ]],
Cell[13591, 339, 89, 1, 70, "Text",
 CellID->19157149],
Cell[13683, 342, 263, 8, 70, "Input",
 CellID->450780366]
}, Open  ]],
Cell[CellGroupData[{
Cell[13983, 355, 58, 1, 70, "Section",
 CellID->1708000],
Cell[14044, 358, 551, 9, 70, "Text",
 CellID->35129489],
Cell[14598, 369, 403, 11, 70, "Text",
 CellID->48905279],
Cell[15004, 382, 873, 27, 70, "Input",
 CellID->17324249],
Cell[15880, 411, 89, 1, 70, "Text",
 CellID->275199393],
Cell[CellGroupData[{
Cell[15994, 416, 1241, 33, 70, "Input",
 CellID->666894011],
Cell[17238, 451, 131177, 2236, 70, "Output",
 CellID->437519550]
}, Open  ]],
Cell[148430, 2690, 358, 7, 70, "Text",
 CellID->130290240],
Cell[CellGroupData[{
Cell[148813, 2701, 1270, 34, 70, "Input",
 CellID->176178541],
Cell[150086, 2737, 86620, 1450, 70, "Output",
 CellID->352096662]
}, Open  ]],
Cell[236721, 4190, 175, 4, 70, "Text",
 CellID->11571494],
Cell[CellGroupData[{
Cell[236921, 4198, 1249, 33, 70, "Input",
 CellID->144480356],
Cell[238173, 4233, 24373, 430, 70, "Output",
 CellID->22095040]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[262595, 4669, 58, 1, 70, "Section",
 CellID->56297006],
Cell[262656, 4672, 589, 16, 70, "Text",
 CellID->253577469],
Cell[CellGroupData[{
Cell[263270, 4692, 1684, 51, 155, "Input",
 CellID->20343141],
Cell[264957, 4745, 27333, 456, 321, "Output",
 CellID->771285384]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[292339, 5207, 55, 1, 70, "Section",
 CellID->256023531],
Cell[292397, 5210, 495, 10, 70, "Text",
 CellID->117098232],
Cell[CellGroupData[{
Cell[292917, 5224, 1907, 55, 173, "Input",
 CellID->826808409],
Cell[294827, 5281, 18882, 317, 306, "Output",
 CellID->100775212]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[313758, 5604, 60, 1, 70, "Section",
 CellID->2809397],
Cell[313821, 5607, 695, 13, 70, "Text",
 CellID->176900651],
Cell[314519, 5622, 1109, 33, 70, "Input",
 CellID->460900760],
Cell[315631, 5657, 539, 9, 70, "Text",
 CellID->76766512],
Cell[316173, 5668, 1508, 41, 70, "Input",
 CellID->146745562],
Cell[317684, 5711, 158, 4, 70, "Text",
 CellID->614303007],
Cell[317845, 5717, 913, 23, 70, "Input",
 CellID->318266012],
Cell[318761, 5742, 218, 5, 70, "Text",
 CellID->42228577],
Cell[CellGroupData[{
Cell[319004, 5751, 263, 8, 28, "Input",
 CellID->448653773],
Cell[319270, 5761, 86807, 1431, 643, "Output",
 CellID->57742385]
}, Open  ]],
Cell[CellGroupData[{
Cell[406114, 7197, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[406182, 7200, 159, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[406378, 7208, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[406453, 7211, 170, 4, 70, "RelatedTutorials",
 CellID->806871991],
Cell[406626, 7217, 184, 4, 70, "RelatedTutorials",
 CellID->334470709],
Cell[406813, 7223, 168, 4, 70, "RelatedTutorials",
 CellID->2248688]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[407020, 7232, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

