(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     25375,        609]
NotebookOptionsPosition[     22458,        520]
NotebookOutlinePosition[     23942,        559]
CellTagsIndexPosition[     23899,        556]
WindowTitle->A2 Graphics - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"A1 CartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/CartesianVectors"], "\<\"Acknowledgments\
\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/Acknowledgments"], "\<\"Table of \
Contents\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Graphics\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/Graphics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"PlotAirfoil\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PlotAirfoil"], "\<\"PlotConvergence\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PlotConvergence"], \
"\<\"PlotSurfacePressureCoefficient\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/tutorial/Graphics\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/Graphics"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/Graphics.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/Graphics", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["A2 Graphics", "Title",
 CellID->509267359],

Cell[TextData[{
 "Some graphics are used so often or are so specialized that it is convenient \
to define ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " functions to generate them."
}], "Text",
 CellID->1534169418],

Cell[BoxData[GridBox[{
   {
    ButtonBox["PlotAirfoil",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PlotAirfoil"], Cell[
    "plots an airfoil in an airfoil-fixed coordinated system", "TableText"]},
   {
    ButtonBox["PlotConvergence",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PlotConvergence"], Cell[
    "plots convergence properties of model", "TableText"]},
   {
    ButtonBox["PlotSurfacePressureCoefficient",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/PlotSurfacePressureCoefficient"], 
    Cell["plots pressure coefficient on airfoil surface", "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell[TextData[{
 "Special graphics functions used in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 "."
}], "Caption",
 CellID->1891092685],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->435059],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[0]:=",
 CellID->273674889],

Cell[TextData[{
 "PlotAirfoil is a straightforward application of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ListPlot, but it is used so often that it a named function in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  In the following example, a discretized NACA-4412 airfoil is plotted in \
an airfoil-fixed coordinate system.  Axes are labeled, and all options of \
ListPlot are accepted."
}], "Text",
 CellID->64486792],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"rP", "=", 
   RowBox[{"AirfoilSurfacePoints", "[", 
    RowBox[{"NACA4DigitAirfoil", "[", 
     RowBox[{"4412", ",", 
      RowBox[{"NDiscretizeUnitSegment", "[", 
       RowBox[{"50", ",", 
        RowBox[{"Layout", "\[Rule]", "HalfCosineSpacing"}]}], "]"}]}], "]"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotAirfoil", "[", "rP", "]"}]}], "Input",
 CellLabel->"In[1]:=",
 CellID->2058623809],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c9rFHkax/Fe9i/ZfyMgcxAP4iFkQTyIF4VGL+NFL3qJl92LQpNcEpDJ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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->26433574]
}, Open  ]],

Cell[TextData[{
 "PlotConvergence is a specialized function used only in one section of the \
tutorial, but it is used several times in that section, and it seemed \
appropriate to define as a separate function.  See ",
 ButtonBox["3.4 Convergence and Timing",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/ConvergenceAndTiming"],
 " for usage."
}], "Text",
 CellID->76441082],

Cell[TextData[{
 "The conventions used in the aerodynamics community for plotting surface \
pressure coefficient are sufficiently different from ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " conventions to warrant a special function.  Surface pressure coefficients \
associated with collocation points are plotted versus chordwise locations on \
the airfoil (",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-values).  For a lifting airfoil at a cruise angle of attack, the upper \
surface of the airfoil has primarily negative pressure coefficients and the \
lower surface, primarily positive pressure coefficients.  For this reason \
negative pressure coefficients are plotted up and positive pressure \
coefficients, down, so that the upper curve corresponds to the upper surface \
of the airfoil.  In the following example, PMVortexLinearNeumann is used to \
solve for potential flow over a NACA-4412 airfoil at an angle of attack of \
five degrees, and pressure coefficient is obtained from Bernouilli's equation \
using fluid speeds at collocation points.  A plot of the discretized airfoil \
is inset if the collection of vectors locating panel endpoints is included in \
the input arguments."
}], "Text",
 CellID->410626370],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "rC", ",", "vC"}], "}"}], "=", 
   RowBox[{"PMVortexLinearNeumann", "[", 
    RowBox[{"rP", ",", 
     RowBox[{"5.", " ", "Degree"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xC", ",", "cp"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Components", "[", "rC", "]"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], ",", 
     RowBox[{"1", "-", 
      RowBox[{"vC", "^", "2"}]}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"PlotSurfacePressureCoefficient", "[", 
  RowBox[{"xC", ",", "cp", ",", "rP"}], 
  "]"}], "\[IndentingNewLine]"}], "Input",
 CellLabel->"In[3]:=",
 CellID->552331023],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3U+IHGd6B+AJuewt/w7JMXvZHPaWnDIQwqKIjeKDUcD4IARBDs3uJXuR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    "], {{0, 330}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 330},
  PlotRange->{{0, 504}, {0, 330}}]], "Output",
 ImageSize->{508, 330},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->613810890]
}, Open  ]],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Graphics"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["A1 CartesianVectors",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/CartesianVectors"]], "RelatedTutorials",
 CellID->147844826],

Cell[TextData[ButtonBox["Acknowledgments",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/Acknowledgments"]], "RelatedTutorials",
 CellID->237435115],

Cell[TextData[ButtonBox["Table of Contents",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/TableOfContents"]], "RelatedTutorials",
 CellID->324670292]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"A2 Graphics - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 11, 0, 9.1830000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "Some graphics are used so often or are so specialized that it is \
convenient to define Aerodynamics functions to generate them. Special \
graphics functions used in Aerodynamics. Load the Aerodynamics packages.", 
    "synonyms" -> {}, "title" -> "A2 Graphics", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/Graphics"}, "SearchTextTranslated" -> ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[592, 21, 78, 1, 70, "TutorialColorBar"],
Cell[673, 24, 191, 5, 70, "LinkTrail"],
Cell[867, 31, 3491, 79, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4383, 114, 48, 1, 70, "Title",
 CellID->509267359],
Cell[4434, 117, 223, 7, 70, "Text",
 CellID->1534169418],
Cell[4660, 126, 681, 17, 70, "DefinitionBox",
 CellID->2096742444],
Cell[5344, 145, 146, 6, 70, "Caption",
 CellID->1891092685],
Cell[5493, 153, 122, 6, 70, "Text",
 CellID->435059],
Cell[5618, 161, 100, 3, 70, "Input",
 CellID->273674889],
Cell[5721, 166, 454, 11, 70, "Text",
 CellID->64486792],
Cell[CellGroupData[{
Cell[6200, 181, 446, 12, 83, "Input",
 CellID->2058623809],
Cell[6649, 195, 5375, 96, 201, "Output",
 CellID->26433574]
}, Open  ]],
Cell[12039, 294, 395, 9, 70, "Text",
 CellID->76441082],
Cell[12437, 305, 1254, 22, 70, "Text",
 CellID->410626370],
Cell[CellGroupData[{
Cell[13716, 331, 758, 23, 83, "Input",
 CellID->552331023],
Cell[14477, 356, 7043, 123, 351, "Output",
 CellID->613810890]
}, Open  ]],
Cell[CellGroupData[{
Cell[21557, 484, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[21625, 487, 150, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[21812, 495, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[21887, 498, 173, 4, 70, "RelatedTutorials",
 CellID->147844826],
Cell[22063, 504, 168, 4, 70, "RelatedTutorials",
 CellID->237435115],
Cell[22234, 510, 170, 4, 70, "RelatedTutorials",
 CellID->324670292]
}, Open  ]]
}, Open  ]],
Cell[22431, 518, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

