(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     31585,       1010]
NotebookOptionsPosition[     24325,        790]
NotebookOutlinePosition[     28587,        901]
CellTagsIndexPosition[     28492,        895]
WindowTitle->2.1  Introduction to Influence Coefficients - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"1.5 Panel Geometry\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PanelGeometry"], "\<\"2.2 Point \
Singularities\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PointSingularities"], "\<\"Table Of \
Contents\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"], \
"\<\"Aerodynamics\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/\
IntroductionToInfluenceCoefficients\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
             "Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/IntroductionToInfluenceCoefficients.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/IntroductionToInfluenceCoefficients", 
             ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["2.1  Introduction to Influence Coefficients", "Title",
 CellID->509267359],

Cell[TextData[{
 "Velocity, velocity potential and stream function at a point due to a \
singularity panel can be written as products of geometric quantities called \
influence coefficients, and strength parameters for the singularity.  These \
strength parameters are constants in an assumed distribution of strength \
density on the panel.  The basic potential-flow singularities used in panel \
methods are source, vortex and doublet.  These singularities can be \
concentrated at a point or distributed over lines or surfaces, and can be \
used in combination.  These variations provide a rich variety of options for \
constructing panel methods for computing potential flow over an object.  The \
current version of ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " is restricted to models using two-dimensional straight-line singularities \
with the strength density of the singularity being a polynomial in distance \
from the beginning of the line segment."
}], "Text",
 CellID->351158547],

Cell[CellGroupData[{

Cell["\<\
2.1.1 Geometric influence coefficients and strength parameters\
\>", "Section",
 CellID->320914423],

Cell["\<\
The sketch below illustrates the local coordinate system used for an isolated \
two-dimensional line singularity.  In keeping with the terminology of panel \
methods, this singularity is referred to as a panel.\
\>", "Text",
 CellID->1534169418],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"AnnotatedArrow", "[", 
    RowBox[{"p_", ",", "q_", ",", "label_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Arrowheads", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{".03", ",", ".4", ",", 
          RowBox[{"Graphics", "[", 
           RowBox[{"Inset", "[", 
            RowBox[{
             RowBox[{"Style", "[", 
              RowBox[{"label", ",", "Medium"}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"Center", ",", "Top"}], "}"}]}], "]"}], "]"}]}], "}"}], 
        ",", 
        RowBox[{"{", 
         RowBox[{".05", ",", "1"}], "}"}]}], "}"}], "]"}], ",", 
     RowBox[{"Arrow", "[", 
      RowBox[{"{", 
       RowBox[{"p", ",", "q"}], "}"}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Graphics", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"AnnotatedArrow", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}], ",", 
       RowBox[{"Style", "[", 
        RowBox[{"\"\<\!\(\*SubscriptBox[\(r\), \(1\)]\)\>\"", ",", "Medium"}],
         "]"}]}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"AnnotatedArrow", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}], ",", 
       RowBox[{
       "Style", "[", "\"\<\!\(\*SubscriptBox[\(r\), \(2\)]\)\>\"", "]"}]}], 
      "]"}], ",", "\[IndentingNewLine]", "Thick", ",", " ", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"PointSize", "[", "Large", "]"}], ",", 
     RowBox[{"Point", "[", 
      RowBox[{"{", 
       RowBox[{"2", ",", "1"}], "}"}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Text", "[", 
      RowBox[{"\"\<P\>\"", ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1.08"}], "}"}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Text", "[", 
      RowBox[{"\"\<\[Delta]\>\"", ",", 
       RowBox[{"{", 
        RowBox[{"0.55", ",", "0.05"}], "}"}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Text", "[", 
      RowBox[{"\"\<\!\(\*SubscriptBox[\(\[Theta]\), \(1\)]\)\>\"", ",", 
       RowBox[{"{", 
        RowBox[{"0.26", ",", "0.05"}], "}"}]}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Text", "[", 
      RowBox[{"\"\<\!\(\*SubscriptBox[\(\[Theta]\), \(2\)]\)\>\"", ",", 
       RowBox[{"{", 
        RowBox[{"1.18", ",", "0.05"}], "}"}]}], "]"}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Axes", "\[Rule]", "True"}], ",", 
   RowBox[{"AxesLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"Ticks", "\[Rule]", "None"}]}], "]"}]}], "Input",
 CellOpen->False,
 CellLabel->"In[0]:=",
 CellID->55184314],

Cell[BoxData[
 GraphicsBox[{
   {Arrowheads[{{0.03, 0.4, 
       GraphicsBox[
        InsetBox[
         BoxData[
          FormBox[
           StyleBox[
            StyleBox[
            "\"\\!\\(\\*SubscriptBox[\\(r\\), \\(1\\)]\\)\"", Medium, 
             StripOnInput -> False], Medium, StripOnInput -> False], 
           TraditionalForm]], {Center, Top}]]}, {0.05, 1}}], 
    ArrowBox[{{0, 0}, {2, 1}}]}, 
   {Arrowheads[{{0.03, 0.4, 
       GraphicsBox[
        InsetBox[
         BoxData[
          FormBox[
           StyleBox[
            StyleBox[
            "\"\\!\\(\\*SubscriptBox[\\(r\\), \\(2\\)]\\)\"", StripOnInput -> 
             False], Medium, StripOnInput -> False], TraditionalForm]], {
         Center, Top}]]}, {0.05, 1}}], ArrowBox[{{1, 0}, {2, 1}}]}, 
   {Thickness[Large], LineBox[{{0, 0}, {1, 0}}], 
    {PointSize[Large], PointBox[{2, 1}], InsetBox["\<\"P\"\>", {2, 1.08}], 
     InsetBox["\<\"\[Delta]\"\>", {0.55, 0.05}], 
     InsetBox["\<\"\\!\\(\\*SubscriptBox[\\(\[Theta]\\), \\(1\\)]\\)\"\>", \
{0.26, 0.05}], 
     InsetBox["\<\"\\!\\(\\*SubscriptBox[\\(\[Theta]\\), \\(2\\)]\\)\"\>", \
{1.18, 0.05}]}}},
  Axes->True,
  AxesLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  ImageSize->504,
  Ticks->None]], "Output",
 ImageSize->{508, 292},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->"local",
 CellLabel->"Out[1]=",
 CellID->100705525]
}, Open  ]],

Cell[TextData[{
 "The panel extends from the origin along the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the length of the panel, ",
 Cell[BoxData[
  StyleBox["\[Delta]", "TR"]], "InlineFormula"],
 ".  A field point at which velocity, velocity potential or stream function \
are computed is labeled P.  Geometric influence coefficients depend on the \
displacement vectors from ends of the panel to P."
}], "Text",
 CellID->344307243],

Cell[TextData[{
 "If the singularity density ",
 Cell[BoxData[
  StyleBox["f", "TI"]], "InlineFormula"],
 " (source, vortex or doublet) of the panel is a constant ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 ", then the velocity components, velocity potential and stream function can \
be written as products of the appropriate influence coefficient (",
 Cell[BoxData[
  StyleBox["ic", "TI"]], "InlineFormula"],
 ") and the strength parameter. "
}], "Text",
 CellID->39044744],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["f", "TI"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   StyleBox["a", "TI"], " "}]], "InlineFormula"]
}], "Text",
 CellID->434040083],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      StyleBox["u", "TI"], ",", 
      StyleBox["w", "TI"]}], "}"}], "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       SubscriptBox[
        StyleBox["ic", "TI"], 
        StyleBox["x", "TI"]], " ", 
       StyleBox["a", "TI"]}], ",", 
      RowBox[{
       SubscriptBox[
        StyleBox["ic", "TI"], 
        StyleBox["z", "TI"]], " ", 
       StyleBox["a", "TI"]}]}], "}"}]}], ",", " ", 
   RowBox[{
    StyleBox["\[Phi]", "TR"], "=", 
    RowBox[{
     SubscriptBox[
      StyleBox["ic", "TI"], 
      StyleBox["\[Phi]", "TR"]], " ", 
     StyleBox["a", "TI"]}]}], ",", " ", 
   RowBox[{
    StyleBox["\[Psi]", "TR"], "=", 
    RowBox[{
     SubscriptBox[
      StyleBox["ic", "TI"], 
      StyleBox["\[Psi]", "TR"]], " ", 
     StyleBox["a", "TI"]}]}]}]], "InlineFormula"]
}], "Text",
 CellID->103946562],

Cell["\<\
If the singularity density varies linearly along the panel, then the velocity \
components, velocity potential and stream function can be written as products \
of appropriate influence coefficients and strength parameters as follows.\
\>", "Text",
 CellID->559574336],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["f", "TI"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{
    StyleBox["a", "TI"], "+", 
    RowBox[{
     StyleBox["b", "TI"], " ", 
     StyleBox["x", "TI"]}]}]}]], "InlineFormula"]
}], "Text",
 CellID->728334774],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      StyleBox["u", "TI"], ",", 
      StyleBox["w", "TI"]}], "}"}], "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox[
         StyleBox["ica", "TI"], 
         StyleBox["x", "TI"]], " ", 
        StyleBox["a", "TI"]}], "+", 
       RowBox[{
        SubscriptBox[
         StyleBox["icb", "TI"], 
         StyleBox["x", "TI"]], " ", 
        StyleBox["b", "TI"]}]}], ",", 
      RowBox[{
       RowBox[{
        SubscriptBox[
         StyleBox["ica", "TI"], 
         StyleBox["z", "TI"]], " ", 
        StyleBox["a", "TI"]}], "+", 
       RowBox[{
        SubscriptBox[
         StyleBox["icb", "TI"], 
         StyleBox["z", "TI"]], " ", 
        StyleBox["b", "TI"]}]}]}], "}"}]}], ",", " ", 
   RowBox[{
    StyleBox["\[Phi]", "TR"], "=", 
    RowBox[{
     RowBox[{
      SubscriptBox[
       StyleBox["ica", "TI"], 
       StyleBox["\[Phi]", "TR"]], " ", 
      StyleBox["a", "TI"]}], "+", 
     RowBox[{
      SubscriptBox[
       StyleBox["icb", "TI"], 
       StyleBox["\[Phi]", "TR"]], " ", 
      StyleBox["b", "TI"]}]}]}], ",", " ", 
   RowBox[{
    StyleBox["\[Psi]", "TR"], "=", 
    RowBox[{
     RowBox[{
      SubscriptBox[
       StyleBox["ica", "TI"], 
       StyleBox["\[Psi]", "TR"]], " ", 
      StyleBox["a", "TI"]}], "+", 
     RowBox[{
      SubscriptBox[
       StyleBox["icb", "TI"], 
       StyleBox["\[Psi]", "TR"]], " ", 
      StyleBox["b", "TI"]}]}]}]}]], "InlineFormula"]
}], "Text",
 CellID->37516354],

Cell[TextData[{
 "A straightforward generalization of these equations serves for strength \
distributions being described for higher-order polynomials.  Functions are \
defined in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " to compute influence-coefficients functions for source, vortex and doublet \
panels of constant and linearly varying strength.  Additionally, \
influence-coefficients are defined for doublet panels of quadratically \
varying strength."
}], "Text",
 CellID->377094181],

Cell[TextData[{
 "Names for all influence-coefficient functions defined in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " begin with IC and continue with descriptive names.  For example, velocity, \
velocity-potential and stream-function influence-coefficient functions for \
vortex panels of linearly varying circulation density are named \
ICVortexLinear, IC\[Phi]VortexLinear and IC\[Psi]VortexLinear.  For \
completeness, influence-coefficient functions are provided for \
two-dimensional point source, vortex and doublet."
}], "Text",
 CellID->52889481]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
2.1.2 Basic relations between two-dimensional vortex and doublet panels\
\>", "Section",
 CellTags->"relation",
 CellID->133089311],

Cell[TextData[{
 "The general solution of Laplace's equation based on one of Green's \
identifies is discussed in Chapter 3 of ",
 ButtonBox["Katz and Plotkin (2001)",
  BaseStyle->"Hyperlink",
  ButtonData->{"References.nb", "katz2001"}],
 ".  This development leads naturally to the use of source, vortex and \
doublet singularities.  There is a relationship between circulation density \
and doublet density for vortex and doublet panels which can be stated as \
follows."
}], "Text",
 CellTags->"relation",
 CellID->179165981],

Cell[TextData[{
 "\t\t\t\t ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox["\[Gamma]", "TR"], "(", 
     StyleBox["x", "TI"], ")"}], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       StyleBox["d\[Mu]", "TI"], "(", 
       StyleBox["x", "TI"], ")"}], 
      StyleBox["dx", "TI"]]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"relation",
 CellID->60328517],

Cell[TextData[{
 "Circulation density is denoted by ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Gamma]", "TR"], 
   RowBox[{"(", 
    StyleBox["x", "TI"], ")"}]}]], "InlineFormula"],
 " and doublet density is denoted by ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Mu]", "TR"], 
   RowBox[{"(", 
    StyleBox["x", "TI"], ")"}]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " denotes the distance along the panel in a panel-local coordinate system."
}], "Text",
 CellTags->"relation",
 CellID->22533550],

Cell[TextData[{
 "The total strength of a vortex panel extending from ",
 Cell[BoxData["0"], "InlineFormula"],
 " to ",
 Cell[BoxData["\[Delta]"], "InlineFormula"],
 " is obtained by integrating the circulation density function over the sheet."
}], "Text",
 CellTags->"relation",
 CellID->154263561],

Cell[TextData[{
 "\t\t\t\t",
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[CapitalGamma]", "=", 
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", "\[Delta]"], 
     RowBox[{
      RowBox[{"\[Gamma]", "(", "x", ")"}], 
      RowBox[{"\[DifferentialD]", "x"}]}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"relation",
 CellID->52774633],

Cell["\<\
Substitute for the circulation density in terms of the doublet density.\
\>", "Text",
 CellTags->"relation",
 CellID->45577279],

Cell[TextData[{
 "\t\t\t\t ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    StyleBox["\[CapitalGamma]", "TR"], "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", 
        StyleBox["\[Delta]", "TR"]], 
       RowBox[{
        FractionBox[
         RowBox[{
          StyleBox["d\[Mu]", "TI"], "(", 
          StyleBox["x", "TI"], ")"}], 
         StyleBox["dx", "TI"]], 
        RowBox[{"\[DifferentialD]", 
         StyleBox["x", "TI"]}]}]}]}], "=", 
     RowBox[{
      RowBox[{
       StyleBox["\[Mu]", "TR"], "(", "0", ")"}], "-", 
      RowBox[{
       StyleBox["\[Mu]", "TR"], "(", 
       StyleBox["\[Delta]", "TR"], ")"}]}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"relation",
 CellID->482647799],

Cell[CellGroupData[{

Cell["Constant-strength doublet panel", "Subsection",
 CellID->299286692],

Cell[TextData[{
 "For example, set the doublet density to a constant, ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Mu]", "TR"], "=", 
   StyleBox["a", "TI"]}]], "InlineFormula"],
 ".  Then the circulation density on the panel is zero and the total \
circulation for the panel is zero."
}], "Text",
 CellTags->"relation",
 CellID->30442929],

Cell[BoxData[
 FormBox[
  RowBox[{"\t\t\t\t", 
   RowBox[{
    StyleBox["\[CapitalGamma]", "TR"], "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "0", 
        StyleBox["\[Delta]", "TR"]], 
       RowBox[{
        FractionBox[
         RowBox[{
          StyleBox["d\[Mu]", "TI"], "(", 
          StyleBox["x", "TI"], ")"}], 
         StyleBox["dx", "TI"]], 
        RowBox[{"\[DifferentialD]", 
         StyleBox["x", "TI"]}]}]}]}], "=", 
     RowBox[{
      RowBox[{
       StyleBox["a", "TI"], "-", 
       StyleBox["a", "TI"]}], "=", "0"}]}]}]}], TraditionalForm]], "Text",
 CellTags->"relation",
 CellID->622982394],

Cell[TextData[{
 "To see what is happening at the end points, look at the integral from \
negative infinity to just inside the panel near ",
 Cell[BoxData[
  RowBox[{
   StyleBox["x", "TI"], "=", "0"}]], "InlineFormula"],
 ".  Since the doublet density is zero off the panel and changes abruptly to ",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " on the panel, the circulation computed is the value of a point vortex \
located at ",
 Cell[BoxData[
  RowBox[{
   StyleBox["x", "TI"], "=", "0"}]], "InlineFormula"],
 "."
}], "Text",
 CellTags->"relation",
 CellID->701435519],

Cell[BoxData[
 FormBox[
  RowBox[{"\t\t\t\t", 
   RowBox[{
    StyleBox[
     RowBox[{"\[CapitalGamma]", "(", "0", ")"}], "TR"], "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", 
        RowBox[{"-", "\[Infinity]"}], 
        StyleBox["0", "TR"]], 
       RowBox[{
        FractionBox[
         RowBox[{
          StyleBox["d\[Mu]", "TI"], "(", 
          StyleBox["x", "TI"], ")"}], 
         StyleBox["dx", "TI"]], 
        RowBox[{"\[DifferentialD]", 
         StyleBox["x", "TI"]}]}]}]}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{
        StyleBox["\[Mu]", "TR"], "(", 
        RowBox[{"-", "\[Infinity]"}], ")"}], "-", 
       RowBox[{
        StyleBox["\[Mu]", "TR"], "(", 
        StyleBox["0", "TR"], ")"}]}], "=", 
      RowBox[{"-", 
       StyleBox["a", "TI"]}]}]}]}]}], TraditionalForm]], "Text",
 CellTags->"relation",
 CellID->249012586],

Cell[TextData[{
 "Likewise, the circulation associated with the endpoint of the panel ",
 Cell[BoxData[
  RowBox[{
   StyleBox["x", "TI"], "=", 
   StyleBox["\[Delta]", "TR"]}]], "InlineFormula"],
 " is obtained by evaluating the integral."
}], "Text",
 CellTags->"relation",
 CellID->330029328],

Cell[BoxData[
 FormBox[
  RowBox[{"\t\t\t\t", 
   RowBox[{
    StyleBox[
     RowBox[{"\[CapitalGamma]", "(", "\[Delta]", ")"}], "TR"], "=", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubsuperscriptBox["\[Integral]", "\[Delta]", 
        StyleBox["\[Infinity]", "TR"]], 
       RowBox[{
        FractionBox[
         RowBox[{
          StyleBox["d\[Mu]", "TI"], "(", 
          StyleBox["x", "TI"], ")"}], 
         StyleBox["dx", "TI"]], 
        RowBox[{"\[DifferentialD]", 
         StyleBox["x", "TI"]}]}]}]}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{
        StyleBox["\[Mu]", "TR"], "(", "\[Delta]", ")"}], "-", 
       RowBox[{
        StyleBox["\[Mu]", "TR"], "(", "\[Infinity]", ")"}]}], "=", 
      StyleBox["a", "TI"]}]}]}]}], TraditionalForm]], "Text",
 CellTags->"relation",
 CellID->710526163],

Cell[TextData[{
 "Thus, a constant-strength doublet panel gives the same flow field as a \
contra-rotating pair of point vortices of strength \[MinusPlus]",
 Cell[BoxData[
  StyleBox["a", "TI"]], "InlineFormula"],
 " at panel endpoints.  In general, the flow field induced by a doublet panel \
whose doublet density function is described by a polynomial of order ",
 Cell[BoxData[
  StyleBox["n", "TI"]], "InlineFormula"],
 " is equivalent to that of a vortex panel whose circulation density is a \
polynomial of order ",
 Cell[BoxData[
  RowBox[{
   StyleBox["n", "TI"], "-", "1"}]], "InlineFormula"],
 " and point vortices at panel endpoints."
}], "Text",
 CellTags->"relation",
 CellID->4484441],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/InfluenceCoefficients"]], "TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->80129204]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["1.5 Panel Geometry",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PanelGeometry"]], "RelatedTutorials",
 CellID->574311372],

Cell[TextData[ButtonBox["2.2 Point Singularities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities"]], "RelatedTutorials",
 CellID->895221539],

Cell[TextData[ButtonBox["Table Of Contents",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/TableOfContents"]], "RelatedTutorials",
 CellID->6988088]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"2.1  Introduction to Influence Coefficients - Wolfram \
Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 11, 0, 18.1998000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "Velocity, velocity potential and stream function at a point due to a \
singularity panel can be written as products of geometric quantities called \
influence coefficients, and strength parameters for the singularity. These \
strength parameters are constants in an assumed distribution of strength \
density on the panel. The basic potential-flow singularities used in panel \
methods are source, vortex and doublet. These singularities can be \
concentrated at a point or distributed over lines or surfaces, and can be \
used in combination. These variations provide a rich variety of options for \
constructing panel methods for computing potential flow over an object. The \
current version of Aerodynamics is restricted to models using two-dimensional \
straight-line singularities with the strength density of the singularity \
being a polynomial in distance from the beginning of the line segment. The \
sketch below illustrates the local coordinate system used for an isolated \
two-dimensional line singularity. In keeping with the terminology of panel \
methods, this singularity is referred to as a panel. The panel extends from \
the origin along the x-axis for the length of the panel, \\[Delta]. A field \
point at which velocity, velocity potential or stream function are computed \
is labeled P. Geometric influence coefficients depend on the displacement \
vectors from ends of the panel to P.", "synonyms" -> {}, "title" -> 
    "2.1  Introduction to Influence Coefficients", "type" -> "Tutorial", 
    "uri" -> "Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "local"->{
  Cell[8725, 236, 1451, 41, 313, "Output",
   CellTags->"local",
   CellID->100705525]},
 "relation"->{
  Cell[15584, 472, 141, 4, 70, "Section",
   CellTags->"relation",
   CellID->133089311],
  Cell[15728, 478, 530, 12, 70, "Text",
   CellTags->"relation",
   CellID->179165981],
  Cell[16261, 492, 390, 16, 70, "Text",
   CellTags->"relation",
   CellID->60328517],
  Cell[16654, 510, 548, 19, 70, "Text",
   CellTags->"relation",
   CellID->22533550],
  Cell[17205, 531, 299, 8, 70, "Text",
   CellTags->"relation",
   CellID->154263561],
  Cell[17507, 541, 344, 13, 70, "Text",
   CellTags->"relation",
   CellID->52774633],
  Cell[17854, 556, 137, 4, 70, "Text",
   CellTags->"relation",
   CellID->45577279],
  Cell[17994, 562, 752, 27, 70, "Text",
   CellTags->"relation",
   CellID->482647799],
  Cell[18847, 596, 338, 10, 70, "Text",
   CellTags->"relation",
   CellID->30442929],
  Cell[19188, 608, 665, 23, 70, "Text",
   CellTags->"relation",
   CellID->622982394],
  Cell[19856, 633, 588, 17, 70, "Text",
   CellTags->"relation",
   CellID->701435519],
  Cell[20447, 652, 904, 31, 70, "Text",
   CellTags->"relation",
   CellID->249012586],
  Cell[21354, 685, 295, 9, 70, "Text",
   CellTags->"relation",
   CellID->330029328],
  Cell[21652, 696, 829, 27, 70, "Text",
   CellTags->"relation",
   CellID->710526163],
  Cell[22484, 725, 698, 17, 70, "Text",
   CellTags->"relation",
   CellID->4484441]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"local", 27052, 842},
 {"relation", 27156, 846}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[624, 21, 78, 1, 70, "TutorialColorBar"],
Cell[705, 24, 191, 5, 70, "LinkTrail"],
Cell[899, 31, 3136, 72, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4060, 107, 80, 1, 70, "Title",
 CellID->509267359],
Cell[4143, 110, 1003, 17, 70, "Text",
 CellID->351158547],
Cell[CellGroupData[{
Cell[5171, 131, 109, 3, 70, "Section",
 CellID->320914423],
Cell[5283, 136, 255, 5, 70, "Text",
 CellID->1534169418],
Cell[CellGroupData[{
Cell[5563, 145, 3159, 89, 19, "Input",
 CellOpen->False,
 CellID->55184314],
Cell[8725, 236, 1451, 41, 313, "Output",
 CellTags->"local",
 CellID->100705525]
}, Open  ]],
Cell[10191, 280, 463, 11, 70, "Text",
 CellID->344307243],
Cell[10657, 293, 492, 13, 70, "Text",
 CellID->39044744],
Cell[11152, 308, 228, 10, 70, "Text",
 CellID->434040083],
Cell[11383, 320, 914, 36, 70, "Text",
 CellID->103946562],
Cell[12300, 358, 277, 5, 70, "Text",
 CellID->559574336],
Cell[12580, 365, 317, 14, 70, "Text",
 CellID->728334774],
Cell[12900, 381, 1571, 60, 70, "Text",
 CellID->37516354],
Cell[14474, 443, 503, 11, 70, "Text",
 CellID->377094181],
Cell[14980, 456, 567, 11, 70, "Text",
 CellID->52889481]
}, Open  ]],
Cell[CellGroupData[{
Cell[15584, 472, 141, 4, 70, "Section",
 CellTags->"relation",
 CellID->133089311],
Cell[15728, 478, 530, 12, 70, "Text",
 CellTags->"relation",
 CellID->179165981],
Cell[16261, 492, 390, 16, 70, "Text",
 CellTags->"relation",
 CellID->60328517],
Cell[16654, 510, 548, 19, 70, "Text",
 CellTags->"relation",
 CellID->22533550],
Cell[17205, 531, 299, 8, 70, "Text",
 CellTags->"relation",
 CellID->154263561],
Cell[17507, 541, 344, 13, 70, "Text",
 CellTags->"relation",
 CellID->52774633],
Cell[17854, 556, 137, 4, 70, "Text",
 CellTags->"relation",
 CellID->45577279],
Cell[17994, 562, 752, 27, 70, "Text",
 CellTags->"relation",
 CellID->482647799],
Cell[CellGroupData[{
Cell[18771, 593, 73, 1, 70, "Subsection",
 CellID->299286692],
Cell[18847, 596, 338, 10, 70, "Text",
 CellTags->"relation",
 CellID->30442929],
Cell[19188, 608, 665, 23, 70, "Text",
 CellTags->"relation",
 CellID->622982394],
Cell[19856, 633, 588, 17, 70, "Text",
 CellTags->"relation",
 CellID->701435519],
Cell[20447, 652, 904, 31, 70, "Text",
 CellTags->"relation",
 CellID->249012586],
Cell[21354, 685, 295, 9, 70, "Text",
 CellTags->"relation",
 CellID->330029328],
Cell[21652, 696, 829, 27, 70, "Text",
 CellTags->"relation",
 CellID->710526163],
Cell[22484, 725, 698, 17, 70, "Text",
 CellTags->"relation",
 CellID->4484441],
Cell[CellGroupData[{
Cell[23207, 746, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[23275, 749, 179, 4, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[23457, 755, 156, 3, 70, "TutorialMoreAbout",
 CellID->80129204]
}, Open  ]],
Cell[CellGroupData[{
Cell[23650, 763, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[23725, 766, 169, 4, 70, "RelatedTutorials",
 CellID->574311372],
Cell[23897, 772, 179, 4, 70, "RelatedTutorials",
 CellID->895221539],
Cell[24079, 778, 168, 4, 70, "RelatedTutorials",
 CellID->6988088]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[24298, 788, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

