(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    725852,      15082]
NotebookOptionsPosition[    710843,      14559]
NotebookOutlinePosition[    712953,      14607]
CellTagsIndexPosition[    712910,      14604]
WindowTitle->2.3  Isolated panel - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"2.2 Point Singularities\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/PointSingularities"], "\<\"3.1 \
Introduction to Panel Methods\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"], "\<\"Table \
of Contents\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"InfluenceCoefficients\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"ICDoubletLinear\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletLinear"], "\<\"ICSourceConstant\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourceConstant"], \
"\<\"ICVortexConstant\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexConstant"], \
"\<\"IC\[Phi]DoubletLinear\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletLinear"], \
"\<\"IC\[Phi]SourceConstant\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourceConstant"], \
"\<\"IC\[Phi]VortexConstant\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexConstant"], \
"\<\"IC\[Psi]DoubletLinear\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICDoubletLinear"], \
"\<\"IC\[Psi]SourceConstant\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICSourceConstant"], \
"\<\"IC\[Psi]VortexConstant\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/ref/ICVortexConstant"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics/tutorial/IsolatedPanel\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/IsolatedPanel"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/IsolatedPanel.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/IsolatedPanel", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["2.3  Isolated panel", "Title",
 CellID->509267359],

Cell["\<\
Some properties of isolated straight-line singularity panels are illustrated \
in this section using examples of a constant-strength source panel, a \
constant-strength vortex panel, and a doublet panel of linearly varying \
strength.\
\>", "Text",
 CellID->20743874],

Cell[TextData[{
 "Load the ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " packages."
}], "Text",
 CellID->819798417],

Cell[BoxData[
 RowBox[{"<<", "Aerodynamics`"}]], "Input",
 CellLabel->"In[0]:=",
 CellID->5378332],

Cell[CellGroupData[{

Cell["2.3.1  Algebraic expressions for influence coefficients", "Section",
 CellID->67289592],

Cell[TextData[{
 "When working with an isolated singularity, a panel local coordinate system \
is used in which the panel extends from the origin along the ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the length of the panel, ",
 Cell[BoxData[
  StyleBox["\[Delta]", "TR"]], "InlineFormula"],
 " (see ",
 ButtonBox["orientation sketch",
  BaseStyle->"Hyperlink",
  ButtonData->{"IntroductionToInfluenceCoefficients.nb", "local"}],
 ").  Computation of all influence coefficients depends on the displacement \
vectors locating the field point (at which the quantity is calculated) with \
respect to panel endpoints.  For all the computations made in this section, \
the field point is specified as an arbitrary location ",
 Cell[BoxData[
  RowBox[{"(", 
   RowBox[{
    StyleBox["x", "TI"], ",", 
    StyleBox["z", "TI"]}], ")"}]], "InlineFormula"],
 ", so that results are functions of ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  StyleBox["z", "TI"]], "InlineFormula"],
 "."
}], "Text",
 CellID->105339135],

Cell[TextData[{
 "Use the constructor (",
 Cell[BoxData[
  ButtonBox["MakeCartesianVectors",
   BaseStyle->"Link",
   ButtonData->"paclet:Aerodynamics/ref/MakeCartesianVectors"]], 
  "InlineFormula"],
 ") for the data type CartesianVectors to specify panel endpoints as a \
collection of two vectors."
}], "Text",
 CellID->23382165],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rP", "=", 
  RowBox[{"MakeCartesianVectors", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "\[Delta]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], "}"}], "]"}]}]], "Input",
 CellLabel->"In[1]:=",
 CellID->434627284],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "\[Delta]"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"0", ",", "0"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{
    0, $CellContext`\[Delta]}, {0, 0}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{91, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->17701381]
}, Open  ]],

Cell["\<\
Specify the field point as a collection of a single vector of the data type \
CartesianVectors.\
\>", "Text",
 CellID->407489582],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rF", "=", 
  RowBox[{"MakeCartesianVectors", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "x", "}"}], ",", 
     RowBox[{"{", "z", "}"}]}], "}"}], "]"}]}]], "Input",
 CellLabel->"In[2]:=",
 CellID->9791574],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", "x", "}"}], "\[UpArrow]", 
   RowBox[{"{", "z", "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{$CellContext`x}, \
{$CellContext`z}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{55, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->366246896]
}, Open  ]],

Cell[CellGroupData[{

Cell["Constant-strength source panel", "Subsection",
 CellID->22443959],

Cell[TextData[{
 "Consider a source panel of constant strength, ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Sigma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   SubscriptBox[
    StyleBox["a", "TI"], 
    StyleBox["\[Sigma]", "TR"]]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Sigma]", "TR"], 
   RowBox[{"(", 
    StyleBox["x", "TI"], ")"}]}]], "InlineFormula"],
 " is the source-density function, and the constant ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["\[Sigma]", "TR"]]], "InlineFormula"],
 " is the strength parameter.  The velocity, velocity potential and stream \
function can be written as the product of the strength parameter and a \
geometric influence coefficient."
}], "Text",
 CellID->470290831],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["u", "TI"], 
    StyleBox["\[Sigma]", "TR"]], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["\[Sigma]", "TR"]], " ", 
    SubscriptBox[
     StyleBox["icu", "TI"], 
     StyleBox["\[Sigma]", "TR"]]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["w", "TI"], 
    StyleBox["\[Sigma]", "TR"]], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["\[Sigma]", "TR"]], " ", 
    SubscriptBox[
     StyleBox["icw", "TI"], 
     StyleBox["\[Sigma]", "TR"]]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["\[Phi]", "TR"], 
    StyleBox["\[Sigma]", "TR"]], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["\[Sigma]", "TR"]], " ", 
    SubscriptBox[
     StyleBox["ic\[Phi]", "TI"], 
     StyleBox["\[Sigma]", "TR"]]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["\[Psi]", "TR"], 
    StyleBox["\[Sigma]", "TR"]], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["\[Sigma]", "TR"]], " ", 
    SubscriptBox[
     StyleBox["ic\[Psi]", "TI"], 
     StyleBox["\[Sigma]", "TR"]]}]}]], "InlineFormula"]
}], "Text",
 CellID->120763874],

Cell[BoxData[GridBox[{
   {
    ButtonBox["ICSourceConstant",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ICSourceConstant"], Cell[
    "velocity influence coefficient", "TableText"]},
   {
    ButtonBox["IC\[Phi]SourceConstant",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ICSourceConstant"], Cell[
    "velocity-potential influence coefficient", "TableText"]},
   {
    ButtonBox["IC\[Psi]SourceConstant",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ICSourceConstant"], Cell[
    "stream-function influence coefficient", "TableText"]}
  }]], "DefinitionBox",
 CellID->80874091],

Cell["\<\
Influence coefficients for source panels of constant strength.\
\>", "Caption",
 CellID->84822298],

Cell["\<\
Compute the velocity influence coefficient.  The result is expressed as a \
collection of one vector.\
\>", "Text",
 CellID->49995632],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ic\[Sigma]", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"ICSourceConstant", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}], ",", 
    RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}]], "Input",
 CellLabel->"In[3]:=",
 CellID->137123347],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"Log", "[", 
        FractionBox[
         RowBox[{
          SuperscriptBox["z", "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["z", "2"]}]], "]"}], 
       RowBox[{"4", " ", "\[Pi]"}]]}], "}"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"{", 
     FractionBox[
      RowBox[{
       RowBox[{"-", 
        RowBox[{"ArcTan", "[", 
         RowBox[{"x", ",", "z"}], "]"}]}], "+", 
       RowBox[{"ArcTan", "[", 
        RowBox[{
         RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}]}], 
      RowBox[{"2", " ", "\[Pi]"}]], "}"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{{
     Rational[-1, 4] Pi^(-1) 
      Log[($CellContext`x^2 + $CellContext`z^2)^(-1) ($CellContext`z^2 + \
($CellContext`x - $CellContext`\[Delta])^2)]}}, {{
     Rational[1, 2] 
      Pi^(-1) (-ArcTan[$CellContext`x, $CellContext`z] + 
       ArcTan[$CellContext`x - $CellContext`\[Delta], $CellContext`z])}}], 
   "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{375, 59},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->8980686]
}, Open  ]],

Cell["\<\
Write the two components of the vector influence coefficient, removing \
extraneous brackets.\
\>", "Text",
 CellID->494657454],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu\[Sigma]", ",", "icw\[Sigma]"}], "}"}], "=", 
  RowBox[{"Flatten", "[", 
   RowBox[{"Components", "[", "ic\[Sigma]", "]"}], "]"}]}]], "Input",
 CellLabel->"In[4]:=",
 CellID->176773044],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{
        SuperscriptBox["z", "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}]], "]"}], 
     RowBox[{"4", " ", "\[Pi]"}]]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}]}], "+", 
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}]}], 
    RowBox[{"2", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{335, 47},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->10269969]
}, Open  ]],

Cell[TextData[{
 "To further simplify the appearance of the expression, replace the ArcTan \
functions with the angles shown in the ",
 ButtonBox["orientation sketch",
  BaseStyle->"Hyperlink",
  ButtonData->{"IntroductionToInfluenceCoefficients.nb", "local"}],
 ". "
}], "Text",
 CellID->16489213],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu\[Sigma]", ",", "icw\[Sigma]"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]], "Input",
 CellLabel->"In[5]:=",
 CellID->703191196],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"Log", "[", 
      FractionBox[
       RowBox[{
        SuperscriptBox["z", "2"], "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
       RowBox[{
        SuperscriptBox["x", "2"], "+", 
        SuperscriptBox["z", "2"]}]], "]"}], 
     RowBox[{"4", " ", "\[Pi]"}]]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], 
    RowBox[{"2", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{181, 47},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->42351100]
}, Open  ]],

Cell["\<\
Compute the velocity-potential influence coefficient, eliminating extraneous \
list brackets, and replacing ArcTan functions with named angles.\
\>", "Text",
 CellID->79030308],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", "ic\[Phi]\[Sigma]", "}"}], "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"IC\[Phi]SourceConstant", "[", 
       RowBox[{"rF", ",", "rP"}], "]"}], ",", 
      RowBox[{"\[Delta]", ">", "0"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{"ic\[Phi]\[Sigma]", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{
        RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
      "\[Theta]2"}]}], "}"}]}], "]"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->493312995],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"2", " ", "z", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["z", "2"]}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"Log", "[", 
     RowBox[{
      SuperscriptBox["z", "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], "]"}]}]}], 
  RowBox[{"4", " ", "\[Pi]"}]]], "Output",
 ImageSize->{362, 37},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[7]=",
 CellID->395880335]
}, Open  ]],

Cell["Compute the stream-function influence coefficient", "Text",
 CellID->283412909],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", "ic\[Psi]\[Sigma]", "}"}], "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"IC\[Psi]SourceConstant", "[", 
       RowBox[{"rF", ",", "rP"}], "]"}], ",", 
      RowBox[{"\[Delta]", ">", "0"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{"ic\[Psi]\[Sigma]", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{
        RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
      "\[Theta]2"}]}], "}"}]}], "]"}]}], "Input",
 CellLabel->"In[8]:=",
 CellID->8664355],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"2", " ", "x", " ", "\[Theta]1"}], "-", 
   RowBox[{"2", " ", "x", " ", "\[Theta]2"}], "+", 
   RowBox[{"2", " ", "\[Delta]", " ", "\[Theta]2"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"Log", "[", 
     FractionBox[
      RowBox[{
       SuperscriptBox["z", "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}]], "]"}]}]}], 
  RowBox[{"4", " ", "\[Pi]"}]]], "Output",
 ImageSize->{246, 47},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[9]=",
 CellID->597020581]
}, Open  ]],

Cell["\<\
Velocity, velocity potential and stream function are obtained by multiplying \
each influence coefficient by the strength parameter. \
\>", "Text",
 CellID->236352304],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "u\[Sigma]", ",", "w\[Sigma]", ",", "\[Phi]\[Sigma]", ",", 
     "\[Psi]\[Sigma]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a\[Sigma]", " ", "icu\[Sigma]"}], ",", 
     RowBox[{"a\[Sigma]", " ", "icw\[Sigma]"}], ",", 
     RowBox[{"a\[Sigma]", " ", "ic\[Phi]\[Sigma]"}], ",", 
     RowBox[{"a\[Sigma]", " ", "ic\[Psi]\[Sigma]"}]}], "}"}]}], 
  ";"}]], "Input",
 CellLabel->"In[10]:=",
 CellID->45587925]
}, Open  ]],

Cell[CellGroupData[{

Cell["Constant-strength vortex panel", "Subsection",
 CellID->177267806],

Cell[TextData[{
 "Consider a vortex panel of constant strength, ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"\[Gamma]", 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   SubscriptBox[
    StyleBox["a", "TI"], "\[Gamma]"]}]], "InlineFormula"],
 ", where ",
 Cell[BoxData[
  RowBox[{"\[Gamma]", 
   RowBox[{"(", 
    StyleBox["x", "TI"], ")"}]}]], "InlineFormula"],
 " is the circulation-density function, and the constant ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], "\[Gamma]"]], "InlineFormula"],
 " is the strength parameter.  The velocity, velocity potential and stream \
function can be written as the product of the strength parameter and a \
geometric influence coefficient."
}], "Text",
 CellID->52645281],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["u", "TI"], "\[Gamma]"], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], "\[Gamma]"], " ", 
    SubscriptBox[
     StyleBox["icu", "TI"], "\[Gamma]"]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["w", "TI"], "\[Gamma]"], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], "\[Gamma]"], " ", 
    SubscriptBox[
     StyleBox["icw", "TI"], "\[Gamma]"]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["\[Phi]", "TR"], "\[Gamma]"], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], "\[Gamma]"], " ", 
    SubscriptBox[
     StyleBox["ic\[Phi]", "TI"], "\[Gamma]"]}]}]], "InlineFormula"],
 "\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["\[Psi]", "TR"], "\[Gamma]"], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], "\[Gamma]"], " ", 
    SubscriptBox[
     StyleBox["ic\[Psi]", "TI"], "\[Gamma]"]}]}]], "InlineFormula"]
}], "Text",
 CellID->1936554],

Cell[BoxData[GridBox[{
   {
    ButtonBox["ICVortexConstant",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ICVortexConstant"], Cell[
    "velocity influence coefficient", "TableText"]},
   {
    ButtonBox["IC\[Phi]VortexConstant",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ICVortexConstant"], Cell[
    "velocity-potential influence coefficient", "TableText"]},
   {
    ButtonBox["IC\[Psi]VortexConstant",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ICVortexConstant"], Cell[
    "stream-function influence coefficient", "TableText"]}
  }]], "DefinitionBox",
 CellID->206350477],

Cell["\<\
Influence coefficients for vortex panels of constant strength.\
\>", "Caption",
 CellID->62955998],

Cell["\<\
Using appropriate functions for influence coefficients and the \
simplifications used in the section on source panels, compute the components \
of velocity influence coefficients.\
\>", "Text",
 CellID->107133949],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icu\[Gamma]", ",", "icw\[Gamma]"}], "}"}], "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Components", "[", 
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{"ICVortexConstant", "[", 
        RowBox[{"rF", ",", "rP"}], "]"}], ",", 
       RowBox[{"\[Delta]", ">", "0"}]}], "]"}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icu\[Gamma]", ",", "icw\[Gamma]"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
    RowBox[{
     RowBox[{"ArcTan", "[", 
      RowBox[{
       RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
     "\[Theta]2"}]}], "}"}]}]}], "Input",
 CellLabel->"In[11]:=",
 CellID->334029669],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], 
    RowBox[{"2", " ", "\[Pi]"}]], ",", 
   FractionBox[
    RowBox[{"Log", "[", 
     FractionBox[
      RowBox[{
       SuperscriptBox["z", "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}]], "]"}], 
    RowBox[{"4", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{173, 47},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[12]=",
 CellID->142034005]
}, Open  ]],

Cell["\<\
Compute the velocity-potential and stream-function influence coefficients.\
\>", "Text",
 CellID->177390599],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", "ic\[Phi]\[Gamma]", "}"}], "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"IC\[Phi]VortexConstant", "[", 
       RowBox[{"rF", ",", "rP"}], "]"}], ",", 
      RowBox[{"\[Delta]", ">", "0"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{"ic\[Phi]\[Gamma]", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{
        RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
      "\[Theta]2"}]}], "}"}]}], "]"}]}], "Input",
 CellLabel->"In[13]:=",
 CellID->19476998],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "2"}], " ", "x", " ", "\[Theta]1"}], "+", 
   RowBox[{"2", " ", "x", " ", "\[Theta]2"}], "-", 
   RowBox[{"2", " ", "\[Delta]", " ", "\[Theta]2"}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"Log", "[", 
     FractionBox[
      RowBox[{
       SuperscriptBox["z", "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}]], "]"}]}]}], 
  RowBox[{"4", " ", "\[Pi]"}]]], "Output",
 ImageSize->{254, 47},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[14]=",
 CellID->74949219]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", "ic\[Psi]\[Gamma]", "}"}], "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"IC\[Psi]VortexConstant", "[", 
       RowBox[{"rF", ",", "rP"}], "]"}], ",", 
      RowBox[{"\[Delta]", ">", "0"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{"ic\[Psi]\[Gamma]", "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{
        RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
      "\[Theta]2"}]}], "}"}]}], "]"}]}], "Input",
 CellLabel->"In[15]:=",
 CellID->4731421],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"2", " ", "z", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{
      SuperscriptBox["x", "2"], "+", 
      SuperscriptBox["z", "2"]}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}], " ", 
    RowBox[{"Log", "[", 
     RowBox[{
      SuperscriptBox["z", "2"], "+", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], "]"}]}]}], 
  RowBox[{"4", " ", "\[Pi]"}]]], "Output",
 ImageSize->{362, 37},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[16]=",
 CellID->470869661]
}, Open  ]],

Cell["\<\
Velocity, velocity potential and stream function are obtained by multiplying \
each influence coefficient by the strength parameter. \
\>", "Text",
 CellID->426238386],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "u\[Gamma]", ",", "w\[Gamma]", ",", "\[Phi]\[Gamma]", ",", 
     "\[Psi]\[Gamma]"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a\[Gamma]", " ", "icu\[Gamma]"}], ",", 
     RowBox[{"a\[Gamma]", " ", "icw\[Gamma]"}], ",", 
     RowBox[{"a\[Gamma]", " ", "ic\[Phi]\[Gamma]"}], ",", 
     RowBox[{"a\[Gamma]", " ", "ic\[Psi]\[Gamma]"}]}], "}"}]}], 
  ";"}]], "Input",
 CellLabel->"In[17]:=",
 CellID->34838295]
}, Open  ]],

Cell[CellGroupData[{

Cell["Doublet panel of linearly varying strength", "Subsection",
 CellID->72624876],

Cell[TextData[{
 "Finally, consider a doublet panel of linearly varying strength, where the \
doublet density can be written as ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Mu]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{
    SubscriptBox[
     StyleBox["a", "TI"], 
     StyleBox["\[Mu]", "TR"]], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["\[Mu]", "TR"]], " ", 
     StyleBox["x", "TI"]}]}]}]], "InlineFormula"],
 ".  The constants ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["a", "TI"], 
   StyleBox["\[Mu]", "TR"]]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  SubscriptBox[
   StyleBox["b", "TI"], 
   StyleBox["\[Mu]", "TR"]]], "InlineFormula"],
 " are the strength parameters for the panel.  Velocity, velocity potential \
and stream function can be written as the product of a geometric influence \
coefficient and the appropriate strength parameter."
}], "Text",
 CellID->717906090],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["u", "TI"], 
    StyleBox["\[Mu]", "TR"]], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["\[Mu]", "TR"]], " ", 
     SubscriptBox[
      StyleBox["icua", "TI"], 
      StyleBox["\[Mu]", "TR"]]}], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["\[Mu]", "TR"]], " ", 
     SubscriptBox[
      StyleBox["icub", "TI"], 
      StyleBox["\[Mu]", "TR"]]}]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["w", "TI"], 
    StyleBox["\[Mu]", "TR"]], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["\[Mu]", "TR"]], " ", 
     SubscriptBox[
      StyleBox["icwa", "TI"], 
      StyleBox["\[Mu]", "TR"]]}], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["\[Mu]", "TR"]], " ", 
     SubscriptBox[
      StyleBox["icwb", "TI"], 
      StyleBox["\[Mu]", "TR"]]}]}]}]], "InlineFormula"]
}], "Text",
 CellID->267001894],

Cell[TextData[{
 "\t\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["\[Phi]", "TR"], 
    StyleBox["\[Mu]", "TR"]], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["\[Mu]", "TR"]], " ", 
     SubscriptBox[
      StyleBox["ic\[Phi]a", "TI"], 
      StyleBox["\[Mu]", "TR"]]}], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["\[Mu]", "TR"]], " ", 
     SubscriptBox[
      StyleBox["ic\[Phi]b", "TI"], 
      StyleBox["\[Mu]", "TR"]]}]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["\[Psi]", "TR"], 
    StyleBox["\[Mu]", "TR"]], "=", 
   RowBox[{
    RowBox[{
     SubscriptBox[
      StyleBox["a", "TI"], 
      StyleBox["\[Mu]", "TR"]], " ", 
     SubscriptBox[
      StyleBox["ic\[Psi]a", "TI"], 
      StyleBox["\[Mu]", "TR"]]}], "+", 
    RowBox[{
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["\[Mu]", "TR"]], " ", 
     SubscriptBox[
      StyleBox["ic\[Psi]b", "TI"], 
      StyleBox["\[Mu]", "TR"]]}]}]}]], "InlineFormula"],
 "\t\t"
}], "Text",
 CellID->654610829],

Cell[BoxData[GridBox[{
   {
    ButtonBox["ICDoubletLinear",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ICDoubletLinear"], Cell[
    "velocity influence coefficient", "TableText"]},
   {
    ButtonBox["IC\[Phi]DoubletLinear",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ICDoubletLinear"], Cell[
    "velocity-potential influence coefficient", "TableText"]},
   {
    ButtonBox["IC\[Psi]DoubletLinear",
     BaseStyle->"Link",
     ButtonData->"paclet:Aerodynamics/ref/ICDoubletLinear"], Cell[
    "stream-function influence coefficient", "TableText"]}
  }]], "DefinitionBox",
 CellID->1009440973],

Cell["\<\
Influence coefficients for doublet panels of linearly varying strength.\
\>", "Caption",
 CellID->98129465],

Cell["\<\
Compute velocity influence coefficients.  Note that there are separate \
influence coefficients for the constant and linear terms in the doublet \
density function.\
\>", "Text",
 CellID->218315512],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"ica\[Mu]", ",", "icb\[Mu]"}], "}"}], "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"ICDoubletLinear", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}], ",", 
    RowBox[{"\[Delta]", ">", "0"}]}], "]"}]}]], "Input",
 CellLabel->"In[18]:=",
 CellID->73802754],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   InterpretationBox[
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       FractionBox[
        RowBox[{"z", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"2", " ", "x"}], "-", "\[Delta]"}], ")"}], " ", 
         "\[Delta]"}], 
        RowBox[{"2", " ", "\[Pi]", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["z", "2"]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["z", "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], ")"}]}]], "}"}], 
      "}"}], "\[UpArrow]", 
     RowBox[{"{", 
      RowBox[{"{", 
       FractionBox[
        RowBox[{"\[Delta]", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["z", "2"], "+", 
           RowBox[{"x", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}]}]}], ")"}]}], 
        RowBox[{"2", " ", "\[Pi]", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", 
           SuperscriptBox["z", "2"]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["z", "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], ")"}]}]], "}"}], 
      "}"}]}],
    Infix[
     Aerodynamics`CartesianVectors`CartesianVectors[{{
       Rational[1, 2] 
        Pi^(-1) $CellContext`z ($CellContext`x^2 + $CellContext`z^2)^(-1) \
($CellContext`z^2 + ($CellContext`x - $CellContext`\[Delta])^2)^(-1) (
         2 $CellContext`x - $CellContext`\[Delta]) $CellContext`\[Delta]}}, {{
       Rational[1, 2] 
        Pi^(-1) ($CellContext`x^2 + $CellContext`z^2)^(-1) ($CellContext`z^2 + \
($CellContext`x - $CellContext`\[Delta])^2)^(-1) $CellContext`\[Delta] \
($CellContext`z^2 + $CellContext`x (-$CellContext`x + \
$CellContext`\[Delta]))}}], "\[UpArrow]"],
    Editable->False], ",", 
   InterpretationBox[
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       FractionBox[
        RowBox[{
         FractionBox[
          RowBox[{"z", " ", "\[Delta]"}], 
          RowBox[{
           SuperscriptBox["z", "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}]], "+", 
         RowBox[{"ArcTan", "[", 
          RowBox[{"x", ",", "z"}], "]"}], "-", 
         RowBox[{"ArcTan", "[", 
          RowBox[{
           RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}]}], 
        RowBox[{"2", " ", "\[Pi]"}]], "}"}], "}"}], "\[UpArrow]", 
     RowBox[{"{", 
      RowBox[{"{", 
       FractionBox[
        RowBox[{
         FractionBox[
          RowBox[{"\[Delta]", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}]}], 
          RowBox[{
           SuperscriptBox["z", "2"], "+", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}]], "-", 
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{"Log", "[", 
           FractionBox[
            RowBox[{
             SuperscriptBox["z", "2"], "+", 
             SuperscriptBox[
              RowBox[{"(", 
               RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
            RowBox[{
             SuperscriptBox["x", "2"], "+", 
             SuperscriptBox["z", "2"]}]], "]"}]}]}], 
        RowBox[{"2", " ", "\[Pi]"}]], "}"}], "}"}]}],
    Infix[
     Aerodynamics`CartesianVectors`CartesianVectors[{{
       Rational[1, 2] 
        Pi^(-1) ($CellContext`z ($CellContext`z^2 + ($CellContext`x - \
$CellContext`\[Delta])^2)^(-1) $CellContext`\[Delta] + 
         ArcTan[$CellContext`x, $CellContext`z] - 
         ArcTan[$CellContext`x - $CellContext`\[Delta], $CellContext`z])}}, {{
       Rational[1, 2] 
        Pi^(-1) (($CellContext`z^2 + ($CellContext`x - \
$CellContext`\[Delta])^2)^(-1) $CellContext`\[Delta] (-$CellContext`x + \
$CellContext`\[Delta]) + 
         Rational[-1, 2] 
          Log[($CellContext`x^2 + $CellContext`z^2)^(-1) ($CellContext`z^2 + \
($CellContext`x - $CellContext`\[Delta])^2)])}}], "\[UpArrow]"],
    Editable->False]}], "}"}]], "Output",
 ImageSize->{516, 104},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]=",
 CellID->194986753]
}, Open  ]],

Cell["\<\
Write components for the velocity influence coefficients associated with each \
strength parameter.\
\>", "Text",
 CellID->13683548],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"icua\[Mu]", ",", "icwa\[Mu]"}], "}"}], "=", 
  RowBox[{"Flatten", "[", 
   RowBox[{"Components", "[", "ica\[Mu]", "]"}], "]"}]}]], "Input",
 CellLabel->"In[19]:=",
 CellID->758730827],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"z", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "x"}], "-", "\[Delta]"}], ")"}], " ", "\[Delta]"}], 
    RowBox[{"2", " ", "\[Pi]", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["z", "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], ")"}]}]], ",", 
   FractionBox[
    RowBox[{"\[Delta]", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["z", "2"], "+", 
       RowBox[{"x", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}]}]}], ")"}]}], 
    RowBox[{"2", " ", "\[Pi]", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["z", "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], ")"}]}]]}], 
  "}"}]], "Output",
 ImageSize->{364, 41},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[19]=",
 CellID->52973983]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icub\[Mu]", ",", "icwb\[Mu]"}], "}"}], "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Components", "[", "icb\[Mu]", "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icub\[Mu]", ",", "icwb\[Mu]"}], "}"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{
        RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
      "\[Theta]2"}]}], "}"}]}], "]"}]}], "Input",
 CellLabel->"In[20]:=",
 CellID->318956780],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      RowBox[{"z", " ", "\[Delta]"}], 
      RowBox[{
       SuperscriptBox["z", "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}]], "+", "\[Theta]1", 
     "-", "\[Theta]2"}], 
    RowBox[{"2", " ", "\[Pi]"}]], ",", 
   FractionBox[
    RowBox[{
     FractionBox[
      RowBox[{"\[Delta]", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}]}], 
      RowBox[{
       SuperscriptBox["z", "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}]], "-", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         SuperscriptBox["z", "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}]], "]"}]}]}], 
    RowBox[{"2", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{307, 48},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[21]=",
 CellID->2938375]
}, Open  ]],

Cell["\<\
Compute the velocity-potential and stream-function influence coefficients.\
\>", "Text",
 CellID->402639517],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]a\[Mu]", ",", "ic\[Phi]b\[Mu]"}], "}"}], "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"IC\[Phi]DoubletLinear", "[", 
       RowBox[{"rF", ",", "rP"}], "]"}], ",", 
      RowBox[{"\[Delta]", ">", "0"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]a\[Mu]", ",", "ic\[Phi]b\[Mu]"}], "}"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{
        RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
      "\[Theta]2"}]}], "}"}]}], "]"}]}], "Input",
 CellLabel->"In[22]:=",
 CellID->68692785],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"\[Theta]1", "-", "\[Theta]2"}], 
    RowBox[{"2", " ", "\[Pi]"}]], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"2", " ", "x", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "\[Theta]1"}], "+", "\[Theta]2"}], ")"}]}], "+", 
      RowBox[{"z", " ", 
       RowBox[{"Log", "[", 
        FractionBox[
         RowBox[{
          SuperscriptBox["z", "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
         RowBox[{
          SuperscriptBox["x", "2"], "+", 
          SuperscriptBox["z", "2"]}]], "]"}]}]}], 
     RowBox[{"4", " ", "\[Pi]"}]]}]}], "}"}]], "Output",
 ImageSize->{279, 47},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[23]=",
 CellID->6140919]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]a\[Mu]", ",", "ic\[Psi]b\[Mu]"}], "}"}], "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"IC\[Psi]DoubletLinear", "[", 
       RowBox[{"rF", ",", "rP"}], "]"}], ",", 
      RowBox[{"\[Delta]", ">", "0"}]}], "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Psi]a\[Mu]", ",", "ic\[Psi]b\[Mu]"}], "}"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{"x", ",", "z"}], "]"}], "\[Rule]", "\[Theta]1"}], ",", 
     RowBox[{
      RowBox[{"ArcTan", "[", 
       RowBox[{
        RowBox[{"x", "-", "\[Delta]"}], ",", "z"}], "]"}], "\[Rule]", 
      "\[Theta]2"}]}], "}"}]}], "]"}]}], "Input",
 CellLabel->"In[24]:=",
 CellID->17690965],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     FractionBox[
      RowBox[{
       SuperscriptBox["z", "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
      RowBox[{
       SuperscriptBox["x", "2"], "+", 
       SuperscriptBox["z", "2"]}]], "]"}], 
    RowBox[{"4", " ", "\[Pi]"}]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{"\[Delta]", "+", 
        RowBox[{"z", " ", 
         RowBox[{"(", 
          RowBox[{"\[Theta]1", "-", "\[Theta]2"}], ")"}]}]}], ")"}]}], "+", 
     RowBox[{"x", " ", 
      RowBox[{"Log", "[", 
       FractionBox[
        RowBox[{
         SuperscriptBox["z", "2"], "+", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"x", "-", "\[Delta]"}], ")"}], "2"]}], 
        RowBox[{
         SuperscriptBox["x", "2"], "+", 
         SuperscriptBox["z", "2"]}]], "]"}]}]}], 
    RowBox[{"4", " ", "\[Pi]"}]]}], "}"}]], "Output",
 ImageSize->{342, 47},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[25]=",
 CellID->905109567]
}, Open  ]],

Cell["\<\
Velocity, velocity potential and stream function are obtained by multiplying \
each influence coefficient by the appropriate strength parameter and summing \
the contributions of the constant and linear terms. \
\>", "Text",
 CellID->332111165],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u\[Mu]", ",", "w\[Mu]", ",", "\[Phi]\[Mu]", ",", "\[Psi]\[Mu]"}],
     "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"a\[Mu]", " ", "icua\[Mu]"}], "+", 
      RowBox[{"b\[Mu]", " ", "icub\[Mu]"}]}], ",", 
     RowBox[{
      RowBox[{"a\[Mu]", " ", "icwa\[Mu]"}], "+", 
      RowBox[{"b\[Mu]", " ", "icwb\[Mu]"}]}], ",", 
     RowBox[{
      RowBox[{"a\[Mu]", " ", "ic\[Phi]a\[Mu]"}], "+", 
      RowBox[{"b\[Mu]", " ", "ic\[Phi]b\[Mu]"}]}], ",", 
     RowBox[{
      RowBox[{"a\[Mu]", " ", "ic\[Psi]a\[Mu]"}], "+", 
      RowBox[{"b\[Mu]", " ", "ic\[Psi]b\[Mu]"}]}]}], "}"}]}], ";"}]], "Input",
 CellLabel->"In[26]:=",
 CellID->124787270]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.3.2  Velocity from velocity potential or stream function", "Section",
 CellID->29918537],

Cell["\<\
There are relations between velocity, velocity potential and stream function \
for potential flow.\t\t\t\t\t\
\>", "Text",
 CellID->363098241],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["u", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["x", "TI"]}]], "=", 
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Psi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t",
 Cell[BoxData[
  RowBox[{
   StyleBox["w", "TI"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"\[PartialD]", 
      StyleBox["\[Phi]", "TR"]}], 
     RowBox[{"\[PartialD]", 
      StyleBox["z", "TI"]}]], "=", 
    RowBox[{"-", 
     FractionBox[
      RowBox[{"\[PartialD]", 
       StyleBox["\[Psi]", "TR"]}], 
      RowBox[{"\[PartialD]", 
       StyleBox["x", "TI"]}]]}]}]}]], "InlineFormula"]
}], "Text",
 CellID->26305678],

Cell[TextData[{
 "Demonstrate that the defined function for influence coefficients in ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " satisfy these conditions for the three isolated panels used as examples.  \
Such checks contribute to confidence in the functions as programmed.  In some \
cases it is necessary to state explicitly properties of the variables."
}], "Text",
 CellID->12274103],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{
      RowBox[{"u\[Sigma]", "-", 
       RowBox[{"D", "[", 
        RowBox[{"\[Phi]\[Sigma]", ",", "x"}], "]"}]}], ",", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "z"}], "}"}], "\[Element]", "Reals"}]}], "]"}], ",", 
    RowBox[{"Simplify", "[", 
     RowBox[{"w\[Sigma]", "-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]\[Sigma]", ",", "z"}], "]"}]}], "]"}]}], "}"}], 
  "\[Equal]", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0"}], "}"}]}]], "Input",
 CellLabel->"In[27]:=",
 CellID->60683208],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[27]=",
 CellID->263539492]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{"u\[Sigma]", "-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]\[Sigma]", ",", "z"}], "]"}]}], "]"}], ",", 
    RowBox[{"Simplify", "[", 
     RowBox[{"w\[Sigma]", "+", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]\[Sigma]", ",", "x"}], "]"}]}], "]"}]}], "}"}], 
  "\[Equal]", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0"}], "}"}]}]], "Input",
 CellLabel->"In[28]:=",
 CellID->243727406],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[28]=",
 CellID->346632648]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{"u\[Gamma]", "-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]\[Gamma]", ",", "x"}], "]"}]}], "]"}], ",", 
    RowBox[{"Simplify", "[", 
     RowBox[{"w\[Gamma]", "-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]\[Gamma]", ",", "z"}], "]"}]}], "]"}]}], "}"}], 
  "\[Equal]", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0"}], "}"}]}]], "Input",
 CellLabel->"In[29]:=",
 CellID->98739334],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[29]=",
 CellID->73869383]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{"u\[Gamma]", "-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]\[Gamma]", ",", "z"}], "]"}]}], "]"}], ",", 
    RowBox[{"Simplify", "[", 
     RowBox[{
      RowBox[{"w\[Gamma]", "+", 
       RowBox[{"D", "[", 
        RowBox[{"\[Psi]\[Gamma]", ",", "x"}], "]"}]}], ",", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"x", ",", "z"}], "}"}], "\[Element]", "Reals"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"\[Delta]", ">", "0"}]}]}], "]"}]}], "}"}], "\[Equal]", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0"}], "}"}]}]], "Input",
 CellLabel->"In[30]:=",
 CellID->17935659],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[30]=",
 CellID->150797085]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{"u\[Mu]", "-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]\[Mu]", ",", "x"}], "]"}]}], "]"}], ",", 
    RowBox[{"Simplify", "[", 
     RowBox[{"w\[Mu]", "-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Phi]\[Mu]", ",", "z"}], "]"}]}], "]"}]}], "}"}], "\[Equal]", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0"}], "}"}]}]], "Input",
 CellLabel->"In[31]:=",
 CellID->756290509],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[31]=",
 CellID->296807279]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Simplify", "[", 
     RowBox[{"u\[Mu]", "-", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]\[Mu]", ",", "z"}], "]"}]}], "]"}], ",", 
    RowBox[{"Simplify", "[", 
     RowBox[{"w\[Mu]", "+", 
      RowBox[{"D", "[", 
       RowBox[{"\[Psi]\[Mu]", ",", "x"}], "]"}]}], "]"}]}], "}"}], "\[Equal]", 
  RowBox[{"{", 
   RowBox[{"0", ",", "0"}], "}"}]}]], "Input",
 CellLabel->"In[32]:=",
 CellID->512433514],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[32]=",
 CellID->30177663]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.3.3  Flow visualization", "Section",
 CellID->34893618],

Cell["\<\
Flow visualization provides valuable checks on plausibility of the computed \
flow field.  It also can show features that should be studied in more detail. \
 In the figures below, the velocity field, equipotential lines and \
streamlines are overlaid to illustrate the flow associated with each panel.\
\>", "Text",
 CellID->33633243],

Cell["\<\
Arrows for the velocity field are green, equipotential lines are red, \
streamlines are blue and the panel is shown in black.  Equipotential lines \
and streamlines should intersect perpendicularly, and streamlines should be \
everywhere parallel to the velocity field.\
\>", "Text",
 CellID->680409942],

Cell[CellGroupData[{

Cell["Constant-strength source panel", "Subsection",
 CellID->147316578],

Cell[TextData[{
 "Set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Delta]", "TR"], "=", "1"}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["a", "TI"], 
    StyleBox["\[Sigma]", "TR"]], "=", "1"}]], "InlineFormula"],
 " to provide unit integrated panel strength, ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     StyleBox["\[Delta]", "TR"]], 
    RowBox[{
     StyleBox["\[Sigma]", "TR"], 
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]}]}], "=", "1"}]], "InlineFormula"],
 "."
}], "Text",
 CellID->4981250],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xMin", ",", "xMax", ",", "zMin", ",", "zMax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "2", ",", 
     RowBox[{"-", "1.2"}], ",", "1.2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u\[Sigma]Plot", ",", "w\[Sigma]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u\[Sigma]", ",", "w\[Sigma]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a\[Sigma]", "\[Rule]", "1."}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1."}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]\[Sigma]Plot", ",", "\[Psi]\[Sigma]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]\[Sigma]", ",", "\[Psi]\[Sigma]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a\[Sigma]", "\[Rule]", "1."}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1."}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV\[Sigma]", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"u\[Sigma]Plot", ",", "w\[Sigma]Plot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{
         RowBox[{"0.005", "<", 
          RowBox[{
           RowBox[{"x", "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}], " ", "&&", " ", 
         RowBox[{"0.005", "<", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"x", "-", "1"}], ")"}], "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"18", ",", "18"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", "Medium"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]\[Sigma]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]\[Sigma]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]\[Sigma]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]\[Sigma]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanel", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
      RowBox[{"AbsoluteThickness", "[", "3", "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Components", "[", 
         RowBox[{"rP", "/.", 
          RowBox[{"\[Delta]", "\[Rule]", "1.0"}]}], "]"}], "]"}], "]"}]}], 
     "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{
  "gV\[Sigma]", ",", "g\[Phi]\[Sigma]", ",", "g\[Psi]\[Sigma]", ",", "gPanel",
    ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.0"}], ",", "2.0", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellOpen->False,
 CellLabel->"In[33]:=",
 CellID->608792814],

Cell[BoxData[
 GraphicsBox[{{{}, 
    {Hue[0.67, 0.6, 0.6], 
     {Arrowheads[{{0.009116798619914617, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9807338717514044, -1.1838697719654656`}, \
{-1.0192661282485957`, -1.2161302280345343`}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9788199149169332, -1.0431054477318142`}, \
{-1.0211800850830668`, -1.0745416110917152`}}]}}, 
     {Arrowheads[{{0.01003260438058273, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9767770061484123, -0.9026378443014492}, \
{-1.0232229938515875`, -0.9326562733456096}}]}}, 
     {Arrowheads[{{0.010494535920320246`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9746523699480947, -0.7625382505618518}, \
{-1.0253476300519053`, -0.7904029259087367}}]}}, 
     {Arrowheads[{{0.010939521084517023`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9725254688599791, -0.6228758215402782}, \
{-1.0274745311400209`, -0.6477124137538397}}]}}, 
     {Arrowheads[{{0.011346123674529836`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9705141070200185, -0.4837023461393182}, \
{-1.0294858929799815`, -0.5045329479783293}}]}}, 
     {Arrowheads[{{0.011688143720156748`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9687725946320472, -0.34502979430757774`}, \
{-1.0312274053679529`, -0.3608525586335991}}]}}, 
     {Arrowheads[{{0.011937480299842693`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9674747188902506, -0.20680544893831998`}, \
{-1.0325252811097494`, -0.21672396282638628`}}]}}, 
     {Arrowheads[{{0.01206948107950648, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9667780233212557, -0.06889697276363918}, \
{-1.0332219766787443`, -0.0722794978245965}}]}}, 
     {Arrowheads[{{0.012069481079506476`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9667780233212557, 
        0.06889697276363882}, {-1.0332219766787443`, 0.0722794978245961}}]}}, 
     {Arrowheads[{{0.011937480299842693`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9674747188902506, 
        0.2068054489383196}, {-1.0325252811097494`, 0.2167239628263859}}]}}, 
     {Arrowheads[{{0.011688143720156748`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9687725946320472, 
        0.34502979430757735`}, {-1.0312274053679529`, 0.3608525586335987}}]}}, 
     {Arrowheads[{{0.011346123674529841`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9705141070200185, 
        0.4837023461393178}, {-1.0294858929799815`, 0.5045329479783289}}]}}, 
     {Arrowheads[{{0.010939521084517023`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9725254688599791, 
        0.6228758215402778}, {-1.0274745311400209`, 0.6477124137538393}}]}}, 
     {Arrowheads[{{0.010494535920320246`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9746523699480947, 
        0.7625382505618513}, {-1.0253476300519053`, 0.7904029259087363}}]}}, 
     {Arrowheads[{{0.01003260438058273, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9767770061484123, 
        0.9026378443014489}, {-1.0232229938515875`, 0.9326562733456093}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9788199149169332, 
        1.0431054477318138`}, {-1.0211800850830668`, 1.0745416110917148`}}]}}, 
     {Arrowheads[{{0.009116798619914617, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9807338717514044, 
        1.1838697719654652`}, {-1.0192661282485957`, 1.2161302280345339`}}]}}, 
     {Arrowheads[{{0.0097764739834501, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8040469254264953, -1.181386143062818}, \
{-0.8430118981029164, -1.2186138569371818`}}]}}, 
     {Arrowheads[{{0.010342744314054715`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.801790582402836, -1.0403841856980935`}, \
{-0.8452682411265758, -1.0772628731254361`}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7992987754129584, -0.899712904175814}, \
{-0.8477600481164532, -0.935581213471245}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7966059841827857, -0.7594847871806495}, \
{-0.8504528393466261, -0.793456389289939}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7937938880921884, -0.6198271343117402}, \
{-0.8532649354372233, -0.6507611009823778}}]}}, 
     {Arrowheads[{{0.012740596380756662`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7910124698751128, -0.48086287631464036`}, \
{-0.8560463536542989, -0.5073724178030071}}]}}, 
     {Arrowheads[{{0.013246569582145843`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7884945676311019, -0.3426707593544241}, \
{-0.8585642558983099, -0.3632115935867528}}]}}, 
     {Arrowheads[{{0.0136278498719416, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7865430632708874, -0.20522498643722126`}, \
{-0.8605157602585244, -0.218304425327485}}]}}, 
     {Arrowheads[{{0.013834467068875216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7854663260855013, -0.06833838779139521}, \
{-0.8615924974439104, -0.07283808279684048}}]}}, 
     {Arrowheads[{{0.013834467068875216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7854663260855013, 
        0.06833838779139484}, {-0.8615924974439104, 0.07283808279684006}}]}}, 
     {Arrowheads[{{0.0136278498719416, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7865430632708874, 
        0.20522498643722087`}, {-0.8605157602585244, 
        0.21830442532748462`}}]}}, 
     {Arrowheads[{{0.013246569582145843`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7884945676311019, 
        0.3426707593544237}, {-0.8585642558983099, 0.3632115935867524}}]}}, 
     {Arrowheads[{{0.012740596380756657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7910124698751128, 
        0.48086287631464}, {-0.8560463536542989, 0.5073724178030067}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7937938880921884, 
        0.6198271343117397}, {-0.8532649354372233, 0.6507611009823774}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7966059841827857, 
        0.759484787180649}, {-0.8504528393466261, 0.7934563892899386}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7992987754129584, 
        0.8997129041758135}, {-0.8477600481164532, 0.9355812134712446}}]}}, 
     {Arrowheads[{{0.010342744314054715`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.801790582402836, 
        1.040384185698093}, {-0.8452682411265758, 1.0772628731254357`}}]}}, 
     {Arrowheads[{{0.009776473983450126, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8040469254264953, 
        1.1813861430628176`}, {-0.8430118981029164, 1.2186138569371816`}}]}}, 
     {Arrowheads[{{0.010477306125975746`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6277414377817475, -1.1785359331371488`}, \
{-0.6663762092770759, -1.221464066862851}}]}}, 
     {Arrowheads[{{0.011183628626149165`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.625138516318523, -1.0371535747662564`}, \
{-0.6689791307403005, -1.080493484057273}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6221500641505607, -0.8961012470138565}, \
{-0.6719675829082626, -0.9391928706332024}}]}}, 
     {Arrowheads[{{0.012768289697852271`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6187690551976331, -0.7555398540556062}, \
{-0.6753485918611902, -0.7974013224149822}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6150435971751111, -0.615681833577597}, \
{-0.6790740498837122, -0.654906401716521}}]}}, 
     {Arrowheads[{{0.014475369768919475`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6111259556165297, -0.4767823051369137}, \
{-0.6829916914422935, -0.5114529889807338}}]}}, 
     {Arrowheads[{{0.015263437675282376`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6073372076773217, -0.33908820015721075`}, \
{-0.6867804393815017, -0.36679415278396615`}}]}}, 
     {Arrowheads[{{0.015890646781373582`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6042084973358147, -0.20270973761578687`}, \
{-0.6899091497230088, -0.2208196741489194}}]}}, 
     {Arrowheads[{{0.016244935134227038`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6023976663336901, -0.06742454185633001}, \
{-0.6917199807251333, -0.07375192873190568}}]}}, 
     {Arrowheads[{{0.016244935134227038`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6023976663336901, 
        0.06742454185632964}, {-0.6917199807251333, 0.07375192873190527}}]}}, 
     {Arrowheads[{{0.015890646781373603`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6042084973358146, 
        0.20270973761578648`}, {-0.6899091497230088, 0.220819674148919}}]}}, 
     {Arrowheads[{{0.015263437675282376`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6073372076773217, 
        0.33908820015721036`}, {-0.6867804393815017, 
        0.36679415278396577`}}]}}, 
     {Arrowheads[{{0.014475369768919472`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6111259556165297, 
        0.47678230513691333`}, {-0.6829916914422935, 0.5114529889807333}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6150435971751111, 
        0.6156818335775965}, {-0.6790740498837122, 0.6549064017165206}}]}}, 
     {Arrowheads[{{0.012768289697852271`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6187690551976331, 
        0.7555398540556058}, {-0.6753485918611902, 0.7974013224149817}}]}}, 
     {Arrowheads[{{0.011949495453044816`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6221500641505607, 
        0.8961012470138561}, {-0.6719675829082628, 0.939192870633202}}]}}, 
     {Arrowheads[{{0.011183628626149165`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.625138516318523, 
        1.037153574766256}, {-0.6689791307403005, 1.0804934840572726`}}]}}, 
     {Arrowheads[{{0.010477306125975791`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6277414377817475, 
        1.1785359331371483`}, {-0.666376209277076, 1.2214640668628507`}}]}}, 
     {Arrowheads[{{0.011199963345734225`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.45199034966053925`, -1.175362977900796}, \
{-0.4891861209276958, -1.224637022099204}}]}}, 
     {Arrowheads[{{0.012072512361463282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4490964672384976, -1.033422350108421}, \
{-0.49208000334973745`, -1.0842247087151085`}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4456318649600362, -0.8917404906758423}, \
{-0.49554460562819885`, -0.9435536269712167}}]}}, 
     {Arrowheads[{{0.014144384224217975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4415037498304014, -0.7505125291145733}, \
{-0.49967272075783364`, -0.8024286473560152}}]}}, 
     {Arrowheads[{{0.015348449856734211`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4366504558557032, -0.6100412537619427}, \
{-0.5045260147325319, -0.6605469815321754}}]}}, 
     {Arrowheads[{{0.01663799156292184, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4311153562503576, -0.4707788430506923}, \
{-0.5100611143378775, -0.517456451066955}}]}}, 
     {Arrowheads[{{0.017936440162617983`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.42520714755944095`, -0.33333548645830935`}, \
{-0.5159693230287942, -0.3725468664828675}}]}}, 
     {Arrowheads[{{0.019074142779961962`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4197667974791728, -0.1983162046793514}, \
{-0.5214096731090623, -0.22521320708535486`}}]}}, 
     {Arrowheads[{{0.019772769634296784`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.41630829113565243`, -0.0657378637744822}, \
{-0.5248681794525826, -0.07543860681375349}}]}}, 
     {Arrowheads[{{0.019772769634296784`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.41630829113565243`, 
        0.06573786377448182}, {-0.5248681794525826, 0.07543860681375308}}]}}, 
     {Arrowheads[{{0.019074142779961962`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4197667974791728, 
        0.19831620467935102`}, {-0.5214096731090623, 
        0.22521320708535447`}}]}}, 
     {Arrowheads[{{0.017936440162617983`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.42520714755944095`, 
        0.33333548645830896`}, {-0.5159693230287942, 0.3725468664828671}}]}}, 
     {Arrowheads[{{0.016637991562921877`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4311153562503575, 
        0.4707788430506919}, {-0.5100611143378776, 0.5174564510669548}}]}}, 
     {Arrowheads[{{0.015348449856734196`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4366504558557032, 
        0.6100412537619423}, {-0.5045260147325318, 0.660546981532175}}]}}, 
     {Arrowheads[{{0.014144384224217975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4415037498304014, 
        0.7505125291145729}, {-0.49967272075783364`, 0.8024286473560147}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4456318649600362, 
        0.8917404906758418}, {-0.49554460562819885`, 0.9435536269712163}}]}}, 
     {Arrowheads[{{0.012072512361463311`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4490964672384976, 
        1.0334223501084205`}, {-0.49208000334973745`, 
        1.0842247087151082`}}]}}, 
     {Arrowheads[{{0.011199963345734225`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.45199034966053925`, 
        1.1753629779007955`}, {-0.4891861209276958, 1.2246370220992036`}}]}}, 
     {Arrowheads[{{0.011912824053351102`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2769808901006054, -1.1719937029687708`}, \
{-0.3112544040170414, -1.228006297031229}}]}}, 
     {Arrowheads[{{0.01297064522774105, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.27394540027974335`, -1.0293099497731446`}, \
{-0.31428989383790334`, -1.0883371090503848`}}]}}, 
     {Arrowheads[{{0.014198637077553304`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.27015835485833006`, -0.8867058067067692}, \
{-0.31807693925931674`, -0.9485883109402897}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2654013659875012, -0.744353715163706}, \
{-0.32283392813014555`, -0.8085874613068825}}]}}, 
     {Arrowheads[{{0.017308930966610733`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.25940254153050746`, -0.6025728171064615}, \
{-0.32883275258713923`, -0.6680154181876566}}]}}, 
     {Arrowheads[{{0.0192658123238217, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.25186426676773377`, -0.46195937889901734`}, \
{-0.3363710273499129, -0.5262759152186302}}]}}, 
     {Arrowheads[{{0.021497164399658777`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.242624229607832, -0.32363491874014894`}, \
{-0.3456110645098147, -0.3822474342010279}}]}}, 
     {Arrowheads[{{0.0238257974448833, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2323151984543792, -0.1895702053836755}, \
{-0.35592009566326754`, -0.23395920638103077`}}]}}, 
     {Arrowheads[{{0.025567923001979964`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22418850814946198`, -0.061887105362916216`}, \
{-0.36404678596818474`, -0.07928936522531946}}]}}, 
     {Arrowheads[{{0.025567923001979975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.22418850814946195`, 
        0.06188710536291589}, {-0.3640467859681848, 0.07928936522531901}}]}}, 
     {Arrowheads[{{0.02382579744488332, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.23231519845437915`, 
        0.1895702053836751}, {-0.3559200956632676, 0.23395920638103038`}}]}}, 
     {Arrowheads[{{0.02149716439965878, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.24262422960783198`, 
        0.3236349187401486}, {-0.34561106450981477`, 
        0.38224743420102747`}}]}}, 
     {Arrowheads[{{0.019265812323821702`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.25186426676773377`, 
        0.46195937889901695`}, {-0.336371027349913, 0.5262759152186297}}]}}, 
     {Arrowheads[{{0.01730893096661073, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.25940254153050746`, 
        0.602572817106461}, {-0.3288327525871393, 0.6680154181876561}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2654013659875012, 
        0.7443537151637055}, {-0.32283392813014555`, 0.8085874613068821}}]}}, 
     {Arrowheads[{{0.014198637077553304`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.27015835485833, 
        0.8867058067067688}, {-0.3180769392593167, 0.9485883109402893}}]}}, 
     {Arrowheads[{{0.012970645227741053`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.27394540027974335`, 
        1.0293099497731442`}, {-0.3142898938379034, 1.0883371090503844`}}]}}, 
     {Arrowheads[{{0.011912824053351102`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.27698089010060534`, 
        1.1719937029687704`}, {-0.31125440401704135`, 
        1.2280062970312287`}}]}}, 
     {Arrowheads[{{0.012572289032024985`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.10286886640235525`, -1.16865860643912}, \
{-0.13242525124470325`, -1.2313413935608801`}}]}}, 
     {Arrowheads[{{0.013818604383719657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09995185624301088, -1.025098049253899}, \
{-0.13534226140404762`, -1.0925490095696302`}}]}}, 
     {Arrowheads[{{0.01530879780135258, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09618119847300861, -0.8813226496434575}, \
{-0.1391129191740499, -0.9539714680036014}}]}}, 
     {Arrowheads[{{0.017116898560857404`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09122496683432636, -0.7373876325651155}, \
{-0.14406915081273214`, -0.815553543905473}}]}}, 
     {Arrowheads[{{0.019351722288825795`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08458181304088923, -0.59344446591864}, \
{-0.15071230460616925`, -0.677143769375478}}]}}, 
     {Arrowheads[{{0.022183664080880648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07545659418490722, -0.44986631351751066`}, \
{-0.1598375234621513, -0.5383689806001367}}]}}, 
     {Arrowheads[{{0.025900636613884837`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.06249876026141974, -0.30761415079728877`}, \
{-0.17279535738563873`, -0.3982682021438881}}]}}, 
     {Arrowheads[{{0.03099785349711698, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.04331785477784085, -0.16964427604576143`}, \
{-0.19197626286921765`, -0.2538851357189448}}]}}, 
     {Arrowheads[{{0.03754378372508139, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.016746676591492676`, -0.04764781363693982}, \
{-0.21854744105556578`, -0.09352865695129586}}]}}, 
     {Arrowheads[{{0.0375437837250814, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.016746676591492635`, 
        0.04764781363693952}, {-0.21854744105556584`, 
        0.09352865695129539}}]}}, 
     {Arrowheads[{{0.030997853497116985`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.043317854777840795`, 
        0.1696442760457611}, {-0.1919762628692177, 0.2538851357189444}}]}}, 
     {Arrowheads[{{0.025900636613884854`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.06249876026141969, 
        0.3076141507972884}, {-0.17279535738563878`, 0.3982682021438877}}]}}, 
     {Arrowheads[{{0.02218366408088066, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07545659418490719, 
        0.4498663135175103}, {-0.1598375234621513, 0.5383689806001364}}]}}, 
     {Arrowheads[{{0.019351722288825823`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08458181304088921, 
        0.5934444659186395}, {-0.1507123046061693, 0.6771437693754777}}]}}, 
     {Arrowheads[{{0.01711689856085741, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09122496683432635, 
        0.7373876325651151}, {-0.14406915081273217`, 0.8155535439054725}}]}}, 
     {Arrowheads[{{0.015308797801352584`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.0961811984730086, 
        0.8813226496434571}, {-0.1391129191740499, 0.953971468003601}}]}}, 
     {Arrowheads[{{0.013818604383719655`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09995185624301087, 
        1.0250980492538986`}, {-0.1353422614040476, 1.0925490095696297`}}]}}, 
     {Arrowheads[{{0.012572289032024982`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.10286886640235525`, 
        1.1686586064391196`}, {-0.13242525124470322`, 
        1.2313413935608797`}}]}}, 
     {Arrowheads[{{0.013127617905417667`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.07028607935322899, -1.1656823597169808`}, {
        0.04736097947030078, -1.2343176402830194`}}]}}, 
     {Arrowheads[{{0.014542990019468325`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.07274129182057976, -1.0212351469461067`}, {
        0.044905767002950014`, -1.096411911877423}}]}}, 
     {Arrowheads[{{0.016272107256669006`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.0759968065403877, -0.8762173814295938}, {
        0.041650252283142064`, -0.9590767362174651}}]}}, 
     {Arrowheads[{{0.018427008297575858`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.08040620629958611, -0.7304975373000477}, {
        0.03724085252394366, -0.8224436391705406}}]}}, 
     {Arrowheads[{{0.021183581531526018`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.08652876449451788, -0.5839016937607147}, {
        0.031118294329011895`, -0.6866865415334034}}]}}, 
     {Arrowheads[{{0.0248449375794905, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.09530166230825723, -0.43616702544173586`}, {
        0.022345396515272536`, -0.5520682686759116}}]}}, 
     {Arrowheads[{{0.03001748678854118, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10846384031919165`, -0.2867564785922891}, {
        0.00918321850433812, -0.41912587434888776`}}]}}, 
     {Arrowheads[{{0.038257143909291476`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{
        0.12991060712495614`, -0.13388960786310045`}, \
{-0.012263548301426369`, -0.2896398039016058}}]}}, 
     {Arrowheads[{{0.05575762470829351, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.17077032920291607`, 
        0.0346919338571513}, {-0.053123270379386314`, \
-0.17586840444538698`}}]}}, 
     {Arrowheads[{{0.055757624708293596`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{
        0.17077032920291627`, -0.034691933857151844`}, \
{-0.053123270379386495`, 0.1758684044453867}}]}}, 
     {Arrowheads[{{0.03825714390929151, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.12991060712495622`, 
        0.1338896078631}, {-0.012263548301426452`, 0.2896398039016055}}]}}, 
     {Arrowheads[{{0.030017486788541194`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10846384031919169`, 0.2867564785922887}, {
        0.009183218504338078, 0.4191258743488874}}]}}, 
     {Arrowheads[{{0.0248449375794905, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.09530166230825726, 0.43616702544173547`}, {
        0.022345396515272502`, 0.5520682686759112}}]}}, 
     {Arrowheads[{{0.02118358153152602, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.0865287644945179, 0.5839016937607142}, {
        0.03111829432901187, 0.6866865415334029}}]}}, 
     {Arrowheads[{{0.01842700829757586, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.08040620629958613, 0.7304975373000473}, {
        0.03724085252394364, 0.8224436391705402}}]}}, 
     {Arrowheads[{{0.016272107256669013`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.07599680654038773, 0.8762173814295934}, {
        0.04165025228314204, 0.9590767362174647}}]}}, 
     {Arrowheads[{{0.014542990019468287`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.07274129182057976, 1.0212351469461063`}, {
        0.044905767002950014`, 1.0964119118774223`}}]}}, 
     {Arrowheads[{{0.013127617905417667`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.070286079353229, 1.1656823597169803`}, {
        0.047360979470300775`, 1.234317640283019}}]}}, 
     {Arrowheads[{{0.013529796440564321`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2425771033473928, -1.1634283164953787`}, {
        0.22801113194672526`, -1.2365716835046212`}}]}}, 
     {Arrowheads[{{0.015071701293711279`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2442224795541291, -1.0182548990235811`}, {
        0.22636575573998893`, -1.0993921597999485`}}]}}, 
     {Arrowheads[{{0.016979080664775204`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.24643595345234368`, -0.8721963280379421}, {
        0.22415228184177435`, -0.9630977896091168}}]}}, 
     {Arrowheads[{{0.01938754787014612, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2494747724971715, -0.7249520932131704}, {
        0.2211134627969465, -0.8279890832574179}}]}}, 
     {Arrowheads[{{0.022503892202001684`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.25372673577531346`, -0.5760728545321517}, {
        0.21686149951880457`, -0.6945153807619663}}]}}, 
     {Arrowheads[{{0.026656977789932944`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2597549925736203, -0.424839216587763}, {
        0.21083324272049775`, -0.5633960775298844}}]}}, 
     {Arrowheads[{{0.03239168694035226, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.26825651510001386`, -0.26997370156017286`}, {
        0.20233172019410417`, -0.435908651381004}}]}}, 
     {Arrowheads[{{0.04058780375872305, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2794566302415091, -0.1089859921565938}, {
        0.19113160505260895`, -0.3145434196081125}}]}}, 
     {Arrowheads[{{0.052037501067732916`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.29007140791395924`, 0.06196091506845908}, {
        0.18051682738015876`, -0.20313738565669476`}}]}}, 
     {Arrowheads[{{0.05203750106773292, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.29007140791395924`, -0.06196091506845952}, {
        0.18051682738015878`, 0.20313738565669442`}}]}}, 
     {Arrowheads[{{0.04058780375872307, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2794566302415091, 0.10898599215659333`}, {
        0.19113160505260895`, 0.31454341960811216`}}]}}, 
     {Arrowheads[{{0.032391686940352264`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.26825651510001386`, 0.2699737015601724}, {
        0.20233172019410414`, 0.4359086513810036}}]}}, 
     {Arrowheads[{{0.026656977789932965`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2597549925736203, 0.4248392165877626}, {
        0.21083324272049772`, 0.5633960775298841}}]}}, 
     {Arrowheads[{{0.022503892202001708`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2537267357753135, 0.5760728545321513}, {
        0.2168614995188045, 0.694515380761966}}]}}, 
     {Arrowheads[{{0.01938754787014614, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.24947477249717154`, 0.72495209321317}, {
        0.22111346279694652`, 0.8279890832574176}}]}}, 
     {Arrowheads[{{0.016979080664775242`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.24643595345234368`, 0.8721963280379416}, {
        0.22415228184177435`, 0.9630977896091165}}]}}, 
     {Arrowheads[{{0.015071701293711279`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2442224795541291, 1.0182548990235807`}, {
        0.22636575573998893`, 1.099392159799948}}]}}, 
     {Arrowheads[{{0.013529796440564321`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2425771033473928, 1.1634283164953783`}, {
        0.22801113194672526`, 1.2365716835046208`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4142645887464283, -1.1622106908760546`}, {
        0.40926482301827805`, -1.2377893091239456`}}]}}, 
     {Arrowheads[{{0.015350155243130659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41484416434413124`, -1.016628806658473}, {
        0.40868524742057505`, -1.1010182521650567`}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41562844033926905`, -0.8699814753065289}, {
        0.40790097142543724`, -0.96531264234053}}]}}, 
     {Arrowheads[{{0.019888137372837355`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4167088557354353, -0.7218798286880315}, {
        0.40682055602927103`, -0.831061347782557}}]}}, 
     {Arrowheads[{{0.023170576259998205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41821461983240005`, -0.5717596670330394}, {
        0.4053147919323063, -0.6988285682610786}}]}}, 
     {Arrowheads[{{0.02749648274328169, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4203015184883747, -0.41881626272621875`}, {
        0.40322789327633163`, -0.5694190313914287}}]}}, 
     {Arrowheads[{{0.03325362540608419, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4230585005155866, -0.2619885349165438}, {
        0.4004709112491197, -0.44389381802463307`}}]}}, 
     {Arrowheads[{{0.040808824602997874`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42619529480784274`, -0.10022005844338491`}, {
        0.3973341169568635, -0.3233093533213214}}]}}, 
     {Arrowheads[{{0.050108447190220146`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42855347429657387`, 0.0664926446110595}, {
        0.3949759374681324, -0.20766911519929518`}}]}}, 
     {Arrowheads[{{0.05010844719022017, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42855347429657387`, -0.06649264461105994}, {
        0.3949759374681324, 0.20766911519929485`}}]}}, 
     {Arrowheads[{{0.0408088246029979, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42619529480784274`, 0.10022005844338444`}, {
        0.3973341169568635, 0.32330935332132105`}}]}}, 
     {Arrowheads[{{0.0332536254060842, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4230585005155866, 0.26198853491654334`}, {
        0.4004709112491197, 0.4438938180246327}}]}}, 
     {Arrowheads[{{0.0274964827432817, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4203015184883747, 0.41881626272621836`}, {
        0.40322789327633163`, 0.5694190313914284}}]}}, 
     {Arrowheads[{{0.023170576259998216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41821461983240005`, 0.5717596670330389}, {
        0.40531479193230624`, 0.6988285682610783}}]}}, 
     {Arrowheads[{{0.019888137372837393`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4167088557354353, 0.7218798286880309}, {0.406820556029271,
         0.8310613477825567}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41562844033926905`, 0.8699814753065285}, {
        0.40790097142543724`, 0.9653126423405296}}]}}, 
     {Arrowheads[{{0.015350155243130659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.41484416434413124`, 1.0166288066584726`}, {
        0.408685247420575, 1.1010182521650562`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4142645887464283, 1.162210690876054}, {
        0.40926482301827805`, 1.2377893091239451`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5857354112535721, -1.1622106908760546`}, {
        0.5907351769817224, -1.2377893091239456`}}]}}, 
     {Arrowheads[{{0.015350155243130659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5851558356558691, -1.016628806658473}, {
        0.5913147525794255, -1.1010182521650567`}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5843715596607314, -0.8699814753065289}, {
        0.5920990285745632, -0.96531264234053}}]}}, 
     {Arrowheads[{{0.019888137372837372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5832911442645651, -0.7218798286880314}, {
        0.5931794439707294, -0.831061347782557}}]}}, 
     {Arrowheads[{{0.023170576259998205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5817853801676004, -0.5717596670330394}, {
        0.5946852080676942, -0.6988285682610786}}]}}, 
     {Arrowheads[{{0.027496482743281685`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5796984815116257, -0.4188162627262188}, {
        0.5967721067236689, -0.5694190313914287}}]}}, 
     {Arrowheads[{{0.033253625406084196`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5769414994844138, -0.2619885349165438}, {
        0.5995290887508807, -0.44389381802463307`}}]}}, 
     {Arrowheads[{{0.04080882460299787, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5738047051921576, -0.1002200584433849}, {
        0.6026658830431371, -0.3233093533213213}}]}}, 
     {Arrowheads[{{0.050108447190220146`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5714465257034265, 0.0664926446110595}, {
        0.605024062531868, -0.20766911519929518`}}]}}, 
     {Arrowheads[{{0.05010844719022017, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5714465257034265, -0.06649264461105994}, {
        0.605024062531868, 0.20766911519929485`}}]}}, 
     {Arrowheads[{{0.04080882460299789, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5738047051921576, 0.10022005844338447`}, {
        0.6026658830431371, 0.32330935332132105`}}]}}, 
     {Arrowheads[{{0.03325362540608421, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5769414994844138, 0.2619885349165434}, {
        0.5995290887508807, 0.44389381802463274`}}]}}, 
     {Arrowheads[{{0.027496482743281685`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5796984815116257, 0.41881626272621836`}, {
        0.596772106723669, 0.5694190313914282}}]}}, 
     {Arrowheads[{{0.023170576259998205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5817853801676004, 0.5717596670330389}, {
        0.5946852080676942, 0.6988285682610782}}]}}, 
     {Arrowheads[{{0.019888137372837372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5832911442645651, 0.7218798286880309}, {
        0.5931794439707294, 0.8310613477825566}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5843715596607314, 0.8699814753065285}, {
        0.5920990285745632, 0.9653126423405296}}]}}, 
     {Arrowheads[{{0.015350155243130659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5851558356558691, 1.0166288066584726`}, {
        0.5913147525794255, 1.1010182521650562`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5857354112535722, 1.162210690876054}, {0.5907351769817225,
         1.2377893091239451`}}]}}, 
     {Arrowheads[{{0.013529796440564321`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7574228966526076, -1.1634283164953787`}, {
        0.7719888680532752, -1.2365716835046212`}}]}}, 
     {Arrowheads[{{0.015071701293711243`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7557775204458713, -1.0182548990235811`}, {
        0.7736342442600115, -1.0993921597999483`}}]}}, 
     {Arrowheads[{{0.016979080664775204`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7535640465476567, -0.8721963280379421}, {
        0.7758477181582261, -0.9630977896091168}}]}}, 
     {Arrowheads[{{0.01938754787014612, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7505252275028289, -0.7249520932131704}, {
        0.7788865372030539, -0.8279890832574179}}]}}, 
     {Arrowheads[{{0.022503892202001687`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.746273264224687, -0.5760728545321517}, {
        0.7831385004811959, -0.6945153807619663}}]}}, 
     {Arrowheads[{{0.026656977789932933`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7402450074263801, -0.4248392165877631}, {
        0.7891667572795027, -0.5633960775298844}}]}}, 
     {Arrowheads[{{0.03239168694035223, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7317434848999866, -0.2699737015601729}, {
        0.7976682798058963, -0.43590865138100393`}}]}}, 
     {Arrowheads[{{0.040587803758723034`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7205433697584913, -0.10898599215659384`}, {
        0.8088683949473916, -0.3145434196081124}}]}}, 
     {Arrowheads[{{0.052037501067732916`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7099285920860411, 0.06196091506845905}, {
        0.8194831726198418, -0.20313738565669473`}}]}}, 
     {Arrowheads[{{0.05203750106773294, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7099285920860411, -0.06196091506845952}, {
        0.8194831726198418, 0.20313738565669442`}}]}}, 
     {Arrowheads[{{0.04058780375872307, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7205433697584913, 0.1089859921565934}, {
        0.8088683949473916, 0.3145434196081121}}]}}, 
     {Arrowheads[{{0.03239168694035227, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7317434848999865, 0.2699737015601725}, {
        0.7976682798058964, 0.43590865138100365`}}]}}, 
     {Arrowheads[{{0.02665697778993295, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7402450074263801, 0.42483921658776264`}, {
        0.7891667572795027, 0.5633960775298841}}]}}, 
     {Arrowheads[{{0.022503892202001715`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.746273264224687, 0.5760728545321513}, {0.783138500481196, 
        0.694515380761966}}]}}, 
     {Arrowheads[{{0.019387547870146144`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7505252275028289, 0.72495209321317}, {0.778886537203054, 
        0.8279890832574176}}]}}, 
     {Arrowheads[{{0.016979080664775204`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7535640465476567, 0.8721963280379417}, {
        0.7758477181582261, 0.9630977896091164}}]}}, 
     {Arrowheads[{{0.015071701293711279`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7557775204458713, 1.0182548990235807`}, {
        0.7736342442600115, 1.099392159799948}}]}}, 
     {Arrowheads[{{0.013529796440564323`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7574228966526076, 1.1634283164953783`}, {
        0.7719888680532753, 1.2365716835046208`}}]}}, 
     {Arrowheads[{{0.01312761790541763, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9297139206467715, -1.1656823597169808`}, {
        0.9526390205296997, -1.2343176402830192`}}]}}, 
     {Arrowheads[{{0.014542990019468325`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9272587081794207, -1.0212351469461067`}, {
        0.9550942329970504, -1.096411911877423}}]}}, 
     {Arrowheads[{{0.016272107256669006`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9240031934596128, -0.8762173814295938}, {
        0.9583497477168584, -0.9590767362174651}}]}}, 
     {Arrowheads[{{0.018427008297575858`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9195937937004144, -0.7304975373000479}, {
        0.9627591474760568, -0.8224436391705408}}]}}, 
     {Arrowheads[{{0.02118358153152602, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9134712355054826, -0.5839016937607147}, {
        0.9688817056709886, -0.6866865415334034}}]}}, 
     {Arrowheads[{{0.02484493757949049, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9046983376917432, -0.4361670254417359}, {
        0.9776546034847281, -0.5520682686759115}}]}}, 
     {Arrowheads[{{0.030017486788541166`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8915361596808088, -0.28675647859228914`}, {
        0.9908167814956623, -0.4191258743488877}}]}}, 
     {Arrowheads[{{0.03825714390929144, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8700893928750443, -0.13388960786310056`}, {
        1.0122635483014268`, -0.28963980390160565`}}]}}, 
     {Arrowheads[{{0.0557576247082935, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8292296707970842, 0.034691933857151094`}, {
        1.053123270379387, -0.17586840444538676`}}]}}, 
     {Arrowheads[{{0.05575762470829359, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8292296707970841, -0.03469193385715165}, {
        1.0531232703793871`, 0.1758684044453866}}]}}, 
     {Arrowheads[{{0.038257143909291504`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8700893928750442, 0.1338896078631001}, {1.012263548301427,
         0.2896398039016054}}]}}, 
     {Arrowheads[{{0.030017486788541194`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8915361596808088, 0.28675647859228875`}, {
        0.9908167814956625, 0.4191258743488873}}]}}, 
     {Arrowheads[{{0.024844937579490505`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9046983376917432, 0.43616702544173547`}, {
        0.9776546034847281, 0.5520682686759112}}]}}, 
     {Arrowheads[{{0.02118358153152602, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9134712355054826, 0.5839016937607142}, {
        0.9688817056709886, 0.6866865415334029}}]}}, 
     {Arrowheads[{{0.01842700829757587, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9195937937004144, 0.7304975373000474}, {0.962759147476057,
         0.8224436391705403}}]}}, 
     {Arrowheads[{{0.016272107256669013`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9240031934596128, 0.8762173814295934}, {
        0.9583497477168585, 0.9590767362174647}}]}}, 
     {Arrowheads[{{0.014542990019468287`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9272587081794207, 1.0212351469461063`}, {
        0.9550942329970504, 1.0964119118774223`}}]}}, 
     {Arrowheads[{{0.013127617905417667`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9297139206467715, 1.1656823597169803`}, {
        0.9526390205296997, 1.234317640283019}}]}}, 
     {Arrowheads[{{0.012572289032024979`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1028688664023556`, -1.16865860643912}, {
        1.1324252512447035`, -1.2313413935608801`}}]}}, 
     {Arrowheads[{{0.013818604383719657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0999518562430113`, -1.025098049253899}, {
        1.135342261404048, -1.0925490095696302`}}]}}, 
     {Arrowheads[{{0.01530879780135259, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.096181198473009, -0.8813226496434575}, {
        1.1391129191740503`, -0.9539714680036014}}]}}, 
     {Arrowheads[{{0.0171168985608574, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0912249668343268`, -0.7373876325651155}, {
        1.1440691508127325`, -0.815553543905473}}]}}, 
     {Arrowheads[{{0.019351722288825802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0845818130408897`, -0.59344446591864}, {
        1.1507123046061698`, -0.677143769375478}}]}}, 
     {Arrowheads[{{0.02218366408088065, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0754565941849077`, -0.4498663135175107}, {
        1.1598375234621519`, -0.5383689806001367}}]}}, 
     {Arrowheads[{{0.025900636613884812`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0624987602614202`, -0.3076141507972888}, {
        1.1727953573856391`, -0.39826820214388803`}}]}}, 
     {Arrowheads[{{0.030997853497116933`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0433178547778412`, -0.16964427604576154`}, {
        1.1919762628692179`, -0.2538851357189447}}]}}, 
     {Arrowheads[{{0.03754378372508135, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0167466765914932`, -0.04764781363693989}, {
        1.2185474410555661`, -0.0935286569512958}}]}}, 
     {Arrowheads[{{0.03754378372508138, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0167466765914932`, 0.047647813636939595`}, {
        1.2185474410555663`, 0.09352865695129531}}]}}, 
     {Arrowheads[{{0.03099785349711697, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0433178547778412`, 0.16964427604576116`}, {
        1.191976262869218, 0.25388513571894433`}}]}}, 
     {Arrowheads[{{0.025900636613884847`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0624987602614202`, 0.30761415079728843`}, {
        1.1727953573856393`, 0.39826820214388764`}}]}}, 
     {Arrowheads[{{0.022183664080880645`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0754565941849077`, 0.44986631351751033`}, {
        1.1598375234621519`, 0.5383689806001363}}]}}, 
     {Arrowheads[{{0.019351722288825802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0845818130408895`, 0.5934444659186395}, {
        1.1507123046061696`, 0.6771437693754776}}]}}, 
     {Arrowheads[{{0.0171168985608574, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0912249668343268`, 0.7373876325651151}, {
        1.1440691508127325`, 0.8155535439054725}}]}}, 
     {Arrowheads[{{0.01530879780135259, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.096181198473009, 0.8813226496434571}, {
        1.1391129191740503`, 0.953971468003601}}]}}, 
     {Arrowheads[{{0.013818604383719657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0999518562430113`, 1.0250980492538986`}, {
        1.135342261404048, 1.0925490095696297`}}]}}, 
     {Arrowheads[{{0.012572289032024979`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1028688664023556`, 1.1686586064391196`}, {
        1.1324252512447035`, 1.2313413935608797`}}]}}, 
     {Arrowheads[{{0.011912824053351105`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.276980890100606, -1.1719937029687708`}, {
        1.311254404017042, -1.228006297031229}}]}}, 
     {Arrowheads[{{0.012970645227741053`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.273945400279744, -1.0293099497731446`}, {
        1.314289893837904, -1.0883371090503848`}}]}}, 
     {Arrowheads[{{0.014198637077553282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2701583548583304`, -0.8867058067067692}, {
        1.318076939259317, -0.9485883109402896}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2654013659875016`, -0.744353715163706}, {
        1.322833928130146, -0.8085874613068825}}]}}, 
     {Arrowheads[{{0.01730893096661073, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.259402541530508, -0.6025728171064615}, {
        1.3288327525871397`, -0.6680154181876565}}]}}, 
     {Arrowheads[{{0.019265812323821695`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2518642667677342`, -0.46195937889901734`}, {
        1.3363710273499134`, -0.52627591521863}}]}}, 
     {Arrowheads[{{0.021497164399658764`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2426242296078325`, -0.323634918740149}, {
        1.3456110645098152`, -0.38224743420102786`}}]}}, 
     {Arrowheads[{{0.023825797444883293`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2323151984543796`, -0.18957020538367553`}, {
        1.355920095663268, -0.23395920638103074`}}]}}, 
     {Arrowheads[{{0.025567923001979954`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2241885081494626`, -0.061887105362916243`}, {
        1.3640467859681853`, -0.07928936522531944}}]}}, 
     {Arrowheads[{{0.025567923001979995`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2241885081494623`, 0.061887105362915903`}, {
        1.3640467859681853`, 0.079289365225319}}]}}, 
     {Arrowheads[{{0.023825797444883293`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2323151984543796`, 0.18957020538367517`}, {
        1.355920095663268, 0.23395920638103032`}}]}}, 
     {Arrowheads[{{0.021497164399658757`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2426242296078325`, 0.3236349187401486}, {
        1.3456110645098152`, 0.3822474342010274}}]}}, 
     {Arrowheads[{{0.019265812323821702`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2518642667677342`, 0.46195937889901695`}, {
        1.3363710273499134`, 0.5262759152186297}}]}}, 
     {Arrowheads[{{0.01730893096661073, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.259402541530508, 0.602572817106461}, {1.3288327525871397`,
         0.6680154181876561}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2654013659875016`, 0.7443537151637055}, {
        1.322833928130146, 0.8085874613068821}}]}}, 
     {Arrowheads[{{0.014198637077553295`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2701583548583304`, 0.8867058067067688}, {
        1.318076939259317, 0.9485883109402893}}]}}, 
     {Arrowheads[{{0.012970645227741053`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2739454002797437`, 1.0293099497731442`}, {
        1.3142898938379037`, 1.0883371090503844`}}]}}, 
     {Arrowheads[{{0.011912824053351105`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.276980890100606, 1.1719937029687704`}, {1.311254404017042,
         1.2280062970312287`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4519903496605397`, -1.175362977900796}, {
        1.4891861209276964`, -1.224637022099204}}]}}, 
     {Arrowheads[{{0.012072512361463282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.449096467238498, -1.033422350108421}, {
        1.492080003349738, -1.0842247087151085`}}]}}, 
     {Arrowheads[{{0.013051586754250076`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4456318649600366`, -0.8917404906758423}, {
        1.4955446056281991`, -0.9435536269712167}}]}}, 
     {Arrowheads[{{0.014144384224217975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.441503749830402, -0.7505125291145733}, {
        1.4996727207578342`, -0.8024286473560152}}]}}, 
     {Arrowheads[{{0.015348449856734177`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4366504558557036`, -0.6100412537619427}, {
        1.5045260147325321`, -0.6605469815321753}}]}}, 
     {Arrowheads[{{0.01663799156292185, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.431115356250358, -0.4707788430506923}, {
        1.510061114337878, -0.5174564510669551}}]}}, 
     {Arrowheads[{{0.01793644016261795, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4252071475594414`, -0.33333548645830935`}, {
        1.5159693230287945`, -0.37254686648286744`}}]}}, 
     {Arrowheads[{{0.019074142779961962`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4197667974791732`, -0.1983162046793514}, {
        1.5214096731090627`, -0.22521320708535486`}}]}}, 
     {Arrowheads[{{0.019772769634296777`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4163082911356528`, -0.06573786377448221}, {
        1.524868179452583, -0.07543860681375349}}]}}, 
     {Arrowheads[{{0.019772769634296777`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4163082911356528`, 0.06573786377448183}, {
        1.524868179452583, 0.07543860681375306}}]}}, 
     {Arrowheads[{{0.019074142779961955`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4197667974791732`, 0.19831620467935104`}, {
        1.5214096731090627`, 0.22521320708535444`}}]}}, 
     {Arrowheads[{{0.01793644016261795, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4252071475594414`, 0.333335486458309}, {
        1.5159693230287945`, 0.3725468664828671}}]}}, 
     {Arrowheads[{{0.016637991562921853`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.431115356250358, 0.4707788430506919}, {1.510061114337878, 
        0.5174564510669548}}]}}, 
     {Arrowheads[{{0.015348449856734208`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4366504558557036`, 0.6100412537619423}, {
        1.5045260147325323`, 0.6605469815321748}}]}}, 
     {Arrowheads[{{0.014144384224217989`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4415037498304017`, 0.7505125291145729}, {
        1.499672720757834, 0.8024286473560148}}]}}, 
     {Arrowheads[{{0.013051586754250076`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4456318649600366`, 0.8917404906758418}, {
        1.4955446056281991`, 0.9435536269712163}}]}}, 
     {Arrowheads[{{0.012072512361463311`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.449096467238498, 1.0334223501084205`}, {1.492080003349738,
         1.0842247087151082`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4519903496605397`, 1.1753629779007955`}, {
        1.4891861209276964`, 1.2246370220992036`}}]}}, 
     {Arrowheads[{{0.010477306125975746`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6277414377817478`, -1.1785359331371488`}, {
        1.6663762092770762`, -1.221464066862851}}]}}, 
     {Arrowheads[{{0.01118362862614915, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6251385163185235`, -1.0371535747662564`}, {
        1.6689791307403008`, -1.080493484057273}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.622150064150561, -0.8961012470138565}, {
        1.671967582908263, -0.9391928706332024}}]}}, 
     {Arrowheads[{{0.012768289697852271`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6187690551976337`, -0.7555398540556062}, {
        1.6753485918611908`, -0.7974013224149822}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6150435971751116`, -0.615681833577597}, {
        1.6790740498837127`, -0.654906401716521}}]}}, 
     {Arrowheads[{{0.014475369768919494`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6111259556165303`, -0.4767823051369137}, {
        1.6829916914422942`, -0.5114529889807338}}]}}, 
     {Arrowheads[{{0.015263437675282373`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6073372076773222`, -0.33908820015721075`}, {
        1.6867804393815022`, -0.3667941527839661}}]}}, 
     {Arrowheads[{{0.015890646781373564`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.604208497335815, -0.20270973761578687`}, {
        1.689909149723009, -0.22081967414891943`}}]}}, 
     {Arrowheads[{{0.016244935134227038`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6023976663336905`, -0.06742454185633001}, {
        1.6917199807251337`, -0.07375192873190567}}]}}, 
     {Arrowheads[{{0.016244935134227038`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6023976663336905`, 0.06742454185632964}, {
        1.6917199807251337`, 0.07375192873190527}}]}}, 
     {Arrowheads[{{0.015890646781373564`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.604208497335815, 0.20270973761578648`}, {
        1.689909149723009, 0.220819674148919}}]}}, 
     {Arrowheads[{{0.015263437675282373`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6073372076773222`, 0.33908820015721036`}, {
        1.6867804393815022`, 0.3667941527839657}}]}}, 
     {Arrowheads[{{0.014475369768919487`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6111259556165303`, 0.47678230513691333`}, {
        1.6829916914422942`, 0.5114529889807333}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6150435971751116`, 0.6156818335775965}, {
        1.6790740498837127`, 0.6549064017165206}}]}}, 
     {Arrowheads[{{0.012768289697852271`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6187690551976337`, 0.7555398540556058}, {
        1.6753485918611908`, 0.7974013224149817}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.622150064150561, 0.8961012470138561}, {1.671967582908263, 
        0.939192870633202}}]}}, 
     {Arrowheads[{{0.01118362862614915, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6251385163185235`, 1.037153574766256}, {
        1.6689791307403008`, 1.0804934840572726`}}]}}, 
     {Arrowheads[{{0.010477306125975746`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6277414377817478`, 1.1785359331371483`}, {
        1.6663762092770762`, 1.2214640668628505`}}]}}, 
     {Arrowheads[{{0.0097764739834501, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8040469254264957`, -1.181386143062818}, {
        1.8430118981029169`, -1.2186138569371818`}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8017905824028364`, -1.0403841856980935`}, {
        1.8452682411265762`, -1.0772628731254361`}}]}}, 
     {Arrowheads[{{0.01093767795645266, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7992987754129588`, -0.899712904175814}, {
        1.8477600481164538`, -0.935581213471245}}]}}, 
     {Arrowheads[{{0.011550188090361782`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7966059841827862`, -0.7594847871806495}, {
        1.8504528393466264`, -0.793456389289939}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7937938880921889`, -0.6198271343117402}, {
        1.8532649354372237`, -0.6507611009823778}}]}}, 
     {Arrowheads[{{0.012740596380756626`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7910124698751133`, -0.48086287631464036`}, {
        1.8560463536542993`, -0.5073724178030071}}]}}, 
     {Arrowheads[{{0.013246569582145828`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7884945676311024`, -0.3426707593544241}, {
        1.8585642558983102`, -0.3632115935867528}}]}}, 
     {Arrowheads[{{0.013627849871941577`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7865430632708879`, -0.20522498643722129`}, {
        1.8605157602585247`, -0.218304425327485}}]}}, 
     {Arrowheads[{{0.013834467068875216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7854663260855017`, -0.06833838779139521}, {
        1.8615924974439109`, -0.07283808279684047}}]}}, 
     {Arrowheads[{{0.013834467068875216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7854663260855017`, 0.06833838779139484}, {
        1.8615924974439109`, 0.07283808279684006}}]}}, 
     {Arrowheads[{{0.013627849871941575`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7865430632708879`, 0.2052249864372209}, {
        1.8605157602585247`, 0.2183044253274846}}]}}, 
     {Arrowheads[{{0.013246569582145828`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7884945676311024`, 0.3426707593544237}, {
        1.8585642558983102`, 0.3632115935867524}}]}}, 
     {Arrowheads[{{0.012740596380756657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.791012469875113, 0.48086287631464}, {1.8560463536542993`, 
        0.5073724178030067}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7937938880921889`, 0.6198271343117397}, {
        1.8532649354372237`, 0.6507611009823774}}]}}, 
     {Arrowheads[{{0.011550188090361782`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7966059841827862`, 0.759484787180649}, {
        1.8504528393466264`, 0.7934563892899386}}]}}, 
     {Arrowheads[{{0.01093767795645266, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7992987754129588`, 0.8997129041758135}, {
        1.8477600481164538`, 0.9355812134712446}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8017905824028364`, 1.040384185698093}, {
        1.8452682411265762`, 1.0772628731254357`}}]}}, 
     {Arrowheads[{{0.0097764739834501, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8040469254264957`, 1.1813861430628176`}, {
        1.8430118981029169`, 1.2186138569371814`}}]}}, 
     {Arrowheads[{{0.009116798619914539, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.980733871751405, -1.1838697719654656`}, {
        2.0192661282485957`, -1.2161302280345343`}}]}}, 
     {Arrowheads[{{0.009569687754266311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9788199149169337`, -1.0431054477318142`}, {
        2.021180085083067, -1.0745416110917152`}}]}}, 
     {Arrowheads[{{0.010032604380582781`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9767770061484127`, -0.9026378443014493}, {
        2.023222993851588, -0.9326562733456097}}]}}, 
     {Arrowheads[{{0.010494535920320246`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9746523699480951`, -0.7625382505618518}, {
        2.0253476300519058`, -0.7904029259087367}}]}}, 
     {Arrowheads[{{0.010939521084516988`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9725254688599796`, -0.6228758215402782}, {
        2.027474531140021, -0.6477124137538397}}]}}, 
     {Arrowheads[{{0.0113461236745298, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.970514107020019, -0.4837023461393182}, {
        2.0294858929799817`, -0.5045329479783293}}]}}, 
     {Arrowheads[{{0.01168814372015665, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9687725946320478`, -0.34502979430757774`}, {
        2.031227405367953, -0.3608525586335991}}]}}, 
     {Arrowheads[{{0.011937480299842693`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.967474718890251, -0.20680544893831998`}, {
        2.03252528110975, -0.21672396282638628`}}]}}, 
     {Arrowheads[{{0.01206948107950648, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9667780233212562`, -0.0688969727636392}, {
        2.0332219766787447`, -0.0722794978245965}}]}}, 
     {Arrowheads[{{0.012069481079506476`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9667780233212562`, 0.0688969727636388}, {
        2.0332219766787447`, 0.07227949782459608}}]}}, 
     {Arrowheads[{{0.011937480299842693`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.967474718890251, 0.2068054489383196}, {2.03252528110975, 
        0.2167239628263859}}]}}, 
     {Arrowheads[{{0.011688143720156648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9687725946320478`, 0.34502979430757735`}, {
        2.031227405367953, 0.36085255863359866`}}]}}, 
     {Arrowheads[{{0.011346123674529803`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.970514107020019, 0.4837023461393178}, {
        2.0294858929799817`, 0.5045329479783289}}]}}, 
     {Arrowheads[{{0.010939521084516988`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9725254688599796`, 0.6228758215402779}, {
        2.027474531140021, 0.6477124137538394}}]}}, 
     {Arrowheads[{{0.010494535920320246`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9746523699480951`, 0.7625382505618513}, {
        2.0253476300519058`, 0.7904029259087363}}]}}, 
     {Arrowheads[{{0.010032604380582781`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9767770061484127`, 0.9026378443014489}, {
        2.023222993851588, 0.9326562733456093}}]}}, 
     {Arrowheads[{{0.009569687754266311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9788199149169337`, 1.0431054477318138`}, {
        2.021180085083067, 1.0745416110917148`}}]}}, 
     {Arrowheads[{{0.009116798619914539, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.980733871751405, 1.1838697719654652`}, {
        2.0192661282485957`, 1.2161302280345339`}}]}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJx0nXl8U8X39wMiLiCCKMoiKCAgi18VRWQ7BUWQHQFBFFFZFFBAQBYVKKIg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    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], 
         LineBox[{3320, 4033, 3319, 4338, 4337, 4725, 4724, 4004, 3663, 5053, 
          3662, 4034, 3324, 4983, 3669, 4005, 3671, 5054, 3670, 4037, 4038, 
          3678, 5056, 3677, 3461, 3330, 4987, 3679, 3534, 4518, 3535, 4519, 
          4735, 4734, 4041, 3339, 4995, 3692, 4010, 3464, 3340, 4996, 3693, 
          3540, 4531, 3541, 4244, 3703, 4279, 4046, 4047, 3711, 3354}], 
         LineBox[{3427, 5028, 3816, 4027, 3498, 4384, 4383, 4278, 3639, 4225, 
          4226, 3826, 5072, 3825, 4071, 4072, 3837, 4866, 3836, 3504, 3437, 
          4703, 3648, 4466, 4467, 3843, 5075, 3842, 4075, 3445, 5042, 3848, 
          4030, 3507, 3446, 5043, 3849, 3652, 3856, 5077, 3855, 3509, 3450, 
          5047, 3857, 3657, 4873, 4874, 4872, 3858, 3510, 3456}], 
         LineBox[{3719, 5098, 3720, 4104, 4103, 4342, 4341, 4213, 4212, 4211, 
          3545, 3702, 5061, 3701, 4099, 4100, 3918, 5003, 3349, 4096, 3338, 
          4393, 4392, 4391, 3537, 3685, 5058, 3684, 4091, 3909, 5083, 3910, 
          4089, 4088, 3329, 4986, 3906, 3676, 3905, 4985, 3328, 4084, 3323, 
          4982, 3901, 3668, 4904, 4905, 4339, 4340, 3322, 4083, 4186}], 
         LineBox[{3808, 5070, 3809, 4145, 4144, 4848, 3807, 4303, 5097, 3636, 
          4687, 3637, 3949, 5029, 3428, 4148, 4149, 3955, 4956, 3954, 4957, 
          4152, 4861, 4862, 4694, 3643, 4693, 3644, 3962, 5038, 3438, 4157, 
          3443, 5040, 3967, 4160, 4159, 3968, 5088, 3969, 3844, 3975, 5105, 
          3974, 4164, 3976, 5090, 3977, 3850, 4968, 4969, 4965, 4967, 4966, 
          4168, 4169}]},
        "0.22999999999999998`"],
       Annotation[#, 0.22999999999999998`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1XtM1mUYxvHHUsAUjGzW6i9QpDLbtK3kkCdSsNMC2/KQJiDgKWCFAnEw
SNuibG62/McywczlmqOsLWdzTSs03Dyt06bN0kpWgpI2KaXPs/64/D7Xdd/3
8/7ex+f3klZSVVQ5JITQ4Z+3U0NIo9pbQmhNDqEOC1NCeEfWxadjvexFKpJv
5Q/Jx2KDrJHmypvwXdk2GkcZo0N4Wd947BsVQg5lWufiJbpI10eG8J7ebn3b
8QhOk/+pbypepis0nepkg/r/th6qr4nPsy712Q9jMz9MPmCdhC18gfVK9TnY
yg+XD1qPxlf5x6xfUH8c2/hb5TfcHEKN7D6+SH4vFuJETFDb4Hu+Qcc97xn9
x/BnfErPRj3p1iP0vWKPXP5peRbmUDalqrWb/9Xcb3SOztPvtFP+lrk+6377
9OJfeBHb5YfMd+B12XTZDJpJD/FJZhPUCvg59AjlycfIk+WFfBHNpXx5mvwO
+Tx+Pn1l3W3/L3G82jjMwAzMxKN4ArMom6ZQLuXQWfrF91xk3232OKX3M34v
LZa1y07LJsez1ns/PoEDdDWehb7l+j7Q16u2j/+cVsh2xfsjy6YBOsZfxd1q
q+hD/h9+GgUzx+M9se5Ue55280PkM2QJeDLeE/xIrYY6+cR45+gPqpd9Kkux
TsTV8RzNLvGcQ/kbaY+e/erJNIwfiQl4AdeqVemtplVUSc9Rs3wlFthrOebj
CmySL8PZfDnOwor4XsmXYhkVUymVUIM83x2qw098ZpvPnM3PolJ+KZVQMdXE
+2G/EXr28LVmZurLo/WyM7J18Wz4VuyL72x8L8004098E05Vb8BevhH71Wvx
NL8m3u/4zuCFeF7Yo16Np/gqfED9vKzSeoeszHNMlj2LFZ6xnMpogp4f6Iie
73Ginu/ieZnbGt8l/ZmywninaLGZRXS3c7mLHpU9wy+kBXSn2fR45vIt5vfx
o+g2WQrejh/H88EkHI6JeBN2xvfMHgH/xUF7XMPtfD9e4S/hZdws22z/SdZZ
sin0IPXwPfH5cGP8ztaT6CSdoLGyDfG3yPwmn9mGb8Z3SXbUd36J77JvC07Q
fw/9qLYXx8S5+Hut9pqZenwdv5V1m63iD5qtxl18Ja7nk9UP8xX8AX4Z7uTL
cW38jVPv4ov5L+Jdwvf5JdgY77/61/wCfj+/EHfw6+TzrOfT4fj/4xm/wQ68
5nf7gJ6D9KT66vgOy1rU13jmLan//x36DzVa57A=
          "]]},
        "0.184`"],
       Annotation[#, 0.184, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1FdszlEYx/HTBI3ZWtFeWAkiRa2oVb1BxApqFKUEsam6slOzlMQIiUQI
kdCiVY2ZujQuxGyMRCVCSmre2Ptz4uLb73me83vO+b/5v307zsnPXp4QQsj0
Z1GzEDa2CGEzNuFWUghJTUMoax5COU7jgfoMN+NtLUMoMHNGbod8MbZhO4pw
Wz9F7oL8RZzHY/VaMzfMruFidRvsUq9TP7Wukqvhq7zHObtx31md9PbK7ZQr
U5ejs94+vV16tdYP4x14hOPmqmX66B+UOShToT6g7otPMp8xCIORiQH6fzlB
digPw3Bk6TdxXkP90eoxGBv39VP1k/UnqSfjiN5ItE8OoQPmuXOY++fycO6k
d91MN3RHGrqYn2//sP1ss11l7scz1Yv1J/DEuId0e7VmXiErniE3hHvwUtnj
MrPjZ5SbzyfVb+1lYLnny1Gf1p/K/WUKzEyzzsV0vJP7jHfWnzjb2QN4hdk8
vTKzs3iQ2VVmZ1uv5kz1YOTHs81MQw6GmJ3OK/RnxGdW53IeZqJAv878G3yJ
3yfnLdErj+/C/kRMijn3B3sJ+Cr3DQPxHiV6C+J3Ez9lx2MC8s384g8ypTIL
7de655X6rvo1L5NZit7WY2RnyJzAU/UHdQ1/jO+JX5h9GK1+xC95ntm56G+d
Ee82ewx37D9Q3+Vn5mpwTd0eHdAWqfbacRW35ibckhtzK67kRK7P9bkeN+AK
/unO3+75xj/wPf6f2mvjHaQg132H4juWaaXuxfuRIdcPdfE55fc56yjv5WS5
PTzF7H6zXWUa6XXkXmZ6Ih3P1Rvkbsqsj88mMy5+RvV4fmL/ktwVXEaqOkGm
BW+XGS2z0twoLlI316+Uq+azfA4VSFb/kUnkQrkR1r/xC93URfpp3EX2nvUp
XuzcrT7PFnRWl6IEhfH3SaY06f9v3T9X76zo
          "]]},
        "0.138`"],
       Annotation[#, 0.138, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl09tPz3Ecx/FPuGHr8KvNUKIZZtOBDlwgig5OGZPTDW02tPE/OFxRDXMo
o2yMNi6YCzc5VHIqsrkgqxxCxIVTtenC4zsXzz0/79fn/fl8vr/v5/vLqDqw
cX9cCGEUd2IhzMeJ5BAOJYRQmxJCWnwI92S9fJdbcR/T1e38AG14iA6kyx/z
EzxCJ55ihvwZP0cX+tTdPJPrnHPYef3GmbJs9THjHJ6YFMKmxBAWGOdiIY7o
XcSLkYcC5CNRb525856/ERcwaJ9ce+bhpHE2UvQVcIM12+3daM1kWRNfs+Yi
X+cxPUutW4ZCLFFPkI83X6IuxTlZEWLyZKyXVWADSuRpsin6K9Vbonelble3
chu/cf4sfqHu4m7u5CZrK5Cnzscua6vw1vgdetGPvqi2xw696dG9YFD2Gbfl
3/g7vmAIXzEgr9K/2ngNypHjzI8YwKh6BNX2KuNSDKPZmj+8L/ptvAq/cVX+
i/fKV3IxfuKK/Afvie7UvkXG73nIe//AWSiUrcDy6DdG9+MetnGTnmrP2aNn
s3ElUu2Thq3m50XfBU9TT8U682sxV3+mvbKRhQ7M1ltqrgwl2G3fcm7Rm2Eu
Zn0SdsqL5cnyadbFOJVvcRJP4nhOREKUY1z0PSBEY/NxfJlH7Dfq+YZ5hJvl
DfK/zhvDTWfdQL2sPvqO9bxUX9L32poevMIn+VlZfXQn1o1ijqwWNdaekp/B
6eheZQdlNdG7Mj6K4/KW2P//9j/1n4tR
          "]]},
        "0.092`"],
       Annotation[#, 0.092, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl00lszGEYx/FXgks3Srm4iMbSqqDpoVy4jqUilFhCXDjVjcSFG7V0UNNq
CRHUzbTaoZ0SbR1Qexe0nWqtPVgOwoGTzz8O3/ye5/cs7zv/NzN3z/6NVZNC
CO9QkxtCY34IF3EBfXkhLM5Rmx7CGEYwigyK+YkZIdSa6ZgWwgHUIYF2nFEv
wl/xH3wz8zWC953+wHl+PdIYdFa52hT+ZDQ7P4lWfhum8lrkTc7rdW61meO4
a6aSnlaP44S4VP8IfRCdRRvkJ+kp1GAL7qnF6VaUqe+gV+gqu1cj5axU9Nto
GrfRjjuYo7eTPqa9KBTPi3z9MbNrUOAus7DWOfP1bKIt9BaasVDvazNvMBB9
a/kgXYpl2GV2vT1XzVXQIjN73W+DeF/0LvJibFMfk4+j345L6pt51/DWnuUo
xW77huh1/oC+YXEGI1gkb7UrhTZURm9u31O1Z3iCF3iOI+qHkcRNxPReRo8d
99W70YXZ8iE7OujD6I3pI9qPPlRE7+dO1TgmLtE7THfSV7SAvqQzaZbfFIu+
uXOO0nJeJ69HvkLcTdfZn+Z1iRvsbEQWsrGSV6hWRuvxRfwZh+z6TX/hJ7/J
vW9E3wgJ+/LVR3FWbUJPRnwQE/Jz6nW8JeKPzviEcXzAe5Twc/UuoLXRm7hn
jjwb2xHnJ/P+/+f+AUsWjSk=
          "]]},
        "0.046`"],
       Annotation[#, 0.046, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0ztsjWEcgPG3DCwo4rK0ek53t0hZEHsZJE3agUQTLPS0ujC0S7UkvSRo
Tzm3JgzauiUMKtrTI1G9YRTp4BIGElpsBur3xvDk+d/ey/d95yQaU0ebykII
U/i2JoTz5SEUMbohhNzaEJbFYX0IWf61MYTvZjLiRc5xmV6Bl+R5/sFDvFJ9
BfL2uGOvn+oX1N/hPT7iAzbhgH3r14Wwnxv4IBese6H3gJ/zfZ7ivfoFjJkp
2rNBbZKfyPep96HVebXmO53fhZRev/ol8WUct6Zb3oOMtYewU7xgbhfvQJ2Z
PrO92C1P6PWIu7FdXiXfzJXmxpx1mh9zBb9VP8ct8X3E+6lXx7560d2eyce5
xBO8oH7EHbba+565pPpdTvCc/jzP8hy/Vn+FdlTFd8uz/JLn0YZt8lGe5mnr
ZniGK3lYvYJHeIvzDjs3KW5GCk1odJ+v8Xdg7gufkJ+N3zx+ZwzJb+EmjsV9
+I330cslVItreMD+abTI8/JB8XWcsiYjz+GzXpaXkMYn+WBci5Pm+s1fi+vk
zXpXxVewR56UXzTTEZ9dPOK+jzznKv3VcY1neyi/rT5pZphL/Ft9grPyM/wH
f5FWf8rjuFH+///wD8HBfGM=
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0slvjXEUh/FjSdyh97bV9paqmfoLbIikG0VoLUUpC2NFSQ2tqei200qQ
VFskhqUqsaHFP0CrW3OwkyBsfH6xePKc8z3n/b3DvfVtR5vb50TEBP5kIrry
Ec8xifvZiJv8V97N7/AeO3MRH7gaNXhgb25ZxDyss5vhDVwoRhRRhhrXtNht
xmOzJfqSfCkvx4x6WbL5NDoKEU3OfSq/ou/FEfWgnfPOv4CL2C3fgz55G7fi
svwSejAkb3fdd/kQXrn3NflL/iKfla2y85br9IuwUl/LK3gh7zDbgl/qrVyS
l6d3QjXq9Tec14ics7MY9+wz8um0qy/ijmsr5be4gm9zQT7G5foRLvJvZ46q
P+KH+hPvcs5GdSt/Tt+ec/o83uhf4ysO6q97jnHzA/r9WKzezE2YMKtK7ylb
wHW8FrPqBh6wM2Wnn1/wIH/DYfMB827PexbnsE/+xM5e7tKfwWn02zuUfns+
5nf8qd7kvTrNTuIUVqdvLl+TvjEq1LVcyY+cuc2Z21Elmy/LIoP1ZiX5cfUJ
dGA43S+9O+65z0P9VXvP9Hf1w/n//+1/6aVfFg==
          "]]},
        RowBox[{"-", "0.046`"}]],
       Annotation[#, -0.046, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0nXdAU8kX72Pvvay9rH3ta13bYO/Yu1ixiwURRUUjgth17V3sXVFRUdGJ
iG5U1IARA0QMkF7oTaS8/CB38uab9/yH/eyZuZlzp51zptzmC1ZPXFRSJBKp
G4pE//tb/C+Fdiv6l06L2QysB1YDq4AVwDJgCbBIBHIC+YEVwCpgNbAe2Ayc
ApwGnAGcBZwDnAucB1wALHKC8tBTRf9SWX3wrAdWA6uAFcAyYAmwSARyAvmB
FcAqYDWwHtgMnAKcBpwBnAWcA5wLnAdcACxygvLQwqJ/yaw+eNYDq4FVwApg
GbAEWCQCOYH8wApgFbAaWA9sBk4BTgPOAM4CzgHOBc4DLgAWOUF5aHjRPzOr
D571wGpgFbACWAYsARaJQE4gP7ACWAWsBtYDm4FTgNOAM4CzgHOAc4HzgAuA
RU5QHtv4YYDxysDqg2c1sApYASwDlgAL4xWTE8gPrABWAauB9cBm4BTgNOAM
4CzgHOBc4DzgAmBhvGLloYuL/mlZffCsB1YDq4AVwDJgCbBIBHIC+YEVwCpg
NbAe2AycApwGnAGcBZwDnAucB1wALHKC8tjsnURWHzzrgdXAKmAFsAxYAizY
V0xOID+wAlgFrAbWA5uBU4DTgDOAs4BzgHOB84ALgAX7ipWHvVehPnjWA6uB
8a8CWAYsARb+MTmB/MAKYBWwGlgPbAZOAU4DzgDOAs4BzgXOAy4AFjlBeejp
ovErmtUHz3pgNbAKWAEsA5YAi0QgJ5AfWAGsAlYD64HNwCnAacAZwFnAOcC5
wHnABcAiJygP7V7UX76y+uBZD6wGVgErgGXAEmCRCOQE8gMrgFXAamA9sBk4
BTgNOAM4CzgHOBc4D7gAWOQE5aFni+rnPasPnvXAamAVsAJYBiwBFolATiA/
sAJYBawG1gObgVOA04AzgLOAc4BzgfOAC4BFTlAeeqGIn7H64FkPrAZWASuA
ZcASYJEI5ATyAyuAVcBqYD2wGTgFOA04AzgLOAc4FzgPuABY5ATlsf09GSLU
B896YDWwClgBLAOWAAv/mJxAfmAFsApYDawHNgOnAKcBZwBnAecA5wLnARcA
C/M5Kw89Ulw/tnRmYD2wGlgFrACWAUuARSKQE8gPrABWAauB9cBm4BTgNOAM
4CzgHOBc4DzgAmCRE5SHHi/i96w+eNYDq4FVwApgGbAEWCQCOYH8wApgFbAa
WA9sBk4BTgPOAM4CzgHOBc4DLgAWOUF5aNfi+Z3VB896YDWwClgBLAOWAItE
ICeQH1gBrAJWA+uBzcApwGnAGcBZwDnAucB5wAXAIicoDz1RVD/RrD541gOr
gVXACmAZsARYJAI5gfzACmAVsBpYD2wGTgFOA84AzgLOAc4FzgMuABY5QXlo
flG89yerD571wGpgFbACWAYsARaJQE4gP7ACWAWsBtYDm4FTgNOAM4CzgHOA
c4HzgAuARU5QHvp3UX9JZPXBsx5YDawCVgDLgCXAIhHICeQHVgCrgNXAemAz
cApwGnAGcBZwDnAucB5wAbDICcpDFxXHF1l98KwHVgOrgBXAMmAJsEgEcgL5
gRXAKmA1sB7YDJwCnAacAZwFnAOcC5wHXAAscoLy0JNF45eB1QfPemA1sApY
ASwDlgBbLW9eTiA/sAJYBawG1gObgVOA04AzgLOAc4BzgfOAC4BFTlAe+rF4
fYrVB896YDWwClgBLAOWAItEICeQH1gBrAJWA+uBzcApwGnAGcBZwDnAucB5
wAXAIicoDy0oXr9l9cGzHlgNrAJWAMuAJcDWEvJyAvmBFcAqYDWwHtgMnAKc
BpwBnAWcA5wLnAdcACxygvII+z1YffCsB1YDq4AVwDJgCTDbX8LqA/IDK4BV
wGpgPbAZOAU4DTgDOAs4BzgXOA+4AJjtL2H1UTS9d0tn9cGzHlgNrAJWAMuA
JcBs/xWrD8gPrABWAauB9cBm4BTgNOAM4CzgHOBc4DzgAmC2/8rGWrK/qH5u
2tgInIRsW/9Oo0J98lxIeBbWhwVOoleLn8eYf34KyI0OzKfXglwL8gSQJ4Bc
CXIlyOUgl4NcCnIpyANBHkiAMT28bymkl4JcDnI5yJUgV4I8AeQJINeCXLAD
hfinEeRGkCeBPJVg/Xcpao9vWP1DewS5FjgBWAksB5YCByITSA8sB1YCJwDr
Hd4nLzc79D9enurQH/F98cz6J5PzzPonY348SIL9qkzO6ouvXyOwFjgBWAks
B5YCi0TYX3h5ILQ3KbAcWAmcAJzi0D75+LIRWAucAKwElgNLgQORId4tBZYD
K4ETgLXARod4PrY3Xp6E86kT356S4H2aoT/rgdXAKmAFsAxYAiwSgRz0kQEr
gFXAamAL7Vik/yMbG4A1wPHAMcCRwGHAAcDBhOdw4CjgOOBEYA3ZXqTfKca8
3ARyC3AyyinPJmCDg5z/PR3INcA6SJ8I8njgREgfB/IY4DhIHwXySOAoSB8O
8jDgcKxPkAcAB2N7gPcdDBwA9RcG8nDgMEgfCfIo4EhIHwPyOOAYSB8P8kTg
eEifbNuPbLMfxYWEY6v8QXF+Krw/5OK/wnikw/ZL/z+/R+D32HzK/77B4XnI
/PihA/0MDsynN4Hc4sBcejHO18mgfwCB9wPjdbLD+M7rnwz2AZPb2Az2rgXq
wwLlSQG5HvIbQG6A/GpIr4H0GkivgvTxkD4e0isgfQykj4H0MkgfCekjIb0E
0odB+jBMD/ZJMNRnMNSnDNKHQ/pwSK+A9FGQPgrSqyB9HKSPg/RqSJ8I6RMh
vR7S6yC9DtKbIb0J0psgfQr0/xSH/s/3Bw08TwPPM4Dc4PB7MJ859HdgaA8m
HM9ArgN5IsgTQR4H8jiQR4E8CuThIA8HeTDIg0EeBu8rDN5XJMgjQR4D8hiQ
x4M8HtsL1IceWA2sAlYAy4AlwCIRyGH+kAErgFXAamA9sJng7/P+ogz9WYjn
SCGeYwTOBv7lkL64Pwn71jKBs4HltvyCn6kFNgJnAktt66km9jyetcDpwJkE
89veK8vPcypwOnCgbTxRs+fxrAVOBc4HLsT8tv3d8ez5PGuBk4BTgX8B/wbO
Bw4kn4rKo2S/x3MScDbwL2A5XVJUn8I4oyQ8JwBrgY3AScDpwJnA2cBSUqLo
r4yVh+cEYCNwKnC6w/N6Fvc/9jyejcBJwKnA6TB/ZAJnA/8C/g2cD1yI8Uma
V7Te8sXGgcBSwrMSOAHYiOmpa3F92FgOLAUOJCAHVgJnAifQD8Xtkf0+z4GE
ZzmwElgLnARstK1vC/OSFjgBOBCZ8CwH1gL/Bk4SxidWHp4Dgdn4xZ7Pcypw
IXCSbb1Sx+qP50BgOeFZSzC/bbxm6XnWAgvtJ4nl51lOeM4EVtreRwp7Hs9S
AnJgI7AS4rNSiM8agbUwnxsd/P/covdDbRwJrAf/MQnsEyOwFjgBWAksB5YC
ByJD+aXAcmAlcIJDfALilQ7xEn78igSOAU4E/1pNlhXxbhtrgE3ARmAzaVP0
/i8LTHm5BdgEbATWAxuAdcBaYDWwBjgROAFYBRzPs+gn8A/gaOBY4O/A34Aj
gL8Cfwb+ABwK/A44BDgI+BDwDZQTkAOHODDXPkRBIA8Ffgf8GfgDcATwV+Dv
wN+Ao4FjgX8C/4D2rwKOB04EToDnBWD7Jsh8/9KCXA9sAE6G5yc5/F5G0Xh6
31YeSG8df3mG/iyC/imC/ibC/hMDHAkcBixxiAfx8nDgKOA4YKgPEbxvEYx3
IhgPRfB+RSlQXhjfRDBeiWD8EcUBRwGHAwdjfyfvi5kKXPzXHo+A9w0cAwzt
1zof8vFRZi/YWArxWSn4v0aQGx3ue0H/m5f/gvxy+H055f1luYP/XVi8f4r9
Ps9ZDv48L5fC86Vwn4jwe/b7P9Df5+WZwML6sJ49r1h/A8vPc4aDv8/Lk+B5
SeCvp4A80IGLy69l5eFZ63A/A58/FeRG+H0j+PeBDv5+cX0nst/jOdXhvgD+
+fkO9wfwcj38XoID8/57oEN8oPivipUX4wW8PMUhXsHLcx3iFbw8D+RKKJ8S
4guBBOMNkB7Oj2uBkxzOp2O8A8/rQ3zBgfn4gNQhHsGfl1Y6nP/GeAme/8Z4
CZ73xngJnv+G9wflDaR8PEIK8QklnDdOADY6nH/GeAovD4TnB8L5WTnI5SBX
AicAGx3OA2M8hpcbyD/F/hUrP+9PZABnAecA5wLnARcAi5xw/xl/Hk8K5/WU
wAnARuBUh/1JfLxGgvEfOI8nhfNpSuAEiMcYgdPh99JBroB4kNLhfBovlxCM
H/HplcBaSK8FeRLIkxzOl/HybAfm40MqiD8lwPmtQIfzWRif4uVa4CSIB/2C
/L9Arof4k9HhPBQvF85r2ONXfHo5nP/RAqfC76c6nC/i5fkOzMefzBDPSoLz
PoHAcjgPowVOhXhUAfwem09tnALxLYlDPIw/3yGH8x9a4HT4/XSId0nhvIYc
zi9oHeJdfP5MPM+B8TI4fyCH/fhGeL4R5JkQD1NAPE2J++0hvRT2mxuBsyF9
NsTTlLDfQQrrvUbgX5D/l8P+b87/svqLvL8hIbi/a2HR897ZOJzwrIbxNhHk
wTDfRDow7g/m/cOvEG+IgfwxDvuH+fyxkF/psN+Me57Vn+fTG+D5Fvh9Begf
A/on8vFNMa6n6yG/AfIL7cTm/4lT+HihOBXihynwvGSIr36F+v7KP9/6vnh5
LMhj4HkG4BCIPyQ4vG/cz4nry3z6CDKv6H08Ycw/PwaeFwvpDSAPhvoMIcUd
5DIVyof7zfn+ksC/HzHuz7VA+7Ng+4T7XaXgHysc/GHef5WBfytz8GeLn2f3
H3n/TbgvRfDvzA7+XPF4IfhbgQ734aF/VfzXfp8a+n+8XOXg7/D+RyD6I+Cv
yBz8EbyfipdLHOx/3j6X4v1CIt5+loE9LYH+FQztPxjaW6qD/crbk0aH+xfQ
fsT7HXh7zAyc5WCvof2H59PRnsLz77y9kudgv6A9hOeBcf5HewHPQ/Lzu95h
/sfzeTh/8vNrDsyHZujPJhjvQmD8SMb1KJ7FFhiPvsJ4EuvAfHoTzCdGmI8M
pGzRfwhxnRiH8xl8ecz888V4fsNA8DwJ337NILeAPAnmswT4vQR4fxaYv8wQ
z7ZAfDIZxs9kyJ8M+ZMc4uf4+1x/tP4eyvn6kMHvR0D9KUAezcut+vPptdDe
tDB/4HklsMes74t/XoTD/I37z/j8kWR0Uf4jrLw4v+P+ND5/DOTXwniohfiC
DtqzGtpnIrz/VKifFGhvJmAj9EcjtEcDeVc0HoQQ4X3g+0N9eXm0w35StPd4
job+aeLne+v7/1XEXux9cizSgH5q0CeR18f6fvj2BPeNOaVAf1JBf46D31PC
75n5+hHD+GJtL3x5DFCfUmgP4fC+ZDC+mRzqh9NHHAHvKxrk0SA3QXsyQnsI
B3tMA+nVkF4P7d1AShX9lTDm9U2C+TsJ5v8weB9S4GSw/1PgecnkffGEx8qH
75N7PyJYvxfBerwI1tdFsF5utUd5luF6HrBIhOt9kB9Y4dBeoXzAsD4qMuP6
HcwH2bb/L5zPEPZjCvazzmYvWGzydJvcdm5QLKxP2PYti/PZexPqo9g+tdm3
4l/A2bb4dKwtfSL5XCQXzhWaeLb+Ps+p5EtxfbPf49tfmC0eI/SbGFu847uN
gwnP8bb4iLDvPYrwbLDFlwS/JgDYYvs9YV95mC1+I9yLGGmLpyaz8hTbc4I9
Fk54hvVvEexHsY43xf3ntdCfoL3BfhMR7B8RwX4Qq7/BsxxYChyIDOWTAstx
vRUY9wvEOuwHwv0WeD6Hn190MH6CvSCG80Ri4XsDQn8Q7kcS/E+DjW3+plj4
noRtfVIsfN9Az+af4v5i8wPFecCC/S3s2xfajXCvp4VncRawcP9yBCvfsuLx
kf0+x6Jgyo+P4bb4tOCHhhGe42znk4U4h2BfC5xoa9/CvaABhGeTzT8X/J9k
W3sX/J1g4HBb/xH0jrL558I8Hkl4/grxkVioT9xfFgv7weC8vMTqbxSxVzE7
AYteA7/hWSwDjgD+ChwJrACOBo514IbF9qeNY6D8CTxLEhzkXH4nLaQHFoUC
s/HL1r7Cob+w+rNxJOX6j70+hfy2+IyFjd9cf2Ln14RzMhZbeuGcWDDIhX0D
wnxkoFz/s47fPCdSbn6yzgd8/wzg5y/r/MDL4yg/nwnny4T+LJzvEjgG5r9w
ys9/YZTv32EwPwZTfv4LoHx/D4f+HwDzY6QDc/aa1V9cWWy/svGEs7+s8y8/
niTC+GHiWSzct2Wbj63zOZ9eA/OxhWdxFnA2jEfJ/PhjHV+5+dtqf/By4X42
2/wtzgPOJ9z4JRbWr4TxKoUfv8TC/VV6pl/x84R7NXSEG9/Ewn1Kgl1qIJx9
YLfHmP6cfWC333D+YlzcH2zrG2I4f+ekA/81xCEezftHEXx7sDLv38VCfoi/
S9D/gvZkrT/u98R4XgzGTxGOhxhPjwD/2jFezT8vCPw3OH8ohvO7ThZytEj8
gJW3ctF/2M4tS8xEUvS8O0TIz/tPjvEdjq3j76/i8tl+X+swf/Dlw/iGjq8f
Mb7/WPB3I3h/yTofcvqIYD4Twfxkbb+cPyhOtvXvMGyPtuelwHgdDuWROfj/
vL+ZCOOXGfxBE4w3Sby9LIb4l1U/fn0L4jHW98vPlzqerfk5+1OC8atInD+h
Pk3w+7jfHO9zCIf+qYD3EwPvJxHmAxOM3xHQPiPg/UTy5bG2D/59mMnMoudf
Y+MNp7/1fXH6OuH+3//Hfk2uvDEQT5BAfCAE2nsk0Re1hwusPvn8EeA/hDic
t+bfnww4nH/f1v7OjQciGB8kGH+OIO7F74uNx1x/d8J4ZwT0RxhvxBiPhfia
UySZUWRQLGO8sCj9Dlv6bL5/irN5f0gE/pH1/fPtLRjeTzC8bxOs1yQBW8Ae
MAML+1+F31cAC/aWwHLYbysDFtq9/bwrv99UD6zj4x/sPiLBPlUAR/HxEXa/
lp49n2cdpA+E/Z7C9/wEFtqBmpWHlyuAhXUvgdNBnsazOJXg8/n1SwVwFNjH
v2F98rfteYJcCnIJcDDYx6kgTwFOhvSZsP8wE/ZDykEuAw6H+JIW5GrgREif
BHIzsIlPLzbD+gOwdbzl+rO1/Px8JwMO5+c/a/l5uRo4EdLLYf+aHPbjSWF9
VAIcDPGzVJCnACdDei3s99LC/jMlrLcqgKN4e97en2zM+pONAyB+p4T0CuAo
wqdPB3kaz+JUSF8I67+FsD6sBLkCOIr3N9j+LEGuB9Y5pMf1ZJ514J+w8Vco
D3AMf/+Gffxl7ZPncLifJw7mcyVwrIO/AffBQPoEh/1LfP5kSJ8E6S2QPg78
FSVwLG/vWsvD268JDuvDfPzWBM8zgr9tgOcHgH12A+y3QOBkWM9JAjmwOBHW
f4ElCWDfJNjyB9nya8GehfNdVvuGt8/CoD5DwZ6RAL/i7ScR2E8isDet4ylf
v2awF/H+mVDwryLAX4b1Y6s9tbWoPMepoA9vX4bC+wjF+7bgfqpwsLe0YL/o
gQ2QXthHYs9fLGfnYoANDvFuW/yBzdfFLOynkgFHEozvFasnxPPSIX0Gz+IM
h/g6n18O94PIgMMpF4+3Mm+vScGekwFH8vF8Fp8U8utAHsXH+61yngMgPhlF
eE4A+0sNrIH0wrxht894eaEwjjMuLr9tf5u4gE8vLoT8WtjvpoX9bTqIl2qA
o/j1Dat+vH2oBtZAeg2sD+aCPRkM8dYEsP/UwBpIrwH7MBnkcor3mfD77YR9
c0L8NhJYDvvrFMDC/Ub2+C+/npkB8d1wWN9RgP0YA/IYsD8TQW5ySI/3j/Dj
n7AvWthPkAbrK+kwfmVC/kzInwH5MyF/NuTPhvxZkD8b8v+C/L8gfw7k/wX5
f0P+35A/F/L/hvz5kD8f8udB/nzIXwj5CyF/AeQvdIgH8fEcBdj3MSCPAXs/
EeRRsF4ux/M6wDHgr8dAPD8TypPJ+//W9sGvF2TC84Ih3p8A/oMaWAPpNeBf
JINcB+v5UbCemQD+hRpYA+k14H/8hvL95veTinNhfeI3PC8A1lOj+PUJ+3zC
ysezBtJreH/EOp/w6x0F4K9EwfqHDpjNzzaWAYdTfj0knPL+hRT8DxlwJKyn
RIJ/ogO5FPbHyoAj+fUYK/P7M9IhfQbP4gyH9Rw+vxb23+qBDfz6jt3+EvqP
bRyw+0s8a8Gf0gMbIL0B1oc0EF+E+xAlGojXAotCHPYHc/mt9jmf/jmwBOzh
z/C8z/C8L5D/E/B3yP+dzy/+DvH2KMj/DfJ/g/xyyP8N8v+A/D8gvxLy/wCO
h/qA+w4l8VAfwBLH+Dofj9fD8+F7rU56eB6wOJny+eF+Wkky5X8f2Pp7vD8J
bC0/t14meg3x7hT+eaI3sP4SDutLBv78iBjvOxTs/WSWnudwuP8gkr/PwNrf
eXk62NtpsD8O7w/QwPl9of8J9m4qz+IUsI/xvD3Yy+LfYD/HwPkRDXAMxEeT
gC1gn1rAvgX71Tp/8+ljHL6nzduzBrBfDQ7ny3l5oMP6H2/PWkBucdiPystj
HL7XytsrBrCPjA72FH7vlLd/IoGlcP4kDOylMAf7ik+vcWD8/iRvf2goxkP5
eGoA2CsBDvYNft+Pty+SIX6aDOdxk3i51X7l04fDeddI/vyLGL8HFgnnXQzA
cjh/GgX7LaPwPjg4HxMJ83Gkw/zNpzfAedIQ2C8RC/tpYvj4lTgB5BaH/MXt
8T8ipOfjdwa+/YoNfHpRmsN9bPx4Hgv7DWIh3qVzuI+Oj2fqIX6oh/V2+J6F
GL5PLXb8ng1XPqcEWM9MgPib43lDPn4rx/Ec9pvJgCPh/GAKxEeSIb6TBCyH
8TUK4g8K4CiC97Hw+1mC4DxBkMP5TV6eAONDIowvauBEGG8sDvc38P0xHexv
2M8kxvPsWuifOrCX9cA6sL+zYT9TNqwf4P4nOC8uMUF7fs7Hq639i99/BOe5
JAm8fSIR4vlPbenhe/dOZrCngK3P5+sLWGLhny+x8OW1Pp/fPxBJ6hX9xyHW
X/n5Tc+ztf9FFvdXxsXz7WvGcL8KrLergOMgfi2H87xRsF9UDud7IyH+nA3x
4iyIF2dCvFkL9psO5IEQL5YAh0G8OB3svXSI/wrxLcGey+BZnA7xYCPEd/XA
OojHCuVg+eG8sgnGGz2MT8I5Z7Z+D+eTVcDxsP81EOzTAIjX5vLjlfgXxFN/
g36/+edZ7VP+eUpWruK/cbD/VgHx2RjYbyuF8VZYJxPirQkgTwR5Bm+PWuuP
j5dmwvicyT9PnAnPC8T7quA8djDs7w0Ee1gk4jkA9vumg72bBvvvUmH/VCDY
0wEgTwJ5skN+3h4WiXgOgPMF6bA/LQ3G11RY70wH+yUN7OsM/vfF6RC/VMB+
hRg4vyCH+43kcL9RJJyHUMJ9OCrgeDj/IIf5Ngrii79gPv3FpxfnQPpcPl4o
/gXxSz3Y3waID2rhfWjh/iENlD8V4pEpEH80QjxRD6xz2N/Av69C3n8QF0A8
Ugr6SMF/CIPzIoEQX5QAh0G8MZ3fXyFOg/hlJuyXyOBZnA77J+Tor8D5lWyw
X7IgfpgJ9k42bx+Js/j4oDgN1u9SYLy1wHglB382EvqbFuQakEdC/9MAh0F7
DYD6FMa9NPY8nuPA3v8B6/tK4DhYT//hcH8Nzz8g3qQEjofzQPGQH++7CYL8
wcBysNe/glwK56ffQbwwDDgQ6icA6kcH/pMO7Ek17F9Qg78n9DP790D59EZI
j98HNfPfhxGbIT4o+Fnse6KQ3gLpLXx6sQnk+P0cIzzPCOnx+5Z6SK+H9Hr4
fQPI4ftVYh3I8fs9Wvg9LaTH722qIb0a0qvh9zUgh+9liRNBjt8PSoDfS4D0
+P1PFaRXQXoV/H48yPH7XD95uZV5uRK+P/QD0v+A9ApIHw3poyF9LMhjQf4d
5N9BLoff+wbpv0F6GaSPgPQRkP4ryL+C/DPIP4NcCr/3AdJ/gPQSSB8K6UMh
/TuQvwN5CMhDQB4IvxcE6YMgvUjEpz8E6Q9B+hsgvwHyQzDeHoLxLRDG6xuQ
/gakDwJ5EMjfAX+A9B9AroLf/wnpf/LpxSoYr1UgTwQ5fK9NbAC5AeRJ0J/x
e8ISKF8mrLdk8czOM9vvw+b8d7v9Y5ML3wcWxmkZcCT421LwhyXAwRi/A381
BfbDC3Ff+/eceP+5EPzlPMifB/kDwZ8ViXgOAPvOCP6nGdgC8cRfsD7zC/zf
LPDfsyC/BPzRMLAv03l/0WqfwvoMft8H/C0Z+GPhcF+DBPzNMPD3jLCfwgxs
AftUCf5PHNz3EAj+krVHchwA/lcS+D8pPFvbC28PG+G+UCP4NxLwf4JhPUUK
/okMOBL8EyXc96kAjoL7KFh8z8YqYBbfs3EM7J+NIYoi+R1b/evAXtXDfmY9
7GdOg/sbUsEehfseReEQ//4M9q8MOA7S/wC5Ehni/wpgIZ6J9+cLdl0YnG/U
Qn8X0rH71eE+PjnEi2QgD6cYn+LbaybEI7IgnpAF/eM3tOffsN6XB+07D9p3
Juw/yeLZPt7bGOP5FvC3gCVwvtDa//B8LvgT0F7wfqhkSI/3qSWBHL+PHg/t
NR78VbjvzCkF1rPgfiqxcF+ffT7kmJ3/FtpbGOXj0QF8e7COx3z8TgPrAWpY
D0jk1wOs6fG+Mn7/fCKsTwbw8QFrf+XXC+Mg3qTj2w9bLxLiLZmwP0sO3+eI
wvUG2P8RCe/TyMfz2X4PE5MXNw8Dk/P2hBK+9xAD8XolzP8xEN+Ww36KSJhP
tSDX8HIx3M/oZHY4j8DHWzXAWofzK/x8GAPxPiX0/xjo7yInPj5WCPE0JZxn
igG5EeQGBzl/H6YB9vclw30VSdAevzqcv+DXgyXAsD9KAudJxMhw36AkAtbf
3sF4Fgasgvh9HPRXJZ7XBn8G4g8i8Pet/Y3nKOBw4GDgALDnX/Ptzdpf+fkN
WCzsXxbs319gXxrAnjMQXE8AOdhDWrjfQQvxNvzeUxSMRwrgaNhP8QPieSHg
j72C/Wmv8Pw1jM8aivfH2MrB0vP7pSJhfk/k52txJr9+b2Ve/hvuM/ntMN7y
46sO9qNaHPw5bnxk+9/s65v8+GiA8TEQxrcAB3+Cj2emQf0I388UxoNgiO8r
4TyPEsbzGCiv8H0bO/PlD4Dy54O/lQf+02/w16TgHwr+q1D/YZA+G/ytLFiP
zIT3FQXjuRxYCv6MDDgS1sdSYX0qFdanUnD/PfgramAN5eeLdFgPSof1oDSY
X+LAXomD/Qg/YL9UPNyvAPF9q7/A82eH+215/gzjtQzu40h0YO75Tolw/4sW
7hvTgj37DsaPd2Cv6uH39PB7etg/K3Li2p+4EOwRkYiv7wBY/5RBewgnuH+R
588wfgaBPsHAz+H8ZiTsz1KDv6SB/QBRDuv5fHuF/UrsPLOwvmaB+wDxfvpA
8PcCoXzBML4nQHwpAc7PCfeJ2fc78OmVkD4O0ieCvaAB+0cN9o8R+psR7DkD
9LdM8P8yYb00A+yzbNjflQ37w7Jgfgmg/HguEvF8CNrPZ5h/o6D/fnNYf+M5
BPyXdBiP03gWpzrcX8HHz3J5Fgvny4T5OxXSpwAnw35oKdSPBDgY9uNqIX6k
BzbA+rgQ/xTmcxlwJOyHFe47YN9zAY7i98NaxzP+fiSwx6z1x8dbVCD/6RCf
gf1wsD8pie8f1vGW26/GztsJ+9/zHOZXzv6wtk9+PpXC/RbhFOc7vv8kwvz2
igQU5X9ge14AtL8AWO++AeMhjO/W8Zw/v5EK47cQr7DfL8j7b8L+VXv9gX3H
xtniv1rY/6NxiB/g9+H4+vwA/S8cOJDy83cw5d9PMOXfzyF4P+mwfywN3sdv
2I+V69A/eXkKyKXgD4fB+9RCvFbn0N/4/a3h8P7N0J4tML6H4f3FYO+E4vgO
3+szwvs1wfvVQnotpNdB+gRInwDpEyG9EtIrIX0cpJdDejmkj4L0Uvw+IaQP
x/Lg999h/3EM9IcPMP58cPjeE/98kYjnQyyeEKKo6Dwl6AeLV5madNxYyqRg
+2E9ymjn6Xd9Yftf8x49yg4/bID4VQ6J6xRyO7BTOp2+Yc62U6Xw/jwFXfSH
6zD/scJ+WRnZnd49suOkVOp9ZI0oeA9+31VPnudnjQiYm0wfdypcdWytjsXD
Go92a5x+4y37Ps7Wro8X/JI4fo9VHObh8qjARGudGf0qtIH9e6yrDx1rPzjW
/v3U7qPOlvWpaf9+arcO1bWdjfi9VDNtkDxeqRyuoV3battkltOz8V3Wp96t
wpLnGF94Xf3e1qpREF8TOW3rMihkiZPwPD1922ftqsGdE2nf9Ts7Nsu373MV
6iunUZlqVB9PZ2/sPErbS7BbVfSk765XC4YlMx5eNzx53GvH75N+qx8TkDjc
/l2fm9ukw5vG/6B/jmwwanwvBbsvYOsf/SYGjH/u8P3QFylPkv71FlhCOlc2
N301W9hXoCZbLhj+8jj2nTrVD63742Iim68e7K47ekYnE+PgsnU8Nj10/L5n
863ldnYeKoxnElr4V2qipbWMbh99NClg6g32fZ/VpTwXKad+ZOO9z41adV1b
JDKeFam+360kfq9TJKopdipRY85bZt/UHD11yOrCN6x9TRy1dZhuyTNWv4cl
mU8GzHJm4995ed2BwS9dGG9ynXFE33ox4ytz2pTNXOXGeMmdTSHiWe6Mt9yM
z9xZuJ7xv9dGdN5/md3vQq9dvr/0yHBvxnPGV4oIXy9mbPj8yvJluA9jj7Hu
Fb/W92Vc+LFVmyiTH+Pdo6IHR7/0J0L7avp+6IhDs50ZT639KLckdWFc9dJF
fVybxYxTnxkm9VvjxtioP/lzzGx3xsqMfsc2izwZfxLFj7p1xYvxq8p+ougR
3oyn76sdu8NTzHjCYW2PMyN8GI86GXzoYQNfxoPP7zG9N/sx7ntl9rD4V4I+
ahrWQ7aw/xxnxj61/BZel7gwrlH234r12i1mPN9n7kLlWjfG0wvrzhnv4s5Y
3H/rnoclPBnf2Kx5UvuaF2PZszGJnqO8Gef9jvZfuUHMuPqZKufPjfRh3LLP
wKDPDX0Z9472+FBo8WM8ZuMNVReJoI+K+oy+Unv9PGfGXeZtPfw11IXx+LFT
xj75azFj/+UrIkp6uDH++UZy6/ocd8abIz+ax5byZPxHfFSnjOtejB8mx685
PcabcZttnjMCNooZry+7ZVP6KB/Gb/ZvPzOssS/jGnX8Q04l+zGec3bfD/Nr
QR8FLTl+0PF/FzozPvpvYtLPty52zp+4+3eHxYxd/u7esNwGN8ZHHu2Zunue
O+OcsZubHizjaU+vX6E/esuLcajP7Aenx3nb8x9o3X/8JjHjt7Hnm38d42N/
Xts/yk5t6sv4L8+DRkWqn/35b8p9mRUm6COji5OfeM9f6sxYGR764/gHF8al
akzZurXLYsYnXkj3P9jsxrhut/PO7gvdGc8rPz2qUgVPxrd+1Jhz9Z4X44yH
HzUDJnszbhoW2tO8Rcw47N6F3OBxPoyXndryyu9PX8ZVfWf4TMz0Y/xoVc9h
TaWCPhLq3LtWne1rnRm/M45u7P/VhfGjRuUUrXstZlxqyNLme3zdGN987Po4
a6k74wkj68y8Uc2T8S/l24IZT9h5eXpxjeflSi7ejFv61LoSt13MOOfjCs9e
U3wYh9cJG3GorS/jgLmNGhp++zH2uOlhGfhF0EckWv3G99Hay86MW1ZJPuZR
eg7jEcoq7U7OWsy4oGvLcllBboxjBp9La+fvzvjxqhvxim6ejA+deiTzV3kx
XhH2ivbc7834crbr97SzYsY19hudR63zYbz1zzXvLo70ZWx+mtn/V9OdjGeO
3fx4fBarHzL9TvWz+QGtGAfsr72h36JJjGN8Jp3rUGkBY/WrRoXPFy5nvCHf
PbDlnLWMt5yZFr9iz3rGPv/0q/nIXj/E/3uzwbkJrH7Iu+clnweU2874fYX1
E8Ma72AcPl1n1HVj9UE+X5+xo9Ioe/kjsj427DzP/n2YKR+Xnn+l78H4xsoP
bmmm6YwHTMo/FdOJ9R/Sdk2d/Be61Yx3vasb6CzzYJwvvzSf3mH9hbgndqrV
ZfdWxukbXRbIHm5n3FHc4kX55qx/kKX+hloDD+5kfOnA/ZVe+buE/k9k9feF
j3o8iHGY65BnzSbPY1z7zSKalLuC8cNDbxbEydYxjiHyRUPZfKggJZLVS+9s
2GbP/yX38e6rbDwiN1rv6NX/IBt/SL+tlZ6lbBTKoyLXbinmeo5j8wMxf7di
EzYfkJOhrrfmTWbjP+lTZP+w8Z4k9VoyZNlqNr6TAZdLhJSpycZzcrDq2W6X
gtj8SnrF1ajfQTqdsecw07jkZ6sZ6z9fG6n2Z/MjGZV/e/3SU9sZNzvi+y5u
lfB+9STM+5zMybKI8dkeymaR85i9QNpHBZm3a4T2ZSbNKkb8oZZvtHEKGbO1
V1hm19B+xZxGRM188wsG1LRxBhGl9RJP7FPPxllENEofOUJRwcY5RNSy+/fL
u0U2tvrXuxT7DNOz+xZzHhHN9mjVeGeqjQuIaKPZTb7aZGORk0g22bV1a21f
N5s9yu9vl5Hno2/UeNjyFVlrs08FuWCfCjzTZp8KLNjLwv6y/E3Xv6W1w/uG
00jUOb9Hv298Jk9s9rMgb26z3/F743vi70uP58vJXps9j98DH1+hUbBXylci
+Bv4Pe89qmOppQ/FEsHfEOQnbP4Gfm8yyLdL1OjVSjLM5n8IcsE/wu91V29y
vvH62vFE8JcE+TmbvwTn9cnz6BKfatQT4ou5pEGgV9vhg1Vknc1/E+Ibgv8m
sOC/8fGPArLp9iC/CcOEeLGEFhRMWHf1uo7UsPmTwnqd4E/y63dp5NytVOf7
V7Wkts2fFORPbP4pfr9614ipA692tpAtNv9W8OfBvybgXxOhPde2lGx190wK
2dOt0/LFanZfCGkU/2dU+uovxNuwoFeJG6nU3f3utUFPMsmV71Xj8/uk0JML
15UO9tUR88j+71d2ttCRJxTL36SmklYD3V909NbSqDPVWgcfzydDAsTtrsUm
0l+r034Mqf2d9Py6ImbUHBUNq+HefZdvMlm4d22/dwusPKFdmd/aX+RI4svs
cm1VVDKmZKkjtxLJ5ybvj8a/+Epl96nlwkoT2f6oW2NRUzn17n1x8L9x6eTK
g2NtOp//SqcWVEqZ3TuRtIys3G1NvXA6a8SgwL4HU8nUEy0/Oq8Ip9mlKn+L
nn+cnih8sPzc7P/IqieTf3neDCdTcjz/y7G8I132bK9wp2wieT9pRRPJ5nek
8Ym/9gYeSyS/Wg7Z2GB3JPFQK0ZcrKOg8oTgt6N3RZOA6mP8akwMIU+XDQrq
+UNBnl61nDOFq2jlv4xnmwXHEfcHl16MnPKdvJr5uPOqDT+J76A2S6qV0tPf
+cM2SbokkqiyL7pKRx+mbs9eFzR4lUCG1741+9DEKPL44o/Ulf4JZEcHbe/l
MWb6ftC9MrefakjYqNPtQz6kkgsbDtXsWENLcnZUXmd4XUjoyk1NqpXSkg6N
s3IvHHhDQ4+cOe2VqCf9XkT/UkzSEemx2r9rDTaTqiv/OT74kIIuNM4cQbek
WsfVbjT1zBfSbHFwh5PW9xRQYv7oglmfyJv6KW6nktPIkoqLq3mMiiX3U9zP
bWtxj05t47YidnEsCf/dbOSWt1fpNBsHFzNLf6Q4PfEdeiJI5J1AUidWOHxo
hrBeqCVeq2fJLx4bxPhlhNv3ox9XMP68MOGsaqMX4znJZZf89vNhHHXn7Lad
44T5y0huBp/ZmL5hOuM5H90qkx5ejEdMnnnOqfVOsnr8k+8zv1whd29OmzM0
S2KTJ9FqoVv63Z/pzPjvHm3ut2f+eBK9V7q8ahvzx5No2Kot+mXMH0+ibis3
5rRj/ngS3XjxYV4T5o8n0R1RZlFt5o8n0QOV2pSpwPzxJDomILD0NuaPJ9GI
5592ejF/PIlO/WYs78H88SSqTC63dxXzx5Po/Iqtqixj/riRzk0OrR3H4gtG
+lysXzj5lQvjhLCncbOZP26kS0Zu3ntwtRvjrMVNG8UzfYz0n3bb3XOZPka6
xZQgrcXsDyOld4c07cj8cSMt0ftrnxbMHzfSP+uX1nZm/riRDs7t/m8/5o8b
qWvson4jmT9upH4hx3VTmD+upd9Tl9eZ4eLMuGKv6Mf/sfiClh7dtHXN2raL
GV+eUb1sIYsvaOmMRhFj37H4gpaqKp2uV4X541q65PeCxElXWXujycb2906P
9Gbc121TSArTR0sL1XHNazN/XEvfzB7s34v541rqL79unsn8cS0dPabyxK1U
0CeB7r4UHjJ4rjPjgWv+XHHjtQvjC8OTN39n8YUE2mTa2NAYdzfGzo+mlE5m
8YUEWuWF7/U1JT0Zh4c+GpXG4gsJdO+HBMu60d6MWxl2SXNZfCGBlnFrdrov
88cTqCbl6YotjXwZv/UY1/9lkh/jKznaqgUsvqCk1+rdbhg335nxlDO1R/QI
c2Hc/lu71y/bL2Z8Z/bIuHnr3RjXdQ/K3z7XnXFUjyH7JpX2ZHz819cGrW56
MZ76cuHNrLHejPdKfjV09RIzXnFpYGL4aB/Go3133+rRxNdensURa8+n+DGu
PKL+P+XeCPrIaYWhLnF3Fjkzjhh2cETKfy6MD2ZmV/dj/oScPh5+I7+Glxvj
pDEd1n+Z7864hKevqU45T8a1Lyjnz2b+hZy2kXZXXJrgzfjcxmHXPTaLGbfe
757wh7MP48CL5xu/aObLuM+TD9PnpPsxfvMh60iJd4I+UnpRU+p75ZXOjBeP
O92h82cXxgVTorPad1vMeKTvomlO29wYj4rOmXVrkTvjNSkD9gVV8mR8rJxf
yKsHXoyfN/lolk7zZpxyetcBw1Yx4x3Xbl9LmuDDuO7Dz6/SW/oyvvEyNSon
249x3/e1k/M/CPoE0p79PtaYvtmZ8YqbOyq2/+HC2CWmetkpAxYzLqP6ubP2
ATfGUe7fSxlXuTO+lTkg7H1dT8ZbN17zvfnKi/HE31WG7l7szbgWfe7T3V/M
2FuctKDlLB/GuoF/Dq7d2ZfxhFJTW5QutZPxi7DdpTKimD7kVOOvrrXeD2Is
/9mwTsWC6YxX1dP1fTvalXGak5ObX72VjF8EjflyKW0t4/wRv3Qlo9czJj+u
ihZSpg/ZvnZi/TdXmT4k3qfwZI13TB9S8tOUqwOfMX1Iiz/uPFh7h+lDhswv
+eriBaYPWXR7+oeIw8J8LCVvtrhtN1pGMg7zHRtrMc9hfKRVWp91a5cyXhDW
JKzXkzWMf68PMz7utp7x0TbLa3S3ty/SIbpa74edttrzv53Zud7U7Yw7nb87
6PvRHfbneYqmHv/K2hP5b9ykZVNqCuWVk75vZykyoiYwPtxOv0Tru5DxzjWX
Fk5ssorxy1n3Ov+c5cG417a9HY/dYP2ZBF5a2n505lbGdRIvHRU1Z/2XRHr8
e2TWf6z/koNlxYefuPkTxefsSYlXhXV1dt6TZN9uqy39U2gPSnJK+iF21Nxl
jM+ec7q0u806xqaoM1FuF9h4SRrv2lzmZiU2PpKwljebnhm+g/GK11H/7N8h
1GcCyTpTv8Q/Pq0YH1686Mr21PmMH56arVw9dC3jRuKd2iaH2XxDBnebnrM4
mc0vZGNrN+9Xddj8Qe7W9ylZt5/9+6jPMl8eLnkrhDz+N/3mjNL275uW7+y+
ZIARv0eVRK46qyY2GP2c8eAOD1JuH95AXm7x/7bh+Uu2H4a8rbX+eOXnZP6x
qZf8Dz4gJ3cGttc3iiSbb89dfqLNSTJpcr2OLQdEkN4eunYx74R9jwHUVfMx
cbmfM+PwMP/jFXQujNd9PPrj3tDFjMVBmmTFCTfG75wbbi+x3p3xmDeD99Rt
4sn4a6+VR9r/58V45p2jZ53WeDNecygu8tY+MeNHH9yn9ZjvwzirdDkl7eHL
uA85M29UxZ2Mvb06a+Rx/uTH+x8dDE11ZGWpC0/uLT1OOr8qXapxTx355/yd
R9qSwvcFkqmyW5Wglcz+TaaVO+dsM4a4MK6TkDFxHLN/k+mVO5ZF7Zj9m0xP
9V0XUoLZi8lUFZD8NqlgPeO2ZVd+ib3kxXj1Cr1COsyb8aylf6fOZPZvMi08
nbJ+ArN/k+nlT/dyhzP7N5kOE7ltG8Ds32Rq/Lt96R7M/g0gtdbKFleNHcH4
+fWZK1rsncV4/d5rEXMnL2Jc5c0FS8fElYwneoeXLujN6o80qf++5iK2HhVA
jEFvm4Xb6488GR/aqdtBVn9koOfcJmNvs/ojbcJcrooO+th/r+bsDkHurP5I
+tyZj5ZMZfVHou9O79OwjzA+WWibqqZWmax+LPThjPCOa5h/YqHtjk9QP2T1
Y6EbC0Me3GX1Y6Gu3Y+Syax+LLSyYeS1Ccyet9Cgc4WVxzH/xEJnT3y8bgzz
TyzUZ0cPyxVWPxYa5XIo4DKrHwv9q7dp0iVWPxa6teawchdZ/VhohDng+QVW
Pwb6oezQc93ZeqGBKirMb3+T+ScG2lparfRZ5p8Y6LxKq90TmX9ioGPbLFlT
h9nzBmqUHm/zD6sfA/Vf/i5uNvNPDLRl5axjYuafGKgq6nDricyeN9B71/8q
6cr8EwPdsjH0x3rmnxjoqJEzn/kz/8RA6zdIO3qK+ScaWjBo/+r/mH+ioZr+
0792Y+uFGtra4pH9mvknGjrjRf8yy9l6oYa+G7KjQgVmz2uo6vaS6i7MP9HQ
XzXH/PGA+ScaWnNTlyZl2Hqhhh5fs+dwB2bPa2iLK3nHpjL/REMDv686JWb+
iYb2rxR/9hbzTzT0/YBJAXLmn8RT8397Z0Qx/ySePq5Wp1pvtl4YT583frhx
EFsvjKenH0Ublq5zY/zyivjucLZeGE/vrq6tjWH+STw91+dGk1VsvTCe7i/T
b1oJtl4YTwfodcNHsfXCePrkU1f1EeafxNNOjzaLfzD/JJ5eO/m2URu2XhhP
m2yt9mwNWy+MoRua6crdXeDMWB64v6SIrRfG0Cf6I5LJbL0whpb8pdrWztON
8fXSJ5oWMv8khn7a13+bgfknMTS9tjruK/NPYmj9c3sGvHL2ZhxwuN44OfNP
YuijKNPWkWy9MIa+a0DvvWL+SQyNnnM4rhtbL4yh5kuLqt5k/kkk9S/d8kby
YmfGH55N8u/w3oXxjeBR59Z0Xsx45VFT1qhNbozr9R524/ACd8a5V+t+K2T+
SSRV1tKVcLvrxfjV9qedYiZ6M5Zlv7nbgq0XRtLXnToMiWH+SSR9uOhozCG2
HhJJL5/NWzM8w4/xka+u5QqYfxJGZ76YtmTDKmfGezPrdNsvc2H8uKzTjOTu
ixlfrZlZ7+R2N8ajqk1Z2WyJO+NBxzqHe1TxZNynYcUO7x95Me56Ub238Uxv
xkeP/MzsLRYzPhQV3mrhJB/G++o/n7K/tS/jXbOv+z395cfY98LRx/Hh/uSx
zf7g9+sFk5MLhkvmXOlNBN7ezUvceuRUxp2Gr7lxuvtCxmczLle/NGQF4zIR
K0Im31rLOOS635rpD9czdt96oeXs516M2015ppgb6s045diyd101Ysajt3sG
dDP4ML62csemHhZfxiWnH5rcK3UnY5fB5zr9kynMR+Hkx5nBpsovBzHeUyPZ
/VXF2YyP/NWk9+ZZSxh3WbzgzMfpaxj/8fTx6ehS6xkXlKtwWsfWp8OJZvrs
U5kztjLWyqeFb/fZzniGd4nw8oodjMNb3f54sONOxk6fJ3+su0MobxTZfju+
0tCToxlXWFArU1o4n/HfMzpdF9d3Y9zxxpUOZ+t7MH7d4mGZjSyeEkWmXKBx
k7ptY+yxXxv7xzwfxgNGu0n9WvsxLl8hIyjdLLT3OPL86LTZOaMnMy67Y7Su
wfcljL+f6yFrFObOuP2mtx3unPVi7HT7Vti5PmLGy0+luWuu7WB81L9P8461
hN9LJJ41Xrglvp7DeM65q7syz69hPM9T2cN9vxfjB9d3r83uv53x37VbSXs8
9iP7l+7ok71Cx9p3aOMpAYlWe7Jxxyd939S9QJ+m58s3GvD7rjqSM/hQTNUh
M4jA0/fMzDfvWMZ4pteTYYa/Bf/FRO78O3h71ZGDGB+9O7+MuZoX440eYxvP
MrH5l4RVmV7vwIsJjNv/82FrtRQ2v5IXPYJy2m9n8ydZc71E/bZl2PxISnW/
2eP5SGafkJlfWuW0DJ3P2FC+0cAAZ/v3fGsfq1nqjzihv5jovgyPAftYvNVE
S58f2rM0s2dMNH7ejdbNmT1jopK0jn87MXvGRMeFRp+/z+wzEyV7z7akzD4z
0U6T5976zOwzE23S+M8uccw+M9Fy69Jisph9ZqLbvn89V8DsMxPN6vt4XtkG
9vKuCjjeoiqzZ0xUW3qjtg6zZ3T084gex1KZfaajfy1+5ryAxVt1NKvWSkMr
Zs/oqMJj8+3TLN6qoz+Wv33iz+wzHR3b8NMfocw+09FXH+Ub85h9pqOdtiij
e7J4q46Oq+t+LpjZZzrqn5T56Ruzz3SUvvMqSGP66GjO+fxO1Zk9o6NdNmyf
25HZM4n08KMxES/Yfq5Eej05zakFi7cm0jehF8Z4sHhrIn1Y5V3OVBZvTaTj
O3hGnmX2WSKVTtjdtiqzZxKp04azW7ex8SGRPj17X57C7LNE+sf8+XUeMPss
kX5NXy4xMPsskR7c6bHiT2bPJNLR9bfWncXirYm07B3/10dYvDWORs3rcqU5
i7fG0bjOJy5teePC+OYfe//pzuKtcXThiQ/Nb7H9XHH0QKUyZaozeyaOrvjY
8tcHtp8rjo7YO8Tix+IZcbTVaNd4JxZvjaPby4Qa85l9FkdXayJcZ7F4axyd
8zY+Lpjt54qjY66mTq/L4q1xtI9fia/rQgV9omgTl+zwg67OjCdrVhUo3rkw
rly1a6XHHRczXn3zrxP5bD9XFF325+EaLVi8NYo2SDkwbXhZT8YfX+49t+I2
G7/plr27Eg+O92Z84k5o7Ri2nyuKrqn1utfKsWw8pyM20ZkFbD9XFG0e/9L7
UBob32nu8JCAP98K+oTTkR2bbKq23Jnx4dCEP/uFuzB26jVrSHDXxYx7Vxm6
YaK3G+OTa/+Y4unqzvjo7H2rBlf0ZHxoeIld1QPZfEn3/u158ccUb8ZvDqx3
+eYtZjywdBfji/E+jKmX0fNSC1/G/ZOvlNqd5cf4hevcQ6vfC/oE0ysnZlf3
X+/MWDJN1mvjdxfG37s+W1e3z2LGn3vt+bhmlxvjhPN3/120wp3xj+bHXx6u
6ck4+spWA33G7Bcqb7OkjmUes1/o0m/Xm1XxZfYL1dY9++TyNGa/UNfph8b0
ac/sFxp/yjdBVuDHeG7sxo1LIv3Jm19aeYfSoSS4buurFb/spntW6n7fTA2F
+SqMBN4fWKbymPFE4NL32j58Gj6PcRffwPnL1i1j3Knj2V1VqqxlPKTevHmL
561nPLNUy97Ubn+SNUm6avXKbWX8xvxtXe+O2xk7Jzz9qZ66g3H091Oj/xUz
e5Ms/LT5af9b/iRm3ox6DRpHkn9nJc2v1+okLZZHkrBeO1bSnFJE4AEtI1ZP
2T6Usbiim8fB+MmM431Su3iUnMv4Yv5zzzv/ujKO/Cdf3uTSMsbWoa/bvw9X
Ma4fLZFuj1/LuGt2fPbzZR6MR9Up1SYjlfkXZEG3llM7bmL+EllrmShZ/K45
41O9mzfNSRvD2BxTd3j75zMZv93nTNy8FzKufadGbNi5ZYwXfpR7Nnq1mvGM
zNEV+iQz/42U8Ov4fRGbX2PIzdrVrv7L7It4svJ8A0v2rUF2vre71ID2rozP
z9SW+fltLeNyceUqNj1uzx8cWNu/M4v/WPO7t9o9bi/zV0mz7j33rl4h2KcS
ovsmHZbX+jJZ6pQ5uU3F1/A9AjN59qepdVyfsyT9H1PMP2n4vXuRqEbx/nOy
s35UlUWxh9n9FiGtZHc/Dv/MOG9W0J6A/bF0yfMbo4Z5p9GF75dOTJpkIGvv
rx7a53YafXdk7vDSFQ2kUd/WT0qUTqM/Gl94cbp8JHmlcH9Q8ZyFbu3lm7Ap
PYMsizl8bNWmJOpaXzq46acM8k9odGRB7STaZFAr0/Jl72mHGW/PGY1GuuL7
pm2uHyLJqsE9RJXum+jfGu+M1BZfyaa2hUc7xhrpkc+TklW95KT8s8m/ZtTV
01Gtnr18tk1LLhrnde20WE9PjQ3aPiEwjSh2/DzuvU1P//z3ysNxqffpq5xb
sl2f1HRy6ECfuyO/kcm6R59uj9fQQ3+cG/b9rYbsm+Tz36Tfavp+98b6/zzU
kD6r5tU+ekBNu/Ytf/5NZAGpt2rfso51NDR1WGAX59MaGj0v+mqGMZ6O8rme
/KWnhhx2VUXv65NAz4avORRy4QHZGTpyR1rTH7Sc75dTygVJRLHy4cHc5T/o
2JaZT9fcyyEV6Z/iDxE/aMUXC7XvFnylh8/VvddRHUUrLTi8Y2twBpn6tY/i
6PZvdKYuak79NlIyvd1uermajNZdtmp/u7IxRDJwVMKNqHDadsX1oPdbjOR6
XKXmN/+T0Y19H3w88iiNBA3s7LE/Q0Yn7f54IWBzDHmwx3LqZMBb6tV/xK3X
/1n7g/Ta4LPkNT0/ZOeeU+UNpG+qaUP8qHf0SvmzwU/pE+qf1+aJYcwn4nJT
XiVvipRo4qbM6vniCxF+/3nx75Ok9d9je3QyklKWoHXNVn4h1Wzln19cfjI9
e3L/HlUzibc+pG9y7yhy2yd9YJ9vSlq1a/+Pvf5UEvmn2P8WyYPIhctuA/y+
xRLzp5/a442SyFPDqQNVLyiJ8H4PFb9fsjt+VoMvzX6TFifKVmi4KJ58EJ0Z
fn2VkYYtMv4a5qEmlbs/L5/7+T495dNv7IwPalJe0yI3Pvgb6bz21VL3Rmoi
1O8/xfVLoP6Jb9TzU/LuqeTtjVVrhm5UE6h/cuD46d25I+Tk8KG2eZ8TdIQE
TVO0nK4lf0c9a7rlmY5Aeybd+fZLhPbtVty+idZn7aN2SVpyN3/tmtseRiL0
n5fF/YcstPWXnsX9hXjz/YlM190vX/3YN/oiQnni/eYUEn73WObf/WLoweTG
XStWTiPQPwn0X9t4wM6bkLez3OaGBunYea2HsbvulNsey8aPIWd+lB/pnsC4
7+XbS676J7D1k4uTnoV9qaYmbqcnZrqNmUE6Py1dY0tbNdENHbk0oOsa8rLB
eJe2A9Xkb03/Sa2qbyJjtp65KZ+hJmRQ/aZTsreTmHhtpthdTdT/Hjl6QruT
pJb12tk2U03GeB9YGzp9ApE8LLnCNVZNUhZ9zx880o0cnLNvXMBrNTn7bt38
WukbyZyKdbsrr6tJM0v+/NCA7aTjkwv16lnrOS1rYK3f/+0ks7+8mlvxookU
7oyKNuf7kGNBld9YqhlJiUn+w1dsmUE85W+yLg80kqHHknTHBm4i0zI2/TVz
nZFobuVvLV/Tn2gmLt27N9dI1vm9GX3DYyDpWK9OfucwI7llOZj9ybyBePx4
vUq+30gmP/7qvuPbTqKIP1fz7QQzmazXDUnzuUqmeWds2FLBTH8ODfOob/X/
yreevuV4TTPt+b2f5MhLF/Ls8wtxYEMzpW8rLfzUejFZvqGp34eWZprY9JdG
tsqNNGy2Y7e6o5mW6r+u+3Kr/xcu1e4v6GmmdYyGzBXW+WnL2lGH6zmZaZsT
8566Wf2/jg3uHf97pJn+M+T7xtVW/+9HaI0zYyaa6dWc1ueCrf7fgRXrLyye
ZaadxiV4P7P6fwNqR18Wu5rp06vn5jyv70uSQ/rdOO1mpk5508kLkx+5sCjg
TpCnmb6fWLtZyEt/ctOn0q4dWWaqtemTE319xzKLmfax6TO865Bt49Rm+tqm
z/Fdqk09Ys1UY9NH83OLZ8NIMy1t06d7r/ruJd6baV2bPr4HHrvpqJm2tenz
VTNh2acn1ufb9GnRP8n10V0zvWbTx/3onnmnrphpZ5s+oabWs7edMdNgmz41
Br+ZtuiwmQ606TP/9NxJo3eb6QebPutD1p3fkWWhM5opJBEzncl3ycH7yywW
uqDyIvFYqz7/vL0tGae20DGlO1c/ZdXnzPv/InrEWuhSn/bz9lr1yf+UmNAw
0kIHiWorBlj1mRdZmF7ivYWOJzVFva36hEY1LKOnFjpna/W2Xa36tIrtVffz
Ewtd+bLq+L+s+vj/nNQm6K6F9t9aNuRfqz7GxNW9T1+x0LBzZcYfsOozRr93
pPiMhY5+WVq9x6rPffP1mYsPW2iEstRGf6s+NVLfrBiz20Kn55Ws7GvVZ+/g
ffO3VLDr046sGX+8pl2f//pMJoEN7fos6tm704eWdn1K/d2osbqjXZ+AjqLK
BT3t+gxop879w8muT2xLqaHrSLs+Xs3uKEZPtOtTt9Gh/xbNsusT9IfHk22u
dn0m1Jp+9ZSbXZ/kqv2OPvK067M5yX3NjiwTdV53PsvV2t7KHlsxZZnFREWh
Q57/sOrzb1/XPuPUJtqh1bktpdssJo0SZjftEWui0w1/NGqw2o1c3zWldMNI
Ew2d+v7HCas+f3d2Nojem6gieOfdS1Z9Xn4b9llLTTSpwRDvu1Z9Rmwhj8Kf
mGhp7xJjg636fP2z98mHd010/+q5B+Kt+ri87+J98oqJnmw2IEtr1Ue/ut2C
rWdM9HJEozlmqz7udf8c7nrYRO/5/H6batUnP6RBh1G7TfR5t5iO2VZ9/Grs
P7ilgolOtumTu/zoreM1TbSkTZ/VYWfeBjY00U42fdSNL6s+tDTRmTZ9Zmy4
9Vvd0UTf2PT5LHtQt7CniUbb9Bn817Ou9Z1MNNmmT/AOyZhuI020jE2fjj/+
WzJ2ookesOlzuecXnyWzTPSUTZ96h6LObXc10Ss2fQ4YfgSfcTPR+zZ9Sg3W
fH3sadcnr8aaG1sqGOmmFl9PBVn16d9ieuzxmkYqmddb0v6VC9na3anqg4ZG
mnDizakBVn3o0LYDP7Y0UtfC5ZOW/C++Na26h6ajka7rtrus1KrPoKU51wp7
Gml33eyFyv+NB16q6PpORpp5uoskxarP2z3Syt1HGukT59KNy4zwJuXOBhLn
iUbat4Q5pLKnmIy4e9J96SwjbRg7xVR3hA/Z80p81cfVSH8H0frNG/iSj1+W
Ks66GanyQLsR7c1+pEr8+EpPPY00ZOkRzx6v/EnppIPTdmTZ9XmjvN5ymcWu
j89Hmuqstusz8Pn3V91j/y99bibvbRBp1+fViXIzRO/t+mzZ2bS1ltr16bu+
V/rHJ3Z9fi0cJ3lw165P8MQl+09csevjOXDbTO8zdn16dDnRZuFhuz7pTe5n
jNht12di6wNxWyro6aCS8e/mz7bOP1cXDD1RU0/9r1xc/s2qz6sWve48aKin
WQsNOSFWfTwuVaoV3lJPb2Zu71dtjRv5q7nKS9tRT+8NriLpMtudqC4EqUS9
9HTXvuj4GSJPcrzJ7uENnfR0/rerpXZc8SJjzrnc6zFST/s2cW91x6pPyUZ/
1xk/UU/3dLtwefn/9DlddsvyWXqaPc2v+zarPqvqxyb4uuqp65YVb49Y9Wl5
8v7IC256GhEwYeoNqz6xdX0Dn3nqaf+3vXQhVn3uRa9tviNLT/vZ9MmZee75
Uoue+tr0GaSUTnJW62mmTZ99LhnmbrF6esOmT1Rc0531I+36NJs3ummh1K7P
8njPYDW16xO04NKED0/s+hQkfjLev2vXZ8SiXzuOXbHrc1jbsvHmM3Z9lEvG
P5l32K5PK8PmccN22/Vp3m9o/x1ZBvrHvMoh5a363J/aInWpxUAvPIztuNWq
T/+1Ja46qw301GlFwWarPh/3/pzePdZAE2e90QZb29vMay8rN4g00ONVp+ly
re1NLzkjKZQaaN/VY2bVt+qzIdbLQ0MNVPVl4JdeVn3KZE1r+/GJgfp16TV4
qlWfo9V7KgPvGuixEcdn9bHq82f72oeOXzFQXc6hYyOt+jwYmjZ4yxkD7XNz
75fpVn3IPFn2/MMGum/GzgpLrfp83nTv9vDdBhpXYfvgDVZ9fr55HrqlgoHW
tOkz4aay6omaBnrWps+bAwUzHzQ00JM2fXp4NLv+saWBJtj0uTZjULqmo12f
esTqU/ey67O75c69DZzs+vyucON795F2fVYmv28xbqJdnzi5afWyWXZ9xj2v
ErLD1a7P6wudy593s+vzt9+EycGedn0mLU8ptSNLR43pcV0lVn1uBoWLl1p0
dFfIbVNX6kIKC27kj1Xr6P7Cbj9LtF1Mpoz029QtVkejmh2WLre2t9tH5mfX
i9TR6s1zFMus7a1kXH+PAqmOBnW9Pva8VZ/pbRukJlIdnT5oamikVZ/77llu
75/oaP7EMr3KjfQmZV5GGu/d1dGIrjHfzln1mV3u/pKjV3S0xfOu1YOt+jyc
sFftdUZH1w/aPSrSqk+Fs0vmzz2so/99UPmarfrM1Q6OG7JbRxtM6k3LUn9y
91i1vC0VdDTLps+00d22naipowds+pQoMa3kw4Y6esimz+0nm/zCW+potE2f
KSvPl9d11NEaNn0KmofuLdFLRx/b9LnxXVO1kZOOzrDpM2F/hcM9R+pogU2f
34M61pkw0a7PlZzxJ1fMsuvjfM+j4U5Xuz7ZC0+eD3Cz63OxfkjzF552fc4e
XtbQu4KW3p9S3q+lizPZoL4YcqKmli6e0qfnUas+E3vGuDxsqKW5JKf2aKs+
HXbVLAxvqaUDp3ytGm7Vp1zMqABdRy0d0mOa7zWrPgntdwws2UtrHQGGlNBZ
9QnxfpHQyElL9436e2ubq17k+Jf0Hb1GammXb01/L7Hqs7Z5h1YTJ1pH9Ptl
+8mt+oxZ5/pu5Swt7bfee1OGVZ/Wb88u8XfV0gN9MoJr/29/xR/fyl9y01JV
4fLs7hbr+La0yq0QTy3t+ja+xxSrPq5ux9U+WVoaYtNnd8M5g5datHSlTZ97
71tdGqvW0nybPvINFlG3WGv5bfr8avV4br1ILR1q06eJfMurfKldnyE+Qxon
Urs+y7tU3iJ9YtfnYNzXmLt37foE7Tvzz5Erdn1i+iw8ufHM/6WP4a9sl8N2
fVqdSJsyeLddn22aNfO8K6ip19shy0rMcSb7/1QPPllTTWcN6xWwUuJCzsyd
1uZRQzXNr60darbqc/Psh4qfW6pp+M7AvpfWupGn0f2T9B3V1HnLy3odXNzJ
27oPIkr1UtOejy/V8i/hSb5Oavm4iZOaNknyr5pg1Sf+0ImT/4xU07Jt3Cr0
H+VNUj5V3DJ5oppmd25XcswGMSmouHXu6llquiL8zDavkT6kyojUQXtc1fTn
0qr516z6NPRzbX3VTU0nltm+SW7V5/9wdd/xVH//A8CNMppISSpbJWUkKupl
ZI9KSmbJzt5UIruUzBKyV8MWCh0re6+4196yKcmo3/19XW+PT/33ct6X+7zv
1z3ndd6PR+fFXfqNuthhGFXELv4iLfaBx4deRLsvDSMLose2h6nQaHoY3SR6
DGJSOhWHh9EK0aOmd/YnP34YVRM9clyltAwtw0iR6BGZUD69VjWMBImeUx/w
8gNoGB0mepgtjY0qcofRNqKHRuCnx/vUYbRE9JD9fBwTkDCM7hE9i3l7iuwj
hlEv0TNyP6JLI2jL03Hx+BI82fLslDtT5r40grJ2DWm+IOTbVdUFUuPpEWQp
/2FwjZBvobczxZSGR5CET/bO1wQP/p6VqwB+BClO0CSdIXhYHHi/MLSMoPec
wixjhHwzfDyztlY1gjI7DMwFCZ73z1JFBtEIyvUJ+fyY4Jl/ZXa/MncEFZ4r
o2wk5JtQ/MlPH1JH0MAPJ+49BM/DtO+/AhNGkL4STbMgwVPy6a2QY8QIGk1M
dtQkeCi/GttrBY0gkz+XjroTPIpNx3LEn4ygqZsd5SmEfPuZWyfqQr3lyXi/
+88rui2PaYwyymLa8nCGvnCr59jy9D9pEh8/teUJf0RLTi685blhq1J+RGzL
s9c42Ouc3JanRqtN+rrKlsfz2n4qC80tD0jfrPbV3/L8vvDqabz5lieHt1Ph
i8OWJ/W8ibP70hA6LUL/8AEh375u17I2mh5CJDoB54cJ+dbTrGyiODyEBK/4
aUidMISfb8R1+fFDyMRSOeygjTnsvieofqBlCDVWUk3aE/KNU+jYtdWqITSh
9v1JH8FzkeyQXD8aQuTjtcflkpxBtWGX+NfcIXTEKbUyi5BvZuF/z71LHUKH
PWXO+BM8HoYLfC8ShtAJxXyyMoInQmDkuF3EEBKiP9Hyi+DJ+vONRT1oCEl2
v47lmfGCmpqag5eeDKGrCTusdQn5dr3ipbcL9RBiI3pEVuL9w+iG0Ir2hof9
dObLbKYhJED07Lz7JaqBYwgZET2LobVJE6eGUAPRg6/uTNsmPITGiZ6y9ZFc
ZrEhREb0fOBf/HJBbssTYkBSeUNly+PyenejleaWx6D+0Dc//S2PMunxviTz
LY/Q2bNjJQ5bnkspT9JdqAeRboCS1x+CB5X294bRDaJy3uZw/RJtEOsR3pPD
NIg+1Lr8TiB4Sn75X2zkGEQOf4MSXxE8EnSjZt9PDaIYxrbjXwmeMp6LkduF
B5FXdu+jS2SE+VompJZFbBDdU5poyyN4vupOrojIDaIrY4vc/AouIPVQgltN
ZRCFi7mKtRM8FS9fq9toDqJnzz0naeXdQTpzzve5/iB6hHvyUvmwJ1TWyuSn
mA8i62MvxP0IHpnRqLEyh0GkZxcyVUnwlKo5XHVf2vKIX2RmNZre8hSzVc0r
DG95gMq6lA+/5fkyzRi8v2XLc7G1VG+lastTmG8q2Ie2PCJR9NvLc7c8BR5F
7SmpW54LJoZJzxO2PJ+V9zraRGx5zgvmy6gFbXk+MeoeFH2y5aHbJ7vzEXU/
annTwchxRxnuWycGvabrR2yl4yKepdow2Eh26CNTP0o9ZHv2AbchyJ++E9vE
0Y/k2uYVc2zNIetZ0fGpU/1oQfpcloOODRyaPJRBKdyPiuN/XdxH7gDuck7C
7GL9yJ8krzo92Rm+J7d/uUT4RmlpO9xQVHQBFYoz0hoq/Wh/T7iVnZMbfNYP
qLfX7EcP/e7nlhI8bGXTqoH6hG/geY01miOe8JRVofuDeT+SHz8vcXvWCxZc
U/SqHPpR5ktG39QSH5iZyvvxeKkfNRA93v4a5obT/YiF6DnKtz4iP9yPPhA9
uc3ROrz4fiRL9CjbSnzb17LlGaUfubJcteV5lOtT1Y22PAducYuX5G550n7X
fUpM3fJIR1gKPE3Y8vSK0r23iNjyOPTmsF8P2vLscVOLFH6y5TnJoMPhvjSA
sro+RvvdVoY27fwaw+kBFO5I3jVDyDeXBDprheEBdJr0wPxOgodr0oyBDz+A
/KXf2bMQPI38lUX0LQPolxSzyyGCx8mJVf931QBKNjZBCYR8Y0UPdvSiAaTm
l03KS/DUbu/IKM0dQJRp65KfCPlmp8inlpw6gLTG2ctYCZ6jwU/X/RIGUM5v
2WFDgqeyazjeKmIA7dppsf0DId+sWUD+RtAA0j8czDVPyLdDRq/nzj8ZQIWn
8mWECJ728Vi8C/UA+kD08MTJnH1NN4BeEj0emtP+OUwDiIfowdEHjzdyDKBn
RA9/wzmJyVMDaIno8fXpjaAQHkBJRE+fmOdPVrEBdJPoObty4spFuQFEQfQ8
y25MuaUygDSJnkEzezI7zQGUTfSc52LSeqE/gHYSPS/6ij++Mx9AekTPaJjh
3gqHAVRA9HzbvcedfLoPce864lFNyDcaezaaOlwfivdrl+cq0wa5bqGokKo+
9Fp2z/dFgsddUoFHO7cP3anNTFO1M4eCd7c/cyb0Ifrkd59wBM8PWjvZmcA+
5GS8xuRHyLdTzr4dua59CH9C+ZFIijMY9kfqu5r3IZiM7p8k5Fu0TOaCjGYf
utbhxNpA8HSmfXWjketD5HmxKlwK7kB7ALenS6gPfXxV4/GIkG/yLjORsRx9
yMhpMaeDkG8ew2Qn79H1IUb1w6OnS32Ae2HeZY2uFwkRPfNPe2bPcPait0RP
Pnu1rqlwL4okelwLc1rj5HqRHtEjfSNGCqfZi/YTPbtn/PJoLXoxT5u34wk5
t17ME8GsF+EW1It57uYr785P6MU8J65dcJ3N7cU8cxOc81zVvZgnz51WTwff
i3lcmNbbQqd7Mc87zZcza3TdaMn2dpGLrjIUq4TWCXJ2I+lafE8HwdMhF/LO
TLgbyc54WDieNIQpsWDfBLluNLPEaEFubw5k54IMuzW7Uc3fb7kyt22AgTfw
Mr1FNzqyLT539zbC/eEKYFN060ZWVBa5rQSPxJEXJJ5B3ahs1/nc10oucIve
v7cgoRv5UZHheZ3dwHzn88LF3G6U83QMF0TweJA9Cz9Z3Y16d9XjfhI8Yb+f
OunhuxHliyzcrTkvSJt7cjNiuhvx0YbhCgiemwkmtOTTPYjUbsMjlnrvTC2u
BykSPdy5pjeCq3qQAtFDj8wcNXN70BzR86fS/DV7Qg+qJXommiwKJgN70FGi
p7XLsifbtQdZEz1fBq3+PjDvQeVET8qkNetlzR7ME/zDRnKXXA/mcVm3NWgT
6sE8xhT2PpEcPZhHZa/DW326HszjcqZwfp2uE/Vpq9w5rKcMic63q4Q4O1EO
02lG86/aUI/Ioi2FO5GoJyNpEg+hftueZJ8i14miViLsQx3M4YiinOKAZidy
O7fXXuKODUgFTbEdsuhEmbZXeS9sdwDzzhe/Vdw60WBa4AT/O2cIPXqmyS+o
E+373hJ/4ooLFOl3JJUndKI4uh3pNPfdYOSds8t6bie6kMH97JkiYf8zf1hV
qLoTtSopmFAxe8JZ4WJuS3wnMp00lfac9wItFz3SlOlOtO3JM3aSch/wqJPc
QzbdhYaJHk3vGOFqXBfKJXrOiK3fCajqwjw7V9SfquV2YZ6h7NzsowldmKfA
fF/PSGAX5gk+ZkWR6tqFeUwH6njtzLswj2TECXURzS7Mw3TD251MrgvzLO4Z
el8t1IV5aqugPYCjC/MkuEf+UaPrwjzl+/a9JJvGI37uVHrJu8owHFYkXoPD
IwFZ5rNh5dqw7ajxdGAVHiWtL1pSEjwc8XSv1XPx6Izara+lhHy7fLzoMmsC
Ht25pbMbEfJNP9VobjwQj7i8aPIjCfnmKUAXmeGKR1OZpXfvv3WGhLxCGSdz
PMrqtdt9S9kFykWNFkETjwq99nWHE/JtuIQ2mlIOj5gjzTKpCZ5tMoXyjUJ4
5J711dvpqCdw1BkuveTAo5Gqo1pjhHyTukYbp0OHR7J9jvw3y3xAdGrKZJ0O
h3kOG0l8OcuJwzxrA6/oLIRxmKdba9owSQ6HeQq/SRT0auIwT6RK2F4GCxzm
eVg/rXfFDYd5tGQl832CcJhHtCxsV3ECDvMcvjRzZzkXh3nW8iU/8lXjME/3
mdfUJngc5ilIm9GOncZhnqzFYkfS6W+oWyqVf7e+MlgO59BU4b6h2c5BKvUK
bTjZnvLWv+obcvJxLLxxyhDGv0ZK3MglxHEuPx45mkNCbgCeKeEbKuwVP9BG
yDfdZE+7wcBvSHTqctY4Id+OhDntfutKGP8to7xOyDecr1mSpTlhnFLhO+1V
F3jpfAeENL8hnR3HJIMJ+Xb9nmrnmuw3dDTweOARJXfYqylrXSb0DfUycPcl
E/KtTkF0x1OObygq6uQpgQUv8BXli79K9w3d5jz1oJCQb8q7wPcPXQcaJHoC
mRRmhTk70ALR086tdtNauAPdJ3oOXtAreidHiIkeLTlLjmHNDlRE9ETfeuB3
xKIDXSR6hox8Fm66EcaJHi7HYPWAIMI40XPPO7q4OqED3SZ6UkPfHyPP60DM
RM98Qp6/aHUH6iN6BHPKftrjO1A00eNU1qiVPt2BefQuXeH7S9eGVhr0Dh4y
UAYN6Yhz5znb0E6Zcw/vVGqDivKYmK1wG1K6zrzEeNoQ5G6ekUuVa0P0Hmxp
Hk7mIK7jem1Msw1ZdNFlO+jawDnDWnVWizakYk3/4j2FA/BZMNzVdGtDQtQH
TPvfO8MxB717oUFt6FAsg8z+ay7A/CjdpjGhDblGPqPlfOAGB7xX71PntSFr
4+euiODZ4y/jIVndhu4K+k+rs3jC9pfBfi74NqRK8kLzB8Gz/qYvOG+6DUnX
vaj2/+oDb0oyGkmm2zFP0if9ygpcO+ZJyzyInlW1Y568t3W5KrntmAfFuqUd
TGjHPFWvBZN6A9sxT1Pg+JsE13bM0/UkMvSeeTvmGXh89TmfZjvm+e68zWtJ
th3zLFjnPywUasc8qyZmdu4c7ZiH/C6LmSxdO+a5evPiV5J9TUhVrmg11kgZ
KuZ3qYhyNqHLQv4X8dXaIPq8u9dRuAnNDEV9WeA1hKzjH0yz5ZpQwQeH3l/3
zeF4+YPlGc0mVH7hmOufuzYQdVvBi9uiCXEz79v7mMoB6FcP0Rm6NaEA8r9R
5GnO8PTl96jYoCa0NPb9tPd1FyAR+HyyJ6EJnTY0ve350A0c6p/kH8xrQrq3
dJOlrrjDlLG6lGp1EwqRV5ulYPOEu9tOtLzAN6EqUSXhqh9e0Bm9rFM73YRW
T0u6Pqn0gYy3pRf+TjUjZaLnwp7F1DJcMxIjespt2Fl9q5rRNNGj/O16iGJu
M/pM9HSKeFLSJjSjMqLnbkzO/fbAZswztW1k+rVrM+ZxMNmvq2PevOVpkGpj
02zGPE8FHGTGZJsxD/2rpM/vhZoxT9RqxykrjmbMc/wOZawgXTPmMXfWoCOZ
bkU3lb9RyRkqwxkeA9WvuFZEY/lW9kmVNvzutXz5tKoV3Y+wpfhFyDcUeL/z
Sm4rMnMNMn7rbA5el70O7U9oRemZt0ZOETwKv15o4QJb0Vt24wM+lA5A+y48
Ktq1FcWFOsr0f3CGb1qJ/frmrSiC0sfpvIoLvNmbwcat2YpqRGtVqwn5plf6
WX9WthVV/lSZtld2hxP2X5NyhFpReRrOi43VE+aONY07c7SiEqO7RxsXvSAX
h+MGulZUxPI990GFDwR7J07/pWtBV4ie+tbw9xc4W9BuooeSNcDEXrgFORM9
4hZexzLkWjDPg4L7I981WzDPRyqreE6LFswze8NA945bC+Y5Ea/BHBHUgnn0
5q70tCe0YJ43F6UiaPJaMM+3pxfUFapbMA9NJy+DN74F88hzcrYXT7dgnr9J
tJ5rXY3oV03/PK+JMlC5n3ju8bERKVeFsenWagOttvjLHYGNKCCUhWs/vyEc
OqceHWjWiH5X/Pjs/NAc2PdZpxyUbUT8D4ePzerZAM+Mb2Y0eyMaDVG2iaV2
gLPVMZ+5/jag8NT8wuvpznApIb8sFdeAlCvYKCluuICMa1OdYG4DemxZcVzL
xQ2uaoy3FwQ2IPyDVaWzV91B/SxJn4R5Azrry2e7h53w/aE5OF4t24ACQgzC
xn56wb1J3vmrHA3oe0x4UXGVD5Cqzzwg56rDPMsuHTYP5eswz2zcF5OfFnWY
Z7Qy6Y5FcB3m6ZnyVxvLq8M8bbSOyne66zBPrdBtqS6SesxTqikjqsJZj3k+
ufGeqZWrxzwZiQzcly3qMU9yzV+WoqB6zBM1O8YglFePeV7SN+1Jx9djnoFm
nbLtXFUIuYhnkZkqw3lZmu9u8lWoKW3CjLteGwK/lNCsWVShO1fUi00EDGFC
0FbYMbgK3aq8KH/pEWH9ec+hs5BXhVKYae9RG9jAa9YOT/PuKpSgSZ6+uMMB
5l/5vB8nqUYxr34u9mQ4g9ye8y16nNUosnXsXNVNF4j1/L7cK1eN6iinL6k/
coPfKxHMGhbVqIbTUFf5mjtctVaSbg+qRlWSfR6SHJ6QMvbH7GpeNarQvZV0
7pcX/NXOCK7FV6Ny1+aqUzU+wHw6VnSlqxbzVObN7nf9WIt5LMUvzZIH1mIe
htpnVb5mtZgHqeJjd8vWYh6j3hMPgthrMc9eYydVhr81mCdvvuJUJK4G89x+
sJ+SNbcG81Bu1+9PDKzBPBn+WZ+4zWswz62DpMHpsjWYhzTuipkgRw3m4fFa
7d7JVYwyve+tDVkqg0PaSs8z+WLEUSmlv9CsDejb797dlsXIifOkwe+zhkBN
+rvPP7gY6RidCfvtbg4q3Mv9e/OLUVzbm5E1I0I9ev3XQEB3MQr9yfr34R4H
GHm4NEhLWoJ8DyQdXM9xhtNJP4eCOEvQA2FuARdNF3Bq/DG8T74ECfN8S/zj
5galy4sjIRYlKEP0ddYlVXfYybY4uj+4BHEraaJHxzzhhsLC2Mu8EhSvfaTu
y4oXRNnNjzN0l6AjFn2df+p9oO3+NvYfXWWY5+m17Wy2H8swj/hxCtaFgDLM
8+sPBYu1WRnmSWunZJ6TKcM8+h+ojlqyl2EeJg/qIzN/SjFPi/qOw+a4Uszj
y7eTaepjKea5RLnrkGlgKeb52bOL8btZKeZ5n7P7oIlsKea567eHYZy9FPNE
vnN//KurEpWM9qzzmyvDismOQuePlWiORPSucqM23DoRtLQaUInEfMoHSQUN
IXeckf+RWSWi3GMaouVmDvQpsaYkspXIqIYqXtPQBmyMTiS5s1ei2t3dq767
CPUOV2b/tr8ViPdaumpuFuH+jJ5j8sFVoOAQ99ShWy7wLLH4BnVuBVqd5Pbw
dSXUO/qyAX6BFSj1eQhvtIo7yHI01ew2r0C3ef/gP3J6QtKQ2vYA2QpE22zk
W7fsBdvj+4COowKV2jQLDtX6gP4Nl0dUXOWYh+LlT0lv+XLMk9JhTr3Nshzz
yDOMNjwOLsc8U2o6IX/zyjGPf1iHukt3Oebh61JmXiX5inlaGCuHnTi/Yh47
DXi3JPcV8xyIyLO0s/iKefLxvGcXgr5iHo3DKSuWeV8xz6oWS/E0/ivmCdgT
Ed6XjdDaiZUSTVtlSBg/LtN+D6GX77advNWuDfmluYs1rAgdYPG+8PicIdRF
Xo4p7vyCBr8/uX3T2xz6HVoUc198Qd9w0qX779nAj6t3fr+X/oIuFlepktA6
ANXJmcTY9SIUnyg/9j3PGQ5vf6jyKqcIUfvVObfruABfH/XfZ6ZFSPjcMYp+
dze4/OnVe3e2IvSd2/jKjZvucCuY85ZTVyF6c+RtWM0JTzAzz95mEVCIrtJ8
H4B1L3CTEc/UkylE5OQnT35s8gGreX0DFsV8zKPF8C2fOzQf88helNt1tjcf
8wjqFdyGY58wD8uTU9lyVp8wz670aArVT58wz3IbrYYO2WfMM7zikWqs8Bnz
NLEskdiGfMY8hdLGqi49nzFPihku2YerAPOEBCmuBloWbHnyvyhH5hdgntDl
pqs8ihnoW9TL6yxOylBzY2WfRGgGCljs+UOO04a/Wezf1HozEE5heWJVhOCh
UQo3P5aJpt48bvv51BxMzB20Pawy0V9Hr75fZoT6uiaa5fWnTETFWtNmR+8A
Lceqh9LIshBtzZ6a+QJnoPRaSCpXyEKHbK8jCz0XEBlkuocLIYzvUcTZe7mB
FUidmuvJQi/vzqgKqrtDQqTF3PZj2ehQXkDjAo8ndP1+lc1klY2idp6RzyTx
ht1qJQ78n7IR+532css2HzCl5E1vzc7DPELvtk8V3cvDPKRK3cdTWPMwT91s
lkFQZy7meRX0JO7hi1zMc/fsnT5D6VzMc7pT6PC19Y+Y5/f93eoiOR8xz9cj
w6Gcph8xT0Dx55a9bB8xj5Ze4N6VzhzMc4zCWHH4RQ7mWUy59KRBOgfzZFxI
brvLeB25/rD22faCML91xA9I595Fl9kpTKzntCGS5set7j5TxJF0QiZD0RCY
OdnG8z1tUNKFpq5jseYwJ14T78HoiEKMRHIb79tAqY7NbeW0B6g90991J6cD
1Fx8/vwZqxs6sDYgK9PoDGcUdSfnTd2RmvRZOg9nF4jUOCunluuJFGXLzfsC
3WCbCXVyIYkPMps2V1k3dAczx55tbApP0LMgBmEmUU9o98q86xPqh1KFS5jO
03rDpRCv4qm+56ih+x6J2qgPXH/2scWkIQqlEz3MXrXswm/fIA2iZ9JlwG6b
ZyQ6TfTkOfz62qwTgdKJHg/L3QzR58NRGNFzxZjd2Iz+Neokeph0z386P/sK
HSR6xtSv7KCseYnUiZ5sFQPNtoRQpET0uCo8+BDrGoLMiR75y4HrFhrB6DnR
c+BisrLo2SDMM3C2KJqaJhDzTHaAlrtROsoJt+DycVWG2b7h62fa09BE3WGT
ywPa8GP8icKwRBoS5XocfVfcEJbnT0uGZqQi8lHbHexB5rC20npB+mgqGlHJ
/mxtbQOk25wFfvl9QNVZPHPNjA5Asfsod8rv9+gDXRKnQKkz7DhQxqpu9B4F
2DBrBt1zgT3Mxow72t+hCyaKss5P3GDf8d20BRLvEA8jtd5bbXdg4M+iMst4
i45Wf3Xp4veEwxfUSI4cfYtonN3DqCm8gUVy7Ve9XwoiPQHZ53E+QH+iON4l
LAalET00LEwf6rhj0SjRs4vBIYepKBZdIHqo9jQX3rsSh0iJnm3beb5+GohD
w0TP31Xveiq7eFRF9KwsDLSrUSRgnqUJ0d6ksATMs9D/avQndyLmmf62MHO5
KBHzTDQo/Qq+koR5hr+m/B0cSMI8/YXkVAJ2yZiH8wBOchiugwVLgFlMnTKs
n0JuJN13QfZBbXg9jw6MXw3s23PQDHhsk10TnQn19Sf+6D3vbCDn+AT/cLc5
6NSGN+yXdYSjlvttaGNsoLWb/M/h0QcwKCdUm3HNATgccw/QSbhBEocax1Xy
+9D3917a/kB3uPfX0WU2xwXCfZmlGfs9gcFc0VQm2w1Uadt6DvP6gDOum4Hh
mTvsDfe1Z3n0BPAy5uWj+p5Qw3ZxN0e9H1z8uG6Ve9EbPN/PJxw77A/RbP5H
vA/4wsltDwKOUkfD5SDXYo4MWXikm6/Q2f0Glj11m0I1NaH5y0+KoIxIMIwR
j3nGawAch8+UKnhGwLVHKreUM83AydnKZfutcDiaxk/ly2gDtR2p59DJ17Dv
yIVhY5w9HBWcXHT6+wqonkkUy0U4g03g8XSB1pewviIfya1FmN9mDO5NJYXC
75wThV7+bsCoGM+ZdD8E7KZOcS9ZEOaDt/39t5WDYYZdIMzwiicUUxyNZGQL
AhNNIYpvvN6wT19TrfVnAAwHXbCTofEFP8dEndq4dOi8ryzFkiABfNtF+f5S
poP0vtfBA3W3oD2ohfSMeRoE2Em5d7Hrw30Wk1bDllR4dPvLBYkBU2BJ+5sQ
LpwKQokRzjcaraFC5KVDQ+QH+HCos506zR5Mq3lkycg+AHsAvcCXZ85Ao1bG
KGT0HsK3X/O3MXWB3GH1SZO6d+BbJG2/L9ENtGzmCt/wvwPJZf148xB3ICXx
8W9++Rb+CHg0V3p4QvLzI3e2r6XAZ/PYv6y23qDIlMN/XjcF7FPQqYd3feEt
Z5s3n0sM0Ki0BeRQy0HBz8JrdvtjwbBc4XfbF02o/5p4OD81FgbTD7QbPTSA
vlD/sVWpOAh08TllwGgO8waOWdAbBzGcaxL8mjZALnTHxcMhHhyfCc6mn3CA
/RRyspV7EuDKolnE6WVn4Org37czOQG4NBJlUitc4FzSoV5lSATX/pnfoTVu
IOdA/jboWyJM7tt5tT3THTSkp2w7LJNATeZYEv1rQr1zoP3SIapkKL0vuXbd
zRsejRZR68Qkw+m02yrBRr5g63ZQxCT0C9AZaZy0v5EMPZe1D9/kzQAJto9+
l2YlYFwn6OAPlwx4VdGX28ynDotOlfRBdRlA8WhhH5m9PvwJWqPhY8qEkoP2
ihHXzIA6lX93g0kmfOl6IHGIwQboKw2pzfIz4fCxol7D7Q7AMhCxfQdlFjjb
/bmfvegMJ1ebSFNuZEFHCTCQDLqA8H6KP1IJWRDuU6zONe4GErwiK0MLWSB3
eiBZsc8dlOWslh6LZxPqKNIlmw5PUNdLXGAOyIakB2yXX9d7g4ELbqaoNxtu
sEkGoXJfuOq+Z8BmIA/Osfx5PmJ1DmrrrpI+CMwDT8uLN18fugnSDMGsHuJ5
8PC9S8LDXXpQotsu7jefC0LR3W8+7DQF0Q8Md4Njc8G5+o2ChZc15C2pu0dc
y4WbkkqyFF72ICAeGRdPmgtnitYuR3k6wwe/3tL3mR+BRviDuJCnCxzrYBnK
1v0Ijp2nmMdK3CCWRY+8kPYjGIZ6HckucYfDpons5SU5cEOlh8m1xBNefhyT
rLPOAcm9Zw8plHgDLQm3fhtrDgjUPTvIUOIL3QXVamw7S+Few8TZ/TufgKvp
3ZqhvkqoqHEY5s6Qg9VCqlqj7EogDR+7KxuvA4570msnvSvh6lj124vCxvDj
9o06S41KaCKhCTT1twKrzNW6xVOVsLxbROkcqT1Mk8XVO5JWAguTIfV2O2cw
UZVtWG2rAJnjgV+bR11gNHGmwTWlAvIO9Yo18jyGu79CGskfVoBC7GLUWUMP
6JMVafK5UgF9x6jXI6K9QDN8oGknewXYph3VJOvyAcZgGpYjJ8ohjsqidZfN
VSDj/uJQploOdnUWz4YX78BksWm9iVs5hIVwpZemmUCbGiMHzYdy0Dk9a9wk
aA1FMxX3c7+VgyWdwn0IsockL7tmLfKv4LaU5Jc26wz+h9mOk/N+hSAc2Zsj
So/AMbvx0VuNr6Bym1dN+sZjuC3v0n7F+ytoXuvpW/fwANkBbp6lzK+gL+ln
/DHLC/icOt0je76C2dnz82YDPhCBeE7ns1TBFTrDonkeeXiwi098l3IVJPCe
23fo3G3QVD+jeudBFdC8743v+WAMIklCRjkpVXAsmob5W68VMC2ev0/VUQUV
4/xmYGQPq3DxuRZ5NRgIXP+UTHi/+GdiMRl81bDtoR0FjdMjKOiSzN6mUw0t
gzUnRN0eQySXTMUtv2qI89G8alXuAQ9t5bs+5FeDNc+UQwKVN2gXK02RjFaD
WPPDN52KvmC19rKjfrwWTuwQ8vHxlwDK5Wje4M+1oBVqwsXyVROiFlN8bz2r
hRa6yMuPWI3g7GzmwBGdWniQSioixGcFdd8/XxjirYXn1z5eT220A73RsuAU
0lqI/mlsxmHpDCsDdVPmrTWQ+fqwV8SeRxDU0y51JrEGBo+OK1OrPYYTXb1R
yw41MDknji9P8oCStrFfRbI18KM0wshtyQtuNc1d9ThUA+shPxdFpH0hyaix
Qm26D/TUnx/62aUEyfdvvnpa3QdJsw2SAVUGkPK8x6gosQ/uX3aJubbbBt7G
6J+be9wHLLT7mDWFnOFd9iQVu04fDFm0KWfxusG7CpuuGxf6QOgG8x5hwuf1
vuv3W98DffBU5F59wS0fYHDcofuFsRcSepZjlcJUAXFaTTiL9sLyI/uKMj5j
MGprtz57uxf8ZX5pZ+ywBRoPkZW5x70Q25jd+WfNGfL5Y90/JPQC7mT+2yRH
N9Dtp9hpXNkLQ5YHxurnPID6hVkw+/demMq24/hp4gNn7dezoqj7oSSR5mR8
lTI881YxpT/cD+yi2yUKHQxh6FUy+9PT/WB6LL47+Y0NXHi7hv8r1g/lGZJh
83+dIejztRD76/2w58jC2yOZbjBRm6Q4adAPBbqlPhJ3PUG8Z3WbrlM/GCcF
GRjt8wWHAbsUiZV+eJ6rOzesLQGBBibUAmP98NDvgdPyNz34MKF9j7WtH96p
WuJEnllDpblKLU1JP7wcinF6wOMMA/PSPCRp/SB+3sRP29AN1uxFns+G90Me
ibuGixChXl7hnen16YdTVeEn3lD4wqcb6pdb5oagUrTnIo/cLbBcKW1i6xqC
/l1R67cYLIEzmkfbtmQI0oq4ZnZvJ+S75MuJsrdD4HABpQnDYwgc/2tPHzQE
Bw6YGo6veoFt9KfuV/uGwOTI3+O66rchdvzbHg3uIeC0LStsmbOCBv4lscPi
QzBzVIZXaed9WLtPb9urNgRaBiNNQ0GE/CsXSIyxGAJxkWM7WFm84VkZi5iA
4SCofiw+b2vlCf0orD3PZBAenrcdsNjjD6QhvTaZqmPw8LXTHh/tRPic/ixI
YX4CNEWUvPb7qEH+ciUYlU3AuQmyDMaLTpArTj7tHjoBdeLqf4UUvaD0BY37
+oEJGK2H3cb6d2Ghiy7FW3wCBE+ptXWfvQ9sHPsb9ppNwOfzlM2lkV5gtv+i
4kPqGdTx9HLA8////5w70rle0s0gifl3mTuKtEGChJU0g2kGPWt4rWTEZQhp
P4Pw1RwzyJFDR0LOwhwYJ7flDp2aQfwXAyP2E+oHr36HgHWhGcRlvBhK8dce
5trH7zGIzaDDwTcCfsU5g1athhS/3Ayi+5L3dFzaBSqL65gVVGaQ549npWb2
bnAm99KKvuYM+kv9BQxk3CHqfUbbI/0ZdJ95tlCb0RN2xLKlh5nPoJ+CLBdu
TnqB/cuQJ1kOM8hS/lqecpEPhHwvzfZYmkEDRM+Jn1e7TKZnkDTR8+Vv758r
wzPIn+hR2WHOIYSfQc5Ezxj9qtzhlhkkQPQ8YH5iSVY9g44TPTTcDKHjaAYd
JXoSBBM/N+TOoH1Ez3k405+TuuWplyvZHpGw5bmreuXk44gtz5JOz1WjoC2P
n4mpg9KTLQ/FJw73Vb8xuKgeMBxtmA4eMgc7L74cA9yaYPgt/wy4SzzfYrOf
OPkLBZOzVGNg8r/wCVi2tsc27CgFM8orHy/vfUI8z6QUkhLkmGlOr13ajNXT
+dGhDnHYjCt8cx7dNbiGxf3nm97k9aljsctPex2KU7pYvKOBfIbvngEWv0oK
dNFMNsFiDlfm3d7D5ljc+f6mcieNNRYzhx15ac5ri8WGnsM9ZMr2WPzB6j1n
mJkjFi9q2Zif8nMGVtG5vG0UpZhXHUWe6bregsUb13ej2lnnw4EJC2hbsKfS
lYolrB/qxvgvOHC5ZL1kdB79c14Aag3/uls3sgXRlrqsJP7Y7M++BPs2+rsg
jjsWBTHHljb7VxPHf4L88ZIf9pEz6LIc15KKZgvW/3StRiz8Sc0YbPZ7Zwo9
uLM4qvWf/qdtwN6wkzYzeRwJeI63jH9c3Ox/ThxfBN4LSmcNb06g46TUZbfp
P6H/9jvNQJdxJMrTDVv9lrmrHueT0W2OL0DURj8c9M/5ECif9/kd2XsjWP9T
s2o7lZGSuX/6oc7DNHOxf+vEZoz150GvgOegtt+fzf7u6DBjfXb79No/r18H
1QeMFprSQ+jEp6WEHQubfUw2+59mwBG1BHfrqh5ijPUTQq+r49bseOf+6a/+
G6Y0qvp9XXuRjubwueKbq9j45nkatBvnaaCGA8bV5pzT//RLnYHCpCPW35S6
ifEyZG/0R0KWJvyJ9/mWif3au9EztpX1ueSlf17/C3AmpO0d2jg0FCu0M1qp
HNks5J5+fr8ZZam7yJ/LKP+nf2oVzGTV/ikVakJroWs6r0aH/unfPgUieTN3
SKg7UOE9+dCio1P/jC/CsIjYS9qsZnSS8pDIT46f2Hgd8XwSl43zSdD236sf
Ay7isP6qrdsqbx20mvin3+oCdG70q0LS+ql9e68uEPah//+vCbvfye9jWXYF
VKM7xH4vGRv9XrD7VTx0UcrocyF660hx4dQ2PNZP2jmA4ZBz1SzWz7tYhMyi
4Pv0P/2mZyBR6stao/JnLH9vTbz6sY+lFsiI73+z/1Yz8f3/t59XMVrb6D8L
w0dm887zILT2v35NjTBHJ0C9l60G/bc/VxWq+fuSbCK6BT4S788/47C8cX9g
87yY7I3zYrD55BiOa1zmIx7m08rm7gS3oI3+zx3Y+UP8G/12ofdU2IEv9CWw
OV5PzL//9veaAbSRfzBKeXym3uEXqLgxno+5ise+F0kb/X8hrvfTAiVjL6rZ
yEfi+CD6TO+Rm0LaD5vfp//2+8oAzo3vE3QO3eJDld9g8/WK/z1vh3j9d3Tl
7nTcqugwBHNUv4rZNkLsr7XZz4uEZGWjvzEss1axnlR0hs3xT8T5Y7Mf2Ob8
8d/+YPMwtTF/bJ7HhBg3+jMDXWfHZV+mSUScP0Ce3WI9mXvmn37oM+iNW2YL
ujoKm/Pff8ex+Y8Yj8KJjfkP/pmfNj2b/Z2J8RwqcMoIYNUYA49L50yqd5YQ
+61v9V+3rpzptrb+DqzE+fu//cba4NjG/A1h/z1vCV78Kme4abUIo7Y6Yg6u
30GOuD5s9h+j0rmel9nSjDb7if/isBVhOd72T7/3NtRenUUmP7oZY/25wUOK
/UiHdwfx89vsN47164ZUI2fXEtOmzfUKah9p2OVc/AXaRp6i1GcX4J9+4v+u
n+BSENCouNU/HLaRxTBlKNfDqVrFq0mEfe2osO+vn9uG4LCS62PvmxWQNeem
0GeXD7rP6+OyArLRdmL/gMiN/gFIhdgv4NJGvwC0eZ7OZj1zkDvw3Wt7PGFf
FDTrLxEDdeodWRJ8eOz8nQTJI6cZkgaB+UpvZ5TNR0RVY/Yp6OEgcChdpLtJ
GQ++KhnXD5vgIO7IoU/BeyrBwuBEZGYN4foncYe+vfsC+75QzdWYjmHn8/yP
6zYGJ2sFC2MlUqH1e5nCSto4dt7XfPUnpQv0ExA+lb/MklyB5TPJs13er7+e
xPodHqw2aJpz0iskJY47df5pFne0II7PgZ7gKc3UX2XAIFttNF8+i/1+/9Kf
x7VetPxzfl4rXL5kKrCSLQmb8buaRzsUBFWx+MxS5WqYvw4W9wb4Otja6WNx
HNcpy/BYEyw2LGw2KmmwwOL9J0T4fqxbY/Gy7fMPPiF2WIz70nec6eRm/YUH
Ee4jJ9I/7sLipQ8lNe9/ymNxQpnTtT97NbDY8FaVTrCKHhavKreFHMPOl8ND
gFR/zVb/UDzQHXUgOc5oi8VajA+ZbC45YHEyvbtQoZ4zfOY6Zyc3iYNdnq4L
9HuLoG9UErY/JnzvPBYjbrwsgRt7bO4qhhWAnEtu0z6NdDAfpKWUyh3E+l98
9hVPSY5qAlv5065jb/Kg72SSXkFdE5QuSCeqlX4EvZybWfkmzXBZnmpeVzMV
Rp0fD/ccxEOXSpT474+5cGqUdzb8MB6Epl6T9ezIhQvE8542z/PdPA9qM5/9
iPu/txv7P9RH3P8Nb+z/UOD0QzirM/jf++82CGG3691t8onnJxJi8qmV/PvL
d7D4wvEXfIUhJlh8Pou27dFBayxWbcK/iDa0hxOi/lmNftj8gDbnl78b88tm
/Yk6NuYXlFW9w0Fua75APf7pp+nqmtFm/bpxfRWy3JgP0V4zkTXzkzP/1KMz
KHJjvibGxWh5o38jiibOrxs/H0f/zP/En39H1zbWI2I8jOg2+lGiMOL69N96
cBAVbKyHyONG49XARiesHnxLXD8367mN67vR8Y31nBh3wsPGIUGcDx5ZiVak
Hq36Bv+9vgn5b/TrRJvr/2b9tVlv/FPvobqNegPl37duM99bgtVrI8R65b/1
VgbS2qh/kC3x/MHN+imbeP7gZrxx/TykbNRnIEWs3zbrn41xrF4kxsMwas4+
dsL/GzjyeD92IdSfm/VJAbHe/G89swTsjyi9eaU6gZtYb26OPyXWw/+tZ35B
90Y9DObE+vm/9QpWr0PRHB3Tfb45+G99sQYdjLiYIZlB0CbW85vjB4n7if/W
E+ugtrGfALeHtayCHgvw3/V6EXg29kvE+AeIPNL2M1qdhDj6JcH8HGx/Bfso
PfdJEfZnm+vvxvU/QWFjPwf/7Pfgn/0k5ifuJ+EH8XzJf9ezzo31DBUePfVy
TAwPKncD+jMbM2Dt9UCObSIOWw++Vx634cuaAksjreFgl1RINfdYv+4+9c96
NQUZz48aiJgQY5IJoG4ebYkjqYDzZiuJu7+kwMr/Pg8EikVa704HDsJv3yV6
Wo50yF/brRORNggr3xFXekAe8fXTUPn5q8+imRNINWa9aK6ZhrcZ0l4VUvZA
drBhbZfENPiRWKmEmd4nXj8LvnEX12mp7Yl5NwvOOHH/wFe5oj6ajJcewSyI
kPlZZRk4A0R5lc2Lz4D/SGdEzp37IJ8aqf6ydQbS0z4f6y6zBqWWT+dwpOMQ
r/Yj5MvzTzDF+2Z79etpkK77PGe2Mw9aQj2cr3tMg6rdlaXD13LBwBIvlNQy
BT/+93fTsf6Y1Nf/1x8TqwdEN+qBje+L2yhQSp7HG+cSz0clxFT0X32Y4zSw
uNOVdiXf2wSLOy6wHOTvtcHi4o/C2dt/O8Hm85bqjecNqIt4np3Bxnl2aPN5
xOb4mEOCR5FkC0T7RZ/u5w1Ae9NjDuHam2GjoIpHP+ksnr60a4G0V02Dytcc
0cX+DEof8xYQ+hHdra/gQnz/c+B26exXV28u0W3/+8EciFzKvsQzWyW6OU6S
oH5zJNy00Jx3/a/CLA4KXgW8Vn9VSLxfzTCs3SeudYEJi+MeXs/gFJHB4nLv
s1OuPjewWPiKOIm59W0sbvfj2pGaqo/Fe6x4QuKx82ybQVZV4Gj4KwssFj98
WFW2whqLdfYwvy4Ts8Pi+yRsvZcKHLD41QIH+2es32gX6J6/2FjQdhKL2bKE
HaFDGYuXdlxdT9bUxGKLo+7MB5P0sPiOsJaYVpQJFqtcFdKNeWmJxXGCVOI2
lTZYzCRufy2kzB6LQ5UGdXORE7QR68HN+befWG+82ag30Ob5kOc36lPU/fvR
vUK7yX/W80no5/c5KRqghzZjHdVnRvRLxP7ZhJi94NP8tf1PYPFjILVh5iT2
POufeh2ZjomyjaMxuDGlx8RyZgZt7i/GNvYXqEXmyGdFiUXIhdClVPLvyFyl
oWiEeR325w3dFTUcQpv7TdWN/SZyemG5kPLhFyj/0Y89XIpHaZbJuNtDQ5CZ
1an9+lwHMj9Wsqg4OAN2TNXF1LeL0SDxeYf1xvMO4NgjkB+VUYLEolPldqU2
wS4/pq9SRd+gOMAj9DRZD8imGk3nZE8jM7VPT7VPjIL9056HcialKJ3p90H7
B+Ow+byNZuN5G+w9fJXRtxwH++9ka9peKIHrNo9NaXNxIPWwhmV4byncl04Q
Lw7GY5/P4OKq+QHjwX+e/+GBav++n3+FN/ubFcIpgYs8FDJpcP4F99xN1TGo
6HB6tftuIvb8pXTjfGD04QRr1pDwFPb9/NIh1HTy+Bh2vucrAYtEPrJl+LwD
n2bDv4iUDNRyC5yX4Efa9Vihc/PIfvgHzU7DJViaKBOzPDqP1ovaDPSOjoNx
w40HGWOziKerONKU/wdM7TA499llCj0XuGUic38O8mS7ip5bDqIzisYmWmZz
8N1ZTGJFehDFCi5QOWiuwSunh1+6YwYRm8DdhJVjw1CTzssTp96FnM7Nzo4X
TQGu85DbF5NOVDXocXXy0hTsiTI7RFfbgaSiCo98EF8CtZYdbTaVneidYOkz
uv0T4JsS87PtbQWCGi5WN4c5MP9CeVsgvgJliUUz8i/NQc+9099aNapRuWzU
dkmneiQnp/FKQagNLh+tvZCtUAauNp4MaZ5tYFJu+eXMegmYZ54/lB/eDpfx
3PLe8p0g+PCGyLn2Lsh3TvfWFRhCUVG2lY8S+iGGiqniCsUgGqF4GLVrez98
qbqkOOMbDkLLYyUtb/shrKrM5PrYUzhwTOa4u+QAzB1s8fkkM4tsVL78bv49
Cr4C0d8/PihAHGlPjClxo6BqpaYZotSIpq8UTD+YmwK7rDLLVd025DK3mB62
bw72enKVHJ9pgrNjF4pjeecgccGo8NOjJug9O2lrlz0HcRPCZhQHWrD7eyUf
rbWo4/9Zbwl1SRQtg87hRLhrbhVmMjAJjAwSXSpOxpBCE/HaKWYSrqjsfki/
YgJ5+2xjfpJNAt+ifGrjnADgnMOu2LJMAu1ApXTXTxP401f0Z+7iJPTyLyrU
VTjB2w9f6UV1J8GO9u3pwkhj4H526dhJCcLr2VZHnxQ4wa6l9l165MXw5N7i
wtHU/cBzyYJD8WAxkPj84ThiJQmK3hSiZ08Vw62rjWcV9quAWUPU9aMSxcCn
zdzlY6MBzw4Im1KqFcO2j8KXBo104YNOo/ucaTFYFfS9cHtpAHVJRuFdbsXQ
XeIzcPSrCUzN/M0sDS0G2SreM0WL5rBLOKz6/bti0BgzLT1Faw08rnwDIagY
Ar7D5MPjtqBUWbXs0lYMFdP76OvAHsz36tIYTRTD6tyYKJOaI/ir/T529Q/h
/fwoMLhn4Qw0Kn/sxv6UQJ2FfuDo7rVLLTF1TfUTJUDOpqiFSsQhZDacJ6et
BGQDKEp9rl2Dm5dMfMNRCfjvHWvnLVWHg8+Fh93elcC9yqEfpoy6gMNvFzMK
LYHdlG1/0DUDiORui1ByK4EM6XIq+qcmoOMc9+uMKWEe8c6hMy41B5Yqq+uH
bpbA6l2VPTrrVjBwANJJxEug3l1+ny9h/xlvsHvn2MkSiI6TOJh11h70c/CG
9QdKwLr0wpHua47ARf6uNJu0BCQGBdgoCJ5xg7jV0skyMHs7IN/EJwbnPkXp
SXeUwZ+xj4sJGVfBd1dEbXVxGUivrpQMyKpD5+1XZ5Tel4EMJYvsi7k7cDw7
OKIptAwmAyTSU0UNwIkigFzVrQxeHDI4UOdrAlXqz0y/3SuDMwk+Lt/bzOFg
qm+rxg3C9Qdu2HjSWIMxiZdIL5RBkeZoYrCQLeRdfxyvy014fYxjV5yWPVAm
u+wcoS8D3RGq3VnujhB3z5CWibQUOkK6Lz5VXrtk4mR83eBAKfRevcPy9rs4
8HnfC00/WQpPe41/WThfg1/BZt9+i5VCUkiszpN1dfgSa8F4+WYp6HJODzaL
6YJXupWmv2kpWLg9OdDnbkCoN23edLqVwkM8p/xUuQnsq7XrY3tJ+H1CpS6/
KSwA3+nAav6+FOpk0295cFhD3KiTXl5xKVx3cFl4ImUL937cTyTtKAVcvMKz
AML+lZ/MZUxhshTuNDNyvfJxhN6+5hhL8iZgGa/tjFrnhL8sM8pxe5tgNpTU
JZNcDljv7lhvY2qCqYPjseuSN0Einus95fEmsMpOlqAlvwP6wxLqFwSb4J6G
Zg77sD54cd6mNBdrAn3SvVxChPxPNnzwMVqxCXRSSl/JJllAVfIrvZZbTfDh
WIFnKqFe/T6eTbvdoAlOrIYUCQnawU7uJiRs3QRJDRZLKMEBTplOmd9zaQL2
OFleuf3OoMneqTC43gTZXFS1DfuZ4FKsmrf/fBM4CPDjfjLJABvzt+ILI4R1
M8NtOuzeDdj+5ubqaGcTdFeMTwvJ3IaJQx1ng+uaoMPiuOP3QH2oC7thBcVN
cOm+luN5NxPIOND+bjKb8Pe9Ahx9LSwgOER15FVyE/jIDhaYplmDI10b8+WI
JhBRXhMJ5rQDjYDrGnP+hM/r+oHCz5EOcGlPa0ikexMkqPOJDu5zhqeXPTTO
/22Gnl0LUuo95NBBSxMRu9AM/ckHstv0pIC1LxJPPdoMzgxj4xPlqmD+4cRh
m65mIMNLOE180yHM97lauLpmkLrS6GFnrw/bZCTfSBQ3ww5+F4Ufj0zgCn1T
z7vsZmigO7nPztcCIga0ju5Lbga5+2bTv6usYTRtQudBeDOcNJVs/qRkBwIP
HaKHnhPqTa1DH51bHMBFjrxf4XEzzCnOh51Xc4bVdi4bdrJmiN4xWqR8mwlW
wlTeh+1phhQuzbz3GjLwW/PR8G6mZvjz9WFtbsINWD767ojHMcL4skIBQ+Rt
+DXQfnP5DCEOfX6jsFoflhJIA8zFmmH9tH29GSEfloxOVQ8qNoNKlZbUkRIL
+MmtTnZLvRla+eJ6DXut4ee0p0i9QTO4rIceAi07+JGRYSdh0wzHa57cZMA5
wA/b7tS8R83Q8tIlaJbwfjOy2YemfrSCxpD0qdP54hCx7PSrbKwVQtyjt1/Q
uw7eFxt2RuBa4X5wxA+t79pg7c7OYlPfCnwttfPonD5oVToJyhW3QgJOzuTa
QxOQ2dUgy5LdCgeHKvsHkQUIXGPX/pXYChU/2Xg+k9rAkZdO1g1hrUCyMjDm
Hm8HlPh6r0S/Vjj/JyZeQcoR1jv9F3V3tcK5wBKGqUZJGOhN4TVgbIXeg67T
C1dV4etwqakxVyuwmBxpnP2kA2+/dyebnmmFzG7tWw6EfHw+tzRkIdYKLdtO
Vf8gfF42SzQsNkqtsMCzdt52xQJurnFr2Wu0wpN3Bu1ODDaEfaFUmJNRK4Rf
qt+dn2kHzFS32x7YtcK7FkHpXwqOwJDwoK6KpAUuf44os6HfBvvcGpw9drfA
+ceG8ieipQh5wHrs0qEWMIqXy/H6owo7ztm1LXO1gADFnTKHU7dhO33l4+wz
LfBIUGTiTJo+kM4x8lqItcCQQYXVB8L7W6s16z6u1AKyr64tc/RYwHIyejKk
3gJq+jNIecUaFj3ohKMMW2DHejb5Tm87mL1tMHzLtgUKQ51lKmkdIZD5ZHXx
YgvEpJEUqIVJQlLMpT2Doy0Q9iEr/AeDKhSwqlwnx7XAAOmIWpGeDjTHGYRx
1LdA6Hu/HWVq+jDK7twjVdwCFzo4TJJdCO8n4RmbUXYL9JJ8qfSLtwBarhgj
36QWKAk9uS9/2BqOJWd/ePu6BfaddJJ452gHoscr52uetYBhcbl15A5HuEOq
edaYsF9k8q6b9r/mAoGly+X92ztBkZ/qsGSjABzYIfXIi74T4h0s7tXeuwoR
1wKFudk7YaEft7uCVQtYXvfMNfB3Qmxi/K07U3qQ2H/ina1YJ1Tj1N/1VJoA
93EHvYNXOmF+L82aRrwlpFuWHi7S7oTnbENcZsgGBPP2dOiadQLrxS+N3Kn2
8OmPxguKB53wUe2100S4EzD+2VGXutoJrZOdU1nJJ0FajFW2a6oTyBcN9NUT
lMHWXbh8W28n+LbXUOjxakJMuZIYX2MnVN3KTFX01oN6Cv1CzeJOqJNe/zZm
ZwIrsvfP+WR2QqOgHJmHniUc8wvIyYrrBJNA+QN739qAan0SX29wJ5j6Wr3X
irGHx3uLPlB7dYK560uxd6+cIE37ReTF9S64/QbfNJjCCjaXj/BRzHTBlHOQ
8GipIgidfF/W0NsFYhSOPdXPNWCF9rzaq8Yu+GoUdIVGWw/QcsX328VdcE/R
JEPE1gQ8+lQfHc/sgr38YrRGTyxBtmKQdj62C3BPd56vbrSBXalWiZ+CuoBn
3WGiF2cPzcF/zrl7dIGr5eDrHyNO4PV+XPvOti5wfie3TZ2MB1IOnnpksI/w
96hKBqqprkCtl3XUPbYucGgVLnnkqwkzCx+/WPJ3gVRgpaxihR7Q3lnptRPr
gsZzi3zqVSYgWH/pr/OVLlDvP8poWGMJNy94MLvqdMG8ZKLb+VEbcE6uBE/z
LshlKPgBY/YQQb/rzpOHXfBgsslYetwJFp91hgfO4UGANr8zNkMWPvIoUHv2
4+E7xduAZV51cKwrcnRowkMU2YNDTeR6cN6Mb9S4GA8Vj2On4i1NYHVnvKpm
Bh5mSaq+OOItoej9/jKlGDzsEnOsSmS3BTcFX36xADzEuD7NfKHuABKTK9EC
bngQRG/CnV84w/rO0XyOCDwYGJCdvkXpCtlzf6hYSfHwTP59puLYLsj8PvV2
iQYPsZnv6CU5FCB9GCdfx4IHRbUj6VQXNSC1t2oylg8Pw+8WJn7Z6sH7ztxn
jmJ4YHt386FptQm8bUk4pXQVDxmq2j6We60guS6oge0OHq6ovrh64oItJFa4
WS5b4mHmegnjkL4DJBRb0DS4Ev7+9cXBSML7+8sTm9v6BA/WXXjVtYxgSA1Z
viHohwfRXzpuVfFBcOEufKglw0HuK7u7ysOsEJyLFITocHB3bM7gA7sSYd8n
NhnDigNT1bjXbGMaIHW7+OlOfhwEHeJyE47Sg6hsMW4HMRzM7niQ3UO4n78o
S6r7r+BAcbVx1GPREq5qiZso3MZBmz3d7v17bSElo4Qq1wIH5DOFXMYcDkC6
XSKF5REO+A2NxArOO0NLYFrT4B8cNPI0qxV47oLDfCG8krM4UCPL7suulAfD
Bmf/+D4czDn92OY/pA4ZZrenyZtwoJFPuTuWXw9Wd0gp6hfjQFQwlOGBtQlI
veV+X56BA+YMNrYbmZYQIEOzgzMWB6QHbr18tWwDuJGfxl6BOFhojYrwPOgA
nJ74ypHHOBgOHImxPucMgzEprF0kg4BuDnUH5O6H3y9yRFR2DkLxc0ZVniMq
QONafKN2/yDssBWod7TUhf/j6rzjufzeP25FA6WhiEhKaaBEGV1CskO2yMrM
3tkrSoM0VZKkNKwKZRzRkE32KN723lv8/L7v9316fPrz6dz32/26rzOuc53r
3IfPvtRBlpsEwly6jnqV1gCGDVdy+UmQ0pV35fE2J9BW7nwmcpQEX7LVDZgV
3MBefDQ3BUjQENV5wN/LC0JsrRmfu5HgrdGn3NAoNbg40FE0b0OCqpcyHbOV
xqBpaxSqZkwCZZ5XPYf0rEFioOFkohYJOlVWu6ePOgKv7dmlBUUSPL0q7lkm
7AZMA2Wf1KVIYPTd/lKvpxfQ5mUZeU6TgFWPvrPSTxXOMl55aDFAgrCcm18z
LhnDM329Bs02EvwgcWp81LKGiZf7tsjUkiCUTnvEcOX5ZWfm1IWKSZC49fHR
36FuwPztBfO1qyS4danOeWubGki/0euyukOCjcKjlzK5TcD91rqcU3EkaOF3
kzCrsobXnrm3eF6ToHrDHz+Gk07w28jBevkDCUSuofOGD9zgyE/3xENOJDiU
ldwgs+M+XM+MvjlO0w0DZwT9BFsPg/XLG9zxLN2wXarJN0bkHJx6cCVNnbsb
OEU22082WQP31RBpaoFuYE18XK5S5gyLl/x/pkp2ww4hIYudbz1BmE1GKd6v
G+5/2/VcwmcnMA7G6UQ5dMPqs5eXEhX1oSNvySzAuBtiWYWFOw5bQ3bUOUcH
9W5oCbzlejzOGW6Zf/Ixku4Gm9m1Z6hfeMJuduqdF0e74U6kxNK9QV3YT/sh
4EpbN2TJFK9+sN8aBAet2hIruyGaS3yNy6gziNRySH3J7wZFTpYbbEpesM5n
e79FYDccG7KYYx/fCYb6Hnatkd0QFx2pS1rQh5RjP0fPxnVDGe/8y/wWa6De
KuBanNINXcu1G4s4XUBj6uqMFOoG86FDVQIqXuAiXv3lvW8BHP9fPMALmC8e
vysT2YDM+vXlkc8YUvxzBI09rABui6wD92+OoX/iD4iIP4iQ4w/on/gDIuIX
w+T4BTrAOT3/5EYhKox+GOPV0YtuUeJlGeR4GRqixEPcyPEQFHyg+5hN0yAq
lk5e9TqzCxHxNjtyvA0R8ZLd5HgJCpHms1xP24sW/8hdyhfsQGn02UJFSreQ
/cfPS+x5JPSOEq9JIMdr0FNKvKaHHK9BGc+HHg+UtqEN/P2PuLN+ofP/jd8g
22/k+A0XOX6DnNPisxW06uGz/gcBe4/faMN/44PItbNB/umWBlRPyvqqFN6I
Hm5QDmXRyIGP1tLvRVob0GlKPEmMHE9C5ZR4lDw5HoWI+KM0Of6IxP4bn0Jn
KfEpL3J8Cu2lY6xtNLmL7i+n2Tw+9x3ZZ2jOuSeVgv6s+/fZoW9omZqc3yBK
zm9AJYf/BJuvzP8i4yMbXd9/Qn8ihTnLHSpxPpMqZf3iDXn9AsfTrMnxNPhI
iaetsv1fPA305aVTxW+Owdl7vCWqtqWAlGloo191QOmOH7fbs38CEX9NJsdf
oTIFDT25OAB+745wUnHVgM+xpzJRvybgWdodPoHYn0DEA7+T44Hg8d94IHyj
xAMZyPFAkP1vPBAuUeK/GuT4LxSyOAuHh4yAaYSTxDfTNihU37dqoXsOojty
Zxj2tgERr8wgxyvhMCVe2UeOVwIRr7xLjldC7cP1e7Lu/gGZuIB9ic0d4EyJ
R3OQ49ErfniDTeHYGPCedM4+6NsNpVKFSnwZI4i8PtcDPynx7ExyPBvum7nQ
ZYX0wICC5I+LAkNAxMPPkuPhQMRXB8jxVSDir5bk+Cs4O79NlM6YgoR65vY/
YqPwT/wW/onf/hvvBV0PI/8HtLM4HwZR4sPEel/rj9YDfVw9EEX7JCPZ6i4y
vpHOsby2B97I9Wk/KY1ASjHn2oNzs3A+S3NMvw5NPwmv/5Hj0R1Q8HhD+8nC
r5BJyVcIJ+crICJfoZycr4CIfAVjcr4CXh8OYGiv5RUxJuejBIzC/TOCbVZj
VhLKrvmNaSv2MvvfeieRzzICDgfE204U5VPinSvzFxnlkaYKG0Rw5HJfnLZq
1QmCDb89ajr4WBZfb9iyEPw7QhPzNa0k9r0/jDAbWb7WPRZsjnl6F01CC5Gf
ssI32nRHAvLtMUeX6zy+NeKE+dcB2sJOP1fM/BHJvSLMHqCxuPrxmVgSfp9v
+r19S+mrMBPrS/qXxN6IB7yDa4fEb243I+H+fLWFVyBdZho8UL9tWTn7CUrO
zww77xv67/0BQ3BolXu3yNEMaEobDPmzdQi0+bZcSnqQDmoM/pvKWarBl//6
XIBvDHLMJXHxXR6E2dc5Qga9STBbZzd3e2IIouhPGm7q+wjZdxxp3gx1A69E
q5LAWUN0IbZQbDVrNa4/50vHWEgRPcBa9CGgyzIFhVDyvwfI+d9oF/fcI6a6
Jry++4qil1h/IvSqkvUiZ9t9zo3TA9D9onmhKTiGUo7Xi1Fkvo7e5fYREFh6
5511m6h/o6iAnA+HPlPaI7m+96AzQtKBJvnlIDEpLBx4pAA6GsvR79gK/Dxu
0Qe086pJYGDgxFUdmY9qfu2dKSwhQc7+J/PpaXkov/otX8OpfpyvsI9yXl8i
+bw+EBop6ukv7geFwLw94cNVMC+p/iuANAye9IuxZkb5kP2JgbbtVRXUeQi1
lXq+AJreqp07X5HA+H/1OQOe6rvvW5CpBt3h/lNThyOBaH9y5PYHWt057MLR
Vf89X37Ff3Cn2Tay6iElv2mFqdUDLxbP6GGeGf2iaRRqjTnzUcJHWwYXzOP5
3fJ7lb3gu9xjlk6naly/3p+9VZG38v7b7L7Uh1yOgdeU9SU+8voSnD6MvG90
r/jf+x8xHduRiwSvBq55Q98BFWdtd+R7f0NEfh0POb8OffHeG0qtUwnVuqqt
dKmfQLhPk2mvYyXObyLbrxro0vIS1Jljoey4BWtNbRX+f3S173x+dgxAXveh
U3pFd+Fu2U3+syUDkCl5LEKe5RbOhwtb/3Vud6IlZueDFTWbrLwwR/YZal/1
C4btc5wZNBt6IGV6UmD0sR4Ez/YYanP3QLal8eW0X9bQP5O26pVgD2zgN7t0
1MMFjt3KYDd26oHYhJSX4hY6MHOeMyfauAfE49iCrHdZw4eDIYbfz/TApsyh
e00KLrBdRmPrNfoe0JjXzTFRDAeOsyep6m2aYDHEikqX5Ts6Rcn3Ok3O90Ia
lPZoSm6PkOeymVEjg4THa02K/f+zvkOVA0zk9UEkT6nfkuT6TeT/QLPSS5Z0
3jw0QKnvtGTDI50fprXGnYNQYbjT43tqKs6n8Fq+4pOdkQa7NzfunV0chNv/
+39plPoyCL16w9x0UcmwffzkswMr9TmUKbFdzfUFmjc3v9zxrQrKvXwlM+IS
EKE/nawfpVmlLcX+qgLZqlr+aMN4RNjbe/Sh83qmWHRiQ+riiekqYPzf3+MQ
UR/nyPUR2f56GK1/rhpOOP0oYF5zGZ1XLNvT+WQG3s3cuKTwbhwty5i94Hs/
Q8l3GUfCL7Z9WspJgx+hbbfv//mE3n6Yvk8aSf8nnzsL4r1EdPQ531N4AI5c
b9m2/0U1JB40kB7ZNAgGnPYCgXJEftEgjByP3ihIXwf87MzZQs8GKPlCNaCf
kL+PXaWZkk82jv45vxH9iUpVrWJuQpY+Sdpr/oyhTYdfOxSIlANx/bOk8eRH
vuVg6KH0qwDG0akvSc/XsfXg/RTE+ZLJ5PMl0bS5hUVzTzdEcYYObDnbjx59
bRksO1FDec5edJNyvuVt8vmWaFut4YBoQQ04hu82Yn3fjdzu7edxsO7G1xPn
XwqTz79EJTm0rrGs3RAvKFLo59mNxgV7U5n7U3C+G9N/z+dEpWijc1pAAnqU
tC3M8FsH4mSjej/gVov3VxDndwqSz+9EPUb0tjNltRCJjkYtDnSgnZuN59q0
xvD1xPmd38jnd6IEyasH4hPqcH5cqaRxdW9tHQwqTXAb8hD5dAtQPDqRt1uu
HXlz6rJ81kvH+XCPKeeTJpPPJ0Ucp+/eV6TOhDjXkB3SVM2oa4qlfcOhYXz9
EOX80izy+aWU358Cw5Lo9ymcdaiZJ8jbQbgI7z9QpJyvOk8+XxW1Cnx3vOdV
DD1oFZ/X7RIUH2Bq98y4A1//lF5QskqpA+K8LQxtUBkeX4hy4jzW1eTzWNEk
Jb+CyM8xpJx/KRv2v/MvUQ/leYjxyYxy3qwe+bxZeEr5/0S59HNx0wSHDpg3
tFuc7Sb2C9QgxvWekatP1UMs5f0R+XBplPNww8jn4UIH5X0R5XSU83Eryefj
wlNnqXv+FcMgSLr/gur9L8rvd6N3J71znHaRgLAnkS9nzyhw4oZ1HdAvfAw4
KEGCf+ob/HP+L/xTv+Cf84Bhs86GbQ+oamE66UBhwXIX/FO/oLDMwp41egxu
BxucWe/dRXm+ZUjxbyrfKNsFRHsi8vOI84rpyecVA9F+iHKl/55fDG/vNtx1
je+GpqHQT/Fb++Cf9gwXNFq8dhzvgbc6fXSq1QP/9h//ntcM//QXMKmeEVLZ
VAbU+zWvqT8eB1uWgn18gs3YnyLy9YzI+XqIyA8g/Id5qcab51b8T+J6Tsm5
rvnbJND/VINOXU5F8V0NveuiSDCWyTdVtjsFgeJCnw7/EL5e8lSZQ86KP6nw
WpZa8WE68mxit/J1bsL++eAvTTYxvyboF/asd3j0Fdba/dIqudcEUt5/eDbu
+QJvAxlQRAUJ+1+DATzGXj9JcEJw/wPvG/mw/Vv8x289JNj5tnZHSlcBvEBz
HbqMfX9/X/6rzKfVfeDYkZ3vqfgNvr1KozGUeAM+Hm9jIvzyoOAQ12jchxT4
9j9758CW5R8W362Ggen4hJiNSS5oCCzp5WgN43KmzLSxylsknJ/HcI0l7NvK
++APeBY9HZIKHFaDJYkPSbC5tKNbLCgdcnPZpy4kkoDp5c43iY4fgKpVzjjg
KQlML3+lfrTmPURNlWsYKvRg/yXyUosPi1gP7Po9NzmoFEepb/lQTD7/Gwm1
rUqsWkA4f+fOwfDXu6Ly4Dap0fOJ6gu0q+Mip+c0gqe3d9C5SFPyA6l6IODg
yER0SxCSDFQlpc524/GSKNcllwNRTvh3L7XS2R+nDUJ8pnfrWMhblEzJ1/yP
fxkwCCnkfE3K7xWAermCzSmJq+jHvbIisaPVoPS/541GbB5JP/v4qyGbz6vI
V/cOelXTev9nej6cy4xYtXVjLBD3l5DvhzhqE6UlgzIoZBu1ezAyjv6pz4gr
Zs6PS7QPj4+Lyzcvanr3gbkF/dedS+No96e1g1n0fcB75s7TMyvj6T3ZacW1
Ab04/3s+5zpyyuoFq/cHA6rPjCF78J1f86gX9ncvCYquG0MzM5WrqA/2wo/F
j1ZUG0dR6/Hpt302veAa9HhodfwoEktmpub4UIX3P0rRipkdkK2C9KXtzzX9
RpDm7CX+kz5VoGlm7hGwNIzEurY/OL+uGoLqJ6cY7wyhA0UCfCaDk/h+VqHL
rQa3JiGps6DyuM8Qsu/r2nRSoRgR4/uhqDOHsk3K0GpOmTVIbhAJSbBApX4p
Oqf/Q69uaAD5X7+UorFYjf2BV0tc5Ub3quHVYReNaxKDiDOVO40v4id8HhwP
1S7qQxLZjXMNZ3vgx53NC5tkBhHR3yST+xv0htI/tZD7J8RC6S/nyP0lqjmi
cWRNXhcej/fnP/xt6tsFHg8yulMfdqCvijH7c4rH4KlH5MaDLN3on/4U/dOf
oplgRpe+z8uQd/HSjvW03chdvHo2z3EZZBhYn1OnE9d3o3Ty+ICSniiwlo71
oJ3dYp51fB3IP8uNP7K8G1leFGeZuE5Cpze/OhepUQcZT1vHLoaREDF+MJDH
D2TswP8miq0L+wvrVCx3sZ/pAoe1H4OSn5NQr+nGUoehTuA+83LNRG4b0qcy
8ogb64QdFqp2yg1tyDhmvGvJcR6E+UPWlLxvQx/p+rpDg+ehwSjv877vbWht
cPG4oXcn9heSdTJ8x193QobLehvS4C90ine1t/atToj/2fhlNKMV8f84Z3X3
WCe8Zj7UtGuyES3pG3InWnWCqDjdg3O7WxAxXh4mj5d4fwMzeTxGcTd2K5le
r0fv1u2Q6JVsRFKM3ujtp1r04JwNlZtoA+pSNHJ4UtuA8/U/B/aifIMGoLrw
5sFhtcaVenRU6UxBA/h6z6xupK5HrfOWG12EG2GbwNmXU/tq0J2OQq7Xow2Q
yyoY0Pq8Fq23EHH66DYJdGez7iQP1SCurobiDyWN2D/Z8EA5t9m/Efi49tFw
UlWji1uXi+3iGkHnzYV4XpUq9Cyb+hIPQxMIiv1xMP1cijjv8Uek3lnxP3hl
PdmvVCMZiv+xSPY/EDF/VibPn9HbkqVNCs+b/tk/MAFUq15dkh3jkSA/xzis
XvvKRE/UTIIoV1sSq9bnNqTsR5qE0Xe2T27n61D2N03ABnbG1va9hhKEP0cV
leKwjpdFgrg+f6nWYnufDi4X5Pl20u20JuX3piEVZHPaKtRziHJue9VBRXdN
yvUzQNWtbZOsRydBXD8q058X/Ucdl8eNSD4R6z5DuX8WAtZsYNOZUc4hygNK
eg57c6pRrp8DqnNzJ6+ELYgT17d9thn1lFXB5QFWRfeN7ihS7p8HqiP9W6cf
yOcQ5dwCb2n3NStSrl8AKgm2uQ3sk+LE9caH5WjWN8rj8oBD6uc7BuQo9y9C
nJ7Uw1KvUzlEOTe97jH63acp1/8BqkiNzou/h8WJ6wMi/jC+fXvqb/nSzyAN
UVnK/UtgPFhnabRdJocoN97/+dItC1nK9ctANfvMuiOqV5y4Pv/apbEIORlc
nj+iNLaVS5pyP5VUG+dJJnHJkzlEuTH/tj3JctIShP/dRva/gZtSfwn/l6if
/OT6CSyU+sxLrs9A1GdNcn0GraV1o+eOdcDOasYjjttKIY7iv98h++9AtM80
cvsEon3eJ7dPINonsR+lhdL+NpPbH0RT2l82uf1BHqX9Lpv/r/0C0X69ye0X
7/c4R56PAPN/2yvwU+Y3T8jzGyD6K8JfJ/qjl+T+CIj+SJjcHwHRn70n92dA
9Gex5P4MiP6Zm9w/A9E/W5D7Z5hzGG+V3VwPwj9tmxSN2oCYr/WT52tA9M+E
/0/0xzvI/TEYUPpjDnJ/DET/bU/uv4Gojz/I8z0wpfTXR8n9NRD9dT25vwZi
fvqQPD+Fbsr88wZ5/gn/jHdAjHfu5PEO2CjzZwfy/BmI+fFT8vwY/hnPgRjP
GcjjORDjuQF5PId/xnPgpYzfueTxG4jxPYk8vgMx348kz/eB8E+I/T+E/xFI
9j+A8FfSyP4KEP7KWbK/Avsp/glx/1aKf/KC7J/AP/4UEP5SEdlfgn/8JfjH
34J//C34J/4BHO08dRMOFeDbZypK/XIM/ol/wD/+H/zj78E//iAlvtcFqsUX
6IdtlTCfOHv7aV7/RcyTD763Sfl6YeYNNT0WJR30935JE6qgb2FQUfGmietP
F/SXxpzWsjwMX7NhbelwF4jw21FpXzaA7BfVxz3bVupvpZeO7XFrSI2+YM1b
3QXrLhvV1+o4Q6L/7P3Kwi4o3kVFO73RE1Z73NLSDemCiYUD7qPl4pDaGTF7
9taKP5IuJsd31hB0NUIfnonrgtGNbqIKjdZAle93Qim5CzQtui/4hzhD0kHP
drmcLhCS2Sioz+EJHKO8to9XdYG+DpVM7sxJ2C64eHVqYxfMFbAdfK1lBGyO
P1+pcHcBz1u9lux6a9iW+qr4+cEuMJfYxKIu7Qyso4H9f8S6QDR+7siX+x5Q
Q//42ueVefAehZrf8vvE4Yx81FE2+y7oVb621SPhHBRfCf3laNwFhYVRCypg
DXIlXmFFGl1wJTX5qCCrM3xmtBfkPrXyfP637T5eI/bf5kDBCdPCjgYhzKMe
d+bdWjUxbxPcF6QXb4r5WAIDw2ytDeYDjBOnSkKcMEvImYbHxLphVg6oKrbO
8sJ87pMU0/FqX8wi8/V8dNMBmE/cW/9jgCYYs5zwaZuf60Mxq1b5rcvmCMOs
bZ/xJn7fFYjlcmRRmEPgnh2zfNtnC6yJEwuvqUDw48CE6l1ZGXDlXrV8PhHB
xsC5pdJBdfgdV+E24IMA2XrVqYrqg8LOmEH3swhEXi9q90qZwPun5mY0/Aiu
xeVI77O4ADt4BJquUyHouONzyCbCGq7Ez6mx1eeBWIQE++tUO5jg+fI94W0e
yEst0Uz3O4LRsxsnBEPyYELsl+fdP85QtEvvQ7Z+HsQezRsW2eAGRxJ2HTgt
lAeKgrHm9Twe8Jh3OL6aIQ+m+P2aPI56Ae2LUKGD63IgfLa+trd9J4xezaeV
Fc4B2kjRAOpeWWi1X6jVN8yB8UifC19ea0CxhshLp8s5QNJrXRveoQ+ZIk6X
wlNywJqFq+H5dRNIYH+j/KQhB9rdKl6FJV2AyKXuHRnUuaDf5O9r/dUafEk7
x0r5c6H6hKCaUrsdWH87V9hxNheeqztG3V1yBO1X9+7M++RCe3DRpTBGF5C+
UW3JkpgLnBnc5p7sbiDgzCS2tyIX9Ho9Vaz3esB2bXlGmM2Fu+xVIvoiXuB3
MuWe3fos+J5rtU0vZS+sf6KQXiGaBSUVzpybz8pB3GJHqZBxFmzMujC/V/Is
COn79USHZ8GJ1Ru7v6sZQEHmNprp1CwQOfDb6ki6CZzd8o5DtzELlDU//KLJ
uQCdziqin6g/gqlPhGb1ih63yh51Dv6P4JlgUvy0wg7oDwVd9NP4CKe7t8dl
DTrCvQiOsLZLH0Fjnt4gbswZ9vZlPJV+9hEMmce3hE+7Qpacek5CyUew4mmt
dFhwB4WEgTr6yY/gKlIUoUPlBV3t63UHJrPhxOCicuKnnZDLppR9qDQbDijd
9N7+SRbuqF/e4fwsGxLejqR9C9IAuyufAz9cyob0D1sT/V/qw6nPi52z6tnQ
9otLhcPSBDjnROUl9mVDD+nI91mvCzAp6PLanyobhrtPn6y9Zg2lVsnMhfWf
YLrfIDvtiR0kxPU50ad8Av+ZTPbTjY7g3cBbq3D5EwTNfdb5QHIGjQ3Gx64b
foLLCyW3dw26Ar/8w4eVwp/g6p/aqqgpd6AOqFvaxPgJri//ZqZa9oQk1v1U
5QsIx0PI7asc+khiqz82SmLWHPI6eiFYH/NMpPPmQ0UWmOO3bP6uyeeIuYg0
HHyr3xXzSMoPqcq3XphZfRP+MDn5Yd7xHFp3xwRiTlv3NvvLQjBmWWf2GDPD
y5gbGsI8aVA4WA3WlR+eq4DmJqfgysLdwMgeGs3UXgEnrqubjn6Wh9TTR3R7
iyqApN6h7r5GGzTd2jkKUytgd+tj58m08zAbf7P98f0KEBZ+o3D8rTk8qpRM
9AyogDqvNfqegdYgtTRgc9aqAjyRhU2mlj107o8ROKRWAddpTCUXbjtBmJ78
5OpjFWD0PPIr+7IL7A+bzurgqgABeaQiZu0O5e8TfPMYKoBqYKhW76cnNI0o
eZ1bUw467Rf0MicPgvfXDyLSO8uB/kDVjNFDReB4yDXJd7wcmh0SGItatCHX
8Uoak3o5NPGmqo4eNQZDuQn7Caty+MbCfM/njzksbTc80BhQDulLtr9Xr9T/
2LFvfXn3yyF24AffnWv2AN8FXySklsOpwU1ZG145we9HMeZXi8ohmyTdUsfk
CgHOdDyObeUg1OhE9djJHbjl7X9rzZbDi4o4XrNaT4g+piaQv64Upv30A7aV
HYYNgoVO+3eVgqzkhNVnHmW4zify/q5YKVx+3CLv7aADa7mSpqk1SuFo+ZPh
YgtjCGPlOG5nXQovVKqONWy5AHTMN70bAkqBrYwmuOuLNQSuosmTuV8K15SF
y8Zd7GF50ZUqJaUUlrvS1wVEOYHPZI80+/dSyHpekkrf5wLzA/qhob9KwelC
p9Z1KXfw6Cj7PjpVCvy7/8xvuu8JTkdrn2RPl4HQ7nCemBcHoUWDwXLT7zKI
7B791qatCKcdjx+y/V4G/e1PgoaeakP6ddupgpQykHvZoXaU0Rh2vH6cw36/
DO6f92hz+W4OV4oqgp0DyqCPdZ1jepA1THVRKxVblYFYeezyqKQ9GNMKb+RR
L4Nv0g++3/NxglJui0av42XgsMNsvWiTC4ieuB9XtbMMts4f0KkTdYdnBsWW
+9aWQX7tdKzbHcp+Tap6SDnxOUhD+jRmn0HJq9wCJpiVr5S4vLl+ETNz9JtD
HNMumN1NXHaZPvfC/EtAbNtLTX/Mu/wD6GcCgzDrdHdQxeiFYr6mcnpBQigc
Nhs0fhearwfX7PWk7ujDMKA7Lps9UA9F6wfXLoqoQYH2ugLZ1np4L6hpuFRk
AA80eaG8vB4q9/7SCXhrBo4akjk6+fUgd3xjYKC7NZxW0z7enlYPOQpyr4LA
AXaoOmTYPKsHtoO5fSdX/KdJpfAjk7frQfOojl+BlRsUKzxN9b1cDzckxzbK
qHiCh9T286oM9UCdJKwjaCIKI3t22dzbUg8mV0ODhk+pgyXTfre2XfUQ3C85
EvnuHPyeOByw73A9aOxeePtggznoNIlFOEvVg5mWTqfNd2uoyJe+m626oi/0
HbuEnwOcfqH4lM6wHgqawl5ejnEGdF3jjYptPZSMf5AZcXODY676mXe96uHn
2s5fOuqesOHwg9BXq+tg+9xFzlPZ4uB3PV72EWsdWN6tV33MqgGDva9pb/DW
QfjwWW3rE4agL/uhwP9wHWju6k1mP2IORU/yAp2k6sD8TGI61crziCx8lzJT
rQNXb/OMbn0HSNCuWtY8VwcFZ1kMLTycYWN6U56cTR0UW48a6Z10g0CmTt9j
nnVQ7V9hrLzOEw6WLnMVzdYBA23jkSNsorCF0Y3ldX8djA7drM3boA6LSv00
N1rqgOvhvl+CBuegM+L8pGN5HexJMHGpqjaDkpKarrP5dZB3Kdljrac1vFun
WC+SXgda6gveMpwOEKOEitgS6mB6E6/UCRNnCIoQ/rR4pw6k/RULmo64gXVJ
0uvfYXVwvd9R1oOeqM+1sHy7X2z20xnMvuGPKqu/mWLOtvW71RNrh9mHT5Df
T9wV83a2nRHhL7wwf1q7cfDWJn/MB/wCE8shCPPi9Fzdz6kQzCUOzgyNr8Mg
PrQIjqyrgQNltVIPBk5C4hpjg6BtNeAdXKPs8/AsvLo+4161uwaMuU/M7T9p
BMksN29xH6mB+F8/q+1czCH9zp5kB6ka2Px2ddKpb9aQwZb3I0+lBi57S/pz
sjnAp8daXUwGNbDI+/HV/FFnyNs5RGVoVQMi137esfjtCoXPQzjeuK3MTyeG
AqrDPeDKlozyPdM1MN8aNlgZvNI9q1+STe2tgQ+zvLECiRpAf+PEp+PNNTA4
x0dta2sIssU0goVlNRDlu+Xw2W3mEEz//bly/oqepbsdo5es4bN0xPa69Boo
8tt250arPVD5n4k6/7wG0peVd7FyO8OJ7E0MffdqVuYtTIvmVa7gO1Pv43y1
BlQCymveBXmAy50DYQ4zK+/z6vDaPktx0L8Zl/6irxbqMia6JNvU4eSVzb9+
N9cCT8n+K4Hd52BvcPiabeW1kDkidNd02gyYfReF1fJroXWuzDLEa6V/dHc0
Dk+vBVo6m+OJi/bQ7NgZkZ9QC5sZgld1gjMU2Ohmzt6tBT5G9WFXajd4aV5K
ErxSC2IsXPWrCj1A5t19p+k1tfDikWSQbTsAU+n12dtba+GQxLOE6FYNqO8M
8j+yuxaOsGRalRYYwtM/HvTVh2uBw+XCXRpNc7BltbvuKFULsYYOs7Er9joq
YLp5vWrtyrjmpS9+3AGo5HUevjWoBfUHJ38NaDlDsbEyj7J1Ldy6IbdebL0b
3PE6mdTvXgs/g5WkwouI+VIrMAiy1G9L1MY8mpYQ9jnOCvPwhTZ18RlnzJ3O
w7+3xHphHoIlEU+eAMxWcqv8utyCMXcoM37V+HEZGjKMjj1f3wJcV3iF9Fcr
AG/ry+oBrhZg3N551u2KHjjSTlw8LNgCX2ZjvhudNoOcfZIMXlItQNMp00VT
bA2r1cKeIrUWiLN0Dzy7xxE03avE6U1aAFoVI59bukDco+11yk4t8EuD68nM
S3eQ2ZcwOD7WAulfrx7Q8JYD5+gbh53aW4BUpz0p6ay78n49PUcqW8B9t7C1
9U9TqLQ0y7PLbwHv3xIN7+ysYblKhW4wpQXyS+N3LdM6rtjpmKLNkxYo2Mdn
wKvuAoaJPJG9N1rg6+XXtxRi3eHhreQSq4lWaI2VD+LeLQsLrhJ7LpJaweJR
LY13kg4Y6BQHOFS1QqqrDOlwnClkH9dtds5vhQPbOWRe2lrDdo7uo+4prVBj
PF91Y7sjXFpyifSKbYWN7y2CFAJdoKmNesDneiuoMfw8TNfjDhPD1vZSzK2Q
xa7k7lsmB8xRYr5LO1ph/TMm03slurDvyLprOQKtYNJyie7YNjOQrW2OuSTV
CmI9wzoJK+/zvMebpGPqrbDWK00r/MTK/2PzzZo2aQUPP1v/vHsucCdb5ft7
5xV7B+9OmhylfH8ioB2OlJVWV/ofwzylVKnEuc0MM4//sxPckU6YFRuM6oVu
e2H+uI0pa2dpAGblrzYh/OUhmNucitSOVIbDwYast/zL///dmzucDFpbQLU9
8EnbZDsIB967W1aiDg79ilF3+1e48ZXh7p0mEDmxKVi5rR3Kx9aMmp2zhvTF
FleaupXyUOGTVSWO8HNVokVWSTsEsphfdexzhSlmB137zyvXP47+uZ5h5flo
p6yHA9vh1xU9F/+svfBQ7kHy2mvtsNqw/oai7FkQuSo5seduO9h8eXqvIMsE
qsraRWXi2kGtqb7nfIU12LFc9jn/qh22z9I/Whp3hNVa/J+937cDMy2PrMqS
KyTcL191P68dqJklBx+u8QKbxi6eMZp2aK+pjdhYcxga+I4VZTC2gwiC/EQd
TZBzv2rnw9oOMXRf7Xn/rMzfv7RslOZuh/v7bHzdyq1h1yaBLAb+lff3gkGu
o8IRokwCDcuOtEPJcf2yiKyV8Sb1J0205Mr7L32jKfzUE8a+hs2w+bUDuzy7
yHGVvbD1ceStfa7tsBdit6WMaoCk64ODx23a4UWMRGizrQmYKcUXyRu3w6ct
/H7nz1vDFZ7XZrra7WBRFJHtlOQIqXPvliyV22E0lCbryDNXqKvMeeAh3Q5e
Ml7vpx55wmD/xG+n8R5YvyNc7kiqFgSJhqo8aeuBhj/HTfeyW8PWENbs0vIe
0Jy+F7k92QW4BY9uZL7ZAylLyuHBrLqwyDLzRzeuB5xNmQ+UtFlDw0RW37O0
HvjGs6DA3+wCH40SLliZ90AZj6237bGXlPrTC7dIm7UezZthlhjLVLLT9cJ8
P8tZ5JteCPhJmMfOM/UCu8n88WJDbYgIGqsz4O6FwZpy/Y0d1vCgyG997uFe
SJRQK32i5ArCi96SL1xWfu/1k+Oin8+CeRer9xvjXoj3lg97vN4a7pSnZaWp
9EJ1ZFvWGS5XODlz5iHDVC94mrl2211Xh6zi3U+y23rh2qqo5ABaaxCIXYh3
KO2F5MEJ4d1xrlB82mRvwbVesHgJR5usNYFz4umIUVwvTCvcDLzWZQ2OsaSM
hXe90HIeJreUEPm9I8iShStOWV8V89oz/htLcgwxp1716BfdY4G5xS5Faa29
HeaQly7aw/rOmNlj1ofVL7n9vT/idWZ+vBdmOd/TvUlyvpjtM52uKbgFYD6e
inhPnA7CTJfElHuYLQRz+VMDLb6BUMwPYpKGtueGgZoQb570wghKjJ5Ja9A8
AQ2zg68zpkdQQyero0umChjnf3jAPz6Chpfvy0jyaENPmF9Y7NAIcry+pyLf
yhAczpx229g3gryY9A/G65jCNOsGs8udI0ishnuiY/4C+P1qUJv/PYIWYnqy
dsdaw6rEpyfsm0dQjkmyn+VJO7huZ3OAVDeC4LZYvKazI2w+eoRdu3oExc1w
hk7JOMOjxQWG4rIRRG1AbXV3iyvs+vJlSvLHCDLJ61QU7XGD1xHXO9K+jKCC
nUUrXZcHpJY3n8wN+atnL8MmTYUrf/XEgaJF7fW/etg8Az1Nbv3VE5WadXXo
7l89a/tGHnk9/KsnaCdfyqq4v3oW9Iw+RyX81eNy685PzqS/egaLS7uS3v7V
Y05LN3s0/a+eVnHxtQUZf/VouTpzqGb/1VPdrymbSzeC9Kp/hfau6Pl6e0BX
Yc1K+bqWsaAVPVknguxqmUbQAR3zSvkVPa97twWZbBxBHi+b+CpX9MTeSrk7
xDqC2n1eotcreqIk5F57bR9Bl5+b2Y6s6AnpbkGruFfuL9+x9ciKHo9Il5oo
3hFUPd1Y4L6ix0ZsbR/nvhV9NxVFDVf0nOuM+5N0cASt593LtLyi58wN0Y0i
h0eQadaqjrgVPSePle8pEBlB75U7sqRX9AiTzMVVxUcQQ3v+jc4VPYe2vM6W
9h5BqhQ94rabX2S4j6A8ih75z763+J1HED9Fj9bWHt9YuxHkRtFjaqdmvdF6
BLVR9DgUftS8bP5Xjw/bLqn583/1XHG4tt/e4K+eu1+nWEnaf/UkbD9Po63x
V0+aU9HQD5W/etB3oUZJhb96SjkffkmT/aunX0pCI3il/yTyiTIFJocqU/Nh
cwn7nuOrYinr2aNoPDp5oHJFL8E+LburbVf0EhwgKnZPlEcbc0GAyamPVoaY
aS588XqsY4r52ZoGtV/zFzDLJg/wceHvSY2iLo3lpfMn7TAXeq+xVXN2xLxJ
NbpuVMYZsxk3p3TUFlfM78cT3wr1uGGm+yrIVr2iV7c/XmcnXQHUco7f9ii/
AuxP7yTS9pYS+03Qpxl7S8um3n++Z94Hi13SmerDY8ikwi7OxHwG5/uQ+6MZ
uKd8JePm1XH07qjAqi8sVUB8/5Et6/BtG95ezBGXbie09ZZT8gUH0ecUs1Z6
tUn8PXPy701C+PrV+XnKQ+hr1nmhKrVxuLUuRjHNrwcxOG3zXwwYx/mw5OvH
oVArfNHxTw9ypZfaIVtEfIe3DTFwusa2co5CAd2mR5XWRP7pPDxZLEm/J92G
rrQbsFdwL8Due/Rrtl9oR16L7IprNBrw97iX6JiOrBfoxOypfY37qFYj/v42
kX9sSM4/pvz+JIyzn4mdGvyJYninZ3gsJ6GcnH+M9GY0JY8yT4Ffb474yLE6
1JI9VPD7bh+0JNoVUvUXU+7vh7g25WwPxnJUY/L1k099H1SQXxDioHx/j8gP
t7yYn2o9MQRNHXdN9vZ/RMwcxUyriodxeTPl9xvIvw/E78eSfx+I3/9B/n3w
ouj77/erGxBPqtCxvb6NsDbbrPub6U8U+Zg1+WBnHfwMsp3T/JwPg2Uzd7Pf
1YE75f0R328k3h/BbhT7/Pd70r3Ix1/VQkygA0bkUgVVY7pQo3Hj88kVP3Vv
hdGBsRU/beuSXK//il+oTbPDK3XDMOJe6rFl1CPyokfQKH1f18fabiDylaTJ
+UpA1Dfie8hEfSOYqG+h5PoGRP0lvo9M1F+CG5MvpqptrEdx7pwy2ovE95Dr
UIPXs++740ZB7vW2EwJGleCiZcv47Pco/NOe4J/2BD2U77/2k7//igQo338V
J3//FaXzX3uwKYUE0iyBz1KjM1H+um2VwY+GgHZb2ZEujkzk5lTQoRQ8BOnT
VwJV1DPQ9GWaS30yTSBlYFo3euAHhF9zLBTPIcE8yVRNVSkbas+nGmue6YMB
i/vFeQ+L8PfOiHzlmlhlmo7YZlhf55V7+KILIvbL77b433559IeyP/0qeX86
ek3Zn36ZvD8d2YmPHFHT7sX5oq2hEi25tL2wFfX1RKR9hGeK6xmroRdUeGL9
r1nmwa7aOTnRoF6ocefqSI37vDJ/GVztQ9P73/0mASR4Tv4+FiK+x0Y8L62X
4NwlYxJU3FuVeEzUGhH7kS3I+5HJ7UmKBDWVt89JsFPOp/h/TvTi3q1HOY9i
hQ/75r8Y5TbBvLBHFqnescZs2mtmW0fnhFkud7fHKnDDzH+rJ0jYi5jfD4H3
RODVJx/TkDLHNMNk9SDQ/+/vqShz8WkV684q1Bez+03h+BClvVciLY+EwE2H
hlB3FZ2SblwlItcXorwavbNwZY6/OoyI+r+TXP9Rk1vNAfnKetiyv3VXXGIL
2ja76f0IUwOlf2pGveD60oT/A7wr3TfErpUJfK85n++hfvfPfp73cMoYPr64
nUXhLPiszxcVxvYebqtB7pfpD3j/0/GyMoeO3g6Iz8zoNo6og/Ck871yazsp
7aMeGhr1wrIrJ6BhjeTkT78BcM4NCC2lnqSUU75bGDABCjT7JhKS+iFMe6bY
t6oHosLiDB9GEN9H7QX/i9sPm54bgdhqJMw500Ppj0bgn/0klOtn4dehnNep
hybgn/0l8M94Bf+MV0BP2a+0red/+5UQsV+pibxfCe27dD37s0sWvDdJECi/
+Q76mZryDYqGcf0m/94wGHc1nKtclwe/DK5X2IeTQHzplfHVVa/gqx2H5MU0
ErxyTT2lZEh5vwGDQK1rfW11kifQ5Fap+nEPAqPwPdfnS9bA8L77ZILJIAhH
r2k3V7CCmbtRHuPtgxB/PCzi51ormHqspxoSNwjX9im1O9HawG57YKe9PITt
w/Cmprnt0xBcYCOpfOfOgi933xmulquGeY16scgLN3H7JMYHon3uJbdPRLRP
XXL7RLyU9tlKbp/oEKV9EvtzHrhLq84ztqLNH8avWP4ZR//0j+if/hCF7mVb
/qI5BftRvaWR5wgi7B9Ntj9KaTrucOXSt5X51iSHX08f+qc+IbHjOs8Pb5gA
CSfXmWT/PjT12o3t3PpuiE+yz+LQ70b33FVLgxMGEPdTvunFLV3o7CkGv9pj
fWjs0LxleHUH+uiqvqVsqR09iY8In1rxDzTL/zO+II6p4/ufK85BI0/bSx6W
30g4g82EK28B2qUll6hoSUj7Kr8ClehvNMLPcv6BXSva5KLql+7ahPIcJ2Pq
a5tQ2X/HQ0S0l+fk9oIWFGRy7OUnYEObgbONUjXapj8lmPZiCno28gguBtQj
UZa0TQwxCJ37sX7RfHMlSmDpuvmS+w3yXoxV+HW+GElM5I13Uq2MGyE/Ndui
fyDqWPnl0cwQ4BA3TLwg8QXRcwYO+rWPgFad6YnK24UolnK/O/l+IO7vJt8P
twyKvulxj4Muh/nTScEKIJ5vPfn5YCNFXzZZH/ywj1zzkakK7XO9tv/JjVog
3udj8vuEBkp/tIHcH8Fp7f0lphtnQUXDbf1MVAsQ77eB/H5Bk2KfEbJ9QED4
8w0Ji04EdSXXK063A+G/fSb7b0DY+ynZ3jApxvT8s/ASvHma+cXMsxM+vK1I
Gk/IR03uda9ofvdAEqU+tZLrExD+ZBTZnwSiPomR6xN8+G//DCH/ra+g0vhi
z9DWaZja0fb25cdRUD4Qg7LsmvF+ACUKE+vnlqXhktqnmvH8wnm7lH+ZDwmX
c9dB3SONbFzORjl/iBjfWCn23Uq2L4jRR/FKfe2BgYQDCodfv6f0J53wO63z
YaKVFuYdCwUZLb/sMSeW/1j/LsQL86/1bKSF0kDMNsdUJGLrKevpAf3A6zqq
wuOtjvl8/N0tIbxemJujG0q4pojrB9HYqS+ubAaqmKFeIj861xBz4dd1ZmU4
njKIurnmuipxPGUQrZJ0EbYxcMbM2t83ZbvshnnvPeNMu2demMVk6z0dTvti
Tpzd8zgLx1MGkcAZku9HHE8ZRFnPHxt9wvGUQXRyUReycTxlEBVrbObOyaXk
EwUMIT3uhvwqHB8aQqaMFwJUsJ4hpEwnsOEB1jOErIL2G0dgPUNImmpzwwms
ZwipwUaqY1jPEDLy27BXCOsZQhdzmdX4sZ4hJOlHnxOF9QyhL49Xqd3AeoaQ
Ui5d51WsZwhVtdB6hmE9Q0h3kYYxBOsZQKousdPm2D4DiKpA9lMr1jOADux+
7EPHZ4FZt28rB7uDHeYC7R+t97CeAdSQdfltPNYzgIbZZX3fYj0DiM6XWiUL
6xlA1x3O32jHegbQfe4T091YzwB6VsVhNIj1DKDkoIWvY1jPAPp0pOngDNbT
jy7t+vngPdbTj/KNj+XvzzPETLpX+OAE1tOPzJdtzlpiPf3I5cgV+iKspx8J
95wza8F6+tFUjGD+KNbTjzJU6ThXyftiFqcezGF0D8C8vVlrgFU+CPPCe8S2
kz0Ec8uNffL7B0Mx51hFux/NI/T0Imma9m8m51QxhyU8tanFenrRtFnfbA7W
04uSpgIl1jvaYU6WYcoXPOeMOfxaY7selTtmk9rntMEJOJ6LxHc4736D9fSi
q0eePLPBenrRjE6osD/W04vMfWy/RmM9vagqTl37JdbTiyS/ivbkYD19iNeY
MWc11tOHnqc3H/TDevrQo5iGJW+spw91GxR2Z2H79KH7zDo989g+fUjSQdmA
DevpQ6SKkxWiWE8fChMUldHGevrQXfm7BmJYTx/qnY28o4D19CGxpIgKXayn
D13Tu7zGCuvpQ7/WBMp4YD09qH/il1A+1tODwnNeDwghQ8zXl4/8pt5rgbmO
+1aRDbZPD9qwc7bBGtunB70XeqESi/X0IF1p7YJqrKcH/dFYJcqg4Iu5Sqip
9jHW04N2fRLakIX19CA36SuK1VhPD/pe3BYyiPX0IPazxxA9IvR0oySt1aG8
hqqYzbXERG5jPd1oDmY3K2E93UhK6ydzKdbTjWSP6oQkYj3dqBfJUvdgPd3o
muJhPz4i/2SFBWu5Fiyxnm7Un0IvUYP1dCMJN99Lk1hPN7ohNpm1eXsI5rZl
mxnhoVDMQl/bj2phPZ3I66usNbWRKmYDOdG4i/mGmP9s7j41iPV0otLLqeLx
TnaYVX1ytx0wdMYs8iF+Uxi1O+Ydw2HMJKynE9Hz2a2RVPTFPCOwj0bZIwCz
belDfy+FIMy/rZj/JGI9nUhjVeClGqynE317OjFDnU/o6ULpjB0GN7F9upCD
4hvSIrZPF5IOe7fuAdbThZT7NiQewXq60Ovdotw92D5dKK3ugp0w1tOFMsJu
fwrEerpQzrFChgpsny7UPunJz4z1dCFzlQ1VwlhPF+p+/sLDAOvpQtZLJ3YE
YT1daFC77stLbJ8OxCO+2ccb26cDzRtGHu/E9ulAh89E6J/aZ4HZ0kH1/jZn
O8zl31cPuGH7dKBenf4rv7GeDkTTW7JXIdELM6fn2+/p2D4diCPk9JEbWE8H
2qecRVOI9XQgkc37qmewng4k0/Lg6YHhUMxqCWudTLB9SMgkUiV0CeshoS8C
VTHmnw0xvynxnUvAekjIffnW83tYDwnFsdXs/Yr1kFDou19+J2jcMduo9NVk
Yj0kdKZngl9IyRdzjJS/VC3WQ0LXrocMsCgGYfZrunJXlSMEsxPfzZMRWA8J
mbneHvyO9bSh6sd1bLzGqph5CnrFQwoMMb9ldznqzW+BWaFmTPm9ix3mcblj
6e5Gzpjzn81IbqJ1x3yDKvNHCpG/s8LnDN21lJV9MW9pjXF09QzA7BNxKaMA
62lDHcf1FzdwhmBW7D0ufX4kFHPaXbbwt58JPe0INX54EnFeFXOcB23jMLZP
OxKkZh1bh/W0o0i5V27cWE87mj3F5cuO9bSjl1bWKAHbpx3pRryjFsB62tHq
5D8yH7F92tG53l2FO7GedvR+Tr7TAutpR4zr7Fe9wfZpR+Yc0XvGsH3aUc7B
rNMihB6q34ifkTP4B2GfFX4WUau4p9AQ8wN55v4JQs8KG5ekJWu62mHe/OLV
xyZCzwp7Wi1ujyDss8LN+1T9xF96YYaBJ20DhH1WWL3Oc2c5oWeFaTOfauxR
CsL84V5xsB9hnxW29Jx4X0fYZ4XZ9Di6DxUQelrRiMv5XF8TVcwyJc2tdVhP
Kzo9HGzvsd8C89A0mz2tmx3m4uX6jNPncX4N4qR7lsFE547ZcbV9xk+spxUV
Mh7PeKDiizliNU2zgBfOv0Hvr/Y03cJ6WtEvxrKmKaynFTHcTG/SHQ3FLMhy
vykb62lEnYYaxhxm+PwQlLH9EJvdV0PMEiFs1IkHLDDHzj90u+Nuhzng2Ho3
aWN8XghKc1ETEFvljpmUHNUn9AqfX4I29Vc/23fGF3P8xrUpGy4FYBZL5b92
TTkI808VJevVXCGYbQds5ULGQjHTXbm2i+oLoacZCfG/3Sxjqor5sDzX0ftf
DDEn/plwYMB6mtERHd2vBdg+zchY14gJYfs0oz2hG7IeYfs0o8G0AtNLSV6Y
03+5Mumq+mLOCd3UEoPt04y4Hl1MW4P1NKOg9K+XPXeEYO4q2nGuB9unGcn/
9hDSLiT01CPSqbdCTOaqmMcbSKv1vhlivhTmkaN10OIvx/tO+nnYYc79dZK1
BtunHkkOyqb3YvuslM+dVv2D7bNSzqDUz6Lmi/n8Wj6ZaGyfesQVtTeKUwXn
36LfW/l/v8D2qUdPYvcfPDwe+vf+3Qe9c7B9atF8udk29guqmNedPuZj/N0Q
s8pZrmm2QxaYNwfzJAd74vxKZN+48Z27iTNmDafNN1/Tu2MWWcNq2/Ya51si
9qdbT29R98Xs/+gay27vAMxOVtf9EdZTi0yFbwzpceP8S6RJddNgEuupRXKl
N3/c+EroqUKaCrkLTy1VMcuK3JBs/mGIebgjNm9cwAJz9hv3XzOX7DB/EePz
XzJ1xszPtWl94Gp8/g+KpF2OpU32wjzd03/o8llfzIcsbM+H+ARgNtE1eXHq
TBDm24o6I/Q8IZiLJFREiyZDMS8ckvG/8p3Q8xOZqtavVrBQxczqkCR/pcgQ
s+9DF/oZbJ+fyMH/llWSlx3mtDTdroNYz0/0apcVaxiDO+ZndzxOt73xwvyI
IczzuIYv5hKJEs0f2D4/UdGUxpCbahDmL8lNoTw7QzB/tjTdUTERijmXuz/D
+xuhpxzNFLeNCVirYlYtus9jUmKIOfIO954tQnj/B5r7NvnJy8cOs5BPJ9+I
mTPm7tuqzk/XuGOOeZuVczbF6+/vf+NhoNfyxRzo8G3vOd8AzM3eCypH1YIw
Hw0XdGHeFfL3eW5fuN8zFYq5Py4mN7+I0FOMkO/JdBpbVcyVyX0X+csMMRuf
0cu3PmyBWfe7pOIJPzvML7lYbNZccMacYECbMrHWHXPcvamJ1lQvzI9+9hwr
0vbFXMowdELPLwBz8W4LE1X1IMxFMr+DZXhDMH8z0U08NhOK+Yt/VdHBYkJP
AUq7bLPY4aCKmff7KfPxKkPMnrv3X5g7aoHZyPLI/bkgfH46iq953LVo6Yz5
ztTOZR9md8zhrInb/rz3wuwtyn/Y18AXs+iB+udLAQGYUyUepJ/QDMLMr2KA
/PhCMD8z5CzNmw/FzGn/u2GpjNDzDX3ubv0jZKeKeZRKwlS1whCzVNgXErWw
BWYGZtvb5wLsMFsWr35mYOGMuYSpZSGc0R2zgHqKZka6F+bo20FvO3R9MS8M
8AeH+wdgfnv9tsATjSDM5wWWmj/sDsHMUmUZXjobirnAuUq4owTvT0SL++Y/
G7ioYr77im6/bq0hZlbuy2KBxywwk/qvnNe+bIe5vkmuYIuNM2bJ/CJNKhZ3
zM+eK/b0Z+L9lWhNRKlXrRHeX4lEj/HRtwXh/ZWon9/qjJZ2EObHnEn3i/eF
YFbb0N8Of/B+S0RLu3//h0pCz3tUH3v3LLenKubIidYl2iZDzE1Ks30L4haY
Bx8H1kxdtcO87BH6e+aiM+bVO4trXDe7Y2YpZi4ey/bCzO5yFtmb+f4tZ1Zu
cgsNwHzXdFhTWC/o7/WZkRXjB0Iwx647ophGdRnzLuPaLw41hJ5IlDLpFEZ3
UxWz/i56a6dRQ8yHEvedTlW2+Hu9WGUj31M7zPctxTMqLjljbki74b9utzvm
bYvt8qcrvDDryR3dGOzli1lF/ovd76gAzHZDdhp/LIIwX7+1VXS7RAjmt6Kf
tx9nuYy5vMWGSqeb0PMSJcfY7wnzV8XcXcphLdtuiFlsT+AT05MWmKm7Xdbu
umWHuVPj3ScnJ2fMRekHRqvY3DG/2Zi4+3CBF+ZIZy6DWza+f3/fWlne60oA
5gNsa8ySDIMw7/jx1bdRKATzBq+g+2voL/99nn3w7ngTtg9YGx5ieP5FCXN8
W+PFzd/PYZZXYN4/UHsB84n4FC5leWwfiPHde1XbENsHtvB70YWxYvtAZN0P
v8y/9oF1wexzPeHYPlB478NxxqfYPkA93Oqe4Bv09//J0r+X0Mf2AZ+YQ2M1
Itg+8GlU+5DdpnBCH4jqj5x73XoKc/32K4oz8vqYFU6puFyrMsf8YlHr5GO/
i5hp9FMY6rdg+0Dah3f25wfdMBuzZNX2/LUPrLfLkXB8gO0D583b1BczsX1A
5JZpqNQzbB9gyu/MCrmB7QNdQxaDRV7YPpCzvY+L6QKh5z3U8dvu1xYBzDcf
ZSttStLBLCyqelm+wAzzzrg8C4ZUW8xhIfd2dZQ7YV7Kqogu+OqG2XWIgS4+
B/cH0L9TyjXwHe4PQHC/hLNgLe4PwPaULn9cJe4PIPG8K2l9Ke4PgOQVGeP/
HfcHwHn7jcZIAaHnG7Bue6DbqqGC+Y1Jkw7iNcZ81veG3vdJK8y+yk/fWpIc
MavpeyfbKrph3mWlleLwd7yBaTeBVBd2P8xdn0MaxE8GYi4vGmuQxPtBvkFW
hVEjJOHxBeLrShpPtmJ/BrqGxe5pWsph3rBuz+RkEPZnQOeMTjlXnyXm6FKV
KNNwR8yXxhzEj3K4YTZlvdXF8Nd/AUXx9zebZPwwswkOPKqzDcR8XXpxDH0I
xkylxXQ6aRn7K+BquePRLQXi/f4GaxFW92dl6pgPzPgbalFbYr5lPe8/eB3H
K+BGvbD6oYc4PgFf40VZjmnieAQUNNbMNTQFY87f4EzyMsHxLYi7FTTS+UYe
c83CqcqCAxcwByRpHd6zFcevgFUrfLH/Ho5XgXaAYYfONRyfgp117vKflHE8
Cgb3R77lYKLoC+gAAcsEKfUOA8zbnts1VZx0xGwumG5echXHK6HB0NujwzoQ
s9+zxxImzMT6ah/wfOX7rmdlhJnGM8puvQxe/wAj/zfCdO5E/GgYVVyVjbyO
1y+H0YmxV2lr8XrfMLpa/kDFEq9fDiM3XiNpBbx+OYwEJaMebsHrL8Noj9XE
HXq8PjaMOKK1Imdwfvsw2piXebUX57cPo5DJawUX8XrfMFpekwcX8HrfMLrE
NZJjiNf7htGUMLeY9sDf53dQVM9UzQ0DSc5Vbs+bp+Fo0J3a5Iwx9M/6O7LS
vqyxM/knTIfT7Gzy7EMMz1J22334gLrin5JO1nQhG/t4aYH6UeDhWDwU6NqB
PuoJ+VXPLoKT41aRlBMd6GYxmyKn9ktoPvjoU4X4b+T3iT4m0nYIRB32PtsW
1oTiGAdjN7RMg0UC4+N7jE3IZqN816rvX8GIQW3q4ZlqRORPGJDzJ1D+Uf2x
h61FqFNN+uKNpmogrj9Hvh58Kb9/lPz7oEnJJxki55NA0n+fB4jn30l+fiD0
dZL1gR1F/wxZPxzs/wSh22rQoPgHNqHpEfgnPwfeXW8osivshYm4TlaGKcr3
uAPGgOqaW6HUOIdE4By12rucSpxPlHDEr+P+/03qNpwPM/8txPz9x6DrLW5A
1lvA/f8c4n94+KVCws9+BzT8iyHhjx6/9vqV0SHZjW/sGVRyf+e1w+7HfGvP
Wztl6Xox2H227+1ZM+5F8T6G5ad39uy801udt5bB11fA1mOA5R0+2D9wvcOs
IuhoUwU9/zITcj7lfs25Kgv9Yg/aB4S8WhDbMXV/cfG7yJC/++2P7NgQ81Zv
8X5D1PUo+y9A179oQNa/wNfvPIOs37E3hq4XegBZL2TvV1lop7/jvL3goRfJ
Er377e9Az+NHPS/tsf0ByHn8+52g5/fDzu8EAKRnfZg=
    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV00lIlVEchvGjYGCUlpoNCiVuFFKC0lBrY5DlxmigjUFJLaxUilIqh4VW
lC7UArWoiEJo0KgbWAQiaSZoC5U2BhZtEiyCMLCw4efi5TnP+Q989+O7aWVV
eyujQgg/5G5iCFOrQriZFMLWtSHc4Ll4E2vc/VsWQqdzl2THh9CGEfVx9wuy
2Xn36hCKpSU5hC28Xc9znFDvsKNTWtw9cTfq7rdc5YMpITRjpr2N2KP+Vm1e
spzz7SyQS/Zm8yY9vTiifs3Oej5gRwOe4Wel1rlO0u08hw/09+ufkyh3b3g0
Ji23U/8zuyPSw6PwuPoJKZcKOSl1MuMdRfQMOk/b27A4y7sSQhjzjH/i/Dbz
f3EMl+rbtvge1O+ZLXAeMtMnrZ7lvpmn0iGj0i9FZiP6L/hN3XjF/mas5J04
zevxCD+FjfZsxyJejK/Vj+J+fhBreTrm8004aX8yZvFYrDb/fmUIifwz5qkv
4Ab+HavMv8IlfASH1Xvx1zrfDZabr8CP/DxuVL+OX/llLDO/Dyf4YXypvhPf
8DwsNR+Dj3gCrlffiC/4ejxgfnZFCLf5PD5Wn8Y2Po4l5m9hDX+ICepD2MT7
cJf5RjzOW/GOejUe4uVYaD4Hd/BCjFEvxT28BAvMp2Auz8B29TjM5FGYY/6d
bymeT+G8/jlM5V8wm0cwmg/gRfPd+NO30YUZ5o/hB34aZ/W34AxvwDSeuvgN
ef+ZOM6/rfH/kWH5JJMSq/8/uMeQzA==
          "]]},
        "0.42600000000000005`"],
       Annotation[#, 0.42600000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV01toznEcx/GvU3bhNNucpjkkFmYXCIs5hdxQsxRTjheU4wU5RAwjYmTO
NqdcOBXDjGjkLIUhMYeNopxKCDm+dvF53r/P7/P9/P7/5+n3dJgyJ2d2nYhI
8nEyKeJWakQp7m0RUUIDm0bMbBnxulFE/cYRx2TZrSOO40DMlpckRxzhb+oe
xUX8Ybwgr9Er5vtZ5zlnAo2nPnoHzZyz/8xMXWcvNHfAXrm9Knt7+D38decW
4ybvU0iZurnOeGDmD22R9dYpwl06O2mD9VW9jbhWZw111Ruqd0vnJ62WZeoV
4FadIlpufVlvBa7UyadOej31Lul8o9OyevaSZSlUY77CfnfrDKqW37FXg/9S
Iv7SGd/vFT9I3rb2O1APyqb9nntanmY9wEyG+c7WH5pE/JY15MfhZ3pDuz3r
lff5SI+pXquIH5hjvrp5xGbPfoRFzqzAAv42Vsvv6ldRludcwTIqdN5B/RO0
jW7TRRpuvlRvsf4hXOu8dTiL347P+aU4iZ+L+c7pj8P5kXhZPhXH8GNxCd8R
+/GZWOn8FOzOJ+B8/YeJEc1rfzvsK/+F7fhPOFv/PDbgb+A1+XH80SZiH07X
n4kv+EXYTb4F3/MFOFk/B+/zE7FcPgyv8n0xr/aO4xE+EdPk3fAsn4a5+u+a
uYv8dzwqf46b+Hs4qvbO4gL+MCbKr+BKvgxH6OfjDL4QS+TzcQI/HQfr98Ih
/GCsL8/D0fwozNJvg735LrhZ3hjT+cBe+nfcyyb8E/xu/gum8m8wgy/FOnwF
rtI/hF/9B3ZgF/1p+JSfh+/Mr8e3/DJsz6e6N6f8/ul4j//g7v2ka/SSKinB
/H/qpaTs
          "]]},
        "0.35500000000000004`"],
       Annotation[#, 0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwN0nloz3Ecx/EPc0QbGxs2IYkxS6uxI8dGm1az1RzF1hwbURh/OOcotESZ
K0eOsKYcy1XI0XJFch85x8hRrhI5Qnj88fw9v6/P+/N+f77fX5/uZbNGVjQJ
IST5eRsZQouoEPZ0CGE3KjqGMBOvrTe3fqh9CNnxIRzm3bEh7MIgudyeyZiE
gW1DKON99kzgTLmUM3g811q/0jmEvbzVGVswQG2s2kPnbJQ3YD1SrI+xft/6
X1RbW4MR1vJxw9pvzPMea8074l2uy5vlFM9Z9mRjEIZgMJKtp3E6+qOnMwZw
pf48tXo+4IyDqEMfsxahEpFyg3qh56U45D95Jd/wPa/5DHpbP29Oa7Uaz634
nHxaLcdZk+NCmILFasU8DuV4qj6fK/CtjXfhIpTZl8VpqPVtcZ1C6IpIpCAR
JfaP5lJkIh/DsNPZ0fricdf8ZP6DHvig57fZLTyP5c94g23OeOn/+IgHaGrO
Dy6yv7FdCOt8633eaHY9V8lXuVH9pv4nyHTWBT6OavNq9B/GJlzFWeTaf1Tf
Qv21vNK8VTxD3swN8mKeIM/iZeYM5Fw5j8+pl/FIeQxXyt05Q+7Hd8yP5b5y
S56j/15MCDHyC05X/8Vd5U88U/8pbiZf5kvqdfwjwT3nqfqn8zN5ASepb+D3
chVP1F/Et+XxfFI9hy/K6VysP4L3y9HcRT2JT8hdeJT+d9Eh7JC/8wH1Bl4r
3+IC/dt5rryPo9Uv8HL5OA/Xv4ynydW8U30Ol8hTOVt/Kg+VszlCvZgL5QLO
1B/P/eVevE49khPlf+5tqv5rHCU/4u/2f+EE+Q0ny0c5yPW8Qn8tf3X3t3Av
/eX8WJ7N7+xfzW/lJdxNTnBvjvn/E/mW/MHd+4lLeI47aGn/fxnGqco=
          "]]},
        "0.28400000000000003`"],
       Annotation[#, 0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03tsjXccx/EfjUtSHJeaFpVZoonLUo6sSmKuVRaX4xJUXBth02k3xSYb
KenG/qCZIEo3kRNBbC7RFhNp2kaChNlEXIs/CEq2TLTC3F7nj/d5P5/v9/f9
Pb/n5Hl65xdNK2wRQhjv53iXEL7uFsKoSAhFPJK/4tFpaljmugBL8QU+x3D1
u11DWMmrUIzeKSE0qN3GvXYhJLUP4ZcPQqjAMHsuNNegvlvehXJkqeepX1ev
tsevzhJVm6k2iGfxdrUYD5An8RRMRtT6XB6PsRiHnMSc+kVnKOKIMxVyR76g
dh717vUCP7h/KYaayUatWjOq3K/Rusc4on8Yv6NGfULiuXk9D7Rnuvop/IE3
1p/kE6hGpn6JdeOcO+462f9RJ58xn6NWZn1b12v12nBInEdtgTXr+Dfrt6R6
fmxEHBV43QGdQ9jTI4QmPuTMD3mn/A/n66+xrgSLsQIF2Ge/2fbNx033W83L
8dz6XI5hkbN8yp8gbt8Uc+lIRiYyMMf66TwX2fgMoxJns3/EXCr+sn9//h8f
odHMK3u3cj2L/8V9lLvHPf/9E1xFC/s0c8z6O56lzDNd4a32PsOl8jm+o3/R
/A1ku1ctV2Kz/faaP4xtOIfTiffD+qPmvjUf5x/tt4kL5O18S/6O58uFXGKf
YTxWzuUa/UU8VZ7Ba+QPeYj8MV+2fxfuJ7fmYvN/d/LeyXc5S/8lp8tP+Uvz
JzlJPsv1+oe4ubtvgJeYX8a35W+4r/7P/Fgu5QXmY/ynPI+r9cdwnZzFeeZb
8gE5wj31+3KV3JOnmX/U0bcoN/FB/Vu8Rb7EE83v4pXyfo7o1/J6uZJzzJfw
UnkzV+gX8xx5CY8wH+WR8ghuqZ/Hk+SJnG0+lQfLfbhMP5kz5Le+jaj5C9xO
vsZN1v/HafJ97i8f5XeJ74k3mI/zM3kH9zGfz9flIn5k/U/8QP6ee8lp3ptj
/v8MviQ3evdeoB4NuIzW1r8HBty+dw==
          "]]},
        "0.21300000000000002`"],
       Annotation[#, 0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1HtsTnccx/FfFyRjq9voQ93XR1CW2GaNu6hIti4pJbTZJkpqmZlLF9UR
KWncaYiaIWR5IiwVUUGW0AltEbd2c9nmUpeVGTLZslo2DfE6f7z7Pp/zvZxz
+pzn6T1jXs7cpBDCfH/udQqhEYu6hFCMIizGV6jsGEL8jRAup4aQxnf1/YY3
HR9Xv/9aCHeic/yD3Mf5kW1DKEgJYQTP4uHOD8MtfQu4p5753IsLea9r3FC7
bccxuYfze5zrzkejOeevqV/j8HoIWzqHUI537N+q75R7+5qv6LmKmLkj5q7q
vyTX8Tr9a7EG6eZW6682t4rr9NSjvbmp6rnIw3nnas1OcdwVXRBDX/MD7E9H
fyyx4wOu4Ur1gxhk1zn7q52bpNbAEzgHEzEZQ/WMteu040xO19Mcs5tP4gQq
5Ff5mJ1VqPX8peZquGd0HbXhXBPVsNM9xznBmfq2cTna6V/is1iGIqzFCixM
dl+ufd49ZHGl502yc7d7b8Wfqq8xX4YncilP07uU99m5wf2VYyUS2IFmM80d
QthlRxNX2Pk7b5X/5Hz1Yn0lmIkFmI3d9k21Nz/6rF1rIc/BP/rHczamR+8X
v4uEvR3NdUNrvIU48vTn8EfIwPsYE92b/cnmUvCj/QP4GXrjoZn/7W7heAo/
QSO+cY3b/k+PcAXBnqecrb/Bs5R5pku8ye4qLpXPcIP6BfO/IiP6LPkQ1tv3
rfn92IwzOIpM/QfMLTKf4BX2reLZcjlflxfzJ/IXXGLPUM6Ux/Nx9ek8QZ7E
xdH7we/JA7ne/g7cX27JheZ/ah9CW/kWD1H/j7vJj/lz89/zK3ItV6tX8NOu
3jEuMP8Z35CLuJ/6Rv5DLuVp5rO5Tv6Yj6iP5ZPyEM6N3jPeKydzqno/Piyn
8kTzD9qFsF1u4u/Ur/MG+SJnmd/GX8p7OFn9BC+TD/E48yU8S17PO9QLOU8u
4FHmB/NoeRQnqefyh3IWZ5hP4bflNC5Tb81x+bnvyWDzZ7mN/DM36f+LY3Jj
9PsgH+AXvitVvNx8gv+Wt3Ca+Xz+RZ7LD/Sv5nvR7wl3l2PR74n/f5wvyg+9
e/+iGjdRj5b6XwIme9eO
          "]]},
        "0.14200000000000002`"],
       Annotation[#, 0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1HtoVnUcx/HfahbKHtc23XSPiOLAmQlKa4lp4mQmC6ZblqMSh2g0L3nB
S0lsMbyiQ2miJsZ8kLQtdTbNEAkzcYrNuaXOS6ZieCkUrSmWFPo6f7x9n8/v
9/1+z/k9nrP+0+aWfpQUQljsn20ZISTFQhjeO4SNmSHkpYZQZ+17+VJKCOd7
hnABfXqE8Es8hDh3yL35mpzNQ/SUZIXwFkrRoe+MmjZuMidLzXozM/lb+bT1
dvurrfW01m5OD26zls5X5QzONXeMeQO5gE/qa1HTzLvM6a6m2owYfyMfs37C
/k/cIKdYPx7V4z6Oq01zxhP8lxx3XamuCvtdf4e9fovnuZFXWR9hxkp+jZ+1
/gySMQ5voBAj7Y3nF/k5HsxnneElLvPsudaedHcODvJQHoaZ9kvkCm7A15in
brK1VvecxBmeI8Xz7jQvlT+2f6BXCAexG/uwN7Iz7+B65Jk9Aq9ijvqtzprj
OsEF7rGZa5Fq9lK/axUWYTWWYaGeCe590jMUcaPZwb23e4Yu/IH9VfprcFeu
5ilqP+UGM9d6hlosRwJb8FjP4/QQvjSjk+vNvMEb5Ttcbn+JukpMwzxUYLt5
75hbHr2H7rWQZ+Fv9YVcjKnONJJfRsLcdH1xdMUQ5KBMfQm/i3yMx+jo2cyP
6cvEafMH8b/oh9t6/on+312/zXdxHZvc44rf6Q+cwRM8QLH6y85S40ztvN7s
Q1wtN/Nl+z/rP4989zrMTVhjXp3+XfgczTiIAvV79C3Wn+Bl5q3gCrmWL8qf
8PvybK40ZzgXyIX8g/2pPEEu5SVyX35FHsyt5qdxrpzM8/W3pfmu5N84z/4j
jst/8kz9BzhJPspH7Nfzg2zvGE/X/yFfkhfxQPvr+JZczVP0F/Mp+T3eb38M
/yjn8eToPeMdcoyzo2+G98nZPFH/zRdC+ELu5J32L/JauYWL9G/mBfJXHLN/
mKvkJh6rv5JnyGt4i/35XCZP51H6h/Lr8igO0bfIb8pFnK8/k4fJA7jGflfO
kf+Lvuvo7w93k89xp/p7nCVf50HyHv7ft3KIP9Of4PvyBh6gv5w75Dl8U/1K
/l1eyn3kLO9No98/h1vk297hhziCX9GKZPVPARwU1AQ=
          "]]},
        "0.07100000000000001`"],
       Annotation[#, 0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1G1M1WUYx/G77IVNJB4EBTKTo0CmIfKgTW3S3Bzp0dTEaQxm5iJ1YcMs
57Ahak5ZLZqZy7kzh4pLcFhkkU9rUVOjWtPU1Bc4TV0OFzbL6bLPefHle373
dV/X/3+f3Yfhr9TMeeOhEEKVP0cSQigYHMI4PIOxyMdh66N5DJ7C0xiFb6zn
ch5GIAcj0Wk9myMYhuF4El9bH8pPIBOPIwtfWR/CGUjDYKTjkPVUHoQkpCAZ
X1rvSg3hVFYI3/N36E0L4RamDwohivNqM7jD3mL+TS7hIkzAWXk8tw90Xj4j
F3IBNmSEsBHrsB4N+MKcZrXdqMY+7EVNYgjlj4XQbc9cTjFvgPfZY14iv63e
McQZsR8HcSBu83ZzCwrTQ3gWJVhu/6fOEfE5xqWesY2bkGj2audfi5XYhAbU
6pnp2Se8Qxm3mf0gJYRd3uER77BE/T39jbgp13OFvWt4n5lbvEMT1iOG7bir
564ZO8zo4xYzr/BW+U+uUl9lXx0WoQbV2GXePHOrcNazankp/rJ/KkdR6UwT
eRxi5ibry0R/jEYE8+1/kRegGNPwXPzdzE/Ql4afzM/jfzEM1/T8Y/bDPr/E
N9GDjz3jku/pOn7Ff7iNqP0XnKXRmX7hD8zu5Hq5iy+on4zfHRR71tH4fcFm
83bq/wwfoit+h1Fqf6u+t/THuMG8DVwtN/E5+R1+WV7GdeaM51J5Kh9Wr+SZ
8mxeJQ/lInkUd5ufxLlyP16h/+fkEAbKF7lQ/Q5nyjf4df0dHORv+bh6C9/O
dMd4sf7X+Ly8knPU3+c/5Hqu0B/lH+WF/Ln6FD4mF3J5/J4lucdygpyhnsMH
5Qyepf+q+idyH+9RP8db5FNcpn8bvyk3c4L6UV4rt/Pz+uv4VXkzb1dfwfPl
xTxJfz5PlifxA/vL+QW5jIvlNB4rZ3Oj/v4cke/5neTr/4EflU9zn/29nC73
cJ7cyvf9Vjr5Xf0xviV/xNn6q/iMvJyv2r+RL8urOUtOd2/afP+R+P8r+Zo7
/DeO43d0o5/9/wOX+sQf
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1HtsjmcYx/Fb90dRVYd2elg2beIUiUWVLGPRRViXFAtaxzaUqLG01c4h
U9IWW9rGprIhTN6Y0ToGkYgtNBIzs44Iw9gfFmbLOs1KRjMZPs8f337v33Pd
1/Xc79PnfTNLyqeVdQshLPEnrlcIf78cwgO0IzkphIL+IcxAu1p5cggzrb/J
CKGQG+Ueru9KC6FIzuBKfV14gnS5Q707vrSeZ0+amSW8AMv1V2CR9WkzF/Jn
cn/799j/gZzFq83K0Bc/IIRMeSA67emHr6yX2fea+nJuNaeCH6mlYl90JnmL
uU1Id63FtZWuVclDo/nWQ3gVT+J30OKe+zHfegG+sz6P7zFRXx1qkSQP5jX2
VKNGrkWpa4uxD2W9PUfna1Ofxn0TQ+jpXnudNZFXqp9IDeEkDuIYjkR21q+5
GdlmvoHRWGb/jhTPxjrGue6xlZuQaPZqz2ktqvAJ6lCpZ7J7X3CGPD5s9rN+
Iex2hpecYZH6x/ob0S7X8Fx7P+IWMxucoQnrEcN2dOnpMmOnGZ3cbOZd/lz+
i4vVV9hXjfkoQyl2mzfD3GJcd69Kfh//2D+B81HkM73JIxEzt4++NMRjOLJQ
aP9UnoUcTMJb0dnMT9CXjJ+i/y0/wau4r+ex2d2sp3M77uAL9/jVc/oDV/A/
HiLf/ls+S6PPdJk/NfsU18jn+Jb6D/p/Rk70HvNR1EffC/0HsBnncBK59h/S
96H+GNeZt4FL5Sa+Ia/iOfJSrjZnDOfKE/hb9SKeLL/HK+RXeJQ8jNvMT+LB
chyX67/UN4Re8m3OVv+X0+Q/eYn+E/w8PYSz3KrezA/lHVyifzHflKt4kPom
/l2u4bn68/lHeTYfVx/PZ+RsLojesz7eYzlBTlUfxMfkVJ6i/576NrmT96rf
4Ab5Iufp38oV8h5OUD/Na+Wj/Lb+al4o1/N29XIulEt4rP4RPE4ey8/sL+B3
5TzOkZP5dTmTG/XHc5b8n+/JCP3nubt8lTvtf8Ap8h0eIh/ip74rp3id/hh3
yFs4U38xX4t+p/ie/Rv5t+h3h9PlFO/NYf+PLL4o3/cOP0IrfkEb4ux/AXdn
zdc=
          "]]},
        RowBox[{"-", "0.07100000000000001`"}]],
       Annotation[#, -0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV021MlWUcx/GLXiiBgQoKxCxPomtuuIj1InPBfBjNQ7lQlyGM1DDBeZjK
cA4Lnwckiha4XDtz1nRNN20tp02dxlyp1Vqp5cOmy9BkOQ+NdLB0fc6LL9/7
d/0fzn0f7hNZ3FARSwkh1PrTnx7C5uwQElwxNoQ3kZkXQgaWOy/NCuHr/BBK
eIu8FTNdH3U2g/8aEcKA2V36Z8mpmSFcs+NJvs7ZzrPQam5QX5fr2fpWyDEM
Oet2FnWWaWas6xyMwRxnffY/0nPXrj7E0IC5asfdQwW32dOOha7zzFXyPXPD
8Vhvrr1NvAbvqJ0wV8MdZraj3vWz5uo4YWYUMnJCeMbcB2ZaUKA+ERPQoK/T
3HPqI9WWuO5w/m7yPngbPsR2nB0TQr59tWrpetOwLCOE9XId96rfxh3MdBZV
P4NvkYp6PRGOq5eod3EnRjzledzjOqzCVmzASv3l7us7n13Gh5Lf3+gQ9nrm
J9z3EvUt5tvRJ7dwpd61fMDOtlz7sRFx7MZDMw/t2GNHgvfb+Qd/JN/lavVG
fc2owQosxV775tpbjUs+ayUvw3390zmKKs/0Mr+AuL2Z5nIxDJMRwXz9b/Bb
KMYsTEvem/1p5rLwg/2T+AHGodfMv3YH1xXchxv42Gdc8z3dxs/4D/2I6r/i
Wdo900/cYfcxbpF7+Ir69+Yvojj5DvFhtNr3qfkvsAM9OIoS/QfNrTYf5w32
beKlcidflpv4bbmOm+15iUvk6fyNehWXy3O4Uc7nF+Xn+YL9GTxRTuGY+R9H
ecfkq1ykPsC58h1+z/xX/PjpEE7zKfX93C9/wouS7yj/Jq/iAvVt/KfcwpXm
o3xeXsBfqr/KJ+Uinpd8z0aG8JmcJueoF/AROYdfN39LvVtO8Ofql7lNPsdl
5ru4Qd7HaeoneJ18mEvNN/NiuZV3q8d4vryIp5ov5FfkqfxI/zx+TS7jYjmL
p8jjud38MI7Ig34nhebP8nD5F07o/5uz5Rs8ST7IQ34rx/h983G+J+/k8ear
+Ve5nm/p38w35TWcJ2d7bw75f0T4nNzrHf4Hp/A7LiBF//+C98sS
          "]]},
        RowBox[{"-", "0.14200000000000002`"}]],
       Annotation[#, -0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV03toz3scx/G3P6bGNmzH77fjkgnpOITlD5c2KYeaS7mTdYzMpdxFmksu
f4yOjhPnuKQlNDSF1EknpKTczsl1jD/ILkzk52DLZeKxP557fl+f9+X7/e73
+/Wct3zysjYRUeBPqn3E7UREWoeIW9yW73DmjxEZuOf6Pu7iAarRyXn5DxFN
Zve4HpsTsUR+mRHR7KxOTz3moARz8dF5A2fZX8oLsBAtzvfZMdGObLUXznK4
kSc5+8AJ+T03owm5+j/zF3zC19Y96O58p+fo4DkOu15kfpWckjsiI+m9sEHv
RrxzlkQ/1z+jrmtELRKukxiJX+y5YE+l62M4jgZ5mvMCu2v0F/JzZzOdNfJ0
fsEz+CXP4nWdzWdGbM+N2IUtqMBeNGchO+KAXSmu9EzPeLfcyMXqq/WV4Vcs
QSkO2TfZ3mLcd58VvBBv9I/iIsz2bEN5ICrszTKXRBp+Qh6m6p/A05GP0RjR
+mz2p5vLxk37+3ATuqHezAe7v2FS62ePJ9jjHo/9jxtwC1/wFkX6a7zLDu/0
H++0+xxvki9zjfpV8/eQ717n+RTK7Tto/gR+x2X8jUL9VeZWma/gzfZt5VJ5
F1fLa3imvIjL7BnChfIo/kd9No+TJ/JquQsPlvvyDfszubccvNT8v50i2smP
eJD6e07Kz3mB+bP8tUvEJb6oXslv5f1cYn4+P5BXci/137hO3sSzzBfxdXkG
n1Ev4AvyIJ5ivqVjxBE5XU6o9+LTcoLHm69V/0tO8VH1at4uX+Mx5v/kZfJh
Tlc/z+vlUzzSfBnPlct5r/pSniqX8DDz/Xm4PIxb9E/hsfIYzpezeYDcg3eY
T+M8+aPfb3/zV7itfIdT+l9xjvyE+8hV/Mlv5RxvMF/Br+U/uIf5Yr4rL+Za
/dv4qbyWc+Uc35uTPo88vibX+w7/j4t4iBsI/d8BkFi+/g==
          "]]},
        RowBox[{"-", "0.21300000000000002`"}]],
       Annotation[#, -0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0nloz3Ecx/H3GH5sw8bm3hzLEWH5wxFSTOZuckSO3OUWfyCJPxAhx1Ja
cuYopOSOpNzkHorMnWMYRSY8/PHcc6/38fl+vtu32cQ5hbOTIiLXj5QGETez
Im5hLMbhbWpEWUrE3syIb7zRTHmdiP3yPtSWH5l7jBJUqxXxhPuaKeUU+Tmn
8gvup36iUUQBp6lNVJuEyahw/htOV5/BMzELSe7wlcsxF/PwUa0qTrjDSVSq
GzHNXZJ4Op9Ra8yV60UkoxKqogqy1atzDSTQxPNSeLY7dXf2E/drqlZLrRm3
UVuo19teH+ySd/9/17SIPZzg8+qtPPuUucF+H4IBGISB2G5mjDvd05/DU/Gp
ZkQvLsBou525PYq9V1r9iCwkozVyMMz8QB6OPPRGNxQ5O2EvHdecn8vf0Qgv
7Xxz9h8MxTs8xSbPeOz9XuEWfuEzCsyXZESs9je4wWudfZyXyhe4RP+S/bvI
86zTfAgrnbfN/j6swwUcQw/zB+zNs1/My5y3nCfL6/m+vIBHytN4kXM6cQ+5
F5/UH8395UE8X27AHeWWfNX5qdxC/pvum7F/navLj7iDfjlnya95iv2j/Lth
xDk+q7+HP8tbebz9SfxAnsvN9dfwC3kpj7JfwFfkEXxEvzufkTtwof2K2hE7
5YScqd+cD8uZPMB+qf4WuYx36d/nVfJlzre/mWfJOzihf5oXy4e4p/1FPEFe
yUX6M3mYPJ672G/LXeUuXGG+kPvK+Zwnp3M7OZtX20/mHPmHb72t/YtcRb7N
Zebfc4b8lHPlA/zTN32cl9gv5g/yBs62P4bvyNO51PwKfiYv5Hpyhu/moP9H
Dl+WX/qGv+AsHuIq/uIf+7ejYg==
          "]]},
        RowBox[{"-", "0.28400000000000003`"}]],
       Annotation[#, -0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1HlMz3Ecx/GP0UJKkcrVxPzDhvmHzTWEshzZHGFsyW2OlY2Y8QfG3Mxm
/YExE7lmoZr7LHPN+YcjsRkqxOSY5tEfz56/1+fzfr1/v+q7X0r2kgmLm4QQ
kv2ojgphe/sQBrcNoVnrEKYmhDAN01Hj7nC7EOp4h5kvZobiiLOvfL9jCN94
GGa7/87ZnINZKDTXhl/b9Qav0NJ7/DC3x3m9vcfMHMcvZw/sq+dTcgZfkMdw
rM483flYgNAqhDNmIrjAngYz5+RiTPG6TC+LE/VyzedhOVqZLzUTwwf0IuND
uCxfwhzzl/Tmcie9lebzsQoJ5q+auYYonWHOUjEc153dwGdUN+52Fo0L9tTI
XbxOx2ik6WbwWIxBBFrgbUwIFVyOUWY2+WwjeXxjj8c17uA6+8rt/c4dUKVX
5/xf4ww+4CV2+rwvEkN4h3v4jVqkmX/WJoSNfs+7vDna34xXy1f4mfub+o/Q
x/uUcBHW27dP/wi24ArOYqD5o3pL9Qt4jX1reZa8lR/LuTxJnsMr7OnLA+Uh
fN59FqfLGbxMTuLecne+Y38Ud5Ub4kJYqF/BzeXn3Mv9N24nv+cc/dP8t0MI
F7nM/SGulffyDP1sfiIv4RT3m/itvJon66fxbXkin3Q/gEvlXpyp/yfWsyNH
yvHuU/iEHM+j9Svd75Zr+KD7x7xBvsWp+rt4kbyfI92X8Eq5iAfpr+CZ8nre
434hT5BncD/9Htxf7sd/zGfyCDmV+8ix3FPuzBv1m3Ky/NMz3UP/GjeTH3CN
+Y8cJ7/kbvJRrvf8FXO+fgF/krdxZ/2p/FCey5Xm1/ErOY8T5DjPTaH/RzLf
kquSfG+gDE9xBw34D4WRq9c=
          "]]},
        RowBox[{"-", "0.35500000000000004`"}]],
       Annotation[#, -0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0k1MzwEcx/FvSigPUUnJVma2TJoLWx42Y8NcQhmH2rAcmJhsMTMOMRsO
mC0dYqZlKw/DqCaxSVww4iJP48CWcGArJS+Hz96/9/fpt/r/8rfsWleVEBE/
ZcrUiHRpyYz4lR6xLTviN17lV+RvakSd2oBaiQzKOT6ifl0/cXzEd/s/pF8y
J0UMmXk7PeIP3jBzU4Y919tLMn+Lp2DD/zvqFRIZEed5qnqb/gSMrIgEGXE3
x90kM+/cTcS7Zkbjc56MlW50qGXYu+TOWLWdauOwkWeq39fPxiaeol6tP9vz
XlzvHaVSJrf5Z3fv4BK+TIonRnTifbknvfodODQtYq65N/wx7zH30M15aq88
P/eeIs+t2Cy13l/nb2qU49IpN2SR2aYpEbvcqceDEyIO4RZ+Al/wPVjGK7HG
nfm4iC/F2/obcRVfg7t5Fs7js7Db/RTM58OTI7bbf4Jj+Gss1P+BGfwTbrV/
DQdz/K+xXf8ifuNnsdz+ZnzJqzBP/xh+4Adwg/2V+IiX4hX9YmzjhVhifyDN
b86Tebp+HrbwdFxt/73+ad6HF/Rf4BHehcvtn8IdvAGT9VtxH2/GxfZrsILX
4hn97biWl+MC+wW4kC/AAfMluIIvxyI+CefwXDxqfxTO4L98iwX2H2Aif4p9
5r9gGn+DM3kT/vYt3ML99uvxKz+JufY34TO+Dd+bP4y9vBozeZrv5rLfYwZ2
8Y++tX5plx7plmH5BzYYjgk=
          "]]},
        RowBox[{"-", "0.42600000000000005`"}]],
       Annotation[#, -0.42600000000000005`, "Tooltip"]& ], {}, {}}}], 
   {GrayLevel[0], AbsoluteThickness[3], LineBox[{{0, 0}, {1., 0}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}, {1.75, 
      FormBox["1.75`", TraditionalForm]}, {2., 
      FormBox["2.`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.1536749816983884`, 
   2.1536749816983884`}, {-1.3536749816983884`, 1.3536749816983884`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 419},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[40]=",
 CellID->2871532]
}, Open  ]],

Cell[TextData[{
 "Note that there is a discontinuity across the panel for the velocity \
component normal to the panel.  Also note that the streamline along the \
negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " axis is thickened compared to the others.  This is suggestive of a branch \
cut."
}], "Text",
 CellID->738393833]
}, Open  ]],

Cell[CellGroupData[{

Cell["Constant-strength vortex panel", "Subsection",
 CellID->671459404],

Cell[TextData[{
 "Set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Delta]", "TR"], "=", "1"}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["a", "TI"], "\[Gamma]"], "=", "1"}]], "InlineFormula"],
 " to provide unit integrated panel strength, ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     StyleBox["\[Delta]", "TR"]], 
    RowBox[{
     StyleBox["\[Gamma]", "TR"], 
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]}]}], "=", "1"}]], "InlineFormula"],
 "."
}], "Text",
 CellID->159660156],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u\[Gamma]Plot", ",", "w\[Gamma]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u\[Gamma]", ",", "w\[Gamma]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a\[Gamma]", "\[Rule]", "1."}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1."}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]\[Gamma]Plot", ",", "\[Psi]\[Gamma]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]\[Gamma]", ",", "\[Psi]\[Gamma]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a\[Gamma]", "\[Rule]", "1."}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1."}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV\[Gamma]", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"u\[Gamma]Plot", ",", "w\[Gamma]Plot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{
         RowBox[{"0.005", "<", 
          RowBox[{
           RowBox[{"x", "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}], " ", "&&", " ", 
         RowBox[{"0.005", "<", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"x", "-", "1"}], ")"}], "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"18", ",", "18"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", "Medium"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]\[Gamma]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]\[Gamma]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]\[Gamma]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]\[Gamma]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "7"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanel", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
      RowBox[{"AbsoluteThickness", "[", "3", "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Components", "[", 
         RowBox[{"rP", "/.", 
          RowBox[{"\[Delta]", "\[Rule]", "1.0"}]}], "]"}], "]"}], "]"}]}], 
     "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{
  "gV\[Gamma]", ",", "g\[Phi]\[Gamma]", ",", "g\[Psi]\[Gamma]", ",", "gPanel",
    ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.0"}], ",", "2.0", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellOpen->False,
 CellLabel->"In[41]:=",
 CellID->79272867],

Cell[BoxData[
 GraphicsBox[{{{}, 
    {Hue[0.67, 0.6, 0.6], 
     {Arrowheads[{{0.009116798619914601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9838697719654657, -1.2192661282485955`}, \
{-1.0161302280345343`, -1.1807338717514042`}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9842819183200495, -1.0800036144948315`}, \
{-1.0157180816799505`, -1.037643444328698}}]}}, 
     {Arrowheads[{{0.010032604380582786`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9849907854779197, -0.9408700526751171}, \
{-1.0150092145220804`, -0.8944240649719417}}]}}, 
     {Arrowheads[{{0.01049453592032024, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9860676623265575, -0.8018182182871996}, \
{-1.0139323376734424`, -0.7511229581833889}}]}}, 
     {Arrowheads[{{0.010939521084517059`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9875817038932192, -0.6627686487870799}, \
{-1.012418296106781, -0.607819586507038}}]}}, 
     {Arrowheads[{{0.011346123674529853`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9895846990804944, -0.5236035400388053}, \
{-1.0104153009195056`, -0.4646317540788422}}]}}, 
     {Arrowheads[{{0.011688143720156731`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9920886178369893, -0.3841685818385412}, \
{-1.0079113821630108`, -0.32171377110263566`}}]}}, 
     {Arrowheads[{{0.011937480299842701`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9950407430559669, -0.24428998699210255`}, \
{-1.004959256944033, -0.1792394247726037}}]}}, 
     {Arrowheads[{{0.012069481079506499`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9983087374695213, -0.10381021197286217`}, \
{-1.0016912625304788`, -0.037366258615373515`}}]}}, 
     {Arrowheads[{{0.012069481079506495`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0016912625304786`, 
        0.03736625861537313}, {-0.9983087374695214, 0.10381021197286178`}}]}}, 
     {Arrowheads[{{0.011937480299842703`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.004959256944033, 
        0.17923942477260332`}, {-0.9950407430559668, 
        0.24428998699210216`}}]}}, 
     {Arrowheads[{{0.011688143720156703`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0079113821630106`, 
        0.3217137711026353}, {-0.9920886178369893, 0.38416858183854075`}}]}}, 
     {Arrowheads[{{0.011346123674529873`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0104153009195056`, 
        0.4646317540788418}, {-0.9895846990804944, 0.5236035400388049}}]}}, 
     {Arrowheads[{{0.010939521084517017`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0124182961067807`, 
        0.6078195865070377}, {-0.9875817038932193, 0.6627686487870794}}]}}, 
     {Arrowheads[{{0.010494535920320257`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0139323376734426`, 
        0.7511229581833885}, {-0.9860676623265575, 0.8018182182871991}}]}}, 
     {Arrowheads[{{0.010032604380582764`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0150092145220801`, 
        0.8944240649719414}, {-0.9849907854779197, 0.9408700526751168}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0157180816799505`, 
        1.0376434443286975`}, {-0.9842819183200495, 1.080003614494831}}]}}, 
     {Arrowheads[{{0.009116798619914589, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0161302280345343`, 
        1.180733871751404}, {-0.9838697719654658, 1.2192661282485953`}}]}}, 
     {Arrowheads[{{0.009776473983450114, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.804915554827524, -1.2194824863382105`}, \
{-0.8421432687018878, -1.1805175136617894`}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8050900680510346, -1.0805623587736346`}, \
{-0.8419687554783771, -1.0370847000498948`}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8055952571169903, -0.941877695175277}, \
{-0.8414635664124214, -0.8934164224717821}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8065436107100611, -0.8033940158172144}, \
{-0.8405152128193506, -0.749547160653374}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.808062428429387, -0.6650296413195764}, \
{-0.8389963951000247, -0.6055585939745416}}]}}, 
     {Arrowheads[{{0.012740596380756667`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8102746410205225, -0.5266345889484169}, \
{-0.8367841825088892, -0.46160070516923063`}}]}}, 
     {Arrowheads[{{0.013246569582145842`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8132589946485415, -0.3879760206041924}, \
{-0.8337998288808701, -0.31790633233698445`}}]}}, 
     {Arrowheads[{{0.01362784987194159, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.816989692319574, -0.2487510543761716}, \
{-0.8300691312098377, -0.17477835738853467`}}]}}, 
     {Arrowheads[{{0.013834467068875209`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8212795642619832, -0.1086513209733224}, \
{-0.8257792592674285, -0.032525149614913276`}}]}}, 
     {Arrowheads[{{0.013834467068875209`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8257792592674285, 
        0.03252514961491289}, {-0.8212795642619832, 0.10865132097332202`}}]}}, 
     {Arrowheads[{{0.0136278498719416, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8300691312098377, 
        0.17477835738853426`}, {-0.816989692319574, 0.24875105437617123`}}]}}, 
     {Arrowheads[{{0.013246569582145842`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8337998288808702, 
        0.31790633233698407`}, {-0.8132589946485416, 0.387976020604192}}]}}, 
     {Arrowheads[{{0.012740596380756657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8367841825088892, 
        0.46160070516923024`}, {-0.8102746410205225, 0.5266345889484164}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8389963951000247, 
        0.6055585939745411}, {-0.808062428429387, 0.665029641319576}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8405152128193506, 
        0.7495471606533737}, {-0.8065436107100611, 0.803394015817214}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8414635664124214, 
        0.8934164224717815}, {-0.8055952571169903, 0.9418776951752764}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8419687554783771, 
        1.0370847000498944`}, {-0.8050900680510346, 1.0805623587736342`}}]}}, 
     {Arrowheads[{{0.009776473983450114, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8421432687018878, 
        1.180517513661789}, {-0.804915554827524, 1.21948248633821}}]}}, 
     {Arrowheads[{{0.010477306125975762`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6255947566665606, -1.2193173857476642`}, \
{-0.6685228903922629, -1.180682614252336}}]}}, 
     {Arrowheads[{{0.011183628626149135`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6253888688839033, -1.0807438366226534`}, \
{-0.6687287781749199, -1.036903222200876}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6255130117197387, -0.9425558182023804}, \
{-0.6686046353390847, -0.8927382994446785}}]}}, 
     {Arrowheads[{{0.012768289697852271`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6261280893497236, -0.8047603565670728}, \
{-0.6679895577090996, -0.7481808199035157}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6274465394599497, -0.6673093440013596}, \
{-0.6666711075988737, -0.6032788912927585}}]}}, 
     {Arrowheads[{{0.014475369768919482`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6297234816075017, -0.5300505149717056}, \
{-0.6643941654513217, -0.45818477914594175`}}]}}, 
     {Arrowheads[{{0.015263437675282363`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.633205847216034, -0.3926627923226784}, \
{-0.6609117998427894, -0.3132195606184985}}]}}, 
     {Arrowheads[{{0.015890646781373582`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6380038552628454, -0.25461503207595015`}, \
{-0.656113791795978, -0.1689143796887561}}]}}, 
     {Arrowheads[{{0.01624493513422706, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6438951300916238, -0.11524939248983951`}, \
{-0.6502225169671995, -0.025927078098396172`}}]}}, 
     {Arrowheads[{{0.01624493513422706, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6502225169671995, 
        0.025927078098395784`}, {-0.6438951300916238, 
        0.11524939248983912`}}]}}, 
     {Arrowheads[{{0.015890646781373582`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.656113791795978, 
        0.16891437968875572`}, {-0.6380038552628454, 
        0.25461503207594977`}}]}}, 
     {Arrowheads[{{0.015263437675282363`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6609117998427894, 
        0.31321956061849804`}, {-0.633205847216034, 0.392662792322678}}]}}, 
     {Arrowheads[{{0.014475369768919463`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6643941654513217, 
        0.4581847791459414}, {-0.6297234816075017, 0.5300505149717052}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6666711075988737, 
        0.603278891292758}, {-0.6274465394599497, 0.6673093440013591}}]}}, 
     {Arrowheads[{{0.012768289697852271`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6679895577090997, 
        0.7481808199035153}, {-0.6261280893497237, 0.8047603565670723}}]}}, 
     {Arrowheads[{{0.011949495453044816`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6686046353390847, 
        0.8927382994446781}, {-0.6255130117197387, 0.9425558182023801}}]}}, 
     {Arrowheads[{{0.011183628626149135`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.66872877817492, 
        1.0369032222008756`}, {-0.6253888688839034, 1.080743836622653}}]}}, 
     {Arrowheads[{{0.010477306125975762`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6685228903922629, 
        1.1806826142523352`}, {-0.6255947566665606, 1.2193173857476636`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4459512131949135, -1.2185978856335782`}, \
{-0.49522525739332157`, -1.1814021143664215`}}]}}, 
     {Arrowheads[{{0.012072512361463297`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.44518705599077374`, -1.0803152974673846`}, \
{-0.4959894145974613, -1.0373317613561448`}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4446816671464303, -0.9426034291576108}, \
{-0.49649480344180474`, -0.8926906884894481}}]}}, 
     {Arrowheads[{{0.014144384224217989`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.44463017617339656`, -0.8055550736990104}, \
{-0.4965462944148385, -0.7473861027715781}}]}}, 
     {Arrowheads[{{0.015348449856734201`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4453353714090012, -0.6692318970854733}, \
{-0.49584109917923386`, -0.6013563382086446}}]}}, 
     {Arrowheads[{{0.016637991562921867`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4472494312859861, -0.5335905261025837}, \
{-0.49392703930224896`, -0.4546447680150637}}]}}, 
     {Arrowheads[{{0.017936440162617962`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.45098254528183845`, -0.398322264205265}, \
{-0.4901939253063966, -0.3075600887359119}}]}}, 
     {Arrowheads[{{0.019074142779961955`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4571397340911158, -0.2625861436972979}, \
{-0.48403673649711926`, -0.1609432680674084}}]}}, 
     {Arrowheads[{{0.019772769634296798`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4657378637744819, -0.12486817945258294`}, \
{-0.4754386068137532, -0.016308291135652735`}}]}}, 
     {Arrowheads[{{0.0197727696342968, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.47543860681375316`, 
        0.01630829113565234}, {-0.4657378637744819, 0.12486817945258256`}}]}}, 
     {Arrowheads[{{0.019074142779961962`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.48403673649711926`, 
        0.160943268067408}, {-0.4571397340911158, 0.2625861436972975}}]}}, 
     {Arrowheads[{{0.017936440162617973`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4901939253063966, 
        0.30756008873591145`}, {-0.45098254528183845`, 
        0.3983222642052646}}]}}, 
     {Arrowheads[{{0.016637991562921874`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.49392703930224896`, 
        0.4546447680150633}, {-0.4472494312859861, 0.5335905261025834}}]}}, 
     {Arrowheads[{{0.015348449856734198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.49584109917923386`, 
        0.6013563382086442}, {-0.44533537140900126`, 0.6692318970854728}}]}}, 
     {Arrowheads[{{0.014144384224217989`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4965462944148385, 
        0.7473861027715777}, {-0.44463017617339656`, 0.8055550736990099}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.49649480344180474`, 
        0.8926906884894478}, {-0.4446816671464303, 0.9426034291576104}}]}}, 
     {Arrowheads[{{0.012072512361463297`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4959894145974613, 
        1.0373317613561444`}, {-0.44518705599077374`, 
        1.0803152974673842`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.49522525739332157`, 
        1.1814021143664213`}, {-0.4459512131949135, 1.218597885633578}}]}}, 
     {Arrowheads[{{0.0119128240533511, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2661113500275943, -1.2171367569582179`}, \
{-0.3221239440900525, -1.1828632430417818`}}]}}, 
     {Arrowheads[{{0.012970645227741062`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.26460406742020326`, -1.0789957761908446`}, \
{-0.32363122669744354`, -1.0386512826326846`}}]}}, 
     {Arrowheads[{{0.014198637077553295`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.26317639494206313`, -0.9416063510240228}, \
{-0.3250588991755836, -0.8936877666230362}}]}}, 
     {Arrowheads[{{0.015631617686746018`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2620007739872351, -0.8051868693066164}, \
{-0.3262345201304117, -0.7477543071639721}}]}}, 
     {Arrowheads[{{0.017308930966610743`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2613963465182258, -0.6700092231753749}, \
{-0.32683894759942095`, -0.6005790121187431}}]}}, 
     {Arrowheads[{{0.01926581232382168, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.261959378899017, -0.5363710273499133}, \
{-0.32627591521862975`, -0.45186426676773417`}}]}}, 
     {Arrowheads[{{0.02149716439965878, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2648113893283839, -0.4044345939215798}, \
{-0.32342390478926286`, -0.3014477590195971}}]}}, 
     {Arrowheads[{{0.0238257974448833, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2719231465601457, -0.2735671544867973}, \
{-0.316312147557501, -0.14996225727790896`}}]}}, 
     {Arrowheads[{{0.025567923001979968`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2854165171276217, -0.14051737420347923`}, \
{-0.30281877699002496`, -0.0006590963847564468}}]}}, 
     {Arrowheads[{{0.025567923001979975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3028187769900249, 
        0.0006590963847560305}, {-0.28541651712762184`, 
        0.14051737420347887`}}]}}, 
     {Arrowheads[{{0.023825797444883313`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.316312147557501, 
        0.14996225727790852`}, {-0.27192314656014577`, 
        0.27356715448679697`}}]}}, 
     {Arrowheads[{{0.02149716439965878, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3234239047892628, 
        0.30144775901959664`}, {-0.26481138932838394`, 
        0.40443459392157943`}}]}}, 
     {Arrowheads[{{0.019265812323821702`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.32627591521862975`, 
        0.4518642667677337}, {-0.261959378899017, 0.5363710273499129}}]}}, 
     {Arrowheads[{{0.017308930966610733`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3268389475994209, 
        0.6005790121187427}, {-0.2613963465182258, 0.6700092231753745}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.32623452013041165`, 
        0.7477543071639716}, {-0.2620007739872351, 0.805186869306616}}]}}, 
     {Arrowheads[{{0.01419863707755331, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3250588991755836, 
        0.8936877666230357}, {-0.26317639494206313`, 0.9416063510240225}}]}}, 
     {Arrowheads[{{0.012970645227741094`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.32363122669744354`, 
        1.0386512826326841`}, {-0.2646040674202032, 1.0789957761908444`}}]}}, 
     {Arrowheads[{{0.01191282405335109, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.32212394409005246`, 
        1.1828632430417816`}, {-0.2661113500275943, 1.2171367569582177`}}]}}, 
     {Arrowheads[{{0.012572289032024972`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08630566526264921, -1.214778192421174}, \
{-0.1489884523844093, -1.1852218075788261`}}]}}, 
     {Arrowheads[{{0.013818604383719674`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08392157866566363, -1.076518731992283}, \
{-0.15137253898139488`, -1.0411283268312463`}}]}}, 
     {Arrowheads[{{0.015308797801352584`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08132264964345727, -0.9391129191740502}, \
{-0.15397146800360123`, -0.8961811984730089}}]}}, 
     {Arrowheads[{{0.0171168985608574, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07856410315335052, -0.8028926802244971}, \
{-0.15673001449370796`, -0.7500484962460914}}]}}, 
     {Arrowheads[{{0.01935172228882581, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07579740709511018, -0.668359363429699}, \
{-0.1594967105519483, -0.602228871864419}}]}}, 
     {Arrowheads[{{0.022183664080880648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.0733957252822162, -0.5363081116974457}, \
{-0.16189839236484227`, -0.4519271824202017}}]}}, 
     {Arrowheads[{{0.025900636613884837`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07232003315022958, -0.40808947503269793`}, \
{-0.1629740844968289, -0.2977928779084789}}]}}, 
     {Arrowheads[{{0.03099785349711697, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07552662898693756, -0.2860939099280415}, \
{-0.15976748866012092`, -0.13743550183666475`}}]}}, 
     {Arrowheads[{{0.0375437837250814, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09470663716635122, -0.1714886175261544}, \
{-0.14058748048070727`, 0.030312146937918727`}}]}}, 
     {Arrowheads[{{0.0375437837250814, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.14058748048070718`, -0.030312146937919157`}, \
{-0.0947066371663513, 0.17148861752615402`}}]}}, 
     {Arrowheads[{{0.030997853497116982`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.1597674886601209, 
        0.1374355018366643}, {-0.0755266289869376, 0.2860939099280412}}]}}, 
     {Arrowheads[{{0.02590063661388485, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.1629740844968289, 
        0.29779287790847847`}, {-0.07232003315022958, 
        0.40808947503269755`}}]}}, 
     {Arrowheads[{{0.022183664080880666`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.1618983923648423, 
        0.45192718242020125`}, {-0.07339572528221619, 0.5363081116974454}}]}}, 
     {Arrowheads[{{0.019351722288825812`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.1594967105519483, 
        0.6022288718644185}, {-0.07579740709511017, 0.6683593634296986}}]}}, 
     {Arrowheads[{{0.01711689856085742, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.156730014493708, 
        0.7500484962460909}, {-0.0785641031533505, 0.8028926802244968}}]}}, 
     {Arrowheads[{{0.015308797801352593`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.15397146800360123`, 
        0.8961811984730084}, {-0.08132264964345727, 0.9391129191740497}}]}}, 
     {Arrowheads[{{0.013818604383719669`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.15137253898139486`, 
        1.041128326831246}, {-0.08392157866566362, 1.0765187319922827`}}]}}, 
     {Arrowheads[{{0.012572289032024979`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.1489884523844093, 
        1.1852218075788254`}, {-0.08630566526264918, 1.2147781924211734`}}]}}, 
     {Arrowheads[{{0.013127617905417648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.09314116969478414, -1.211462549941464}, {
        0.024505889128745628`, -1.1885374500585357`}}]}}, 
     {Arrowheads[{{0.01454299001946831, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.09641191187742296, -1.0727412918205796`}, {
        0.021235146946106814`, -1.04490576700295}}]}}, 
     {Arrowheads[{{0.016272107256669013`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10025320680570056`, -0.9348203359521523}, {
        0.01739385201782921, -0.9004737816949067}}]}}, 
     {Arrowheads[{{0.01842700829757586, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10479658034701135`, -0.7980532651231155}, {
        0.01285047847651842, -0.754887911347473}}]}}, 
     {Arrowheads[{{0.021183581531526007`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.11021595329810921`, -0.662999352729812}, {
        0.007431105525420562, -0.6075888825643061}}]}}, 
     {Arrowheads[{{0.024844937579490488`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.11677415102885275`, -0.530595779955316}, {
        0.0008729077946770247, -0.4576395141623314}}]}}, 
     {Arrowheads[{{0.03001748678854118, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{
        0.12500822729006422`, -0.4025814873780152}, {-0.007361168466534437, \
-0.30330086556316166`}}]}}, 
     {Arrowheads[{{0.038257143909291476`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{
        0.13669862743101757`, -0.2828517835955444}, {-0.0190515686074878, \
-0.14067762816916188`}}]}}, 
     {Arrowheads[{{0.05575762470829351, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{
        0.16410369856303403`, -0.18253503508526903`}, {-0.04645663973950426, 
        0.04135856449703336}}]}}, 
     {Arrowheads[{{0.055757624708293596`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.04645663973950441, -0.04135856449703393}, {
        0.16410369856303414`, 0.18253503508526883`}}]}}, 
     {Arrowheads[{{0.03825714390929151, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.019051568607487843`, 0.1406776281691614}, {
        0.13669862743101763`, 0.2828517835955441}}]}}, 
     {Arrowheads[{{0.030017486788541194`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.007361168466534451, 0.3033008655631612}, {
        0.12500822729006422`, 0.4025814873780148}}]}}, 
     {Arrowheads[{{0.0248449375794905, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.0008729077946770109, 0.45763951416233095`}, {
        0.11677415102885276`, 0.5305957799553157}}]}}, 
     {Arrowheads[{{0.02118358153152602, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.007431105525420548, 0.6075888825643055}, {
        0.11021595329810922`, 0.6629993527298116}}]}}, 
     {Arrowheads[{{0.018427008297575858`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.012850478476518427`, 0.7548879113474726}, {
        0.10479658034701134`, 0.798053265123115}}]}}, 
     {Arrowheads[{{0.016272107256669017`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.01739385201782919, 0.9004737816949062}, {
        0.10025320680570057`, 0.9348203359521519}}]}}, 
     {Arrowheads[{{0.014542990019468334`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.021235146946106787`, 1.0449057670029493`}, {
        0.09641191187742298, 1.0727412918205792`}}]}}, 
     {Arrowheads[{{0.013127617905417648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.024505889128745628`, 1.1885374500585355`}, {
        0.09314116969478414, 1.2114625499414637`}}]}}, 
     {Arrowheads[{{0.013529796440564333`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2718658011516803, -1.2072829857003338`}, {
        0.19872243414243773`, -1.1927170142996661`}}]}}, 
     {Arrowheads[{{0.015071701293711267`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2758627480352427, -1.0677518913188349`}, {
        0.19472548725887534`, -1.0498951675046948`}}]}}, 
     {Arrowheads[{{0.01697908066477521, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2807448484326464, -0.9287888946288142}, {
        0.18984338686147162`, -0.9065052230182449}}]}}, 
     {Arrowheads[{{0.019387547870146127`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2868126126691828, -0.7906512430854068}, {
        0.18377562262493524`, -0.7622899333851818}}]}}, 
     {Arrowheads[{{0.022503892202001698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2945153807619663, -0.6537267357753135}, {
        0.1760728545321517, -0.6168614995188045}}]}}, 
     {Arrowheads[{{0.026656977789932933`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3045725481181197, -0.518578521985385}, {
        0.16601568717599835`, -0.4696567721322625}}]}}, 
     {Arrowheads[{{0.032391686940352236`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.31826159255747455`, -0.38590357392354324`}, {
        0.15232664273664348`, -0.3199787790176336}}]}}, 
     {Arrowheads[{{0.040587803758723034`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.33807283137281835`, -0.25592721847680316`}, {
        0.13251540392129968`, -0.1676021932879031}}]}}, 
     {Arrowheads[{{0.052037501067732896`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.36784326800963596`, -0.12536552556101807`}, {
        0.1027449672844821, -0.015810945027217603`}}]}}, 
     {Arrowheads[{{0.05203750106773292, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10274496728448204`, 0.015810945027217208`}, {
        0.36784326800963596`, 0.1253655255610177}}]}}, 
     {Arrowheads[{{0.04058780375872307, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.1325154039212996, 0.16760219328790266`}, {
        0.3380728313728184, 0.25592721847680283`}}]}}, 
     {Arrowheads[{{0.03239168694035226, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.15232664273664343`, 0.31997877901763316`}, {
        0.3182615925574746, 0.38590357392354285`}}]}}, 
     {Arrowheads[{{0.02665697778993296, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.1660156871759983, 0.46965677213226203`}, {
        0.30457254811811973`, 0.5185785219853847}}]}}, 
     {Arrowheads[{{0.0225038922020017, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.17607285453215168`, 0.6168614995188041}, {
        0.2945153807619663, 0.653726735775313}}]}}, 
     {Arrowheads[{{0.019387547870146127`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.18377562262493524`, 0.7622899333851813}, {
        0.2868126126691828, 0.7906512430854062}}]}}, 
     {Arrowheads[{{0.016979080664775232`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.18984338686147156`, 0.9065052230182443}, {
        0.28074484843264647`, 0.9287888946288136}}]}}, 
     {Arrowheads[{{0.01507170129371127, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.19472548725887537`, 1.0498951675046941`}, {
        0.2758627480352427, 1.0677518913188344`}}]}}, 
     {Arrowheads[{{0.013529796440564327`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.19872243414243776`, 1.1927170142996657`}, {
        0.2718658011516803, 1.2072829857003333`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.44955401500629866`, -1.2024998828640752`}, {
        0.37397539675840763`, -1.197500117135925}}]}}, 
     {Arrowheads[{{0.015350155243130648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.453959428635645, -1.0619029878735429`}, {
        0.36956998312906136`, -1.0557440709499866`}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4594302893993537, -0.9215107932804454}, {
        0.3640991223653526, -0.9137833243666136}}]}}, 
     {Arrowheads[{{0.019888137372837372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.466355465429616, -0.7814147380883764}, {
        0.3571739463350903, -0.771526438382212}}]}}, 
     {Arrowheads[{{0.023170576259998205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4752991564963728, -0.6417440315971059}, {
        0.3482302552683335, -0.6288442036970121}}]}}, 
     {Arrowheads[{{0.027496482743281668`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.48706609021495806`, -0.5026544596648452}, {
        0.33646332154974823`, -0.4855808344528022}}]}}, 
     {Arrowheads[{{0.03325362540608418, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5027173474363977, -0.36423497110382186`}, {
        0.3208120643283085, -0.341647381837355}}]}}, 
     {Arrowheads[{{0.04080882460299786, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5233093533213213, -0.22619529480784276`}, {
        0.30022005844338495`, -0.1973341169568635}}]}}, 
     {Arrowheads[{{0.05010844719022016, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5488455857875305, -0.08737700370833855}, {
        0.2746838259771758, -0.05379946687989713}}]}}, 
     {Arrowheads[{{0.05010844719022018, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2746838259771758, 0.05379946687989674}, {
        0.5488455857875306, 0.08737700370833816}}]}}, 
     {Arrowheads[{{0.0408088246029979, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.30022005844338484`, 0.19733411695686312`}, {
        0.5233093533213214, 0.22619529480784237`}}]}}, 
     {Arrowheads[{{0.03325362540608421, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.32081206432830844`, 0.3416473818373546}, {
        0.5027173474363978, 0.36423497110382147`}}]}}, 
     {Arrowheads[{{0.027496482743281696`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3364633215497482, 0.48558083445280176`}, {
        0.4870660902149581, 0.5026544596648449}}]}}, 
     {Arrowheads[{{0.023170576259998216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.34823025526833346`, 0.6288442036970117}, {
        0.47529915649637283`, 0.6417440315971055}}]}}, 
     {Arrowheads[{{0.019888137372837393`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.35717394633509025`, 0.7715264383822117}, {
        0.46635546542961603`, 0.781414738088376}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3640991223653526, 0.9137833243666131}, {
        0.4594302893993537, 0.9215107932804449}}]}}, 
     {Arrowheads[{{0.015350155243130648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3695699831290613, 1.0557440709499861`}, {
        0.4539594286356449, 1.0619029878735424`}}]}}, 
     {Arrowheads[{{0.013740997549939769`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.37397539675840763`, 1.1975001171359243`}, {
        0.4495540150062986, 1.2024998828640745`}}]}}, 
     {Arrowheads[{{0.013740997549939763`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6260246032415928, -1.1975001171359247`}, {
        0.5504459849937019, -1.2024998828640752`}}]}}, 
     {Arrowheads[{{0.015350155243130659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6304300168709391, -1.0557440709499866`}, {
        0.5460405713643555, -1.0619029878735429`}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6359008776346479, -0.9137833243666136}, {
        0.5405697106006467, -0.9215107932804454}}]}}, 
     {Arrowheads[{{0.019888137372837372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6428260536649102, -0.771526438382212}, {
        0.5336445345703845, -0.7814147380883764}}]}}, 
     {Arrowheads[{{0.023170576259998205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6517697447316669, -0.6288442036970121}, {
        0.5247008435036277, -0.6417440315971059}}]}}, 
     {Arrowheads[{{0.02749648274328167, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6635366784502522, -0.48558083445280215`}, {
        0.5129339097850424, -0.5026544596648453}}]}}, 
     {Arrowheads[{{0.03325362540608419, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6791879356716919, -0.34164738183735494`}, {
        0.49728265256360266`, -0.3642349711038219}}]}}, 
     {Arrowheads[{{0.04080882460299787, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6997799415566155, -0.19733411695686343`}, {
        0.47669064667867905`, -0.22619529480784284`}}]}}, 
     {Arrowheads[{{0.05010844719022017, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7253161740228247, -0.05379946687989704}, {
        0.45115441421246993`, -0.08737700370833865}}]}}, 
     {Arrowheads[{{0.05010844719022019, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.45115441421246993`, 0.08737700370833826}, {
        0.7253161740228248, 0.05379946687989665}}]}}, 
     {Arrowheads[{{0.040808824602997874`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.476690646678679, 0.22619529480784245`}, {
        0.6997799415566155, 0.19733411695686304`}}]}}, 
     {Arrowheads[{{0.03325362540608419, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.49728265256360266`, 0.3642349711038215}, {
        0.6791879356716919, 0.3416473818373545}}]}}, 
     {Arrowheads[{{0.02749648274328171, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5129339097850423, 0.5026544596648449}, {
        0.6635366784502523, 0.48558083445280176`}}]}}, 
     {Arrowheads[{{0.023170576259998216`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5247008435036277, 0.6417440315971055}, {0.651769744731667,
         0.6288442036970117}}]}}, 
     {Arrowheads[{{0.019888137372837372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5336445345703844, 0.781414738088376}, {0.6428260536649101,
         0.7715264383822117}}]}}, 
     {Arrowheads[{{0.01735114455407311, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5405697106006467, 0.9215107932804449}, {
        0.6359008776346479, 0.9137833243666131}}]}}, 
     {Arrowheads[{{0.015350155243130659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5460405713643555, 1.0619029878735424`}, {
        0.6304300168709391, 1.0557440709499861`}}]}}, 
     {Arrowheads[{{0.01374099754993978, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5504459849937018, 1.2024998828640747`}, {
        0.6260246032415928, 1.1975001171359245`}}]}}, 
     {Arrowheads[{{0.013529796440564323`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8012775658575627, -1.1927170142996661`}, {
        0.7281341988483202, -1.2072829857003338`}}]}}, 
     {Arrowheads[{{0.015071701293711265`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.805274512741125, -1.0498951675046946`}, {
        0.7241372519647578, -1.0677518913188349`}}]}}, 
     {Arrowheads[{{0.01697908066477522, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8101566131385288, -0.9065052230182448}, {
        0.719255151567354, -0.9287888946288141}}]}}, 
     {Arrowheads[{{0.019387547870146123`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8162243773750653, -0.7622899333851817}, {
        0.7131873873308178, -0.7906512430854068}}]}}, 
     {Arrowheads[{{0.022503892202001687`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8239271454678487, -0.6168614995188045}, {
        0.7054846192380342, -0.6537267357753135}}]}}, 
     {Arrowheads[{{0.026656977789932926`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8339843128240021, -0.4696567721322624}, {
        0.6954274518818808, -0.518578521985385}}]}}, 
     {Arrowheads[{{0.032391686940352236`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.847673357263357, -0.31997877901763355`}, {
        0.681738407442526, -0.3859035739235433}}]}}, 
     {Arrowheads[{{0.04058780375872305, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8674845960787008, -0.16760219328790302`}, {
        0.6619271686271821, -0.2559272184768032}}]}}, 
     {Arrowheads[{{0.052037501067732896`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8972550327155183, -0.015810945027217492`}, {
        0.6321567319903646, -0.12536552556101818`}}]}}, 
     {Arrowheads[{{0.05203750106773292, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6321567319903645, 0.12536552556101782`}, {
        0.8972550327155183, 0.01581094502721709}}]}}, 
     {Arrowheads[{{0.04058780375872307, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6619271686271821, 0.2559272184768029}, {
        0.8674845960787009, 0.1676021932879026}}]}}, 
     {Arrowheads[{{0.03239168694035226, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6817384074425259, 0.38590357392354296`}, {
        0.847673357263357, 0.31997877901763316`}}]}}, 
     {Arrowheads[{{0.026656977789932933`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6954274518818808, 0.5185785219853847}, {
        0.8339843128240021, 0.469656772132262}}]}}, 
     {Arrowheads[{{0.022503892202001715`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7054846192380342, 0.653726735775313}, {0.8239271454678488,
         0.616861499518804}}]}}, 
     {Arrowheads[{{0.019387547870146123`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7131873873308177, 0.7906512430854064}, {
        0.8162243773750651, 0.7622899333851813}}]}}, 
     {Arrowheads[{{0.01697908066477522, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7192551515673541, 0.9287888946288138}, {
        0.8101566131385289, 0.9065052230182444}}]}}, 
     {Arrowheads[{{0.015071701293711265`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7241372519647578, 1.0677518913188344`}, {
        0.805274512741125, 1.0498951675046941`}}]}}, 
     {Arrowheads[{{0.013529796440564323`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7281341988483202, 1.2072829857003333`}, {
        0.8012775658575627, 1.1927170142996657`}}]}}, 
     {Arrowheads[{{0.013127617905417648`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9754941108712548, -1.188537450058536}, {
        0.9068588303052163, -1.2114625499414642`}}]}}, 
     {Arrowheads[{{0.014542990019468313`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9787648530538937, -1.0449057670029498`}, {
        0.9035880881225776, -1.0727412918205796`}}]}}, 
     {Arrowheads[{{0.016272107256669006`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9826061479821713, -0.9004737816949067}, {
        0.8997467931943, -0.9348203359521523}}]}}, 
     {Arrowheads[{{0.018427008297575858`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.987149521523482, -0.754887911347473}, {
        0.8952034196529891, -0.7980532651231155}}]}}, 
     {Arrowheads[{{0.02118358153152599, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9925688944745799, -0.607588882564306}, {
        0.8897840467018914, -0.662999352729812}}]}}, 
     {Arrowheads[{{0.024844937579490488`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9991270922053235, -0.45763951416233134`}, {
        0.8832258489711479, -0.5305957799553161}}]}}, 
     {Arrowheads[{{0.030017486788541173`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.007361168466535, -0.3033008655631616}, {
        0.8749917727099363, -0.4025814873780152}}]}}, 
     {Arrowheads[{{0.03825714390929145, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0190515686074881`, -0.14067762816916182`}, {
        0.863301372568983, -0.2828517835955444}}]}}, 
     {Arrowheads[{{0.05575762470829351, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0464566397395045`, 0.04135856449703354}, {
        0.8358963014369667, -0.18253503508526922`}}]}}, 
     {Arrowheads[{{0.05575762470829362, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8358963014369665, 0.182535035085269}, {
        1.0464566397395048`, -0.041358564497034095`}}]}}, 
     {Arrowheads[{{0.038257143909291476`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.863301372568983, 0.28285178359554414`}, {
        1.0190515686074881`, 0.14067762816916135`}}]}}, 
     {Arrowheads[{{0.030017486788541183`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8749917727099363, 0.40258148737801486`}, {
        1.007361168466535, 0.30330086556316116`}}]}}, 
     {Arrowheads[{{0.02484493757949048, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8832258489711478, 0.5305957799553157}, {
        0.9991270922053234, 0.45763951416233095`}}]}}, 
     {Arrowheads[{{0.021183581531526007`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8897840467018913, 0.6629993527298116}, {
        0.9925688944745799, 0.6075888825643055}}]}}, 
     {Arrowheads[{{0.01842700829757587, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8952034196529892, 0.798053265123115}, {0.9871495215234821,
         0.7548879113474725}}]}}, 
     {Arrowheads[{{0.016272107256669034`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8997467931943, 0.9348203359521519}, {0.9826061479821714, 
        0.9004737816949061}}]}}, 
     {Arrowheads[{{0.014542990019468353`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9035880881225775, 1.0727412918205792`}, {
        0.9787648530538938, 1.0449057670029493`}}]}}, 
     {Arrowheads[{{0.013127617905417659`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9068588303052163, 1.2114625499414635`}, {
        0.9754941108712549, 1.1885374500585355`}}]}}, 
     {Arrowheads[{{0.012572289032024979`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1489884523844096`, -1.185221807578826}, {
        1.0863056652626495`, -1.2147781924211738`}}]}}, 
     {Arrowheads[{{0.013818604383719657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1513725389813954`, -1.0411283268312463`}, {
        1.0839215786656642`, -1.076518731992283}}]}}, 
     {Arrowheads[{{0.015308797801352542`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1539714680036015`, -0.8961811984730088}, {
        1.0813226496434578`, -0.93911291917405}}]}}, 
     {Arrowheads[{{0.0171168985608574, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1567300144937083`, -0.7500484962460914}, {
        1.0785641031533508`, -0.8028926802244971}}]}}, 
     {Arrowheads[{{0.019351722288825802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1594967105519487`, -0.602228871864419}, {
        1.0757974070951106`, -0.668359363429699}}]}}, 
     {Arrowheads[{{0.02218366408088065, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1618983923648427`, -0.4519271824202017}, {
        1.0733957252822166`, -0.5363081116974457}}]}}, 
     {Arrowheads[{{0.025900636613884812`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1629740844968293`, -0.29779287790847897`}, {
        1.07232003315023, -0.4080894750326979}}]}}, 
     {Arrowheads[{{0.03099785349711695, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1597674886601212`, -0.13743550183666478`}, {
        1.075526628986938, -0.2860939099280415}}]}}, 
     {Arrowheads[{{0.03754378372508135, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1405874804807077`, 0.030312146937918602`}, {
        1.0947066371663519`, -0.1714886175261543}}]}}, 
     {Arrowheads[{{0.03754378372508138, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0947066371663519`, 0.171488617526154}, {
        1.1405874804807077`, -0.030312146937919088`}}]}}, 
     {Arrowheads[{{0.03099785349711697, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.075526628986938, 0.2860939099280412}, {
        1.1597674886601212`, 0.1374355018366643}}]}}, 
     {Arrowheads[{{0.025900636613884833`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.07232003315023, 0.4080894750326976}, {1.1629740844968293`,
         0.2977928779084785}}]}}, 
     {Arrowheads[{{0.022183664080880666`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0733957252822166`, 0.5363081116974454}, {
        1.1618983923648427`, 0.45192718242020125`}}]}}, 
     {Arrowheads[{{0.019351722288825802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0757974070951106`, 0.6683593634296986}, {
        1.1594967105519487`, 0.6022288718644185}}]}}, 
     {Arrowheads[{{0.01711689856085741, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0785641031533508`, 0.8028926802244967}, {
        1.1567300144937083`, 0.7500484962460908}}]}}, 
     {Arrowheads[{{0.015308797801352553`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0813226496434578`, 0.9391129191740497}, {
        1.1539714680036015`, 0.8961811984730084}}]}}, 
     {Arrowheads[{{0.013818604383719657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.083921578665664, 1.0765187319922827`}, {
        1.1513725389813951`, 1.041128326831246}}]}}, 
     {Arrowheads[{{0.012572289032024979`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0863056652626497`, 1.2147781924211736`}, {
        1.1489884523844098`, 1.1852218075788257`}}]}}, 
     {Arrowheads[{{0.011912824053351105`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.322123944090053, -1.182863243041782}, {
        1.2661113500275947`, -1.217136756958218}}]}}, 
     {Arrowheads[{{0.012970645227741053`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.323631226697444, -1.0386512826326848`}, {
        1.2646040674202037`, -1.0789957761908449`}}]}}, 
     {Arrowheads[{{0.014198637077553295`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.325058899175584, -0.8936877666230362}, {
        1.2631763949420636`, -0.9416063510240228}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.326234520130412, -0.7477543071639721}, {
        1.2620007739872354`, -0.8051868693066164}}]}}, 
     {Arrowheads[{{0.01730893096661073, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3268389475994213`, -0.6005790121187431}, {
        1.2613963465182263`, -0.6700092231753749}}]}}, 
     {Arrowheads[{{0.01926581232382168, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.32627591521863, -0.45186426676773417`}, {
        1.2619593788990173`, -0.5363710273499133}}]}}, 
     {Arrowheads[{{0.021497164399658753`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3234239047892633`, -0.3014477590195971}, {
        1.2648113893283845`, -0.40443459392157977`}}]}}, 
     {Arrowheads[{{0.02382579744488329, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3163121475575015`, -0.149962257277909}, {
        1.2719231465601462`, -0.27356715448679725`}}]}}, 
     {Arrowheads[{{0.025567923001979964`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3028187769900255`, -0.0006590963847564885}, {
        1.2854165171276222`, -0.1405173742034792}}]}}, 
     {Arrowheads[{{0.025567923001979954`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2854165171276222`, 0.14051737420347882`}, {
        1.3028187769900252`, 0.000659096384756086}}]}}, 
     {Arrowheads[{{0.02382579744488329, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2719231465601462`, 0.2735671544867969}, {
        1.3163121475575013`, 0.14996225727790857`}}]}}, 
     {Arrowheads[{{0.021497164399658767`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2648113893283843`, 0.40443459392157943`}, {
        1.323423904789263, 0.30144775901959664`}}]}}, 
     {Arrowheads[{{0.019265812323821702`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2619593788990175`, 0.5363710273499129}, {
        1.3262759152186303`, 0.4518642667677337}}]}}, 
     {Arrowheads[{{0.01730893096661073, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2613963465182263`, 0.6700092231753745}, {
        1.3268389475994213`, 0.6005790121187427}}]}}, 
     {Arrowheads[{{0.01563161768674601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2620007739872356`, 0.805186869306616}, {
        1.3262345201304122`, 0.7477543071639716}}]}}, 
     {Arrowheads[{{0.014198637077553295`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2631763949420636`, 0.9416063510240223}, {
        1.325058899175584, 0.8936877666230357}}]}}, 
     {Arrowheads[{{0.012970645227741053`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2646040674202037`, 1.0789957761908444`}, {
        1.323631226697444, 1.0386512826326844`}}]}}, 
     {Arrowheads[{{0.011912824053351105`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2661113500275947`, 1.2171367569582174`}, {
        1.322123944090053, 1.1828632430417814`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.495225257393322, -1.1814021143664217`}, {
        1.445951213194914, -1.2185978856335784`}}]}}, 
     {Arrowheads[{{0.012072512361463282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4959894145974617`, -1.0373317613561448`}, {
        1.4451870559907742`, -1.0803152974673846`}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.496494803441805, -0.8926906884894482}, {
        1.4446816671464306`, -0.9426034291576109}}]}}, 
     {Arrowheads[{{0.014144384224218001`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4965462944148389`, -0.7473861027715781}, {
        1.4446301761733968`, -0.8055550736990104}}]}}, 
     {Arrowheads[{{0.015348449856734189`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4958410991792344`, -0.6013563382086448}, {
        1.4453353714090018`, -0.6692318970854734}}]}}, 
     {Arrowheads[{{0.016637991562921846`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4939270393022495`, -0.45464476801506376`}, {
        1.4472494312859867`, -0.5335905261025837}}]}}, 
     {Arrowheads[{{0.017936440162617952`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.490193925306397, -0.3075600887359119}, {
        1.4509825452818388`, -0.39832226420526495`}}]}}, 
     {Arrowheads[{{0.01907414277996195, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4840367364971196`, -0.1609432680674084}, {
        1.4571397340911163`, -0.2625861436972979}}]}}, 
     {Arrowheads[{{0.019772769634296784`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4754386068137537`, -0.016308291135652762`}, {
        1.4657378637744825`, -0.12486817945258291`}}]}}, 
     {Arrowheads[{{0.019772769634296784`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4657378637744825`, 0.12486817945258252`}, {
        1.4754386068137537`, 0.016308291135652374`}}]}}, 
     {Arrowheads[{{0.01907414277996195, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4571397340911163`, 0.2625861436972975}, {
        1.4840367364971196`, 0.16094326806740802`}}]}}, 
     {Arrowheads[{{0.017936440162617962`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.450982545281839, 0.3983222642052646}, {
        1.4901939253063972`, 0.3075600887359115}}]}}, 
     {Arrowheads[{{0.016637991562921846`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4472494312859865`, 0.5335905261025833}, {
        1.4939270393022492`, 0.4546447680150633}}]}}, 
     {Arrowheads[{{0.015348449856734189`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4453353714090016`, 0.6692318970854729}, {
        1.4958410991792341`, 0.6013563382086443}}]}}, 
     {Arrowheads[{{0.014144384224217975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.444630176173397, 0.8055550736990099}, {
        1.4965462944148389`, 0.7473861027715777}}]}}, 
     {Arrowheads[{{0.013051586754250093`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4446816671464309`, 0.9426034291576103}, {
        1.4964948034418053`, 0.8926906884894477}}]}}, 
     {Arrowheads[{{0.012072512361463311`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4451870559907742`, 1.0803152974673842`}, {
        1.495989414597462, 1.0373317613561444`}}]}}, 
     {Arrowheads[{{0.011199963345734237`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.445951213194914, 1.2185978856335777`}, {1.495225257393322,
         1.181402114366421}}]}}, 
     {Arrowheads[{{0.010477306125975746`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6685228903922633`, -1.1806826142523357`}, {
        1.6255947566665612`, -1.219317385747664}}]}}, 
     {Arrowheads[{{0.01118362862614915, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6687287781749205`, -1.036903222200876}, {
        1.6253888688839038`, -1.0807438366226534`}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.668604635339085, -0.8927382994446785}, {
        1.6255130117197392`, -0.9425558182023804}}]}}, 
     {Arrowheads[{{0.012768289697852282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6679895577091002`, -0.7481808199035157}, {
        1.626128089349724, -0.8047603565670728}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6666711075988743`, -0.6032788912927585}, {
        1.6274465394599502`, -0.6673093440013596}}]}}, 
     {Arrowheads[{{0.014475369768919472`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6643941654513221`, -0.4581847791459418}, {
        1.6297234816075021`, -0.5300505149717056}}]}}, 
     {Arrowheads[{{0.015263437675282363`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6609117998427898`, -0.3132195606184984}, {
        1.6332058472160345`, -0.39266279232267837`}}]}}, 
     {Arrowheads[{{0.015890646781373568`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6561137917959785`, -0.16891437968875614`}, {
        1.638003855262846, -0.2546150320759501}}]}}, 
     {Arrowheads[{{0.016244935134227048`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6502225169672, -0.025927078098396193`}, {
        1.6438951300916242`, -0.11524939248983948`}}]}}, 
     {Arrowheads[{{0.016244935134227045`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6438951300916242`, 0.11524939248983909`}, {
        1.6502225169671998`, 0.025927078098395805`}}]}}, 
     {Arrowheads[{{0.01589064678137358, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.638003855262846, 0.25461503207594977`}, {
        1.6561137917959785`, 0.16891437968875575`}}]}}, 
     {Arrowheads[{{0.015263437675282363`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6332058472160345`, 0.39266279232267803`}, {
        1.6609117998427898`, 0.3132195606184981}}]}}, 
     {Arrowheads[{{0.014475369768919487`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6297234816075021`, 0.5300505149717053}, {
        1.6643941654513221`, 0.45818477914594136`}}]}}, 
     {Arrowheads[{{0.013622335338254642`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6274465394599502`, 0.6673093440013591}, {
        1.6666711075988743`, 0.603278891292758}}]}}, 
     {Arrowheads[{{0.012768289697852282`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.626128089349724, 0.8047603565670723}, {
        1.6679895577091002`, 0.7481808199035153}}]}}, 
     {Arrowheads[{{0.011949495453044802`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6255130117197392`, 0.94255581820238}, {1.668604635339085, 
        0.8927382994446781}}]}}, 
     {Arrowheads[{{0.01118362862614915, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6253888688839038`, 1.080743836622653}, {
        1.6687287781749205`, 1.0369032222008756`}}]}}, 
     {Arrowheads[{{0.010477306125975746`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.625594756666561, 1.2193173857476638`}, {1.668522890392263,
         1.1806826142523354`}}]}}, 
     {Arrowheads[{{0.0097764739834501, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8421432687018882`, -1.1805175136617894`}, {
        1.8049155548275244`, -1.2194824863382105`}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8419687554783775`, -1.0370847000498948`}, {
        1.805090068051035, -1.0805623587736346`}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8414635664124217`, -0.893416422471782}, {
        1.8055952571169906`, -0.9418776951752769}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.840515212819351, -0.7495471606533741}, {
        1.8065436107100614`, -0.8033940158172145}}]}}, 
     {Arrowheads[{{0.01216112908340843, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.838996395100025, -0.6055585939745416}, {
        1.8080624284293876`, -0.6650296413195764}}]}}, 
     {Arrowheads[{{0.012740596380756667`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8367841825088898`, -0.46160070516923063`}, {
        1.810274641020523, -0.5266345889484169}}]}}, 
     {Arrowheads[{{0.01324656958214583, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8337998288808706`, -0.3179063323369845}, {
        1.8132589946485418`, -0.38797602060419234`}}]}}, 
     {Arrowheads[{{0.01362784987194159, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.830069131209838, -0.17477835738853467`}, {
        1.8169896923195743`, -0.2487510543761716}}]}}, 
     {Arrowheads[{{0.013834467068875205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.825779259267429, -0.03252514961491328}, {
        1.8212795642619837`, -0.10865132097332239`}}]}}, 
     {Arrowheads[{{0.013834467068875205`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8212795642619837`, 0.108651320973322}, {1.825779259267429,
         0.032525149614912895`}}]}}, 
     {Arrowheads[{{0.01362784987194159, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8169896923195745`, 0.2487510543761712}, {
        1.8300691312098383`, 0.17477835738853428`}}]}}, 
     {Arrowheads[{{0.01324656958214584, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.813258994648542, 0.387976020604192}, {1.8337998288808708`,
         0.3179063323369841}}]}}, 
     {Arrowheads[{{0.012740596380756657`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.810274641020523, 0.5266345889484164}, {
        1.8367841825088898`, 0.46160070516923024`}}]}}, 
     {Arrowheads[{{0.01216112908340845, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8080624284293876`, 0.665029641319576}, {
        1.8389963951000252`, 0.6055585939745411}}]}}, 
     {Arrowheads[{{0.011550188090361797`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8065436107100616`, 0.8033940158172139}, {
        1.8405152128193512`, 0.7495471606533736}}]}}, 
     {Arrowheads[{{0.010937677956452644`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8055952571169909`, 0.9418776951752765}, {
        1.841463566412422, 0.8934164224717817}}]}}, 
     {Arrowheads[{{0.010342744314054698`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8050900680510351`, 1.0805623587736342`}, {
        1.8419687554783777`, 1.0370847000498944`}}]}}, 
     {Arrowheads[{{0.0097764739834501, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8049155548275244`, 1.21948248633821}, {
        1.8421432687018882`, 1.180517513661789}}]}}, 
     {Arrowheads[{{0.009116798619914601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.016130228034535, -1.1807338717514044`}, {
        1.983869771965466, -1.2192661282485957`}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.015718081679951, -1.037643444328698}, {
        1.98428191832005, -1.0800036144948315`}}]}}, 
     {Arrowheads[{{0.010032604380582736`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0150092145220806`, -0.8944240649719418}, {
        1.9849907854779203`, -0.9408700526751171}}]}}, 
     {Arrowheads[{{0.010494535920320246`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.013932337673443, -0.7511229581833889}, {
        1.986067662326558, -0.8018182182871996}}]}}, 
     {Arrowheads[{{0.01093952108451705, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.012418296106781, -0.6078195865070382}, {
        1.9875817038932195`, -0.66276864878708}}]}}, 
     {Arrowheads[{{0.011346123674529826`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.010415300919506, -0.4646317540788422}, {
        1.989584699080495, -0.5236035400388053}}]}}, 
     {Arrowheads[{{0.011688143720156717`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.007911382163011, -0.3217137711026357}, {
        1.9920886178369897`, -0.3841685818385412}}]}}, 
     {Arrowheads[{{0.011937480299842701`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0049592569440335`, -0.1792394247726037}, {
        1.9950407430559673`, -0.24428998699210255`}}]}}, 
     {Arrowheads[{{0.012069481079506502`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0016912625304792`, -0.037366258615373515`}, {
        1.9983087374695216`, -0.10381021197286217`}}]}}, 
     {Arrowheads[{{0.01206948107950649, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9983087374695219`, 0.10381021197286176`}, {
        2.001691262530479, 0.03736625861537314}}]}}, 
     {Arrowheads[{{0.011937480299842701`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9950407430559673`, 0.24428998699210216`}, {
        2.0049592569440335`, 0.17923942477260332`}}]}}, 
     {Arrowheads[{{0.011688143720156707`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9920886178369899`, 0.38416858183854075`}, {
        2.007911382163011, 0.32171377110263527`}}]}}, 
     {Arrowheads[{{0.011346123674529857`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9895846990804948`, 0.5236035400388048}, {
        2.0104153009195063`, 0.46463175407884183`}}]}}, 
     {Arrowheads[{{0.010939521084517033`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9875817038932198`, 0.6627686487870794}, {
        2.012418296106781, 0.6078195865070376}}]}}, 
     {Arrowheads[{{0.010494535920320265`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9860676623265578`, 0.8018182182871991}, {
        2.013932337673443, 0.7511229581833885}}]}}, 
     {Arrowheads[{{0.010032604380582736`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9849907854779203`, 0.9408700526751167}, {
        2.0150092145220806`, 0.8944240649719414}}]}}, 
     {Arrowheads[{{0.009569687754266346, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.98428191832005, 1.080003614494831}, {2.015718081679951, 
        1.0376434443286975`}}]}}, 
     {Arrowheads[{{0.009116798619914601, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.983869771965466, 1.219266128248595}, {2.016130228034535, 
        1.1807338717514038`}}]}}}}, GraphicsComplexBox[CompressedData["
1:eJx0nXdAVMcX79fea9TYS2JL7LHXwdh772IXKxZErOiKIJZYYu8So7FHVFRU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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV0l1ojgEYh/F72LLNhtfma6tZStQsJ5QtB6ZMTibbcjJCKAr5iEIcjCQO
RsqmkGKKWdgyi83SSEn5PDFGUtQwyrTNmN8O/l3PdX+9e989+eu3r9yWFBE/
JTE5YpI0ZEf0TYrYNC3iNzby6/IvPaJWbUBthQxKHR9Wv6E/elxEr/0f8l2y
x0cMmXmXE/EHb5lpkr+ez9obY76Zp+H5kTvqaySyIi7wdPVW/QyMKRFJMuzu
dHfHmOl2dzTeM5OMz3gKbnSjTS3L3iV3xqptVUvFyzxbvUN/Gl7haeq79Gd5
3o3lPqNCKuU2/+RuCy7ii6UoM+I+dki7vNVvw6GpEQXmuvhj/spcp5uFaq89
d/mcYs+d2CI1I3+f73RTzsgTaZdSs02JiP3u1OOxjIjjuI3XYjc/iOv4Dqx2
ZxGW8uX4QH8DVvBVeIDPxCI+D1+6Pxnn8lTcY//1RP97/hEX6g/hDN6L2+3f
xZSR74WP9BtxYHrERdxsfyu+5/uwQP80fuVHcb39cnzB12Kr/lJ8yBdilf1k
vMYTmKdfgHd4Hlba75ngHeH92KDfjSf5cyyzfw738quYGPnd8TBvwWX2q3EL
r8EL+ntwNd+MJfbn4xJegsn6VbiCl2Gx/RxcwGfjKf1MnMOTcL79p97J8fwN
9pv/hbn8MxbyJhzFO/CI/Xrs827U4Wz7G7GL78Qe8yfwCz+E+TzXe9Ps95+D
z/k379qgPJIP8lJSzf8HayuOOw==
          "]]},
        "0.42600000000000005`"],
       Annotation[#, 0.42600000000000005`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV01toznEcx/HfsuUwGxvbMM1wYYXcUEhqwxyyTSNhK6dh5BhTtrgYyeFi
FqEcUkwhOWUWIbYlSzYiUXO6mJo5beqZLHk9F5/n/f/8vp/P9/8c/s/IVVsK
N8eEEDK8dMSHUDU0hOmDQogdEMKy1BCKqJi+mV1ICaETj8j8kMmmi85+4vP0
EH5hDq0x78JVWEKr6ZJcMr636wO1Uj/3+C13zHnE3ssyV6jbWbN9EbzGz8c6
Pg8H6pTqrqcNFPqHcEMmDk/Z80+mlr9NS1zf01uKaXrb5XdQGfWXvyuTiOf0
eg8O4SH/gNbKP9Bbh8P1dsmXUwWlyj+SeUzxOjnOZtIMqnfWQF+pI7rbWQLV
2fONz3Q9l+bRHN35mE95FEd96VNiCE34lGbLHPTecnFBtIcF0R3Yad9Te3tw
dPR+ej3Oe7teir+ojU55v5/T3J/eUK8hvl8slP+YHEK1z/kajyX4/Lifb8KP
5s3672iq+9RjLVXZd17/Oh2nJrpPufI39cr1a/CAfYdwE38CW/nduILfipX2
TMNcfi4+Ml+NC/nFWMGPwin8BHxpfwqO4/tgmf6rJM8U/wknm//FEfx33Kx/
F+P4J9hofhW7h/m9sVR/I77nd+FY86P4ld+PK/UL8QW/HOvMZ2EDPxmL9GPx
Mp+EGeZj8Q6fgYv02weGcIaP4BXzVjzCt2C+/mncyV/CJPN63MvX4mz9SlzP
V+FZ8zIs5ksxW38i5vDZGGtehAV8Pk7VH4aT+DFYbZ6AWXzAifrPPNuJ/FuM
yHdhOt+G4/mbGMM/xH36NdH/q+fxJI7RL8F3/DZslz+MX/g9mMmne25u+f6z
sIXv8Oz9oUb6QC+pj/x/bqKpxw==
          "]]},
        "0.35500000000000004`"],
       Annotation[#, 0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV0mtozmEYx/F7NtvYZo+zjWHSlkNSZBbltIkYbSiR0aamMBQ5xAsk8cIx
FEYxOaQ0NbNohJRkcwpzHGViiC0me8FnL77P9/nd13Xd//v/704vWlVQGhVC
GOwnISWEB71CqMUiFOJTYgjfE0I40zOEFt6vp7l7COfks4jIL/TV4znikkN4
yVP1vOcEuYET+QNPs17VN4TpnGStyFoxlqLN/o3c1fpyXoGViHKGn9yM1ViD
r9ZiUeUMV9GhRwglzhLFy/iatX4c3TuEGHRALDqiv/VO3BnxSPO8BC51pvH2
ful8A60lW0vnIdbWqU0xl4NT8un2d00KoZzj+Yb1TM+u1jfL/9mYgTzMxAk9
xc70Sn09l+JXF9+D81FkdgKPQbn36tnHOZGIkcjEQv1zeRGyMQOTUWbviLkU
PLb/cG7DIDSZabN3rP/z+Qc+4qhnvPd+X/Gs/fvYp5Xz9b/rFsI+3+ApH7R3
De+Q7/E79Vrz9cj2rFtciT32O2X+Eg7hHq4jV3+FuY3my3mn/XbxCvkwv5Y3
82J5FW+1zzjOlafxTfUiLpDn8SY5ncfKI/iR/XvwMDmO15p/0tUdkhs4S/0v
95e/8Urz1Rwj3+U76he5NTWEk1xifjm/kTfwUPUD/EXewUvM5/NDuZCr1HP4
tpzFC8xH83k5wmnqQ/mKnMZzzH+OhHBc/s0X1F/zXrmO88wf43XyOY6o3+Jt
ciVPNb+Vl8l7uEx9LS+US3ii+VE8SZ7I0eoLeJacx9nmU3i0nMH71BM5U/7n
ro8yf5+T5Bf8W38zp8ofebhcwUGu4e3my7nFHT/CGeaLuV5ezZ/17+ZGeQsP
kFPdm8u+fybXyU3u3h/cwVs8Qpz+/wyvocc=
          "]]},
        "0.28400000000000003`"],
       Annotation[#, 0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1HtojmEYx/Ebbf9svBg7YOIPK5YUYSlGaCW0zXHCbDFyLOSQQyM5lEPO
x0jLKRKFFEsjJYU5yzk225Dl8A5z/Lx/fPe9f/d1X9fzvM+e9+1cPC9/bpMQ
Qn9/6hNCqEwOIS4Swh2O57vcPC2ERNy3foB7eISHaGV/XZsQonq3W+ckhTBb
rksMocHeW2eqUIgpKMIP+9XcwvxpXILp+G1/txkjzWitVmMviWs5z943Tpa/
cgOiSHW+kX/hJ/7E5iDd/kb3EXEfh61n6J8v18stkZjic2G5syvwxV4Kulln
4m37EN4g2ToF2RhqzmVzjlgfxTFUy2Ps9zf7ifMD+J298fZqeSzX8Diu40Je
2TaEU81D2Jwawk6sQxkO4HcLtA7hkFlRPumeaniP/ImL1Zc6V4qpmI9ZOGJe
gbnFeOo6i3gOvjmfw7kocm8DuDfKzG2jLx0J6IEMTHB+FE9EFoZhUOzezI/o
S8Vd8zO5EZ3xXk+j2XHW42LPGFXY6xqvPeMPeIgm5jRwrvMvfZYtPtN93mZ2
Oa+Rb/BL9Vux54ks16rgc9gU+1/qP40duIFLGOL8GX1L9JfxWvPW8yx5Jz+T
l/FkeS6XmtOPh8g5fEW9iPPk0bxU7sR95e5caX4Sd5PjeYH+e628R/Ir7qP+
k9Pljzxb/0VuJl/na+onuaFdCAe5RP9Mfi4v5q7qW7lOXsOF+nP5jjyJL6gP
5qtyHy7Q35SPyxHuoN6Vz8sdOF9/bcsQ9stRPqH+jDfLt3m4/n28UD7GEfUK
XiWf46H6S3m6vIkPqC/gCXIJZ+vvyQPlbG6qXsAj5OGcpT+Ve8ldeIt6AmfI
f31/e+q/yYnyY446/5nT5CrOlM/wP9+Vcl6tv4y/yLu4i/5ifiLPi/0+OL+B
q+Xl3FFO896c9fwz+Lb83rv3HdfwApWId/4/NA6/WQ==
          "]]},
        "0.21300000000000002`"],
       Annotation[#, 0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV021olXUYx/FroaVbnU2nbjZLrR0wCcHIRi9SaUvIBdMJMtNsy5Zu5oaK
U5LaQmdq28HHSkWRg2goomAmxQgdhYSaFaH4/EJzbqJMnNESpc9efM/3/v2v
h/u+D+eM/qC+vC4jIqp93MuKWD0kopvLh0VMR/bwiAQWOp+cG3G0IGISN8tr
UOL6e2fFfOvpiB6zm/S/JQ/Ijrhkx0C+zEOc52KtuV59W11P1bdIrsN/zr5y
Vuos28ww13kYijJnXfY/0tNpVxfqUI8Zaj94hnJeZ896zHY93Ny7fMfcU3is
N9/eBl6OSrU2c+9zq5kUal2PNFfD3WYGIZEX8by5z8w0olA9iRdRr2+DuRfU
c9TmuW51/mHfc3ALvkQKvwyNKLCvWi1LbyYWJCKa5Br+W/0mOlDirFT9BNox
ALV6kpxWL1bfxluQ80zESs/YhAasRzOW6Z/muU71fad82L0zPO8e7/wkz1df
Zz6Fu/Jqnqv3Uz5gZ2u+/ViDNHbgoZmHgyN22dHD++28yV/Ld7hKfYW+RszD
YtRij30V9lbhgnst449xX/8ULkOld3qDX0Xa3lxzI5CJcUhilv5yno0ivI3J
fc9mf8JcHn63fyz3YhQ6zfTa3c/1TL6L6/jGPa75nrrwF8KeB1ym/4p3SXmn
P3mj3W28Sj7JV9RPmz+PIvc6zkfQYt9u8wexGSfxI4r1HzK33Hyam+37gmvl
LXxR/oTfkxdxoz2vc7E8hX9Sr+Rp8gxeIY/k1+SX+az9g/kluT8vMf/HIP8h
+SpPUP+XR8i3eaH5Y/yE/DO3q+/nB89G7ORq8zV8SW7gMeob+Ja8iueaL+Pf
5Dl8VP1NPiFP4Iq+3xnvkxNcoD6Gv5MLeLr5jpyI7XIPf6t+kVvlM1xqfhsv
lfdyQv04N8lHuMR8I38kt/AO9SU8S67miebH8yR5ImeoV/A7cikXmc/jV+RC
TqlnclJ+5H8y3vyvnCWf4x793ZwvX+ex8iF+7L/Sxp+bT/M9eSsXmq/i83Id
d+hfyzfklfycnO93c9j3n+Qzcqff3j9ox2WcRX/9/wNoAMoG
          "]]},
        "0.14200000000000002`"],
       Annotation[#, 0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03tsTnccx/GfW6iootbyPOIyjZJFQla1iEtUinRJbcIqo1Fd3ebWuhWT
ou6phqm4ZSKPZrTLhrBITMREVoKVZZmwmT/IzEbtUqKRib3OH+++f5/z/X2/
53eentOveMnkxa1CCPP8ad0phCdpITThMbqnhDA1NYQpeKy2pHsIBdbfxEP4
gKvkJNcP9QyhUI7zUn0teIGY/FS9Az6znmFPTzOLeRbK9JeixPq8mR/xTjnV
/lr7F8pv8iqz4vrap4fQT+6Lf+zphiPWC+zro17GF8wp5Wa1HjganUnebe6n
iLlW59pK15bJA6P51plczuN5Aurcsx5F1rPwnXUDLiNXXyU2IEUewJ/Ysxbr
5Q2Y69oc1EXP2Nnv5nyN6lM4NTmETu51zFlTeJX6mR4hnMVXOI2TkaPzcz2y
zByB4Vhk/6E3QsiwTnCOe+znGqSYvcbvtA4rsB2bsFzPJPe+6gx5fMLs4N61
ztCO56hv01+NJnkjF0bPxF+YucMZarAZCRzESz0vuzmLGc1cb+ZvvFd+wkXq
5fZVoBilmI9a8wrMLcJt91rOC/Cv/bmcj5meaSS/jUT0v9YXRxIGIwPT7H+f
P0Q2JmJMdDbzk/Wl4Yb5g7gFffBIT4vZbaynchPuY5973PM7/YEf8RrPkG//
Xc9S7Zl+4F1mn+NKuYHvql/TfwvZ0fvHp1Bl3mH9X2I3GnAWOfYf17dSf4I3
mbeF58s1fEdezTPkhVxhzjucI+fyefWZPEmezOVybx4mv8WN5nflgXJbLtN/
s2sIneVfOUv9BcflP/lj/We4lXyJL6rX87OYb5ZL9M/jn+UVnKm+k3+XK7lQ
fz5/L0/nr9XH8rdyFhdE7xkflZM5pp7Jp+UYv6f/YZcQDsjNfEz9Du+Qr3Oe
/v28VP6ck9Uv8Dr5FI/TX8Gz5So+qF7G0+QSHqV/CI+WR3GIvkV+V87jbP1p
PFTuz9XqSZwh/+c7GaL/CneUf+Jm+//idPk+D5KP8yvfyjlerz/Bf8t7uL/+
Ir4lL+KH9m/lB/Ia7iWne29O+P0z+Lr8yDv8HBfxCxrR1v7/ActSzAE=
          "]]},
        "0.07100000000000001`"],
       Annotation[#, 0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1HtM1WUcx/Gn9A9bSNwRSE1OIl5REa1FLpqbw4K8DWcyWVrzkhOal3IN
m4KXKbNFy8tk7oyhYabOC5ZoylzUyijXvGN/4DRqMV3YdC6Xvc4fb97n83yf
7/f3e86ew5AFlTOXPxFCqPDnm7gQxqWHMB5jMBZ5OG19FI/GcIzECJyyPoxz
8TxyMBSt1rM5gsEYgudw0vpAHoRMPIssfG19AGcgFelIw1fWkzkFCUhCIk5Y
b08O4XxWCN/xt7iTGsJdvJYSQgmuqb3OLfYW8GV5Ik/AC7giT+Ij/Z2XL8n5
PA4bMkLYiPWoRQ2Om9OkthdL8AU+R2V8CGXPhNBhzyxOMu9p77PPvHh+X71l
gDPiSxzF4ZjN28vNyE8L4UVMxDL7G5wj4nOUizxjB9cj3uw1zr8WK7EZNVih
p9Szf/AOxXzI7MdJITR6h77e4R31Tfrr0COv53J7P+T9Zm71DvWoRRS78FDP
QzMazOjlZjNv8WfyX1yhvtq+aryFSixGo3ll5lbgimet4KX42/4pXIL5zvQS
j0fU3ER9meiHUYhgjv3TeS4KMBWTY+9mfpy+VPxsfi4/wCB063lg9pM+z+Ye
dGG7Z/zme/oDv+I/3EOJ/Z3OUudMF/hjs1t5ndzOneo/xu4GCjzrTOy+YIt5
e/QfwCdoj91hFNl/UN8q/VGuMW8DL5br+ar8Ac+T3+VqcyZxkTyFT6vP51J5
Bq+WB/IEeQR3mJ/Aw+Q+XKX/l8QQ+ss3OF/9PmfKf/IS/S0c5HPcpt7M9zJD
2M0L9S/ia/JKzlHfxr/L67hcfwn/JL/Jx9Rf4bNyPpfF7lmCeyzHyRnqOXxU
zuA39N9W3yn38j71q7xVPs/F+nfwe3ITx6mf4bXyEX5VfzW/LW/hXepVPEde
yIX68/hluZAfx+4eT5OLuUBO5bFyNtfp78cR+V+/kzz93/NT8kXutf8Op8ld
nCsf5Ed+K638kf4o35U/5Wz9FXxJXsa37d/IN+U1nCWnuTeHfP+R2P8rudsd
/gdtuI4O9LH/f6NyxCE=
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03tsjXccx/Ffa1FZFaWlF/5oE2pjLl23iBJFWJeUTaViRCOduCatMJdI
mdW1GqJCJ5PlTLBulLXSRUSQLS5d1aVIS0lGZrZMtqxkIgR7nT/efZ/P73t5
nuf0ORklZYWlMSGElf580yeEmIQQRqWGUNM3hJyeIUScnZA7uofQnhzCLfRP
CuF6egjp3Can8j05jd8xM61fCNNRiDZzN/Rc4+P29NOz086+3CBfdd6qXuks
2VmrPUl8zVlv/lXuw4PtHW9fFk/gZnMtei5wnT099FTYkcBH5PPOm9R/5sNy
d+cXo/34Fxf1JnrGJu6U031ep+9zNPr8I+p9F3H8A291PtqOLZzLXZzH4g1M
xgeYhDFq+fw2d+UhfNMzDOWZ7n2ws9c9PAcHeQSPxGL1afJC/h7folRfkbMW
1yzkRPfxpvs9ZF8Cr1RvTPH/wRE04FjUnvkg1yLb7lF4D0v07/OsmT5HOM81
argaCXav9r2uxXJsQQWWmZni2k3uIZ+P2v2qdwj73UMX9zBPfbP5KjySv+DZ
etfwd3Zucw/V2IAI9uKZmWd27LOjk2vt/I13y39xsfoKfeWYi1IswH77iuwt
RptrLeNF0f+n/olcgDmeaTSPRMTeXuZSEYchyMQM/R/xTORgMsZG783+eHNJ
uGx/Fj/FADw089TuGJ+n86PoO489rnHX9/QHWvESj1Ggv8OzVHmmq7zD7pO8
Xj7HHeq/RN8N5LjWaa5HpX1fmz+MnTiHE8jTX2fuM/MRrrBvIy+Qq7ldXsWz
5MVcbs/7nCdP5FPqc3iK/DGvkPvzu/Jb3GJ/Tx4kx3KZ+SuJfjfyHc5W/49T
5T95oflGfp0Wwk98Vr2WH8tfcYn5+XxLXs4D1bfz7/J6nm2+gC/Jn/Bx9XF8
Rs7mouh71st7LMfLKeoDuUFO4anmH6h/KXfyIfV23iY3c775Gl4qH+B49dO8
Vq7n8ebL+VO5kveql/EMuYRzzQ/jMXIuv4q+e/yhnM85chIPlzO4ynwcZ8rP
/U6Gmb/A3eQb3Kn/b06W73GWXMcv/FZO8jrzEf5H3sUZ5ov5pryEH+jfxPfl
1ZwmJ3tvjvp/ZHKz/NA7/ARncRstiNX/P8Ue1dg=
          "]]},
        RowBox[{"-", "0.07100000000000001`"}]],
       Annotation[#, -0.07100000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV02tMVgUcx/E/vbBFphkUD1rJU8gqRstYLwSLRmu0sBthXiZr2qTLFgo5
qbCZXYFq0RRZrj1z1XClm7hWc4qsfLRSipqmRa4wkVpOF7ac62Lzc158+Z7f
+V/OOZzzpJcsq2vKiYjl/hy/PGIUrYURT2MlnsUz6MuLmJEfcXBaRDH/ou8Y
rnU8oD42MeJoco53ydc4f+vkiKUFEbO5kSudr8DP+pp5up7lXMQtvMk1jqiN
2LFTvtr5Xueu4h3JnPPD6sMcl0R0XxGxDuX29+jb697W83d6DiFl7mNzh/Qf
kIf4Nf2d6ECpuXb9u829ykN6vsEUc/PU52MBBp3bY/Yhx1NRiBRKzN9gfymu
R5sdd3OW+9S3ocyu/fbvdu5BtZ/4fq7DA6jHLD3Vdn3u+A4u1fNvym7+DJ/i
Q/ki3mlnP/Z4/hfNZXl6ch21Ss4mNWxwz2nOcJW+bu7CRP2t3sUqtOAVrEHz
pIg5rv2Fe6jhLZ733GURG937BXY/ov6y+U6ckJ/nhcm3wZvs7HB/XXgBGfTg
rJmzdmywY5x77TzGa+XfuUF9hb42PIwn0YiN9tXb25C8R9dq5sfwh/5qrsUi
zzSLb0LG3snmUpiQ/N+Qxlz99/I8lONOzE7uzf5cc3n4yv4SPoMrMWbmjN3h
uI5PYATrXOOI/9Ov+Bb/4TRq9Q97lk7PNMRv2L2dV8tZHlb/MvndoNy1+nkr
2u17x/wHeBNZfIIq/ZvNPWU+w2uS98yNchcfllfyAvlxbrPnFq6Sq3mH+iKe
I9/HK+RpfLN8HQ/aP4lnyDncZP7rKREXyz/yTPW/OCX/xo+a/4j/n+o75AH1
Xj4tv82LzS/l7+UWLlZ/nY/Lq3mh+VreL8/nbeq38S55Jtcn39mlEe/JuXKB
ejH3yQV8j/lR9fXyOL+vfpg75H1cY76bl8nvcq56P6+St/Lt5tt4idzOPepN
PFdezBXmy7hSruBzybfHd8k1XC7n8Y1yEXean8Bp+W+/kzLze/lC+QCP6z/J
+fIIl8ib+R+/le38nPkMn5Lf4iLzDXxQfoJH9b/ER+VWLpTzfTdbvI8075PH
fMN/YgA/YBA5+s8DM1bYlQ==
          "]]},
        RowBox[{"-", "0.14200000000000002`"}]],
       Annotation[#, -0.14200000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03lsz3ccx/GPiKOu0tJaZWkFWZZUtL/IoqToQtHNLWg0UuKctUVXWcqM
baIyUXHF9SM0ZCFq2UJdjSiJKrJMHCVzH8GW1UI6C2KP3x/PPr+v9/H5fn+/
fn89Z5ZMKG4RQhjpz6+JISxKDiEnPoQSHsoL+dMP1PCF6wWYi/mYh2z1u91C
+IrLUIqeXUO4rfYH7nUIoWXHEKJJIezEQGcW2rutvkPejm34RD1fvVH9qDN2
eZaI2mS1TJ7Cm9XGcbo8msdiDCLmR/BIDEMuhsf21C97hhKO90zF3Jkb1C7g
rHv9i1Xu/wOy7AzAGbVmHHG/5+ae4bB+NQ7htPqo2OfmlZzhzA/Vj+ME3pk/
xjU4in76K8zleu4q1+19H3Vyrf3hapXm27r+Rq8NBzvfquWb+Zr3m6/obg4r
EcUWNHdCgu+vRwhNvM8z3+cN8lMu0C81V47p+BKzsdt5E51bgKvut5Dn4m/z
OZyHaZ5lAPdD1Lmd7CWjFT5GGiaZH82TEcEwDIo9m/Pj7CXgovP78Cv0wCM7
r5z9HuPxDHew0T1u+e4f4ze8wQvkmW/0Wdb4TJd5rbNreLlcx4365+1fQcS9
TnI1VsfeL/s/YR3qcASDzR+wt9h+lFc47zueLVfyNbmMp8rzuNw5/XmwnMPH
9afxZ/IYLpVTOFP+iBuc35F7y4GL7F/qEkI7+SZn6L/kZPkJz7H/C79L8V5x
rf4+fiFv5UL7s/i6vIh76f/ID+XlnG8/jy/IU/hn/Ww+JWfwRPtvO4ewV46T
k/R78WE5iT+3/0B/s9zEVfrXuEKu51z7m7hY3sNx+id5qVzNQ+yX8wx5NW/R
L+JJciFn2U/ngXIWv429ezxCzuWInMB95VReY78Vp8mv/TbS7Z/j1vLv3GT+
T06U73Af+QD/57dSw8vsR/kveT2n2i/gK/J8fmD+e74rL+HucqL35qD/RxrX
y4+8w/+gFjfQgGD+f0favhM=
          "]]},
        RowBox[{"-", "0.21300000000000002`"}]],
       Annotation[#, -0.21300000000000002`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1Htoj1Ecx/Gj0GguG9tcagsjTGy5bQwrJjOkn1yamMuGPzCE2hjxByLk
OmllNLkPJWmWLOVOZIxCuSZyiT8Imdfvj/fez+d8z/ec59lznl+3eSWRpc1C
CH39eR8bQss2IRxODKEKS5NCWIK3xlsYr+kQQk7nEM5yVccQDiFbnm9OEeZi
eLsQ5vFxcwo5S57FmTybq43f6BrCUT5gjwoMVpuh1mifPfJu7EK68anGG4z/
ww5j2zHBWD7uGfuD1e5jp/XOuZe78n453fUoc3KQjZEYgX7Gh/BQDEJPewzm
Mv3j1K7wSXucwmn0sdYalCFWfq4+yfU61PifvJHveZ63fBm9jddbp7XaEdet
+KpcqzbGXkUJIRRjrVoBRzATj9RLeAE+t3XfnIcC84Zyf1R6tjadQkhEc/RG
CqaYP4GnIgOjMQz77B2jLw63rZ/KP9AFb/T8sPY/TMYHvMBuezyLvnPcx298
RZ75jfEhbPGsd3mbtS9yuVzPjerX9T9Ehr1q+Qw2We+g/mPYjnpciL4L80/o
W66/ktdbbwMXyTu4QV7J0+WFXGqdgTxCzuFL6gU8Xp7IK+TOnC734lvWj+Ue
clNcCIv13+FW8lMeoP6dE+V3XKz/PP/t4gxwnXo1f5UruFD/fH4sL+Pu6lv5
tVzOM/Tn8U15Gp9Vz+bL8gCORM9qe9+WHCMnqHfnGjmB8/W/Ut8rf+Ej6g28
Wb7BudFvg5fIVRyjXstl8hkeqb+U58ibeJ/6Yp4iF3Km/jTOkjP5j/kRHivn
coYcx/3kZN6ivzmnyD+d2zT917iF/IC/mP+R4+UXnCqf4F/O+kVeo7+SP8k7
OVn/TH4oL+JX5m/kl/IqTpLjo9+e95ES/Y2IflvO8DfU4QluoQn/AeKSq1g=

          "]]},
        RowBox[{"-", "0.28400000000000003`"}]],
       Annotation[#, -0.28400000000000003`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV03loz3Ecx/G3oyjn2NxbzfXHsC3SyGwKhZTIkSMK8cfkiD+QZDRDbqPm
PkqOTa7lzpFjrqi5ytSGEoUoNDIevz9ee35en9f79fn+vr/fZ+kzF4yb3yAi
2vpzpm3E/c4RZ/FAu4j9lN8qYl77iHfNIxq3iCiT5XWMKMd8zJPvT444wVfq
nsRl/HG8Kq/V28cPtJ7qnGk0hXL0jpi5ZL/aTENnLzV32N5Fe6/t7eX38ned
uw+3+jxbKEt3vDOqzPylHbL+OiW4W6eUNlnf1tuM63SKKUNvqN59nToqkmXp
rcWdOiW0yvqmXiGu0VlN3fX66d3Q+UHnZY3sJctSqNb8dfu9rftQjfyRvVr8
lxJRTxXe7y0/RN4l8Q6USXl0yHPPy9OsB5vpY76n9duWEd9lf2kMfaBq2uZZ
rxK/CT2mOvpCI8y/aONdPfshbnBmBa7gb+AL+R39p5Sd+O6xjIqcV6p/lDYm
3pPOUa75Y3oL9ffgSuetwln8JqziF+NEfg4udU5fzOXz8YJ8Mo7kR+MivgNm
8T2w0vnNsCtfnxRRoP8Am/IvMVP+DVP49zhb/zT+6RRxDa/Ij+AXfhdO15+J
z/gFmC5fj7X8CpykPwLv8RPwlHwQXuYzcaz+79YRB/kmfLI8Hcv5ZBylXyPf
wX/GQ/IqXMvfxWH623EefwCbyC/hMr4MByfuOM7gi7BEXoDj+OmYo5+BA/gc
/G1+LA7nh2E23xp78alYrN8I0/if7mOG/i1szD/Bz+Y/YhJfjd34Y/jLna/A
5fp78BO/GVP1p+BTfi7WmC/EN/wSbMcnuTfH/R5pif/LxJ3uEPGVrtBzqqR6
+g8uf6cM
          "]]},
        RowBox[{"-", "0.35500000000000004`"}]],
       Annotation[#, -0.35500000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV0ltoznEcx/HfpqYWNraZw1Ybbqa2JdpqW26mGDcjxMWUibKxyaERM2WW
bC62JTuUYxrNqG1plhxaM7LIyAUycjM1kpDJ8HouPr1/79/30PP/P//00sp1
FVEhhG9yMSGE10khtCeGkDs3hDaeg+1Y5e7ftBBanFslKy6ERuxRf+7+jyx1
LkoOYbU0zA5hGW/S04sj6mftaJEGdzfdPXH3W07xgfkh1GOGvcexS31I7Zdk
OufZmS919mbxWj038JF6s51H+X07anA/PyBHnKtloZ2H8Jr+u/q/S5S7QR6N
idPt1N9td4908SgsUy+XnbJbdkm1jHlHPXoGnN/ZWxOZ5a2zQhj2GydneDbz
f3EYY/UVRN6D+mWz2c59ZjqlNvJOzVyRerkn3ZFnNduhv9IztWG1/TVYyhtw
hO/FDXw7VtmzBPP5crylvglX8TW4hydjFl+EQ/bHYjqfnOlZzT/GqfwVZqp/
xUT+EbeZv4m/54VwB/vVL+FnfgZLzG/FF7wC09RP4nt+GDeaX4kP+XrsUs/D
2zwTi81PxIdwnsfwBPU0vM4TsMj8qHoTH8cL6iN4gg9ioflGLOfnMEa9Dw/y
TiwwX4VbeC02q5fhWl6COeYzMJfn4IT+YlzBCzGbx+FinoJ15qMxlf/wDWWY
f4BT+FMc1z+G8fwNLuAd+NO30Bv5Hs234Sd+GlPMb8ZnfAeO6j+Gb/k+TOLx
vpur/o9UHOQf5oTwRfrlpQzJpPwHngCQpQ==
          "]]},
        RowBox[{"-", "0.42600000000000005`"}]],
       Annotation[#, -0.42600000000000005`, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0nXl8U8X39wMiLiCCKMoiKCAgi18VRWQ7BUWQHQFBFFFZFFBAQBYVKKIg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    "], {{{}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], 
         LineBox[{3320, 4033, 3319, 4338, 4337, 4725, 4724, 4004, 3663, 5053, 
          3662, 4034, 3324, 4983, 3669, 4005, 3671, 5054, 3670, 4037, 4038, 
          3678, 5056, 3677, 3461, 3330, 4987, 3679, 3534, 4518, 3535, 4519, 
          4735, 4734, 4041, 3339, 4995, 3692, 4010, 3464, 3340, 4996, 3693, 
          3540, 4531, 3541, 4244, 3703, 4279, 4046, 4047, 3711, 3354}], 
         LineBox[{3427, 5028, 3816, 4027, 3498, 4384, 4383, 4278, 3639, 4225, 
          4226, 3826, 5072, 3825, 4071, 4072, 3837, 4866, 3836, 3504, 3437, 
          4703, 3648, 4466, 4467, 3843, 5075, 3842, 4075, 3445, 5042, 3848, 
          4030, 3507, 3446, 5043, 3849, 3652, 3856, 5077, 3855, 3509, 3450, 
          5047, 3857, 3657, 4873, 4874, 4872, 3858, 3510, 3456}], 
         LineBox[{3719, 5098, 3720, 4104, 4103, 4342, 4341, 4213, 4212, 4211, 
          3545, 3702, 5061, 3701, 4099, 4100, 3918, 5003, 3349, 4096, 3338, 
          4393, 4392, 4391, 3537, 3685, 5058, 3684, 4091, 3909, 5083, 3910, 
          4089, 4088, 3329, 4986, 3906, 3676, 3905, 4985, 3328, 4084, 3323, 
          4982, 3901, 3668, 4904, 4905, 4339, 4340, 3322, 4083, 4186}], 
         LineBox[{3808, 5070, 3809, 4145, 4144, 4848, 3807, 4303, 5097, 3636, 
          4687, 3637, 3949, 5029, 3428, 4148, 4149, 3955, 4956, 3954, 4957, 
          4152, 4861, 4862, 4694, 3643, 4693, 3644, 3962, 5038, 3438, 4157, 
          3443, 5040, 3967, 4160, 4159, 3968, 5088, 3969, 3844, 3975, 5105, 
          3974, 4164, 3976, 5090, 3977, 3850, 4968, 4969, 4965, 4967, 4966, 
          4168, 4169}]},
        "0.22999999999999998`"],
       Annotation[#, 0.22999999999999998`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1XtM1mUYxvHHUsAUjGzW6i9QpDLbtK3kkCdSsNMC2/KQJiDgKWCFAnEw
SNuibG62/McywczlmqOsLWdzTSs03Dyt06bN0kpWgpI2KaXPs/64/D7Xdd/3
8/7ex+f3klZSVVQ5JITQ4Z+3U0NIo9pbQmhNDqEOC1NCeEfWxadjvexFKpJv
5Q/Jx2KDrJHmypvwXdk2GkcZo0N4Wd947BsVQg5lWufiJbpI10eG8J7ebn3b
8QhOk/+pbypepis0nepkg/r/th6qr4nPsy712Q9jMz9MPmCdhC18gfVK9TnY
yg+XD1qPxlf5x6xfUH8c2/hb5TfcHEKN7D6+SH4vFuJETFDb4Hu+Qcc97xn9
x/BnfErPRj3p1iP0vWKPXP5peRbmUDalqrWb/9Xcb3SOztPvtFP+lrk+6377
9OJfeBHb5YfMd+B12XTZDJpJD/FJZhPUCvg59AjlycfIk+WFfBHNpXx5mvwO
+Tx+Pn1l3W3/L3G82jjMwAzMxKN4ArMom6ZQLuXQWfrF91xk3232OKX3M34v
LZa1y07LJsez1ns/PoEDdDWehb7l+j7Q16u2j/+cVsh2xfsjy6YBOsZfxd1q
q+hD/h9+GgUzx+M9se5Ue55280PkM2QJeDLeE/xIrYY6+cR45+gPqpd9Kkux
TsTV8RzNLvGcQ/kbaY+e/erJNIwfiQl4AdeqVemtplVUSc9Rs3wlFthrOebj
CmySL8PZfDnOwor4XsmXYhkVUymVUIM83x2qw098ZpvPnM3PolJ+KZVQMdXE
+2G/EXr28LVmZurLo/WyM7J18Wz4VuyL72x8L8004098E05Vb8BevhH71Wvx
NL8m3u/4zuCFeF7Yo16Np/gqfED9vKzSeoeszHNMlj2LFZ6xnMpogp4f6Iie
73Ginu/ieZnbGt8l/ZmywninaLGZRXS3c7mLHpU9wy+kBXSn2fR45vIt5vfx
o+g2WQrejh/H88EkHI6JeBN2xvfMHgH/xUF7XMPtfD9e4S/hZdws22z/SdZZ
sin0IPXwPfH5cGP8ztaT6CSdoLGyDfG3yPwmn9mGb8Z3SXbUd36J77JvC07Q
fw/9qLYXx8S5+Hut9pqZenwdv5V1m63iD5qtxl18Ja7nk9UP8xX8AX4Z7uTL
cW38jVPv4ov5L+Jdwvf5JdgY77/61/wCfj+/EHfw6+TzrOfT4fj/4xm/wQ68
5nf7gJ6D9KT66vgOy1rU13jmLan//x36DzVa57A=
          "]]},
        "0.184`"],
       Annotation[#, 0.184, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1FdszlEYx/HTBI3ZWtFeWAkiRa2oVb1BxApqFKUEsam6slOzlMQIiUQI
kdCiVY2ZujQuxGyMRCVCSmre2Ptz4uLb73me83vO+b/5v307zsnPXp4QQsj0
Z1GzEDa2CGEzNuFWUghJTUMoax5COU7jgfoMN+NtLUMoMHNGbod8MbZhO4pw
Wz9F7oL8RZzHY/VaMzfMruFidRvsUq9TP7Wukqvhq7zHObtx31md9PbK7ZQr
U5ejs94+vV16tdYP4x14hOPmqmX66B+UOShToT6g7otPMp8xCIORiQH6fzlB
digPw3Bk6TdxXkP90eoxGBv39VP1k/UnqSfjiN5ItE8OoQPmuXOY++fycO6k
d91MN3RHGrqYn2//sP1ss11l7scz1Yv1J/DEuId0e7VmXiErniE3hHvwUtnj
MrPjZ5SbzyfVb+1lYLnny1Gf1p/K/WUKzEyzzsV0vJP7jHfWnzjb2QN4hdk8
vTKzs3iQ2VVmZ1uv5kz1YOTHs81MQw6GmJ3OK/RnxGdW53IeZqJAv878G3yJ
3yfnLdErj+/C/kRMijn3B3sJ+Cr3DQPxHiV6C+J3Ez9lx2MC8s384g8ypTIL
7de655X6rvo1L5NZit7WY2RnyJzAU/UHdQ1/jO+JX5h9GK1+xC95ntm56G+d
Ee82ewx37D9Q3+Vn5mpwTd0eHdAWqfbacRW35ibckhtzK67kRK7P9bkeN+AK
/unO3+75xj/wPf6f2mvjHaQg132H4juWaaXuxfuRIdcPdfE55fc56yjv5WS5
PTzF7H6zXWUa6XXkXmZ6Ih3P1Rvkbsqsj88mMy5+RvV4fmL/ktwVXEaqOkGm
BW+XGS2z0twoLlI316+Uq+azfA4VSFb/kUnkQrkR1r/xC93URfpp3EX2nvUp
XuzcrT7PFnRWl6IEhfH3SaY06f9v3T9X76zo
          "]]},
        "0.138`"],
       Annotation[#, 0.138, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl09tPz3Ecx/FPuGHr8KvNUKIZZtOBDlwgig5OGZPTDW02tPE/OFxRDXMo
o2yMNi6YCzc5VHIqsrkgqxxCxIVTtenC4zsXzz0/79fn/fl8vr/v5/vLqDqw
cX9cCGEUd2IhzMeJ5BAOJYRQmxJCWnwI92S9fJdbcR/T1e38AG14iA6kyx/z
EzxCJ55ihvwZP0cX+tTdPJPrnHPYef3GmbJs9THjHJ6YFMKmxBAWGOdiIY7o
XcSLkYcC5CNRb525856/ERcwaJ9ce+bhpHE2UvQVcIM12+3daM1kWRNfs+Yi
X+cxPUutW4ZCLFFPkI83X6IuxTlZEWLyZKyXVWADSuRpsin6K9Vbonelble3
chu/cf4sfqHu4m7u5CZrK5Cnzscua6vw1vgdetGPvqi2xw696dG9YFD2Gbfl
3/g7vmAIXzEgr9K/2ngNypHjzI8YwKh6BNX2KuNSDKPZmj+8L/ptvAq/cVX+
i/fKV3IxfuKK/Afvie7UvkXG73nIe//AWSiUrcDy6DdG9+MetnGTnmrP2aNn
s3ElUu2Thq3m50XfBU9TT8U682sxV3+mvbKRhQ7M1ltqrgwl2G3fcm7Rm2Eu
Zn0SdsqL5cnyadbFOJVvcRJP4nhOREKUY1z0PSBEY/NxfJlH7Dfq+YZ5hJvl
DfK/zhvDTWfdQL2sPvqO9bxUX9L32poevMIn+VlZfXQn1o1ijqwWNdaekp/B
6eheZQdlNdG7Mj6K4/KW2P//9j/1n4tR
          "]]},
        "0.092`"],
       Annotation[#, 0.092, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl00lszGEYx/FXgks3Srm4iMbSqqDpoVy4jqUilFhCXDjVjcSFG7V0UNNq
CRHUzbTaoZ0SbR1Qexe0nWqtPVgOwoGTzz8O3/ye5/cs7zv/NzN3z/6NVZNC
CO9QkxtCY34IF3EBfXkhLM5Rmx7CGEYwigyK+YkZIdSa6ZgWwgHUIYF2nFEv
wl/xH3wz8zWC953+wHl+PdIYdFa52hT+ZDQ7P4lWfhum8lrkTc7rdW61meO4
a6aSnlaP44S4VP8IfRCdRRvkJ+kp1GAL7qnF6VaUqe+gV+gqu1cj5axU9Nto
GrfRjjuYo7eTPqa9KBTPi3z9MbNrUOAus7DWOfP1bKIt9BaasVDvazNvMBB9
a/kgXYpl2GV2vT1XzVXQIjN73W+DeF/0LvJibFMfk4+j345L6pt51/DWnuUo
xW77huh1/oC+YXEGI1gkb7UrhTZURm9u31O1Z3iCF3iOI+qHkcRNxPReRo8d
99W70YXZ8iE7OujD6I3pI9qPPlRE7+dO1TgmLtE7THfSV7SAvqQzaZbfFIu+
uXOO0nJeJ69HvkLcTdfZn+Z1iRvsbEQWsrGSV6hWRuvxRfwZh+z6TX/hJ7/J
vW9E3wgJ+/LVR3FWbUJPRnwQE/Jz6nW8JeKPzviEcXzAe5Twc/UuoLXRm7hn
jjwb2xHnJ/P+/+f+AUsWjSk=
          "]]},
        "0.046`"],
       Annotation[#, 0.046, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0ztsjWEcgPG3DCwo4rK0ek53t0hZEHsZJE3agUQTLPS0ujC0S7UkvSRo
Tzm3JgzauiUMKtrTI1G9YRTp4BIGElpsBur3xvDk+d/ey/d95yQaU0ebykII
U/i2JoTz5SEUMbohhNzaEJbFYX0IWf61MYTvZjLiRc5xmV6Bl+R5/sFDvFJ9
BfL2uGOvn+oX1N/hPT7iAzbhgH3r14Wwnxv4IBese6H3gJ/zfZ7ivfoFjJkp
2rNBbZKfyPep96HVebXmO53fhZRev/ol8WUct6Zb3oOMtYewU7xgbhfvQJ2Z
PrO92C1P6PWIu7FdXiXfzJXmxpx1mh9zBb9VP8ct8X3E+6lXx7560d2eyce5
xBO8oH7EHbba+565pPpdTvCc/jzP8hy/Vn+FdlTFd8uz/JLn0YZt8lGe5mnr
ZniGK3lYvYJHeIvzDjs3KW5GCk1odJ+v8Xdg7gufkJ+N3zx+ZwzJb+EmjsV9
+I330cslVItreMD+abTI8/JB8XWcsiYjz+GzXpaXkMYn+WBci5Pm+s1fi+vk
zXpXxVewR56UXzTTEZ9dPOK+jzznKv3VcY1neyi/rT5pZphL/Ft9grPyM/wH
f5FWf8rjuFH+///wD8HBfGM=
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0slvjXEUh/FjSdyh97bV9paqmfoLbIikG0VoLUUpC2NFSQ2tqei200qQ
VFskhqUqsaHFP0CrW3OwkyBsfH6xePKc8z3n/b3DvfVtR5vb50TEBP5kIrry
Ec8xifvZiJv8V97N7/AeO3MRH7gaNXhgb25ZxDyss5vhDVwoRhRRhhrXtNht
xmOzJfqSfCkvx4x6WbL5NDoKEU3OfSq/ou/FEfWgnfPOv4CL2C3fgz55G7fi
svwSejAkb3fdd/kQXrn3NflL/iKfla2y85br9IuwUl/LK3gh7zDbgl/qrVyS
l6d3QjXq9Tec14ics7MY9+wz8um0qy/ijmsr5be4gm9zQT7G5foRLvJvZ46q
P+KH+hPvcs5GdSt/Tt+ec/o83uhf4ysO6q97jnHzA/r9WKzezE2YMKtK7ylb
wHW8FrPqBh6wM2Wnn1/wIH/DYfMB827PexbnsE/+xM5e7tKfwWn02zuUfns+
5nf8qd7kvTrNTuIUVqdvLl+TvjEq1LVcyY+cuc2Z21Elmy/LIoP1ZiX5cfUJ
dGA43S+9O+65z0P9VXvP9Hf1w/n//+1/6aVfFg==
          "]]},
        RowBox[{"-", "0.046`"}]],
       Annotation[#, -0.046, "Tooltip"]& ], {}, {}}}], 
   {GrayLevel[0], AbsoluteThickness[3], LineBox[{{0, 0}, {1., 0}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}, {1.75, 
      FormBox["1.75`", TraditionalForm]}, {2., 
      FormBox["2.`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.1536749816983884`, 
   2.1536749816983884`}, {-1.3536749816983884`, 1.3536749816983884`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 419},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[47]=",
 CellID->305833234]
}, Open  ]],

Cell[TextData[{
 "Note that there is a discontinuity across the panel of the velocity \
component tangent to the panel.  Also note that the equipotential line along \
the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " axis is thickened compared to the others.  This is suggestive of a branch \
cut."
}], "Text",
 CellID->643240816]
}, Open  ]],

Cell[CellGroupData[{

Cell["Doublet panel of linearly varying strength", "Subsection",
 CellID->485419430],

Cell[TextData[{
 "Set ",
 Cell[BoxData[
  RowBox[{
   StyleBox["\[Delta]", "TR"], "=", "1"}]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  RowBox[{
   StyleBox[
    SubscriptBox[
     StyleBox["a", "TI"], "\[Mu]"], "TI"], "=", 
   RowBox[{"1", "/", "2"}]}]], "InlineFormula"],
 " and  ",
 Cell[BoxData[
  RowBox[{
   StyleBox[
    SubscriptBox[
     StyleBox["b", "TI"], "\[Mu]"], "TI"], "=", "1"}]], "InlineFormula"],
 "  to provide a unit integrated panel strength, ",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", 
     StyleBox["\[Delta]", "TR"]], 
    RowBox[{
     StyleBox["\[Mu]", "TR"], 
     RowBox[{"(", 
      StyleBox["x", "TI"], ")"}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]}]}], "=", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox["\[Integral]", "0", 
      StyleBox["\[Delta]", "TR"]], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox[
         StyleBox["a", "TI"], 
         StyleBox["\[Mu]", "TR"]], "+", 
        RowBox[{
         SubscriptBox[
          StyleBox["b", "TI"], 
          StyleBox["\[Mu]", "TR"]], " ", 
         StyleBox["x", "TI"]}]}], ")"}], 
      RowBox[{"\[DifferentialD]", 
       StyleBox["x", "TI"]}]}]}], "=", "1"}]}]], "InlineFormula"],
 ".  ",
 " Nonlinear scaling is used for magnitudes of vectors to enhance visibility \
in the far field."
}], "Text",
 CellID->977679975],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"u\[Mu]Plot", ",", "w\[Mu]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"u\[Mu]", ",", "w\[Mu]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a\[Mu]", "\[Rule]", "0.5"}], ",", 
      RowBox[{"b\[Mu]", "->", "1."}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1."}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Phi]\[Mu]Plot", ",", "\[Psi]\[Mu]Plot"}], "}"}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Phi]\[Mu]", ",", "\[Psi]\[Mu]"}], "}"}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a\[Mu]", "\[Rule]", "0.5"}], ",", 
      RowBox[{"b\[Mu]", "->", "1."}], ",", 
      RowBox[{"\[Delta]", "\[Rule]", "1."}]}], "}"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gV\[Mu]", "=", 
   RowBox[{"VectorPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"u\[Mu]Plot", ",", "w\[Mu]Plot"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
     RowBox[{"RegionFunction", "\[Rule]", 
      RowBox[{"Function", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"x", ",", "z"}], "}"}], ",", 
        RowBox[{
         RowBox[{"0.005", "<", 
          RowBox[{
           RowBox[{"x", "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}], " ", "&&", " ", 
         RowBox[{"0.005", "<", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"x", "-", "1"}], ")"}], "^", "2"}], "+", 
           RowBox[{"z", "^", "2"}]}]}]}]}], "]"}]}], ",", 
     RowBox[{"VectorColorFunction", "->", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"RGBColor", "[", 
         RowBox[{"0", ",", "1", ",", "0"}], "]"}], "&"}], ")"}]}], ",", 
     RowBox[{"VectorPoints", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"18", ",", "18"}], "}"}]}], ",", 
     RowBox[{"VectorScale", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{"Large", ",", "Automatic", ",", 
        RowBox[{
         RowBox[{"Sqrt", "[", "#5", "]"}], "&"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Phi]\[Mu]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Phi]\[Mu]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Red"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"g\[Psi]\[Mu]", "=", 
   RowBox[{"ContourPlot", "[", 
    RowBox[{"\[Psi]\[Mu]Plot", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "xMin", ",", "xMax"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"z", ",", "zMin", ",", "zMax"}], "}"}], ",", 
     RowBox[{"Contours", "\[Rule]", "13"}], ",", 
     RowBox[{"ContourShading", "\[Rule]", "None"}], ",", 
     RowBox[{"ContourStyle", "\[Rule]", "Blue"}], ",", 
     RowBox[{"PlotPoints", "\[Rule]", "25"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gPanel", "=", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"GrayLevel", "[", "0", "]"}], ",", 
      RowBox[{"AbsoluteThickness", "[", "3", "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"Transpose", "[", 
        RowBox[{"Components", "[", 
         RowBox[{"rP", "/.", 
          RowBox[{"\[Delta]", "\[Rule]", "1.0"}]}], "]"}], "]"}], "]"}]}], 
     "}"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{
  "gV\[Mu]", ",", "g\[Phi]\[Mu]", ",", "g\[Psi]\[Mu]", ",", "gPanel", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"FrameLabel", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<x\>\"", ",", "\"\<z\>\""}], "}"}]}], ",", 
   RowBox[{"FrameTicks", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.0"}], ",", "2.0", ",", "0.25"}], "]"}], "]"}], ",", 
      RowBox[{"Chop", "[", 
       RowBox[{"Range", "[", 
        RowBox[{
         RowBox[{"-", "1.2"}], ",", "1.2", ",", "0.2"}], "]"}], "]"}], ",", 
      "None", ",", "None"}], "}"}]}], ",", 
   RowBox[{"ImageSize", "\[Rule]", 
    RowBox[{"7", " ", "72"}]}], ",", 
   RowBox[{"RotateLabel", "\[Rule]", "False"}]}], "]"}]}], "Input",
 CellOpen->False,
 CellLabel->"In[48]:=",
 CellID->115028120],

Cell[BoxData[
 GraphicsBox[{{{}, 
    {Hue[0.67, 0.6, 0.6], 
     {Arrowheads[{{0.007006085956855114, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0231716916247224`, -1.1949768083818337`}, \
{-0.9768283083752777, -1.2050231916181664`}}]}}, 
     {Arrowheads[{{0.0073399671617991975`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.023517274539664, -1.0508263615691498`}, \
{-0.976482725460336, -1.0668206972543797`}}]}}, 
     {Arrowheads[{{0.007681217422131775, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0233662212549006`, -0.9062565771170494}, \
{-0.9766337787450994, -0.9290375405300095}}]}}, 
     {Arrowheads[{{0.008022208840003176, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0225308985294257`, -0.7613243586569295}, \
{-0.9774691014705743, -0.791616817813659}}]}}, 
     {Arrowheads[{{0.008351654359839984, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.020807246706861, -0.6161659308748674}, \
{-0.979192753293139, -0.6544223044192506}}]}}, 
     {Arrowheads[{{0.00865399079833627, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0180113167986624`, -0.47102425296533185`}, \
{-0.9819886832013375, -0.5172110411523155}}]}}, 
     {Arrowheads[{{0.008909628880796949, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.014043390390729, -0.3262593942422799}, \
{-0.9859566096092708, -0.379622958698897}}]}}, 
     {Arrowheads[{{0.009096939943481954, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0089706478108351`, -0.1823149200989676}, \
{-0.9910293521891648, -0.24121449166573866`}}]}}, 
     {Arrowheads[{{0.009196476448135895, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-1.0030898059073599`, -0.03961939056510577}, \
{-0.9969101940926403, -0.10155708002312991`}}]}}, 
     {Arrowheads[{{0.009196476448135895, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9969101940926403, 
        0.10155708002312952`}, {-1.0030898059073599`, 
        0.039619390565105383`}}]}}, 
     {Arrowheads[{{0.009096939943481963, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9910293521891648, 
        0.24121449166573827`}, {-1.0089706478108353`, 
        0.18231492009896721`}}]}}, 
     {Arrowheads[{{0.008909628880796949, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9859566096092708, 
        0.37962295869889656`}, {-1.014043390390729, 0.32625939424227945`}}]}}, 
     {Arrowheads[{{0.008653990798336272, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9819886832013376, 
        0.5172110411523152}, {-1.0180113167986624`, 0.4710242529653314}}]}}, 
     {Arrowheads[{{0.008351654359839948, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9791927532931392, 
        0.6544223044192502}, {-1.0208072467068607`, 0.6161659308748669}}]}}, 
     {Arrowheads[{{0.008022208840003217, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9774691014705742, 
        0.7916168178136586}, {-1.022530898529426, 0.761324358656929}}]}}, 
     {Arrowheads[{{0.0076812174221317815`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9766337787450994, 
        0.9290375405300091}, {-1.0233662212549006`, 0.9062565771170489}}]}}, 
     {Arrowheads[{{0.007339967161799229, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.976482725460336, 
        1.0668206972543792`}, {-1.0235172745396641`, 1.0508263615691493`}}]}}, 
     {Arrowheads[{{0.007006085956855081, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.9768283083752777, 
        1.2050231916181657`}, {-1.0231716916247222`, 1.194976808381833}}]}}, 
     {Arrowheads[{{0.007500748201117313, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8488155660144169, -1.1977740823014618`}, \
{-0.7982432575149948, -1.2022259176985381`}}]}}, 
     {Arrowheads[{{0.007916189900583657, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8497692097358438, -1.0534225881643504`}, \
{-0.797289613793568, -1.064224470659179}}]}}, 
     {Arrowheads[{{0.008352546623810837, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8502561450807914, -0.9084448053632134}, \
{-0.7968026784486202, -0.9268493122838455}}]}}, 
     {Arrowheads[{{0.0088026457589213, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8500159573998738, -0.7628362461740803}, \
{-0.797042866129538, -0.7901049302965081}}]}}, 
     {Arrowheads[{{0.009253616917066613, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8487196095970849, -0.6166891672818526}, \
{-0.7983392139323268, -0.6538990680122654}}]}}, 
     {Arrowheads[{{0.009684442341790622, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8460012051279963, -0.47026075079186047`}, \
{-0.8010576184014154, -0.517974543325787}}]}}, 
     {Arrowheads[{{0.010064110746727563`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8415533680607088, -0.3240423633544476}, \
{-0.8055054554687028, -0.3818399895867293}}]}}, 
     {Arrowheads[{{0.010352963975760361`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8353069440866947, -0.17876717550034293`}, \
{-0.8117518794427171, -0.24476223626436333`}}]}}, 
     {Arrowheads[{{0.010510657131267259`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8276383604198423, -0.035256325358817876`}, \
{-0.8194204631095694, -0.10592014522941781`}}]}}, 
     {Arrowheads[{{0.010510657131267254`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8194204631095694, 
        0.10592014522941741`}, {-0.8276383604198422, 
        0.035256325358817495`}}]}}, 
     {Arrowheads[{{0.010352963975760361`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.811751879442717, 
        0.24476223626436294`}, {-0.8353069440866946, 
        0.17876717550034255`}}]}}, 
     {Arrowheads[{{0.010064110746727563`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8055054554687029, 
        0.38183998958672893`}, {-0.8415533680607089, 0.3240423633544472}}]}}, 
     {Arrowheads[{{0.009684442341790634, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.8010576184014154, 
        0.5179745433257866}, {-0.8460012051279963, 0.47026075079186}}]}}, 
     {Arrowheads[{{0.009253616917066613, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7983392139323268, 
        0.6538990680122649}, {-0.8487196095970849, 0.6166891672818522}}]}}, 
     {Arrowheads[{{0.0088026457589213, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7970428661295379, 
        0.7901049302965077}, {-0.8500159573998737, 0.76283624617408}}]}}, 
     {Arrowheads[{{0.008352546623810837, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7968026784486202, 
        0.9268493122838452}, {-0.8502561450807914, 0.908444805363213}}]}}, 
     {Arrowheads[{{0.007916189900583657, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7972896137935679, 
        1.0642244706591786`}, {-0.8497692097358437, 1.05342258816435}}]}}, 
     {Arrowheads[{{0.007500748201117313, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.7982432575149948, 
        1.2022259176985377`}, {-0.8488155660144169, 1.1977740823014613`}}]}}, 
     {Arrowheads[{{0.008025206245612558, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6741747593762378, -1.2015253082320987`}, \
{-0.6199428876825855, -1.1984746917679012`}}]}}, 
     {Arrowheads[{{0.008540151062896743, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6759128539086937, -1.0571682289804198`}, \
{-0.6182047931501297, -1.0604788298431096`}}]}}, 
     {Arrowheads[{{0.009097876597824343, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6773173535468907, -0.911956808718522}, \
{-0.6168002935119328, -0.9233373089285369}}]}}, 
     {Arrowheads[{{0.00969527190114388, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.67807440083792, -0.7657668231435139}, \
{-0.6160432462209032, -0.7871743533270745}}]}}, 
     {Arrowheads[{{0.010322200653426799`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.677716654283233, -0.6185501534055535}, \
{-0.6164009927755904, -0.6520380818885645}}]}}, 
     {Arrowheads[{{0.01095545921089755, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6756004177194082, -0.4704542297442611}, \
{-0.6185172293394151, -0.5177810643733863}}]}}, 
     {Arrowheads[{{0.011550000140529156`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6709753391381158, -0.3220247984727213}, \
{-0.6231423079207075, -0.3838575544684556}}]}}, 
     {Arrowheads[{{0.012031998483918541`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6632965237209923, -0.1744238910103314}, \
{-0.6308211233378311, -0.2491055207543748}}]}}, 
     {Arrowheads[{{0.012308486731555051`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6528563070865913, -0.02933943216569508}, \
{-0.6412613399722322, -0.1118370384225406}}]}}, 
     {Arrowheads[{{0.012308486731555055`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6412613399722322, 
        0.11183703842254022`}, {-0.6528563070865913, 
        0.029339432165694686`}}]}}, 
     {Arrowheads[{{0.012031998483918541`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6308211233378311, 
        0.24910552075437448`}, {-0.6632965237209922, 0.174423891010331}}]}}, 
     {Arrowheads[{{0.011550000140529156`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6231423079207076, 
        0.3838575544684552}, {-0.6709753391381159, 0.3220247984727209}}]}}, 
     {Arrowheads[{{0.010955459210897545`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6185172293394152, 
        0.5177810643733859}, {-0.6756004177194083, 0.4704542297442607}}]}}, 
     {Arrowheads[{{0.010322200653426808`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6164009927755904, 
        0.652038081888564}, {-0.677716654283233, 0.618550153405553}}]}}, 
     {Arrowheads[{{0.009695271901143885, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6160432462209033, 
        0.7871743533270742}, {-0.6780744008379201, 0.7657668231435134}}]}}, 
     {Arrowheads[{{0.009097876597824343, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6168002935119327, 
        0.9233373089285365}, {-0.6773173535468906, 0.9119568087185216}}]}}, 
     {Arrowheads[{{0.008540151062896729, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6182047931501297, 
        1.0604788298431091`}, {-0.6759128539086936, 1.0571682289804192`}}]}}, 
     {Arrowheads[{{0.008025206245612577, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.6199428876825855, 
        1.1984746917679008`}, {-0.6741747593762379, 1.2015253082320982`}}]}}, 
     {Arrowheads[{{0.008565816631221584, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.49887743615106495`, -1.2063280600069013`}, \
{-0.4422990344371701, -1.1936719399930984`}}]}}, 
     {Arrowheads[{{0.009196978732669614, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.5015182947260126, -1.0622953411538643`}, \
{-0.4396581758622225, -1.055351717669665}}]}}, 
     {Arrowheads[{{0.009902846916367303, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.5040987198894635, -0.917229155303492}, \
{-0.4370777506987716, -0.9180649623435669}}]}}, 
     {Arrowheads[{{0.010690969293289007`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.5063272639533714, -0.770837480950108}, \
{-0.4348492066348637, -0.7821036955204806}}]}}, 
     {Arrowheads[{{0.011564972963119645`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.5076796151144062, -0.6228035036871528}, \
{-0.4334968554738289, -0.6477847316069653}}]}}, 
     {Arrowheads[{{0.012515744954074016`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.5072414794861432, -0.47289174699905223`}, \
{-0.4339349911020919, -0.5153435471185952}}]}}, 
     {Arrowheads[{{0.013499259993655723`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.5035245737251036, -0.32128319373562375`}, \
{-0.4376518968631314, -0.3845991592055531}}]}}, 
     {Arrowheads[{{0.014392889503343288`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4945975117001496, -0.16938491223938554`}, \
{-0.44657895888808546`, -0.2541444995253207}}]}}, 
     {Arrowheads[{{0.014960972808844782`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.47959036603795024`, -0.020764213789599556`}, \
{-0.4615861045502848, -0.12041225679863612`}}]}}, 
     {Arrowheads[{{0.014960972808844779`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.46158610455028487`, 
        0.12041225679863574`}, {-0.4795903660379502, 
        0.020764213789599174`}}]}}, 
     {Arrowheads[{{0.01439288950334329, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4465789588880855, 
        0.25414449952532037`}, {-0.49459751170014954`, 
        0.16938491223938512`}}]}}, 
     {Arrowheads[{{0.013499259993655723`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.43765189686313144`, 
        0.3845991592055527}, {-0.5035245737251036, 0.3212831937356233}}]}}, 
     {Arrowheads[{{0.012515744954074022`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4339349911020919, 
        0.5153435471185949}, {-0.5072414794861432, 0.47289174699905184`}}]}}, 
     {Arrowheads[{{0.01156497296311965, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4334968554738289, 
        0.6477847316069648}, {-0.5076796151144062, 0.6228035036871522}}]}}, 
     {Arrowheads[{{0.010690969293289007`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4348492066348637, 
        0.7821036955204801}, {-0.5063272639533714, 0.7708374809501074}}]}}, 
     {Arrowheads[{{0.009902846916367313, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4370777506987715, 
        0.9180649623435665}, {-0.5040987198894635, 0.9172291553034916}}]}}, 
     {Arrowheads[{{0.009196978732669623, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.43965817586222244`, 
        1.0553517176696647`}, {-0.5015182947260126, 1.062295341153864}}]}}, 
     {Arrowheads[{{0.008565816631221577, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.4422990344371701, 
        1.1936719399930982`}, {-0.49887743615106495`, 
        1.2063280600069008`}}]}}, 
     {Arrowheads[{{0.009101344010657443, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.32243387184887096`, -1.2121190640675896`}, \
{-0.2658014222687758, -1.1878809359324103`}}]}}, 
     {Arrowheads[{{0.00985977208960438, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3259398674171582, -1.068859670888223}, \
{-0.26229542670048855`, -1.0487873879353065`}}]}}, 
     {Arrowheads[{{0.010733582591328943`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3297805572558835, -0.9245479940423578}, \
{-0.25845473686176323`, -0.9107461236047011}}]}}, 
     {Arrowheads[{{0.0117488049062129, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3338121487714561, -0.7787546003128184}, \
{-0.25442314534619065`, -0.77418657615777}}]}}, 
     {Arrowheads[{{0.012940220288670586`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.33768566177504605`, -0.6308693187951443}, \
{-0.25054963234260064`, -0.6397189164989738}}]}}, 
     {Arrowheads[{{0.014352681181793435`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3405998760408944, -0.48002305715016824`}, \
{-0.24763541807675232`, -0.5082122369674792}}]}}, 
     {Arrowheads[{{0.016027151032747714`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.34065617365925654`, -0.3250840046827664}, \
{-0.24757912045839017`, -0.3807983482584105}}]}}, 
     {Arrowheads[{{0.017900660423818403`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.333309576383729, -0.1655712973825998}, \
{-0.2549257177339177, -0.2579581143821065}}]}}, 
     {Arrowheads[{{0.019428217492058197`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.3109124370716635, -0.007020708118457106}, \
{-0.27732285704598325`, -0.13415576246977856`}}]}}, 
     {Arrowheads[{{0.019428217492058215`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2773228570459833, 
        0.13415576246977823`}, {-0.31091243707166344`, 
        0.0070207081184566755`}}]}}, 
     {Arrowheads[{{0.017900660423818413`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.25492571773391776`, 
        0.25795811438210614`}, {-0.333309576383729, 0.16557129738259935`}}]}}, 
     {Arrowheads[{{0.016027151032747728`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.24757912045839017`, 
        0.38079834825841014`}, {-0.3406561736592566, 
        0.32508400468276594`}}]}}, 
     {Arrowheads[{{0.014352681181793439`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.24763541807675232`, 
        0.5082122369674789}, {-0.3405998760408944, 0.4800230571501678}}]}}, 
     {Arrowheads[{{0.012940220288670586`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2505496323426007, 
        0.6397189164989734}, {-0.3376856617750461, 0.6308693187951439}}]}}, 
     {Arrowheads[{{0.011748804906212908`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.25442314534619065`, 
        0.7741865761577696}, {-0.33381214877145615`, 0.778754600312818}}]}}, 
     {Arrowheads[{{0.010733582591328937`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.25845473686176323`, 
        0.9107461236047008}, {-0.3297805572558835, 0.9245479940423573}}]}}, 
     {Arrowheads[{{0.00985977208960438, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.26229542670048855`, 
        1.048787387935306}, {-0.3259398674171582, 1.0688596708882225`}}]}}, 
     {Arrowheads[{{0.00910134401065743, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.2658014222687758, 
        1.1878809359324098`}, {-0.32243387184887096`, 1.212119064067589}}]}}, 
     {Arrowheads[{{0.00960410944543542, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.1443177025465889, -1.2185758903682942`}, \
{-0.09097641510046958, -1.1814241096317057`}}]}}, 
     {Arrowheads[{{0.010490592201358322`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.14840190963751507`, -1.0765588169071887`}, \
{-0.08689220800954342, -1.0410882419163408`}}]}}, 
     {Arrowheads[{{0.011535893496510703`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.15322438721642767`, -0.9337202074295488}, \
{-0.08206973043063082, -0.9015739102175103}}]}}, 
     {Arrowheads[{{0.012787191600286095`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.15886838334521694`, -0.7896411391360545}, \
{-0.07642573430184155, -0.763300037334534}}]}}, 
     {Arrowheads[{{0.014318955254674862`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.16537376883130323`, -0.6436811135518485}, \
{-0.06992034881575525, -0.6269071217422696}}]}}, 
     {Arrowheads[{{0.016264048094914926`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.17268359689389068`, -0.49478529334492133`}, \
{-0.06261052075316781, -0.4934500007727261}}]}}, 
     {Arrowheads[{{0.018899009764876854`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.18045612927104704`, -0.34087387377762624`}, \
{-0.05483798837601146, -0.36500847916355067`}}]}}, 
     {Arrowheads[{{0.022894574731997065`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.18653694448378946`, -0.17630682069060172`}, \
{-0.048757173163269024`, -0.2472225910741045}}]}}, 
     {Arrowheads[{{0.02934937585238119, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.16769432031441195`, 
        0.015205044091578149`}, {-0.06759979733264654, \
-0.15638151467981382`}}]}}, 
     {Arrowheads[{{0.029349375852381208`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.0675997973326467, 
        0.1563815146798136}, {-0.16769432031441175`, \
-0.015205044091578704`}}]}}, 
     {Arrowheads[{{0.02289457473199707, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.048757173163269024`, 
        0.24722259107410421`}, {-0.18653694448378944`, 
        0.17630682069060127`}}]}}, 
     {Arrowheads[{{0.018899009764876865`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.05483798837601142, 
        0.3650084791635503}, {-0.18045612927104707`, 0.3408738737776258}}]}}, 
     {Arrowheads[{{0.016264048094914933`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.06261052075316778, 
        0.4934500007727257}, {-0.1726835968938907, 0.49478529334492094`}}]}}, 
     {Arrowheads[{{0.014318955254674864`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.06992034881575523, 
        0.6269071217422691}, {-0.16537376883130323`, 0.6436811135518481}}]}}, 
     {Arrowheads[{{0.012787191600286107`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.07642573430184149, 
        0.7633000373345336}, {-0.15886838334521697`, 0.789641139136054}}]}}, 
     {Arrowheads[{{0.011535893496510703`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.0820697304306308, 
        0.9015739102175098}, {-0.15322438721642767`, 0.9337202074295482}}]}}, 
     {Arrowheads[{{0.010490592201358322`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.08689220800954342, 
        1.0410882419163403`}, {-0.14840190963751507`, 
        1.0765588169071882`}}]}}, 
     {Arrowheads[{{0.009604109445435428, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.09097641510046955, 
        1.1814241096317053`}, {-0.14431770254658893`, 
        1.2185758903682937`}}]}}, 
     {Arrowheads[{{0.010043663194458218`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.03587790038311767, -1.2250757585386924`}, {
        0.0817691584404121, -1.1749242414613077`}}]}}, 
     {Arrowheads[{{0.01104698066557282, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.03176470170136062, -1.0846200807092694`}, {
        0.08588235712216916, -1.03302697811426}}]}}, 
     {Arrowheads[{{0.012247377965771002`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.0266662859020379, -0.9437966391552104}, {
        0.09098077292149187, -0.8914974784918486}}]}}, 
     {Arrowheads[{{0.013705815717546262`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.02033344448630897, -0.8023530273160481}, {
        0.0973136143372208, -0.7505881491545404}}]}}, 
     {Arrowheads[{{0.015516083847034288`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.012462239516063442`, -0.6599489589010629}, {
        0.10518481930746633`, -0.6106392763930552}}]}}, 
     {Arrowheads[{{0.01784124011684196, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.002640309639429725, -0.5162301071876967}, {
        0.11500674918410005`, -0.47200518692995075`}}]}}, 
     {Arrowheads[{{0.021022725964844946`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.009926892360792597, -0.3712433223073923}, {
        0.12757395118432235`, -0.33463903063378453`}}]}}, 
     {Arrowheads[{{0.025994774584153214`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.027659694899201198`, -0.2278762603373115}, {
        0.14530675372273097`, -0.19565315142739473`}}]}}, 
     {Arrowheads[{{0.036143128429005494`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{-0.05321176661158247, -0.1196706299243905}, {
        0.17085882543511224`, -0.021505840663845185`}}]}}, 
     {Arrowheads[{{0.03614312842900553, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.17085882543511224`, 
        0.02150584066384452}, {-0.05321176661158247, 
        0.11967062992439038`}}]}}, 
     {Arrowheads[{{0.025994774584153238`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.14530675372273105`, 
        0.19565315142739434`}, {-0.027659694899201268`, 
        0.22787626033731115`}}]}}, 
     {Arrowheads[{{0.021022725964844956`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.1275739511843224, 
        0.33463903063378414`}, {-0.009926892360792652, 
        0.3712433223073919}}]}}, 
     {Arrowheads[{{0.01784124011684198, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.1150067491841001, 0.47200518692995036`}, {
        0.0026403096394296693`, 0.5162301071876964}}]}}, 
     {Arrowheads[{{0.015516083847034297`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.10518481930746636`, 0.6106392763930547}, {
        0.012462239516063414`, 0.6599489589010624}}]}}, 
     {Arrowheads[{{0.013705815717546263`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.09731361433722081, 0.75058814915454}, {
        0.020333444486308958`, 0.8023530273160476}}]}}, 
     {Arrowheads[{{0.012247377965771007`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.0909807729214919, 0.8914974784918481}, {
        0.02666628590203788, 0.94379663915521}}]}}, 
     {Arrowheads[{{0.011046980665572826`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.08588235712216917, 1.0330269781142596`}, {
        0.0317647017013606, 1.084620080709269}}]}}, 
     {Arrowheads[{{0.010043663194458218`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.0817691584404121, 1.174924241461307}, {
        0.03587790038311767, 1.2250757585386918`}}]}}, 
     {Arrowheads[{{0.010391348145098457`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2182638652492098, -1.2307674674089528`}, {
        0.2523243700449082, -1.1692325325910469`}}]}}, 
     {Arrowheads[{{0.01149076304532334, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.21482719156982916`, -1.091888550630164}, {
        0.2557610437242889, -1.0257585081933653`}}]}}, 
     {Arrowheads[{{0.0128170077802107, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2104168595996828, -0.9531791045120694}, {
        0.2601713756944352, -0.8821150131349895}}]}}, 
     {Arrowheads[{{0.014435218177906885`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.20473394397200273`, -0.8145828342863991}, {
        0.2658542913221153, -0.7383583421841894}}]}}, 
     {Arrowheads[{{0.016430560399691416`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.19744345462679, -0.6760266948076884}, {
        0.27314478066732806`, -0.5945615404864295}}]}}, 
     {Arrowheads[{{0.018908312377579957`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.18828966224245322`, -0.5375366325249461}, {
        0.2822985730516648, -0.4506986615927013}}]}}, 
     {Arrowheads[{{0.02196667761065987, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.17748118139340127`, -0.39967450580902136`}, {
        0.29310705390071673`, -0.3062078471321555}}]}}, 
     {Arrowheads[{{0.025505242810149466`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.16696663353925362`, -0.26450512444888413`}, {
        0.3036216017548644, -0.15902428731582213`}}]}}, 
     {Arrowheads[{{0.028333456996041465`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.162982402107615, -0.13355718761283242`}, {
        0.30760583318650303`, -0.007619282975403277}}]}}, 
     {Arrowheads[{{0.028333456996041465`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.30760583318650303`, 0.007619282975402861}, {
        0.162982402107615, 0.13355718761283203`}}]}}, 
     {Arrowheads[{{0.02550524281014948, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.30362160175486447`, 0.15902428731582172`}, {
        0.16696663353925356`, 0.26450512444888374`}}]}}, 
     {Arrowheads[{{0.021966677610659877`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2931070539007168, 0.3062078471321551}, {
        0.17748118139340124`, 0.399674505809021}}]}}, 
     {Arrowheads[{{0.018908312377579967`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.28229857305166484`, 0.4506986615927009}, {
        0.1882896622424532, 0.5375366325249458}}]}}, 
     {Arrowheads[{{0.016430560399691416`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.27314478066732806`, 0.5945615404864292}, {
        0.19744345462678997`, 0.6760266948076881}}]}}, 
     {Arrowheads[{{0.014435218177906887`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2658542913221153, 0.7383583421841889}, {
        0.2047339439720027, 0.8145828342863987}}]}}, 
     {Arrowheads[{{0.012817007780210709`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.26017137569443527`, 0.8821150131349891}, {
        0.21041685959968276`, 0.953179104512069}}]}}, 
     {Arrowheads[{{0.011490763045323336`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2557610437242889, 1.025758508193365}, {
        0.21482719156982918`, 1.0918885506301637`}}]}}, 
     {Arrowheads[{{0.010391348145098457`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.2523243700449082, 1.1692325325910466`}, {
        0.2182638652492098, 1.2307674674089526`}}]}}, 
     {Arrowheads[{{0.010623071759023932`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4025421000522177, -1.2347473676660254`}, {
        0.42098731171248865`, -1.1652526323339745`}}]}}, 
     {Arrowheads[{{0.011791926313725576`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.40047053777716446`, -1.0970980341564973`}, {
        0.42305887398754183`, -1.020549024667032}}]}}, 
     {Arrowheads[{{0.01321172096517739, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.39770380716254106`, -0.9600894823313861}, {
        0.42582560460216523`, -0.8752046353156728}}]}}, 
     {Arrowheads[{{0.014952462859907678`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3939564380802897, -0.8238353604649308}, {
        0.4295729736844166, -0.7291058160056577}}]}}, 
     {Arrowheads[{{0.01709516530929886, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.38884336385646046`, -0.6884129459224627}, {
        0.43468604790824583`, -0.5821752893716552}}]}}, 
     {Arrowheads[{{0.01970868241350384, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3819193675551102, -0.5537654668974012}, {
        0.4416100442095961, -0.4344698272202462}}]}}, 
     {Arrowheads[{{0.022774988769970198`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3728748829163935, -0.41948523168476426`}, {
        0.45065452884831286`, -0.2863971212564126}}]}}, 
     {Arrowheads[{{0.026022781612183406`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.36193260648172576`, -0.28437584226966117`}, {
        0.4615968052829805, -0.13915356949504512`}}]}}, 
     {Arrowheads[{{0.028791626885130688`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.3498371421710532, -0.14581324510595456`}, {
        0.47369226959365296`, 0.004636774517718892}}]}}, 
     {Arrowheads[{{0.028791626885130716`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.47369226959365307`, -0.004636774517719308}, {
        0.34983714217105316`, 0.14581324510595423`}}]}}, 
     {Arrowheads[{{0.02602278161218342, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4615968052829806, 0.1391535694950447}, {
        0.3619326064817257, 0.2843758422696608}}]}}, 
     {Arrowheads[{{0.02277498876997021, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.45065452884831286`, 0.2863971212564122}, {
        0.3728748829163934, 0.4194852316847639}}]}}, 
     {Arrowheads[{{0.01970868241350385, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4416100442095961, 0.43446982722024574`}, {
        0.3819193675551102, 0.5537654668974009}}]}}, 
     {Arrowheads[{{0.01709516530929886, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.43468604790824583`, 0.5821752893716547}, {
        0.38884336385646046`, 0.6884129459224623}}]}}, 
     {Arrowheads[{{0.014952462859907683`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42957297368441666`, 0.7291058160056573}, {
        0.3939564380802897, 0.8238353604649303}}]}}, 
     {Arrowheads[{{0.01321172096517739, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42582560460216523`, 0.8752046353156724}, {
        0.39770380716254106`, 0.9600894823313857}}]}}, 
     {Arrowheads[{{0.011791926313725576`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.42305887398754183`, 1.0205490246670317`}, {
        0.40047053777716446`, 1.097098034156497}}]}}, 
     {Arrowheads[{{0.010623071759023932`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.4209873117124886, 1.165252632333974}, {
        0.40254210005221763`, 1.234747367666025}}]}}, 
     {Arrowheads[{{0.010719786744117589`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5879972956445769, -1.2362769871147221`}, {
        0.5884732925907177, -1.1637230128852778`}}]}}, 
     {Arrowheads[{{0.011925766026694515`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5877858475280142, -1.0991800545382862`}, {
        0.5886847407072805, -1.0184670042852433`}}]}}, 
     {Arrowheads[{{0.01340302419569935, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5873964127244171, -0.9629976471326734}, {
        0.5890741755108774, -0.8722964705143855}}]}}, 
     {Arrowheads[{{0.01523619929665253, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.586651280433113, -0.8280084061787518}, {
        0.5898193078021815, -0.7249327702918366}}]}}, 
     {Arrowheads[{{0.017533546267245972`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5851645595205852, -0.6945514150062057}, {
        0.5913060287147093, -0.5760368202879125}}]}}, 
     {Arrowheads[{{0.02041454570604357, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5820828197600455, -0.5629297985528489}, {
        0.5943877684752491, -0.4253054955647986}}]}}, 
     {Arrowheads[{{0.02394806260139348, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5755906475781919, -0.4329934198021396}, {
        0.6008799406571027, -0.2728889331390372}}]}}, 
     {Arrowheads[{{0.027968495951180873`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5625236386581566, -0.3028561605971437}, {
        0.6139469495771381, -0.12067325116756254`}}]}}, 
     {Arrowheads[{{0.031777616130180625`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5404152776897956, -0.16691269147134558`}, {
        0.6360553105454991, 0.025736220883109895`}}]}}, 
     {Arrowheads[{{0.03177761613018063, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6360553105454991, -0.025736220883110283`}, {
        0.5404152776897955, 0.1669126914713452}}]}}, 
     {Arrowheads[{{0.027968495951180894`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6139469495771381, 0.12067325116756213`}, {
        0.5625236386581565, 0.3028561605971434}}]}}, 
     {Arrowheads[{{0.02394806260139348, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.6008799406571027, 0.27288893313903684`}, {
        0.5755906475781919, 0.43299341980213923`}}]}}, 
     {Arrowheads[{{0.020414545706043553`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5943877684752491, 0.42530549556479824`}, {
        0.5820828197600455, 0.5629297985528484}}]}}, 
     {Arrowheads[{{0.017533546267245993`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5913060287147094, 0.5760368202879119}, {
        0.5851645595205852, 0.6945514150062052}}]}}, 
     {Arrowheads[{{0.01523619929665253, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5898193078021816, 0.7249327702918361}, {
        0.5866512804331131, 0.8280084061787514}}]}}, 
     {Arrowheads[{{0.013403024195699354`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5890741755108775, 0.8722964705143851}, {
        0.5873964127244171, 0.962997647132673}}]}}, 
     {Arrowheads[{{0.011925766026694515`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5886847407072804, 1.0184670042852428`}, {
        0.5877858475280141, 1.0991800545382857`}}]}}, 
     {Arrowheads[{{0.010719786744117589`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.5884732925907177, 1.1637230128852774`}, {
        0.5879972956445768, 1.2362769871147217`}}]}}, 
     {Arrowheads[{{0.010668438268448283`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.773594021776991, -1.2349928482483166`}, {
        0.7558177429288919, -1.1650071517516833`}}]}}, 
     {Arrowheads[{{0.01187107292770184, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7753721220290645, -1.097555641031937}, {
        0.7540396426768184, -1.0200914177915925`}}]}}, 
     {Arrowheads[{{0.013355333017371471`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7776283907159846, -0.9609572551646295}, {
        0.7517833739898984, -0.8743368624824295}}]}}, 
     {Arrowheads[{{0.015225064322060422`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7804650679858176, -0.8255258555257496}, {
        0.7489466967200654, -0.7274153209448389}}]}}, 
     {Arrowheads[{{0.017638774520897047`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7838961680129582, -0.6918182502612205}, {
        0.7455155966929248, -0.5787699850328974}}]}}, 
     {Arrowheads[{{0.02084698310108077, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7875402732243529, -0.5608702276517535}, {
        0.7418714914815299, -0.42736506646589395`}}]}}, 
     {Arrowheads[{{0.02524467550223336, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7895183348950526, -0.4346913498095779}, {
        0.7398934298108303, -0.27119100313159894`}}]}}, 
     {Arrowheads[{{0.031307609759735726`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7825609404856545, -0.31620020365037343`}, {
        0.7468508242202283, -0.10732920811433282`}}]}}, 
     {Arrowheads[{{0.0384138677313928, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7467775922168922, -0.199345782710719}, {
        0.7826341724889907, 0.05816931212248333}}]}}, 
     {Arrowheads[{{0.03841386773139282, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7826341724889908, -0.058169312122483774`}, {
        0.746777592216892, 0.19934578271071868`}}]}}, 
     {Arrowheads[{{0.03130760975973575, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7468508242202284, 0.10732920811433236`}, {
        0.7825609404856545, 0.31620020365037316`}}]}}, 
     {Arrowheads[{{0.02524467550223337, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7398934298108303, 0.2711910031315985}, {
        0.7895183348950527, 0.4346913498095775}}]}}, 
     {Arrowheads[{{0.020846983101080793`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.74187149148153, 0.42736506646589345`}, {0.787540273224353,
         0.5608702276517532}}]}}, 
     {Arrowheads[{{0.01763877452089703, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7455155966929247, 0.5787699850328971}, {
        0.7838961680129581, 0.69181825026122}}]}}, 
     {Arrowheads[{{0.015225064322060427`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7489466967200653, 0.7274153209448384}, {
        0.7804650679858176, 0.8255258555257492}}]}}, 
     {Arrowheads[{{0.013355333017371471`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7517833739898984, 0.874336862482429}, {0.7776283907159846,
         0.960957255164629}}]}}, 
     {Arrowheads[{{0.01187107292770184, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7540396426768184, 1.020091417791592}, {0.7753721220290645,
         1.0975556410319365`}}]}}, 
     {Arrowheads[{{0.010668438268448283`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.7558177429288919, 1.165007151751683}, {0.773594021776991, 
        1.2349928482483161`}}]}}, 
     {Arrowheads[{{0.010465880423080593`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9581911518529398, -1.231063979952932}, {
        0.9241617893235314, -1.168936020047068}}]}}, 
     {Arrowheads[{{0.011617198907947655`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9616020709405398, -1.0924158898850103`}, {
        0.9207508702359314, -1.0252311689385192`}}]}}, 
     {Arrowheads[{{0.013038780524483186`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9660283268579744, -0.9541088285570656}, {
        0.9163246143184968, -0.8811852890899933}}]}}, 
     {Arrowheads[{{0.014840015454680876`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9718751766450171, -0.8162170078734398}, {
        0.9104777645314542, -0.7367241685971486}}]}}, 
     {Arrowheads[{{0.017205708989382982`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9797719350829831, -0.6788924406462065}, {
        0.9025810060934881, -0.5916957946479116}}]}}, 
     {Arrowheads[{{0.020484881767525196`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9907772917819481, -0.5425497622024616}, {
        0.8915756493945232, -0.44568553191518573`}}]}}, 
     {Arrowheads[{{0.025453452729713305`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0069090394779232`, -0.4086116417748356}, {
        0.8754439016985479, -0.29727071116634124`}}]}}, 
     {Arrowheads[{{0.03434062549263583, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.032491941665014, -0.2836495961965986}, {
        0.8498609995114572, -0.13987981556810772`}}]}}, 
     {Arrowheads[{{0.05676214149484382, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.058823451411794, -0.222440755533662}, {0.8235294897646773,
         0.08126428494542631}}]}}, 
     {Arrowheads[{{0.056762141494843925`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8235294897646774, -0.08126428494542726}, {
        1.0588234514117938`, 0.22244075553366216`}}]}}, 
     {Arrowheads[{{0.03434062549263589, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.849860999511457, 0.13987981556810725`}, {
        1.0324919416650142`, 0.28364959619659824`}}]}}, 
     {Arrowheads[{{0.025453452729713305`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8754439016985479, 0.29727071116634085`}, {
        1.0069090394779232`, 0.4086116417748352}}]}}, 
     {Arrowheads[{{0.020484881767525213`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.8915756493945232, 0.44568553191518534`}, {
        0.9907772917819482, 0.5425497622024613}}]}}, 
     {Arrowheads[{{0.017205708989383003`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9025810060934881, 0.591695794647911}, {0.9797719350829832,
         0.678892440646206}}]}}, 
     {Arrowheads[{{0.014840015454680876`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9104777645314541, 0.7367241685971483}, {0.971875176645017,
         0.8162170078734394}}]}}, 
     {Arrowheads[{{0.013038780524483186`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9163246143184968, 0.8811852890899928}, {
        0.9660283268579744, 0.9541088285570651}}]}}, 
     {Arrowheads[{{0.011617198907947655`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9207508702359314, 1.0252311689385187`}, {
        0.9616020709405398, 1.0924158898850098`}}]}}, 
     {Arrowheads[{{0.010465880423080573`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{0.9241617893235314, 1.1689360200470675`}, {
        0.9581911518529399, 1.2310639799529313`}}]}}, 
     {Arrowheads[{{0.010124016883565685`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1408582826559983`, -1.2252010779605762`}, {
        1.0944358349910612`, -1.174798922039424}}]}}, 
     {Arrowheads[{{0.011176829551549517`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1451297987830207`, -1.084811810842115}, {
        1.0901643188640386`, -1.0328352479814145`}}]}}, 
     {Arrowheads[{{0.012461424078421294`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1505431679238711`, -0.9440346752744642}, {
        1.084750949723188, -0.8912594423725948}}]}}, 
     {Arrowheads[{{0.014066512771326483`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1575020226553687`, -0.8025031762611465}, {
        1.0777920949916906`, -0.7504380002094418}}]}}, 
     {Arrowheads[{{0.01613845785175807, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.166600788678621, -0.6595096590931426}, {
        1.0686933289684384`, -0.6110785762009754}}]}}, 
     {Arrowheads[{{0.018941884216876405`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1787358998745918`, -0.5135427239274986}, {
        1.0565582177724675`, -0.4746925701901489}}]}}, 
     {Arrowheads[{{0.02301403398023973, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1951331370959248`, -0.3608021411543516}, {
        1.0401609805511347`, -0.34508021178682524`}}]}}, 
     {Arrowheads[{{0.02957919774754836, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2151821907246803`, -0.18924319744719334`}, {
        1.020111926922379, -0.23428621431751293`}}]}}, 
     {Arrowheads[{{0.040417165220601514`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.1996849772371163`, 0.0388573989693525}, {
        1.035609140409943, -0.18003386955758818`}}]}}, 
     {Arrowheads[{{0.040417165220601556`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0356091404099432`, 0.18003386955758816`}, {
        1.199684977237116, -0.03885739896935324}}]}}, 
     {Arrowheads[{{0.0295791977475484, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.020111926922379, 0.23428621431751268`}, {
        1.2151821907246805`, 0.1892431974471928}}]}}, 
     {Arrowheads[{{0.023014033980239764`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0401609805511345`, 0.3450802117868249}, {
        1.1951331370959248`, 0.36080214115435116`}}]}}, 
     {Arrowheads[{{0.0189418842168764, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0565582177724675`, 0.47469257019014854`}, {
        1.1787358998745918`, 0.5135427239274981}}]}}, 
     {Arrowheads[{{0.016138457851758096`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0686933289684384`, 0.6110785762009749}, {
        1.1666007886786212`, 0.6595096590931422}}]}}, 
     {Arrowheads[{{0.014066512771326483`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0777920949916906`, 0.7504380002094415}, {
        1.1575020226553687`, 0.8025031762611462}}]}}, 
     {Arrowheads[{{0.012461424078421294`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0847509497231882`, 0.8912594423725944}, {
        1.1505431679238713`, 0.9440346752744637}}]}}, 
     {Arrowheads[{{0.011176829551549517`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.0901643188640386`, 1.032835247981414}, {
        1.1451297987830207`, 1.0848118108421145`}}]}}, 
     {Arrowheads[{{0.010124016883565685`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.094435834991061, 1.1747989220394233`}, {1.140858282655998,
         1.2252010779605755`}}]}}, 
     {Arrowheads[{{0.009671005245730915, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3211584749076861`, -1.2184376785281241`}, {
        1.2670768192099615`, -1.181562321471876}}]}}, 
     {Arrowheads[{{0.010591286792003737`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3254091886225918`, -1.076303600887245}, {
        1.2628261054950558`, -1.0413434579362844`}}]}}, 
     {Arrowheads[{{0.01168674134835193, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.330483965414536, -0.933193864100761}, {
        1.2577513287031117`, -0.9021002535462979}}]}}, 
     {Arrowheads[{{0.013009457066716537`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.33646798069161, -0.7885029106549882}, {
        1.2517673134260379`, -0.7644382658156001}}]}}, 
     {Arrowheads[{{0.014631797501090841`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3432804232992943`, -0.6412042378515956}, {
        1.2449548708183533`, -0.6293839974425223}}]}}, 
     {Arrowheads[{{0.016649315656531227`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3502753152763773`, -0.48953416650942266`}, {
        1.2379599788412703`, -0.4987011276082248}}]}}, 
     {Arrowheads[{{0.019154816231893988`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3549902866398353`, -0.33065650801673696`}, {
        1.2332450074778123`, -0.3752258449244399}}]}}, 
     {Arrowheads[{{0.022062645755349348`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3497450754152127`, -0.16196188096537922`}, {
        1.2384902187024351`, -0.261567530799327}}]}}, 
     {Arrowheads[{{0.02447962428512691, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.3193916600534499`, 0.008305950316841407}, {
        1.2688436340641978`, -0.14948242090507707`}}]}}, 
     {Arrowheads[{{0.024479624285126914`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2688436340641978`, 0.14948242090507674`}, {
        1.3193916600534497`, -0.008305950316841837}}]}}, 
     {Arrowheads[{{0.022062645755349372`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.238490218702435, 0.26156753079932676`}, {
        1.3497450754152125`, 0.16196188096537875`}}]}}, 
     {Arrowheads[{{0.019154816231893995`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2332450074778123`, 0.37522584492443956`}, {
        1.3549902866398353`, 0.33065650801673657`}}]}}, 
     {Arrowheads[{{0.01664931565653123, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2379599788412703`, 0.49870112760822444`}, {
        1.3502753152763773`, 0.4895341665094222}}]}}, 
     {Arrowheads[{{0.014631797501090872`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2449548708183533`, 0.6293839974425219}, {
        1.3432804232992945`, 0.6412042378515952}}]}}, 
     {Arrowheads[{{0.013009457066716565`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2517673134260376`, 0.7644382658155998}, {1.33646798069161,
         0.7885029106549878}}]}}, 
     {Arrowheads[{{0.01168674134835193, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2577513287031117`, 0.9021002535462975}, {
        1.330483965414536, 0.9331938641007607}}]}}, 
     {Arrowheads[{{0.010591286792003737`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2628261054950558`, 1.041343457936284}, {
        1.3254091886225918`, 1.0763036008872446`}}]}}, 
     {Arrowheads[{{0.00967100524573094, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.2670768192099615`, 1.1815623214718753`}, {
        1.3211584749076863`, 1.2184376785281235`}}]}}, 
     {Arrowheads[{{0.009145555953560324, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4992124326812184`, -1.211771783683759}, {
        1.4419640379070175`, -1.1882282163162412`}}]}}, 
     {Arrowheads[{{0.009919967319078328, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.5027900946744148`, -1.0683132205059256`}, {
        1.4383863759138213`, -1.049333838317604}}]}}, 
     {Arrowheads[{{0.010811552890389509`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.506679925759189, -0.9236549836504255}, {
        1.4344965448290468`, -0.9116391339966334}}]}}, 
     {Arrowheads[{{0.011839946056530382`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.5106486640280523`, -0.7772796045919068}, {
        1.4305278065601836`, -0.7756615718786817}}]}}, 
     {Arrowheads[{{0.013021145313838921`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.514127179356414, -0.628499089518965}, {
        1.4270492912318222`, -0.642089145775153}}]}}, 
     {Arrowheads[{{0.014352710185245816`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.5158743632822629`, -0.4765554369842168}, {
        1.425302107305973, -0.5116798571334306}}]}}, 
     {Arrowheads[{{0.01577605960072504, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.5134848615652148`, -0.32115627372564287`}, {
        1.4276916090230214`, -0.38472607921553403`}}]}}, 
     {Arrowheads[{{0.017103193521098903`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.5033451782666256`, -0.16404547611707823`}, {
        1.4378312923216103`, -0.25948393564762806`}}]}}, 
     {Arrowheads[{{0.017960306819247793`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4832238288939692`, -0.01113508392984492}, {
        1.457952641694267, -0.13004138665839077`}}]}}, 
     {Arrowheads[{{0.017960306819247793`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.457952641694267, 0.13004138665839038`}, {
        1.4832238288939692`, 0.011135083929844532`}}]}}, 
     {Arrowheads[{{0.017103193521098906`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4378312923216103`, 0.2594839356476277}, {
        1.5033451782666256`, 0.16404547611707782`}}]}}, 
     {Arrowheads[{{0.015776059600725045`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4276916090230212`, 0.38472607921553365`}, {
        1.5134848615652146`, 0.3211562737256424}}]}}, 
     {Arrowheads[{{0.014352710185245823`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.425302107305973, 0.5116798571334302}, {
        1.5158743632822629`, 0.4765554369842164}}]}}, 
     {Arrowheads[{{0.013021145313838927`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.427049291231822, 0.6420891457751526}, {
        1.5141271793564137`, 0.6284990895189646}}]}}, 
     {Arrowheads[{{0.011839946056530382`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4305278065601836`, 0.7756615718786812}, {
        1.5106486640280523`, 0.7772796045919064}}]}}, 
     {Arrowheads[{{0.010811552890389509`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.434496544829047, 0.911639133996633}, {1.5066799257591892`,
         0.9236549836504251}}]}}, 
     {Arrowheads[{{0.009919967319078328, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4383863759138211`, 1.0493338383176034`}, {
        1.5027900946744146`, 1.068313220505925}}]}}, 
     {Arrowheads[{{0.009145555953560324, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.4419640379070175`, 1.1882282163162408`}, {
        1.4992124326812184`, 1.2117717836837585`}}]}}, 
     {Arrowheads[{{0.00858728006577067, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6755169737467204`, -1.2058884123453024`}, {
        1.618600673312104, -1.1941115876546975`}}]}}, 
     {Arrowheads[{{0.009221415379578095, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6781353769090865`, -1.061673892382375}, {
        1.6159822701497377`, -1.0559731664411547`}}]}}, 
     {Arrowheads[{{0.009925532987128357, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6806236328321467`, -0.9163499257407348}, {
        1.6134940142266774`, -0.9189441919063241}}]}}, 
     {Arrowheads[{{0.010700156745055249`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.682617777716551, -0.7696279536842879}, {
        1.6114998693422733`, -0.7833132227863006}}]}}, 
     {Arrowheads[{{0.011535814164948668`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6834892850795657`, -0.6212622642113677}, {
        1.6106283619792583`, -0.6493259710827504}}]}}, 
     {Arrowheads[{{0.012402895589344298`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6822503331759406`, -0.4712407782646347}, {
        1.6118673138828838`, -0.5169945158530127}}]}}, 
     {Arrowheads[{{0.01323675664576884, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.677594410166949, -0.32016573412462795`}, {
        1.6165232368918752`, -0.3857166188165489}}]}}, 
     {Arrowheads[{{0.013925670723378076`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6683816092946573`, -0.1697373280072995}, {
        1.6257360377641668`, -0.2537920837574068}}]}}, 
     {Arrowheads[{{0.014325606634628051`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6547944091636662`, -0.022728823433440598`}, {
        1.6393232378951579`, -0.11844764715479508`}}]}}, 
     {Arrowheads[{{0.014325606634628051`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.639323237895158, 0.11844764715479471`}, {
        1.6547944091636662`, 0.022728823433440196`}}]}}, 
     {Arrowheads[{{0.013925670723378063`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.625736037764167, 0.2537920837574064}, {
        1.6683816092946573`, 0.16973732800729907`}}]}}, 
     {Arrowheads[{{0.013236756645768847`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6165232368918752`, 0.38571661881654856`}, {
        1.677594410166949, 0.32016573412462757`}}]}}, 
     {Arrowheads[{{0.012402895589344301`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6118673138828836`, 0.5169945158530124}, {
        1.6822503331759404`, 0.4712407782646343}}]}}, 
     {Arrowheads[{{0.011535814164948668`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6106283619792585`, 0.6493259710827499}, {
        1.683489285079566, 0.6212622642113672}}]}}, 
     {Arrowheads[{{0.010700156745055249`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6114998693422733`, 0.7833132227863001}, {
        1.682617777716551, 0.7696279536842875}}]}}, 
     {Arrowheads[{{0.009925532987128357, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6134940142266776`, 0.9189441919063237}, {
        1.6806236328321469`, 0.9163499257407344}}]}}, 
     {Arrowheads[{{0.009221415379578095, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6159822701497377`, 1.055973166441154}, {
        1.6781353769090865`, 1.0616738923823743`}}]}}, 
     {Arrowheads[{{0.008587280065770702, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.6186006733121039`, 1.194111587654697}, {
        1.6755169737467204`, 1.205888412345302}}]}}, 
     {Arrowheads[{{0.008028758297853768, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8506782479933586`, -1.2010928493128115`}, {
        1.7963805755360538`, -1.1989071506871882`}}]}}, 
     {Arrowheads[{{0.008539700059346058, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8523447454731603`, -1.0566136853592072`}, {
        1.7947140780562523`, -1.0610333734643222`}}]}}, 
     {Arrowheads[{{0.009087707144693643, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8536142066338799`, -0.9112639040277603}, {
        1.7934446168955327`, -0.9240302136192987}}]}}, 
     {Arrowheads[{{0.009665390037027943, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8541434314961254`, -0.764951358946884}, {
        1.7929153920332874`, -0.7879898175237046}}]}}, 
     {Arrowheads[{{0.010256893674952106`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8534511954428632`, -0.6176998628982734}, {
        1.7936076280865494`, -0.6528883723958446}}]}}, 
     {Arrowheads[{{0.01083369517726316, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8509427531532634`, -0.46977215459604016`}, {
        1.7961160703761492`, -0.5184631395216073}}]}}, 
     {Arrowheads[{{0.011351224449672936`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8460453715915575`, -0.32181693302975045`}, {
        1.8010134519378551`, -0.38406541991142645`}}]}}, 
     {Arrowheads[{{0.011750518851388916`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8385117242194495`, -0.17492910126993133`}, {
        1.8085470993099628`, -0.24860031049477493`}}]}}, 
     {Arrowheads[{{0.011970451973653085`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8288085278542399`, -0.030423430263999733`}, {
        1.818250295675173, -0.11075304032423594`}}]}}, 
     {Arrowheads[{{0.011970451973653089`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8182502956751727`, 0.11075304032423557`}, {
        1.8288085278542396`, 0.030423430263999338`}}]}}, 
     {Arrowheads[{{0.011750518851388921`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.808547099309963, 0.24860031049477457`}, {
        1.8385117242194497`, 0.17492910126993091`}}]}}, 
     {Arrowheads[{{0.011351224449672936`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.8010134519378551`, 0.38406541991142606`}, {
        1.8460453715915575`, 0.32181693302975006`}}]}}, 
     {Arrowheads[{{0.010833695177263155`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7961160703761492`, 0.5184631395216068}, {
        1.8509427531532634`, 0.46977215459603977`}}]}}, 
     {Arrowheads[{{0.010256893674952106`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7936076280865494`, 0.6528883723958442}, {
        1.8534511954428632`, 0.6176998628982729}}]}}, 
     {Arrowheads[{{0.009665390037027943, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7929153920332872`, 0.7879898175237041}, {
        1.8541434314961251`, 0.7649513589468836}}]}}, 
     {Arrowheads[{{0.009087707144693643, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7934446168955327`, 0.9240302136192983}, {
        1.8536142066338799`, 0.9112639040277598}}]}}, 
     {Arrowheads[{{0.008539700059346058, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.7947140780562523`, 1.0610333734643218`}, {
        1.8523447454731603`, 1.0566136853592067`}}]}}, 
     {Arrowheads[{{0.008028758297853768, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.796380575536054, 1.198907150687188}, {1.8506782479933588`,
         1.2010928493128112`}}]}}, 
     {Arrowheads[{{0.007492307397928591, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0252220458758527`, -1.1974032084097663`}, {
        1.9747779541241481`, -1.2025967915902338`}}]}}, 
     {Arrowheads[{{0.007901144369542324, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0260943838089274`, -1.0529878824118677`}, {
        1.9739056161910735`, -1.0646591764116615`}}]}}, 
     {Arrowheads[{{0.008326356320812377, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.026460084098497, -0.9079588779032902}, {
        1.973539915901504, -0.9273352397437687}}]}}, 
     {Arrowheads[{{0.008758809389375953, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0260559377308955`, -0.7623389274694037}, {
        1.9739440622691053`, -0.7906022490011848}}]}}, 
     {Arrowheads[{{0.009183861512803575, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0245694905833975`, -0.616259914997018}, {
        1.9754305094166034`, -0.6543283202971}}]}}, 
     {Arrowheads[{{0.009580186234789435, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.021687266307054, -0.4700179957687653}, {
        1.978312733692947, -0.5182172983488822}}]}}, 
     {Arrowheads[{{0.009919908724160433, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0171949119368873`, -0.32410828424453825`}, {
        1.9828050880631136`, -0.3817740686966386}}]}}, 
     {Arrowheads[{{0.010171468533881662`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.011120452047686, -0.17918832818375774`}, {
        1.9888795479523151`, -0.24434108358094853`}}]}}, 
     {Arrowheads[{{0.010306039523791392`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{2.0038556328616566`, -0.03592443630178467}, {
        1.996144367138344, -0.10525203428645101`}}]}}, 
     {Arrowheads[{{0.010306039523791392`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9961443671383443`, 0.10525203428645064`}, {
        2.0038556328616566`, 0.035924436301784275`}}]}}, 
     {Arrowheads[{{0.010171468533881673`, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.988879547952315, 0.24434108358094814`}, {
        2.011120452047686, 0.17918832818375735`}}]}}, 
     {Arrowheads[{{0.009919908724160391, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9828050880631138`, 0.3817740686966382}, {
        2.017194911936887, 0.3241082842445378}}]}}, 
     {Arrowheads[{{0.00958018623478945, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9783127336929467`, 0.5182172983488817}, {
        2.021687266307054, 0.4700179957687649}}]}}, 
     {Arrowheads[{{0.009183861512803497, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9754305094166036`, 0.6543283202970995}, {
        2.024569490583397, 0.6162599149970176}}]}}, 
     {Arrowheads[{{0.008758809389376046, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9739440622691051`, 0.7906022490011844}, {
        2.026055937730896, 0.7623389274694031}}]}}, 
     {Arrowheads[{{0.008326356320812384, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.973539915901504, 0.9273352397437683}, {2.026460084098497, 
        0.9079588779032897}}]}}, 
     {Arrowheads[{{0.007901144369542324, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9739056161910735`, 1.0646591764116613`}, {
        2.0260943838089274`, 1.0529878824118675`}}]}}, 
     {Arrowheads[{{0.007492307397928591, 1.}}], 
      {RGBColor[0, 1, 0], 
       ArrowBox[{{1.9747779541241481`, 1.2025967915902331`}, {
        2.0252220458758527`, 1.1974032084097657`}}]}}}}, 
   GraphicsComplexBox[CompressedData["
1:eJx0nXdcU9f//+O2zjqq1lXqtu49avuOdVate9eBe++JA+MARx24rRNbB3VS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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], 
         LineBox[{5292, 4121, 6226, 4120, 7093, 5974, 5975, 5324, 4125, 5310, 
          4126, 5325, 6015, 5309, 4124, 6228, 6229, 5718, 5717, 6431, 3713, 
          4684, 4358, 6716, 4357, 5023, 4371, 4998, 4985, 3649, 5340, 5353, 
          5308, 5379, 5366, 3864, 5418, 5431, 5323, 5405, 5392, 3759, 5448, 
          5461, 5291, 5487, 5474, 3895, 4828, 4841, 4562, 4813, 5882, 3569, 
          5869, 5184, 4549, 5092, 5091, 6037, 6036, 5292}]},
        "0.679`"],
       Annotation[#, 0.679, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0k8og3Ecx/EP583s8Wf+jFJSuDhoUorsYE5KU8rJIhdNy0W2GQelHLRh
LiwZDg6UPznsgJLxNEqOO8xxRbhIkvJeDq8++/2ez/N9fs/TGnxTg/4iSUu4
qpQSdumS3CJPa6TfEinN+hY3MHGHTq7fkw/oLpdeyQ+r9E1mDOmR63kyC5dN
Gq2Q/Dig6yX7cMhsG+mEyb6LbIaD/ifrYn6nyDyy6KE/UyYtwmB+gvRwxhjZ
wj3paukJSaRwhC/umSYjmMAGlnFOf4AcQSvc6EA7fYOsh3iXKlgQpu8hvWhE
F9rwQr+f5w8hwJkmyTFypVYaJzOl0hl7OXrDfAuT9Z5D2keAmdec3wcLrNhh
P4ldOOnneEYd+Uy+Me8dTcyOMi/OfpB1lHWInMU6M+KFb4I1rGKbbogMI4gI
5tBLP8Zcd2EGeYFj9k8wz/MX8MPZN+3//40/qutUcA==
          "]]},
        "0.5820000000000001`"],
       Annotation[#, 0.5820000000000001, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0j1IlWEUwPFj2dIt9fbhzUQhbNCgxMIolYIGDSSV7s2hySJa+lCkkiRs
qCiCooiEMiMagiglhxrcMhK1JbJJwWwSg5AiREqo36Xhz/895zkf732fu+V4
++GzORHxGr2FES3JiPvczOc2R4yuj+jJi2hcF7FYEFG2IWIrdooTxRFVvIYn
1H1fG9GtJ6P3Ih/hn/I/8NiMEblXcm95iN/xCr0rkYvV4gTG7VklHuML3uU8
PmMSX53XoBbbUYkd2Is6e3LNreBSlNg5uTFiBmP4hXk8yY9oN7vVjhM8o+4O
nxb3cYfza3xM3M3lzp9xl3iIXzhvM/8UOuzL8EF8U1fpHWtRhjQacEl9glP4
a39JUUQSu9Tv5gPYhqNowhv1N7kXJ9GDTiyqH+RhPMUnjKJC/V37G3yDfk56
p6vchf16ptzXHIaRk/0OXKin3HM1xsXFnIcB9fWcxnP5M9yWrVM/5a7n8NH8
D7zk9yy48yt69qmtwR5ns/a+d/aFM3Jp3FC/bMZvuT98XVyfinjkrMA37eMk
9/M9+YecLz60KeKB50EMZGeJb/NlO1+6j1ueqzGtZ9rcKrkmcUvq///5H74s
Zr0=
          "]]},
        "0.485`"],
       Annotation[#, 0.485, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl009Mz3Ecx/F3szkQEtI/B4YKc0gZWpEpw2LL2mK2wpoN07DlT36X0OZv
XTowW6tmDG0OsmXTpVrWxb9ThxwcIsy/WByYR3N47vl5fz6vz/f9/Xx+39/i
A/WVx5IiYhzNWRHfUiK2pUYkZkecTYv4or44P2JkTkQ+N5o7h+/mE1wxN+JE
ZsROPsm9csvkluPdrIgc/ij3Ac+t98v16ZNrnIdr5jeaO2Rcwo+tbVkYUYYW
a5vMXedSbuUVclfRoH4iu9J40rus4l/cJHPK3iTv8XtexF7996AaF+2pcrYO
Z+uQS6jbuQudWKu+zFewXr4YTbKfnWPS3LC9z2XGeAQFetQsiDiKu7KVXIYH
9sziTDwzX8A5WCA/oQ7jXh6buleUyDd41/NI8fxbXO48rZxrz2BGxEt0oBfd
+GnPcU6gDm24hB75Ct6DPJSiEPnyKZyNv/qnOc8MNMqXcyWWYANWY1x+q/67
Ue+dDnMN7sk/dM8N7v4258id4Vr1Ba63fpOPqFt41PpBrlIf43br7/WfwBBG
8QrZcoumzotp+hXxOneQz3X2PJJ5ihu4j068tudjesQPvMV0Z/vDbfLVeu3Q
cxenyhXzZnU577P+Qm4EPRhCH4bluvgBEp7fgmZclU+e+n043e91gbt9Y2/c
U5bvYMhzn6EfA0jWI59n8n7rNajFUhThk+9pyPMKjO/IFfIaDJobwHbjn/Zm
WBuXPa3+mvL///kPfImHiA==
          "]]},
        "0.388`"],
       Annotation[#, 0.388, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Ftoz2Ecx/HHMiyHYTaHEbuwuREyh4miDEnKKEoodoOMyLHFHMZkuLB2
whwix5nD0nLc0LScj+2ChM0xLlhuWLx+uXjv3fM8n+/zfX7P7/9bysLlWTlt
Qggj/WnqGkJynxDec19e0z2Ed51DeNYzhOfxITzlZ/zZen5yCJ94Q5cQ3sok
GX8xbrSeqjYNA/HdXK7MB5mCHiHsxHCkY3NSCC3WN/FPzuOsbiGU8Ewu5Xs4
b+/R9i3TvxyluGauhItxWW1z7xBGqKnVs4rvq6uwdgixeq3CSqxHVUII0+yX
LbMU89Ck/jVinLWFf+Mb4uwXg0z5+uis+t7kOrnBnGGczjOt7+vlfNiIQuQj
Tq7VPbbVsw8ncJ4zdORdavITQ9iDXJyRP4oGNde5HtVoxCPMlc/Ua4KeY7mb
3HSeajybi6z/kYt17rf8E1/wVO6I/U+jFNdwCdnyw5xjlOcbwzHuLY37IlnN
E5lXqMcPfESFmmV6zdJzIb+S28NLjMs4x/pWXmC8jlOtH+PVxlV8yvp8+y9G
jn5ZPAmfovv0HjKQghnIxAb5OE5Eq/7Jni0ew+TTeTwGYQ6moVq+gIuQjVys
QIv8Wa7BYTzGHaTJ79U/0x3s53hn2sKrMU5No3fVjBr8xQ/0iH7jzjIcd417
c6fo/clP5Bk4YX4pz49y8o3efTMe2r+Bf3meGzxFzXcuRKW5KoxVe5EvYJR8
Ne/mw1zIl/ioutNcYHyKd/IZ3sYneQcf8Uzb+aq7rzFXHH1T5jZH3xhv4TLO
44NcgXIciO4CRXqUO9ciZCPFuQbgiue5Fe1h3zrOsDYaaThuLpX7yQ211t64
A9phiPm99jwX/Tacp5LX81p89dvor+dt9/6Ab0bfMNfyQ36h53Pf8ku8wHv/
T27JvuFE+9yQqzOuRfvoO9dncvf//9P+ATpovFs=
          "]]},
        "0.29100000000000004`"],
       Annotation[#, 0.29100000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1G1sTmcYwPG7mw9FVvQp1YpgY774UJVgYZnxlEiJt41kZlFNBJtZ6FNd
xGQ0GYosrZd5WRBmU9IJ27qqNkRa3RZvia620qEvyDRhskSsM7+Tffj3f+7r
vq7rvs45fc6QRStmf5gUQljrT016CKlIywyhLxp7h1Br3dArhBjXiRWmhFDP
VbEQfugTwgXXldzAv8qvcl0zIIQf+bSc38VOu+4j9ttLIVSLNYv9Iv+M+E9c
zT9zjb1+nI4Lzrvs3Hq+wgPF2tW19wuhAw/VDNezzDzJaSHcs95rfZ/HpIaw
Rc5411Pttbo+Z6+Ns+x1d90DC/UcKSePs3mj/XV8mL/C22rnYh46zdYu70/7
bfyAOzjVTLv0PMnXMkI4xY38xMwz1M3ETjN+yzu4UM1qJDBLjyIuje4D27HU
egni9pZxDpeZ5V/9uszwjDc5b75+L7i+LXYHm+WNE6uQexwlcjoxxZk1/LdY
A1+M5ucmZKtf0Nc5OBLNypNQrqYn90e9eDYPQ0z+I+tnqEQbmjBefoEZPkWK
/ns4bvZt/Kqa857HZexHJY7hsZoVvAb5KMNn0fOTn8vzMBxvYBSy5KdwJrrM
kuZ+kvGx/DjPxGCMxQjclZ/j/FlYbqYlvADfyK/wjgo860M8TN5qfs96PS+3
/wUvtd7KzfbzeI71B/yl/Q7nP0IdmnEFmfIGRPeLJOe9xqM9gyzOV3NSTjV2
4SgO4Kqa+/1D+Au30M29/cNl8uc6a6ozp3NveeN4gnWc37F/SV4TTqEOZ9Ag
7yCXY43+W1GMzfJ7mGOdd5XKGd79i9zFyWqK5ZdgLXajFLPVjHLWWGeO4LPy
avkT6zqO209yf8nRb8HcT6N3iiR5N7gV73oPy5CP3Oi5O7PIDN9xhWd0lA9j
n5rFznwfeUjgI/RUc8KM32Ov/HKu1asKR/A1xui3wV6LPjdR6HuRQAEOmG0/
3nI9B0/1m6jvPbG76K42iK2ytxL/6fNEfLr4NJS419fFFotNtD8JD5zxuXnf
dD0BrfbuoNMzeZmH4pXou6I+K/p96F/Mt3z7hsgfjEG4Gn3X0CJepN9t9RlR
L87kS+oS7rfFeoM5/uDJ1jc5Zn1DXYH1dev10beVc6LfdfT/Yn3d/krrVbgm
1kusUSzhrFj07U3//3v/HLG+6R0=
          "]]},
        "0.194`"],
       Annotation[#, 0.194, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1H1MlWUYx/HbbJEUIIjAOZblsv7BxbQXXW21orVmbSabthZtjtbKRKiw
MCvD0TRUUvijV5e9zAluTkVEQ4myQ2QhYNap0FbLfCltQ3vZiFh97j++fM99
Pb/7uu7nOTxnWkV1WdW4EMJ7/gxMDGFXYQgHc0K4lCfg6ewQ+rNC6JjkOmdN
CWFPbgj7rPciLxnCoH17ZVP2dXAP7+Ov1FfJH+XP1C5Xy0I2jurVzYfU83ky
CpBW/4S/VE/yFFyBIfU8vY7r1edaSu0w9/BPaof4iPXncS++wFRnq80L4Rd7
C+09J3eP9fPu6UxBCKdxtcw0fCf/bZyjx/fRWJMfwmqslKvHb/ok9Um5/xG9
Gux7xOc1XMGv8HP6b+Dl1q/GNf/tOf2FKj2XojGeS71S76UIeo7DVtda8JIz
FMS5nE6E0G5Wk3ozmmLeegP/5/pGTlpfbG6xHgm1MbOSPMN6sznlMuNjVqYJ
69Qaead7WsvncLdncoD/VOvlPpkTnMZMe8snh7AYW/Sax3ei1Z5MLkSP+kye
jjz5YesxdOAE0rhVvsbZ6pCl/5tc6lk08rX2HHSv/XgHHdiGC/ZU8QpUoBmr
0SY/lxfiOtyGWSiRz+IERp1lkvvJwHL5Up6HqzAbxTglf5f596PSmR7jcmyV
3+651niG7/N0uWf5YetVXOn66/y49Xoecn0Rl1kv4U2unzR/GCkMYQAJuWS8
XwTz5vBNnkEJV9izS6YTr6EFmzEY/1+LQjiPHzHevf3DzfIL4v+0mfdxjtwt
fLt1KT/o+mG5NNqQwn70yr3LrVih/3rUo0F+gnOs9F3lcpHv/iIe5Qx76uXX
4gW8gSbMt2eWWbPNLOZuuS5+Mb4nXOp6cH8ZOOvcI/E7RZA7xj/jId/DYlRg
bnzuZtY6w27e7hm18Ad4255HzXwCi7AM1ci0Z4cztuMt+Vbu0msLj3pnN/FZ
rrN/jr4340a5Bpkb+LjeT3q3quP75ZwbUebzfIzofYcZv8utU3+Gl6hX4l/9
/lAb5fN8AcMYk0nr/Q3uNesB8ztlEj5f4/MMFHk+Z5zpV5mTvM26FS0okbs+
vjNmn/YMX1Y7xQPxN1Wtj/vj7x+KzPvI+bvRZW6+7A/6HbA+xrXmTpW7Ep/G
3/b4O6eeiHtRGH8r9fyYv1bP5YnIQa/6/jhTfZk+l/mciU58GH9v1Wvi++N+
jjhfttmDak9Z5zpXu8wetMXZeu2OZ5ark+vnDOtLsBM74lx7/gf/UvxB
          "]]},
        "0.097`"],
       Annotation[#, 0.097, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV01lsjlkcB+BTS9XS1VYkdgm9EBIhIyFRsdRaQtIikYqMIpapmIkgJhLJ
TIgl1Bp7tUXRKokZIpGKG8tY2guCGbUNhpZaqoN63ovn++Wc8z/b+35vjzlL
pi6OCSFk+slMCCE5NYQUEkgikcn628h4WtKaVkzS30LG0YxYmjNRfxPZlMaO
IcTIwAT9X7W/0cAX/me8/vLkELZ3CeGMPJcYQo+UEAq0e8qudOewdjc5LjqP
PKAdL1uTyEHtBFlmfJt18rXz5VZKO+vnBKc4SYa6W/I2N6nkDpvVH5clFHOM
o5Sqz3LebJa2DyGPn4hJCqHOvu/ZRC2vGKX+vKzrEMIVedW61bKKAe4409xc
Ctp5lnIExea0lB24rH+A7EWy+hrtL5ylmiqGqs9rG8Ia2lh/p0x31g2ytzmX
OoVwnb2c5ShvzVkkV5DDFtZRpj5DTqcPwxhI/+h5y1QanCXFfWL5RX26nERX
BpPGU/Uj7T+Zhc70o5xJofoSzyrPezkoe6lbLmdp/yoXGt8u52mvl3eNz5ZT
tBfIPcaf2L+GCu5yg1R1naL70qhuiBzkGfSXOeaUqvmDfIrYx1/mPPefrOUh
Tdzts9yifpo1xthzvExQ94Mcrp0us4xfU1dFGRX8yRV1+2UxK6y/nrX8pj7O
OVZ7V0myo3cfIxtkrDlr1f/OSnawmUxzBtprsD3T5EV1F+Qq7QqZbrzReWPd
8aWsj94pjfEh3JOPmOE95JJDRvTc7fmzM5yWJd5JkTzEbmvPted8ZrOMxdF3
bs5JZyxnl/piecFaBbLBPnvkPoZG3130LXI++k+bO4ssZpAd3cseh521gEMU
coSN1q2M/rPudUfWW7efdQqje8u+1DjjG0YbH8sGY2PkJ7X1vsPPfOATHyP6
6+R7annH26hG/xtZwyte819Up/+FfMkz/uU5dfqfyKc84jHV0Xr6/5b/cJ+H
PCDDfb4Dge7Ptw==
          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwV1GtMlmUYwPHbD+9kOVOLY1Qkr1mgL1AKooJGs3I20042MIlQ8pBA2GF0
mKZxsDBjqR1M1OaSNtY008qsrdFsK8sv8QE1SqpZ6kLTDlIb6/d8+PN/ruu+
7uu+ngPv2Or6e+qGhRB2+PPY5SFsRElaCNPx96gQEmNCeDEzhIlcmh7CX3LT
uATTseiqEE5cGcJJPqXmR/6Vj8sdQ51+G1Cs3xQU2jMZy+UmiW/GTRjQN1e+
Rj4hnogJOCt/o/zD8m3IlcvB9XLjUCn3Eq515gJnZ/GaK0JYjfnidDUZSMVP
eqVxuppW95TBtyWHMAsb0Ia79WrCHHvvRFNqCMnqmvkO5z4qtxILXVdgn9qL
7vMDXiR+EM9bX2DPR85oEP8rTriOmalZ3IJMs1zmvLNRrZkeQKP806hR96n1
HZw12rPmDgyODGEXHzJLJ7/hjH38HpLU1+qxKnomWIbFeNNcb5tvN7rUv89P
OreLZ9tTpddSlONkhneGIeec539wGjEzDnGZ+m7zPOteDvFneudwobiA51l/
xZmv4Rmsx1rE1A2aMTgzlUfxc2YYzq32rE3xDtGITvXbcdieg9yNvejBkeg5
qS9zVqkzi3lk9K74dvG93G79krph5u3jcziFo+o69N+NzfgYe1AVfePmmOQe
i3hIjzinI82eb9X0ohsD+Blb7Vmubr4zK7k3+gb5EfEWXmF9DS8UP8HZ1ndy
g7iL37FeoX8NVngnd/Gt+CV6nt5DIa7BXJThKfUxHoNB56e5txFIqC/gEozD
fZiNveqbuB1VaMRKnFf/Lu/HNnyDzxFX3+b8Ms/gdR5hptXcgKn29HhX/diP
/zCA0fZkmyUfX4hTOCl6f+pv4bnYJb+UK6I69T3efT++1v8wX3A/n/Ase87w
OWxE3DPaxK8iFn2j+uy0p5oXYwmGzNbkf+UFrMOHZj+AOdY22zdcj0zXSbxF
XO+6DgfEtc49jmPowzpnVKvJQ4qaZFS6TqDTuVnictcVuCTu8B4H+XG0u17F
B6PvS6894u3czVt5G750/RYfMfM8M7a4/kqulVN8m8noN0OvPlfLl0ffndqj
OCH3vT0/oA/fyd3PY+3JxnX4zd54FKu9gU+Lx/OZyHJxPZdE35bc73J5nlc+
CrAs+h2W/0N+cvT7jCLUyhfJX5CfKp6GXL3W65XDxdYuWpvCf0Y1PEG+0mzN
nlML8sUPifP4Zf3qMUPdTJTqNwMz0SD/P9z648U=
          "]]},
        RowBox[{"-", "0.097`"}]],
       Annotation[#, -0.097, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Gtsz1ccx/EjGqIMRdu5dPpva5qt000sZIj/sliVaqkErf3TrNMa7Yqo
yNwi7rpYE5cNm2ypW1LLTPrAJRKr8GAXPCirZEXHbLPM2B4gkobXLx68+/6d
7/l8zzm/32kbq1xcWtcthFDvx+YhISRSQtjAm7ARJ/qGsDgthJP8ytAQ6jyX
DgqhV39181Plp6aHUIRpSFZvkdsjc5w7+oUw8MUQrnODWrp83PiR8XT5YpSg
Xr3Bem/xfB4fjfVXec6Qv+D5jp6XZUciFwlnGq6eIfuHuZfk/uThPFffHKwf
EEKlXEwuR67EeAY+UMtSG6H2j54NagvURqiNVJtp/J7+XM9lPA/l0TdRr5V7
wx6jkY8H+l/nGvVRfN/4VX4NeVikfm1gCNWcE72HNduNf8Ftz9+r/c7ZuIV9
xr/xVfM3+axxJ8dwA3uNr/MV85ncgUy1CutfVivnrcj2jbJw23mGOMdstSnu
oADv4kZ0B96pGEVItc5HXIcp+gqxQ89d478xy7uXYpjeObLbo+9nPBtHzDeq
rdazBknWfuAsQbYbeshU8RZzW9Hbc7vfpUt8GQmsVF+F6ugukOl3aSIfRRNO
yx/mz+1znL9BT/vUerelSGABKrHHub+2/yE0y3/H9fZo5gI9FdaqxlzcHOx+
0PWCu+OH+AtJ3qmL4/KtzrLS+57mM9bO5THG+Vxs/lN77sbH2IJ1SJJ77IxP
kYq+9l4VfQverGddagjbsAKH5ffjvJ6T3IpjaMOP0d3Jx+01wZ5juY9cIU82
LuVG84/kgvN28L+4g4tyX1r/EHbiBL5FhXyec4z2jm9ylzWyOB1pen6WaUcr
7uEW9ur5UK7Englul9vGVca7eKH5tVxuvIxj5r/iJcbNfNB8mfXnY6E7mc5v
R38D0fd0D2MwDEWIY7l8EvfHY/unebdk5Mnn83hkYxYKcCz6O+ZGVGAFanBf
/gi34Av8hLPIiv4n2T/uG3zGyc60hpdgnJ42d9WJFjzBPfTTE3OWUThnPIh7
RvcnP4mL0KRezWVRTr7N3XfiB+uf5/+8zyl+R8//kZHj+9RyDXroed8aB+Vr
uMHfxCd46lwZ5hapdccB8yncxL9aswO7UJjy/H/5MyPGzO4=
          "]]},
        RowBox[{"-", "0.194`"}]],
       Annotation[#, -0.194, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl1Gtoz1Ecx/HDUG4btrn9RwgjUSL3DElqqI21yVouG0syQ0KEyWUPKLdy
mUVs8YAHy8zITB64PmBlcyki5S6KuS28fnnw3vv3PefzPef8fuff+i8uylzZ
KoQwzp8uSSG0dA2hPBbCb36fGMJbvMMs9areIaRzMWfFh7AL79Q1XczpLcb4
BM/dQ5jAl+RqUYMruIwK+Wt8lq/yGa7jDt1CuKhvn7qa90fr8hJzRzhXXeJ5
Adc63+Ie5rAmOYRB9h/Pg3ktX7f3N+s1Yxq+RHbWS/zZWjf4lnWecwOGy+fo
K8BJ75DOk1Ghpx0nRmsaH879EC//Qf0TVXiOBoyVL/KtNqG99Q9xmjOXcn89
db1CuIOjqEIlPulZzuuQh70owXn5GZyJAZiAERgm356T8d1ZErxPHNbKp3E6
YhiFVLyUnxLdHwqdaQnn4JT8Wd+ryLcs535yq3m+ejMXmj/A+erd3Gg+l+eo
l/Fh8y/s/wH1aMRdJMt1j94XLXKjeaRvMIzz9JyTuYj9OI1juKfnVc8QPuIp
/qIZe+UzrDHdnjO5o9wYnqhO4yzzt+UacB71qMENuTKuwDrr78YW7JBv6xwb
3VVnTnL3f/AdcXq2yO/EehzEHszWM8Jeo+yZylfkanmDup7TzLe4izjv+Jq/
RXeKls4hNPEzZLuHAuRhhvxJv9sTWKG+Gf1meQ22O1ed+W18lUt4tfFiXLDv
EHt+cYeVKNN/HJPk2shN5D/O8Bf5xgvwQ08nc4Wel2EpvurNMFZt7rVv8AYP
jU01Ng07jGfipT2zOWbPF56HRr9xfoDe8g/t0xjtpw7WzZG9Z60F3FrdCpnm
M/DK2Qaqmzw/whM8xn35x9Z6Go1xg/oBFlpjqzvZhpjxPs6Qa6wUc2Vj7iUF
g6w5T93XfIpcnLoN2mKR+leEnoSk///b/gHFmbjm
          "]]},
        RowBox[{"-", "0.29100000000000004`"}]],
       Annotation[#, -0.29100000000000004`, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl01toz2Ecx/GHolE2tv8cy2wXYptDZDm0v6SZyLHV5jCbkeXCf6Qhm8WN
uSGk5LCIKYdNTcxutLUbpzK7GHLhkLiZWWGHUvL65eLd+/k+z+c5/H6/55dZ
UbU5MSKEMIR500IYkxrClUkhfJsSQtv4EI7ou6M+nhLC6FgITdr3cBcPJsil
hfAVQ6hTD00N4S//5kEMYNjYVn07zJltvTLu1/8DO7XLcWhyCFn2O8yL00NY
hqXosO8rZ+rCCnTgMfonhtDJT637gbuRK1tszm5cd9Y1nI/G5BBGcWq0nv5c
zsA4+V71MFrwAd3Ik084cw2SrH+B4856imeY88S7eY5LaMEt9Jmzl6tRitM4
gWb5At6ETCzBHGTLJ3EMg86S7HlG4qB8nNdgKhZgJj7LL7f/WlQ6UwUX44b8
be8t4b02cIbcAS5R13Kl8fO8S13PPca38Tr1Hr5o/JP9e9GOHrxATC49el78
kVvI872DbC41p0nmEc7hJi7jpTlffL/veI+/GMBp+Y3WWGnPQh4rt4iXquNc
ZPyZXDea0Y5WdMpd4UZUW78edbjvvpyN9jM3xZmSsVpmnLqAC7EKDdEz6Fuv
vREbojPpi+l7aO803u7eBFxTj+CB6A7jqnqQW+Wr9O9HHhLYh+/27MM736ZN
tkT2hHoLv9b3lrv4Dee4U9k4at5Hz5ql/YlrojurPQe12scwUX6WeiXuWbfI
/ifxU/+v6B+K/i3foJyny5TxGbkcc4uQhLnqfOunGY+n/P+v/wGbAY9v
          "]]},
        RowBox[{"-", "0.388`"}]],
       Annotation[#, -0.388, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[1, 0, 0], LineBox[CompressedData["
1:eJwl0ktI1FEYhvHP0ZpNTo5jTo4x5aJF2KIWQYvGgtyMWXQho6tkNI2CFdGq
IGpTBHa1zCASK2jTQlq0Ghq6ELpz4VZqLdEuwtTqJy0envOd857v/M+Zaeu7
cPB8TURUkGyMuJuNuIf7aGyKyK+OuGM8hMVcxAJaGyI+Nkd8QknmtrV1cgnj
fpRxDuPpiM8yp4x7l9f0H0Cb/bu4inf4IfOBv8jP8jTa9etZE3EGY/YWeQde
piLqOI2q+XbOY5X8nPoXJjCLaWyTH8xEXEFS/2EuuMMtXm9PpSViEqOYwCt8
t6fMl3ECQ7iON/KdvB8bsB2bsUk+yRn89C317lODS/IFLqIFW7ER3+Q7nN+F
km86zT0Yl3/tbQZbI55xXu4iH1Ff5ZL1B9ynvskz1o9xt/osj1j/6vw5vMcM
ppCRa1q+L37LTfEkjhrv9NYdKKir9peNr/k9n3u7rPUDayOa+bC5MX261Cl1
kbuxB3/N5+T32rsPTzGKk0hg3nvXckqmHvPy/fo9NK7Va7ceCX7rHgveZxGP
rNWZG+YV/JgH7HnCK9Wd9owY/5FdwiE9K85JO+eFPo3cgKzMDffaYi6jPo6l
3P//+j/jk2MR
          "]]},
        RowBox[{"-", "0.485`"}]],
       Annotation[#, -0.485, "Tooltip"]& ], {}, {}}}], 
   GraphicsComplexBox[CompressedData["
1:eJx0nXmAjVX8/29JSipLSSFEJFtS9jq3tFKKiEIpIlJky1JcopWyG6HGllGW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    "], {{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {{}, 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0LlKQ1EUheGVRkFQjJhE7SzEofBNbHwE20DszXNYOYBDLxJ8A+3sHao4
4hCjcZ4Kv2Dx87PW2Xefe+/4wuJ8rZBkDs8jyfZQ8sgdPKE6nDzwhr7Fm/zD
v/jEN76wbG5gNDksJhW5jC2zbW7rznlN7uFeHOj6ze/zq/yGF3zgHatmm7zC
Z7xj7pKvcIF1/TUf6Se653KffVUueZelseTY2WQpmUJ0N5XkFndoyHvY7X6f
3MI9CvKM+WmceL5uT1lXs3fQ/gbPuu+0+P+//gCrJDmH
          "]]},
        "0.396`"],
       Annotation[#, 0.396, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0jtMk1EUB/ADDKA8Kk/7MIEmSFve8lDRxIFRwYGBsIEI6qLM7BASYGVl
4DGwAXF2pNZVN1tXYYeRhF/D8Mv/nvOd7977Nc0ur819q4mIDVIdEYPJiCFe
dUYk1ZPpiJyckH3yA8s8ZUyvV47LrPmL1ojZroj3vGWGdxzon5rbNbPDNgvt
EcccMeC8fgqctUU0pSK+6jfKU3Wffo48B/onzjuU82zZq/Q44hdHzth3VtH6
Jy+cPcVLSvqTZhP2aKG5mubf8MyzR/Lcvq0ywXOzT8xkaFPnPKuxDvbc6Z/c
dL8NaqmjW6+HP/ZLqH/Lde91eL+dfuuMLMi0XGKR19Wz5LT8KFfIM6Kul6Ny
mCsu+U+huqdvLFtX+Kz+Ur03n1jl1l2+u8ONuWseqBso6j2UF977K8tUWPQ7
/dDLp+//C3fgCD1N
          "]]},
        "0.33`"],
       Annotation[#, 0.33, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0r9LVnEUx/GjLZaL6aOPulk4iC46WFkN6qIVBE46uAgOYmggFELZoOAf
oI9Cgm4tkYWrNSjRlIqCPwZ/LI8KGSjhjwyMXpeGN5/zOefc7znfe29FV39b
X05EZDBZEvGrMOJRccRjPMEf/kL+N05xjjNspSK2b0bs6vlYFPGDHuEn0siq
FacjSpBCKdK47rkbeFEesaDnJc3n83BP3IB63MUd7Ou5Tx+guTSiCY34ZOae
ObvqBXSH1snXokZvNQrMe662Ie6nTxGem7Z/g3u9MbNDrjLZKzmPv83fwl/x
LP2AVf0D6nPiS71b/CC/TjexgSv97+wwrieDdrTId5r9UL5VXE9H0aeW4ouS
nD0W+Wv2zsUEv0cPzTnAEL/CX9LXyd3oK1pNx8ointkjq28fK87r5b/TZSyh
yrxhNOptwpjcOObt9QWf8Vb9q9wUHbHLMHqcX5PczXnfaDedkd82Zwe54vfJ
9xdPqJ/oz9BjuuZdZAv//0//AJIbXOE=
          "]]},
        "0.264`"],
       Annotation[#, 0.264, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0jlPlFEUgOEDCNoygyxDXAaXwAiVjBbCMDKGEQUXCgK4FNQKamOinSau
qBUUUmpjJ/APrLWw0hgLkBoTK1kKfW4o3rzfWe45904mPz07NlMTER/Qnon4
hWpTxCqP5yI6GiM2sxEffW/xX+xgSbzNR9Rvt0bM4BbuYBZH5YstETX6Auvm
1XIdTsmfxjE9X+0q8FlxGfXqe/BHf6Ut4hwG5Ss4oW9IvKPWoOe7/VvNEd94
mw+YdRgraoe4b3/EQe5wtphq3Mv5NEutHyUU5ErO9ONpeqsdT9Ib+ZOdK/xM
vMzPeUBfGW3mtCOHbnMyyKLPvONyq77X0Iii36QXJ5FX73H+pZ659FvYMczn
E3ZUccYbL/FlXMQoRtI99JYwqncEV+WuoKL/Bt/EdTS55wt913wv6ntj72tM
pTfIT/Jb+Vdyc8jo/+JOWd6He9iL93LvcNd3nXvX82dxA/+z86dc1Zwh/Ehn
1Iryj9J7+TFv8kMu8APu4rL9v7nk3AAumDOMDbkJMzrd8b54TTwvXsjt/j//
A/2eUig=
          "]]},
        "0.198`"],
       Annotation[#, 0.198, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl09tPznEcwPGv+0yHp/QYTwcXoZOe6EB3JmOonorF5tAFmanJYYyL4oYZ
c+G0UQ73LpA/QDZqNsaVnBJ6cmHDFSbx+s3Fe+/v5/j99ntWUWdPqntWCOEX
RrJDKIyH8JiLuCcWQiwrhM58ltvJOVyhVolyPBcf1ZODver54mNzQziOMecL
80J4yb12FeppF4+LG8V94iIcNpeQ6zdzAidRIr9Z7xf5EhzRsyi6i39EvZzk
Pq7mGWzVv9zcabl68UV7LuEyGuTfuvMd3uSEcFvva54QZ/JVcRavNndN/wAG
0WhuUs963qFnsZ61eq6rzRFnIjuaRVrfB/Uc54/8WTyFZrMD3rQx2s1NPMgt
8je4OdrHt3ATKflT5rp93zJ79kXfmlNy59WWOJdiXfR78Tb3rZE/h4R4t/4C
7uJGPds5L5r35la04YzeDfblyjeJm9GCfrlW/W1IYRPacSg3hANosC+e5269
ddGdfBYV3lBstsb7a/FeLaZ/gqfM33NPmoe4Qt99/eXR91Ev5SFxGSeRsK+K
l5nP4GquQhJLxfPVK3kBp83ny8dx1+5P7rjDkzyuNqZv2nv/4DfG1P7ytL/j
ijtnnMv0dTgPOO+xpwu7UKBnpT0NqMMK1OOJnbU8qV7DaR6VG4ne6k3J6P14
5q6f6q/Unzq3uiPurkf68rhFPCr/Qn2Em8W58sPqDyPkvpsf5gfib9HvoP6V
e+2fbW6Lbz3KC+WKsV8+Q9whXyBO4KCZVbH//9//AInihm0=
          "]]},
        "0.132`"],
       Annotation[#, 0.132, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1FtMiGEcx/EnxhxmFBWN0aYlZg43CpHJucyhw5ityIRNuXCjKNOVqRyG
cSO2bMh56agD48bmsLlzuHHj3AVzzOnzXnz7/X+/5/9/3qen9y15S/m6spgQ
wlQ/MseEMCQxhG+jQzgdH8IZXIoNYRKdKK9PCOEIlvKp/G/61Xo//Ub/0Cny
f/Q7/5f+oMFMmnwA/cnH0F90IJ0mn4q4cSHEol++iX+rfofbzvKetjhbK1r4
7CR7jwqhjY/Rnzs2hNXIQUycXM9TPa20XU87vcx30E7rHbJB5lbIOmXdsjt0
JT9Y3ml9KM3hu+Rd/B10q5tkPfSemT7n6lV/oJ/wEd36xtMJKHCeYfry6XB6
V2+u+RH27tHXiweyz3rv0y+RyibTFGw1F2+uhCbQmbJndIb7mYnh7m8cX+ye
N6PSvkl4aI8M6+lYLn+j57HsEY6aOYaRmCQvtZ5MT/NnkMuvxlN1mTM1qvvU
zz37Ip/uLBl4wV/i56rn4YCeZvvsp9WowmHUoRYZzrXWXmvwBZ/5KnfxWr3A
fCaa7PfKvvPVL+ll/omeg+oc9Xl1Lq3hW+25zuw52oBMz94T7UWT+AtoxEG/
cw3GmrvIJ9Ji87Wen4XxsgnYa7YiukNsdG8bkKx3h95V9jwRvV/0JN0tK8ch
+663R4lzzdKbxx/n87BVNltWYOaKmUJ6lVaY24vDejfpK9U3R19+9DejZ/l0
WsvXYZ66QTaXntRfhO38fH6jPRfSBSjUW4BF6nPWs2g9X6q/jM/m90Xfkd/r
tnM0o5I/ZX0bdulZrOeomSPYKevQU+IZ7XQLrdJfjf1YrveWe7uJA9H7Zz1N
vUx+g16P3gN5sXxK9D9Dfo1ejd4NeZE8Vb0k+jbpFTSh13OnW++hv5y1TZ0S
fZfRd6zul/0HqYWq4A==
          "]]},
        "0.066`"],
       Annotation[#, 0.066, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV0ttPj3EcwPGPZhhlckpF5t9gs7nDZiYzLKFfB/3K+hXJWGYOG2XWrJ9c
OFU/VDTSkWTuXOACXbvvwuRw25bXc/Ha5/d9P9/n+zw9a1sqU960JCI+0lgY
MVMU8Y5patb6vSmiXn9r/YYpjumTerU+YT3OGBX6mH5SH7V+zQhH9VG9Uh+2
fsHz5Lo2QoWeZUgb5JX2kkNaNzmtnydajn3aHfq0Xu5pPezWuri0IaKdrRsj
ytil3aaqOKJ4nedoJWa/dY7H9NHLBA94yH3O2/fIbDV/uO+WOW7daa7y7IL1
ER88a8HvjhKdb97lO7N85Yr7rnKda/yz9y9/knd3Vg/Z5Fs7Z4b33LTvBp1M
utZhNtNChiam7et2Lev+u/xy3jy/k+/o2hADdHmnZ+ZTLtu/070/fY+0d0/7
Oz5ZN5ifzTqzzP5Fs9TcwmZm7T1HG3PO2OOs7Z6zgy9aqfPaqdZTFDgrnyLt
YsK+C9T5HrUUam20ameT/6nk/4DV2hmatQxHtMPkay2c1hpZ4R0O6iu1Bus0
9SzTy/Xl+qnkedSyVD+g5yXvaJ2iijx9v764JuKE9XEqWbBvrz7l+n8V2Xcs

          "]]},
        "0"],
       Annotation[#, 0, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwV1ElsjlEYxfHbmFoVamjNEZWUmBILdoKKsdEaSmxa2pj6tYqWEiJpDXss
TCUR0a5QiakVpWK2QLGhqCAxz5aU312c/u957nnufd63b77hxesWlieFELL8
aesdwv30EB7gYaylI7SsbwjHB4WwHAupe/8QCvC+Wop1kL8Q1xkhzMTu2I1C
nxCScbxMV/wn1xk7sAuOUx9Ll+Rm6Esf6A7+r/2cASH04/vSWn5avxCa5PbI
NeJavrP6bL6ZHyC3Qm8n6/l6+/MZVC7TiWbJXbbXhVpoL38V19lLjvfx1/hu
NEjfYBpI1/n19lMp37kp8W73lNNN6336bmC5eXrJ5PK3+Gy+kk+jAn091Tbp
qaKRzh1FWXRPfU2cF1djgkpogr0i73hIvNeZbfEdOXM7P5QqrTfSH5nZWKOW
RXe819Vqt3EN3sV2dx7zf6x2TqZz0tQSfCmN0zOFr1Z/bP8JTeOv4FQ8Y/+f
eertd2CSuQLtiM9ovwan4z4zJGg/LXZuPi2iJDO0y7+il3SSf+3MFue/sX8V
32KGM3Y5Px038BWUY+7H/CY9j7AKn2AuNmEeVsltpi2UK/9Mfav6U9yGbTgf
h9l/gdNjzox5lG2OlPi9UjL1oFQaIZcwayaW4mvZMjxkvsN0kOZ57locL7PB
XiVV0AL1E+aoo3pa4o7P+r/Qe9nTzn+HX/mJeJSfhEv1nZI/TQ1UqO+bTBGW
yHyQmUwf42x8sfp3+yuwlP+knk2fqYxfqf7Dfgk2mrPA+RexEL/G9ynzBX/K
fMMG/jv+5n/F59XXLL9S/jKuwhrPN1emGufg6Pitxbp3etb6HOWoj4nfnvoO
9fPWzZ7nQPyNsG6I347Mc+uduJvq3L0LW+Ufyj3EVnyEHXr/A+O2pdE=
          "]]},
        RowBox[{"-", "0.066`"}]],
       Annotation[#, -0.066, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl1FtMz2Ecx/Eny41JSkgbm0056yA3urNc2bRKUzaijQszx1I3auhsCZli
uKMDmxsxNyWkDTnM8cqYw6SrNjKGvH5z8e79fL7P9/t7nv+vf80v21OwOyaE
MMWP+MQQKmeHsHROCMvwSB7GdJQmh5DAT9CWEsJj3qqWyFVmvumvSgjhOw8m
hXB/Rgjj1kP8g5/qe2j9zOwDHtLziK/Kw/za/oS+59a/+S/+RHP6VvEBZ5Vj
P+bqreCv3G5+lNNQ6x51qEeGu9ToyVRvkpujulo2GtQbcUqtDaeRo35MbY3+
Dvks4t3lnOdP45P2cu2d4LZoFjvcq9791pkt1ZemL19exOflhdwg59kvk9Pl
QrnSXIZ1OR9EBZrVN+nr8dwr6EKR87q5RH2H+WwzVXo73W2DvZXydvVi+536
CtUucQFf5o3qo567nrfp+6z/qzyCmfJHOYk/8SxO5Wn8nuP4HX/Q+xELrEtn
+qI4OwZv5UlchM3qY3KsdSvfcMebiJOnIs2zutyhW63afhfX8GRzsbhl7zr3
oil6F2jEK7OvMeH8dntncGeW9xS91+jd45rauJ5aZ9Thns89iL3ewV2u01+P
3uh3p7cVLchSy8SYs9M5A03qubwWR/WXOLcYQX2X/MbzT8oD8mHrI7jtjAHs
tN/Pq83moFKu11eH5XKfvRV8Ub6AHuemchr++jxzPOsPp/AvZ/xAh55+n60P
+fZ+Rn8PnC3Pi77fXI1DGDD3RS3XOSN8XG8rWlBmZiW/dKes6L1yNuc644V1
gdlkM3vlfXiolqeWqFaJxXqXYEvy//8R/wCHOJ4b
          "]]},
        RowBox[{"-", "0.132`"}]],
       Annotation[#, -0.132, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl08lvzGEYwPGnqiQSoqN02hLambopyRxaJNrSppaL9IDgQJerHjlYLobG
gc44WS62NnF10C1CREITxDLhYEn/AnS0sVx8fnH45vs+z/ss70ynjX1DvScq
ImIG2dqI7TURGd7GO+sj2riTt/Iu/pyKWLEmop4HV0d8c56UP1cd8d35tbo3
qyL+1EW85b/8RW3ZXUn8g9/zXGK1O9y3oyh+ak5W7cl0RDMv4LDcZrPPym0R
X/S2YeTlcsjLX8AVuREUkrfLH9W32J4OPZfcd3KDHWMYxXm7D6k9iANoUNvn
HfvlB3mduJ/X8gDn5fea26x3IzLYbWaWj+hv4h7xBm7EQ/snMI49+irVdHGH
OV/NzMpXOTcl70y+c37kfsKuSYxjGlMomPvBZ/iI42Yfwyxm3N3DKO7ijLlj
fJpL5lX6+zzAc7kXSOuvwyt7lsm/5Nvm3MEtLNKX5jb7rrpv5VrxfS6KC7iM
IXFOvt27U9wiHpbfxNXJd8tLfd6yeVU8x0u4Qk3K/mqsxBNveox59df13cQN
7DN3udqM/Cnnd84lLou71f/kLvO60eIzzMo90zeF6eT3iF47f+v5hWtqCmr6
5UZ4gIv8yZx59wtoRY8965PfGgac29GBbnQhV/P/f+QfGqlvrA==
          "]]},
        RowBox[{"-", "0.198`"}]],
       Annotation[#, -0.198, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0t9PzXEcx/G3jdiIRcRhtvoHtEZXmpl154rapHQRN8y045wsP/rB3KDy
Y2uVaNVFh3DRnQsRkuHCZjslbmwuktY0c2VreTQXzz0/7/fn9X5/zndVWFd/
6MyKiBjB7i0R6Y0RE1sjUjyeiDiq90U9jSt5EWUFEXtRti1iVubj5oiEuz1y
59RvnRv4jdkxuZdIucvoneUHnOadcp/sW29+A+5tipjR7+UfPJjPMrOYwRx+
YsHMsPwv/s2nZE9iRH3eTNK7F/i2PQc5o77qrgRd6m4MubvLPcjqN3OR3YXI
2vnQb1zt+9ZgJXKwChm5++jDOrm1SMpWer+Ns+Zv8Dtvfvbd0zjurkuvjrv5
BBfLfTXbIvfH+3+dq1GDTr/tu8wB/Vr1sLfaze137uBH6kV7q2RuqY/wTZ60
s9xMv/sBtNqTtv+ZXo7ZUW5QX9bfJ9/BjerX+t+Wv0fdo87lJ+rHOI0K+6/J
H+br3O5+SK6Nn5pNOTc6L7prkt/FCe9txyX1RUxhEs/tqJAd5Up+wRO+ZV52
zq7y5b8xv7Kz1F0tjqFV5o69H/Rr1O+5mpPyO/THzC8V/P///QcN3m/g
          "]]},
        RowBox[{"-", "0.264`"}]],
       Annotation[#, -0.264, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0s1LlGEUxuGTFGFGUTrOGH1J7mtXCyFXtTfISFetMyXIygyCIAX7A1Q0
M4vM0akWGdSmnJkajaRILBWiMIWidkYtu15a/Lifc5+v5314a8+0NZ7bEBFT
qEtFrFdG/Ebbroh7OyIeVEXcpf01EQPoQ7v8Mn8Je/Xsxh6Uiffpq8V+jOib
z0Tcpu/TEe8wLW7dGXFdfkXcTUu4yuuhW+VL6ivoK3En/63zF7Vf0S/u5Q/Q
LszJXaGveQX5Iob0P3SXQZpPdiIjn8YJd1+tjljDN/zCTyz6rlY6oadd3Vnn
Ed9UQB6b7LnB30iL4hKdwSzq1d7hjaLD+QImzTntDsPJ3fiVSS1vlPdC/hiO
Iyv3mRbsL2JGPIs32IyUvipMy+WxxlvFilkpWo0jOIxms1uQxnbfvA2L/B/m
f8eC8zwuOV/ES4ypXTDrE/8jsu77iD7hDcv9NeMPHvN65Br1NHvDPD2JJkzJ
PcVzPV3iZzTrrhMYRw6TaNHXS29iycxl3Febs+caOpN3s+MyzdFb6se8+RD9
oGcLbaIV9BQ9r6Y82UMPeKM6HFR/CEedGzL//+l/nAJ3Rg==
          "]]},
        RowBox[{"-", "0.33`"}]],
       Annotation[#, -0.33, "Tooltip"]& ], 
      TagBox[
       TooltipBox[
        {RGBColor[0, 0, 1], LineBox[CompressedData["
1:eJwl0r1LlWEYx/FLXHT0mC8FDtIfEI0O4qbpkjUoCjYIpZAlOCnmaC0WkmK+
gYpYizacwlrMzYJzfH+lJdFRyqIXDR38HBy+fK/7fn7Xfd3Pc05pc/vdx1kR
0YfdaxHlhRElxRFP1D2oVFfhe17Ej4KINjxCXlFEAtevRNyUm8uPOLga8ZYP
eYJ3eIq3eJK3M8+dc6DvvjkPcOysX/iJlGc5nIsB+SX5l/zOfqtskjvMalHP
yvQmIgbNH8BrjGEc2Xqq9dZgX/YbPqvn9X/iRSygX+5c/gwfZU6d+R9/cYJ/
6Lb/Qd+6/Ij8Go/yBg/xKg/zMr/iFa6Qb9Tb564z7viC662fcx3/wW+Zat/1
Fr5a3+A7Zj2Tv81PuZa3fatp2aQ7bqk3sYEGs3bN2sOR3BvrezJNKHNeUv97
dbv9h55PWn+xn8YyBjO/l/20/hTqM+/g3BWsYkbvhLlpdSqD+3U6I81dicv/
ygVHHmTm
          "]]},
        RowBox[{"-", "0.396`"}]],
       Annotation[#, -0.396, "Tooltip"]& ], {}, {}}}], 
   {GrayLevel[0], AbsoluteThickness[3], LineBox[{{0, 0}, {1., 0}}]}},
  Frame->True,
  FrameLabel->{
    FormBox["\"x\"", TraditionalForm], 
    FormBox["\"z\"", TraditionalForm]},
  FrameTicks->{{{-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.75, 
      FormBox[
       RowBox[{"-", "0.75`"}], TraditionalForm]}, {-0.5, 
      FormBox[
       RowBox[{"-", "0.5`"}], TraditionalForm]}, {-0.25, 
      FormBox[
       RowBox[{"-", "0.25`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.25, 
      FormBox["0.25`", TraditionalForm]}, {0.5, 
      FormBox["0.5`", TraditionalForm]}, {0.75, 
      FormBox["0.75`", TraditionalForm]}, {1., 
      FormBox["1.`", TraditionalForm]}, {1.25, 
      FormBox["1.25`", TraditionalForm]}, {1.5, 
      FormBox["1.5`", TraditionalForm]}, {1.75, 
      FormBox["1.75`", TraditionalForm]}, {2., 
      FormBox["2.`", TraditionalForm]}}, {{-1.2, 
      FormBox[
       RowBox[{"-", "1.2`"}], TraditionalForm]}, {-1., 
      FormBox[
       RowBox[{"-", "1.`"}], TraditionalForm]}, {-0.7999999999999999, 
      FormBox[
       RowBox[{"-", "0.7999999999999999`"}], 
       TraditionalForm]}, {-0.5999999999999999, 
      FormBox[
       RowBox[{"-", "0.5999999999999999`"}], 
       TraditionalForm]}, {-0.3999999999999999, 
      FormBox[
       RowBox[{"-", "0.3999999999999999`"}], 
       TraditionalForm]}, {-0.19999999999999996`, 
      FormBox[
       RowBox[{"-", "0.19999999999999996`"}], TraditionalForm]}, {0, 
      FormBox["0", TraditionalForm]}, {0.20000000000000018`, 
      FormBox["0.20000000000000018`", TraditionalForm]}, {
     0.40000000000000013`, 
      FormBox["0.40000000000000013`", TraditionalForm]}, {0.6000000000000001, 
      FormBox["0.6000000000000001`", TraditionalForm]}, {0.8, 
      FormBox["0.8`", TraditionalForm]}, {1.0000000000000002`, 
      FormBox["1.0000000000000002`", TraditionalForm]}, {1.2, 
      FormBox["1.2`", TraditionalForm]}}, None, None},
  ImageSize->504,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-1.1920937271229854`, 
   2.1920937271229857`}, {-1.3920937271229854`, 1.3920937271229854`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  RotateLabel->False]], "Output",
 ImageSize->{508, 421},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[54]=",
 CellID->70145224]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.3.4 Jump conditions", "Section",
 CellID->229580123],

Cell["\<\
Jump conditions are discontinuities of velocity component, velocity potential \
or stream function across the panel. The magnitude of a jump in value across \
a singularity is related to the strength of the singularity.  Quantities with \
jump conditions are considered in this section.  See individual function \
pages for each singularity for more detailed information.\
\>", "Text",
 CellID->147816116],

Cell[CellGroupData[{

Cell["Constant-strength source panel", "Subsection",
 CellID->164768312],

Cell["\<\
For a source panel, tangent component velocity and velocity potential are \
continuous across the panel, but nomal-component velocity and stream function \
are discontinuous with jumps related to the strength of the source panel.\
\>", "Text",
 CellID->13291475],

Cell["\<\
Note that the jump in normal-component velocity is equal to the strength of \
the source panel.\
\>", "Text",
 CellID->162142882],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"icw\[Sigma]Minus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icw\[Sigma]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icw\[Sigma]Plus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icw\[Sigma]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icw\[Sigma]Jump", "=", 
   RowBox[{"icw\[Sigma]Plus", "-", "icw\[Sigma]Minus"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"w\[Sigma]Jump", "=", 
  RowBox[{"a\[Sigma]", " ", "icw\[Sigma]Jump"}]}]}], "Input",
 CellLabel->"In[55]:=",
 CellID->152001704],

Cell[BoxData["a\[Sigma]"], "Output",
 ImageSize->{19, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[58]=",
 CellID->386286322]
}, Open  ]],

Cell["Compute the jump in stream function across the source panel.", "Text",
 CellID->82001642],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Psi]\[Sigma]Minus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"ic\[Psi]\[Sigma]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Psi]\[Sigma]Plus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"ic\[Psi]\[Sigma]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Psi]\[Sigma]Jump", "=", 
   RowBox[{"ic\[Psi]\[Sigma]Plus", "-", "ic\[Psi]\[Sigma]Minus"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Psi]\[Sigma]Jump", "=", 
  RowBox[{"a\[Sigma]", " ", "ic\[Psi]\[Sigma]Jump"}]}]}], "Input",
 CellLabel->"In[59]:=",
 CellID->301350322],

Cell[BoxData[
 RowBox[{"a\[Sigma]", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "x"}], "+", "\[Delta]"}], ")"}]}]], "Output",
 ImageSize->{71, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[62]=",
 CellID->330826737]
}, Open  ]],

Cell["\<\
Note that the jump in stream function is consistent with the jump in \
normal-component velocity.\
\>", "Text",
 CellID->93140049],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"w\[Sigma]Jump", "\[Equal]", 
  RowBox[{"-", 
   RowBox[{"D", "[", 
    RowBox[{"\[Psi]\[Sigma]Jump", ",", "x"}], "]"}]}]}]], "Input",
 CellLabel->"In[63]:=",
 CellID->114614373],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[63]=",
 CellID->125439526]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Constant-strength vortex panel", "Subsection",
 CellID->486795743],

Cell["\<\
For a vortex panel, normal component velocity and stream function are \
continuous across the panel, but tangent-component velocity and velocity \
potential are discontinuous with jumps related to the strength of the vortex \
panel.\
\>", "Text",
 CellID->48476432],

Cell["\<\
Note that the jump in tangent-component velocity is equal to the strength of \
the vortex panel.\
\>", "Text",
 CellID->478267279],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"icu\[Gamma]Minus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icu\[Gamma]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icu\[Gamma]Plus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"icu\[Gamma]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"icu\[Gamma]Jump", "=", 
   RowBox[{"icu\[Gamma]Plus", "-", "icu\[Gamma]Minus"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"u\[Gamma]Jump", "=", 
  RowBox[{"a\[Gamma]", " ", "icu\[Gamma]Jump"}]}]}], "Input",
 CellLabel->"In[64]:=",
 CellID->263300400],

Cell[BoxData["a\[Gamma]"], "Output",
 ImageSize->{18, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[67]=",
 CellID->17961284]
}, Open  ]],

Cell["\<\
Compute the jump in velocity potential across the vortex panel.\
\>", "Text",
 CellID->41162955],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ic\[Phi]\[Gamma]Minus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"ic\[Phi]\[Gamma]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Phi]\[Gamma]Plus", "=", 
   RowBox[{"Limit", "[", 
    RowBox[{"ic\[Phi]\[Gamma]", ",", 
     RowBox[{"z", "\[Rule]", "0"}], ",", 
     RowBox[{"Assumptions", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"\[Delta]", ">", "0"}], ",", 
        RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
       "}"}]}], ",", 
     RowBox[{"Direction", "\[Rule]", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic\[Phi]\[Gamma]Jump", "=", 
   RowBox[{"ic\[Phi]\[Gamma]Plus", "-", "ic\[Phi]\[Gamma]Minus"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Phi]\[Gamma]Jump", "=", 
  RowBox[{"a\[Gamma]", " ", "ic\[Phi]\[Gamma]Jump"}]}]}], "Input",
 CellLabel->"In[68]:=",
 CellID->224778660],

Cell[BoxData[
 RowBox[{"a\[Gamma]", " ", 
  RowBox[{"(", 
   RowBox[{"x", "-", "\[Delta]"}], ")"}]}]], "Output",
 ImageSize->{62, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[71]=",
 CellID->63310290]
}, Open  ]],

Cell["\<\
The jump in velocity potential is consistent with the jump in \
tangent-component velocity.\
\>", "Text",
 CellID->319382839],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"u\[Gamma]Jump", "\[Equal]", 
  RowBox[{"D", "[", 
   RowBox[{"\[Phi]\[Gamma]Jump", ",", "x"}], "]"}]}]], "Input",
 CellLabel->"In[72]:=",
 CellID->20069353],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[72]=",
 CellID->134183500]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Doublet panel of linearly varying strength", "Subsection",
 CellID->22834914],

Cell["\<\
Jump conditions for a doublet panel are similar to those for a vortex panel; \
normal component velocity and stream function are continuous across the \
panel, but tangent-component velocity and velocity potential are \
discontinuous with jumps related to the strength of the doublet panel.\
\>", "Text",
 CellID->742533898],

Cell[TextData[{
 "Note that the jump in tangent component velocity is just the circulation \
density for the doublet panel (see ",
 ButtonBox["Section 2.1.2",
  BaseStyle->"Hyperlink",
  ButtonData->{"IntroductionToInfluenceCoefficients.nb", "relation"}],
 "). "
}], "Text",
 CellID->374706227],

Cell[TextData[{
 "\t\t\t",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     StyleBox["\[Gamma]", "TR"], "(", 
     StyleBox["x", "TI"], ")"}], "=", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        StyleBox["d\[Mu]", "TI"], "(", 
        StyleBox["x", "TI"], ")"}], 
       StyleBox["dx", "TI"]]}], "=", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        FractionBox["\[DifferentialD]", 
         RowBox[{"\[DifferentialD]", 
          StyleBox["x", "TI"]}]], 
        RowBox[{"(", 
         RowBox[{
          SubscriptBox[
           StyleBox["a", "TI"], 
           StyleBox["\[Mu]", "TR"]], "+", 
          RowBox[{
           SubscriptBox[
            StyleBox["b", "TI"], 
            StyleBox["\[Mu]", "TR"]], " ", 
           StyleBox["x", "TI"]}]}], ")"}]}]}], "=", 
      RowBox[{"-", 
       SubscriptBox[
        StyleBox["b", "TI"], 
        StyleBox["\[Mu]", "TR"]]}]}]}]}], TraditionalForm]]]
}], "Text",
 CellID->102395795],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icua\[Mu]Minus", ",", "icub\[Mu]Minus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icua\[Mu]", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icub\[Mu]", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icua\[Mu]Plus", ",", "icub\[Mu]Plus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"icua\[Mu]", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"icub\[Mu]", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"icua\[Mu]Jump", ",", "icub\[Mu]Jump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"icua\[Mu]Plus", "-", "icua\[Mu]Minus"}], ",", 
     RowBox[{"icub\[Mu]Plus", "-", "icub\[Mu]Minus"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"u\[Mu]Jump", "=", 
  RowBox[{
   RowBox[{"a\[Mu]", " ", "icua\[Mu]Jump"}], "+", 
   RowBox[{"b\[Mu]", " ", "icub\[Mu]Jump"}]}]}]}], "Input",
 CellLabel->"In[73]:=",
 CellID->32443855],

Cell[BoxData[
 RowBox[{"-", "b\[Mu]"}]], "Output",
 ImageSize->{26, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[76]=",
 CellID->177081688]
}, Open  ]],

Cell["\<\
Compute the jump in velocity potential across the doublet panel.\
\>", "Text",
 CellID->86872070],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]a\[Mu]Minus", ",", "ic\[Phi]b\[Mu]Minus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"ic\[Phi]a\[Mu]", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"ic\[Phi]b\[Mu]", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]a\[Mu]Plus", ",", "ic\[Phi]b\[Mu]Plus"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Limit", "[", 
      RowBox[{"ic\[Phi]a\[Mu]", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}], ",", 
     RowBox[{"Limit", "[", 
      RowBox[{"ic\[Phi]b\[Mu]", ",", 
       RowBox[{"z", "\[Rule]", "0"}], ",", 
       RowBox[{"Assumptions", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Delta]", ">", "0"}], ",", 
          RowBox[{"0", "\[LessEqual]", "x", "\[LessEqual]", "\[Delta]"}]}], 
         "}"}]}], ",", 
       RowBox[{"Direction", "\[Rule]", 
        RowBox[{"-", "1"}]}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ic\[Phi]a\[Mu]Jump", ",", "ic\[Phi]b\[Mu]Jump"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ic\[Phi]a\[Mu]Plus", "-", "ic\[Phi]a\[Mu]Minus"}], ",", 
     RowBox[{"ic\[Phi]b\[Mu]Plus", "-", "ic\[Phi]b\[Mu]Minus"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[Phi]\[Mu]Jump", "=", 
  RowBox[{
   RowBox[{"a\[Mu]", " ", "ic\[Phi]a\[Mu]Jump"}], "+", 
   RowBox[{"b\[Mu]", " ", "ic\[Phi]b\[Mu]Jump"}]}]}]}], "Input",
 CellLabel->"In[77]:=",
 CellID->323169014],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "a\[Mu]"}], "-", 
  RowBox[{"b\[Mu]", " ", "x"}]}]], "Output",
 ImageSize->{63, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[80]=",
 CellID->138870219]
}, Open  ]],

Cell["\<\
The jump in velocity potential is consistent with the jump in \
tangent-component velocity.\
\>", "Text",
 CellID->167480315],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"u\[Mu]Jump", "\[Equal]", 
  RowBox[{"D", "[", 
   RowBox[{"\[Phi]\[Mu]Jump", ",", "x"}], "]"}]}]], "Input",
 CellLabel->"In[81]:=",
 CellID->289153844],

Cell[BoxData["True"], "Output",
 ImageSize->{32, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[81]=",
 CellID->98788664]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.3.5 Branch cuts", "Section",
 CellID->455404028],

Cell[CellGroupData[{

Cell["Constant-strength source panel", "Subsection",
 CellID->181897434],

Cell[TextData[{
 "Flow visualization suggests a branch cut along the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the stream function.  Compute the properties of the stream \
function as the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed in a counterclockwise direction."
}], "Text",
 CellID->297567032],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalDelta]ic\[Psi]\[Sigma]", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]\[Sigma]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]\[Sigma]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[CapitalDelta]\[Psi]\[Sigma]", "=", 
  RowBox[{"a\[Sigma]", " ", "\[CapitalDelta]ic\[Psi]\[Sigma]"}]}]}], "Input",
 CellLabel->"In[84]:=",
 CellID->81019929],

Cell[BoxData[
 RowBox[{"a\[Sigma]", " ", "\[Delta]"}]], "Output",
 ImageSize->{29, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[85]=",
 CellID->789528760]
}, Open  ]],

Cell[TextData[{
 "This verifies that the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is a branch cut for the stream function implying that it is not a \
single-valued function."
}], "Text",
 CellID->52759497]
}, Open  ]],

Cell[CellGroupData[{

Cell["Constant-strength vortex panel", "Subsection",
 CellID->651722584],

Cell[TextData[{
 "Flow visualization suggests a branch cut along the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the velocity potential.  Compute the properties of the velocity \
potential as the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis is crossed in a counterclockwise direction."
}], "Text",
 CellID->493568425],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalDelta]ic\[Phi]\[Gamma]", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]\[Gamma]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]\[Gamma]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[CapitalDelta]\[Phi]\[Gamma]", "=", 
  RowBox[{"a\[Gamma]", " ", "\[CapitalDelta]ic\[Phi]\[Gamma]"}]}]}], "Input",
 CellLabel->"In[86]:=",
 CellID->311225619],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "a\[Gamma]"}], " ", "\[Delta]"}]], "Output",
 ImageSize->{36, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[87]=",
 CellID->478300053]
}, Open  ]],

Cell["\<\
This verifies that the negative x-axis is a branch cut for the velocity \
potential, implying that it is not a single-valued function.\
\>", "Text",
 CellID->30475487]
}, Open  ]],

Cell[CellGroupData[{

Cell["Doublet panel of linearly varying strength", "Subsection",
 CellID->209793762],

Cell[TextData[{
 "Flow visualization does not suggest a branch cut for either the velocity \
potential or stream function.  ",
 "Compute the properties of the velocity potential and stream function as the \
negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 " axis is crossed in a counterclockwise direction."
}], "Text",
 CellID->632727],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalDelta]ic\[Phi]\[Mu]", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]\[Mu]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Phi]\[Mu]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[CapitalDelta]\[Phi]\[Mu]", "=", 
  RowBox[{"a\[Mu]", " ", "\[CapitalDelta]ic\[Phi]\[Mu]"}]}]}], "Input",
 CellLabel->"In[90]:=",
 CellID->141491386],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[91]=",
 CellID->135577583]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalDelta]ic\[Psi]\[Mu]", "=", 
   RowBox[{
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]\[Mu]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", 
       RowBox[{"-", "1"}]}]}], "]"}], "-", 
    RowBox[{"Limit", "[", 
     RowBox[{"ic\[Psi]\[Mu]", ",", 
      RowBox[{"z", "\[Rule]", "0"}], ",", 
      RowBox[{"Assumptions", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"\[Delta]", ">", "0"}], ",", 
         RowBox[{"x", "<", "0"}]}], "}"}]}], ",", 
      RowBox[{"Direction", "\[Rule]", "1"}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[CapitalDelta]\[Psi]\[Mu]", "=", 
  RowBox[{"a\[Mu]", " ", "\[CapitalDelta]ic\[Psi]\[Mu]"}]}]}], "Input",
 CellLabel->"In[92]:=",
 CellID->37149076],

Cell[BoxData["0"], "Output",
 ImageSize->{11, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[93]=",
 CellID->905257254]
}, Open  ]],

Cell[TextData[{
 "This verifies that there is no branch cut along the negative ",
 Cell[BoxData[
  StyleBox["x", "TI"]], "InlineFormula"],
 "-axis for the velocity potential or stream function of the doublet panel, \
implying that both functions are single valued."
}], "Text",
 CellID->793178211]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.3.6 Equivalent flow fields for vortex and doublet panels", "Section",
 CellID->497897766],

Cell[TextData[{
 "There is a relation between doublet density and circulation density for \
two-dimensional line singularities as discussed in more detail in ",
 ButtonBox["Section 2.1.2",
  BaseStyle->"Hyperlink",
  ButtonData->{"IntroductionToInfluenceCoefficients.nb", "relation"}],
 "."
}], "Text",
 CellID->342911820],

Cell[TextData[{
 "\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[DifferentialD]", 
      StyleBox["\[Mu]", "TR"]}], 
     RowBox[{"\[DifferentialD]", 
      StyleBox["x", "TI"]}]]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->315517505],

Cell["\<\
Write the equivalent circulation density for a doublet panel of linearly \
varying strength.\
\>", "Text",
 CellID->218711756],

Cell[TextData[{
 "\t\t\t\t",
 Cell[BoxData[
  RowBox[{
   RowBox[{
    StyleBox["\[Gamma]", "TR"], 
    RowBox[{"(", 
     StyleBox["x", "TI"], ")"}]}], "=", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      FractionBox["\[DifferentialD]", 
       RowBox[{"\[DifferentialD]", 
        StyleBox["x", "TI"]}]], 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox[
         StyleBox["a", "TI"], 
         StyleBox["\[Mu]", "TR"]], "+", 
        RowBox[{
         SubscriptBox[
          StyleBox["b", "TI"], 
          StyleBox["\[Mu]", "TR"]], " ", 
         StyleBox["x", "TI"]}]}], ")"}]}]}], "=", 
    RowBox[{"-", 
     SubscriptBox[
      StyleBox["b", "TI"], 
      StyleBox["\[Mu]", "TR"]]}]}]}]], "InlineFormula"],
 "\t\t\t\t"
}], "ExampleText",
 CellID->902684560],

Cell[TextData[{
 "A doublet panel of linearly varying strength is equivalent to the \
superposition of a constant-strength vortex panel of strength ",
 Cell[BoxData[
  RowBox[{"-", 
   SubscriptBox[
    StyleBox["b", "TI"], 
    StyleBox["\[Mu]", "TR"]]}]], "InlineFormula"],
 " and point vortices of strength ",
 Cell[BoxData[
  RowBox[{"-", 
   SubscriptBox[
    StyleBox["a", "TI"], 
    StyleBox["\[Mu]", "TR"]]}]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  RowBox[{
   SubscriptBox[
    StyleBox["a", "TI"], 
    StyleBox["\[Mu]", "TR"]], "+", 
   RowBox[{
    SubscriptBox[
     StyleBox["b", "TI"], 
     StyleBox["\[Mu]", "TR"]], " ", 
    StyleBox["\[Delta]", "TR"]}]}]], "InlineFormula"],
 " at panel endpoints.  Compute the velocity field due to the \
constant-strength vortex panel and two point vortices at panel endpoints."
}], "Text",
 CellID->26256628],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalGamma]", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "a\[Mu]"}], ",", 
     RowBox[{"a\[Mu]", "+", 
      RowBox[{"b\[Mu]", " ", "\[Delta]"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vVortexPoints", "=", 
   RowBox[{
    RowBox[{"ICVortexPoint", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}], ".", "\[CapitalGamma]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vVortexPanel", "=", 
   RowBox[{
    RowBox[{"-", "b\[Mu]"}], " ", 
    RowBox[{"ICVortexConstant", "[", 
     RowBox[{"rF", ",", "rP"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vV", "=", 
   RowBox[{"vVortexPoints", "+", "vVortexPanel"}]}], ";"}]}], "Input",
 CellLabel->"In[94]:=",
 CellID->345799449],

Cell["\<\
The velocity influence coefficients for a doublet panel of linearly varying \
strength were computed earlier and named ica\[Mu] and icb\[Mu].  Compute the \
velocity field for the doublet panel.\
\>", "Text",
 CellID->373792664],

Cell[BoxData[
 RowBox[{
  RowBox[{"vD", "=", 
   RowBox[{
    RowBox[{"a\[Mu]", " ", "ica\[Mu]"}], "+", 
    RowBox[{"b\[Mu]", " ", "icb\[Mu]"}]}]}], ";"}]], "Input",
 CellLabel->"In[98]:=",
 CellID->252617228],

Cell["Subtract the two velocity fields.", "Text",
 CellID->366544510],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"vV", "-", "vD"}], ",", 
   RowBox[{"Assumptions", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Element", "[", 
       RowBox[{"x", ",", "Reals"}], "]"}], ",", " ", 
      RowBox[{"Element", "[", 
       RowBox[{"z", ",", "Reals"}], "]"}], ",", 
      RowBox[{"\[Delta]", ">", "0"}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[99]:=",
 CellID->549191169],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{"{", "0", "}"}], "}"}], "\[UpArrow]", 
   RowBox[{"{", 
    RowBox[{"{", "0", "}"}], "}"}]}],
  Infix[
   Aerodynamics`CartesianVectors`CartesianVectors[{{0}}, {{0}}], "\[UpArrow]"],
  Editable->False]], "Output",
 ImageSize->{83, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[99]=",
 CellID->160412478]
}, Open  ]],

Cell["\<\
Both components of the resulting velocity field are zero, showing that the \
two velocity fields are equal.\
\>", "Text",
 CellID->54964601],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["InfluenceCoefficients",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/guide/InfluenceCoefficients"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["2.2 Point Singularities",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PointSingularities"]], "RelatedTutorials",
 CellID->70597402],

Cell[TextData[ButtonBox["3.1 Introduction to Panel Methods",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"]], \
"RelatedTutorials",
 CellID->67757421],

Cell[TextData[ButtonBox["Table of Contents",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/TableOfContents"]], "RelatedTutorials",
 CellID->41761111]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"2.3  Isolated panel - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 11, 0, 23.2230000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "Some properties of isolated straight-line singularity panels are \
illustrated in this section using examples of a constant-strength source \
panel, a constant-strength vortex panel, and a doublet panel of linearly \
varying strength. Load the Aerodynamics packages. When working with an \
isolated singularity, a panel local coordinate system is used in which the \
panel extends from the origin along the x-axis for the length of the panel, \
\\[Delta] (see orientation sketch). Computation of all influence coefficients \
depends on the displacement vectors locating the field point (at which the \
quantity is calculated) with respect to panel endpoints. For all the \
computations made in this section, the field point is specified as an \
arbitrary location (x,z), so that results are functions of x and z.", 
    "synonyms" -> {}, "title" -> "2.3  Isolated panel", "type" -> "Tutorial", 
    "uri" -> "Aerodynamics/tutorial/IsolatedPanel"}, "SearchTextTranslated" -> 
  ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[600, 21, 78, 1, 70, "TutorialColorBar"],
Cell[681, 24, 191, 5, 70, "LinkTrail"],
Cell[875, 31, 4242, 92, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[5142, 127, 56, 1, 70, "Title",
 CellID->509267359],
Cell[5201, 130, 277, 6, 70, "Text",
 CellID->20743874],
Cell[5481, 138, 125, 6, 70, "Text",
 CellID->819798417],
Cell[5609, 146, 98, 3, 70, "Input",
 CellID->5378332],
Cell[CellGroupData[{
Cell[5732, 153, 93, 1, 70, "Section",
 CellID->67289592],
Cell[5828, 156, 1076, 29, 70, "Text",
 CellID->105339135],
Cell[6907, 187, 332, 10, 70, "Text",
 CellID->23382165],
Cell[CellGroupData[{
Cell[7264, 201, 302, 10, 70, "Input",
 CellID->434627284],
Cell[7569, 213, 450, 15, 36, "Output",
 CellID->17701381]
}, Open  ]],
Cell[8034, 231, 139, 4, 70, "Text",
 CellID->407489582],
Cell[CellGroupData[{
Cell[8198, 239, 239, 8, 70, "Input",
 CellID->9791574],
Cell[8440, 249, 391, 13, 36, "Output",
 CellID->366246896]
}, Open  ]],
Cell[CellGroupData[{
Cell[8868, 267, 71, 1, 70, "Subsection",
 CellID->22443959],
Cell[8942, 270, 812, 26, 70, "Text",
 CellID->470290831],
Cell[9757, 298, 1332, 54, 70, "Text",
 CellID->120763874],
Cell[11092, 354, 645, 17, 70, "DefinitionBox",
 CellID->80874091],
Cell[11740, 373, 108, 3, 70, "Caption",
 CellID->84822298],
Cell[11851, 378, 144, 4, 70, "Text",
 CellID->49995632],
Cell[CellGroupData[{
Cell[12020, 386, 269, 8, 70, "Input",
 CellID->137123347],
Cell[12292, 396, 1396, 43, 80, "Output",
 CellID->8980686]
}, Open  ]],
Cell[13703, 442, 137, 4, 70, "Text",
 CellID->494657454],
Cell[CellGroupData[{
Cell[13865, 450, 240, 7, 70, "Input",
 CellID->176773044],
Cell[14108, 459, 837, 29, 68, "Output",
 CellID->10269969]
}, Open  ]],
Cell[14960, 491, 298, 8, 70, "Text",
 CellID->16489213],
Cell[CellGroupData[{
Cell[15283, 503, 452, 15, 70, "Input",
 CellID->703191196],
Cell[15738, 520, 690, 24, 68, "Output",
 CellID->42351100]
}, Open  ]],
Cell[16443, 547, 186, 4, 70, "Text",
 CellID->79030308],
Cell[CellGroupData[{
Cell[16654, 555, 776, 24, 70, "Input",
 CellID->493312995],
Cell[17433, 581, 782, 27, 58, "Output",
 CellID->395880335]
}, Open  ]],
Cell[18230, 611, 85, 1, 70, "Text",
 CellID->283412909],
Cell[CellGroupData[{
Cell[18340, 616, 774, 24, 70, "Input",
 CellID->8664355],
Cell[19117, 642, 698, 22, 68, "Output",
 CellID->597020581]
}, Open  ]],
Cell[19830, 667, 177, 4, 70, "Text",
 CellID->236352304],
Cell[20010, 673, 490, 15, 70, "Input",
 CellID->45587925]
}, Open  ]],
Cell[CellGroupData[{
Cell[20537, 693, 72, 1, 70, "Subsection",
 CellID->177267806],
Cell[20612, 696, 734, 22, 70, "Text",
 CellID->52645281],
Cell[21349, 720, 1070, 42, 70, "Text",
 CellID->1936554],
Cell[22422, 764, 646, 17, 70, "DefinitionBox",
 CellID->206350477],
Cell[23071, 783, 108, 3, 70, "Caption",
 CellID->62955998],
Cell[23182, 788, 223, 5, 70, "Text",
 CellID->107133949],
Cell[CellGroupData[{
Cell[23430, 797, 852, 27, 70, "Input",
 CellID->334029669],
Cell[24285, 826, 661, 23, 68, "Output",
 CellID->142034005]
}, Open  ]],
Cell[24961, 852, 118, 3, 70, "Text",
 CellID->177390599],
Cell[CellGroupData[{
Cell[25104, 859, 776, 24, 70, "Input",
 CellID->19476998],
Cell[25883, 885, 718, 23, 68, "Output",
 CellID->74949219]
}, Open  ]],
Cell[CellGroupData[{
Cell[26638, 913, 775, 24, 70, "Input",
 CellID->4731421],
Cell[27416, 939, 783, 27, 58, "Output",
 CellID->470869661]
}, Open  ]],
Cell[28214, 969, 177, 4, 70, "Text",
 CellID->426238386],
Cell[28394, 975, 490, 15, 70, "Input",
 CellID->34838295]
}, Open  ]],
Cell[CellGroupData[{
Cell[28921, 995, 83, 1, 70, "Subsection",
 CellID->72624876],
Cell[29007, 998, 980, 32, 70, "Text",
 CellID->717906090],
Cell[29990, 1032, 1084, 44, 70, "Text",
 CellID->267001894],
Cell[31077, 1078, 1123, 45, 70, "Text",
 CellID->654610829],
Cell[32203, 1125, 641, 17, 70, "DefinitionBox",
 CellID->1009440973],
Cell[32847, 1144, 117, 3, 70, "Caption",
 CellID->98129465],
Cell[32967, 1149, 208, 5, 70, "Text",
 CellID->218315512],
Cell[CellGroupData[{
Cell[33200, 1158, 320, 10, 70, "Input",
 CellID->73802754],
Cell[33523, 1170, 4465, 125, 125, "Output",
 CellID->194986753]
}, Open  ]],
Cell[38003, 1298, 142, 4, 70, "Text",
 CellID->13683548],
Cell[CellGroupData[{
Cell[38170, 1306, 235, 7, 70, "Input",
 CellID->758730827],
Cell[38408, 1315, 1289, 44, 62, "Output",
 CellID->52973983]
}, Open  ]],
Cell[CellGroupData[{
Cell[39734, 1364, 715, 23, 70, "Input",
 CellID->318956780],
Cell[40452, 1389, 1257, 43, 69, "Output",
 CellID->2938375]
}, Open  ]],
Cell[41724, 1435, 118, 3, 70, "Text",
 CellID->402639517],
Cell[CellGroupData[{
Cell[41867, 1442, 871, 27, 70, "Input",
 CellID->68692785],
Cell[42741, 1471, 877, 29, 68, "Output",
 CellID->6140919]
}, Open  ]],
Cell[CellGroupData[{
Cell[43655, 1505, 871, 27, 70, "Input",
 CellID->17690965],
Cell[44529, 1534, 1156, 39, 68, "Output",
 CellID->905109567]
}, Open  ]],
Cell[45700, 1576, 254, 5, 70, "Text",
 CellID->332111165],
Cell[45957, 1583, 728, 21, 70, "Input",
 CellID->124787270]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[46734, 1610, 96, 1, 70, "Section",
 CellID->29918537],
Cell[46833, 1613, 152, 4, 70, "Text",
 CellID->363098241],
Cell[46988, 1619, 844, 33, 70, "Text",
 CellID->26305678],
Cell[47835, 1654, 400, 8, 70, "Text",
 CellID->12274103],
Cell[CellGroupData[{
Cell[48260, 1666, 610, 20, 70, "Input",
 CellID->60683208],
Cell[48873, 1688, 162, 5, 36, "Output",
 CellID->263539492]
}, Open  ]],
Cell[CellGroupData[{
Cell[49072, 1698, 485, 16, 70, "Input",
 CellID->243727406],
Cell[49560, 1716, 162, 5, 36, "Output",
 CellID->346632648]
}, Open  ]],
Cell[CellGroupData[{
Cell[49759, 1726, 484, 16, 70, "Input",
 CellID->98739334],
Cell[50246, 1744, 161, 5, 36, "Output",
 CellID->73869383]
}, Open  ]],
Cell[CellGroupData[{
Cell[50444, 1754, 695, 21, 70, "Input",
 CellID->17935659],
Cell[51142, 1777, 162, 5, 36, "Output",
 CellID->150797085]
}, Open  ]],
Cell[CellGroupData[{
Cell[51341, 1787, 470, 15, 70, "Input",
 CellID->756290509],
Cell[51814, 1804, 162, 5, 36, "Output",
 CellID->296807279]
}, Open  ]],
Cell[CellGroupData[{
Cell[52013, 1814, 470, 15, 70, "Input",
 CellID->512433514],
Cell[52486, 1831, 161, 5, 36, "Output",
 CellID->30177663]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[52696, 1842, 63, 1, 70, "Section",
 CellID->34893618],
Cell[52762, 1845, 345, 6, 70, "Text",
 CellID->33633243],
Cell[53110, 1853, 313, 6, 70, "Text",
 CellID->680409942],
Cell[CellGroupData[{
Cell[53448, 1863, 72, 1, 70, "Subsection",
 CellID->147316578],
Cell[53523, 1866, 654, 25, 70, "Text",
 CellID->4981250],
Cell[CellGroupData[{
Cell[54202, 1895, 4795, 135, 19, "Input",
 CellOpen->False,
 CellID->608792814],
Cell[59000, 2032, 197426, 3597, 70, "Output",
 CellID->2871532]
}, Open  ]],
Cell[256441, 5632, 348, 9, 70, "Text",
 CellID->738393833]
}, Open  ]],
Cell[CellGroupData[{
Cell[256826, 5646, 72, 1, 70, "Subsection",
 CellID->671459404],
Cell[256901, 5649, 635, 24, 70, "Text",
 CellID->159660156],
Cell[CellGroupData[{
Cell[257561, 5677, 4527, 126, 19, "Input",
 CellOpen->False,
 CellID->79272867],
Cell[262091, 5805, 196969, 3589, 70, "Output",
 CellID->305833234]
}, Open  ]],
Cell[459075, 9397, 356, 9, 70, "Text",
 CellID->643240816]
}, Open  ]],
Cell[CellGroupData[{
Cell[459468, 9411, 84, 1, 70, "Subsection",
 CellID->485419430],
Cell[459555, 9414, 1400, 51, 70, "Text",
 CellID->977679975],
Cell[CellGroupData[{
Cell[460980, 9469, 4690, 131, 19, "Input",
 CellOpen->False,
 CellID->115028120],
Cell[465673, 9602, 215262, 3922, 70, "Output",
 CellID->70145224]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[680996, 13531, 60, 1, 70, "Section",
 CellID->229580123],
Cell[681059, 13534, 415, 7, 70, "Text",
 CellID->147816116],
Cell[CellGroupData[{
Cell[681499, 13545, 72, 1, 70, "Subsection",
 CellID->164768312],
Cell[681574, 13548, 272, 5, 70, "Text",
 CellID->13291475],
Cell[681849, 13555, 139, 4, 70, "Text",
 CellID->162142882],
Cell[CellGroupData[{
Cell[682013, 13563, 1218, 34, 70, "Input",
 CellID->152001704],
Cell[683234, 13599, 167, 5, 36, "Output",
 CellID->386286322]
}, Open  ]],
Cell[683416, 13607, 95, 1, 70, "Text",
 CellID->82001642],
Cell[CellGroupData[{
Cell[683536, 13612, 1263, 34, 70, "Input",
 CellID->301350322],
Cell[684802, 13648, 263, 9, 36, "Output",
 CellID->330826737]
}, Open  ]],
Cell[685080, 13660, 140, 4, 70, "Text",
 CellID->93140049],
Cell[CellGroupData[{
Cell[685245, 13668, 201, 6, 70, "Input",
 CellID->114614373],
Cell[685449, 13676, 162, 5, 36, "Output",
 CellID->125439526]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[685660, 13687, 72, 1, 70, "Subsection",
 CellID->486795743],
Cell[685735, 13690, 275, 6, 70, "Text",
 CellID->48476432],
Cell[686013, 13698, 140, 4, 70, "Text",
 CellID->478267279],
Cell[CellGroupData[{
Cell[686178, 13706, 1218, 34, 70, "Input",
 CellID->263300400],
Cell[687399, 13742, 166, 5, 36, "Output",
 CellID->17961284]
}, Open  ]],
Cell[687580, 13750, 106, 3, 70, "Text",
 CellID->41162955],
Cell[CellGroupData[{
Cell[687711, 13757, 1263, 34, 70, "Input",
 CellID->224778660],
Cell[688977, 13793, 242, 8, 36, "Output",
 CellID->63310290]
}, Open  ]],
Cell[689234, 13804, 135, 4, 70, "Text",
 CellID->319382839],
Cell[CellGroupData[{
Cell[689394, 13812, 180, 5, 70, "Input",
 CellID->20069353],
Cell[689577, 13819, 162, 5, 36, "Output",
 CellID->134183500]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[689788, 13830, 83, 1, 70, "Subsection",
 CellID->22834914],
Cell[689874, 13833, 334, 6, 70, "Text",
 CellID->742533898],
Cell[690211, 13841, 294, 8, 70, "Text",
 CellID->374706227],
Cell[690508, 13851, 980, 36, 70, "Text",
 CellID->102395795],
Cell[CellGroupData[{
Cell[691513, 13891, 2487, 71, 70, "Input",
 CellID->32443855],
Cell[694003, 13964, 181, 6, 36, "Output",
 CellID->177081688]
}, Open  ]],
Cell[694199, 13973, 107, 3, 70, "Text",
 CellID->86872070],
Cell[CellGroupData[{
Cell[694331, 13980, 2573, 71, 70, "Input",
 CellID->323169014],
Cell[696907, 14053, 232, 8, 36, "Output",
 CellID->138870219]
}, Open  ]],
Cell[697154, 14064, 135, 4, 70, "Text",
 CellID->167480315],
Cell[CellGroupData[{
Cell[697314, 14072, 175, 5, 70, "Input",
 CellID->289153844],
Cell[697492, 14079, 161, 5, 36, "Output",
 CellID->98788664]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[697714, 14091, 56, 1, 70, "Section",
 CellID->455404028],
Cell[CellGroupData[{
Cell[697795, 14096, 72, 1, 70, "Subsection",
 CellID->181897434],
Cell[697870, 14099, 380, 10, 70, "Text",
 CellID->297567032],
Cell[CellGroupData[{
Cell[698275, 14113, 1003, 27, 70, "Input",
 CellID->81019929],
Cell[699281, 14142, 196, 6, 36, "Output",
 CellID->789528760]
}, Open  ]],
Cell[699492, 14151, 241, 7, 70, "Text",
 CellID->52759497]
}, Open  ]],
Cell[CellGroupData[{
Cell[699770, 14163, 72, 1, 70, "Subsection",
 CellID->651722584],
Cell[699845, 14166, 386, 10, 70, "Text",
 CellID->493568425],
Cell[CellGroupData[{
Cell[700256, 14180, 1004, 27, 70, "Input",
 CellID->311225619],
Cell[701263, 14209, 214, 7, 36, "Output",
 CellID->478300053]
}, Open  ]],
Cell[701492, 14219, 177, 4, 70, "Text",
 CellID->30475487]
}, Open  ]],
Cell[CellGroupData[{
Cell[701706, 14228, 84, 1, 70, "Subsection",
 CellID->209793762],
Cell[701793, 14231, 357, 9, 70, "Text",
 CellID->632727],
Cell[CellGroupData[{
Cell[702175, 14244, 986, 27, 70, "Input",
 CellID->141491386],
Cell[703164, 14273, 159, 5, 36, "Output",
 CellID->135577583]
}, Open  ]],
Cell[CellGroupData[{
Cell[703360, 14283, 985, 27, 70, "Input",
 CellID->37149076],
Cell[704348, 14312, 159, 5, 36, "Output",
 CellID->905257254]
}, Open  ]],
Cell[704522, 14320, 297, 7, 70, "Text",
 CellID->793178211]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[704868, 14333, 97, 1, 70, "Section",
 CellID->497897766],
Cell[704968, 14336, 322, 8, 70, "Text",
 CellID->342911820],
Cell[705293, 14346, 398, 16, 70, "ExampleText",
 CellID->315517505],
Cell[705694, 14364, 136, 4, 70, "Text",
 CellID->218711756],
Cell[705833, 14370, 778, 30, 70, "ExampleText",
 CellID->902684560],
Cell[706614, 14402, 871, 28, 70, "Text",
 CellID->26256628],
Cell[707488, 14432, 783, 25, 70, "Input",
 CellID->345799449],
Cell[708274, 14459, 238, 5, 70, "Text",
 CellID->373792664],
Cell[708515, 14466, 210, 7, 70, "Input",
 CellID->252617228],
Cell[708728, 14475, 69, 1, 70, "Text",
 CellID->366544510],
Cell[CellGroupData[{
Cell[708822, 14480, 434, 13, 70, "Input",
 CellID->549191169],
Cell[709259, 14495, 418, 14, 36, "Output",
 CellID->160412478]
}, Open  ]],
Cell[709692, 14512, 150, 4, 70, "Text",
 CellID->54964601],
Cell[CellGroupData[{
Cell[709867, 14520, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[709935, 14523, 179, 4, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]],
Cell[CellGroupData[{
Cell[710151, 14532, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[710226, 14535, 178, 4, 70, "RelatedTutorials",
 CellID->70597402],
Cell[710407, 14541, 198, 5, 70, "RelatedTutorials",
 CellID->67757421],
Cell[710608, 14548, 169, 4, 70, "RelatedTutorials",
 CellID->41761111]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[710816, 14557, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

