(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     18636,        513]
NotebookOptionsPosition[     14742,        392]
NotebookOutlinePosition[     16940,        453]
CellTagsIndexPosition[     16850,        447]
WindowTitle->PanelNumbering - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[Cell[
    BoxData[
     ActionMenuBox[
      FrameBox["\<\"URL \[RightGuillemet]\"\>",
       StripOnInput->False], {"\<\"Aerodynamics/tutorial/PanelNumbering\"\>":>
      None, "\<\"Copy Mathematica url\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell["Aerodynamics/tutorial/PanelNumbering"]}, Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; Null], 
      Delimiter, "\<\"Copy web url\"\>":>
      Module[{DocumentationSearch`Private`nb$}, 
       DocumentationSearch`Private`nb$ = NotebookPut[
          Notebook[{
            Cell[
             BoxData[
              MakeBoxes[
               Hyperlink[
               "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/PanelNumbering.html"], StandardForm]], "Input", TextClipboardType -> 
             "PlainText"]}, Visible -> False]]; 
       SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
       FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
       NotebookClose[DocumentationSearch`Private`nb$]; 
       Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
        NotebookLocate[{
          URL[
           StringJoin[
           "http://reference.wolfram.com/mathematica/", 
            "Aerodynamics/tutorial/PanelNumbering", ".html"]], None}]}]},
      Appearance->None,
      MenuAppearance->Automatic]],
     LineSpacing->{1.4, 0}]], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["PanelNumbering", "Title",
 CellID->509267359],

Cell["\<\
Three conventions are used for numbering points on the surface (contour) of \
an airfoil:  clockwise, counterclockwise and fore-to-aft.  This software uses \
the clockwise convention, but must be able to convert data from the other \
conventions for data imported from other sources.\
\>", "Text",
 CellID->1534169418],

Cell["\<\
Clockwise:  Start at the lower-surface trailing edge and proceed clockwise \
around the airfoil, going forward along the lower surface of the airfoil to \
the leading edge and continuing aft along the upper surface to the trailing \
edge.  This is the convention used by Katz and Plotkin (2001)\
\>", "TutorialAbstract",
 CellID->25677726],

Cell["\<\
Counterclockwise: Start at the upper-surface trailing edge and proceed \
counterclockwise around the airfoil, going forward along the upper surface of \
the airfoil to the leading edge and continuing aft along the lower surface to \
the trailing edge.  This is the convention used by Michael Selig in his \
original collection of data on the UIUC Airfoil Data Site.  The data file fo \
the Eppler 205 airfoil (e205.dat) is an example of using this convention.\
\>", "TutorialAbstract",
 CellID->70750406],

Cell["\<\
Fore-to-aft: Start at the leading edge and proceed aft along the upper \
surface of the airfoil to the trailing edge.  Repeat for the lower surface, \
leading edge to trailing edge.  This is the convention used by David Lednicer \
of Analytical Methods, Inc., whose collection of airfoil has been added to \
the UIUC Airfoil Data Ste. The Clark Y airfoil (clarky.dat) is an example of \
this convention.\
\>", "TutorialAbstract",
 CellID->405104752],

Cell["Load the package.", "MathCaption",
 CellID->836781195],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<Aerodynamics`AirfoilGeometry`\>\"", 
  "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->2058623809],

Cell["\<\
The function AirfoilFromDataFile, must be able to import data from files \
using these numbering conventions.  This is handled by the option \
NumberingConvention.\
\>", "Text",
 CellID->138628418],

Cell["\<\
List the options for the function AirfoilFromDataFile and note that the \
default option for NumberingConvention is Counterclockwise. \
\>", "MathCaption",
 CellID->167576114],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "AirfoilFromDataFile", "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->14035536],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"NumberingConvention", "\[Rule]", "Counterclockwise"}], 
  "}"}]], "Output",
 ImageSize->{280, 15},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->243621817]
}, Open  ]],

Cell["\<\
Look at the names of the files stored in the folder Airfoil Geometry, \
assuming the aerodynamics software is installed in the base directory.\
\>", "MathCaption",
 CellID->48545141],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{
   "$BaseDirectory", "<>", 
    "\"\<\\Applications\\Aerodynamics\\Airfoil Data\>\""}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"FileNames", "[", "]"}]}], "Input",
 CellLabel->"In[3]:=",
 CellID->71328127],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"clarky.dat\"\>", ",", "\<\"e205.dat\"\>", 
   ",", "\<\"e374.dat\"\>", ",", "\<\"l7769.dat\"\>", 
   ",", "\<\"n642415.dat\"\>", ",", "\<\"naca23012.dat\"\>", 
   ",", "\<\"naca4412.dat\"\>", ",", "\<\"naca4412m.dat\"\>", 
   ",", "\<\"naca642415.dat\"\>", ",", "\<\"naca747a315.dat\"\>", 
   ",", "\<\"s1223.dat\"\>", ",", "\<\"sd7003.dat\"\>", 
   ",", "\<\"usa35b.dat\"\>", ",", "\<\"williams-flapA.dat\"\>", 
   ",", "\<\"williams-mainA.dat\"\>"}], "}"}]], "Output",
 ImageSize->{505, 50},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->18332394]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"ResetDirectory", "[", "]"}], ";"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->89265534],

Cell["Look at some of the file e205.dat.", "MathCaption",
 CellID->158189714],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{"Import", "[", 
    RowBox[{
    "$BaseDirectory", "<>", 
     "\"\<\\Applications\\Aerodynamics\\Airfoil Data\\e205.dat\>\""}], "]"}], 
   ",", "4"}], "]"}]], "Input",
 CellTags->{"3.6.3", "3.6"},
 CellLabel->"In[5]:=",
 CellID->73827751],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"E205\"\>", ",", "\<\"(10.48%)\"\>"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1.`", ",", "0.`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.99655`", ",", "0.00039`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.98649`", ",", "0.00174`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.97049`", ",", "0.00427`"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.94916`", ",", "0.00778`"}], "}"}], ",", 
    RowBox[{"\[LeftSkeleton]", "50", "\[RightSkeleton]"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.92195`", ",", 
      RowBox[{"-", "0.0038`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.9486`", ",", 
      RowBox[{"-", "0.00252`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.97017`", ",", 
      RowBox[{"-", "0.00125`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.98635`", ",", 
      RowBox[{"-", "0.00036`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"0.99651`", ",", 
      RowBox[{"-", "0.00003`"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1.`", ",", "0.`"}], "}"}]}], "}"}],
  Short[#, 4]& ]], "Output",
 ImageSize->{463, 67},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellTags->{"3.6.3", "3.6"},
 CellLabel->"Out[5]//Short=",
 CellID->624058879]
}, Open  ]],

Cell["\<\
This excerpt of data verifies that the numbering convention used in the file \
is counterclockwise.\
\>", "TutorialAbstract",
 CellID->105072402],

Cell["\<\
Use the function AirfoilFromDataFile to import the data and manipulate it for \
use in the aerodynamics software. \
\>", "MathCaption",
 CellID->68685394],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"r", ",", "spline"}], "}"}], "=", 
   RowBox[{"AirfoilFromDataFile", "[", 
    RowBox[{
    "$BaseDirectory", "<>", 
     "\"\<\\Applications\\Aerodynamics\\Airfoil Data\\e205.dat\>\""}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"gR", "=", 
   RowBox[{"PlotAirfoil", "[", "r", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Show", "[", 
  RowBox[{"gR", ",", 
   RowBox[{"PlotStyle", "[", 
    RowBox[{"{", "}"}], "]"}]}], "]"}]}], "Input",
 CellLabel->"In[6]:=",
 CellID->45929110],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt3c+LFOkdx/EO+UvybwyEHJY9LHsQA+Ih5KLQ6CV70Yte9KIXhUYvCks8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    "], {{0, 180}, {504, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{504, 180},
  PlotRange->{{0, 504}, {0, 180}}]], "Output",
 ImageSize->{508, 180},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[6]=",
 CellID->114415095]
}, Open  ]],

Cell["Need to use color to show numbering convention.", "TutorialAbstract",
 CellID->201324096]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"PanelNumbering - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 11, 0, 36.6702000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "Three conventions are used for numbering points on the surface (contour) \
of an airfoil: clockwise, counterclockwise and fore-to-aft. This software \
uses the clockwise convention, but must be able to convert data from the \
other conventions for data imported from other sources. Load the package. The \
function AirfoilFromDataFile, must be able to import data from files using \
these numbering conventions. This is handled by the option \
NumberingConvention.", "synonyms" -> {}, "title" -> "PanelNumbering", "type" -> 
    "Tutorial", "uri" -> "Aerodynamics/tutorial/PanelNumbering"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "3.6.3"->{
  Cell[6929, 203, 299, 10, 70, "Input",
   CellTags->{"3.6.3", "3.6"},
   CellID->73827751],
  Cell[7231, 215, 1314, 40, 88, "Output",
   CellTags->{"3.6.3", "3.6"},
   CellID->624058879]},
 "3.6"->{
  Cell[6929, 203, 299, 10, 70, "Input",
   CellTags->{"3.6.3", "3.6"},
   CellID->73827751],
  Cell[7231, 215, 1314, 40, 88, "Output",
   CellTags->{"3.6.3", "3.6"},
   CellID->624058879]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"3.6.3", 16437, 430},
 {"3.6", 16637, 437}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[595, 21, 78, 1, 70, "TutorialColorBar"],
Cell[676, 24, 191, 5, 70, "LinkTrail"],
Cell[870, 31, 1917, 42, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[2812, 77, 51, 1, 70, "Title",
 CellID->509267359],
Cell[2866, 80, 328, 6, 70, "Text",
 CellID->1534169418],
Cell[3197, 88, 349, 6, 70, "TutorialAbstract",
 CellID->25677726],
Cell[3549, 96, 514, 8, 70, "TutorialAbstract",
 CellID->70750406],
Cell[4066, 106, 459, 8, 70, "TutorialAbstract",
 CellID->405104752],
Cell[4528, 116, 60, 1, 70, "MathCaption",
 CellID->836781195],
Cell[4591, 119, 141, 4, 70, "Input",
 CellID->2058623809],
Cell[4735, 125, 207, 5, 70, "Text",
 CellID->138628418],
Cell[4945, 132, 185, 4, 70, "MathCaption",
 CellID->167576114],
Cell[CellGroupData[{
Cell[5155, 140, 120, 3, 70, "Input",
 CellID->14035536],
Cell[5278, 145, 246, 8, 36, "Output",
 CellID->243621817]
}, Open  ]],
Cell[5539, 156, 192, 4, 70, "MathCaption",
 CellID->48545141],
Cell[CellGroupData[{
Cell[5756, 164, 284, 9, 70, "Input",
 CellID->71328127],
Cell[6043, 175, 644, 14, 71, "Output",
 CellID->18332394]
}, Open  ]],
Cell[6702, 192, 122, 4, 70, "Input",
 CellID->89265534],
Cell[6827, 198, 77, 1, 70, "MathCaption",
 CellID->158189714],
Cell[CellGroupData[{
Cell[6929, 203, 299, 10, 70, "Input",
 CellTags->{"3.6.3", "3.6"},
 CellID->73827751],
Cell[7231, 215, 1314, 40, 88, "Output",
 CellTags->{"3.6.3", "3.6"},
 CellID->624058879]
}, Open  ]],
Cell[8560, 258, 155, 4, 70, "TutorialAbstract",
 CellID->105072402],
Cell[8718, 264, 164, 4, 70, "MathCaption",
 CellID->68685394],
Cell[CellGroupData[{
Cell[8907, 272, 573, 18, 83, "Input",
 CellID->45929110],
Cell[9483, 292, 5107, 91, 201, "Output",
 CellID->114415095]
}, Open  ]],
Cell[14605, 386, 95, 1, 70, "TutorialAbstract",
 CellID->201324096]
}, Open  ]],
Cell[14715, 390, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

