(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     12963,        353]
NotebookOptionsPosition[      9064,        257]
NotebookOutlinePosition[     11767,        313]
CellTagsIndexPosition[     11724,        310]
WindowTitle->Preface - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Related Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Table of Contents\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/tutorial/TableOfContents"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/Aerodynamics"], "\<\"CartesianVectors\"\>":>
       Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/CartesianVectors"], \
"\<\"AirfoilGeometry\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/AirfoilGeometry"], \
"\<\"InfluenceCoefficients\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/InfluenceCoefficients"], \
"\<\"PanelMethods\"\>":>Documentation`HelpLookup[
       "paclet:Aerodynamics/guide/PanelMethods"], "\<\"Graphics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Graphics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/PrefaceToAerodynamics\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/PrefaceToAerodynamics"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/PrefaceToAerodynamics.html"], StandardForm]], "Input", 
              TextClipboardType -> "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/PrefaceToAerodynamics", ".html"]], 
           None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["Preface", "Title",
 CellID->509267359],

Cell[TextData[{
 "When I taught undergraduate aerodynamics at the University of Florida, I \
often wished for a collection of relatively easy-to-use tools for computing \
more complex flows than those illustrated in textbooks.  ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " 1.1 is a start of such a collection.  The current version is limited to \
steady incompressible two-dimensional potential flow over an airfoil."
}], "Text",
 CellID->34163350],

Cell[TextData[{
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " contains a collection of functions for constructing panel methods \
including three fully implemented panel methods.  The easy-to-use aspect is \
addressed by providing over 40 functions to accomplish well-defined tasks, \
and for displaying final results using conventions of the aerodynamics \
community.  These tasks range from importing data files for airfoil geometry \
to plotting model results for surface pressure coefficient.  Each function is \
fully documented and illustrated with examples."
}], "Text",
 CellID->233074666],

Cell[TextData[{
 "The needs of either a survey or an in-depth study of two-dimensional \
potential flow over an airfoil can be met by ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ".  The three panel methods implemented and their documentation can be used \
as a brief exposure to potential-flow aerodynamics.  To accommodate an \
in-depth study, the content of all packages is available to users for \
critique and modification.  Functions are defined for commonly used \
two-dimensional influence coefficients and for presentation graphics, so  \
emphasis can be placed on model construction."
}], "Text",
 CellID->154329092],

Cell[TextData[{
 "The collection of tutorials is organized in the form of an interactive \
book.  The vast majority of the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " coding is shown in the tutorials, but if routine coding for preparing \
graphics detracts from the narrative, these cells are closed.  You may open \
them if you want to look at details.  Examples are usually presented with \
input specification grouped together to enable you to change input values \
easily and rerun the example for other cases.  A convenient link to the \
tutorials is the table of contents listed below under the heading Related \
Tutorials."
}], "Text",
 CellID->140730767],

Cell[TextData[{
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " consists of five packages.  The main guide page for ",
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " provides links to guide pages for each package.  All guide pages are \
listed below under the heading More About.  Additionally, function names are \
indexed in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Help, providing quick access to symbol documentation when working on a \
problem."
}], "Text",
 CellID->136038040],

Cell[TextData[{
 "Those interested primarily in a ready-to-use solver for potential-flow over \
an airfoil using a panel method, should visit the site developed by ",
 ButtonBox["Kevin Jones",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://aa.nps.edu/~jones/online_tools/panel2/"], None}],
 ", at the Naval Postgraduate School."
}], "Text",
 CellID->334044537],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[Cell[BoxData[
 ButtonBox["Aerodynamics",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/Aerodynamics"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->1567025153],

Cell[TextData[Cell[BoxData[
 ButtonBox["CartesianVectors",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/CartesianVectors"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->285068269],

Cell[TextData[Cell[BoxData[
 ButtonBox["AirfoilGeometry",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/AirfoilGeometry"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->403490632],

Cell[TextData[Cell[BoxData[
 ButtonBox["InfluenceCoefficients",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/InfluenceCoefficients"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->929488745],

Cell[TextData[Cell[BoxData[
 ButtonBox["PanelMethods",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/guide/PanelMethods"]], "InlineFormula"]], \
"TutorialMoreAbout",
 CellID->4762702],

Cell[TextData[ButtonBox["Graphics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Graphics"]], "TutorialMoreAbout",
 CellID->47881637]
}, Open  ]],

Cell[CellGroupData[{

Cell["RELATED TUTORIALS", "RelatedTutorialsSection",
 CellID->415694126],

Cell[TextData[ButtonBox["Table of Contents",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/TableOfContents"]], "RelatedTutorials",
 CellID->806871991]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Preface - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 11, 0, 43.9710000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "When I taught undergraduate aerodynamics at the University of Florida, I \
often wished for a collection of relatively easy-to-use tools for computing \
more complex flows than those illustrated in textbooks. Aerodynamics 1.1 is a \
start of such a collection. The current version is limited to steady \
incompressible two-dimensional potential flow over an airfoil. Aerodynamics \
contains a collection of functions for constructing panel methods including \
three fully implemented panel methods. The easy-to-use aspect is addressed by \
providing over 40 functions to accomplish well-defined tasks, and for \
displaying final results using conventions of the aerodynamics community. \
These tasks range from importing data files for airfoil geometry to plotting \
model results for surface pressure coefficient. Each function is fully \
documented and illustrated with examples. The needs of either a survey or an \
in-depth study of two-dimensional potential flow over an airfoil can be met \
by Aerodynamics. The three panel methods implemented and their documentation \
can be used as a brief exposure to potential-flow aerodynamics. To \
accommodate an in-depth study, the content of all packages is available to \
users for critique and modification. Functions are defined for commonly used \
two-dimensional influence coefficients and for presentation graphics, so \
emphasis can be placed on model construction.", "synonyms" -> {}, "title" -> 
    "Preface", "type" -> "Tutorial", "uri" -> 
    "Aerodynamics/tutorial/PrefaceToAerodynamics"}, "SearchTextTranslated" -> 
  ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[588, 21, 78, 1, 70, "TutorialColorBar"],
Cell[669, 24, 191, 5, 70, "LinkTrail"],
Cell[863, 31, 3247, 73, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[4135, 108, 44, 1, 70, "Title",
 CellID->509267359],
Cell[4182, 111, 459, 9, 70, "Text",
 CellID->34163350],
Cell[4644, 122, 604, 11, 70, "Text",
 CellID->233074666],
Cell[5251, 135, 635, 12, 70, "Text",
 CellID->154329092],
Cell[5889, 149, 673, 13, 70, "Text",
 CellID->140730767],
Cell[6565, 164, 507, 14, 70, "Text",
 CellID->136038040],
Cell[7075, 180, 371, 9, 70, "Text",
 CellID->334044537],
Cell[CellGroupData[{
Cell[7471, 193, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[7539, 196, 200, 6, 70, "TutorialMoreAbout",
 CellID->1567025153],
Cell[7742, 204, 207, 6, 70, "TutorialMoreAbout",
 CellID->285068269],
Cell[7952, 212, 205, 6, 70, "TutorialMoreAbout",
 CellID->403490632],
Cell[8160, 220, 217, 6, 70, "TutorialMoreAbout",
 CellID->929488745],
Cell[8380, 228, 197, 6, 70, "TutorialMoreAbout",
 CellID->4762702],
Cell[8580, 236, 148, 3, 70, "TutorialMoreAbout",
 CellID->47881637]
}, Open  ]],
Cell[CellGroupData[{
Cell[8765, 244, 72, 1, 70, "RelatedTutorialsSection",
 CellID->415694126],
Cell[8840, 247, 170, 4, 70, "RelatedTutorials",
 CellID->806871991]
}, Open  ]]
}, Open  ]],
Cell[9037, 255, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

