(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     12363,        422]
NotebookOptionsPosition[      8504,        305]
NotebookOutlinePosition[     10418,        350]
CellTagsIndexPosition[     10375,        347]
WindowTitle->Aerodynamics: Table Of Contents - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[" ", "TutorialColorBar",
 CellMargins->{{Inherited, Inherited}, {-5, 0}}],

Cell[TextData[{
 ButtonBox["Aerodynamics",
  BaseStyle->{"Link", "LinkTrail"},
  ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"],
 StyleBox[" > ", "LinkTrailSeparator"]
}], "LinkTrail"],

Cell[BoxData[GridBox[{
   {Cell["AERODYNAMICS TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Aerodynamics\"\>":>
       Documentation`HelpLookup["paclet:Aerodynamics/guide/Aerodynamics"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"Aerodynamics/tutorial/TableOfContents\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["Aerodynamics/tutorial/TableOfContents"]}, Visible -> 
            False]]; SelectionMove[
         DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/Aerodynamics/\
tutorial/TableOfContents.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "Aerodynamics/tutorial/TableOfContents", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 ": Table Of Contents"
}], "Title",
 CellID->225410711],

Cell[TextData[{
 StyleBox["Aerodynamics",
  FontSlant->"Italic"],
 " is a collection of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " application packages for constructing numerical models for computing \
aerodynamic properties of airfoils.  Although the long-term intent is to \
provide a collection of computational tools for different flow physics and \
for two- and three-dimensional flows, this version is limited to steady, \
two-dimensional potential flow over airfoils.   The software consists of five \
packages, and the tutorials may be considered an incomplete draft of an \
electronic book on aerodynamics."
}], "Text",
 CellID->1534169418],

Cell[TextData[ButtonBox["Preface",
 BaseStyle->"Link",
 ButtonData->
  "paclet:Aerodynamics/tutorial/PrefaceToAerodynamics"]], "Subsection",
 CellID->140154496],

Cell[CellGroupData[{

Cell["Chapter 1  Airfoil Geometry", "Subsection",
 CellID->273035715],

Cell[TextData[{
 "\t1.1  ",
 ButtonBox["Introduction to Airfoil Geometry",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/IntroductionToAirfoilGeometry"]
}], "Text",
 CellID->232035282],

Cell[TextData[{
 "\t1.2  ",
 ButtonBox["Airfoils from Data Files",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/AirfoilsFromDataFiles"]
}], "Text",
 CellID->309722458],

Cell[TextData[{
 "\t1.3  ",
 ButtonBox["Airfoils from Thickness and Camber Properties",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/tutorial/AirfoilsFromThicknessAndCamberProperties"]
}], "Text",
 CellID->132244910],

Cell[TextData[{
 "\t1.4  ",
 ButtonBox["Airfoils from Conformal Mapping",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/AirfoilsFromConformalMapping"]
}], "Text",
 CellID->70020217],

Cell[TextData[{
 "\t1.5  ",
 ButtonBox["Panel Geometry",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/PanelGeometry"]
}], "Text",
 CellID->149476908]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 2 InfluenceCoefficients", "Subsection",
 CellID->30482394],

Cell[TextData[{
 "\t2.1  ",
 ButtonBox["Introduction to Influence Coefficients",
  BaseStyle->"Link",
  ButtonData->
   "paclet:Aerodynamics/tutorial/IntroductionToInfluenceCoefficients"]
}], "Text",
 CellID->28892229],

Cell[TextData[{
 "\t2.2  ",
 ButtonBox["Point Singularities",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/PointSingularities"]
}], "Text",
 CellID->401496844],

Cell[TextData[{
 "\t2.3  ",
 ButtonBox["Isolated Panel",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/IsolatedPanel"]
}], "Text",
 CellID->221349426]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 3 PanelMethods", "Subsection",
 CellID->19579000],

Cell[TextData[{
 "\t3.1  ",
 ButtonBox["Introduction to Panel Methods",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/IntroductionToPanelMethods"]
}], "Text",
 CellID->391717129],

Cell[TextData[{
 "\t3.2  ",
 ButtonBox["Validation Example",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/ValidationExample"]
}], "Text",
 CellID->226269871],

Cell[TextData[{
 "\t3.3  ",
 ButtonBox["Implementation",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/Implementation"]
}], "Text",
 CellID->23172448],

Cell[TextData[{
 "\t3.4  ",
 ButtonBox["Flow Visualization",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/FlowVisualization"]
}], "Text",
 CellID->15577002],

Cell[TextData[{
 "\t3.5  ",
 ButtonBox["Convergence and Timing",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/ConvergenceAndTiming"]
}], "Text",
 CellID->197670982],

Cell[TextData[{
 "\t3.6  ",
 ButtonBox["Aerodynamic Coefficients",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/AerodynamicCoefficients"]
}], "Text",
 CellID->26729679],

Cell[TextData[ButtonBox["\t3.7  Analysis of an Airfoil",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/AnalysisOfAnAirfoil"]], "Text",
 CellID->543713775]
}, Open  ]],

Cell[TextData[ButtonBox["References",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/References"]], "Subsection",
 CellID->111528304],

Cell[CellGroupData[{

Cell["Appendices", "Subsection",
 CellID->1052532],

Cell[TextData[{
 "\tA1 ",
 ButtonBox["CartesianVectors",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/CartesianVectors"]
}], "Text",
 CellID->226534904],

Cell[TextData[{
 "\tA2 ",
 ButtonBox["Graphics",
  BaseStyle->"Link",
  ButtonData->"paclet:Aerodynamics/tutorial/Graphics"]
}], "Text",
 CellID->46165208]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["Acknowledgments",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/tutorial/Acknowledgments"]], "Subsection",
 CellID->47173671],

Cell[CellGroupData[{

Cell["MORE ABOUT", "TutorialMoreAboutSection",
 CellID->23220180],

Cell[TextData[ButtonBox["Aerodynamics",
 BaseStyle->"Link",
 ButtonData->"paclet:Aerodynamics/guide/Aerodynamics"]], "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"Aerodynamics: Table Of Contents - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, 
  "Metadata" -> {
   "built" -> "{2009, 11, 25, 11, 0, 46.5762000}", "context" -> 
    "Aerodynamics`", "keywords" -> {}, "index" -> True, "label" -> 
    "Aerodynamics Tutorial", "language" -> "en", "paclet" -> "Aerodynamics", 
    "status" -> "", "summary" -> 
    "Aerodynamics is a collection of Mathematica application packages for \
constructing numerical models for computing aerodynamic properties of \
airfoils. Although the long-term intent is to provide a collection of \
computational tools for different flow physics and for two- and \
three-dimensional flows, this version is limited to steady, two-dimensional \
potential flow over airfoils. The software consists of five packages, and the \
tutorials may be considered an incomplete draft of an electronic book on \
aerodynamics. 1.1 Introduction to Airfoil Geometry 1.2 Airfoils from Data \
Files", "synonyms" -> {}, "title" -> "Aerodynamics: Table Of Contents", 
    "type" -> "Tutorial", "uri" -> "Aerodynamics/tutorial/TableOfContents"}, 
  "SearchTextTranslated" -> ""},
FrontEndVersion->"7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "7.0 for Microsoft Windows (64-bit) (February 18, 2009)",
   StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[612, 21, 78, 1, 70, "TutorialColorBar"],
Cell[693, 24, 191, 5, 70, "LinkTrail"],
Cell[887, 31, 2394, 55, 70, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3306, 90, 121, 5, 70, "Title",
 CellID->225410711],
Cell[3430, 97, 661, 14, 70, "Text",
 CellID->1534169418],
Cell[4094, 113, 160, 4, 70, "Subsection",
 CellID->140154496],
Cell[CellGroupData[{
Cell[4279, 121, 69, 1, 70, "Subsection",
 CellID->273035715],
Cell[4351, 124, 203, 6, 70, "Text",
 CellID->232035282],
Cell[4557, 132, 187, 6, 70, "Text",
 CellID->309722458],
Cell[4747, 140, 231, 7, 70, "Text",
 CellID->132244910],
Cell[4981, 149, 200, 6, 70, "Text",
 CellID->70020217],
Cell[5184, 157, 169, 6, 70, "Text",
 CellID->149476908]
}, Open  ]],
Cell[CellGroupData[{
Cell[5390, 168, 72, 1, 70, "Subsection",
 CellID->30482394],
Cell[5465, 171, 218, 7, 70, "Text",
 CellID->28892229],
Cell[5686, 180, 179, 6, 70, "Text",
 CellID->401496844],
Cell[5868, 188, 169, 6, 70, "Text",
 CellID->221349426]
}, Open  ]],
Cell[CellGroupData[{
Cell[6074, 199, 63, 1, 70, "Subsection",
 CellID->19579000],
Cell[6140, 202, 197, 6, 70, "Text",
 CellID->391717129],
Cell[6340, 210, 177, 6, 70, "Text",
 CellID->226269871],
Cell[6520, 218, 169, 6, 70, "Text",
 CellID->23172448],
Cell[6692, 226, 176, 6, 70, "Text",
 CellID->15577002],
Cell[6871, 234, 184, 6, 70, "Text",
 CellID->197670982],
Cell[7058, 242, 188, 6, 70, "Text",
 CellID->26729679],
Cell[7249, 250, 171, 3, 70, "Text",
 CellID->543713775]
}, Open  ]],
Cell[7435, 256, 149, 3, 70, "Subsection",
 CellID->111528304],
Cell[CellGroupData[{
Cell[7609, 263, 50, 1, 70, "Subsection",
 CellID->1052532],
Cell[7662, 266, 172, 6, 70, "Text",
 CellID->226534904],
Cell[7837, 274, 155, 6, 70, "Text",
 CellID->46165208]
}, Open  ]],
Cell[CellGroupData[{
Cell[8029, 285, 158, 3, 70, "Subsection",
 CellID->47173671],
Cell[CellGroupData[{
Cell[8212, 292, 65, 1, 70, "TutorialMoreAboutSection",
 CellID->23220180],
Cell[8280, 295, 158, 3, 70, "TutorialMoreAbout",
 CellID->1567025153]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[8477, 303, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

